
require "common/scripts/moxie/moxie"
require "common/scripts/moxie/shell"

require "buggynight/scripts/fixup/strict"
require "buggynight/scripts/fixup/logger"
require "buggynight/scripts/android/android"
require "buggynight/scripts/global"
require "buggynight/scripts/prefetch"
require "buggynight/scripts/buggynight"
require "buggynight/scripts/endofshow"

-- Only needed for testing animations in it's own scene
require "buggynight/scripts/testplayground"

-- Launcher:
require "buggynight/scripts/launcher/launcher"


local Logger = TaggedLog("Application", false)
local launcherEnabled = true

Application.onRotation = function(deviceRotation)
    Shell.dispatch("onRotation", {deviceRotation})
end

Application.onInterceptTouch = function(windowX, windowY, x, y, eventTime, onTSB)
    -- dispatch to current state
    return Shell.dispatch("onInterceptTouch", {windowX=windowX, windowY=windowY, x=x, y=y, eventTime=eventTime, onTSB=onTSB})
end

Application.onPPStateChange = function(window)
    -- dispatch to current state
    Shell.dispatch("onPPStateChange", {window=window})
end

Application.onPPLocSizeChanged = function(appBecameFullScreen, window)
    -- dispatch to current state
    Shell.dispatch("onPPLocSizeChanged", {appBecameFullScreen=appBecameFullScreen, window=window})
end

Application.onSystemUIVizUpdate = function(newVisibility)
    Context.onSystemUIVizUpdate(newVisibility)
end

Application.onActivityEvent = function(eventString)
    -- debug only: exit doodle on dummy activity destruction
    if (eventString == "onStop") then
        Global.shutdown = true
    end

    -- dispatch to current state
    Shell.dispatch("onActivityEvent", {eventString=eventString})
end

Application.onChangeDisplaySize = function(screenWidth, screenHeight, availWidth, availHeight)
    Logger:message("Change display size " .. screenWidth .. ", " .. screenHeight .. ", " .. availWidth .. ", " .. availHeight)

    Global.screen_size.x = screenWidth
    Global.screen_size.y = screenHeight
    Global.avail_size.x = availWidth
    Global.avail_size.y = availHeight
    Global.updateScreenSize()

    -- default clip rect to full screen
    Global.setRenderOverTSB(true)

    -- dispatch to current state
    Shell.dispatch("onChangeDisplaySize", {screenWidth=screenWidth, screenHeight=screenHeight, availWidth=availWidth, availHeight=availHeight})
end

Application.onVolumeUp = function(newVolume)
    Logger:message("Volume up " .. newVolume)

    -- dispatch to current state
    Shell.dispatch("onVolumeUp", {newVolume=newVolume})
end

Application.onVolumeDown = function(newVolume)
    Logger:message("Volume down " .. newVolume)

    -- dispatch to current state
    Shell.dispatch("onVolumeDown", {newVolume=newVolume})
end

Application.onScreenEvent = function(newState)
    Logger:message("Screen event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onScreenEvent", {newState=newState})
end

Application.onBatteryEvent = function(newState)
    Logger:message("Battery event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onBatteryEvent", {newState=newState})
end

Application.onHeadsetEvent = function(newState)
    Logger:message("Headset event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onHeadsetEvent", {newState=newState})
end

Application.onBluetoothEvent = function(newState)
    Logger:message("Bluetooth event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onBluetoothEvent", {newState=newState})
end

Application.onKeyPress = function(keyCode, action, isLongPress, downTime)
    Logger:message("Keypress event " .. keyCode .. ", " .. action .. ", " .. downTime)

    -- dispatch to current state
    Shell.dispatch("onKeyPress", {keyCode=keyCode, action=action, isLongPress=isLongPress, downTime=downTime})
end

Application.onTouchDown = function(pointerId, x, y, eventTime, touchPhase)
    Logger:message("Touch down " .. pointerId .. ", " .. x .. ", " .. y .. ", " .. eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchDown", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    Events.send(Application, "touchDown", Vector(x, y, 0, 0))

    return true
end

Application.onTouchUp = function(pointerId, x, y, eventTime, touchPhase)
    Logger:message("Touch up " .. pointerId .. ", " .. x .. ", " .. y .. ", " ..eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchUp", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})

    Events.send(Application, "touchUp", Vector(x,y, 0, 0))
    return true
end

Application.onTouchMove = function(pointerId, x, y, eventTime, touchPhase)
    Logger:message("Touch move " .. pointerId .. ", " .. x .. ", " .. y .. ", " .. eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchMove", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})

    Events.send(Application, "drag", Vector(x, y, 0, 0))
    return true
end

Application.onRequestExit = function()
    Logger:message("Exit requested")
    return true
end

Application.onTerminate = function()
	Shell.onShutdown();
	Global.onShutdown();
end

-- Now that everything is loaded, lock down the global
-- table so no more global variables can accidentally be
-- created
Strict.globalLock(true)

if (launcherEnabled) then
    -- Go through the home screen launcher state
    Shell.start("launcher_start")
else
    -- Go straight to the show
    if (Avatar.setWatchdogPaused) then
        Avatar.setWatchdogPaused(true)
    end
    Global.init()
    Global.initSpotLight()
    AsyncLoader.prefetch()
    Android.acquireScreen()
    Android.setAlphaRange(1, 1)
    InputManager:enableSensors(true)
    Global.initShow()
    Global.lockHeading = false
    Global.scene.mainCamera.controller:enableTouchControls(true)
    Global.scene.mainCamera.controller:UseSensor()
    Global.scene.mainCamera.controller:releaseSoftClamp()
    Shell.start("buggynight")
end
