
local Logger = TaggedLog.new("SoundPlayer", false)

SoundPlayer = Manager:new {
    name = "SoundPlayer",

    currentSegment = nil,
    pendingEvents = {},
    segmentSounds = {},

    quantizedBed = nil,

    segmentTimer = 0,
}

function SoundPlayer:onStart()
    Events.listen(self, "segmentPlaying")
end

function SoundPlayer:onDestroy()
    Events.stopListening(self, "segmentPlaying")
end

function SoundPlayer:onUpdate()
    if (self.currentSegment) then
        self.segmentTimer = self.segmentTimer + Time.dt()
        local frame = self.segmentTimer * 24 + self.currentSegment.startFrame

--        Global.AppDebugText3 = "Frame = " .. tostring(math.floor(frame))
        if SceneData.AudioLatencyOffset then
            frame = frame + SceneData.AudioLatencyOffset
        end

        local stillPending = {}
        for _, event in ipairs(self.pendingEvents) do
            if event.endFrame then
                local endFrame = event.endFrame + self:getQuantizedDelay(event.quantized)
                if endFrame <= frame then
                    local sound = SoundManager:getSound(event.name)
					if sound == nil then
						return
					end
                    if event.quantized then
                        Logger:message("frame: " .. tostring(frame) .. " sound: " .. event.name .. "(" .. tostring(sound) .. ") ending quantized delay = " .. tostring(self:getQuantizedDelay(event.quantized) .. " bed = " .. tostring(self.quantizedBed)))
                    end
                    sound.soundControl.autorelease = false
                    sound:stop()
                else
                    table.insert(stillPending, event)
                end
            else
                local startFrame = event.startFrame or self.currentSegment.startFrame
                if event.startOffset then
                    startFrame = startFrame + event.startOffset
                end
                startFrame = startFrame + self:getQuantizedDelay(event.quantized)
                if (startFrame <= frame) then
                    local sound = SoundManager:getSound(event.name)
					if sound == nil then
						return
					end
                    if event.quantized then
                        Logger:message("frame: " .. tostring(frame) .. " sound: " .. event.name .. "(" .. tostring(sound) .. ") starting quantized delay = " .. tostring(self:getQuantizedDelay(event.quantized) .. " bed = " .. tostring(self.quantizedBed)))
                    end
                    self.segmentSounds[sound.name] = sound
                    sound.soundControl:setAudioType(event.type)
                    sound.soundControl.autorelease = false
                    sound.soundControl.trackPosition = event.trackPosition
                    if (not sound:isPlaying()) then
                        sound:setLooping(event.looping)
                    end
                    if (event.fadeTime) then
                        sound.soundControl:setTimedFade(0, event.fadeTime, event.fadeStart, event.fadeEnd)
                    else
                        Logger:message("triggered " .. tostring(sound.name) .. " at volume " .. tostring(sound.soundControl.currentVolume))
                        sound.soundControl:setVolume(event.volume or SceneData.AudioTypes[event.type].defaultVolume or 1)
                        sound:play()
                    end
                    if event.quantizedBed then
                        self.quantizedBed = sound
                    end
                else
                    table.insert(stillPending, event)
                end
            end
        end
        self.pendingEvents = stillPending
    end
end

function SoundPlayer:segmentPlaying(segmentName)
    Logger:message("segment = " .. tostring(segmentName))

    local stillPlaying = {}
    for name, sound in pairs(self.segmentSounds) do
        Logger:message("checking sound: " .. tostring(name))
        if (sound:isPlaying()) then
            stillPlaying[name] = sound
        else
            SoundManager:releaseSound(sound)
            if self.quantizedBed == sound then self.quantizedBed = nil end
        end
    end
    self.segmentSounds = stillPlaying

    self.segmentTimer = 0
    self.currentSegment = SceneData.audio[segmentName]
    local pendingEvents = {}
    if self.currentSegment and self.currentSegment.sounds then
        for _, event in ipairs(self.currentSegment.sounds) do
            local type = nil
            if event.type then
                type = SceneData.AudioTypes[event.type]
            end
            if type then
                if type.enabled then
                    table.insert(pendingEvents, event)
                end
            else
                Log.warning("SoundPlayer: invalid audio type: " .. tostring(event.type) .. " for sound: " .. tostring(event.name))
            end
        end
    end
    self.pendingEvents = pendingEvents
end

function SoundPlayer:getQuantizedDelay(interval)
    local quantizedDelay = 0
    if interval then
        local currentTime = self:getQuantizedBedTime() / interval
        local nextBlock = math.ceil(currentTime)
        quantizedDelay = (nextBlock - currentTime) * interval
--        Logger:message("quantized delay = " .. self:getQuantizedBedTime() .. ", interval = " .. interval .. ", nextBlock = " .. nextBlock .. ", delay = " .. quantizedDelay)
    end
    return quantizedDelay
end

function SoundPlayer:getQuantizedBedTime()
    if (self.quantizedBed) then -- and self.quantizedBed:isPlaying()) then
        return self.quantizedBed.soundControl:getSeekPosition()
    else
        return 0
    end
end
