
require "common/scripts/moxie/events"
require "buggynight/scripts/components/camera/orientationremapper"

local Logger = TaggedLog.new("CameraController", false)



CameraController  = {
	touchAnchor	= nil,
	rotationScaleX = 0.14,
	currentRotationScaleX = 0.14,
	rotationScaleY = 0.13,
	rotY = 15,
	rotX = 0.0,
	quatY = Quaternion.new(),
	quatX =  Quaternion.new(),
	rotZ = 0.0,
	smoothCamera        = true,
	cameraSpeed         = 16,
	boomLength          = 1,
	cameraOrbitSquashY  = 0.5,
	touchControls       = false,
	constrainedTouch    = true,
	rotationFinal       = Vector.zero:copy(),
	touchStartTime      = 0.0,
	touchTime           = 0.0,
	cameraCrane         = Vector.zero:copy(),
	calib               = nil,
	lastUpdateSecs      = nil,
	touchDelta          = nil,
	dragPosition        = nil,

	startYOffset        = 128,
	currentHeading      = 0,
	rotSource           = Quaternion.new(),
	startYOffsetQuat    = Quaternion.new(),
	lastYawQuat         = Quaternion.new(),
	lastYaw             = 0,

	yClampStart         = nil,
	yClampFinal         = 0,
	headingClamp        = Quaternion.new(),
	hitQuad             = {x=0, y=0, width=0, height=0},
	hitQuadSize         = .875,

	lockHeading         = false,

	-- x rotation limit
	softClampReady      = false,
	softClamp           = true,
	remapper            = shallowcopy(OrientationRemapper),

	-- good way in stuff
	gwiLock             = true,
	useSensor           = false,

	-- touch handling checking
	isTouchDown         = false,
	isTouchUp           = true,

	target				= nil,
	targetThreshold		= 0.0,
}

CameraController.new = function(self)
	local o = {}
	setmetatable(o, self)
	self.__index = self
	return o
end

function CameraController:onStart()
	self:lookAtHeading(self.startYOffset)

	local size = math.clamp01(self.hitQuadSize)
	self.hitQuad = {
		xMin=Global.screen_size.x*(1 - size),
		yMin=Global.screen_size.y*(1 - size),
		xMax=Global.screen_size.x*(size),
		yMax=Global.screen_size.y*(size)
	}

	if Platform.win32 or Platform.mac then
		self.constrainedTouch = false
	end

	self.calib = Quaternion.new()
	if Platform.android or Platform.iOS then
		self.calib:setAxisAngle(Vector.right, math.rad(-90))
	else
		self.calib:setAxisAngle(Vector.right, math.rad(0))
	end

	self:setInitialRotation()
	self.softClamp = true

--[[
	self.debugSphere = Global.scene:createActor( "camera_debugSphere", SceneData.actors.DebugSphere )
	self.debugSphere:setTranslation( Vector.new( -5.0, 2.0, 5.0 ) )
	self.debugSphere:setScale( Vector.new( 0.25, 0.25, 0.25 ) )
	self.debugSphere.materials.Mat_Sphere:setParameter("u_diffuseColor", Vector.new( 0.0, 0.0, 1.0, 0.25 ) )
	self:setTarget( self.debugSphere, 30.0 )
--]]
end

function CameraController:releaseSoftClamp()
	self.softClampReady = true
	self:enableTouchControls(true)
end

function CameraController:enableTouchControls(enabled)
	self.touchControls = enabled

	if (self.touchControls) then
		Events.listen(self, "touchDown")
		Events.listen(self, "touchUp")
		Events.listen(self, "drag")
	else
		Events.stopListening(self, "touchDown")
		Events.stopListening(self, "touchUp")
		Events.stopListening(self, "drag")
	end
end

function CameraController:setInitialRotation()
	self.rotY = 0
end

function CameraController:touchDown(touchPosition)
	if self.isTouchDown then
		Logger:message("Touch: UP NOT RECVD")
		self.isTouchUp = true
		self:resetInput(touchPosition)
	end
	self.isTouchDown = true

	self.touchAnchor = touchPosition
	self.touchStartTime = os.time()

	if (touchPosition.x < self.hitQuad.xMax and touchPosition.y < self.hitQuad.yMax and
		touchPosition.x > self.hitQuad.xMin and touchPosition.y > self.hitQuad.yMin) then
		self.lockHeading = true
	end

	self.isTouchUp = false
end

function CameraController:resetInput(touchPosition)
	if (self.touchAnchor == nil) then return end

	self.touchTime = os.time() - self.touchStartTime

	local toAnchor = (self.touchAnchor - touchPosition)

	self.dragPosition = nil
	self.lockHeading = false
	self.isTouchDown = false
end

function CameraController:touchUp(touchPosition)
	self.isTouchUp = true
	self:resetInput(touchPosition)
end

function CameraController:drag(touchDelta)

	if (not self.dragPosition) then
		self.dragPosition = Vector(touchDelta.x, touchDelta.y)
	end

	local pos = Vector(touchDelta.x, touchDelta.y)

	self.touchDelta = Vector(0.5*(pos.x - self.dragPosition.x), 0.5*(pos.y - self.dragPosition.y), 0.0, 0.0)

	self.dragPosition = pos

	local localT = self.object:getLocal()
	local rollAngle, rollQuat = localT.rotation:getRoll()
	self.touchDelta = self.touchDelta:rotateBy(rollQuat:inverse())

	--update rotation
	self.rotY = self.rotY + self.touchDelta.y
	self.rotX = self.rotX - self.touchDelta.x
	self.quatX = Quaternion.axisAngle(Vector.right, self.rotY * self.currentRotationScaleX)
	self.quatY = Quaternion.axisAngle(Vector.up, -self.rotX * self.rotationScaleY)
end

function CameraController:lookAt(objT)
	local lookDir = objT.position - self.object:getWorld().position
	local rotation = Quaternion.lookAt(lookDir)

	-- rotY is calculated from y motion with mouse/finger and converted into pitch
	self.rotY, self.quatX = rotation:getPitch()
	self.rotX, self.quatY = rotation:getYaw()
end

function CameraController:lookAtHeading(heading)
	self.lastYaw, self.lastYawQuat = self.object:getLocal().rotation:getYaw()
	self.currentHeading = heading

	local northOffset = self.currentHeading - self.lastYaw

	self.startYOffsetQuat = Quaternion.axisAngle(Vector.up, northOffset)
end

function CameraController:_getHeadingClamp(yaw)
	if (not self.yClampStart) then
		self.yClampStart = yaw - self.yClampFinal
	end
	self.yClampFinal = yaw - self.yClampStart

	return Quaternion.axisAngle(Vector.up, self.yClampFinal):inverse()
end

function CameraController:UseSensor()
	self.useSensor = true
end

function CameraController:setTarget( obj, threshold )
	self.target = obj
	self.targetThreshold = math.rad( threshold )
end

local function quatLookAt( dir, up )
	local right				= Vector.cross( dir, up )
	right:normalize()
	
	up						= Vector.cross( right, dir )
	up:normalize()

	local basis				= Matrix.identity:copy()
	basis.m00				= right.x
	basis.m10				= right.y
	basis.m20				= right.z
	basis.m01				= up.x
	basis.m11				= up.y
	basis.m21				= up.z
	basis.m02				= -dir.x
	basis.m12				= -dir.y
	basis.m22				= -dir.z

	local q 				= Quaternion.new()
	q:setMatrix( basis )

	return q
end

function CameraController:getDirectionToTarget()
	local targetWorld	= self.target:getWorld()
	local cameraWorld	= self.object:getWorld()

	local dir			= targetWorld.position - cameraWorld.position
	dir:normalize()

	return dir
end

function CameraController:lookAtTarget( localT )
	local dir				= self:getDirectionToTarget()

	local cameraForward		= localT:forward()

	local angle				= math.acos( Vector.dot( dir, cameraForward ) )
	local t					= math.clamp01( angle / self.targetThreshold )

	dir						= Vector.lerp( t*t, dir, cameraForward )
	dir:normalize()

	return quatLookAt( dir, localT:up() )
end

function CameraController:onUpdate()
	local localT = self.object:getLocal()
	local lastRotation = localT.rotation
	
	if (self.useSensor) then
		self.rotSource = InputManager:getSensorRotation()
	end

	self.lastYaw, self.lastYawQuat = self.rotSource:getYaw()

	local useLockHeading = self.lockHeading or Global.lockHeading
	if (useLockHeading) then
		self.headingClamp = self:_getHeadingClamp(self.lastYaw)
	else
		self.yClampStart = nil
	end

	if (self.constrainedTouch) then
		self.rotY = 0
		self.quatX = Quaternion.new()
	end
	
	
	--Global.AppDebugText1 = tostring(self.softClamp)

	if (self.softClamp) then
		if (self.softClampReady) then
			
			localT.rotation, self.softClamp = self.remapper:remapOrientation(self.rotSource:copy())
		else 
			localT.rotation = self.remapper:remapOrientation(self.rotSource:copy())
		end
		
	else
		localT.rotation = self.rotSource:copy()
	end

	-- combine all offsets into final rotation
	localT.rotation = self.startYOffsetQuat*self.quatY*self.quatX*self.headingClamp*localT.rotation

	-- calculate time independently of Time.dt() so that we can still look around when paused
	local currTimeSecs = Time.seconds()
	self.lastUpdateSecs = self.lastUpdateSecs or currTimeSecs
	local dt = currTimeSecs - self.lastUpdateSecs

	-- Slerp towards goal
	if (not self.softClamp and self.smoothCamera and self.cameraSpeed ~= 0) then
		localT.rotation = Quaternion.slerp(dt * self.cameraSpeed, lastRotation, localT.rotation )
	end

--[[
	local t = math.clamp01( math.cos( Time.seconds() ) * 0.5 + 0.5 ) - 3.5*math.pi
	local x = math.cos( t ) * 5.0
	local y = math.sin( t ) * 5.0
	self.debugSphere:setTranslation( Vector.new( x, 2.0, y ) )
--]]

	-- look at target
	if self.target then
		localT.rotation = self:lookAtTarget( localT )
	end

	local camLook = Vector(0.0, 0.0, -1.0, 0)
	camLook = camLook:rotateBy(localT.rotation)
	camLook.w = 0
	camLook:normalize()

	-- Create a new forward vector that is squashed on Y in an ellipsoid
	local forward = Vector(
		-- X
		camLook.x,

		-- Y ( solve for Y in: x^2 + (y/0.5)^2 + z^2 = 1.0  y = 0.5*( sqrt(1.0 - x^2 - z^2) ))
		math.sign(camLook.y)*self.cameraOrbitSquashY *
			math.sqrt(math.clamp(1.0 - camLook.x*camLook.x - camLook.z*camLook.z, 0.0, 5.6e300)),

		-- Z
		camLook.z
	)

	-- add position back in now that rotation has been applied
	localT.position = self.cameraCrane + forward*self.boomLength
	self.object:setLocal(localT)
end
