

FloatyDriver = {
    startHeading = 0,
    endHeading = 0,
    startColor = Vector(1, 1, 1, 0),
    endColor = Vector(1, 1, 1, 1),
    colorTimer = 0,
    fadeOn = true

}

function FloatyDriver:new(params)
    local newObj = shallowcopy(params) or {}
    setmetatable(newObj, self)
    self.__index = self

    return newObj
end

function FloatyDriver:onStart()
    self.material = self.object:findMaterial(self.materialName)
    self.object.animation["Loop"]:play()
    self.alive = false
    self.fadeOn = true
    self.timeLeft = self.lifeSpan + self.startDelay
end

function FloatyDriver:onUpdate()

    -- work out the color
    if (self.alive and self.fadeOn) then 
        self.colorTimer = math.clamp01(self.colorTimer + Time.dt())
    else 
        self.colorTimer = math.clamp01(self.colorTimer - Time.dt())
    end
    
    local amount = math.easeInOutSine(math.clamp(self.colorTimer, 0, 1))
    local matColor = Vector.lerp(amount, self.startColor, self.endColor)
    self.material:setParameters({ u_Color = matColor})
        
    -- wait a second before getting started
    if (not self.alive) then 
        self.startDelay = self.startDelay - Time.dt()
        if (self.startDelay < 0) then 
            self.alive = true 
            self:setUp()
        end
    else 

        -- find the transition time left and kill it when done
        self.timeLeft = self.timeLeft - Time.dt()
        if (self.timeLeft < 1 and self.alive) then
            self.fadeOn = false
        end
        
        
        -- do the rotation
        if (self.timeLeft < 0) then
            return -- dont rotate is the timer is over
        end
        
        local normalizedTimeLeft =  1 - (self.timeLeft/self.lifeSpan)
        local rotationAmount = math.easeInOutQuart(math.clamp(normalizedTimeLeft, 0, 1))
        
        local objectSRT = self.object:getLocal()
        objectSRT.rotation = Quaternion.slerp(rotationAmount, Quaternion.axisAngle(Vector.up, self.startHeading), Quaternion.axisAngle(Vector.up, self.endHeading))
        self.object:setLocal(objectSRT)
    end
end


function FloatyDriver:setUp()
    self.startHeading = Global.scene.mainCamera:getWorld().rotation:getYaw()
    self.endHeading = Global.worldRot
    self.alive = true
end


