
FootprintFader  = {}
FootprintFader.__index = FootprintFader

function FootprintFader:new(params)
    local o = shallowcopy(params) or {}

    setmetatable(o, self)
    self.__index = self

    return o
end


function FootprintFader:onStart()
    self.material = self.object:findMaterial(self.matName)
    if (self.fadeObjectName ~= nil) then
        local fadeSlice = ActorManager:getActor(self.fadeObjectName, false);
        local faceSlicePos = fadeSlice:getBoneWorld(self.fadeObjectBone)
        self.material:setParameters({ u_trailPosition = faceSlicePos.position})
    end 

    self.currentColor = Vector.new( 1.0, 1.0, 1.0, 0.0 )
    Events.listen(self, "segmentPlaying")

    self:fadeIn(self.fadeInDelay, self.fadeInSpeed)
end

function FootprintFader:onUpdate()

	if (self.timer > 0 and self.fadeObjectName ~= nil) then
        self.timer = self.timer - (Time.dt() / self.speed)
        local amount = math.easeInOutSine(math.clamp01(self.timer))
        self.currentColor = Vector.lerp(1 - amount, self.startColor, self.endColor)
        self.material:setParameters({ u_Color = self.currentColor})
        
        local fadeSlice = ActorManager:getActor(self.fadeObjectName, false);
        local faceSlicePos = fadeSlice:getBoneWorld(self.fadeObjectBone)
        self.material:setParameters({ u_trailPosition = faceSlicePos.position})
	end 
end


function FootprintFader:segmentPlaying(segmentName)        
	if (segmentName == self.fadeOutSegment) then
		self:fadeOut(self.fadeOutDelay, self.fadeOutSpeed)
	end
end

FootprintFader.fadeIn = function(self, delay, speed)
    self.timer = 1 + (delay / speed);
    self.speed = speed;
	self.startColor = self.currentColor
	self.endColor = Vector.new( 1.0, 1.0, 1.0, 1.0 )
end

FootprintFader.fadeOut = function(self, delay, speed)
    self.timer = 1 + (delay / speed);
    self.speed = speed;
	self.startColor = self.currentColor
	self.endColor = Vector.new( 1.0, 1.0, 1.0, 0.0 )
end
