
LightTracker  = {}
LightTracker.__index = LightTracker

function LightTracker:new(params)
	local newObj = shallowcopy(params) or {}
	setmetatable(newObj, self)
	self.__index = self
	self.debug = false

	return newObj
end

function LightTracker:createDebugSphere( position, color )
	local sphere = Global.scene:createActor("light_debugsphere", SceneData.actors.DebugSphere)
	sphere:setTranslation( position )
	sphere:setScale( Vector.new( 0.125, 0.125, 0.125 ) )
	sphere.materials.Mat_Sphere:setParameter("u_diffuseColor", color )

	self.track:addChild( sphere )
end

function LightTracker:onStart()
	self.trackingActor = ActorManager:getActor(self.trackingActor)
	self.light = Global.scene:createLight(self.lightName, self.lightDef)
	self.light:setTranslation( self.lightPosition )
	
	self.track = Global.scene:createTransform( self.lightName.."Tracker" )
	self.track:addChild( self.light )

	if self.lightTarget then
		--local dir = self.lightTarget - self.lightPosition
		local dir = self.lightPosition - self.lightTarget
		dir:normalize()

		self.light:setRotation( Quaternion.lookAt( dir, Vector.up ) )
	end
	
	if self.debug then
		self:createDebugSphere( Vector.zero, Vector.new( 1.0, 0.0, 0.0, 0.25 ) )

		if self.lightPosition then
			self:createDebugSphere( self.lightPosition, Vector.new( 0.0, 1.0, 0.0, 0.25 ) )
		end

		if self.lightTarget then
			self:createDebugSphere( self.lightTarget, Vector.new( 0.0, 0.0, 1.0, 0.25 ) )
		end
	end
end

function LightTracker:onDestroy() 
	self.light:destroy()
	self.track:destroy()
end 

function LightTracker:onUpdate()
	local bonePos = self.trackingActor:getBoneWorld(self.trackingBone)
	self.track:setLocal( bonePos )
end
