
local Logger = TaggedLog("Locator", false)

local AlwaysShowDebug = false

Locator = {
    targetPosition = Vector.zero:copy(),
    radius = 1,
}

function Locator:new(params)
    local o = shallowcopy(params) or {}
    setmetatable(o, self)
    self.__index = self
    return o
end

function Locator:onStart()
    self.label = tostring(self.object.name) .. "-locatorTarget"
    if not self.fixedPosition then
        self.target = Global.scene:createTransform(self.label .. "-locatorTarget")
        if self.target then
            local transform = self.target:getLocal()
            transform.position = self.targetPosition:copy()
            self.target:setLocal(transform)
            self.object:addChild(self.target)
        else
            Log.error("Locator unable to create target transform for: " .. tostring(self.object.name))
        end
    end
    if AlwaysShowDebug or self.debugVisible then
        self:_createDebugSphere()
    end
end

function Locator:onDestroy()
    if self.fixedPosition and self.debugSphere then
        self.debugSphere:destroy()
    end
end

function Locator:getTrackingPosition()
    if self.fixedPosition or not self.target then
        return self.targetPosition
    end
    return self.target:getWorld().position
end

function Locator:_createDebugSphere()
    local debugSphere = Global.scene:createActor(self.label .. "_debugsphere", SceneData.actors.DebugSphere)
    if debugSphere then
        debugSphere:setVisibility(true)
        if self.debugColor then
            debugSphere.materials.Mat_Sphere:setParameter("u_diffuseColor", self.debugColor)
        end
        local transform = debugSphere:getLocal()
        transform.position = self.targetPosition:copy()
        transform.scale = Vector(self.radius, self.radius, self.radius)
        debugSphere:setLocal(transform)
        if not self.fixedPosition then
            self.object:addChild(debugSphere)
        end

        self.debugSphere = debugSphere
    end
end
