
PositionTracker  = {}
PositionTracker.__index = PositionTracker

function PositionTracker.new(trackingActor, boneName, onlyY)
    local this = {
        trackingActor = trackingActor,
        boneName = boneName,
        onlyYAxis = onlyY,
        active = true,
        trackRot = false,
    }
    return setmetatable(this, PositionTracker)
end

function PositionTracker:onStart()
    Events.listen(self, "onActorDestroyed")
end

function PositionTracker:trackRotation()
    self.trackRot = true;
end

function PositionTracker:onActorDestroyed(name)
    -- Stop updating if our tracking target is destroyed
    if (self.trackingActor.name == name) then
        -- self.trackingActor = nil
        self.active = false;
    end
end

function PositionTracker:onLateUpdate()
    if (self.active and self.trackingActor) then
        local trans = self.trackingActor
        if (self.boneName) then
            trans = self.trackingActor:getBoneWorld(self.boneName)
        end
        if (trans == nil) then
            trans = self.trackingActor:getLocal()
        end

        if (trans) then
            local goalPos = nil
            if (self.onlyYAxis) then
                goalPos = Vector(0, trans.position.y, 0)
            else
                goalPos = trans.position
            end


			
            self.object:setTranslation(goalPos)
			
			if (self.trackRot) then
				local trackingRot = self.trackingActor:getWorld()
				self.object:setRotation(trackingRot.rotation)
			end
        end
    end
end
