
local Logger = TaggedLog("ScreenFader", false)

ScreenFader = {
    -- configurable params
    matName = "Mat_ScreenFade",
    fromColor = Vector(0, 0, 0, 1),
    toColor = Vector(0, 0, 0, 0),
    delay = 0,
    speed = 2,
    onComplete = nil,
    fadeOnStart = false,

    -- internal
    material = nil,
    timer = 0,
    startColor = Vector(0, 0, 0, 1),
    endColor = Vector(0, 0, 0, 0),
}

function ScreenFader:new(params)
    local newObj = shallowcopy(params) or {}
    setmetatable(newObj, self)
    self.__index = self

    return newObj
end

function ScreenFader:onStart()
    self.material = self.object:findMaterial(self.matName)

    -- parent our game object to the camera so that it is always covers the screen
    local screenSpace = Global.scene.screenSpace
    
    if (not screenSpace) then
        Log.warning("Unable to find screen space object for screen fader")
        return
    end
    
    screenSpace:addChild(self.object)
    
    if (self.fadeOnStart) then
        self:fadeIn()
    end
end

function ScreenFader:onUpdate()
	if (self.timer < 1) then
        Global.AppDebugText1 = self.timer
		self.timer = self.timer + (Time.dt() * self.speed)
        local amount = math.easeInOutSine(math.clamp(self.timer, 0, 1))
        local matColor = Vector.lerp(amount, self.startColor, self.endColor)
        self.material:setParameters({ u_Color = matColor})
		if (self.timer >= 1) then
            if (self.onComplete) then self.onComplete() end
		end
	end
end

ScreenFader.fadeIn = function(self, delay, onComplete)
    self.delay = delay or self.delay
    self.onComplete = onComplete or self.onComplete

    self.timer = -self.delay * self.speed
	self.startColor = self.fromColor
	self.endColor = self.toColor
end

ScreenFader.fadeOut = function(self, delay, onComplete)
    self.delay = delay or self.delay
    self.onComplete = onComplete or self.onComplete

    self.timer = -self.delay * self.speed
    self.startColor = self.toColor
    self.endColor = self.fromColor
end
