
local Logger = TaggedLog("SecondaryAnimationControl", false)

SecondaryAnimationControl = {}

function SecondaryAnimationControl:new(params)
    local o = shallowcopy(params) or {}
    self.__index = self
    setmetatable(o, self)

    return o
end

function SecondaryAnimationControl:onStart()
    self.currentlyPlaying = 1
    self.triggeredThisSegment = false
    Events.listen(self, "segmentPlaying")
    self.trigger = Global.scene.actors[self.triggerName].components["LookAtDetector"]
    self:ResetAnim()
end

function SecondaryAnimationControl:ResetAnim() 
    self.object = status(self.object)

    if self.object then
        self.object.animation[self.idleAnim]:play()
    end
end

function SecondaryAnimationControl:onDestroy() 
    Events.stopListening(self, "segmentPlaying")
end 

function SecondaryAnimationControl:segmentPlaying(segmentName)
    self:ResetAnim()
    self.triggeredThisSegment = false
end

function SecondaryAnimationControl:onUpdate()
    if (self.trigger:lookedAt()) then
        if not self.triggeredThisSegment then
            self.triggeredThisSegment = true

            -- Play the current animation in the cycle
            local currentlyPlaying = self.currentlyPlaying
            local animation = self.animations[currentlyPlaying]
            local sound = animation.sound
            Logger:message("Playing animation: " .. animation.name)
            self.object.animation[animation.name]:play()
            if sound then
                Delay:runAfter(sound.delay, function()
                    Logger:message("Playing sound: " .. sound.name)
                    SoundManager:playSound(sound.name, sound.volume or 1, sound.looping, sound.type)
                end)
            end

            -- Set up for the next animation
            self.currentlyPlaying = self.currentlyPlaying + 1
            if self.currentlyPlaying > #self.animations then
                self.currentlyPlaying = 1
            end
        end
    end 
end
