

SceneData.lights.Sun = {
	type		= LT_DIRECTIONAL,
	wrap		= 1.0,
}

SceneData.lights.SpotLight = {
	type        = LT_SPOT,
	range       = { 2.0, 70.0 },	-- hard coded in lighting.rsfx
	spotAngle   = { 15.0, 20.0 },	-- hard coded in lighting.rsfx
	wrap        = 1.0,
}

SceneData.lights.FillLight = {
	type        = LT_SPOT,
	range       = { 0.0, 60.0 },	-- hard coded in lighting.rsfx
	spotAngle   = { 15, 22.0 },		-- hard coded in lighting.rsfx
	wrap        = 1.0,
}

SpotLightControl = {
    lightColor = Vector.new( 2.5, 2.5, 2.5, 1.0 ),
    
    ambientColor = Vector.new( 2.0, 2.0, 2.0, 1.0),
    clearColor = Vector.new( 1.0, 1.0, 1.0, 1.0 ),
    blackColor = Vector.new( 0.0, 0.0, 0.0, 1.0 ),
    darkColor = Vector.new( 0.1, 0.1, 0.15, 1.0 ),
    fillColor = Vector.new( 0.2, 0.2, 0.2, 1.0 ),
    redColor = Vector.new( 2, 0.2, 0.2, 1.0 ),
    
    nearRadius = 0,
    farRadius = 0,
    timer = 0,
	lightOffset = Vector.new( 0.0, 0.0, 0.9 ),
	lightLookAt = Vector.new( 0.0, 0.0, -100.0 ),
}

function SpotLightControl:new(amount)
	local this = {
		amount = amount,
	}
	SpotLightControl:createSun()
	SpotLightControl:createSpotlight()
	SpotLightControl:createFillLight()
	setmetatable(this, self)
	self.__index = self
	return this
end

function SpotLightControl:onStart()
	self.timer = 0
	Events.listen(self, "segmentPlaying")
end

function SpotLightControl:createSun()
	self.sun = Global.scene:createLight("SunLight", SceneData.lights.Sun)
	
	local q = Quaternion.new()
	q:setAxisAngle( Vector.new( 1.0, 0.0, 0.0 ), math.pi*-0.4 ) 
	self.sun:setLocal({ rotation = q })
end

function SpotLightControl:createSpotlight()
	self.spotlight = Global.scene:createLight("SpotLight", SceneData.lights.SpotLight)

	local m = Matrix.lookAt( self.lightOffset, self.lightLookAt, Vector.up )
	local q = Quaternion.new()
	q:setMatrix( m )

	self.lightOffsetTransform = SRT.new( self.lightOffset, q )
end

function SpotLightControl:createFillLight()
	self.fillLight = Global.scene:createLight("FillLight", SceneData.lights.FillLight)

	local m = Matrix.lookAt( self.lightOffset, self.lightLookAt, Vector.up )
	local q = Quaternion.new()
	q:setMatrix( m )

end

function SpotLightControl:onDestroy() 
	Events.stopListening(self, "segmentPlaying")
    Global.AppDebugText2 = "Destroying Lights"
	if(status(self.sun) ~= nil) then
		self.sun:destroy()
	end
	if(status(self.spotlight) ~= nil) then
		self.spotlight:destroy();
	end
end 

function SpotLightControl:segmentPlaying(segmentName)
	if (segmentName == "EndC") then
		self:fadeLightToBlack(2.8)
	end
end

function SpotLightControl:onUpdate()
	local lightColor
	local nightColor 

	local camera = Global.scene.mainCamera:getWorld()
	local lookat = Matrix.lookAt( Vector.zero, camera:forward(), Vector.up )
	lookat:invert()

	camera.rotation:setMatrix( lookat )

	local transform = self.lightOffsetTransform * camera
	self.spotlight:setLocal( transform )
	self.fillLight:setLocal( transform )

	if (self.timer < 0) then
		self.timer = self.timer + Time.dt() * 1
	else 
		if (self.clearToBlack and self.timer < 1) then
			self.timer = self.timer + Time.dt() * 1
			Global.clearColor:set(0, 0, 0, math.clamp01(self.timer))
            self.spotlight:setDiffuse(Vector.lerp(math.clamp01(self.timer), self.lightColor, self.blackColor))
		end
		
		if (self.blackToLight and self.timer < 1) then
			self.timer = self.timer + Time.dt() * 1
			
            self.spotlight:setDiffuse(self.lightColor)
            self.fillLight:setDiffuse(Vector.lerp(math.clamp01(self.timer), self.blackColor, self.fillColor))
            self.sun:setDiffuse(Vector.lerp(math.clamp01(self.timer), self.blackColor, self.darkColor))
	   end

		if (self.lightToBlack and self.timer < 1) then
			self.timer = self.timer + Time.dt() * 1
			local newColor = Vector.lerp(math.clamp01(self.timer), self.darkColor, self.blackColor)
            self.spotlight:setDiffuse(newColor)
            self.fillLight:setDiffuse(Vector.lerp(math.clamp01(self.timer), self.fillColor, self.blackColor))
            self.sun:setDiffuse(newColor)
		end

		if (self.blackToClear and self.timer < 1) then
			Global.clearColor:set(0, 0, 0, math.clamp01(1 - self.timer))
			self.timer = self.timer + Time.dt() * 1
            self.spotlight:setDiffuse(Vector.lerp(math.clamp01(self.timer), self.blackColor, self.lightColor))
		end
	end
end

SpotLightControl.fadeClearToBlack = function(self, delay)

	self.timer = -delay
	self.clearToBlack = true
	self.blackToLight = false
	self.lightToBlack = false
	self.blackToClear = false
end

SpotLightControl.fadeBlackToLight = function(self, delay)
         
    self.timer = -delay
    self.clearToBlack = false
    
    self.blackToLight = true
    self.lightToBlack = false
    self.blackToClear = false
    self.spotlight:setSpotAngle(14.5, 15.00);

    SoundManager:prefetch("light_switch", Global.resources)
    Delay:runAfter(delay, function() SoundManager:playSFX("light_switch", 1, false) end)
end

SpotLightControl.fadeLightToBlack = function(self, delay)
	self.timer = -delay
	self.clearToBlack = false
	self.blackToLight = false
	self.lightToBlack = true
	self.blackToClear = false
end

SpotLightControl.fadeBlackToClear = function(self, delay)

	self.timer = -delay
	self.clearToBlack = false
	self.blackToLight = false
	self.lightToBlack = false
	self.blackToClear = true
    self.spotlight:setSpotAngle(80, 90);
end

SpotLightControl.setToClear = function(self)

    self.timer = 0
    self.clearToBlack = false
    self.blackToLight = false
    self.lightToBlack = false
    self.blackToClear = false
    self.sun:setDiffuse(self.blackColor)
    self.fillLight:setDiffuse(self.blackColor) 

    self.spotlight:setDiffuse(self.lightColor) 
    self.spotlight:setSpotAngle(80, 90);
end

