
SquirrelLightDriver  = {}
SquirrelLightDriver.__index = SquirrelLightDriver

function SquirrelLightDriver:new(params)
	local o = shallowcopy(params) or {}

	setmetatable(o, self)
	self.__index = self
	self.debug = false

	return o
end

function SquirrelLightDriver:createDebugSphere( position, color )
	local sphere = Global.scene:createActor("light_debugsphere", SceneData.actors.DebugSphere)
	sphere:setTranslation( position )
	sphere:setScale( Vector.new( 0.125, 0.125, 0.125 ) )
	sphere.materials.Mat_Sphere:setParameter("u_diffuseColor", color )

	self.track:addChild( sphere )
end

function SquirrelLightDriver:onStart()
	self.trackingActor = ActorManager:getActor(self.trackingActor)
	self.light = Global.scene:createLight(self.lightName, self.lightDef)	
	self.light:setTranslation( self.lightPosition )
	
	self.track = Global.scene:createTransform( self.lightName.."Tracker" )
	self.track:addChild( self.light )

	if self.lightTarget then
		--local dir = self.lightTarget - self.lightPosition
		local dir = self.lightPosition - self.lightTarget
		dir:normalize()
		
		self.light:setRotation( Quaternion.lookAt( dir, Vector.up ) )
	end
	
	if self.debug then
		self:createDebugSphere( Vector.zero, Vector.new( 1.0, 0.0, 0.0, 0.25 ) )

		if self.lightPosition then
			self:createDebugSphere( self.lightPosition, Vector.new( 0.0, 1.0, 0.0, 0.25 ) )
		end

		if self.lightTarget then
			self:createDebugSphere( self.lightTarget, Vector.new( 0.0, 0.0, 1.0, 0.25 ) )
		end
	end
end

function SquirrelLightDriver:onDestroy()
	self.light:destroy()
end

function SquirrelLightDriver:onUpdate()
	local bonePos = self.trackingActor:getBoneWorld(self.trackingBone)
	self.track:setLocal( bonePos )

	local lightSwitch = self.trackingActor:getBoneLocal(self.trackingTarget)
	local lightSwitchPos = lightSwitch.position.y
	local switchAmount = math.inverseLerp(self.switchRange.x, self.switchRange.y, lightSwitchPos);

	local range0 = math.lerp( self.lightStartRange[1], self.lightEndRange[1], switchAmount )
	local range1 = math.lerp( self.lightStartRange[2], self.lightEndRange[2], switchAmount )

	self.light:setRange( range0, range1 )
end
