
-- Simple class to wrap the actor bone APIs into a single object
Bone = {}

function Bone:new(actor, bonePath)
    local this = { actor = actor, bonePath = bonePath }

    setmetatable(this, self)
    self.__index = self
    return this
end

function Bone:getWorld()
    return self.actor:getBoneWorld(self.bonePath)
end

function Bone:getLocal()
    return self.actor:getBoneLocal(self.bonePath)
end

function Bone:setLocal(transform)
    self.actor:setBoneLocal(self.bonePath, transform)
end

function Bone:setLocalTranslation(position)
    local transform = self:getLocal()
    transform.position = position
    self.actor:setBoneLocal(self.bonePath, transform)
end

function Bone:setLocalScale(scale)
    local transform = self:getLocal()
    transform.scale = scale
    self.actor:setBoneLocal(self.bonePath, transform)
end

function Bone:setLocalRotation(rotation)
    local transform = self:getLocal()
    transform.rotation = rotation
    self.actor:setBoneLocal(self.bonePath, transform)
end

function Bone:setVisibility(visible)
    self.actor:setBoneVisibility(self.bonePath, visible)
end

-- Patch in a getBone() to the actor class to all direct access from the actor itself
function Actor:getBone(bonePath)
    return Bone:new(self, bonePath)
end