
Global = {}
Global.resources = System.createResourceGroup("global")

require "common/scripts/moxie/moxie"
require "common/scripts/moxie/replay"
require "common/scripts/moxie/toolbox/toolbox"

require "buggynight/scripts/fixup/fixup"
require "buggynight/scripts/fixup/logger"
require "buggynight/scripts/fixup/events"
require "buggynight/scripts/fixup/bone"
require "buggynight/scripts/util"
require "buggynight/scripts/views"

-- data
require "buggynight/scripts/scenedata/actors"
require "buggynight/scripts/scenedata/sets"
require "buggynight/scripts/scenedata/triggers"
require "buggynight/scripts/scenedata/segments"
require "buggynight/scripts/scenedata/audio"

-- components
require "buggynight/scripts/components/animationdriver"
require "buggynight/scripts/components/camera/cameracontroller"
require "buggynight/scripts/components/camera/lockcameraactor"
require "buggynight/scripts/components/camera/debugcamera"
require "buggynight/scripts/components/cameramodifier"
require "buggynight/scripts/components/locator"
require "buggynight/scripts/components/lookatdetector"
require "buggynight/scripts/components/lookatdetectorcore"
require "buggynight/scripts/components/screenfader"
require "buggynight/scripts/components/simpleparticles"
require "buggynight/scripts/components/positiontracker"
require "buggynight/scripts/components/floatydriver"
require "buggynight/scripts/components/textureanimator"
require "buggynight/scripts/components/textureblender"
require "buggynight/scripts/components/spotlightcontrol"
require "buggynight/scripts/components/lighttracker"
require "buggynight/scripts/components/coloranimator"
require "buggynight/scripts/components/secondaryanimationcontrol"
require "buggynight/scripts/components/squirrellightdriver"
require "buggynight/scripts/components/footprintfader"

-- controllers/managers
require "buggynight/scripts/manager"
require "buggynight/scripts/actormanager"
require "buggynight/scripts/endofshow"
require "buggynight/scripts/inputmanager"
require "buggynight/scripts/delay"
require "buggynight/scripts/segment"
require "buggynight/scripts/segmentmanager"

-- audio
require "buggynight/scripts/audio/soundmanager"
require "buggynight/scripts/audio/soundplayer"
require "buggynight/scripts/audio/soundcontrol"

-- stats
require "buggynight/scripts/stats" -- please to not require anything after this file.

local Logger = TaggedLog("Global", false)

Global.screen_size      = Vector(384, 640, 0, 0)
Global.avail_size       = Vector(384, 640, 0, 0)
Global.clearColor       = Color(0, 0, 0, 0)
Global.launcherActive   = false
Global.lockHeading      = false
--Global.useSensor        = true --moved to intro.lua

Global.worldRot             = 0
Global.worldRotCompensation = 0
Global.frame 				= 0

if (Platform.candidate) then
	Global.showFPS          = false
	Global.debugRender      = false
	Global.renderOverTSB    = true
	Toolbox.enabled         = false
end

if (Platform.debug or Platform.release) then
	Global.showFPS          = false
	Global.debugRender      = true
	Global.renderOverTSB    = true
end

if (Platform.branch == "R") then
	Global.showFPS          = false
	Global.debugRender      = false
	Toolbox.enabled         = false
end

local toolboxInitialized = false

Global.storyManager = nil

Global.init = function()
    math.randomseed(os.time())

    if StoryManager then
        Global.storyManager = StoryManager:getFSM("buggynight")
    end

	Global.scene = Scene.create()

	InputManager:init(Global.scene)
	Android.setAlphaRange(0, 1)

	Delay:init(Global.scene)
	SoundManager:init(Global.scene)
	SoundPlayer:init(Global.scene)
	ActorManager:init(Global.scene)

	Global.scene.cameraRig  = Global.scene:createTransform("cameraRig")
	Global.scene.mainCamera = Global.scene:createCamera("camera001")

	Global.scene.cameraRig:addChild(Global.scene.mainCamera)
	Global.scene.mainCamera:setPerspective(35 * (1184 / 1280), 0.9, 100) -- match long axis to unity's aspect
	Global.scene.cameraRig:setPosition(Vector(0, 2.5, 0))
	Global.scene.cameraRig.boomLength = 1
	Global.scene.mainCamera.controller = Global.scene.mainCamera:addComponent("cameraControl", CameraController:new())
	
	AnalyticsManager.initialize()
	AnalyticsManager.sendInitialStats()

	if(Platform.win32 or Platform.mac) then
		Global.scene.mainCamera.debugcontroller = Global.scene.mainCamera:addComponent("debugcamera", DebugCameraController)
	end
	
	mainView:setCamera( Global.scene.mainCamera )

	-- create a transform for screen space items
	local screenSpace = Global.scene:createTransform("ScreenSpace")
	Global.scene.mainCamera:addChild(screenSpace)
	local screenSpaceSRT = screenSpace:getLocal()
	screenSpaceSRT.rotation = Quaternion.fromEulerAngles(Vector(0, 0, 0))
	screenSpaceSRT.position = Vector(0, 0, -5)
	screenSpace:setLocal(screenSpaceSRT)
	Global.scene.screenSpace = screenSpace

	-- Common
	Global.resources:addTexture( "common:textures/missingtexture.dds" )
	Global.resources:addTexture( "buggynight:textures/firefly.dds" )
	Global.resources:prefetch()


    local startScene = Platform.scene
    if startScene ~= nil and startScene ~= "" then
        Context.onCheckpoint(startScene)
    end

    RenderDebug.setEnabled(true)
end

Global.initShow = function()
--    AsyncLoader.generateStreamingData()

    SegmentManager:init(Global.scene)
end

Global.initSpotLight = function()   
    local controlRoot = Global.scene:createActor("GlobalControllers", LaunchData.actors.LauncherBug)
    controlRoot:setVisibility(false)
    local spotLightController = controlRoot:addComponent("SpotLightController", SpotLightControl:new(0.2))
    spotLightController:setToClear()

    controlRoot:addComponent("CameraModifier", CameraModifier:new())
end

Global.startShow = function()
	-- Check for HitchProfiler before calling it
	if(HitchProfiler ~= nil) then
		HitchProfiler.resetFrameCounter()
	end
	
	AnalyticsManager:onShowBegan()

	Global.startShowTime = os.clock()
	Global.setRenderOverTSB(false)
	
	-- we are finally displaying the show, take control of the audio stream
	JavaAudio.requestFocus(1)

	Replay:init(Global.scene.mainCamera);
end

Global.onInterrupted = function()
    AnalyticsManager:userLeft()
    AnalyticsManager:onShowEnded()
end	

Global.showEnded = function()
    Global.endShowTime = os.clock()

    -- kind of a name mismatch but this function is indeed only invoked on the proper completion 
    -- of the show
    AnalyticsManager:showComplete()
end

Global.onShutdown = function()
	if (Global.scene) then
		Global.scene:destroy()
		Global.scene = nil
	end

	if (mainView) then
		mainView:destroy()
		mainView = nil
    end

	if Global.resources then
		Global.resources:destroy()
	end

    if Global.storyManager then
        Global.storyManager:destroy()
        Global.storyManager = nil
    end

    AsyncLoader.shutdown()
	System.gc()
end

Global.onUpdate = function()
	if (Global.scene == nil) then return end
	
--	AsyncLoader:cleanup();

	InputManager:update()

	ActorManager:preCreateUpdate()

	AnalyticsManager:update()

    if toolboxInitialized then
        Toolbox:update()
    end

    if (Global.scene) then
		Global.scene:update()
	end

	if (Input.isKeyDown(KB_F12, true)) then
		mainView:saveScreenshot()
	end  
	
	if(System.isStatManagerEnabled()) then
		Stat.updateAll();
	end
end

Global.drawInfo = function()
	local textScale = 1.0
	if (Platform.win32 or Platform.mac) then
		textScale = 0.5
	end

	local y = 10
	local yStep = 30 * textScale
	local nextLine = function()
		y = y + yStep
		return y - yStep
	end
	if (Global.debugRender) then
		-- TODO: refactor this segment info out of Global
		local segmentName = SegmentManager:currentSegmentName()
		if (segmentName == "") then
			segmentName = "gwi"
		end
		local title = "[" .. Platform.package .. " | "..segmentName.."]"
		local fpsString = "["..FPS.string().."]"
		local osdEnabled = "No OSD"
		if ( OSD and OSD.enabled ) then
			osdEnabled = "OSD"
		end
		local versionString = "["..System.versionString().."]"

		local textStyle = { color = Color.new(0.8, 0.8, 0.8), font = "common:fonts/arial_default_24.fnt", scale = textScale }
		Renderer.drawText(10, nextLine(), textStyle, title)
		Renderer.drawText(10, nextLine(), textStyle, versionString)
		Renderer.drawText(10, nextLine(), textStyle, "[time: "..Time.string().."]")
		Renderer.drawText(10, nextLine(), textStyle, "["..segmentName.."]")
		Renderer.drawText(10, nextLine(), textStyle, fpsString)
        nextLine()
        Renderer.drawText(10, nextLine(), textStyle, Global.AppDebugText1 or "")
        Renderer.drawText(10, nextLine(), textStyle, Global.AppDebugText2 or "")
        Renderer.drawText(10, nextLine(), textStyle, Global.AppDebugText3 or "")

		if (Memory) then
			local mbs = Memory.numBytesAllocated()/1048576.0
			local blocks = Memory.numBlocksAllocated()
			Renderer.drawText(10, nextLine(), textStyle, "[mem: "..string.format("%.2f", mbs).." | #:"..blocks.."]")
		end

		if (Platform.record and Platform.record ~= "") then
			textStyle = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
			Renderer.drawText(10, nextLine(), textStyle, "[record: "..Platform.record.."]")
		end

		if (Platform.replay and Platform.replay ~= "") then
			--			textStyle = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
			Renderer.drawText(10, nextLine(), textStyle, "[replay: "..Platform.replay.."]".."[x"..string.format("%.2f", Time.timescale()).."]")
		end
	end

	if (Global.showFPS) then
		local textStyle = { color = Color.new(1.0, 1.0, 1.0), font = "common:fonts/arial_default_24.fnt", scale = 2.0 }
		local fpsString = ""..string.format("%.2f fps", FPS.average())..""
		if Global.startShowTime ~= nil then
			local time = os.clock() - Global.startShowTime
			if Global.endShowTime ~= nil then
				time = Global.endShowTime - Global.startShowTime
			else
				Global.frame = Global.frame + 1
			end
			--fpsString = fpsString..", "..string.format("%0.2f", time).." s, "..tostring(Global.frame).." frames"
		end
		Renderer.drawText(10, nextLine(), textStyle, fpsString)
	end
end

Global.onReshape = function(width, height)
	Log.message("Reshape: "..tostring(width)..", "..tostring(height));
	mainView:resize( width, height )

    if not toolboxInitialized then
        Log.message("Initializing toolbox size = " .. tostring(width) .. "x" .. tostring(height))
        Toolbox:init()
        Toolbox:hide()
        toolboxInitialized = true
    end

    Events.send("renderer", "onReshape", Vector(width, height, 0, 0))
end

Global.onRender = function()
	mainView:setClearColor( Global.clearColor )
	Global.scene:render( mainView )
	Global:drawInfo()

	Renderer.display( mainView )
end

Global.updateScreenSize = function()
	Logger:message("updating screen size: " .. Global.screen_size.x .. ", " .. Global.screen_size.y)
	if (Platform.android) then
		if (Global.renderOverTSB) then
			-- Use the whole screen
			mainView:setClipRect(0, 0, Global.screen_size.x, Global.screen_size.y)
		else
			-- Clip out the TSB area
			-- mainView:setClipRect(0, Global.screen_size.y - Global.avail_size.y, Global.avail_size.x, Global.avail_size.y)
		end
	end
end

Global.setRenderOverTSB = function(render)
	Global.renderOverTSB = render
	Global.updateScreenSize()
end
