
Launcher.intro = {

    bugReady = false,

    onStartup = function(self)

        Android.acquireScreen()

        ActorManager:startPreCreate()        

        self.interrupted = Context.isBlacklisted()
        self.crossFadeAnimation = true
        InputManager:enableSensors(true)

        -- analytics
        AnalyticsManager:setAppAtTimeOfPrompt()

        -- set up visuals
        local launcherRig = Global.scene.actors.LauncherRig
        local launcherBug = Global.scene.actors.LauncherBug

        local prompt = Context.getPromptType()

        local moverTransitionDelay = 0.0
        local fadeToBlackDelay = 0.0
        local sensorDelayOffset = 4.0

        -- GWI TODO - fast-way-in end animations aren't stitching correctly into the show
        if (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART or prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE) then

            self.animationName = "ReplayEnd"
            fadeToBlackDelay = 0
            SoundManager:playSFX("00launch_jump", 1, false)
            moverTransitionDelay = 1.46
            Android.onFadeInForest()
        elseif (prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE) then
            self.animationName = "ContinueEnd"
            moverTransitionDelay = 0.92
            fadeToBlackDelay = 0
            sensorDelayOffset = 0
            self.crossFadeAnimation = false
            Android.onFadeInForest()
        else
            self.animationName = "Intro"
            moverTransitionDelay = 2.4
            fadeToBlackDelay = 1.25
            SoundManager:playSFX("00launch_82_take_jump_land_look", 1, false)
            Android.reset()
        end

        local whiteFadeDelay = 2.0
        local launcherBugStartDuration = launcherBug.animation[self.animationName]:getDuration()
        if self.crossFadeAnimation then
            launcherBug.components.LauncherBugMover:startTransition(launcherBugStartDuration, moverTransitionDelay)
        end

        launcherBug:setVisibility(true)
        launcherBug.animation[self.animationName]:play()
        launcherRig.animation[self.animationName]:play()
        Delay:runAfter(launcherBugStartDuration - 0.1, function() self.bugReady = true end)
        Delay:runAfter(launcherBugStartDuration + sensorDelayOffset, function() Global.lockHeading = false end)


        self.TSBClipTimer       = whiteFadeDelay
        
        Global.lockHeading      = true
        Global.scene.mainCamera.controller:UseSensor()

        local spotLightControl = Global.scene.actors.GlobalControllers
        
        spotLightControl.components["SpotLightController"]:fadeClearToBlack(fadeToBlackDelay)
        
        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()
        local prompt = Context.getPromptType()

        if (self.interrupted or Global.shutdown) then
            return "launcher_interrupted"
        end

        if (not self.TSBClipped) then
            self.TSBClipTimer = self.TSBClipTimer - Time.dt()
            if (self.TSBClipTimer < 0) then
                if (Platform.android or Platform.iOS) then
                    -- mainView:setClipRect(0, Global.screen_size.y - Global.avail_size.y, Global.avail_size.x, Global.avail_size.y)
                end
                self.TSBClipped = true
            end
        end

        if (self.bugReady) then
            return "buggynight"
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onRender = function(self)
        if (not Android.isPaused()) then
            Global.onRender()
        end
    end,

    onShutdown = function(self)
        Global.scene.mainCamera.controller:releaseSoftClamp()
        if self.crossFadeAnimation then
            Global.scene.actors.LauncherBug.animation.IntroLocal:crossfade(0.25, false)
            local introRig = ActorManager:getActor("IntroRig", true)
            introRig.animation.Intro:play()
            introRig.animation.Intro:setPaused(false)
        else
            Global.scene.actors.LauncherBug:destroy()
        end
    end,

    onEvent = function(self, event)
        if(event.type == "onActivityEvent") then
            if (event.data.eventString == "onDummyActivityGainedFocus") then
                Android.onResume()
            elseif (event.data.eventString == "onDummyActivityLostFocus") then
                Android.onPause()
            elseif (event.data.eventString == "onStop") then
                self.interrupted = true
                Android.onResume()
            end
        elseif(event.type == "onChangeDisplaySize") then
            Global.setRenderOverTSB(false)
        elseif(event.type == "onBatteryEvent") then
            if (event.data.newState == 0) then
                self.interrupted = true
                Android.onResume()
            end
        end
    end
}
