
LauncherBugMover = {}
LauncherBugMover.__index = LauncherBugMover;

LauncherBugMover.new = function(newStartTarget, newStartTargetTransform, newEndTarget, newEndTargetTransform, startScale, endScale)
    local self = {
        waiting = true,
        ready = false,
        startTarget = newStartTarget,
        startTargetTransform = newStartTargetTransform,
        endTarget = newEndTarget,
        endTargetTransform = newEndTargetTransform,
        startScale = startScale or 1,
        endScale = endScale or 1,
        transitionActive = false,
        transTimer = 0,
        transDuration = 1,
    }
    return setmetatable(self, LauncherBugMover)
end

LauncherBugMover.onStart = function(self)
end

LauncherBugMover.onLateUpdate = function(self)

    if self.transitionActive then
        self.transTimer = self.transTimer + Time.dt()
    end

    local startSRT = self.startTarget:getBoneWorld(self.startTargetTransform)
    local endSRT = self.endTarget:getBoneWorld(self.endTargetTransform)
    if (endSRT == nil) then return end
    
    local finalSRT = self.object:getLocal()
    local amount = math.clamp01(self.transTimer / self.transDuration)
    finalSRT.position = Vector.lerp(amount, startSRT.position, endSRT.position)
    finalSRT.rotation = Quaternion.slerp(amount, startSRT.rotation, endSRT.rotation)

    self.object:setLocal(finalSRT)
end

LauncherBugMover.startTransition = function(self, duration, startDelay)
    self.transitionActive = true
    self.transDuration = duration - startDelay
    self.transTimer = -startDelay
end

LauncherBugMover.isReady = function(self)
    return self.ready
end
