
require "buggynight/scripts/launcher/load"

Launcher.load1 = {
    -- whether or not we have already passed through this state
    loaded = false,

    -- next state after loading completes
    nextState = nil,

    -- whether or not to wait for a screen cycle
    doIdle = false,
    doIdleOnNotification = false,

    onStartup = function(self)

        Android.reset()

        -- choose the next state
        self.nextState = nil
        self.doIdle = false
        self.doIdleOnNotification = false
        
        local prompt = Context.getPromptType()

        if (prompt == PROMPT_HAT_FLOURISH_NOTIFICATION) then
            self.nextState = "launcher_notify"
        elseif (prompt == PROMPT_HAT_FLOURISH_NOTIFICATION_IDLE) then
            self.nextState = "launcher_notify"
            self.doIdle = true
        elseif (not Context.bypassIdle() and (prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE or prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE)) then
            self.nextState = "launcher_load2"
            self.doIdle = true
            self.doIdleOnNotification = true
        else
            self.nextState = "launcher_load2"
        end

        -- Initialize the scene & camera and start prefetching of the hat assets
        self:prefetch()

        -- tick continuously
        return true
    end,

    onUpdate = function(self)
        -- go to next state when loading completes
        if (self.loaded or System.getPendingResourceCount() == 0) then
            self:setupScene()
            self.loaded = true

            if (self.doIdle) then
                Launcher.idle.nextState = self.nextState
                Launcher.idle.idleOnNotification = self.doIdleOnNotification
                return "launcher_idle"
            else
                return self.nextState
            end
        end
    end,

    onRender = function(self)
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    prefetch = function(self)
        if (self.loaded) then return end

        Global.resources:addActor(LaunchData.actors.LauncherBug)
        Global.resources:addActor(LaunchData.actors.LauncherRig)
        Global.resources:addActor(SceneData.actors.IntroRig)
        Global.resources:prefetch()
        Global.init()

        if StoryManager and StoryManager.pauseStory then
            StoryManager.pauseStory()
        end
    end,

    setupScene = function(self)
        if (self.loaded) then return end

        Global.initSpotLight()

        local launcherRig = Global.scene:createActor("LauncherRig", LaunchData.actors.LauncherRig)
        launcherRig:setVisibility(false)
        Global.scene.screenSpace:addChild(launcherRig)

        -- Use the ActorManager for the intro rig as it needs to have an animDriver to work correctly in the intro segment
        local introRig = ActorManager:getActor("IntroRig", true)
        local anim = introRig.animDriver:init("Intro", introRig.animation, false)
        anim:play()
        anim:setPaused(true)
        anim:setTime(0)
        introRig:setVisibility(false)

        local launcherbug = Global.scene:createActor("LauncherBug", LaunchData.actors.LauncherBug)
        launcherbug:addComponent("LauncherBugMover", LauncherBugMover.new(launcherRig, "BugPosition|RootNode", introRig, "BugPosition|RootNode"))
    end
}
