
Launcher.load2 = {
    -- true to bypass screen idle for the initial prompt
    skipFirstIdle = (System.getEnv == nil) or Platform.win32 or Platform.mac or (System.getEnv("DevMode") == "true") or (Platform.primeTime ~= nil and not Platform.primeTime),

    -- whether or not we have already passed through this state
    loaded = false,


    -- next state after loading completes
    nextState = nil,

    -- whether or not to wait for a screen cycle
    doIdle = false,

    onStartup = function(self)

        Android.reset()        

        if StoryManager and StoryManager.resumeStory then
            StoryManager.resumeStory()
        end

        -- choose the next state
        self.nextState = nil
        self.doIdle = false
        self.finishedIntro = true
        self.animation = nil

        Global.lockHeading      = true

        local prompt = Context.getPromptType()

        if (prompt == PROMPT_HAT_ON_TSB) then
            self.nextState = "launcher_prompt"
            self.doIdle = not self.skipFirstIdle
        elseif (prompt == PROMPT_HAT_UNHIDE) then
            self.nextState = "launcher_fast_prompt"
            self.doIdle = true
        elseif (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART
            or prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE
            or prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE) then

            if (prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE) then                
                self.startAnimation = "ContinueStart"
                self.loopAnimation = "ContinueLoop"
            else
                self.startAnimation = "ReplayStart"
                self.loopAnimation = "ReplayLoop"
            end

            Android.acquireScreen()
            InputManager:enableSensors(true)
            
            self.nextState = "launcher_intro"
            self.animation = self.startAnimation            
            self.finishedIntro = false
            self:playAnimation()

            if (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART) then
                Avatar.cancelNotification()
                Context.onCheckpoint("")
            end
        else
            Log.fatal("Reached launcher_load2 in inapplicable scenario")
        end

        -- start loading assets in the background
        self:prefetch()

        self.interrupted = Context.isBlacklisted()
        self.exiting = false
        self.skipFirstIdle = false

        -- tick continuously
        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()

        if (self.exiting) then
            -- we are done loading and moving that bug off screen
            self.loaded = true

            -- go to next state
            if (self.doIdle) then
                Launcher.idle.nextState = self.nextState
                Launcher.idle.idleOnNotification = false
                return "launcher_idle"
            else
                return self.nextState
            end
        elseif ( self.finishedIntro) then
            self.exiting = true
        end

        if (self.animation and (self.interrupted or Global.shutdown)) then
            return "launcher_interrupted"
        end

        if (self.animation and not self:animationPlaying()  ) then
            self.animation = self.loopAnimation
            self:playAnimation()
            if (self.loaded or System.getPendingResourceCount() == 0) then
                self.finishedIntro = true
            end
        end
    end,

    onRender = function(self)
        if (not (Android.isPaused() or self.interrupted or not self.animation)) then
            Global.onRender()
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onShutdown = function(self)
    end,

    onEvent = function(self, event)
        if (self.animation) then
            if(event.type == "onActivityEvent") then
                if (event.data.eventString == "onDummyActivityGainedFocus") then
                    Android.onResume()
                elseif (event.data.eventString == "onDummyActivityLostFocus") then
                    Android.onPause()
                elseif (event.data.eventString == "onStop") then
                    self.interrupted = true
                    Android.onResume()
                end
            elseif(event.type == "onChangeDisplaySize") then
                Global.setRenderOverTSB(false)
            end
        end
    end,

    prefetch = function(self)
        if (self.loaded) then return end
        Global.resources:addAudio("buggynight:audio/00launch_82_take_jump_land_look.ogg")
        Global.resources:prefetch()

        local prompt = Context.getPromptType()
        if (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART
            or prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE
            or prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE) then
            Launcher.load2.prefetchAddl()
        end
    end,

    playAnimation = function(self)
        if (self.animation) then
            Global.scene.actors.LauncherBug.animation[self.animation]:play()
            Global.scene.actors.LauncherRig.animation[self.animation]:play()
        end
    end,

    animationPlaying = function(self)
        return self.animation and Global.scene.actors.LauncherBug.animation[self.animation]:isPlaying()
    end,

    prefetchAddl = function()
        -- prefetch misc audio files
        Global.resources:addAudio("buggynight:audio/light_switch.ogg")
        Global.resources:addAudio("buggynight:audio/ambience_loop.ogg")
        Global.resources:addAudio("buggynight:audio/distantfrogs_loop.ogg")
        Global.resources:addAudio("buggynight:audio/cricket1_startup.ogg")
        Global.resources:addAudio("buggynight:audio/cricket2_startup.ogg")
        Global.resources:addAudio("buggynight:audio/starry_night.ogg")
        Global.resources:addAudio("buggynight:audio/00launch_jump.ogg")
        Global.resources:addParticleSystem("buggynight:particles/firefliesforeground.mxp")
        Global.resources:addParticleSystem("buggynight:particles/firefliesbackground.mxp")
        Global.resources:prefetch()

        -- prefetch the first segment
        local checkPoint = Context.getCheckpoint()
        if (checkPoint) then
            AsyncLoader.prefetchForSegment(checkPoint)
        else
            AsyncLoader.prefetch()
        end
    end
}
