
require "buggynight/scripts/analyticsmanager"

Launcher.prompt = {
    new = function(self, animName)
        local this = { 
            animName = animName, 
            returning = false,
        }
        setmetatable(this, self)
        self.__index = self
        return this
    end,

    onStartup = function(self)
        Android.reset()

        -- prevent cloud service from killing us
        if (Avatar.setCanPreempt ~= nil) then
            Avatar.setCanPreempt(false)
        end

        -- record when hat is initially shown on the TSB
        if (Context.getPromptType() == PROMPT_HAT_ON_TSB) then
            Context.onTSBHatShown()
        end
        
        -- notify analytics we are showing the hat, plus which app was active at the time
        AnalyticsManager:GWIPromptBegun()

        -- set up visuals
        local launcherBug = Global.scene.actors.LauncherBug
        local launcherRig = Global.scene.actors.LauncherRig

        launcherBug:setVisibility(true)
        launcherBug.animation[self.animName]:play()
        launcherRig.animation[self.animName]:play()

        Global.lockHeading      = true

        Launcher.onHatPrompt()

        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()
        
        -- check if animation is done
        local launcherRig = Global.scene.actors.LauncherRig
        if (not launcherRig.animation[self.animName]:isPlaying()) then
            return "launcher_wait"
        end

        -- hide the hat immediately if we enter a blacklist state
        if (Context.isBlacklisted()) then
            return "launcher_hide"
        end

        -- long timeout
        if (Context.isTSBHatTimedOut()) then
            return "launcher_timeout"
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onRender = function(self)
        Global.onRender()
    end,

    onShutdown = function(self)
    end
}
