
-- A base class for manager singletons that need to be components of
-- something so that they can be part of the game loop (i.e. onUpdate(),
-- etc).  Calling init() on the manager subclass will setup a game object
-- and then attach the subclass as a component of it.
Manager = {}

Manager.new = function(self, params)
    local o = params or {}
    o.name = o.name or "Manager"

    setmetatable(o, self)
    self.__index = self

    return o
end

function Manager:init(scene)
    if (self.managerGameObj) then self:shutdown() end
    self.managerGameObj = scene:createTransform(self.name .. "GameObj")

    if (self.managerGameObj.components == nil or self.managerGameObj.components[self.name] == nil) then
        self.managerGameObj:addComponent(self.name, self)
    end
end

function Manager:shutdown()
    if (self.managerGameObj) then
        self.managerGameObj:destroy()
        self.managerGameObj = nil
    end
end
