
SceneData = SceneData or {}
SceneData.actors = SceneData.actors or {}
SceneData.lights = SceneData.lights or {}

-- Brian, if you need to copy paste one an animation manually, here is an example:
-- LogA        = { "buggynight:anims/annie_log.mxa", LogAStart, LogAEnd, false, true },

local function anim(animDef, animFileName)
    return {
        -- name, startFrame, endFrame, looping
        "buggynight:anims/" .. animFileName .. ".mxa", animDef.frames[1], animDef.frames[2], animDef.looping,
    }
end

local function animations(actorName, segments)
    local anims = {}
    for _, segment in ipairs(segments) do
        for animName, animData in pairs(segment.anims) do
            anims[animName] = anim(animData, actorName .. "_" .. segment.name)
        end
    end
    return anims
end

local IntroSegment = {
    name = "intro",
    anims = {
        Intro =      { frames = { 94, 170}, looping = false },
        IntroB =      { frames = { 171, 280}, looping = false },
    },
}

local RockSegment = {
    name = "rock",
    anims = {
        RockA =      { frames = {-56, 122}, looping = false },
        RockA_Idle = { frames = {-56, 122}, looping = true  },
        RockB =      { frames = {124, 346}, looping = false },
        RockB_Idle = { frames = {335, 346}, looping = true  },
        RockC =      { frames = {347, 432}, looping = false },
        RockC_Idle = { frames = {432, 432}, looping = true },
        RockD =      { frames = {434, 666}, looping = false },
    },
}

local LogSegment = {
    name = "log",
    anims = {
        LogA =      { frames = {438,  652}, looping = false },
        LogA_Idle = { frames = {438,  652}, looping = true },
        LogB =      { frames = {654,  856}, looping = false },
        LogB_Idle = { frames = {843,  856}, looping = true  },
        LogC =      { frames = {857, 933}, looping = false },
        LogC_Idle = { frames = {933,  933}, looping = true  },
        LogD =      { frames = {934, 1196}, looping = false },
    },
}

local TwigSegment = {
    name = "twig",
    anims = {
        TwigA =      { frames = {1151, 1382}, looping = false },
        TwigA_Idle = { frames = {1151, 1382}, looping = true },
        TwigB =      { frames = {1384, 1519}, looping = false },
        TwigB_Idle = { frames = {1517, 1536}, looping = true  },
        TwigC =      { frames = {1520, 1587}, looping = false },
        TwigC_Idle = { frames = {1587, 1587}, looping = true  },
        TwigD =      { frames = {1589, 1840}, looping = false },
    },
}

local SquirrelSegment = {
    name = "squirrel",
    anims = {
        SquirrelA =      { frames = {1626, 1872}, looping = false },
        SquirrelA_Idle = { frames = {1626, 1872}, looping = true },
        SquirrelB =      { frames = {1874, 2000}, looping = false },
        SquirrelB_Idle = { frames = {2000, 2000}, looping = true  },
        SquirrelC =      { frames = {2001, 2135}, looping = false },
        SquirrelC_Idle = { frames = {2135, 2135}, looping = true },
        SquirrelD =      { frames = {2136, 2191}, looping = false },
        SquirrelD_Idle = { frames = {2191, 2191}, looping = true },
        SquirrelE =      { frames = {2192, 2365}, looping = false },
    },
}

local FrogSegment = {
    name = "frog",
    anims = {
        FrogA =      { frames = {2370, 2578}, looping = false },
        FrogA_Idle = { frames = {2370, 2578}, looping = true },
        FrogB =      { frames = {2580, 2752}, looping = false },
        FrogB_Idle = { frames = {2752, 2752}, looping = true  },
        FrogC =      { frames = {2753, 3072}, looping = false },
        FrogC_Idle = { frames = {3072, 3072}, looping = true },
        FrogD =      { frames = {3074, 3263}, looping = false },
    },
}

local EndSegment = {
    name = "end",
    anims = {
        EndA =      { frames = {3179, 3444}, looping = false },
        EndA_Idle = { frames = {3446, 3445}, looping = true },
        EndB =      { frames = {3447, 3764}, looping = false },
        EndB_Idle = { frames = {3764, 3764}, looping = true },
        EndC =      { frames = {3766, 3923}, looping = false },
    },
}

local ThatsAllFolksSegment = {
    name = "end",
    anims = {
        ThatsAllFolks =      { frames = {3921, 3925}, looping = false },
    },
}

local BugLightColor = Color.new( 0.75, 0.75, 0.75, 1 )

local EyeParameters = {
	u_DarkColor = Color.new( 0.2, 0.2, 0.2, 1 ), 
	u_TexContrast = 0.3, 
	u_TexOffset = 0.3,
	u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
	u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
	u_MainTex_ST = Vector.new(1, 1, 0, 0), 
	u_Emissive = 1.0,
}

local SoundLocatorColor = Color(0, 0, 1, 0.2)
local pupilZOffset = -0.001

SceneData.actors.Annie = {
    model = "buggynight:models/annie_osd.mxm",
    animation = animations("annie", { RockSegment, LogSegment, TwigSegment, EndSegment }),
    targetBone = "Annie:bug1_mox_cn_upperbody|Annie:bug1_MOXIE_skeleton|RootNode",
    materials = {
		{
			name = "Annie:Mat_Bug", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING,
			textures = { u_MainTex = "buggynight:textures/bug01_diffuse.dds", }, 
			parameters = { 
				u_DarkColor = BugLightColor,
				u_TexContrast = 1.0, 
				u_TexOffset = -0.5, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
        { 
			name = "Annie:Mat_BugEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_eyeball", 	
			renderGroup = 2,
			--shaderFlags = SCF_LIGHTING, 
			textures = { }, 
			parameters = EyeParameters, 
		},
		{ 
			name = "Annie:Mat_BugPupil",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = pupilZOffset, },
		},
	},
}

SceneData.actors.Gracie = {
    model = "buggynight:models/gracie_osd.mxm",
    animation = animations("gracie", { RockSegment, LogSegment, TwigSegment, EndSegment }),
    targetBone = "Gracie:bug1_mox_cn_upperbody|Gracie:bug1_MOXIE_skeleton|RootNode",
    materials = {
		{
			name = "Gracie:Mat_Bug", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/bug01_diffuse.dds", }, 
			parameters = { 
				u_DarkColor = BugLightColor, 
				u_TexContrast = 1.0, 
				u_TexOffset = -0.5, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
        { 
			name = "Gracie:Mat_BugEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_eyeball", 	
			renderGroup = 2, 
			--shaderFlags = SCF_LIGHTING, 
			textures = { }, 
			parameters = EyeParameters, 
		},
		{ 
			name = "Gracie:Mat_BugPupil",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = pupilZOffset, },
		},
	},
}

SceneData.actors.John = {
    model = "buggynight:models/john_osd.mxm",
    animation = animations("john", { RockSegment, LogSegment, TwigSegment, SquirrelSegment, EndSegment }),
    targetBone = "John:bug1_mox_cn_upperbody|John:bug1_MOXIE_skeleton|RootNode",
    materials = {
		{
			name = "John:Mat_Bug", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/bug01_diffuse.dds", }, 
			parameters = { 
				u_DarkColor = BugLightColor, 
				u_TexContrast = 1.0, 
				u_TexOffset = -0.5, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
        { 
			name = "John:Mat_BugEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_eyeball", 	
			renderGroup = 2, 
			--shaderFlags = SCF_LIGHTING, 
			textures = { }, 
			parameters = EyeParameters, 
		},
		{ 
			name = "John:Mat_BugPupil",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = pupilZOffset, },
		},
	},
}

SceneData.actors.Tessa = {
    model = "buggynight:models/tessa_osd.mxm",
    animation = animations("tessa", { RockSegment, LogSegment, TwigSegment, EndSegment }),
    targetBone = "Tessa:bug1_mox_cn_upperbody|Tessa:bug1_MOXIE_skeleton|RootNode",
    materials = {
		{
			name = "Tessa:Mat_Bug", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/bug01_diffuse.dds", }, 
			parameters = { 
				u_DarkColor = BugLightColor, 
				u_TexContrast = 1.0, 
				u_TexOffset = -0.5, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
        { 
			name = "Tessa:Mat_BugEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_eyeball", 	
			renderGroup = 2, 
			--shaderFlags = SCF_LIGHTING, 
			textures = { }, 
			parameters = EyeParameters, 
		},
		{ 
			name = "Tessa:Mat_BugPupil",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = pupilZOffset, },
		},
	},
}

SceneData.actors.Wah = {
    model = "buggynight:models/wah_osd.mxm",
    animation = animations("wah", { IntroSegment, RockSegment, LogSegment, TwigSegment, SquirrelSegment, FrogSegment, EndSegment, ThatsAllFolksSegment }),
    targetBone = "Wah:bug1_mox_cn_upperbody|Wah:bug1_MOXIE_skeleton|RootNode",
    materials = {
		{
			name = "Wah:Mat_Bug", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/bug01_diffuse.dds", }, 
			parameters = { 
				u_DarkColor = BugLightColor, 
				u_TexContrast = 1.0, 
				u_TexOffset = -0.5, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
        { 
			name = "Wah:Mat_BugEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_eyeball", 	
			renderGroup = 2, 
			--shaderFlags = SCF_LIGHTING, 
			textures = { }, 
			parameters = EyeParameters, 
		},
		{ 
			name = "Wah:Mat_BugPupil",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = pupilZOffset, },
		},
	},
}

SceneData.actors.ThePlant = {
    model = "buggynight:models/theplant.mxm",
    animation = animations("theplant", { TwigSegment }),
    targetBone = "ThePlant:PlantMain1_Skel|ThePlant:ThePlant_MOXIE_skeleton|RootNode",
    materials = {
		{ 
            name = "ThePlant:HeroPlant", 
            shader = "buggynight:shaders/unlit.rsfx", 
            technique = "vertcolors_and_matcolor_solid", 
            renderGroup = 2, 
            shaderFlags = SCF_LIGHTING, 
            textures = { u_MainTex = "buggynight:textures/flowerplant.dds", },

            parameters = { 
                u_Color = Color.new( 1, 1, 1, 0 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_Emissive = 0.0,
            }, 
        },
    },
}

SceneData.actors.TheLeafPlant = {
    model = "buggynight:models/theleafplant.mxm",
    animation = animations("theleafplant", { TwigSegment }),
    boneRadius = 1.5,
    materials = {
		{ 
            name = "TheLeafPlant:Plants", 
            shader = "buggynight:shaders/unlit.rsfx", 
            technique = "vertcolors_and_matcolor", 
            renderGroup = 2, 
            shaderFlags = SCF_LIGHTING, 
            textures = { u_MainTex = "buggynight:textures/plantsparts01.dds", },

            parameters = { 
                u_Color = Color.new( 1, 1, 1, 1 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_Emissive = 0.0,
            }, 
        },
    },
}

SceneData.actors.HomePlant = {
    model = "buggynight:models/homeplant.mxm",
    animation = animations("homeplant", { RockSegment, LogSegment, TwigSegment, SquirrelSegment, FrogSegment, EndSegment }),
    materials = {
		{ name = "HomePlant:Mat_HomePlantTwigs", shader = "buggynight:shaders/unlit.rsfx", technique = "texture", renderGroup = 2, shaderFlags = SCF_LIGHTING, textures = { u_MainTex = "buggynight:textures/homeplantparts.dds", }, parameters = { u_Emissive = 0.0, u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, },
		{ name = "HomePlant:Mat_HomePlantLeaves", shader = "buggynight:shaders/unlit.rsfx", technique = "vertcolors_and_matcolor", renderGroup = 2, shaderFlags = SCF_LIGHTING, textures = { u_MainTex = "buggynight:textures/homeplantparts.dds", }, parameters = { u_Emissive = 0.0, u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, },
    },
}

SceneData.actors.Frog = {
    model = "buggynight:models/frog_osd.mxm",
    animation = animations("frog", { RockSegment, LogSegment, TwigSegment, SquirrelSegment, FrogSegment, EndSegment }),
    targetBone = "Frog:frog_mox_cn_spine1|Frog:frog_MOXIE_skeleton|RootNode",
    materials = {
        {    
			name = "Frog:Mat_Frog", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodymultitexture", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_BaseTex = "buggynight:textures/frog_diffuse_end.dds",  u_BlendTex = "buggynight:textures/frog_diffuse_end.dds" }, 
			parameters = { 
				u_DarkColor = BugLightColor, 
				u_TexContrast = 1.0, 
				u_TexOffset = -0.5, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( 0, -1, 0, 0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0, 
				u_BlendAmount = 0.0, 
			}, 
		},
        { 
			name = "Frog:Mat_FrogEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_eyeball", 	
			renderGroup = 2, 
			textures = { }, 
			parameters = EyeParameters,
		},
		{ 
			name = "Frog:Mat_FrogPupil",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = pupilZOffset, },
		},
	},
    components = { 
        { 
            name = "TextureBlender", 
            params = { 
                materialName = "Frog:Mat_Frog", 
                trackingTarget = "Frog:TextureFader|Frog:TextureFaderFrame|Frog:frog_geometry|RootNode", 
                switchRange = Vector(0.0, 10.0),
            }
        }
    },
}

SceneData.actors.EndMasking = {
    model = "buggynight:models/endmasking.mxm",
    animation = animations("endmasking", { EndSegment }),
    materials = {
		{ name = "EndMasking:Mat_EndMasking", shader = "buggynight:shaders/unlit.rsfx", technique = "vertcolors_and_matcolor", renderGroup = 2,  textures = { u_MainTex = "buggynight:textures/white.dds", }, parameters = { u_Emissive = 0.0, u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, },
    },
    components = { 
        { 
            name = "ColorAnimator", 
            params = { 
                materialName = "EndMasking:Mat_EndMasking", 
                trackingTarget = "EndMasking:Fader|EndMasking:EndMasking_MOXIE_skeleton|RootNode", 
                animatedParameter = "u_Color", 
                startColor = Vector(1,1,1,1), 
                endColor = Vector(0,0,0,0), 
                range = Vector(0, 1),
            }
        },
    },
}


-- the frog uses different textures in a couple scenes
SceneData.actors.FrogRock = deepcopy(SceneData.actors.Frog)
SceneData.actors.FrogRock.materials[1].textures = { u_BaseTex = "buggynight:textures/frog_diffuse_rock.dds", u_BlendTex = "buggynight:textures/frog_diffuse_end.dds"} 

SceneData.actors.FrogLog = deepcopy(SceneData.actors.Frog)
SceneData.actors.FrogLog.materials[1].textures = { u_BaseTex = "buggynight:textures/frog_diffuse_stump.dds", u_BlendTex = "buggynight:textures/frog_diffuse_end.dds"}

SceneData.actors.FrogTwig = deepcopy(SceneData.actors.Frog)
SceneData.actors.FrogTwig.materials[1].textures = { u_BaseTex = "buggynight:textures/frog_diffuse_plant.dds", u_BlendTex = "buggynight:textures/frog_diffuse_end.dds"}

SceneData.actors.FrogSquirrel = deepcopy(SceneData.actors.Frog)
SceneData.actors.FrogSquirrel.materials[1].textures = { u_BaseTex = "buggynight:textures/frog_diffuse_squirrel.dds", u_BlendTex = "buggynight:textures/frog_diffuse_end.dds"}

SceneData.actors.Squirrel = {
    model = "buggynight:models/squirrel_osd.mxm",
    animation = animations("squirrel", { SquirrelSegment }),
	materials = {
		{
			name = "squirrel:SquirrelBody", 
		 	shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple_fullLight", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/squirrel_diffuse_02.dds", }, 
			parameters = { 
				u_DarkColor = Color.new( 0.75, 0.75, 0.75, 1 ), 
				u_TexContrast = 1.0, 
				u_TexOffset = -1.0, 
				u_AmbientLightColor = Color.new( 0.75, 0.75, 0.75, 1 ), 
				u_LightDirection = Vector.new( -0.65, -0.8, -1.0, 0.0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
		{
			name = "squirrel:Mat_SquirrelFeet", 
		 	shader = "buggynight:shaders/unlit.rsfx", 
			technique = "texture", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/squirrel_diffuse_02.dds", }, 
			parameters = { 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0, 
			}, 
		},
        {
            name = "squirrel:SquirrelTail", 
            shader = "buggynight:shaders/unlit.rsfx", 
            technique = "transparent_with_color_fullLight", 
            renderGroup = 2, 
            shaderFlags = SCF_LIGHTING, 
            textures = { u_MainTex = "buggynight:textures/squirel_taildiffuse_02.dds", }, 
            parameters = { 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_Emissive = 0.0, 
            }, 
        },
		{ 
			name = "squirrel:SquirrelEyes",
			shader = "buggynight:shaders/character.rsfx", 
			technique = "character_pupil", 
			renderGroup = 2, 
			textures = {},
			parameters = { u_zOffset = 0.0, },
		},
    },
}

SceneData.actors.SquirrelHat = {
    model = "buggynight:models/squirrelhat_osd.mxm",
    animation = animations("squirrelhat", { SquirrelSegment }),
	materials = {
		{
			name = "squirrelHat:Mat_SquirrelHat", 
		 	shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple_fullLight", 
			renderGroup = 2, 
			shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/nightcap.dds", }, 
			parameters = { 
				u_DarkColor = Color.new( 0.75, 0.75, 0.75, 1 ), 
				u_TexContrast = 1.0, 
				u_TexOffset = -1.0, 
				u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), 
				u_LightDirection = Vector.new( -0.65, -0.8, -1.0, 0.0 ), 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_Emissive = 0.0,
			}, 
		},
    },
}

SceneData.actors.Broom = {
    model = "buggynight:models/broom.mxm",
    animation = animations("broom", { SquirrelSegment }),
	materials = {
        {
            name = "Broom1:Broom1",
            shader = "buggynight:shaders/unlit.rsfx", 
            technique = "texture",
            renderGroup = 3,
            shaderFlags = SCF_LIGHTING, 
            textures = { u_MainTex = "buggynight:textures/broom.dds", },
            parameters = { 
                u_Color = Color.new( 1, 1, 1, 1 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_Emissive = 0.0,
            }, 
        },
    },
}

SceneData.actors.SquirrelHouse = {
    model = "buggynight:models/squirrelhouse.mxm",
    animation = animations("squirrelhouse", { SquirrelSegment }),

    materials = {
        { 
            name = "Squirrelhouse:TreeWallA", 
            shader = "buggynight:shaders/unlit.rsfx", 
            technique = "three_texture_and_overlay", 
            renderGroup = 2, 
            shaderFlags = SCF_LIGHTING, 
            textures = { 
                u_TopTex = "buggynight:textures/treehouselight.03.dds", 
                u_MiddleTex = "buggynight:textures/treehouselight.02.dds", 
                u_BaseTex = "buggynight:textures/treehouselight.01.dds",
                u_TileTex = "buggynight:textures/overlaytexture.dds",
            }, 
            parameters = { 
                u_Color = Color.new( 1, 1, 1, 0.8 ), 
                u_TopTexFade = 0.0, 
                u_MiddleTexFade = 0.0, 
                u_TopTex_ST = Vector.new(1, 1, 0, 0), 
                u_MiddleTex_ST = Vector.new(1, 1, 0, 0), 
                u_BaseTex_ST = Vector.new(1, 1, 0, 0), 
                u_Emissive = 0.0, 
            }, 
        },
        { 
            name = "Squirrelhouse:TreeWallAInside", 
            shader = "buggynight:shaders/unlit.rsfx", 
            technique = "three_texture_and_overlay", 
            renderGroup = 2, 
            textures = { 
                u_TopTex = "buggynight:textures/treehouselight.03.dds", 
                u_MiddleTex = "buggynight:textures/treehouselight.02.dds", 
                u_BaseTex = "buggynight:textures/treehouselight.01.dds",
                u_TileTex = "buggynight:textures/overlaytexture.dds",
            }, 
            parameters = { 
                u_Color = Color.new( 1, 1, 1, 0.99 ), 
                u_TopTexFade = 0.0, 
                u_MiddleTexFade = 0.0, 
                u_TopTex_ST = Vector.new(1, 1, 0, 0), 
                u_MiddleTex_ST = Vector.new(1, 1, 0, 0), 
                u_BaseTex_ST = Vector.new(1, 1, 0, 0), 
                u_Emissive = 1.0, 
            }, 
        },
    },
    components = { 
        { 
            name = "TextureAnimator", 
            params = { 
                trackingTarget = "Squirrelhouse:SwitchObj|Squirrelhouse:SquirrelHouse_geometry|RootNode", 
                switchRange = Vector(2.286, 4.286),
            }
        }
    },
}

SceneData.actors.Owl = {
    model = "buggynight:models/owl_osd.mxm",
    animation = {
        OwlIdle = { "buggynight:anims/owl_owl1.mxa", 1, 2, true },
        Owl1 = { "buggynight:anims/owl_owl1.mxa", 1, 92, false },
        Owl2 = { "buggynight:anims/owl_owl2.mxa", 1, 112, false },
        Owl3 = { "buggynight:anims/owl_owl3.mxa", 1, 82, false },
        Owl4 = { "buggynight:anims/owl_owl4.mxa", 1, 82, false },
    },

    materials = {
        { 
			name = "Owl:Mat_Owl", 
			shader = "buggynight:shaders/unlit.rsfx", 
			technique = "texture_and_matcolor", 
			renderGroup = 2, 
            shaderFlags = SCF_LIGHTING, 
			textures = { u_MainTex = "buggynight:textures/owl_diffuse.dds", }, 
			parameters = { 
				u_MainTex_ST = Vector.new(1, 1, 0, 0), 
				u_MaterialColor = Vector.new(1, 1, 1, 1),
			},
        },
        { 
			name = "Owl:Mat_OwlEye", 
			shader = "buggynight:shaders/character.rsfx", 
			technique = "characterbodysimple", 	
			renderGroup = 2, 
			textures = { }, 
			parameters = EyeParameters,
		},
    },
    components = { 
        { 
            name = "ColorAnimator", 
            params = { 
                materialName = "Owl:Mat_Owl", 
                trackingTarget = "Owl:Owl_ColorFader_Skel|Owl:Owl_MOXIE_skeleton|RootNode", 
                animatedParameter = "u_MaterialColor", 
                startColor = Vector(1,1,1,1), 
                endColor = Vector(0,0,0,0), 
                range = Vector(1.613, 2.613),
            }
        },
        { 
            name = "SecondaryAnimationControl", 
            params = { 
                triggerName = "Trigger_Owl",
                idleAnim = "OwlIdle",
                animations = {
                    { name = "Owl1", sound = { name = "owl_hoo1", delay = 1, type = "Canopy"} },
                    { name = "Owl2", sound = { name = "owl_hoo2", delay = 1, type = "Canopy"} },
                    { name = "Owl3", sound = { name = "owl_hoowoo", delay = 1, type = "Canopy"} },
                    { name = "Owl4", sound = { name = "owl_hoowoohoo2", delay = 1, type = "Canopy"} },
                },
            },
        },
    },
}

SceneData.actors.OwlTreeCover = {
    model = "buggynight:models/owltreecover.mxm",
    materials = {
        { 
            name = "TreeCoverOwl", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            technique = "vertexcolor", 
            renderGroup = 2, 
            textures = { u_MainTex = "buggynight:textures/overlaytexture2.dds", }, 
            parameters = { 
                u_Emissive = 0.0, 
                u_Color = Color.new( 1, 1, 1, 1 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
            },
        },
    }
}
SceneData.actors.Ground = {
    model = "buggynight:models/ground.mxm",
    materials = {
		{ 
            name = "Mat_Ground", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            renderGroup = 0, 
            technique = "texture_with_color", 
            textures = { u_MainTex = "buggynight:textures/white.dds", }, 
            parameters = { 
                u_Emissive = 0.0, 
                u_Color = Color.new( 0 /256, 0/256, 0/256, 1 ), 
               -- u_Color = Color.new( 32 /256, 23/256, 14/256, 1 ), 
            } 
        },
    },
}

SceneData.actors.RockFootprints = {
    model = "buggynight:models/footprints_rock.mxm",
    materials = {
		{ 
            name = "Footprints_B", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            renderGroup = 2, 
            technique = "trail_fader", 
            textures = { u_MainTex = "buggynight:textures/footprints_blue.dds", }, 
            parameters = { 
                u_Color = Color.new( 0, 0, 0, 0 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_fadeRadius = Vector.new( 4.0, 5.5 ),
           } 
        },
    },
    components = { 
        { 
            name = "FootprintFader", 
            params = { 
                matName = "Footprints_B", 
                fadeObjectName = "HomePlant", 
                fadeObjectBone = "HomePlant:TrailPos|HomePlant:HomePlant_MOXIE_skeleton|RootNode", 
                fadeOutSegment = "RockB",
                fadeInDelay = 0,
                fadeInSpeed = 5,
                fadeOutDelay = 0,
                fadeOutSpeed = 2,
            }
        },
    },
}

SceneData.actors.LogFootprints = {
    model = "buggynight:models/footprints_log.mxm",
    materials = {
		{ 
            name = "Footprints_B", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            renderGroup = 2, 
            technique = "trail_fader", 
            textures = { u_MainTex = "buggynight:textures/footprints_blue.dds", }, 
            parameters = { 
                u_Color = Color.new( 0, 0, 0, 0 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_fadeRadius = Vector.new( 4.0, 5.5 ),
           } 
        },
    },
    components = { 
        { 
            name = "FootprintFader", 
            params = { 
                matName = "Footprints_B", 
                fadeObjectName = "HomePlant", 
                fadeObjectBone = "HomePlant:TrailPos|HomePlant:HomePlant_MOXIE_skeleton|RootNode", 
                fadeOutSegment = "LogB",
                fadeInDelay = 0,
                fadeInSpeed = 5,
                fadeOutDelay = 0,
                fadeOutSpeed = 2,
            }
        },
    },
}



SceneData.actors.TwigFootprints = {
    model = "buggynight:models/footprints_twig.mxm",
    materials = {
		{ 
            name = "Footprints_B", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            renderGroup = 2, 
            technique = "trail_fader", 
            textures = { u_MainTex = "buggynight:textures/footprints_blue.dds", }, 
            parameters = { 
                u_Color = Color.new( 0, 0, 0, 0 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_fadeRadius = Vector.new( 4.0, 5.5 ),
           } 
        },
    },
    components = { 
        { 
            name = "FootprintFader", 
            params = { 
                matName = "Footprints_B",
                fadeObjectName = "HomePlant", 
                fadeObjectBone = "HomePlant:TrailPos|HomePlant:HomePlant_MOXIE_skeleton|RootNode", 
                fadeOutSegment = "TwigB",
                fadeInDelay = 0,
                fadeInSpeed = 5,
                fadeOutDelay = 0,
                fadeOutSpeed = 2,
            }
        },
    },
}

SceneData.actors.SquirrelFootprints = {
    model = "buggynight:models/footprints_squirrel.mxm",
    materials = {
		{ 
            name = "Footprints_B", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            renderGroup = 2, 
            technique = "trail_fader", 
            textures = { u_MainTex = "buggynight:textures/footprints_blue.dds", }, 
            parameters = { 
                u_Color = Color.new( 0, 0, 0, 0 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_fadeRadius = Vector.new( 4.0, 5.5 ),
           } 
        },
    },
    components = { 
        { 
            name = "FootprintFader", 
            params = { 
                matName = "Footprints_B", 
                fadeObjectName = "HomePlant", 
                fadeObjectBone = "HomePlant:TrailPos|HomePlant:HomePlant_MOXIE_skeleton|RootNode", 
                fadeOutSegment = "SquirrelB",
                fadeInDelay = 0,
                fadeInSpeed = 5,
                fadeOutDelay = 0,
                fadeOutSpeed = 2,
            }
        },
    },
}
SceneData.actors.FrogFootprints = {
    model = "buggynight:models/footprints_frog.mxm",
    materials = {
		{ 
            name = "Footprints_B", 
            shader = "buggynight:shaders/unlit.rsfx", 
            shaderFlags = SCF_LIGHTING, 
            renderGroup = 2, 
            technique = "trail_fader", 
            textures = { u_MainTex = "buggynight:textures/footprints_blue.dds", }, 
            parameters = { 
                u_Color = Color.new( 0, 0, 0, 0 ), 
                u_MainTex_ST = Vector.new(1, 1, 0, 0), 
                u_fadeRadius = Vector.new( 4.0, 5.5 ),
           } 
        },
    },
    components = { 
        { 
            name = "FootprintFader", 
            params = { 
                matName = "Footprints_B", 
                fadeObjectName = "HomePlant", 
                fadeObjectBone = "HomePlant:TrailPos|HomePlant:HomePlant_MOXIE_skeleton|RootNode", 
                fadeOutSegment = "FrogB",
                fadeInDelay = 0,
                fadeInSpeed = 5,
                fadeOutDelay = 0,
                fadeOutSpeed = 2,
            }
        },
    },
}

SceneData.actors.FireFliesForeground = {
    position = {0.0, 0.0, 0.0},
    components = {
        { 
            name = "SimpleParticles", 
            params = { 
                emitterName = "FireFlyForegroundParticles", 
                mpxFileName = "firefliesforeground", 
                position = Vector(0.0, -40.0, 0.0),
            }
        }
    }
}
SceneData.actors.FireFliesBackground = {
    position = {0.0, 0.0, 0.0},
    components = {
        { 
            name = "SimpleParticles", 
            params = { 
                emitterName = "FireFlyForegroundParticles", 
                mpxFileName = "firefliesbackground", 
                position = Vector(0.0, -40.0, 0.0),
            }
        }
    }
}

SceneData.actors.Floaty = {
    position = {0.0, 0.0, 0.0},
    model = "buggynight:models/floaty.mxm",
    animation = { Loop = { "buggynight:anims/floaty_loop.mxa", 1, 293 , true, true } },
    materials = { { name = "Mat_Floaty", shader = "buggynight:shaders/unlit.rsfx", technique = "transparent_with_color_nodepth", renderGroup = 5, shaderFlags = SCF_LIGHTING, textures = { u_MainTex = "buggynight:textures/firefly.dds", }, parameters = { u_Emissive = 0.0, u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, },},
    components = { { name = "FloatyDriver", params = { materialName = "Mat_Floaty", startDelay = 2, lifeSpan = 10  } } },
}


SceneData.actors.DebugSphere = {
    model = "buggynight:models/unitsphere.mxm",
    materials = {
        {
            name = "Mat_Sphere",
            shader = "common:shaders/simple.rsfx",
            technique = "default_alphaBlend",
            renderGroup = 2,
            textures =
            {
                u_diffuseMap = "buggynight:textures/white.dds",
            },
            parameters =
            {
                u_diffuseColor = Color.new( 0.0, 1.0, 0.0, 0.2 ),
            }
        },
    },
}

SceneData.actors.DebugAxis = {
    model = "buggynight:models/axis.mxm",
    materials = {
        { 
        	name = "Mat_Axis", shader = "buggynight:shaders/unlit.rsfx",
	     	technique = "vertexcolor",
	     	renderGroup = 7,
	     	textures = { },
	     	parameters = { u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, 
	    },
    },
}

SceneData.actors.DistantFrogLocator = {
    components = {
        {
            name = "Locator",
            params = {
                targetPosition = Vector(0, 2, -10),
                fixedPosition = true,
                debugColor = Color(1, 1, 1, 0.2),
            },
        }
    },
}

SceneData.actors.Cricket1Locator = {
    components = {
        {
            name = "Locator",
            params = {
                targetPosition = Vector(10, 2, 0),
                fixedPosition = true,
                debugColor = Color(1, 1, 0, 0.2),
            },
        }
    },
}

SceneData.actors.Cricket2Locator = {
    components = {
        {
            name = "Locator",
            params = {
                targetPosition = Vector(-10, 2, 0),
                fixedPosition = true,
                debugColor = Color(0, 1, 1, 0.2),
            },
        }
    },
}

SceneData.actors.OwlLocator = {
    components = {
        {
            name = "Locator",
            params = {
                targetPosition = Vector(0, 2, 10),
                fixedPosition = true,
                debugColor = Color(0.5, 0.5, 0.5, 0.2),
            },
        }
    },
}

SceneData.actors.RockScatterLocator = {
    components = {
        {
            name = "Locator",
            params = {
                  targetPosition = Vector(6.6, 0.28, -0.75),
                  debugColor = SoundLocatorColor,
            },
        }
    },
}

SceneData.actors.LogScatterLocator = {
    components = {
        {
            name = "Locator",
            params = {
                targetPosition = Vector(-5.95, 0.15, -0.62),
                debugColor = SoundLocatorColor,
            },
        }
    },
}

SceneData.lights.SquirrelLight = {
    type        = LT_SPOT,
    range       = { 6.0, 7.5 },
    spotAngle   = { 17.0, 18.0 },
	wrap		= -0.2,
    --ambient     = Vector.new( 0.0, 0.0, 1.0 ),
    diffuse     = Vector.new( 10.0, 10.0, 7.0 ),
}

SceneData.actors.SquirrelLightObject = {
	position = { 0.0, 0.0, 0.0 },
	components = {
		{
			name = "SquirrelLightDriver",
			params =
			{
				lightName 		= "SquirrelLight",
				lightDef		= SceneData.lights.SquirrelLight,
				lightPosition	= Vector.new(  0.5,  2.5,  2.0 ),
                lightTarget     = Vector.new( -3.0, -4.1, -3.5 ),
				trackingActor	= "SquirrelHouse",
				trackingBone	= "Squirrelhouse:HouseLight|Squirrelhouse:SquirrelHouse_geometry|RootNode",
                trackingTarget  = "Squirrelhouse:SwitchObj|Squirrelhouse:SquirrelHouse_geometry|RootNode", 
                switchRange     = Vector(3.286, 4.286),
                lightStartRange = { 0.0, 0.01 },
                lightEndRange   = SceneData.lights.SquirrelLight.range,
			}
		}
	}
}
