
SceneData = SceneData or {}

local GroundAttenuation = {
    { pitch =  1, atten = 0.2 },
    { pitch =  0, atten = 0.8 },
    { pitch = -1, atten = 0.2 },
}

local EnvironmentAttenuation = {
    { pitch =  0.8, atten = 0 },
    { pitch =  0,   atten = 1 },
}

local CanopyAttenuation = {
    { pitch = 1,    atten = 1 },
    { pitch = 0.25, atten = 0 },
}

SceneData.AudioTypes = {
    SFX =           { enabled = true, masterVolume = 1, verticalAttenuation = GroundAttenuation, },
    Morsing =       { enabled = true, masterVolume = 1, verticalAttenuation = GroundAttenuation, },
    Music =         { enabled = true, masterVolume = 1, defaultVolume = 1, verticalAttenuation = EnvironmentAttenuation, },
    MusicSFX =      { enabled = true, masterVolume = 1, verticalAttenuation = EnvironmentAttenuation, },
    Environment =   { enabled = true, masterVolume = 0.7, verticalAttenuation = EnvironmentAttenuation, },
    Canopy =        { enabled = true, masterVolume = 1, verticalAttenuation = CanopyAttenuation, },
    Ambient =       { enabled = true, masterVolume = 1, },
    Cricket =       { enabled = true, masterVolume = 0.2, },
}

SceneData.AudioLatencyOffset = 3  -- In frames

-- Constants to make it easier to copy sound blocks around
local CricketLoopOffset  = 6 * 24    -- duration of the startup cricket sound
local OwlLoopOffset      = 90        -- delay from the start of environment sound return to bring up owls

local RockCRibbit        = 376       -- when the ribbit starts in RockC
local RockCEnvStart      = 607       -- start frame to bring back the environment sounds after ribbit in RockC

local LogCRibbit         = 939       -- when the ribbit starts in LogC
local LogCEnvStart       = 1157      -- start frame to bring back the environment sounds after ribbit in LogC

local TwigCRibbit        = 1588     -- when the ribbit starts in TwigC
local TwigCEnvStart      = 1680     -- start frame to bring back the environment sounds after ribbit in TwigC

local SquirrelDRibbit    = 2240     -- when the ribbit starts in SquirrelC
local SquirrelDEnvStart  = 2290     -- start frame to bring back the environment sounds after ribbit in SquirrelC

local FrogCRibbit        = 2759     -- when the ribbit starts in FrogC

local MorseFadeLength    = 1        -- duration of the fade in for morsing
local MorseFadeVolume    = 1

SceneData.audio = {

    Intro = {
        startFrame = 94,
        sounds = {
            { name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeStart = 0, fadeEnd = 1, looping = true },
            { name = "distantfrogs_loop", type = "Environment", fadeTime = 2, fadeStart = 0, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket",  startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },
            { name = "starry_night", type = "Canopy", fadeTime = 2, fadeStart = 0, fadeEnd = 0.2, looping = true },

            { name = "00_94_land_look_morse_take", type = "SFX", trackPosition = "IntroRig", startFrame = 109, },
        },
    },

    IntroB = {
        startFrame = 171,
        sounds = {
            { name = "01_mx_loop", type = "Music", startFrame = 220, looping = true, fadeTime = 3, fadeStart = 0.05, fadeEnd = 1,},
            { name = "ambience_loop", type = "Ambient", startFrame = 220, fadeTime = 3, fadeEnd = 0.5, looping = true },
            { name = "00_155_skatter", type = "SFX", trackPosition = "IntroRig", startFrame = 202, },
        }
    },

    RockA = {
        startFrame = -56,
        sounds = {
            -- Checkpoint environmental sound and music loop
            { name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeEnd = 0.5, looping = true },
            { name = "distantfrogs_loop", type = "Environment", fadeTime = 2, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket",  startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },
            { name = "starry_night", type = "Canopy", fadeTime = 2, fadeEnd = 0.2, looping = true },
            { name = "01_mx_loop", type = "Music", looping = true, fadeTime = 3, fadeEnd = 1,},

            { name = "01_here_comp", type = "Morsing", trackPosition = "Wah", startFrame = -56, fadeTime = MorseFadeLength, fadeStart = 0, fadeEnd = MorseFadeVolume, looping = true },
        },
    },

    RockB = {
        startFrame = 124,
        sounds = {
            { name = "01_here_comp", type = "Morsing", endFrame = 124 },

            { name = "01_152_skitter", type = "MusicSFX", trackPosition = "Wah", startFrame = 152, },
            { name = "01_154_mx_skitter", type = "Music", startFrame = 154, trackPosition = "Wah"},

            { name = "01_335_346_rockloop", type = "SFX", trackPosition = "Wah", startFrame = 324, looping = true },

            { name = "01_mx_loop", type = "Music", startFrame = 124, fadeTime = 1, fadeEnd = 0 },
            { name = "02_mx_loop_end", type = "Music", startFrame = 124, volume = 1 },
        },
    },

    RockC = {
        startFrame = 347,
        sounds = {
            { name = "ribbit", type = "SFX", startFrame = RockCRibbit, trackPosition = "FrogRock", volume = 0.5},
            { name = "01_335_346_rockloop", type = "SFX", endFrame = RockCRibbit + 14 },
            { name = "01_402_bugfreak", type = "SFX", trackPosition = "Wah", startFrame = 405, },

            -- Shut down environmental sounds at ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = RockCRibbit, fadeTime = 2, fadeStart = 0.7, fadeEnd = 0, looping = true, },
            { name = "cricket1_loop", type = "Cricket", endFrame = RockCRibbit, looping = true },
            { name = "cricket2_loop", type = "Cricket", endFrame = RockCRibbit, looping = true },
            { name = "owl_loop", type = "Environment", endFrame = RockCRibbit, looping = true },
        },
    },

    RockD = {
        startFrame = 434,
        sounds = {
            { name = "01_436_tonguezap", type = "SFX", trackPosition = "FrogRock", volume = 0.3, startFrame = 430, },
            { name = "01_455_slurp", type = "SFX", trackPosition = "FrogRock", startFrame = 455, volume = 0.3 },
            { name = "01_495_slurp_boing", type = "SFX", trackPosition = "FrogRock", startFrame = 496, },
            { name = "01_589_rock_skatter", type = "SFX", trackPosition = "RockScatterLocator", startFrame = 578, },

            -- Bring them back up after ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = RockCEnvStart, fadeTime = 2, fadeStart = 0, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", startFrame = RockCEnvStart, trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startFrame = RockCEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", startFrame = RockCEnvStart, trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket", startFrame = RockCEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startFrame = RockCEnvStart, startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },

            { name = "02_mx_loop", type = "Music", startFrame = 625, looping = true, fadeTime = 3, fadeStart = 0.05, fadeEnd = 1,},
        }
    },

    LogA = {
        startFrame = 438,
        sounds = {
            -- Checkpoint environmental sound and music loop
            { name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeEnd = 0.5, looping = true },
            { name = "distantfrogs_loop", type = "Environment", fadeTime = 2, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket",  startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },
            { name = "starry_night", type = "Canopy", fadeTime = 2, fadeEnd = 0.2, looping = true },
            { name = "02_mx_loop", type = "Music", looping = true, fadeTime = 3, fadeEnd = 1,},

            { name = "02_bob_comp", type = "Morsing", trackPosition = "Wah", startFrame = 438, fadeTime = MorseFadeLength, fadeStart = 0, fadeEnd = MorseFadeVolume, looping = true },
        },
    },

    LogB = {
        startFrame = 654,
        sounds = {
            { name = "02_mx_loop", type = "Music", startFrame = 654, fadeTime = 1, fadeEnd = 0 },
            { name = "02_mx_loop_end", type = "Music", startFrame = 654, volume = 1 },
            { name = "02_bob_comp", type = "Morsing", endFrame = 654 },

            { name = "02_657", type = "SFX", trackPosition = "Wah", startFrame = 657, },

            { name = "02_683_skitter", type = "MusicSFX", trackPosition = "Annie", startFrame = 685 },
            { name = "02_mx_683_skitter", type = "Music", startFrame = 683, trackPosition = "Wah" },

            { name = "02_843_logloop", type = "SFX", trackPosition = "Annie", startFrame = 843, looping = true },

        },
    },

    LogC = {
        startFrame = 857,
        sounds = {
            { name = "02_843_logloop", type = "SFX", endFrame = 857 },

            { name = "02_857_bugpushlogpop", type = "SFX", startFrame = 857, trackPosition = "Annie", },

        },
    },

    LogD = {
        startFrame = 934,
        sounds = {
            { name = "ribbit", type = "SFX", startFrame = LogCRibbit, trackPosition = "FrogLog", volume = 1, },

            { name = "02_952_shiver", type = "SFX", startFrame = 952, trackPosition = "Tessa", volume = 0.2 },
            { name = "02_968_tonguezap", type = "SFX", startFrame = 968, volume = 0.5, trackPosition = "FrogLog", },
            { name = "02_993_slurp_boing", type = "SFX", startFrame = 993, trackPosition = "FrogLog", },
            { name = "02_1098_reemerge", type = "SFX", startFrame = 1099, trackPosition = "Wah", },
            { name = "02_skatter", type = "SFX", trackPosition = "LogScatterLocator", startFrame = 1165, },

            -- Shut down environmental sounds at ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = LogCRibbit, fadeTime = 2, fadeStart = 0.7, fadeEnd = 0, looping = true, },
            { name = "cricket1_loop", type = "Cricket", endFrame = LogCRibbit, looping = true },
            { name = "cricket2_loop", type = "Cricket", endFrame = LogCRibbit, looping = true },
            { name = "owl_loop", type = "Environment", endFrame = LogCRibbit, looping = true },

            -- Bring them back up after ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = LogCEnvStart, fadeTime = 2, fadeStart = 0, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", startFrame = LogCEnvStart, trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startFrame = LogCEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", startFrame = LogCEnvStart, trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket", startFrame = LogCEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startFrame = LogCEnvStart, startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },

            { name = "03_mx_loop", type = "Music", startFrame = 1165, looping = true, fadeTime = 3, fadeStart = 0.05, fadeEnd = 1,},
        },
    },

    TwigA = {
        startFrame = 1151,
        sounds = {
            -- Checkpoint environmental sound and music loop
            { name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeEnd = 0.5, looping = true },
            { name = "distantfrogs_loop", type = "Environment", fadeTime = 2, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket",  startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },
            { name = "starry_night", type = "Canopy", fadeTime = 2, fadeEnd = 0.2, looping = true },
            { name = "03_mx_loop", type = "Music", looping = true, fadeTime = 3, fadeEnd = 1,},

            { name = "03_jane_comp", type = "Morsing", trackPosition = "Wah", startFrame = 1144, fadeTime = MorseFadeLength, fadeStart = 0, fadeEnd = MorseFadeVolume, looping = true },
        },
    },

    TwigB = {
        startFrame = 1384,
        sounds = {
            { name = "03_jane_comp", type = "SFX", endFrame = 1384 },
            { name = "03_1385_bugboing", type = "SFX", startFrame = 1385, trackPosition = "Wah"},

            { name = "03_mx_loop", type = "Music", startFrame = 1387, fadeTime = 1, fadeEnd = 0 },
            { name = "03_mx_loop_end", type = "Music", startFrame = 1387, volume = 1 },
            { name = "03_1412_mx_skitter", type = "Music", startFrame = 1420, trackPosition = "Wah"},
			{ name = "03_1415_skitter", type = "MusicSFX", startFrame = 1420, trackPosition = "Wah" },
            { name = "03_1513_twigwobble_loop", type = "SFX", startFrame = 1513, looping = true, fadeTime = 0.5, fadeStart = 0, fadeEnd = 1, trackPosition = "ThePlant" },
        },
    },

    TwigC = {
        startFrame = 1520,
        sounds = {
            { name = "03_1533_twig", type = "SFX", startFrame = 1533, trackPosition = "FrogTwig" },
            { name = "03_1513_twigwobble_loop", type = "SFX", startFrame = 1538, looping = true, fadeTime = 0.5, fadeEnd = 0, trackPosition = "ThePlant" },
        },
    },

    TwigD = {
        startFrame = 1589,
        sounds = {
            { name = "03_1589_ribbit", type = "SFX", startFrame = 1589, trackPosition = "FrogTwig" },
            { name = "03_1601_look_drop_skatter", type = "SFX", startFrame = 1601, trackPosition = "ThePlant" },

            -- Shut down environmental sounds at ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = TwigCRibbit, fadeTime = 2, fadeStart = 0.7, fadeEnd = 0, looping = true, },
            { name = "cricket1_loop", type = "Cricket", endFrame = TwigCRibbit, looping = true },
            { name = "cricket2_loop", type = "Cricket", endFrame = TwigCRibbit, looping = true },
            { name = "owl_loop", type = "Environment", endFrame = TwigCRibbit, looping = true },

            -- Bring them back up after ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = TwigCEnvStart, fadeTime = 2, fadeStart = 0, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", startFrame = TwigCEnvStart, trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startFrame = TwigCEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", startFrame = TwigCEnvStart, trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket", startFrame = TwigCEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startFrame = TwigCEnvStart, startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },

            { name = "04_mx_loop", type = "Music", startFrame = 1810, looping = true, fadeTime = 3, fadeStart = 0.05, fadeEnd = 1,},
        },
    },

    SquirrelA = {
        startFrame = 1626,
        sounds = {
            -- Checkpoint environmental sound and music loop
            { name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeEnd = 0.5, looping = true },
            { name = "distantfrogs_loop", type = "Environment", fadeTime = 2, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket",  startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },
            { name = "starry_night", type = "Canopy", fadeTime = 2, fadeEnd = 0.2, looping = true },
            { name = "04_mx_loop", type = "Music", looping = true, fadeTime = 3, fadeStart = 0.5, fadeEnd = 1,},

            { name = "04_larry_comp", type = "Morsing", trackPosition = "Wah", startFrame = 1626, fadeTime = MorseFadeLength, fadeStart = 0, fadeEnd = MorseFadeVolume, looping = true },
        },
    },

    SquirrelB = {
        startFrame = 1874,
        sounds = {
            { name = "04_larry_comp", type = "Morsing", trackPosition = "Wah", endFrame = 1874 },
            { name = "04_1876_quiver", type = "SFX", startFrame = 1876, trackPosition = "Wah"},
            { name = "04_1976_squirrelcrashloop", type = "SFX", startFrame = 1976, looping = true, volume = 1, trackPosition = "Wah" },

            { name = "04_mx_loop", type = "Music", startFrame = 1873, fadeTime = 1, fadeEnd = 0 },
            { name = "04_mx_loop_end", type = "Music", startFrame = 1873, volume = 1 },
			{ name = "04_1899_skitter", type = "MusicSFX", startFrame = 1899, trackPosition = "Wah"},
            { name = "04_1901_skitter_mx", type = "Music", startFrame = 1901, trackPosition = "Wah"},
        },
    },

    SquirrelC = {
        startFrame = 2001,
        sounds = {
            { name = "04_lamp_switch_on_or_off", type = "SFX", startFrame = 2008, trackPosition = "Wah", volume = 0.5},
            { name = "04_2032_door_sweep_hmph_door", type = "SFX", volume = 1, startFrame = 2032, trackPosition = "Wah" },
            { name = "04_1976_squirrelcrashloop", type = "SFX", startFrame = 2032, fadeTime = 1, fadeEnd = 0 },
            { name = "04_lamp_switch_on_or_off", type = "SFX", startFrame = 2122, trackPosition = "Wah", volume = 0.5},
        }
    },

    SquirrelD = {
        startFrame = 2137,
        sounds = {
            { name = "04_2136_ribbit_zap", type = "SFX", volume = 0.3, startFrame = 2137, trackPosition = "FrogSquirrel", volume = 0.7},

            -- Shut down environmental sounds at ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = SquirrelDRibbit, fadeTime = 2, fadeStart = 0.7, fadeEnd = 0, looping = true, },
            { name = "cricket1_loop", type = "Cricket", endFrame = SquirrelDRibbit, looping = true },
            { name = "cricket2_loop", type = "Cricket", endFrame = SquirrelDRibbit, looping = true },
            { name = "owl_loop", type = "Environment", endFrame = SquirrelDRibbit, looping = true },
        },
    },

    SquirrelE = {
        startFrame = 2192,
        sounds = {
            { name = "04_2191_sos_gulp_boing", type = "SFX", startFrame = 2191, trackPosition = "FrogSquirrel"},
            { name = "04_skatter", type = "SFX", startFrame = 2291, trackPosition = "Wah"},

            -- Bring them back up after ribbit
            { name = "distantfrogs_loop", type = "Environment", startFrame = SquirrelDEnvStart, fadeTime = 2, fadeStart = 0, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", startFrame = SquirrelDEnvStart, trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startFrame = SquirrelDEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", startFrame = SquirrelDEnvStart, trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket", startFrame = SquirrelDEnvStart, startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startFrame = SquirrelDEnvStart, startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },

            { name = "05_mx_loop", type = "Music", startFrame = 2315, looping = true, fadeTime = 3, fadeStart = 0.05, fadeEnd = 1,},
        },
    },

    FrogA = {
        startFrame = 2370,
        sounds = {
            -- Checkpoint environmental sound and music loop
            { name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeEnd = 0.5, looping = true },
            { name = "distantfrogs_loop", type = "Environment", fadeTime = 2, fadeEnd = 0.7, looping = true, trackPosition = "DistantFrogLocator", },
            { name = "cricket1_startup", type = "Cricket", trackPosition = "Cricket1Locator", },
            { name = "cricket1_loop", type = "Cricket", startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket1Locator", },
            { name = "cricket2_startup", type = "Cricket", trackPosition = "Cricket2Locator", },
            { name = "cricket2_loop", type = "Cricket",  startOffset = CricketLoopOffset, looping = true, trackPosition = "Cricket2Locator", },
            { name = "owl_loop", type = "Environment", startOffset = OwlLoopOffset, looping = true, trackPosition = "OwlLocator", },
            { name = "starry_night", type = "Canopy", fadeTime = 2, fadeEnd = 0.2, looping = true },
            { name = "05_mx_loop", type = "Music", looping = true, fadeTime = 3, fadeEnd = 1,},

            { name = "05_all_comp", type = "Morsing", trackPosition = "Wah", fadeTime = MorseFadeLength, fadeStart = 0, fadeEnd = MorseFadeVolume, looping = true },
        },
    },

    FrogB = {
        startFrame = 2580,
        sounds = {
            { name = "05_all_comp", type = "Morsing", endFrame = 2580 },
            { name = "05_2580_freak_skitter_eyesup", type = "MusicSFX", startFrame = 2580, trackPosition = "Wah"},

            { name = "05_mx_loop", type = "Music", startFrame = 2580, fadeTime = 1, fadeEnd = 0 },
            { name = "05_mx_loop_end", type = "Music", startFrame = 2580, volume = 1 },
            { name = "05_2602_mx_skitter", type = "Music", startFrame = 2602, trackPosition = "Wah"},
        },
    },

    FrogC = {
        startFrame = 2753,
        sounds = {
            { name = "rrrrribbit", type = "SFX", startFrame = FrogCRibbit, trackPosition = "Frog", volume = 0.7},

            -- Checkpoint environmental sound and music loop
            { name = "distantfrogs_loop", type = "Environment", startFrame = FrogCRibbit, fadeTime = 2, fadeStart = 0.7, fadeEnd = 0, looping = true, },
            { name = "cricket1_loop", type = "Cricket", endFrame = FrogCRibbit, looping = true },
            { name = "cricket2_loop", type = "Cricket", endFrame = FrogCRibbit, looping = true },
            { name = "owl_loop", type = "Environment", endFrame = FrogCRibbit, looping = true },

            { name = "05_2791_chase", type = "SFX", startFrame = 2798, trackPosition = "Wah" },
            { name = "05_2809_mx_chase", type = "Music", startFrame = 2809, volume = 1, trackPosition = "Wah" },
            { name = "05_3046_mx_stretch", type = "Music", startFrame = 3046, trackPosition = "Wah" },
        },
    },

    FrogD = {
        startFrame = 3074,
        sounds = {
            { name = "05_3074_stretch_bang", type = "SFX", startFrame = 3074, trackPosition = "Wah" },
        },
    },

    EndA = {
        startFrame = 3179,
        sounds = {
            { name = "06_3213", type = "SFX", startFrame = 3214, trackPosition = "Wah" },
        },
    },

    EndB = {
        startFrame = 3447,
        sounds = {
            { name = "06_3445_escape", type = "SFX", startFrame = 3449, trackPosition = "Wah" },
            { name = "06_alive_comp", type = "Morsing", trackPosition = "Wah", startFrame = 3507, },
            { name = "06_badfrog_comp", type = "Morsing", trackPosition = "Annie", startFrame = 3609 },
            { name = "06_3631_runaway_skitterout", type = "MusicSFX", startFrame = 3631, trackPosition = "Wah" },

            { name = "06_mx_3411_bugwait", type = "Music", startFrame = 3411, volume = 1, trackPosition = "Wah" },
            { name = "06_mx_3451_end", type = "Music", startFrame = 3451, volume = 1, trackPosition = "Wah"  },
        },
    },

    EndC = {
        startFrame = 3766,
        sounds = {
            { name = "06_3836", type = "SFX", startFrame = 3836, trackPosition = "Wah" },
        },
    },

    ThatsAllFolks = {
        startFrame = 3926,
		sounds = {
			{ name = "ambience_loop", type = "Ambient", fadeTime = 2, fadeEnd = 0, looping = true },
            { name = "starry_night", type = "Canopy", fadeTime = 2, SfadeEnd = 0, looping = true },
		},
	},
}
