
require "client/scripts/story"


Segment = {}
Segment.__index = Segment

Segment.new = function(def)
	local ob = {};
	ob.def = def or {};
	ob.name = def.name or "";
	ob.playing = def.playing or false;
	ob.hasBegun = def.hasBegun or false;
	ob.duration = def.duration or 0;
	ob.looping = def.looping or false;
	ob.active = def.active or {};
	ob.activatedObjs = def.activatedObjs or {};
	return setmetatable(ob, Segment);
end

function Segment:onUpdate()
 	if (self:currentTime() > self.startTime + self.duration)  then
 		if (self.looping) then
 			self:startPlaying();
 		else
 			self.playing = false;
 			Log.message("Segment finished: " .. self.streamNumber);
    		Events.send(self, "segmentFinished", self.streamNumber);
 		end	
 	end
    local trigger = self:lookingAtTrigger();
    --Log.error(" Animation time"  .. scene.actors["Gracie"].animation[self.name]:getTime())
    if (trigger) then
        Log.message("looking at trigger : " .. trigger);
    else
    end
end
function Segment:isPlaying() 
	return self.playing;
end
function Segment:getName()
	return self.name;
end

function Segment:startPlaying(sn)
    Log.message("Starting to play: " .. self.name);

    self:activateObjects();
    self.streamNumber = sn;
    self.playing = true;
    self.startTime = self:currentTime();
    self:playAnimations();

    self.hasBegun = true;

end

function Segment:stopPlaying()
    self.playing = false;
end

function Segment:deactivateObjects(oldactorlist, newactorlist)

end

creator = {
    Actor = function(objectName, def)
    return scene:createActor(objectName, def[objectName]);
    end,
    Trigger = function(objectName, def)
    return scene:createActor(objectName, def[objectName]);
    end,
    Camera = function(objectName,def)
    return scene:createActor(objectName, def[objectName]);
    end,
    Set = function(objectName,def)
    return scene:createActor(objectName, def[objectName]);
    end,
    Light = function(objectName,def)
    return scene:createActor(objectName, def[objectName]);
    end,
}

function Segment:createObject(objectName)
    Log.message("Creating " .. objectName);
    Log.message(table.tostring(def[objectName]));
    local a = creator[def[objectName].objectType](objectName, def);
    self.activatedObjs[objectName] = a;
     -- Add any components needed
    if (def[objectName].components) then
        Log.message(table.tostring(def[objectName].components));
        for _, compData in ipairs(def[objectName].components) do
            Log.message("entry " ..table.tostring( compData) );
            local class = _G[compData.name]
            --Log.message("class " .. class );
            if (class == nil) then
                Log.message("Unknown component type: " .. tostring(compData.name))
            else
                a:addComponent(compData.name, class:new(compData.params))
                Log.message("Added component: " .. compData.name .. " to " .. a.name)
            end
        end
    end
    return a;
end

function Segment:activateObjects() 
    --TODO check against multiple scene lists, not just actors
	for _, objectName in ipairs(self.active) do
        if (scene.actors ~= nil) then		
            self.activatedObjs[objectName] = scene.actors[objectName] or self:createObject(objectName);
        else
             self.activatedObjs[objectName] = self:createObject(objectName);
        end
	end
end

function Segment:lookingAtTrigger()


    for _,target in pairs(self.activatedObjs) do
        if (target) then
            local detector = self.def[target:getName()];
             if (detector and detector.components) then
                 detector = detector.components["LookAtDetector"]
                 Log.message("found component for" .. target:getName());
            --     if (detector and detector:lookedAt()) then
            --         return target
            --     end
             end
        end
    end

    return nil
end

function Segment:playAnimations()
    for _, actorName in ipairs(self.active) do
    	if (self.activatedObjs[actorName] and self.activatedObjs[actorName].animation and self.activatedObjs[actorName].animation[self.name]) then
      	 	self.activatedObjs[actorName].animation[self.name]:play();
    	end
    end
end


function Segment:currentTime()
    return os.clock();
end




