
Color = {}
Color.__index = Color

Color.copy = function(self) 
	return Color.new(self.r, self.g, self.b, self.a);
end

Color.unpack = function(self)
	return self.r, self.g, self.b, self.a;
end

Color.new = function(r, g, b, a)
	return setmetatable({ r = r or 0.0, g = g or 0.0, b = b or 0.0, a = a or 1.0 }, Color)
end

Color.__add = function(a, b)
	return Color.new(a.r + b.r, a.g + b.g, a.b + b.b, a.a + b.a);
end

Color.__sub = function(a, b)
	return Color.new(a.r - b.r, a.g - b.g, a.b - b.b, a.a - b.a);
end

Color.__mul = function(a, b)
	return Color.new(a.r * b.r, a.g * b.g, a.b * b.b, a.a * b.a);
end

Color.__eq = function(a, b)
	return a.r == b.r and a.g == b.g and a.b == b.b and a.a == b.a;
end

Color.__lt = function(a, b)
	return a.r < b.r and a.g < b.g and a.b < b.b and a.a < b.a;
end

Color.__le = function(a, b)
	return a.r <= b.r and a.g <= b.g and a.b <= b.b and a.a <= b.a;
end

Color.scale = function(self, s)
	if (type(s) == "table") then
		self.r = self.r * s.r;
		self.g = self.g * s.g;
		self.b = self.b * s.b;
		self.a = self.a * s.a;		
	elseif (type(s) == "number") then
		self.r = self.r * s;
		self.g = self.g * s;
		self.b = self.b * s;
		self.a = self.a * s;
	end
end


Color.__tostring = function(self)
	return "{"..self.r..","..self.g..","..self.b..","..self.a.."}";
end

Color.set = function(self, r, g, b, a)
	self.r = (r or 0);
	self.g = (g or 0);
	self.b = (b or 0);
	self.a = (a or 1);
end

Color.lerp = function(t, a, b) 
	local out = a + (b - a) * t
	return out
end

Color.white = Color.new(1.0, 1.0, 1.0, 1.0)
Color.grey = Color.new(0.5,0.5,0.5,1.0)
Color.black = Color.new(0.0, 0.0, 0.0, 1.0)
Color.yellow = Color.new(1.0, 1.0, 0.0, 1.0)
Color.cyan = Color.new(0.0, 1.0, 1.0, 1.0)
Color.magenta = Color.new(1.0, 0.0, 1.0, 1.0)
Color.red = Color.new(1.0, 0.0, 0.0, 1.0)
Color.blue = Color.new(0.0, 0.0, 1.0, 1.0)
Color.green = Color.new(0.0, 1.0, 0.0, 1.0)

setmetatable(Color, { __call = function(_, ...) return Color.new(...) end })
