-- Prebuilt components
-- Debug camera controller (works best in windows) 
-- WASD moves the camera forward, left, back and right
-- left-click and drag to rotate the camera
-- at a minimum, a component is just a data container.
FlyCameraController = 
{
	linearVelocity = Vector.new(0,0,0),		-- linear velocity
	angularVelocity = Vector.new(0,0,0),	-- angular velocity
	moveSpeed = 10.0,						-- units per second
	turnSpeed = math.rad(50),							
	lookSpeed = math.rad(50),
	linearDamp = 0.95,
	angularDamp = 0.95,
	sensitivity = 60,
	lastUpdate = os.clock(),
}

FlyCameraController.__index = FlyCameraController

function FlyCameraController.new(ob) 
	return setmetatable(ob or {}, FlyCameraController);
end

-- components can also act as behaviors
function FlyCameraController:onStart() 
end

-- demonstrates the use of the Input library to query buttons
function FlyCameraController:onUpdate()
	if(GUI.isBlocking()) then
		return;
	end
	
	if(not(self.object:isActive())) then
		return;
	end

	local linearAcc = Vector(0,0,0);
	
	if (Input.isKeyDown(KB_W)) then
		linearAcc.z = linearAcc.z - self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_S)) then
		linearAcc.z = linearAcc.z + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_A)) then
		linearAcc.x = linearAcc.x - self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_D)) then
		linearAcc.x = linearAcc.x + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_Z)) then
		linearAcc.y = linearAcc.y + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_X)) then
		linearAcc.y = linearAcc.z - self.moveSpeed;
	end
	
	local t = os.clock();
	local dt = t - self.lastUpdate;
	self.lastUpdate = t;
	self.linearVelocity = self.linearVelocity + linearAcc * 2.0 * dt;
	self.linearVelocity.x = math.min(math.max(self.linearVelocity.x, -self.moveSpeed*dt), self.moveSpeed*dt);
	self.linearVelocity.y = math.min(math.max(self.linearVelocity.y, -self.moveSpeed*dt), self.moveSpeed*dt);
	self.linearVelocity.z = math.min(math.max(self.linearVelocity.z, -self.moveSpeed*dt), self.moveSpeed*dt);
	
	local angularAcc = Vector(0,0,0);
	if(Input.isMouseMoving() and Input.getMouseButton(MB_L_BUTTON)) then
		local mouseDelta = Input.getMouseDelta() * self.sensitivity;
		if mouseDelta.x < 0.0 then mouseDelta.x = -0.1 elseif mouseDelta.x > 0.0 then mouseDelta.x = 0.1 end
		if mouseDelta.y < 0.0 then mouseDelta.y = -0.1 elseif mouseDelta.y > 0.0 then mouseDelta.y = 0.1 end
		angularAcc.x = -(mouseDelta.y) * self.lookSpeed;
		angularAcc.y = -(mouseDelta.x) * self.turnSpeed;
	end
	
	self.angularVelocity = self.angularVelocity + angularAcc * 2.0 * dt;
	self.angularVelocity.x = math.min(math.max(self.angularVelocity.x, -self.lookSpeed*dt), self.lookSpeed*dt);
	self.angularVelocity.y = math.min(math.max(self.angularVelocity.y, -self.turnSpeed*dt), self.turnSpeed*dt);
	
	local localT = self.object:getLocal();
	
	local y = Quaternion.new(); 
	local p = Quaternion.new(); 
		
	y:setAxisAngle(Vector.up, self.angularVelocity.y);
	p:setAxisAngle(Vector.right:rotateBy(localT.rotation), self.angularVelocity.x);
	
	local deltaRot = p * y;
	
	localT.position = localT.position + self.linearVelocity:rotateBy(localT.rotation);
	localT:applyRotation(deltaRot);
	
	self.object:setLocal(localT);

	-- apply damping
	self.linearVelocity = self.linearVelocity * self.linearDamp;
	self.angularVelocity = self.angularVelocity * self.angularDamp;
end