
local __scrollButtonImpl = function(bar, button, dir)
    button.onButtonPressed = function(self)
        local secs = os.clock()
        if(self.lastTime == nil) then
            self.lastTime = secs
        end
        local dt = secs - self.lastTime
        if(self.pressed) then
            if(self.delay == nil) then
                bar:setScrollPosition(bar.pos + bar.step*dir)
                self.delay = bar.step_delay
                return
            end
            self.delay = self.delay - dt
            if(self.delay <= 0) then
                bar:setScrollPosition(bar.pos + bar.step*dir)
                self.delay = bar.step_repeat
            end
        end
        self.lastTime = secs
    end

    button.onButtonUp = function(self)
        self.delay = nil
    end
end

-- horizontal scroll bar
GUI.hscrollbar = function(parent)
    local size = GUI.theme.hscrollbar.track.image.size
    local bar = GUI.frame(parent, px(0,0), size, "")
    bar:dock(DOCK_BOTTOM)

    local track = GUI.frame(bar, px(0,0), GUIAutoSize(FILL_PARENT, FILL_PARENT), "hscrollbar.track")
    track:anchor(true, false, true, true)
    local left = __button(bar, GUIAligned(ALIGN_CENTER_LEFT), GUIAutoSize(WRAP_CONTENT, WRAP_CONTENT), "", "hscrollbar.left")
    left:anchor(true, false, false, true)
    local right = __button(bar, GUIAligned(ALIGN_CENTER_RIGHT), GUIAutoSize(WRAP_CONTENT, WRAP_CONTENT), "", "hscrollbar.right")
    right:anchor(false, false, true, true)

    local grip = __button(bar, GUIAligned(ALIGN_CENTER), GUIAutoSize(WRAP_CONTENT,WRAP_CONTENT), "", "hscrollbar.grip")

    bar.pos = 0.0
    bar.min = 0.0
    bar.max = 1.0
    bar.step = 0.125
    bar.step_delay = 0.5
    bar.step_repeat = 0.05

    __scrollButtonImpl(bar, left, -1)
    __scrollButtonImpl(bar, right, 1)

    bar.setScrollRange = function(min, max)
        bar.min = min
        bar.max = max
    end

    bar.setScrollPosition = function(self, p)
    -- clamp and store requested position
        p = math.min(bar.max, math.max(bar.min, p))
        bar.pos = p

        -- calculate normalized position (0 >= t <= 1.0)
        local t = (p - bar.min) / (bar.max - bar.min)

        -- transform t to pixel coordinates
        local minx	= left:getWidth()
        local maxx	= bar:getWidth() - right:getWidth() - grip:getWidth()
        local x 	= t * (maxx - minx) + minx--math.min(maxy, math.max(miny, p + miny))

        -- move the grip
        grip:moveTo(x, 0)
    end

    bar.getScrollPosition = function(self)
        return bar.pos
    end

    grip.__show = function(self)
        if(bar.disabled) then
            return
        end
        local minx	= left:getWidth()
        local maxx	= bar:getWidth() - right:getWidth()
        local width = maxx - minx

        if(width < grip:getWidth()) then
            grip:hide()
        else
            grip:show()
        end
    end

    grip.onButtonDown = function(self)
        self.drag = true
        self.dragoffset = GUI.getCursorPosition() - self:getWorldPosition()
        return true
    end

    grip.onButtonUp = function(self)
        self.drag = false
        return true
    end

    grip.onButtonPressed = function(self)
        if(self.drag) then
            local newpos = GUI.getCursorPosition() - self.dragoffset - bar:getWorldPosition()
            local minx	= left:getWidth()
            local maxx	= bar:getWidth() - right:getWidth() - grip:getWidth()
            local t = (newpos.x - minx) / (maxx - minx)
            bar.pos = t * (bar.max - bar.min)
            bar:setScrollPosition(bar.pos)
        end
        return true
    end

    bar.disable = function(self)
        left:disable()
        right:disable()
        grip:hide()
        bar.disabled = true
    end

    bar.enable = function(self)
        left:enable()
        right:enable()
        grip:__show()
        bar.disabled = false
    end

    local scrollbarHandler =
    {
        onMouseDown = function(self)
            if(bar.disable) then
                return
            end

            -- clicking on anything other than the grip or the buttons
            -- results in page up/page dwn
            if(not(parent.view) or parent.view:getNumChildren() == 0) then
                return
            end

            local content = parent.view:getChild(1)
            local viewSize = parent.view:getSize()
            local contentSize = content:getSize()
            local mpos = GUI.getCursorPosition() - bar:getWorldPosition()
            local gripPos = grip:getLocalPosition()
            local gripSize = grip:getSize()

            local pageStep = (viewSize.x/contentSize.x) * (bar.max - bar.min)
            if(gripPos.x+gripSize.x*0.5 > mpos.x) then
                bar:setScrollPosition(bar.pos - pageStep)
            else
                bar:setScrollPosition(bar.pos + pageStep)
            end
        end,

        onResize = function(self, w, h)
            self:setScrollPosition(bar.pos)
            grip:__show()
        end,
    }

    bar:addEventHandler(scrollbarHandler)

    return bar
end

-- vertical scrollbar
-- this should be pretty much the same as the horizontal scrollbar
-- just transposed.
GUI.vscrollbar = function(parent)
    local size = GUI.theme.vscrollbar.track.image.size
    local bar = GUI.frame(parent, px(0,0), size, "")
    bar:dock(DOCK_RIGHT)

    local track = GUI.frame(bar, px(0,0), GUIAutoSize(WRAP_CONTENT, FILL_PARENT), "vscrollbar.track")
    track:anchor(false, true, true, true)
    local up = __button(bar, GUIAligned(ALIGN_TOP_CENTER), GUIAutoSize(WRAP_CONTENT, WRAP_CONTENT), "", "vscrollbar.up")
    up:anchor(false, true, true, false)
    local down = __button(bar, GUIAligned(ALIGN_BOTTOM_CENTER), GUIAutoSize(WRAP_CONTENT, WRAP_CONTENT), "", "vscrollbar.down")
    down:anchor(false, false, true, true)

    local grip = __button(bar, GUIAligned(ALIGN_CENTER), GUIAutoSize(WRAP_CONTENT,WRAP_CONTENT), "", "vscrollbar.grip")

    bar.pos = 0.0
    bar.min = 0.0
    bar.max = 1.0
    bar.step = 0.125
    bar.step_delay = 0.5
    bar.step_repeat = 0.05

    __scrollButtonImpl(bar, up, -1)
    __scrollButtonImpl(bar, down, 1)

    bar.setScrollRange = function(min, max)
        bar.min = min
        bar.max = max
    end

    bar.setScrollPosition = function(self, p)
    -- clamp and store requested position
        p = math.min(bar.max, math.max(bar.min, p))
        bar.pos = p

        -- calculate normalized position (0 >= t <= 1.0)
        local t = (p - bar.min) / (bar.max - bar.min)

        -- transform t to pixel coordinates
        local miny	= up:getHeight()
        local maxy	= bar:getHeight() - down:getHeight() - grip:getHeight()
        local y 	= t * (maxy - miny) + miny--math.min(maxy, math.max(miny, p + miny))

        -- move the grip
        grip:moveTo(0, y)
    end

    grip.__show = function(self)
        if(bar.disabled) then
            return
        end
        local miny	= up:getHeight()
        local maxy	= bar:getHeight() - down:getHeight()
        local height = maxy - miny
        if(height > grip:getHeight()) then
            grip:show()
        else
            grip:hide()
        end
    end

    bar.disable = function(self)
        up:disable()
        down:disable()
        grip:hide()
        bar.disabled = true
    end

    bar.enable = function(self)
        up:enable()
        down:enable()
        grip:__show()
        bar.disabled = false
    end

    bar.getScrollPosition = function(self)
        return bar.pos
    end

    grip.onButtonDown = function(self)
        self.drag = true
        self.dragoffset = GUI.getCursorPosition() - self:getWorldPosition()
        return true
    end

    grip.onButtonUp = function(self)
        self.drag = false
        return true
    end

    grip.onButtonPressed = function(self)
        if(self.drag) then
            local newpos = GUI.getCursorPosition() - self.dragoffset - bar:getWorldPosition()
            local miny	= up:getHeight()
            local maxy	= bar:getHeight() - down:getHeight() - grip:getHeight()
            local t = (newpos.y - miny) / (maxy - miny)
            bar.pos = t * (bar.max - bar.min)
            bar:setScrollPosition(bar.pos)
        end
        return true
    end

    local scrollbarHandler =
    {
        onMouseDown = function(self)
            if(bar.disable) then
                return
            end

            if(not(parent.view) or parent.view:getNumChildren() == 0) then
                return
            end

            local content = parent.view:getChild(1)
            local viewSize = parent.view:getSize()
            local contentSize = content:getSize()
            local mpos = GUI.getCursorPosition() - bar:getWorldPosition()
            local gripPos = grip:getLocalPosition()
            local gripSize = grip:getSize()

            local pageStep = (viewSize.y/contentSize.y) * (bar.max - bar.min)
            if(gripPos.y+gripSize.y*0.5 > mpos.y) then
                bar:setScrollPosition(bar.pos - pageStep)
            else
                bar:setScrollPosition(bar.pos + pageStep)
            end
        end,

        onResize = function(self, w, h)
            self:setScrollPosition(bar.pos)
            grip:__show()
        end,
    }

    bar:addEventHandler(scrollbarHandler)

    return bar
end

GUI.scrollframe = function(parent, pos, size, mode, style)
    local frame = GUI.frame(parent, pos, size, "")
    --	frame:anchor(true, true, true, true)
    frame:dock(DOCK_FILL)

    if(mode == nil) then
        mode = "both"
    end

    local hscroll = nil
    local vscroll = nil

    if(mode == "horizontal" or mode == "both") then
        hscroll = GUI.hscrollbar(frame)
        hscroll:setScrollPosition(0)
    end

    if(mode == "vertical" or mode == "both") then
        vscroll = GUI.vscrollbar(frame)
        vscroll:setScrollPosition(0)
    end

    local view = GUI.frame(frame, pos, size, style)
    view:dock(DOCK_FILL)

    local scrollHandler = {
        onMouseWheel = function(self, delta)
            local content = view:getChild(1)
            local contentSize = content:getSize()

            delta = -delta -- invert delta
            vscroll:setScrollPosition(vscroll:getScrollPosition() + (delta/contentSize.y))
            return true
        end,
        onPassiveUpdate = function(self)
            local viewSize = view:getSize()
            if(view:getNumChildren() == 0) then
                return false
            end

            local content = view:getChild(1)

            local contentSize = content:getSize()
            local offsetx = 0
            local offsety = 0

            if(vscroll) then
                local ty = vscroll.pos / (vscroll.max - vscroll.min)
                local maxy = (contentSize.y - viewSize.y)
                if(maxy < 0) then
                    maxy = 0
                end
                if(viewSize.y > contentSize.y) then
                    vscroll:disable()
                else
                    vscroll:enable()
                end
                offsety = ty * maxy
            end

            if(hscroll) then
                local tx = hscroll.pos / (hscroll.max - hscroll.min)
                local maxx = (contentSize.x - viewSize.x)
                if(maxx < 0) then
                    maxx = 0
                end

                if(viewSize.x > contentSize.x) then
                    hscroll:disable()
                else
                    hscroll:enable()
                end
                offsetx = tx * maxx
            end

            content:moveTo(-offsetx, -offsety)
            return false
        end
    }

    frame:addEventHandler(scrollHandler)

    frame.view = view

    return view
end
