
--
-- Patch the slider to make it usable.  Need to clean this up
-- and move it back into the common code
--

local hoveredHandler = function(name)
    return {
        onMouseEnter = function(self)
            self:setStyle(name..".hovered")
            self.hovered = true
            return false
        end,
        onMouseLeave = function(self)
            self:setStyle(name..".default")
            self.hovered = false
            return false
        end,
    }
end

-- implements a simple horizontal slider control
-- x: horizontal position of the slider
-- y: vertical position of the slider
-- initialValue: initial slider value
-- minValue: minimum slider value
-- maxValue: maximum slider value
-- numTicks:
GUI.slider = function(parent, pos, width, caption, initialValue, minValue, maxValue, numTicks)
    initialValue = initialValue or 0
    minValue = minValue or 0
    maxValue = maxValue or 100
    numTicks = numTicks or 0

    local range = maxValue - minValue
    assert(range ~= 0)

    local trackSize = GUI.theme.slider.track.image.size
    local labelSize = GUI.textSize(caption, "slider.text")
    local gripSize = GUI.theme.slider.grip.default.image.size
    local halfGripSize = px(gripSize.x * 0.5, gripSize.y * 0.5)

    local slider = GUI.frame(parent, px(pos.x, pos.y), px(width, labelSize.y + gripSize.y), "")
    slider.caption = caption
    local label = GUI.label(slider, px(0, 0), px(width, labelSize.y), caption, "slider.text")
    label:anchor(true, false, true, false)

    local controlSize = px(width, gripSize.y)
    local control = GUI.frame(slider, px(0, labelSize.y), controlSize, "")
    control:anchor(true, false, true, false)
    local trackPos = px(halfGripSize.x, (controlSize.y - trackSize.y)*0.5)
    local trackWidth = controlSize.x - gripSize.x

    local track = GUI.frame(control, trackPos, px(trackWidth, trackSize.y), "slider.track")
    track:anchor(true, false, true, false)
    local fill = GUI.frame(control, trackPos, px(trackWidth, trackSize.y), "slider.fill")
    local grip = GUI.frame(control, px(0,0), gripSize, "slider.grip.default")

    local snap = function(x)
        if numTicks > 1 then
            local r = range / numTicks
            return math.floor((x + r * 0.5) / r) * r
        end
        return x
    end
    
    local controlHandler = {
        onMouseDown = function(self)
            slider.drag = true
            return true
        end,
        onMouseUp = function(self)
            slider.drag = false
            return true
        end,
        onUpdate = function(self)
            if slider.drag then
                local value = snap(slider:posToValue(GUI.getCursorPosition().x))
                slider:setValue(value)
            end
            return true
        end,
        onPassiveUpdate = function(self)
            label:setText(slider.caption..": "..string.format("%.2f", slider.value))
        end,
        onResize = function(self, w, h)
            slider:updatePosition()
        end,
    }

    slider.posToValue = function(self, pos)
        local trackSize = track:getSize()
        local trackPos = track:getWorldPosition()
        local value = (pos - trackPos.x) * range / trackSize.x + minValue
        return math.max(minValue, math.min(maxValue, value))
    end

    slider.valueToPos = function(self, value)
        local trackSize = track:getSize()
        return (value - minValue) * trackSize.x / range
    end

    slider.updatePosition = function(self)
        local pos = self:valueToPos(self.value)
        grip:moveTo(pos, 0)
        fill:resize(pos, trackSize.y)
    end

    slider.setValue = function(self, value)
        value = math.min(maxValue, math.max(minValue, value))
        if self.value ~= value then
            local oldValue = self.value
            self.value = value
            self:updatePosition()
            if self.onValueChanged then
                self:onValueChanged(value, oldValue)
            end
        end
    end

    local eatEventsHandler = {
        onMouseDown = function(self)
            return true
        end,
        onMouseUp = function(self)
            return true
        end,
    }
    slider:addEventHandler(eatEventsHandler)

    slider:setValue(initialValue)

    control:addEventHandler(controlHandler)
    grip:addEventHandler(hoveredHandler("slider.grip"))

    return slider
end
