
GUI.treelist = function(parent, pos, size)
    local treelist = GUI.list(parent, GUIPos(0, 0), GUISize(SZ_FILL, 28), "", false)
    treelist:anchor(true, true, true, true)
    treelist.__istreelist = true
    treelist.items = {}
    treelist.indent = 0
    treelist.indent_step = 16

    treelist.deselect = function(self)
        treelist.selected = status(treelist.selected)
        if(treelist.selected) then
            treelist.selected:setStyle("treelist.item")
            if(treelist.onItemDeselected) then
                treelist:onItemDeselected(treelist.selected)
            end
            treelist.selected = nil
        end
    end

    treelist.select = function(self, item)
        treelist.selected = status(treelist.selected)

        if(type(item) == "string") then
            item = self.items[item]
        end

        if(treelist.selected) then
            self:deselect()
        end

        if(item and item ~= treelist.selected) then
            treelist.selected = item
            treelist.selected:setStyle("treelist.item_selected")
            if(treelist.onItemSelected) then
                treelist:onItemSelected(item)
            end
        else
            treelist.selected = nil
        end
    end

    treelist.__newitem = function(parent, name)
        local item = GUI.frame(parent, GUIPos(0,0),  GUISize(SZ_FILL, 28), "treelist.item")
        item.name = name
        item:anchor(true, false, true, false)
        local minimizeButton = GUI.minimizer(item, GUIAligned(ALIGN_CENTER_LEFT), false)
        minimizeButton:move(parent.indent, 0)

        local label = GUI.label(item, GUIPos(minimizeButton:getWidth() + parent.indent, 0), GUISize(SZ_FILL, 28), name, "h1")
        item:anchor(true, false, true, false)
        label:setTextAlign(ALIGN_CENTER_LEFT)

        -- create item sublist
        item.sublist = GUI.list(parent, GUIPos(0,0), GUISize(SZ_FILL, 28), "", false)
        item.sublist:hide()
        item.sublist:anchor(true, false, true, false)
        item.sublist.items = {}
        item.sublist.__istreelist = true
        item.sublist.button = minimizeButton
        item.sublist.indent = parent.indent + treelist.indent_step

        item.sublist.__destroy = function(self)
            parent.items[name] = nil
            item.sublist:destroy()
            item:destroy()
        end

        item.collapse = function(self, recursively)

            if(recursively) then
                for _, it in pairs(self.sublist.items) do
                    it.sublist.button:minimize()
                    it:collapse(recursively)
                end
            end

            self.sublist:hide()

            treelist.selected = status(treelist.selected)
            if(treelist.selected) then
                if(not(treelist.selected:isVisible())) then
                    treelist:select(self)
                end
            end
        end

        item.expand = function(self, recursively)
            if(recursively) then
                for _, it in pairs(self.sublist.items) do
                    it.sublist.button:maximize()
                    it:expand(recursively)
                end
            end
            self.sublist:show()
        end

        function minimizeButton:onMinimize()
            local recursively = false
            if(Input.isModifierDown(KB_LCTRL) or Input.isModifierDown(KB_RCTRL)) then
                recursively = true
            end
            item:collapse(recursively)
        end

        function minimizeButton:onMaximize()
            local recursively = false
            if(Input.isModifierDown(KB_LCTRL) or Input.isModifierDown(KB_RCTRL)) then
                recursively = true
            end
            item:expand(recursively)
        end

        local visibilityHandler = {
            onPassiveUpdate = function(self)
                if(Util.numKeys(item.sublist.items)==0) then
                    self:hide()
                else
                    self:show()
                end
            end
        }

        local clickHandler = {
            onMouseDblClick = function(item)
                if(treelist.selected ~= item) then
                    treelist:select(item)
                else
                    treelist:deselect()
                end
                if(treelist.onItemDblClick) then
                    treelist:onItemDblClick(item)
                end
            end,
            onMouseDown = function(item)
                if(treelist.selected ~= item) then
                    treelist:select(item)
                else
                    treelist:deselect()
                end
                if(treelist.onItemClick) then
                    treelist:onItemClick(item)
                end
            end,
        }

        item:addEventHandler(clickHandler)
        minimizeButton:addEventHandler(visibilityHandler)

        return item
    end

    treelist.populate = function(self, table, parent)
        for key, value in pairs(table) do
            local node = self:addNode(parent, key)
            self:populate(value, node)
        end

        if(parent and parent.items) then
            for key, value in pairs(parent.items) do
                if(table[key] == nil) then
                    if(value.sublist) then
                        value.sublist:destroy()
                    end
                    value:destroy()
                    parent.items[key] = nil
                end
            end
        end
    end

    treelist.clear = function(self)
        for _, item in pairs(self.items) do
            item.sublist:__destroy()
        end
        self.items = {}
    end

    treelist.addNode = function(self, parent, name)
        if(parent==nil) then
            parent = self
        end

        assert(parent.__istreelist)

        local item = parent.items[name]
        if(item == nil) then
            item = self.__newitem(parent, name)
            parent.items[name] = item
        end

        return item.sublist
    end

    treelist.collapse = function(self, recursively)
        for _, item in pairs(self.items) do
            item.sublist.button:minimize()
            item:collapse(recursively)
        end
    end

    treelist.expand = function(self, recursively)
        for _, item in pairs(self.items) do
            item.sublist.button:maximize()
            item:expand(recursively)
        end
    end

    return treelist
end

