
-- templatized URIClass
URIClass = function(name, rtti, basedir, ext) 
	local out = _G[name] or {}
	out.basedir = basedir; 
	out.rtti = rtti

	out.uri = function(filename)
		local uri = ".:".._G[name].basedir.."/"..filename;
		if(type(ext) == "table") then
			for i, v in ipairs(ext) do
				
			end
		end
		return ".:".._G[name].basedir.."/"..filename..".ext";
	end

	return out;
end

-- these are native resource types that are referenced by URI.
Model			= URIClass("Model", "ModelResource", "/model", "mxm");
Animation		= URIClass("Animation", "AnimResource", "/anims", "mxa");
Shader 			= URIClass("Shader", "ShaderFxResource", "/shaders", "rsfx");
Texture			= URIClass("Texture", "TextureResource", "/textures", "dds");
Font			= URIClass("Font", "FontResource", "/fonts", "fnt");
Audio			= URIClass("Audio", "AudioResource", "/sound", { "mp3", "ogg", "wav" } );
ParticleSystem	= URIClass("ParticleSystem", "ParticleSystemResource", "/particles", "mxp" );
Flipbook		= URIClass("Flipbook", "FlipbookResource", "/flipbooks", "mfb" );
VideoCube		= URIClass("VideoCube", "VideoCubeResource", "/videos", "mvc" )

-- Override the addResource function to keep track of uris added to it
ResourceGroup._superAddResource = ResourceGroup.addResource
function ResourceGroup:addResource( type, uri )
    self.uris = self.uris or {}
    self.uris[uri] = true
    ResourceGroup._superAddResource(self, type, uri)
end

-- Resource grouping
function ResourceGroup:addModel( uri )
    self:addResource( Model.rtti, uri )
end

function ResourceGroup:addAnimation( uri )
    self:addResource( Animation.rtti, uri )
end

function ResourceGroup:addShader( uri )
    self:addResource( Shader.rtti, uri )
end

function ResourceGroup:addTexture( uri )
    self:addResource( Texture.rtti, uri )
end

function ResourceGroup:addFont( uri )
    self:addResource( Font.rtti, uri )
end

function ResourceGroup:addAudio( uri )
    self:addResource( Audio.rtti, uri )
end

function ResourceGroup:addParticleSystem( uri )
    self:addResource( ParticleSystem.rtti, uri )
end

function ResourceGroup:addFlipbook( uri )
    self:addResource( Flipbook.rtti, uri )
end

function ResourceGroup:addVideoCube( uri )
    self:addResource( VideoCube.rtti, uri )
end

function ResourceGroup:addMaterial( def )
	if def.shader then
		self:addShader( def.shader )
	end

	if def.textures then
		for _, uri in pairs(def.textures) do
			self:addTexture( uri )
		end
	end

	if def.flipbooks then
		for _, uri in pairs(def.flipbooks) do
			self:addFlipbook( uri )
		end
	end
end

function ResourceGroup:addActor( def, includeAnimsOnlyForSegmentName )
	if def.model then
		self:addModel( def.model )
	end

	if def.animation then
        if includeAnimsOnlyForSegmentName then
            if def.animation[includeAnimsOnlyForSegmentName] then
                self:addAnimation( def.animation[includeAnimsOnlyForSegmentName][1] )
                local idleName = includeAnimsOnlyForSegmentName .. "_Idle"
                if def.animation[idleName] then
                    self:addAnimation( def.animation[idleName][1] )
                end
            end
        else
            for _, anim in pairs(def.animation) do
			    self:addAnimation( anim[1] )
            end
        end
	end

	if def.materials then
		for _, matdef in ipairs(def.materials) do
			self:addMaterial( matdef )
		end
	end		
end
