

Vector = {}
Vector.__index = Vector

Vector.copy = function(self) 
	return Vector.new(self.x, self.y, self.z, self.w);
end

Vector.copy_inplace = function(self, ob) 
	self.x = ob.x;
	self.y = ob.y;
	self.z = ob.z;
	self.w = ob.w;
end

Vector.length = function(self) 
	return math.sqrt(self.x*self.x+self.y*self.y+self.z*self.z+self.w*self.w);	
end

Vector.lengthSq = function(self)
	return self.x*self.x+self.y*self.y+self.z*self.z+self.w*self.w;	
end

Vector.setLength = function(self, len) 
	self:normalize();
	self.x = self.x * len;
	self.y = self.y * len;
	self.z = self.z * len;
--	self = self * len;
end

Vector.normalize = function(self)
	local len = self:length();

	if len > 0.0 then
		local inv_len = 1.0/len;
		self.x = self.x * inv_len;
		self.y = self.y * inv_len;
		self.z = self.z * inv_len;
		self.w = self.w * inv_len;
	end

	return len;
end

Vector.normalized = function(self) 
	local out = self:copy();
	out:normalize();
	return out;
end

Vector.unpack = function(self)
	return self.x, self.y, self.z, self.w;
end

Vector.new = function(x, y, z, w)
--	if(type(x) == "table") then
--		x, y, z, w = Vector.unpack(x);
--	end
	return setmetatable({ x = x or 0.0, y = y or 0.0, z = z or 0.0, w = w or 1.0 }, Vector)
end

Vector.__add = function(a, b)
	return Vector.new(a.x + b.x, a.y + b.y, a.z + b.z, a.w + b.w);
end

Vector.__sub = function(a, b)
	return Vector.new(a.x - b.x, a.y - b.y, a.z - b.z, a.w - b.w);
end

Vector.__mul = function(a, b)
    if (type(a)=="number") then
        local c = a; 
        a = b;
        b = c;
    end
	if (type(b)=="number") then
		return Vector.new(a.x * b, a.y * b, a.z * b, a.w * b);
	elseif (type(b=="table")) then
		return Vector.new(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
	end
end

Vector.__div = function(a, b)
    if (type(a)=="number") then
        local c = a; 
        a = b;
        b = c;
    end
	if (type(b)=="number") then
		return Vector.new(a.x / b, a.y / b, a.z / b, a.w / b);
	elseif (type(b=="table")) then
		return Vector.new(a.x / b.x, a.y / b.y, a.z / b.z, a.w / b.w);
	end
end


Vector.__eq = function(a, b)
	return a.x == b.x and a.y == b.y and a.z == b.z and a.w == b.w;
end

Vector.__lt = function(a, b)
	return a.x < b.x and a.y < b.y and a.z < b.z and a.w < b.w;
end

Vector.__le = function(a, b)
	return a.x <= b.x and a.y <= b.y and a.z <= b.z and a.w <= b.w;
end

Vector.scale = function(self, s)
	if (type(s) == "table") then
		self.x = self.x * s.x;
		self.y = self.y * s.y;
		self.z = self.z * s.z;
		self.w = self.w * s.w;		
	elseif (type(s) == "number") then
		self.x = self.x * s;
		self.y = self.y * s;
		self.z = self.z * s;
		self.w = self.w * s;
	end
end

Vector.__tostring = function(self)
	return "{"..string.format("%.2f", self.x)..","..string.format("%.2f", self.y)..","..string.format("%.2f", self.z)..","..string.format("%.2f", self.w).."}";
end

Vector.set = function(self, x, y, z, w)
	self.x = (x or 0);
	self.y = (y or 0);
	self.z = (z or 0);
	self.w = (w or 1);
end

-- returns the dot product between two vectors
Vector.dot = function(a, b) 
	return a.x * b.x + a.y * b.y + a.z * b.z + a.w * b.w;
end

-- returns the cross product between two vectors
-- note: the 4D cross product is undefined behavior, the W component is ignored. 
Vector.cross = function(a, b)
	return Vector.new(a.y*b.z-a.z*b.y, a.z*b.x-a.x*b.z, a.x*b.y-a.y*b.x, 0.0);
end

-- returns the angle between two vectors
Vector.angle = function(a, b)
	local cosTheta = Vector.dot(a, b);
	local denom = a:length() * b:length(); 
	if (denom > 0.0) then
		return math.acos(cosTheta) / denom;
	end
	return 0;
end

Vector.distance = function(a, b) 
	local diff = b - a;
	return diff:length();
end

Vector.distanceSq = function(from, to)
	local diff = b - a;
	return diff:lengthSq();
end

Vector.project = function(self, normal)
	if (normal.lengthSq() > 1.0) then 
		normal.normalize();
	end	
	return normal * Vector.dot(self, normal);
end

Vector.reflect = function(self, normal) 
	return self - 2.0*Vector.project(self, normal);
end

Vector.lerp = function(t, a, b) 
	local out = a + (b - a) * t
	return out
end

Vector.slerp = function(t, a, b) 
end

Vector.rotateBy = function(self, q) 
--	local xv; 
--	local uv;
--	local uuv;
--	local vec = Vector(q.x, q.y, q.z, q.w)
--	uv = vec:cross(self);
--	uuv = vec:cross(uv);
--	
--	uv = uv * 2.0 * q.w;
--	uuv = uuv * 2.0;
--	return self + uv + uuv;
	
	local out = q * Quaternion(self.x,self.y,self.z,0.0) * q:inverse();
	return Vector(out.x,out.y,out.z); 
end

local _x = 0.0;
local _y = 0.0;
local _z = 0.0;
local _w = 0.0;

Vector.rotateByInline = function(self, v, q)
	--_x = q.w*v.x + q.y*v.z - q.z*v.y;
	--_y = q.w*v.y - q.x*v.z + q.z*v.x;
	--_z = q.w*v.z + q.x*v.y - q.y*v.x;
	--_w =-q.x*v.x - q.y*v.y - q.z*v.z;

	--self.x = -_w*q.x + _x*q.w - _y*q.z + _z*q.y;
	--self.y = -_w*q.y + _x*q.z + _y*q.w - _z*q.x;
	--self.z = -_w*q.z - _x*q.y + _y*q.x + _z*q.w;
	local out = q * Quaternion(v.x,v.y,v.z,0.0) * q:inverse();
	self.x = out.x;
	self.y = out.y;
	self.z = out.z;
end

Vector.one = Vector.new(1.0, 1.0, 1.0, 1.0)
Vector.zero = Vector.new(0.0, 0.0, 0.0, 0.0)
Vector.up = Vector.new(0.0, 1.0, 0.0, 0.0)
Vector.forward = Vector.new(0.0, 0.0, -1.0, 0.0)
Vector.right = Vector.new(1.0, 0.0, 0.0, 0.0)

setmetatable(Vector, { __call = function(_, ...) return Vector.new(...) end })
