-- a simple horizontal progress bar.
-- implementation details are similar to the slider control.
GUI.progressBar = function(parent, pos, width, caption, initialValue)
    initialValue = initialValue or 0;

    local trackSize = _T.progressBar.track.image.size;
    local npatch = _T.progressBar.track.image.ninepatch;
    local npwidth = npatch[1]+npatch[2];
    local textSize = GUI.textSize(caption, "progressBar.text");

    local progressBar = GUI.frame(parent, px(pos.x, pos.y), px(width, textSize.y + trackSize.y), "");
    local label = GUI.label(progressBar, px(0, 0), px(textSize.x, textSize.y), caption, "progressBar.text");
    label:dock(DOCK_TOP);

    local control = GUI.frame(progressBar, px(0, 0), nx(1,1), "");
    local track = GUI.frame(control, px(0,0), px(width, trackSize.y), "progressBar.track");
    local fill = GUI.frame(control,  px(0,0), px(width, trackSize.y), "progressBar.fill");
    control:dock(DOCK_TOP);

    local maxX = width;
    local clamp = function(x, xmin, xmax)
        if(x > xmax) then
            return xmax;
        end
        if(x < xmin) then
            return xmin;
        end
        return x;
    end

    local passiveHandler = {
        onPassiveUpdate = function(self)
            self.progress = clamp(self.progress, 0, 1);
            fill:resize(math.max(npwidth, progressBar.progress*maxX), npwidth);
        end
    };

    progressBar.advance = function(self, v)
        self.progress = clamp(self.progress + v, 0, 1);
        return self.progress >= 1.0;
    end

    progressBar.setProgress = function(self, v)
        self.progress = v;
    end

    progressBar.reset = function(self)
        self.progress = 0.0;
    end

    progressBar.progress = initialValue;
    fill:resize(progressBar.progress*maxX, trackSize.y);

    progressBar:addEventHandler(passiveHandler);

    return progressBar;
end
