
SRT = {}
SRT.__index = SRT

SRT.copy = function(self) 
	return SRT.new(self.position, self.rotation, self.scale);
end

SRT.copy_inplace = function(self, ob)
	self.position:copy_inplace(ob.position);
	self.rotation:copy_inplace(ob.rotation);
	self.scale:copy_inplace(ob.scale);
end

SRT.reset = function(self)
	self.position.x = 0;
	self.position.y = 0;
	self.position.z = 0;
	self.position.w = 1;

	self.rotation.x = 0;
	self.rotation.y = 0;
	self.rotation.z = 0;
	self.rotation.w = 1;

	self.scale.x = 1;
	self.scale.y = 1;
	self.scale.z = 1;
	self.scale.w = 1;
end

SRT.new = function(position, rotation, scale)
	local p	= position	and position:copy() or Vector(0,0,0);
	local r	= rotation	and rotation:copy() or Quaternion.new(0,0,0,1);
	local s = scale		and scale:copy()	or Vector.new(1,1,1);
	return setmetatable({ position = p, rotation = r, scale = s }, SRT)
end

SRT.applyScale = function(self, s)
	if(type(s) == "table") then
		self.scale.x = self.scale.x * s.x;
		self.scale.y = self.scale.y * s.y;
		self.scale.z = self.scale.z * s.z;
	elseif(type(s) == "number") then
		self.scale.x = self.scale.x * s;
		self.scale.y = self.scale.y * s;
		self.scale.z = self.scale.z * s;
	end
end

SRT.applyRotation = function(self, rot)
	self.rotation = rot * self.rotation;
end

SRT.applyTranslation = function(self, offset)
	self.position = self.position + offset;
end

SRT.__tostring = function(self) 
	return "{ S = "..tostring(self.scale)..", R = "..tostring(self.rotation)..", T = "..tostring(self.position).."}";
end

SRT.__add = function(a, b)
	return SRT.new(a.position + b.position, a.rotation + b.rotation, a.scale + b.scale);
end

SRT.__sub = function(a, b)
	return SRT.new(a.position - b.position, a.rotation - b.rotation, a.scale - b.scale);
end

SRT.__mul = function(a, b)
	local sc  = a.scale * b.scale;
	local rot = a.rotation * b.rotation;
	local pos = rot.applyRotation(a.position * sc) + b.position;	
	return SRT.new(pos, rot, scale);
end

SRT.__eq = function(a, b)
	return a.position == b.position and a.rotation == b.rotation and a.scale == b.scale;
end

SRT.__lt = function(a, b)
	return a.position < b.position and a.rotation < b.rotation and a.scale < b.scale;
end

SRT.__le = function(a, b)
	return a.position <= b.position and a.rotation <= b.rotation and a.scale <= b.scale;
end

SRT.right = function(self) 
	return Vector.rotateBy(Vector.right, self.rotation);
end

SRT.forward = function(self)
	return Vector.rotateBy(Vector.forward, self.rotation);
end

SRT.up = function(self) 
	return Vector.rotateBy(Vector.up, self.rotation);
end

SRT.getMatrix = function(self)
	local rx = 2.0 * self.rotation.x
	local ry = 2.0 * self.rotation.y
	local rz = 2.0 * self.rotation.z
	local twx = rx * self.rotation.w
	local twy = ry * self.rotation.w
	local twz = rz * self.rotation.w
	local txx = rx * self.rotation.x
	local txy = ry * self.rotation.x
	local txz = rz * self.rotation.x
	local tyy = ry * self.rotation.y
	local tyz = rz * self.rotation.y
	local tzz = rz * self.rotation.z

	return Matrix.new(
		self.scale.x * (1.0 - (tyy + tzz)), 	self.scale.y * (txy - twz), 			self.scale.z * (txz + twy), 		self.position.x,
		self.scale.x * (txy + twz), 			self.scale.y * (1.0 - (txx + tzz)), 	self.scale.z * (tyz - twx), 		self.position.y,
		self.scale.x * (txz - twy), 			self.scale.y * (tyz + twx), 			self.scale.z * (1.0 - (txx + tyy)),	self.position.z,
		0, 0, 0, 1
	)
end

SRT.identity = SRT.new(Vector.zero:copy(), Quaternion.identity:copy(), Vector.one:copy());

setmetatable(SRT, { __call = function(_, ...) return SRT.new(...) end })
