
StateMachine = {}
StateMachine.__index = StateMachine;
StateMachine.new = function(statetable)
	return setmetatable({ states = statetable or {} }, StateMachine);
end

StateMachine.addState = function(self, name, state)
	if(self.default == nil) then
		self.default = name;
	end
	
	if(self.states == nil) then
		self.states = {};
	end
	
	self.states[name] = state;
	self.states[name].name = name;
end

StateMachine.setDefault = function(self, statename)
	self.default = statename;
end

StateMachine.start = function(self, statename)
	if(statename == nil) then
		statename = self.default;
	end
	if(self.states ~= nil and self.states[statename] ~= nil) then	
		Log.message("Starting: "..statename);
		self.activeState = statename;
		return self.states[self.activeState]:onStartup();
	else
		Log.fatal("Could not find state: "..statename)
	end
	return false;
end

StateMachine.stop = function(self)
	if(self.activeState == nil) then
		return;
	end
	
	Log.message("Stopping: "..self.activeState);
	if(self.states[self.activeState].onShutdown ~= nil) then
		self.states[self.activeState]:onShutdown();
	end

	self.activeState = nil
end

StateMachine.update = function(self)
	local nextState = self.activeState;
	if(self.states[self.activeState] ~= nil) then
		if(self.states[self.activeState].onUpdate ~= nil) then
			nextState = self.states[self.activeState]:onUpdate();
		end
	end
	
	return nextState;
end

StateMachine.evaluate = function(self)
	if(self.states == nil) then
		return;
	end
	
	if(self.activeState == nil) then
		-- attempt to set to a default state
		self:start(self.default);
	end
	
	if(self.activeState == nil) then
		return;
	end
	
	local nextState = self:update();
	if(nextState ~= nil and self.activeState ~= nextState) then
		self:stop();
		self:start(nextState);
	end	
end

StateMachine.onReshape = function(self, width, height)
	if(self.states[self.activeState] ~= nil) then
		if(self.states[self.activeState].onReshape ~= nil) then
			self.states[self.activeState]:onReshape(width, height);
		end
	end
end

StateMachine.onRender = function(self)
	if(self.states[self.activeState] ~= nil) then
		if(self.states[self.activeState].onRender ~= nil) then
			self.states[self.activeState]:onRender();
		end
	end
end

StateMachine.onUpdate = function(self)
	self:evaluate();
end

StateMachine.onShutdown = function(self)
	self:stop();
end

StateMachine.onEvent = function(self, event)
	if(self.states[self.activeState] ~= nil) then
		if(self.states[self.activeState].onEvent ~= nil) then
			return self.states[self.activeState]:onEvent(event);
		end
	end
	return nil;
end

setmetatable(StateMachine, { __call = function(_, ...) return StateMachine.new(...) end })
