PlayerControls = {}
PlayerControls.__index = PlayerControls

-- listeners should implement these functions:
local events = 
{
	onNext 		= "onPlayerNext",		-- go to the next segment
	onPrev 		= "onPlayerPrev",		-- go to the previous segment
	onPause 	= "onPlayerPause",		-- pause the show
	onResume 	= "onPlayerResume",		-- resume the show
}

PlayerControls.new = function()
	-- create the root frame for the transport controls.
	local window = GUI.window(GUIAligned(ALIGN_BOTTOM_CENTER), GUISize(310, 120), "Player", true, false)
	window:anchor(ANCHOR_BOTTOM)

	-- info labels 
	local infobar = GUI.frame(window, GUIAligned(ALIGN_TOP_CENTER), GUIRelative(0.98, 0.5), "")
	infobar:anchor(ANCHOR_NONE)
	
	local label = GUI.label(infobar, GUIAligned(ALIGN_TOP_LEFT), GUIRelative(0.5, 1.0), "Unknown", "h1")
	label:anchor(ANCHOR_NONE)
	label:setTextAlign(ALIGN_CENTER_LEFT)
		
	local timecode = GUI.label(infobar, GUIAligned(ALIGN_TOP_RIGHT), GUIRelative(0.5, 1.0), "[00:00:00]", "h1")
	timecode:anchor(ANCHOR_NONE)
	timecode:setTextAlign(ALIGN_CENTER_RIGHT)

    Events.listen(window, "segmentPlaying")

	window.onUpdate = function(self)
		timecode:setText("[ " .. Time.string() .. " ]")
	end

    window.segmentPlaying= function(self, segmentName)
        label:setText("[ " .. segmentName .. " ]")
    end

	-- transport buttons next, pause/resume, play
	local buttonSize
	buttonSize = GUIRelative(0.3, 0.5)
	local prevButton = GUI.button(window, GUIAligned(ALIGN_BOTTOM_LEFT), 	buttonSize, 	"Prev")
	prevButton:anchor(ANCHOR_NONE)
	local playButton = GUI.button(window, GUIAligned(ALIGN_BOTTOM_CENTER), 	buttonSize, 	"Pause")
	playButton:anchor(ANCHOR_NONE)
	local nextButton = GUI.button(window, GUIAligned(ALIGN_BOTTOM_RIGHT), 	buttonSize, 	"Next")
	nextButton:anchor(ANCHOR_NONE)

	-- implement button events
	function prevButton:onButtonDown()
		window:dispatch(events.onPrev)
	end

    function playButton:updateCaption()
        if self.paused then
            self:setCaption("Resume")
        else
            self:setCaption("Pause")
        end
    end

    function playButton:updateState()
        local paused = Time.isPaused()
        if paused ~= self.paused then
            self.paused = Time.isPaused()
            self:updateCaption()
        end
    end
    playButton:updateState()
    playButton:addEventHandler({
        onPassiveUpdate = playButton.updateState,
    })

    function playButton:onButtonDown()
		self.paused = not(self.paused)
		if(self.paused) then
            window:dispatch(events.onPause)
		else
			window:dispatch(events.onResume)
		end
        self:updateCaption()
	end
		
	function nextButton:onButtonDown()
		window:dispatch(events.onNext)
	end

	-- move the tool up on device so it doesn't interfere with the TSB.
	if Platform.android  or Platform.iOS then
		window:move(0, -96)
	end

	return window
end
