
local Logger = TaggedLog.new("AlignActorManager", true)

AlignActorManager = Manager:new {
    name = "AlignActorManager",
}

function AlignActorManager:init(scene)
    Manager.init(self, scene)

    -- Listen for the "Align" events
    Events.listen(self, "Align")
end

function AlignActorManager:onDestroy()
    Events.stopListening(self, "Align")
end

function AlignActorManager:Align(actor, bonePath, frame)
    Logger:message("AlignActorManager: event recieved: actor = " .. tostring(actor.name) .. ", bonePath = " .. tostring(bonePath) .. ", frame = " .. tostring(frame))

    -- See if the actor is on screen.  Ignore the event if it is offscreen
    -- or align the specified object to the target bone if it is visible
    local targetPos = ActorManager:getActorTrackingPos(actor.name)
    local actorOnScreen = Util.getWindowPosition(targetPos, Global.curScreenWidth, Global.curScreenHeight)
    if actorOnScreen then
        local trackingOffset = actor:getBoneWorld(bonePath)
        local groupName = string.match(bonePath, "align(%a+)|")
        if trackingOffset and groupName then
            local groupActors = ActorManager:getActorsInGroup(string.lower(groupName))
            local yaw, yawQuat = trackingOffset.rotation:getYaw()
            trackingOffset.rotation = yawQuat
            trackingOffset.scale = Vector(1,1,1);

            for _, alignActor in ipairs(groupActors) do
                Logger:message("Aligning " .. alignActor.name .. " to " .. actor.name .. "'s bone: " .. bonePath..string.format(" rot %.3f, scale <%.3f, %.3f, %.3f> ", 
                					trackingOffset.rotation:getYawAngle(), trackingOffset.scale.x, trackingOffset.scale.y, trackingOffset.scale.z))
                alignActor:setLocal(trackingOffset)
            end
        end
    end
end
