
--enable the maximum lua jit optimization level.
--jit.opt.start(3);

require "common/scripts/moxie/moxie"
require "common/scripts/moxie/shell"
require "episode3/scripts/fixup/scene"

require "episode3/scripts/project"

require "episode3/scripts/scenelauncher"
require "episode3/scripts/fixup/strict"
require "episode3/scripts/fixup/logger"
require "episode3/scripts/android/android"
require "episode3/scripts/global"
require "episode3/scripts/resourceloader"
-- require "episode3/scripts/scenedata/scenes/editor/editor"
require "episode3/scripts/scenedata/scenes/duet/duet"
-- require "episode3/scripts/scenedata/scenes/tests/numbers/numbers"
-- require "episode3/scripts/scenedata/scenes/tests/tracktest/tracktest"
-- require "episode3/scripts/scenedata/scenes/tests/fovtesting/fovtesting"
-- require "episode3/scripts/scenedata/scenes/tests/treetest/treetest"
-- require "episode3/scripts/scenedata/scenes/tests/cambounds/cambounds"
-- require "episode3/scripts/scenedata/scenes/tests/lookattest/lookattest"
-- require "episode3/scripts/scenedata/scenes/tests/colortest/colortest"
-- require "episode3/scripts/scenedata/scenes/tests/glowtest/glowtest"
-- require "episode3/scripts/scenedata/scenes/tests/perftests/perftests"
require "episode3/scripts/endofshow"


-- Only needed for testing animations in it's own scene
require "episode3/scripts/testplayground"

-- Launcher:
require "episode3/scripts/launcher/launcher"

local Logger = TaggedLog("Application", false)

Application.onRotation = function(deviceRotation)
    Shell.dispatch("onRotation", {deviceRotation})
end

Application.onInterceptTouch = function(windowX, windowY, x, y, eventTime, onTSB)
    -- dispatch to current state
    return Shell.dispatch("onInterceptTouch", {windowX=windowX, windowY=windowY, x=x, y=y, eventTime=eventTime, onTSB=onTSB})
end

Application.onPPStateChange = function(window)
    -- dispatch to current state
    Shell.dispatch("onPPStateChange", {window=window})
end

Application.onPPLocSizeChanged = function(appBecameFullScreen, window)
    -- dispatch to current state
    Shell.dispatch("onPPLocSizeChanged", {appBecameFullScreen=appBecameFullScreen, window=window})
end

Application.onActivityEvent = function(eventString)
    -- debug only: exit doodle on dummy activity destruction
    if (eventString == "onStop") then
        Global.shutdown = true
    end

    -- dispatch to current state
    Shell.dispatch("onActivityEvent", {eventString=eventString})
    AnalyticsManager:onActivityEvent(eventString)
end

Application.onChangeDisplaySize = function(screenWidth, screenHeight, availWidth, availHeight)
    Logger:message("Change display size " .. screenWidth .. ", " .. screenHeight .. ", " .. availWidth .. ", " .. availHeight)

    Global.screen_size.x = screenWidth
    Global.screen_size.y = screenHeight
    Global.avail_size.x = availWidth
    Global.avail_size.y = availHeight
    Global.updateScreenSize()

    -- dispatch to current state
    Shell.dispatch("onChangeDisplaySize", {screenWidth=screenWidth, screenHeight=screenHeight, availWidth=availWidth, availHeight=availHeight})
end

Application.onVolumeUp = function(newVolume)
    Logger:message("Volume up " .. newVolume)

    -- dispatch to current state
    Shell.dispatch("onVolumeUp", {newVolume=newVolume})
end

Application.onVolumeDown = function(newVolume)
    Logger:message("Volume down " .. newVolume)

    -- dispatch to current state
    Shell.dispatch("onVolumeDown", {newVolume=newVolume})
end

Application.onScreenEvent = function(newState)
    Logger:message("Screen event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onScreenEvent", {newState=newState})
end

Application.onBatteryEvent = function(newState)
    Logger:message("Battery event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onBatteryEvent", {newState=newState})
end

Application.onHeadsetEvent = function(newState)
    Logger:message("Headset event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onHeadsetEvent", {newState=newState})
end

Application.onBluetoothEvent = function(newState)
    Logger:message("Bluetooth event " .. newState)

    -- dispatch to current state
    Shell.dispatch("onBluetoothEvent", {newState=newState})
end

Application.onKeyPress = function(keyCode, action, isLongPress, downTime)
    Logger:message("Keypress event " .. keyCode .. ", " .. action .. ", " .. downTime)

    -- dispatch to current state
    Shell.dispatch("onKeyPress", {keyCode=keyCode, action=action, isLongPress=isLongPress, downTime=downTime})
end

Application.onTouchDown = function(pointerId, x, y, eventTime, touchPhase)
    Logger:message("Touch down " .. pointerId .. ", " .. x .. ", " .. y .. ", " .. eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchDown", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    if(pointerId == 0) then
   		Events.send(Application, "touchDown", Vector(x, y, 0, 0))
    end

    return true
end

Application.onTouchUp = function(pointerId, x, y, eventTime, touchPhase)
    Logger:message("Touch up " .. pointerId .. ", " .. x .. ", " .. y .. ", " ..eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchUp", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    if(pointerId == 0) then
    	Events.send(Application, "touchUp", Vector(x,y, 0, 0))
    end

    return true
end

Application.onTouchMove = function(pointerId, x, y, eventTime, touchPhase)
    Logger:message("Touch move " .. pointerId .. ", " .. x .. ", " .. y .. ", " .. eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchMove", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    if(pointerId == 0) then
    	Events.send(Application, "drag", Vector(x, y, 0, 0))
    end

    return true
end

Application.onRequestExit = function()
    Logger:message("Exit requested")
    return true
end

Application.onTerminate = function()
	Shell.onShutdown();
    ResourceLoader:shutdown()
	Global.onShutdown();
end

Application.setupConfiguration = function()
--    Platform.config = "debug,    targets:foobar"
    if Platform.config then
        Log.message("Config = " .. Platform.config)
        -- parse out the config file names
        for configName in string.gmatch(Platform.config, "%w+") do
            local fileName = Project.configDir .. configName .. ".lua"
            Log.message("Config file name = " .. fileName)
            local configure, err = loadfile(fileName)
            if configure then
                configure()
            else
                Log.error("Unable to load config file " .. fileName .. ": " .. tostring(err))
            end
        end
    end
end
-- New bake system doesn't work!

Application.setupConfiguration()

-- Now that everything is loaded, lock down the global
-- table so no more global variables can accidentally be
-- created

-- Disabled for
--Strict.globalLock(true)

-- Initialize the show.  We should refactor this

--uncomment the next line to enable the profiler.
--Shell.__onInitialize = Shell.onInitialize;

Shell.onInitialize = function()
	--uncomment the next line to enable the profiler.
	--Shell.__onInitialize();

    SceneLauncher:loadSceneData()

    ResourceLoader:init()
    Global.init()

    ActorManager:init(Global.scene)
    -- Setting custom clearcolor
	
    if Platform.scene and Platform.scene ~= "" then
        local scene = SceneLauncher:getScene()
        local segment = Platform.segment
        ResourceLoader:prefetchSegment(segment)
        Shell.start(scene)
    else
        Shell.start("launcher_start")
    end
	return true
end
