
local Logger = TaggedLog("ActorFader", false)

ActorFader = {
    -- configurable params
    fromColor	= Color(1, 1, 1, 0),
    toColor		= Color(1, 1, 1, 1),
    
    delay		= 0.0,
    duration	= 1.0,

    timer 		= 0.0,
}

function ActorFader:new(params)
    local newObj = shallowcopy(params) or {}
    setmetatable(newObj, self)
    self.__index = self

    return newObj
end

function ActorFader:onStart()
    self.timer = self.delay + self.duration
    Logger:message("Starting to fade "..self.object.name.." : "..self.timer)
end

function ActorFader:onUpdate()		
    if (self.timer >= 0.0) then
        self.timer = self.timer - Time.dt()		
		
		-- blendAmount goes 1 to 0, so flip the lerp
		local blendAmount	= math.easeInOutSine(math.clamp01(self.timer / self.duration))
		local matColor		= Color.lerp(blendAmount, self.toColor, self.fromColor)
		
	    --Global.AppDebugText1 = string.format("%.5f : %.5f", self.timer, blendAmount)	
		
	    if self.object.materials then
	        for name, material in pairs(self.object.materials) do
	    		material:setParameters({ u_Color = matColor})
	        end
	    end
    end
end

