
local Logger = TaggedLog("AnimatedFader", true)

AnimatedFader = {
    -- configurable params
    matName = "Mat_ScreenFade",
    fromColor = Vector(0, 0, 0, 1),
    toColor = Vector(0, 0, 0, 0),
    controlBoneData = nil,

    -- internal
    material = nil,
    controlBone = nil,
}

function AnimatedFader:new(params)
    if (params.controlBoneData == nil) then
        Log.error("Need to specify the controlBoneData in the params to AnimatedFader")
        return nil
    end

    local newObj = shallowcopy(params) or {}
    setmetatable(newObj, self)
    self.__index = self

    return newObj
end

function AnimatedFader:onStart()
    self.material = self.object:findMaterial(self.matName)
    self.controlBone = ActorManager:getBone(self.controlBoneData)

    -- parent our game object to the camera so that it is always covers the screen
    local screenSpace = Global.scene.screenSpace
    if (not screenSpace) then
        Log.warning("Unable to find screen space object for screen fader")
        return
    end
    screenSpace:addChild(self.object)
end

function AnimatedFader:onUpdate()
    -- Use the control bone's local Y to drive the fade between the two colors
    local amount = self.controlBone:getLocal().position.y
    local matColor = Vector.lerp(amount, self.fromColor, self.toColor)
	self.material:setParameters({ u_Color = matColor})
end
