
SceneData = SceneData or {}
SceneData.actors = SceneData.actors or {}

SceneData.actors.DebugAxisCopy = {
    model = "episode3:models/axis.mxm",
    dumpMaterials = true,
    dumpBones = true,
    materials = {
        { 
        	name = "Mat_Axis", shader = "episode3:shaders/debug.rsfx",
	     	technique = "vertex",
	     	renderGroup = 7,
	     	textures = { },
	     	parameters = { u_Color = Color.new( 1, 1, 1, 0.5 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, 
	    },
    },
}

Axis = {
	history = {},
	camHistory = {},
	attachTo = nil,
	axis = nil,
	width = 1280,
	height = 720,
	persp = 10000;
	xSlope = 1,
	ySlope = 1,
	aspect = 1,
	showRotation = true,
	step = 30,
	textStyle = nil,
	track = false,
	drawTracking = false,
	
	-- static
	textLocations = {},
	registeredUpdate = false,
	onPreUpdate = function(self)
		Axis.textLocations = {} -- clear
	end,
}

axisUniqueIndex = 0

Axis.new = function(self, data)
    local o = data or {};
    setmetatable(o, self);
    self.__index = self;
	
	o.history = {}
	o.camHistory = {}
	
    return o;
end

function Axis:onStart()
	
	if(not Axis.registeredUpdate) then
		Axis.registeredUpdate = true;
		Global.addPreUpdate(Axis);
	end
	
	self.textStyle = { color = Color.new(0, 1.0, 0.0), font = "common:fonts/arial_default_14.fnt", scale = 1.0 }
	
	if(self.showRotation) then
		self.step = 45
	end
	axisUniqueIndex = axisUniqueIndex + 1;
	
	self.width 	= Global.curScreenWidth;
	self.height = Global.curScreenHeight;
	
	self.persp = 1/(Global.nearFar.f - Global.nearFar.n);
	
	self.aspect = Global.curScreenWidth/Global.curScreenHeight;
    
    self.ySlope = math.tan(math.rad(Global.fov)*0.5)
    self.xSlope = self.ySlope*self.aspect;
    
    local depth = -(Global.nearFar.f - Global.nearFar.n)/2;
	
	
	mainView:project(Vector(0,0,0))
	
    Events.listen(self, "onReshape");
end

function Axis:onUpdate()
	local parent = ActorManager:getActor(self.attachTo.parentName)
	if(parent) then
		local parentT = parent:getBone(self.attachTo.bonePath):getWorld()
		local dist = (parentT.position - Global.scene.mainCamera:getWorld().position)
		
		parentT.scale = Vector(1,1,1)
		
		local scale = dist:length() - Global.nearFar.n
		
		-- cull out results in front of the near plane
		if(scale <= 0) then
			return;
		end
		
		-- the final scale of axis
		scale = (self.persp*scale)*(self.ySlope*Global.nearFar.f*0.1) + 1.0; 
		
		-- render labels in screen space
		local ndc = mainView:project(parentT.position)
		
		-- some simple culling
		if(ndc.y > 1.1 or ndc.y < -1.1 or ndc.x > 1.2 or ndc.x < -1.2 or ndc.z < 0 or ndc.z > 1) then
			return
		end
		
		ndc.x = 0.5*self.width*(ndc.x + 1)
		ndc.y = 0.5*self.height*(1 - ndc.y)
		
		-- a quick way to reduce text overlap - create a hash tag with the rough (x,y) coordinates, if hashes match a previous entry increment stackCount which will step the text down
		local key = string.format("<%.1f,%.1f>", math.floor(ndc.x/1000)*1000, math.floor(ndc.y/100)*100);
		--[[ Text locations get cleared at the top of the frame (outside of this update) ]]--
		Axis.textLocations[key] = Axis.textLocations[key] or {}
		local stack = Axis.textLocations[key]
		local stackCount = #stack;
		
		-- draw text
		Renderer.drawText(ndc.x, ndc.y + stackCount*self.step, self.textStyle, self.attachTo.bonePath)
		Renderer.drawText(ndc.x, ndc.y + stackCount*self.step + 15, self.textStyle, string.format("pos: <%.2f,%.2f,%.2f>", parentT.position.x, parentT.position.y, parentT.position.z))
		if(self.showRotation) then
			Renderer.drawText(ndc.x, ndc.y + stackCount*self.step + 30, self.textStyle, string.format("rot: <%.2f,%.2f,%.2f>", parentT.rotation:getPitch(), parentT.rotation:getYawAngle(), parentT.rotation:getRollAngle()))
		end
		
		-- update the locations table so all axis objects can compare text positions
		table.insert(Axis.textLocations[key], Vector(ndc.x, ndc.y));
		
		RenderDebug.axes(parentT, scale)		
				
		if(self.drawTracking and self.track and self.attachTo.parentName == MainCamera.controller.targetActorName) then
			
		
			local cameraT =	Global.scene.mainCamera.controller.object:getWorld()
			local camForward = Vector.rotateBy(Vector.forward, cameraT.rotation)
			
			local camDist = (parentT.position - cameraT.position)
			local camTarget = camForward:normalized()*camDist:length() + cameraT.position
			
			if( Global.scene.mainCamera.controller.active) then
				table.insert(self.history, parentT)
--				table.insert(self.camHistory, camTarget)
			end
			
			local n = table.getn(self.history)
			local s = 15
			local prevPos = Vector(0,0,0)
			local color = Color.new(0.2, 0.6, 0.7)
			local camColor = Color.new(1.0, 1.0, 1.0)
			local lasti = n
			for i = n, 1, -1 do
				local tr = self.history[i]
				
				local pos = tr.position
				local dist = Vector.distance(pos, prevPos)
				
				if(i == n or dist > 10 or (lasti - i) > 10) then
					RenderDebug.axes(tr, s)
					if(i < n) then
						RenderDebug.line(pos, prevPos, color)
--						RenderDebug.line(pos, self.camHistory[i], camColor)
					end
					prevPos = pos
					--s = s * 0.99
				end
			end		
		end
	end 
end

function Axis:onReshape(screenSize)
	self.width  	= screenSize.x
	self.height  	= screenSize.y
	self.persp = 1/(Global.nearFar.f - Global.nearFar.n);
end


