
Billboard = {
	width  = 1,
	height = 1,
	aspect = 1,
	xSlope = 1,
	ySlope = 1,
	camera = nil,
	angle = 0,
	
	billboardAxisX = true,
	billboardAxisY = true;
}

Billboard.new = function(self, data)
    local o = data or {};
    setmetatable(o, self);
    self.__index = self;
    return o;
end

function Billboard:onStart()
	Events.listen(self, "onReshape");
	self.width = Global.curScreenWidth;
	self.height = Global.curScreenWidth;
	if(self.camera == nil) then
		self.camera = Global.scene.mainCamera;
	end
	self:configure()
	
end

function Billboard:onUpdate(cameraTransform)
	if (self.object == nil) then
		return
	end
	
	local cameraWorld = nil
	local cameraLocal = nil
	local camLook 	= nil
	local camRight 	= nil
	local camUp 	= nil
		
	if(cameraTransform) then
		cameraWorld = cameraTransform 
		cameraLocal = cameraTransform  
		camLook 	= cameraTransform.camForward:copy()
		camRight 	= cameraTransform.camRight:copy()
		camUp	 	= cameraTransform.camUp:copy()
	else
--		cameraWorld 	= Global.scene.mainCamera:getWorld()  
--		cameraLocal 	= Global.scene.mainCamera:getLocal() 
--		camLook 	= Vector.rotateBy(Vector.forward, cameraLocal.rotation)
-- 		camRight 	= Vector.rotateBy(Vector.right, cameraLocal.rotation)
--		camUp 	= Vector.rotateBy(Vector.up, cameraLocal.rotation) 
	end
	
	local toMe = self.object:getWorld().position - cameraWorld.position;
	
	local dist = toMe:length()
	dist = dist == 0 and 1 or dist
	local planeDistX = Vector.dot(toMe, camRight)
	local planeDistY = Vector.dot(toMe, camUp)
	
	-- these angles match the FOV skew at the (x,y) screen position
	local yaw = (-math.deg(math.tan(planeDistX/dist)*self.aspect))
	local pitch = (-math.deg(math.tan(planeDistY/dist)))
	
	-- first match the camera's plane
	local localT = self.object:getLocal()
	localT.rotation = Quaternion.lookAt(camLook)
	
	-- then tweak the x and y to fit the FOV skew
	if(self.billboardAxisY) then
		localT.rotation = localT.rotation*Quaternion.axisAngle(Vector.up, yaw);
		if(self.debug) then
			DrawText:addText("BillboardAngleX", "------------ Yaw <%.3f> -------------", yaw)
		end
	end
	
	if(self.billboardAxisX) then
		localT.rotation = localT.rotation*Quaternion.axisAngle(Vector.right, pitch)
		if(self.debug) then
			DrawText:addText("BillboardAngleY", "------------ Pitch <%.3f> -------------", pitch)
		end
	end
	
	self.object:setLocal(localT)
	
	
end

function Billboard:onReshape(screenSize)
	self.width 	= screenSize.x
	self.height = screenSize.y
	self:configure()
end

function Billboard:configure()
	self.aspect = self.width/self.height;
end