

ButterflyController = 
{	               		 
	butterflyWaitTime = 1.5,     		 -- Butterfly re-activation time
	waitTime = 1.0,             		 -- Duration for butterfly in wait state
	butterflyToCenterSpeed = 0.03,       -- Speed at which butterfly flies to center
	butterflyToActorSpeedValue = 0.008,  -- Speed at which butterfly moves to actor
	butterflyToHorizonSpeedValue = 0.008,-- Speed at which butterfly moves to horizon
	rotateAngleFromScreen = 1.0,         -- Butterfly starts at this angle away from center
	lostTimerThreshold = 3,              -- Butterfly activation delay		
	
	-- Do not modify these values
	flipbooksPlaying = false,			 -- Flipbook does not play initially
	butterflyActive = false,			 -- Initial activity of butterfly
	blendAmount = 0,					 -- Alpha value of butterfly
	lostTimer = 0,
	currentTime = 0,
	waitCounter = 0,
}


ButterflyController.new = function(self, data)    
    local o = data or {}
    setmetatable(o, self)
    self.__index = self    

	return o
end

ButterflyController.onStart = function(self)

	Events.listen(self, "segmentPlaying")

	-- Store current segment
	self.currentSegment = SegmentManager:currentSegmentName()

	-- Store current transform
	self.currentTransform = self.object:getLocal()

	-- Handle to native implementation of actorslayer
	self.actorSlayerHandle = MainCamera.controller.actorsLayer

	-- Butterfly should not activate in these segments
	self.ignoreSegments = 
	{
		Cell_Boy = 1,
		Cell_Girl = 2,
	}

	-- Butterfly ignores horizon pull in these segments
	self.ignoreHorizonSegments = 
	{
		boytree = 1,
		boytree_intro_a = 2,
		boytree_girlblinks = 3,
		boytree_boyoutro = 4,
		boytree_girloutro = 5,
		boytree_dance = 6,
		boytree_girlwalks_a = 7,
		boytree_girlwalks_b = 8,
		boytree_girljumps = 9,
		girltree = 10,
		girltree_intro_a = 11,
		girltree_intro_b = 12,
		girltree_dance = 13,
		girltree_girlblinks = 14,
		girltree_girlwalks_a = 15,
		girltree_girlwalks_b = 16,
		girltree_girljumps = 17,
		girltree_girloutro = 18,
	}
	
	-- Butterfly flipbook initializations
	self.material = self.object.materials.Butterfly_MAT
	self.glowMaterial = self.object.materials.Butterfly_Glow_MAT
    self.flipBook = self.object:addComponent(
    	"ButterflyFlipbook", 
    	SimpleFBDriver.new("episode3:flipbooks/butterfly.mfb", 
    		self.material, true)
    )
    self.glowFlipBook = self.object:addComponent(
    	"ButterflyGlowFlipbook", 
    	SimpleFBDriver.new("episode3:flipbooks/butterfly_glow.mfb", 
    		self.glowMaterial, true)
    )
    self.material = self.object:findMaterial("Butterfly_MAT")
	self.glowMaterial = self.object:findMaterial("Butterfly_Glow_MAT")
	
	-- Make butterfly invisible
	self:blend(0)

	-- If not starting from any of the ignore segments, initialize butterfly's 
	--  flipbooks
	if (not self.ignoreSegments[self.currentSegment]) then
		self.flipBook:play()
		self.glowFlipBook:play()
		self.flipbooksPlaying = true
	end

	-- Choose actor for the butterfly to follow
	if(self.actor == nil and self.lastViewedActor) then
		self.actor = ActorManager:getActor(ActorManager.proxyActors[self.lastViewedActor])
	end
	if (self.actor == nil) then	
		self.actor = ActorManager:getActor(ActorManager.proxyActors["girl"])
	end
	if (self.actor == nil) then
		self.actor = ActorManager:getActor(ActorManager.proxyActors["boy"])
	end	

	-- If starting from a segment that should ignore the horizon pull, set
	--   appropriate pull (speed) values.
	if(self.ignoreHorizonSegments[self.currentSegment]) then
		self.butterflyToActorSpeed = self.butterflyToActorSpeedValue * 2
		self.butterflyToHorizonSpeed = 0.0
	else
		self.butterflyToActorSpeed = self.butterflyToActorSpeedValue
		self.butterflyToHorizonSpeed = self.butterflyToHorizonSpeedValue
	end
end

-- Set blend (alpha) values for the butterfly
-- direction: -1 decreases alpha; 0 sets alpha to 0; 1 increases alpha
ButterflyController.blend = function(self, direction)

	self.dt = Time.dt()	
	self.blendAmount = math.clamp01(self.blendAmount + self.dt * direction)

	-- set material alpha to the blend amount
	self.material:setParameters({ u_Alpha = self.blendAmount})		
	self.glowMaterial:setParameters({ u_Alpha = self.blendAmount})		

	-- When butterfly is invisble, wait for butterflyWaitTime before deactivating it
	if(self.blendAmount == 0) then
		self.currentTime = self.currentTime - self.dt * 3.0
		if(self.currentTime <= 0) then
			self.butterflyActive = false
		end
	else
		self.currentTime = self.butterflyWaitTime
	end

end

-- When butterfly is activated, place it appropriately in world space
ButterflyController.activateButterfly = function(self)

	self.butterflyActive = true
	self.actor = nil
	-- Choose actor for the butterfly to follow
	if(self.lastViewedActor) then
		self.actor = ActorManager:getActor(ActorManager.proxyActors[self.lastViewedActor])
	end
	if (self.actor == nil) then	
		self.actor = ActorManager:getActor(ActorManager.proxyActors["girl"])
	end
	if (self.actor == nil) then
		self.actor = ActorManager:getActor(ActorManager.proxyActors["boy"])
	end	

	if(self.actor) then
		-- Find the bone to follow (target or glow center bone based on availability)
        local boneTransform = nil        
        if(self.actor.namedBones and self.actor.namedBones.glowBone) then
        	-- Get Glow Bone
        	boneTransform = self.actor:getBoneWorld(self.actor.glowBonePath)
        elseif (self.actor.namedBones and self.actor.namedBones.targetBone) then
        	-- Get Target Bone
        	boneTransform = self.actor:getBoneWorld(self.actor.targetBonePath)	            	
        end

        if(boneTransform ~= nil) then

        	-- Store necessary vectors for positioning butterfly
			local cameraForward = MainCamera.camForward:normalized()
			local originToActor = (boneTransform.position 
				- Vector(0,0,0,1)):normalized()
			local originToButterfly = Vector.rotateBy(Vector.forward,
				 self.object:getWorld().rotation)

			-- Find axis around which to rotate butterfly in world space
			local axis = Vector.cross(originToActor, 
				cameraForward)

			-- Direction where the butterfly should get generated
			local butterflySpawnDirection = Quaternion.axisAngle(axis, 
				math.deg(self.rotateAngleFromScreen))

			-- The direction vector 
			local newDir = Vector.rotateBy(MainCamera.camForward, butterflySpawnDirection)
			local modifiedButterflyRot = Quaternion.lookAtWithUp(newDir, Vector.up)

			-- Store camera's roll to cancel it when applying rotation to butterfly
			local roll, rollQ = MainCamera:getWorld().rotation:getRoll()
			self.currentTransform.rotation = 
				MainCamera:getWorld().rotation * rollQ:inverse() + modifiedButterflyRot

			local angleButterflyCenter = Vector.dot(originToButterfly, 
				MainCamera.camForward)

			if(angleButterflyCenter < 0) then
				local modifiedButterflyRot = Quaternion.lookAtWithUp(newDir,
					Vector.up * -1)
				self.currentTransform.rotation = MainCamera:getWorld().rotation 
				* rollQ:inverse() + modifiedButterflyRot
		    end
			
            self.object:setLocal(self.currentTransform)

            -- Force full alpha when butterfly appears
            self.blendAmount = 1

            self.state = "ButterflyToCenter"
        end		
    end

    self.waitCounter = 0
end

-- Update Actor reference on segment transitions
ButterflyController.segmentPlaying = function(self, segmentName)
	
	-- Store current segment
	self.currentSegment = segmentName

	-- Initiate flipbooks if they have not been initiated already
	if(not self.flipbooksPlaying) then
		self.flipBook:play()
		self.glowFlipBook:play()
		self.flipbooksPlaying = true
	end

	-- Reset actor as proxy actors will point to a new actor every segment
	self.actor = nil

	-- Choose actor for the butterfly to follow
	if(self.lastViewedActor) then
		self.actor = ActorManager:getActor(ActorManager.proxyActors[self.lastViewedActor])
	end
	if (self.actor == nil) then	
		self.actor = ActorManager:getActor(ActorManager.proxyActors["girl"])
	end
	if (self.actor == nil) then
		self.actor = ActorManager:getActor(ActorManager.proxyActors["boy"])
	end		

	-- If current segment is a segment that should ignore the horizon pull, set
	--   appropriate pull (speed) values.
	if(self.ignoreHorizonSegments[self.currentSegment]) then
		self.butterflyToActorSpeed = self.butterflyToActorSpeedValue * 2
		self.butterflyToHorizonSpeed = 0.0		
	else
		self.butterflyToActorSpeed = self.butterflyToActorSpeedValue
		self.butterflyToHorizonSpeed = self.butterflyToHorizonSpeedValue
	end	
end

ButterflyController.onUpdate = function(self)

	-- For segments that ignore the butterfly, avoid this function
	if self.ignoreSegments[self.currentSegment] then
		return
	end

	self.dt = Time.dt()

	-- Start counting down lost timer once all actors are offscreen
	if (self.actorSlayerHandle.allActorsOffscreen == true) then
		self.lostTimer = self.lostTimer + self.dt
	else
		self.lostTimer = 0
		self.angleButterflyActor = 0
		self.angleButterflyCenter = 0	
		if (self.blendAmount <= 0) then
			self.lastViewedActor = self.actorSlayerHandle.lastViewedActor	
		end
	end	

	-- When player is lost, activate butterfly
	if(self.lostTimer > self.lostTimerThreshold and not self.butterflyActive) then		
		self:activateButterfly()
    end

    if(self.butterflyActive) then

    	-- If butterfly has reached actor or user has found actors, 
    	--  deactivate butterfly    	
    	if(self.actor) then
	        local boneTransform = nil
	        if(self.actor.namedBones and self.actor.namedBones.glowBone) then
	        	-- Get Glow Bone
	        	boneTransform = self.actor:getBoneWorld(self.actor.glowBonePath)
	        elseif (self.actor.namedBones and self.actor.namedBones.targetBone) then
	        	-- Get Target Bone
	        	boneTransform = self.actor:getBoneWorld(self.actor.targetBonePath)	            	
	        end

	    	-- Move butterfly towards actor
	    	if(boneTransform ~= nil) then
		    	local lookAtRotation = Quaternion.lookAtWithUp((
		    		boneTransform.position - self.currentTransform.position):normalized(),
		    	Vector.up)
 				local cameraForward = MainCamera.camForward
                local originToActor = boneTransform.position:normalized()
                local originToButterfly = Vector.rotateBy(Vector.forward, 
                	self.object:getWorld().rotation)

                local butterflyRight = Vector.cross(originToButterfly, Vector.up)
                local butterflyFixedHorizon = Vector.cross(Vector.up, butterflyRight)

                local butterflyDynamicHorizon = Vector(butterflyFixedHorizon.x, 
                	butterflyFixedHorizon.y - originToActor.y, 
                	butterflyFixedHorizon.z):normalized()

                local toHorizonRotation = Quaternion.lookAtWithUp(
                	butterflyDynamicHorizon, Vector.up)


		    	self.angleButterflyActor = Vector.dot(originToButterfly, originToActor)
		    	self.angleButterflyCenter = Vector.dot(originToButterfly, MainCamera.camForward)
		    	self.angleButterflyHorizon = Vector.dot(butterflyDynamicHorizon, 
		    		Vector.up)		    	

		    	-- Store butterfly's direction (used to flip it in the shader)
		    	self.butterflyDirection = Vector.dot(butterflyRight, originToActor)
				
				-- Pull butterfly towards Center
		    	if(self.state == "ButterflyToCenter") then
			    	
			    	-- Store camera's roll to cancel it when applying rotation to butterfly
			    	local roll, rollQ = MainCamera:getWorld().rotation:getRoll()
			    	
			    	local lerpValueCenter = math.clamp(
			    		math.abs(self.angleButterflyCenter + 1) * 0.5, 0.4, 1)
			    		* self.butterflyToCenterSpeed
					
					self.currentTransform.rotation = Quaternion.lerp(
						lerpValueCenter,
						self.currentTransform.rotation,
						MainCamera:getWorld().rotation * rollQ:inverse()
					)
					
					self.butterflyDirection = Vector.dot(butterflyRight, 
						MainCamera.camForward)
					
					if(self.angleButterflyCenter > 0.999) then
						self.state = "ButterflyWait"
					end
				elseif (self.state == "ButterflyWait") then
					if(self.angleButterflyCenter > 0.98) then
						self.waitCounter = self.waitCounter + self.dt
					else
						self.waitCounter = 0
						self.state = "ButterflyToCenter"
					end
					if(self.waitCounter > self.waitTime) then
						self.state = "ButterflyToActor"
						self.easeOut = 0
					end
				elseif (self.state == "ButterflyToActor") then

					-- Pull butterfly towards Actor
					local lerpValueActor = math.clamp( ( 
						self.angleButterflyActor + 1 ) * 0.5, 0.15, 1.0)
						* self.butterflyToActorSpeed * self.easeOut

		    		self.currentTransform.rotation = Quaternion.lerp(
		    			lerpValueActor,
		    			self.currentTransform.rotation, 
		    			lookAtRotation
		    		)

		    		-- Pull butterfly towards the Horizon (applicable segments)
		    		local lerpValueHorizon = math.clamp( ( 
						self.angleButterflyHorizon + 1 ) * 0.5, 0.01, 1.0)
					 	* self.butterflyToHorizonSpeed

			    	self.currentTransform.rotation = Quaternion.lerp(
			    		lerpValueHorizon, 
			    		self.currentTransform.rotation,
			    		toHorizonRotation
			    	)

			    	-- Ease out it used to avoid abrupt movements by butterfly
			    	self.easeOut = math.clamp01(self.easeOut + self.dt * 0.4)					
					
				end
				
				-- Send blend and flip values to shader and set transform
				self.material:setParameters({ u_flip = math.sign(
					self.butterflyDirection)})
				self.glowMaterial:setParameters({ u_flip = math.sign(
					self.butterflyDirection)})
		    	self.object:setLocal(self.currentTransform)		    	
			end

			-- If any actor is on screen or if the butterfly is close enough 
			--   to the actor
			if(self.actorSlayerHandle.allActorsOffscreen == false 
				or self.angleButterflyActor > 0.995) then
	    		self:blend(-1)
	    	else   -- Butterfly is in action. Blend in
	    		self:blend(1)
	    		-- If butterfly goes too far away from view, reset 
	    		if(self.angleButterflyCenter < 0.7) then
	    			self:activateButterfly()
	    		end
    		end
		
	    end
    end
end

ButterflyController.onDestroy = function(self)
end