
BoundsManager = {
	boundsOwnerName = nil,
	debug = false,
	
	leftBound = nil,
	rightBound = nil,
	boundList = {},
	enabled = false,
	
	-- static
	enabledCounter = 0,
}

BoundsManager.new = function(self, data)
    local o = data or {};
    setmetatable(o, self);
    self.__index = self;
    return o;
end

function BoundsManager:onStart()
	if (self.object == nil) then
		return
	end
	
	self.leftBound = ActorManager:getActor("BoundL_"..self.object.name, true);
	self.rightBound = ActorManager:getActor("BoundR_"..self.object.name, true);
	self.boundList = {}
	self.boundList["left"] 	= self.leftBound
	self.boundList["right"] = self.rightBound
	
	if(not self.useBounds) then
		self.onUpdate = function() end
		self.enableBounds = function() end
	end
	
end

function BoundsManager:onUpdate()
	if(self.enabled) then
		local actor = ActorManager:getActor(self.boundsOwnerName); 
		if(actor) then
			self:updatePostPosition(actor)
		end
	end
end

function BoundsManager:enableBounds(enable, actor)
	self.enabled = enable;
	if(enable) then
	
		BoundsManager.enabledCounter = BoundsManager.enabledCounter + 1;
		
		self.boundsOwnerName = actor.name;
		if(self.debug) then
			DrawText:color(1,1,0):addText("BoundOwner"..self.object.name, "BoundsManager: STARTED tracking %s", self.boundsOwnerName)
		end
		
		self:updatePostPosition(actor)
	else
		BoundsManager.enabledCounter = BoundsManager.enabledCounter - 1;
		
		if(self.debug and self.boundsOwnerName) then
			DrawText:color(1,1,0):addText("BoundOwner"..self.object.name, "BoundsManager: STOPPED tracking %s", self.boundsOwnerName)
		end
	end
end

function BoundsManager:updatePostPosition(actor)
	
	if(actor and actor.leftBound and actor.rightBound) then
		-- update bounds
		local posL = actor.leftBound:getWorld()
		local posR = actor.rightBound:getWorld();
		self.leftBound:setLocal(posL);
		self.rightBound:setLocal(posR);
	end
end
