
CameraGUI = {}
CameraGUI.__index = CameraGUI

local sliderLength = 300

function CameraGUI:new()
    local window = GUI.window(GUIAligned(ALIGN_CENTER), GUISize(320, 300), "Camera")
    window:anchor(ANCHOR_BOTTOM)

    local cameraController = Global.scene.mainCamera.controller
    window.cameraController = cameraController
    local initialFOV = cameraController.object.fov or Global.fov or 10

    local fovSlider = GUI.slider(window, GUIAligned(ALIGN_CENTER), sliderLength, "FOV", initialFOV, 10, 179)
    fovSlider:dock(DOCK_TOP)
    local passiveHandler = {
        onPassiveUpdate = function(self)
            self:setValue(cameraController.object.fov or Global.fov or 10)
        end,
    }
    fovSlider:addEventHandler(passiveHandler)
    fovSlider.onValueChanged = function(self, fov)
        Global.scene.mainCamera.fov = fov
        Global.fov = fov
        Global.scene.mainCamera:setPerspective(Global.fov, Global.nearFar.n, Global.nearFar.f)
    end

    local trackingSlider = GUI.slider(window, GUIAligned(ALIGN_CENTER), sliderLength, "Tracking strength", 0.5, 0, 1)
    trackingSlider:dock(DOCK_TOP)
    local passiveHandler = {
        onPassiveUpdate = function(self)
            trackingSlider.caption = "Tracking strength [" .. SegmentManager:currentSegmentName() .. "]"
            if(cameraController.trackingLayer) then
                self:setValue(cameraController.trackingLayer.strength)
            end
        end,
    }
    trackingSlider:addEventHandler(passiveHandler)
    trackingSlider.onValueChanged = function(self, strength)
        cameraController.trackingLayer:updateTrackingStrength(strength)
    end
    
    -- Boom slider
    local boomSlider = GUI.slider(window, GUIAligned(ALIGN_CENTER), sliderLength, "BoomSquash", .5, 0.0, 1)
    boomSlider:dock(DOCK_TOP)
    local passiveHandler = {
        onPassiveUpdate = function(self)
        	if(MainCamera.controller.boomLayer) then
            	self:setValue(MainCamera.controller.boomLayer.cameraOrbitSquashY or .5)
            end
        end,
    }
    boomSlider:addEventHandler(passiveHandler)
    boomSlider.onValueChanged = function(self, squash)
        MainCamera.controller.boomLayer.cameraOrbitSquashY = squash;
    end
    
     -- Bounds toggle
    local boundsToggle = GUI.checkbox(window, GUIAligned(ALIGN_CENTER), GUISize(100, 30), false, "Bounds")
    boundsToggle:dock(DOCK_TOP)
    local passiveHandler = {
    	toggle = boundsToggle;
        onPassiveUpdate = function(self)
        	if(MainCamera.controller.boundsLayer) then
	        	if(MainCamera.controller.boundsLayer.enabled) then
	        		self:check()
	        	else
	        		self:uncheck()
	        	end
        	end
--            self:setValue(MainCamera.controller.boundsLayer.enabled or false)
        end,
    }
    boundsToggle:addEventHandler(passiveHandler)
    boundsToggle.onclick = function(self)
        MainCamera.controller.boundsLayer.enabled = self.checked;
    end
    
     -- Actors layer
    local actorLocatorToggle = GUI.checkbox(window, GUIAligned(ALIGN_CENTER), GUISize(100, 30), false, "ActorLocator")
    actorLocatorToggle:dock(DOCK_TOP)
    local passiveHandler = {
    	toggle = actorLocatorToggle;
        onPassiveUpdate = function(self)
        	if(MainCamera.controller.actorsLayer) then
	        	if(MainCamera.controller.actorsLayer.debug) then
	        		self:check()
	        	else
	        		self:uncheck()
	        	end
        	end
        end,
    }
    actorLocatorToggle:addEventHandler(passiveHandler)
    actorLocatorToggle.onclick = function(self)
        MainCamera.controller.actorsLayer.debug = self.checked;
    end

    return window
end

