
local Logger = TaggedLog("CameraRig", true)

CameraRig = createClass(
	{
		-- each layer is is a transform that represents a camera rotation offset
		layers = {},
		layersDict = {},
		
		-- resulting transform of all layers
		worldT = SRT.new(),
		camForward = Vector.new(),
		camUp = Vector.new(),
		camRight = Vector.new(),
		initialized = false,
	});

function CameraRig:updateLayers(transform)
	camerarig:updateLayersNative(self)
end

	
function CameraRig:destroyLayers()
	if(self ~= nil and self.layers ~= nil) then
		for i, layer in ipairs(self.layers) do
			
			if(layer.onDestroy) then
				layer:onDestroy()		
			end
			
		end
	end
end

function CameraRig:addLayer(layer, object)
	
	assert(layer.name ~= CameraLayer.name and self.layersDict[layer.name] == nil, "Layer with name '"..layer.name.."' is not unique");
	
	assert(layer.verifyInterface, "Layer with name '"..layer.name.."' does not support CameraLayer interface");
	
	layer:verifyInterface()
	
	layer.object = object
	layer.worldT = SRT.new();
	layer.camForward = Vector.new();
	layer.camUp = Vector.new();
	layer.camRight = Vector.new();
	
	table.insert(self.layers, layer)
	self.layersDict[layer.name] = layer;


	if(layer.onCreate) then
			layer:onCreate()
	end	

	return layer;
end

function CameraRig:startLayers()
	
    -- initialize basis vectors
    self.initialized = true
    self.camForward = Vector.rotateBy(Vector.forward, self.worldT.rotation);
	self.camRight = Vector.rotateBy(Vector.right, self.worldT.rotation);
	self.camUp = Vector.rotateBy(Vector.up, self.worldT.rotation);
	
	for i, layer in ipairs(self.layers) do
		if(layer.onStart) then
			layer:onStart()
		end
	end
end

function CameraRig:getLayer(name)
	return self.layersDict[name]
end