
--[[
Todo:
- a button to reset the camera back to where it started
- a button to set the debug camera to match the real one
]]


-- at a minimum, a component is just a data container.
DebugCameraController = 
{
	linearVelocity = Vector.new(0,0,0),		-- linear velocity
	angularVelocity = Vector.new(0,0,0),	-- angular velocity
	moveSpeed = 100.0,						-- units per second
	step 	  = 5.0,
	turnSpeed = math.rad(10),							
	lookSpeed = math.rad(200),
	linearDamp = 0.15,
	angularDamp = 0.15,
	sensitivity = 60,
	frame = 0,
	blink = 1.0;
	lastUpdateSecs		= nil,
	screenWidth = 1,
	screenHeight = 1,
	textStyle = nil,
	textStyleControl = nil,
}

-- components can also act as behaviors
function DebugCameraController:onStart() 
	self.active = false
	Events.listen(self, "onReshape");
	self.textStyle = { color = Color.new(1,0,0,1), font = "common:fonts/arial_default_24.fnt", scale = 1.0 }
	self.textStyleControl = { color = Color.new(1,0,0,1), font = "common:fonts/arial_default_14.fnt", scale = 1.0 }
end

function DebugCameraController:onReshape(viewport)
	self.screenWidth = viewport.x;
	self.screenHeight = viewport.y;
end

-- demonstrates the use of the Input library to query buttons
function DebugCameraController:onUpdate()

	if(Input.isKeyDown(KB_Q))then
		self.moveSpeed = math.max(self.moveSpeed + self.step, 10.0);
		--self.lookSpeed = math.min(math.rad(100), self.lookSpeed*1.1);
		self.turnSpeed = math.min(math.rad(100), self.turnSpeed*1.1);
	end
	if(Input.isKeyDown(KB_E))then
		self.moveSpeed = math.max(self.moveSpeed - self.step, 10.0);
		--self.lookSpeed = math.max(0.1, self.lookSpeed*0.9);
		self.turnSpeed = math.max(0.1, self.turnSpeed*0.9);
	end
	
	self.frame = self.frame + 1;
	Renderer.drawText(600, 20, self.textStyleControl, string.format("<DEBUG_CAMERA> Speed:%.1f", self.moveSpeed));
	Renderer.drawText(600, 40, self.textStyleControl, string.format("Q speed up, E speed down, Z move up, X move down, WASD forward/back/strafe MouseL look"));
		
	local linearAcc = Vector(0,0,0);
	
	if (Input.isKeyDown(KB_W)) then
		linearAcc.z = linearAcc.z - self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_S)) then
		linearAcc.z = linearAcc.z + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_A)) then
		linearAcc.x = linearAcc.x - self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_D)) then
		linearAcc.x = linearAcc.x + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_Z)) then
		linearAcc.y = linearAcc.y + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_X)) then
		linearAcc.y = linearAcc.z - self.moveSpeed;
	end
	
	local currTimeSecs = os.clock();
	if(self.lastUpdateSecs == nil) then
		self.lastUpdateSecs = currTimeSecs;
	end
	
	local dt = currTimeSecs - self.lastUpdateSecs;
	self.lastUpdateSecs = currTimeSecs;
	
	self.linearVelocity = self.linearVelocity + linearAcc * 2.0 * dt;
	self.linearVelocity.x = math.min(math.max(self.linearVelocity.x, -self.moveSpeed*dt), self.moveSpeed*dt);
	self.linearVelocity.y = math.min(math.max(self.linearVelocity.y, -self.moveSpeed*dt), self.moveSpeed*dt);
	self.linearVelocity.z = math.min(math.max(self.linearVelocity.z, -self.moveSpeed*dt), self.moveSpeed*dt);
	
	local angularAcc = Vector(0,0,0);
	if(Input.isMouseMoving() and Input.getMouseButton(MB_L_BUTTON)) then
		local mouseDelta = Input.getMouseDelta() * self.sensitivity;
		angularAcc.x = -(mouseDelta.y/self.screenHeight) * self.lookSpeed*2.0;
		angularAcc.y = -(mouseDelta.x/self.screenWidth) * self.turnSpeed;
	end
	
	self.angularVelocity = self.angularVelocity + angularAcc * 2.0 * dt;
	self.angularVelocity.x = math.min(math.max(self.angularVelocity.x, -self.lookSpeed*dt), self.lookSpeed*dt);
	self.angularVelocity.y = math.min(math.max(self.angularVelocity.y, -self.turnSpeed*dt), self.turnSpeed*dt);
	
	local localT = self.object:getLocal();
	
	local y = Quaternion.new(); 
	local p = Quaternion.new(); 
		
	y:setAxisAngle(Vector.up, self.angularVelocity.y);
	p:setAxisAngle(Vector.right:rotateBy(localT.rotation), self.angularVelocity.x);
	
	local deltaRot = p * y;
	
	localT.position = localT.position + self.linearVelocity:rotateBy(localT.rotation);
	localT:applyRotation(deltaRot);
	
	self.object:setLocal(localT);

	-- apply damping
	self.linearVelocity = self.linearVelocity * self.linearDamp;
	self.angularVelocity = self.angularVelocity * self.angularDamp;
end
