
local Logger = TaggedLog("ActorClampLayer", true)

--- ActorClamp
-- clamps the Y-Axis of the camera to an actor
-- 
ActorClampLayer = createClass({
    clampToActor		= nil;
    targetBoneName		= nil; -- optional parameter
    pastFirstFrame		= false;
    actorHeadingClamp	= Quaternion.new();
},deepcopy(CameraLayer));


ActorClampLayer.name	= "actorClamp";

function ActorClampLayer:onStart()

end

--- getActorHeadingClamp
-- uses the current world rotation of the camera to compute a rotation that places the camera back at its heading clamp 
function ActorClampLayer:_getActorHeadingClamp(currentRotation)
	local rigTr = Global.scene.cameraRig:getWorld();
	local camAngle, camYaw = currentRotation:getYaw();
	
	local camLook = Vector.rotateBy(Vector.forward:copy(), camYaw);
	local camRight = Vector.rotateBy(Vector.right:copy(), camYaw);
	
	local worldT = self.targetBoneName and self.clampToActor[self.targetBoneName]:getWorld() or self.clampToActor:getWorld()
	
	local toHeading = worldT.position - rigTr.position
	toHeading.y = 0;
	
	local debugOffset = 4.0
	
	local sign = -math.sign(Vector.dot(camRight, toHeading));
	local angle = sign*math.deg(Vector.angle(camLook, toHeading)) + debugOffset;
	local deltaRot = Quaternion.axisAngle(Vector.up:copy(), angle);
    
    local forward = Vector.rotateBy(Vector.forward, Global.scene.mainCamera:getWorld().rotation)
    forward:normalize()
    
    if (self.pastFirstFrame and self.pitchThreshold > 0 and math.abs(forward.y) < self.pitchThreshold) then
        self.clampToActor = nil;
    end
    
    if (self.duration > 0) then
        self.duration = self.duration - Time.dt();
        if (self.duration < 0) then
            self.clampToActor = nil;
        end
    end
	self.pastFirstFrame = true;
	return deltaRot;
end

--- setActorHeadingClamp
-- clamps the camera to the actor passed in ( only affects the Y-axis of the camera)
function ActorClampLayer:setActorHeadingClamp(actor, duration, pitchThreshold, targetBoneName)
    self.duration = duration;
    self.pitchThreshold = pitchThreshold;
	self.targetBoneName = targetBoneName;
	self.clampToActor = actor;
end