
 local Logger = TaggedLog("ActorVignette", true)
 local glowParameters={}

 local min  = math.min
 local lerp = math.lerp

--- Actor Locator subclass
-- a locator is setup for each actor in order to track its position and scale used to for final render
ActorVignette = createClass({

	name = "not_set",
	color = Color(0,1,1, 0),
	actor = nil,
	debugSphere = nil,
	debugSphereTL = nil,
	debugSphereBR = nil,
	uniqueFrameCounter = 0,
	dist = 0,
	fadeTimeScale = 2.0,
	distV = {},
	--minDist = 0,

	points = {
		{ position = Vector(), sphere = {}},
	},
	
	boneGroups = {
		dog = {
			glowCenter 	= "glowBone",
			glowTop 	= "glowTop",
			glowBottom 	= "glowBottom",
			up			= "glowUp",
			intensity	= "glowIntensity",
		},
		boy = {
			glowCenter 	= "glowBone",
			glowTop 	= "glowTop",
			glowBottom 	= "glowBottom",
			up			= "glowUp",
			intensity	= "glowIntensity",
		}
	},
	
	bones = nil,
	indexName = nil, -- which glow position to map to boy = 0, girl = 1, dog = 2
	
	scale = 1, 		-- scale passed to actor locator
	debugScale = 1, -- scale for debug rendering
	timer = 0,
	maxAlpha = 1.0,

	position = Vector(),

	--- Initialize the actor
	init = function(self)
		
		self.glowIndex = self.groupToIndex[self.indexName or self.name];
	    self:getGroupActor()
	    
	    self.points = {}
	    
	    SceneData.actors[self.name.."Sphere_TL"] = shallowcopy(SceneData.actors[self.name.."Sphere"])
	    SceneData.actors[self.name.."Sphere_BR"] = shallowcopy(SceneData.actors[self.name.."Sphere"])
	    
	    self.points[1] = {}
	    self.points[1].position = Vector()
		self.position = Vector()
	    if(self.debug) then
		    self.points[1].sphere 	= ActorManager:getActor(self.name.."Sphere", true)
		    self.points[1].sphere:setVisibility(true)
		    self.points[1].sphere:setLocal(SRT.new())
	    end
	    
	    if(self.debug) then
	    	self.points[2] = {}
	    	self.points[2].position = Vector()
		    self.points[2].sphere 	= ActorManager:getActor(self.name.."Sphere_TL", true)
		    self.points[2].sphere:setVisibility(false)
		    self.points[2].sphere:setLocal(SRT.new())
	    	
	    	self.points[3] = {}
	    	self.points[3].position = Vector()
		    self.points[3].sphere 	= ActorManager:getActor(self.name.."Sphere_BR", true)
		    self.points[3].sphere:setVisibility(false)
		    self.points[3].sphere:setLocal(SRT.new())
	    end
	    
		-- listen for flipbook frame change
	    Events.listen(self, "exposureChanged");
	    
	    self.uniqueFrameCounter = 0;

	    self.distV[0] = 0
		self.distV[1] = 0
		self.distV[2] = 0

		local vignette = Global.scene.actors.Vignette
		if (vignette) then
			self.vignetteMaterial = vignette:findMaterial("Mat_Quad")
		end		
	end,
	
	exposureChanged = function(self, name)
		if(self.actor and name == self.actor.name) then
			self.uniqueFrameCounter = self.uniqueFrameCounter + 1;
		end
	end,
	
	glowIndex = 0,
	
	groupToIndex = {
		boy = 0,
		girl = 1,
		dog = 2,
	},

	
	offsetPosition = Vector(),

	-- hide the character locator
	hide = function(self)
		self.timer = 0
		self.color.a = lerp(self.color.a, 0, Time.dt()*self.fadeTimeScale)
		
		self.offsetPosition = self.position--self.points[1].position;
		actorglow.hide(self.glowIndex, self.color.a, self.offsetPosition.x, self.offsetPosition.y, self.offsetPosition.z)
	end,
	
	--- show the character location
	show = function(self)
		self.timer = (self.timer + Time.dt())
		self.timer = min(1, self.timer)
		self.color.a = math.lerp(self.color.a, self.maxAlpha, Time.dt()*self.fadeTimeScale)
		
		self.offsetPosition = self.position--self.points[1].position;
		actorglow.show(self.actor, self.glowIndex, self.color.a, self.offsetPosition.x, self.offsetPosition.y, self.offsetPosition.z)
	end,
	
	--- Update the actors glow
	update = function(self)

		if ( self.actor ) then
			--once per frame setup.
			if ( self.glowIndex == 0 ) then
				local camT = MainCamera:getWorld();
				duet.setCamera(camT.rotation.x, camT.rotation.y, camT.rotation.z, camT.rotation.w, camT.position.x, camT.position.y, camT.position.z)
				-- TODO: delete this dummy stuff
				if( actorglow.dummy ) then
					actorglow.setGlowSettings( not ActorManager.disableGlow, Global.opacity )
				else
					actorglow.setGlowSettings( not ActorManager.disableGlow, false, Global.opacity )
				end
			end

			--update the actor glow for every actor.
			local intensityName = self.actor.namedBones[self.bones.intensity] or "none"
			local upName = self.actor.namedBones[self.bones.up] or "none"
			local centerName = self.actor.namedBones[self.bones.glowCenter] or "none"
			local TLName = self.actor.namedBones[self.bones.glowTop] or "none"
			local BRName = self.actor.namedBones[self.bones.glowBottom] or "none"

			if (self.glowIndex ~= nil) then
				self.distV[self.glowIndex] = self.dist
			end

		
			Global.minDist = min(min(self.distV[0], self.distV[1]),self.distV[2])	
			
			local vignette = Global.scene.actors.Vignette
			if(self.vignetteMaterial == nil) then
				self.vignetteMaterial = vignette:findMaterial("Mat_Quad")
			end

			if(Global.minDist > 0 and Global.minDist < 3) then
	    		if(self.vignetteMaterial) then
	    			if(Global.minDist) then
	    				local amount = math.clamp01(1 - Global.minDist * 3.0)
	    				self.vignetteMaterial:setParameters({ u_Alpha = amount})
	    			else
	    				self.vignetteMaterial:setParameters({ u_Alpha = 0})
	    			end
	    		end
			else
		    	if(self.vignetteMaterial) then
		    		self.vignetteMaterial:setParameters({ u_Alpha = 1})
		    	end
			end

			actorglow.update(self.actor, self.glowIndex, self.dist, intensityName, upName, centerName, TLName, BRName)
		end
	end,
		
	getGroupActor = function(self)
		for i, group in ipairs(self.boneGroups) do
			local actor = ActorManager:getActor(ActorManager.proxyActors[group.name])
			if(actor) then
				self.bones = group.bones;
				self.actor = actor;
				self.activeActor = true
				return actor;
			end
		end
		
		-- not actors found these become nil
		self.bones = nil;
		self.actor = nil;
		self.activeActor = false
		
		return nil;
	end,
	
	getGlowCenter = function(self)
		if(self.activeActor) then
			return self.actor[self.bones.glowCenter]
		end
		return nil
	end,
	
	getGlowTop = function(self)
		if(self.activeActor) then
			return self.actor[self.bones.glowTop]
		end
		return nil
	end,
	
	getGlowBottom = function(self)
		if(self.activeActor) then
			return self.actor[self.bones.glowBottom]
		end
		return nil
	end,
})
