
require "episode3/scripts/components/camera/layers/actorglow"

local Logger = TaggedLog("ActorSlayer", true)

ActorsLayer = createClass({
    -- the distance of the plane the locator is project to
    projectionDist = 1050;
    
    -- the scale of the locator object
    scale = 1;
    aspect = 1;
    
 	width = 720;
 	height = 1184;
 	
 	fovFudge = 1.1;
 	targetFovFudge = 1.1;
 	updateFov = false;
 	
 	debug = false;
 	
 	allActorsOffscreen	= false;
 	allOffscreenTimer	= 0;
 	offscreenTimeThreshold = 1.0;   -- the amount of time that all actors must be offscreen until all actors are really considered offscreen
 	offscreenStartPosition = nil;
 	fadeTimeScale = 2.0;

	rightEdge = nil;
	topEdge = nil;
	actors = nil;
 	
},deepcopy(CameraLayer));

ActorsLayer.name	= "actorsLayer";

function ActorsLayer:onStart()

	self.fov = Global.fov*self.fovFudge;
	self.aspect = Global.curScreenWidth/Global.curScreenHeight

	-- screen edge debug
	self.rightEdge 	= Global.scene:createTransform("rightEdge")
	self.topEdge 	= Global.scene:createTransform("topEdge")
	
	Global.scene.mainCamera:addChild(self.rightEdge)
	Global.scene.mainCamera:addChild(self.topEdge)

	self:onReshape(Vector(Global.curScreenWidth or 1184, Global.curScreenHeight or 720))
	
	if(self.debug) then
		-- Debug models for actor locator
		SceneData.actors.girlSphere  = {
		    model = "episode3:models/screenquad.mxm",
		    position = { 50*math.cos(math.pi/2 - math.pi/3), 0, -50*math.sin(math.pi/2 - math.pi/3)},
		    rotation = Quaternion.new(),
		    materials = {
		        {
		            name = "Mat_Quad",
		            shader = "episode3:shaders/launcher.rsfx",
		            technique = "transparent_with_color",
		            renderGroup = 16,
		            textures =
		            {
		            	u_MainTex = "episode3:textures/testtexture.dds",
		            },
		            parameters =
		            {
		                u_Color = Color.new( 1.0, 0.5, 1.0, 0.1 ),
		            }
		        },
		    },
		    
		    
		    components = { {name = "Billboard"}},
		}
		
		SceneData.actors.boySphere  = {
		    model = "episode3:models/screenquad.mxm",
		    position = { 50*math.cos(math.pi/2 + math.pi/3), 0, -50*math.sin(math.pi/2 + math.pi/3)},
		    rotation = Quaternion.new(),
		    materials = {
		        {
		            name = "Mat_Quad",
		            shader = "episode3:shaders/launcher.rsfx",
		            technique = "transparent_with_color",
		            renderGroup = 15,
		            textures =
		            {
		                u_MainTex = "episode3:textures/testtexture.dds",
		            },
		            parameters =
		            {
		                u_Color = Color.new( 1.0, 0.1, 1.0, 0.1 ),
		            }
		        },
		    },
		    
		    components = { {name = "Billboard"}},
		}
		
		SceneData.actors.dogSphere  = {
		    model = "episode3:models/screenquad.mxm",
		    position = { 50*math.cos(math.pi/2 + math.pi/3), 0, -50*math.sin(math.pi/2 + math.pi/3)},
		    rotation = Quaternion.new(),	
		    materials = {
		        {
		            name = "Mat_Quad",
		            shader = "episode3:shaders/launcher.rsfx",
		            technique = "transparent_with_color",
		            renderGroup = 15,
		            textures =
		            {
		                u_MainTex = "episode3:textures/testtexture.dds",
		            },
		            parameters =
		            {
		                u_Color = Color.new( 0.5, 0.5, 0.5, 0.1 ),
		            }
		        },
		    },
		    
		    components = { {name = "Billboard"}},
		}
	end
	
	local boy = ActorVignette:new(
		{
			name="boy", 
			boneGroups = {
				{
					name = "boy",
					bones={
						glowCenter 	= "glowBone",
						glowTop 	= "glowTop",
						glowBottom 	= "glowBottom",
						up			= "glowUp",
						intensity	= "glowIntensity",
					}
				},
				{
					name = "girl",
					bones={
						glowCenter 	= "boyGlowBone",
						glowTop 	= "boyGlowTop",
						glowBottom 	= "boyGlowBottom",
						up			= "boyGlowUp",
						intensity	= "boyGlowIntensity",
					}
				},
				{
					name = "dog",
					bones={
						glowCenter 	= "boyGlowBone",
						glowTop 	= "boyGlowTop",
						glowBottom 	= "boyGlowBottom",
						up			= "boyGlowUp",
						intensity	= "boyGlowIntensity",
					}
				},
			},
			color = Color(0,1,1),
			fadeTimeScale = self.fadeTimeScale, 
			debug = self.debug
		})
	local girl = ActorVignette:new(
		{
			name="girl",
			boneGroups = {
				{
					name="girl",
					bones={
						glowCenter 	= "glowBone",
						glowTop 	= "glowTop",
						glowBottom 	= "glowBottom",
						up			= "glowUp",
						intensity	= "glowIntensity",
					}
				},
				{
					name = "boy",
					bones={
						glowCenter 	= "girlGlowBone",
						glowTop 	= "girlGlowTop",
						glowBottom 	= "girlGlowBottom",
						up			= "girlGlowUp",
						intensity	= "girlGlowIntensity",
					}
				},
				{
					name = "dog",
					bones={
						glowCenter 	= "girlGlowBone",
						glowTop 	= "girlGlowTop",
						glowBottom 	= "girlGlowBottom",
						up			= "girlGlowUp",
						intensity	= "girlGlowIntensity",
					}
				},
			},
			color = Color(1,0,1), 
			fadeTimeScale = self.fadeTimeScale,
			debug = self.debug})
	local dog = ActorVignette:new(
		{
			name="dog", 
			boneGroups = {
				{
					name = "dog",
					bones={
						glowCenter 	= "glowBone",
						glowTop 	= "glowTop",
						glowBottom 	= "glowBottom",
						up			= "glowUp",
						intensity	= "glowIntensity",
					}
				},
				{
					name="boy",
					bones = {
						glowCenter 	= "dogGlowBone",
						glowTop 	= "dogGlowTop",
						glowBottom 	= "dogGlowBottom",
						up			= "dogGlowUp",
						intensity	= "dogGlowIntensity",
					}
				},
				{
					name="girl",
					bones = {
						glowCenter 	= "dogGlowBone",
						glowTop 	= "dogGlowTop",
						glowBottom 	= "dogGlowBottom",
						up			= "dogGlowUp",
						intensity	= "dogGlowIntensity",
					}
				},
			},
			color = Color(1,1,1), 
			fadeTimeScale = self.fadeTimeScale,
			debug = self.debug
		})
	boy:init(); girl:init(); dog:init()
	self.actors = {
		boy,
		girl,
		dog
	}

    Events.listen(self, "onReshape");
	Events.listen(self, "segmentPlaying");

	if (actorslayer and actorslayer.onStartNative) then
		actorslayer.onStartNative(self)
	end
end

function ActorsLayer:getLineSegmentToTarget(actor)

	local camT = self.worldT
	-- optimization
	local camLook = self.camForward:copy();
	local camRight = self.camRight:copy();
	local camUp = self.camUp:copy();
	
	-- actor target bone position in world space
	local worldPos = actor:getGlowCenter():getWorld().position
	
	-- get vector pointing at the boy (from camera)
	local toTarget = worldPos - camT.position
	-- project vector to boy against the camera 'look' - provides distance from camera plane
	local length = (camLook:dot(toTarget))
	local sign = Vector.dot(toTarget, camRight);
	-- gets the vector that points at the target and is perpendicular to the camera look ray
	toTarget = toTarget - camLook*length
	
	local rotToTarget = Quaternion.lookAt(worldPos - camT.position);
--			DrawText:addText("rot to "..actor.name, "Rot from %s, %.3f", actor.name, (camT.rotation:inverse()*rotToTarget):getYawAngle())
	
	-- project to normalized screen space (w divide) then move it to the projection plane
	toTarget = toTarget*(1/math.abs(length))*self.projectionDist
	
	-- an object that represents the perpendicular line segment from camera ray to target
	local segToTarget = {
		startPos = (camT.position + camLook*self.projectionDist),
		endPos = (camT.position + camLook*self.projectionDist + toTarget),
		dir = ((camT.position + camLook*self.projectionDist + toTarget)-(camT.position + camLook*self.projectionDist)),
	}
	
	return segToTarget;
end

function ActorsLayer:getFrustumCollisionPoint(lineSegmentToTarget)
	-- distance to collision point
	local length = math.floatMax
	local outColPoint = nil
	for _, plane in ipairs(self.frustum) do
		
		if(Vector.dot(lineSegmentToTarget.dir, plane.normal) > 0) then
--				RenderDebug.point(plane.point, Color(0, 1, 0), 2)
		else
--				RenderDebug.point(plane.point, actor.color, 2)
			
			-- distance to end of segment wrt plane
			local endDist 		= Vector.dot(lineSegmentToTarget.endPos, plane.normal)
			local begDist 		= Vector.dot(lineSegmentToTarget.startPos, plane.normal)
			local planeDist 	= Vector.dot(plane.point, plane.normal)
			local segLength 	= (endDist - begDist)
			local toPlaneDist	= (planeDist - begDist)/segLength;
			
			local toColPoint = (lineSegmentToTarget.endPos - lineSegmentToTarget.startPos)*toPlaneDist
			
			-- the point of collision with frustum plane
			local colPoint = lineSegmentToTarget.startPos + toColPoint
			
			local testLength = colPoint:lengthSq()
			
			-- save only the nearest collision point (from camera ray) for this ray cast
			if(testLength < length) then
				length = testLength
				outColPoint = colPoint
			end
		end
	end
	
	return outColPoint
end

function ActorsLayer:onUpdate(dt)
	--local timeBegin = duet.hiresTimerGet();

	local camT = self.worldT
	
	-- optimization
	local camLook = self.camForward:copy()
	local camRight = self.camRight:copy()
	local camUp = self.camUp:copy()
	
	local allActorsWereOffscreen = self.allActorsOffscreen;
	
	if(self.updateFov) then
		self.fovFudge = math.lerp(self.fovFudge, self.targetFovFudge, dt*2.0)
		self.fov = Global.fov*self.fovFudge;
		
		self:updateScreenEdge()
--		DrawText:color(0,1,0):addText("fovupdate", "Updating FOV: %.3f", self.fovFudge)
		if(math.abs(self.fovFudge - self.targetFovFudge) < 0.0019) then
			self.updateFov = false;
--			DrawText:color(0,1,0):addText("fovupdate", "FOV Settled at: %.3f", self.fovFudge)
		end
	end
	
	local onScreenCount = 0;
	
	-- find all actors onscreen
	for _, actor in ipairs(self.actors) do
		
		-- get the latest actor from the actor group, actors change per segment this will make sure
		-- you have the actor that represents the boy group or the girl group etc...
		actor:getGroupActor()
		
		actor.active = false
		if(actor.actor and actor:getGlowCenter()) then
			
			actor.active = true
			
			-- actor target bone position in world space
			local worldPos = actor:getGlowCenter():getWorld().position
			worldPos.w = 1;
--			worldPos = worldPos:rotateBy(moverLayerCorrection);
			
			local ndc;
			local camPos	= MainCamera:getWorld().position;
			local camLook 	= MainCamera.camForward:copy();
			local camRight 	= MainCamera.camRight:copy();
			local camUp 	= MainCamera.camUp:copy();
			local zNear		= Global.nearFar.n;
			local zFar		= Global.nearFar.f;

			ndc = util.toClipSpace(	worldPos,
									camPos, camLook, camRight, camUp,
									self.fov, self.aspect,
									zNear, zFar);

--			Log.message("FOV: "..self.fov.. string.format("NDC <%.3f, %.3f, %.3f>", ndc.x, ndc.y, ndc.z))
			local onscreen = false
			if( ndc.y < 1.0 and ndc.y > -1.0 and ndc.x < 1.0 and ndc.x > -1.0 and ndc.z > 0 and ndc.z < 1) then
				-- debug is off screen or not
--				DrawText:addText(actor.name.."_onscreen", "%s is ON screen", actor.name);
				onscreen = true
				onScreenCount = onScreenCount + 1
				
				
			-- debug is off screen or not
--			else
--				DrawText:addText(actor.name.."_onscreen", "%s is OFF screen", actor.name);
			
			end
			
			actor.worldPos = worldPos
			actor.onscreen = onscreen
		
		end
	end
	
	-- if all actors are off screen start off screen timer
	if(onScreenCount == 0) then
		-- mark this position
		if(not self.offscreenStartPosition) then
			self.offscreenStartPosition = camT.rotation:copy()
		end

		self.allActorsOffscreen, self.allOffscreenTimer = actorslayer.updateOffscreenTimer(	dt,
																							self.allOffscreenTimer,
																							self.offscreenTimeThreshold,
																							self.offscreenStartPosition,
																							camT.rotation,
																							self.allActorsOffscreen);
	else
		self.allActorsOffscreen = false;
		self.allOffscreenTimer	= 0;
		self.offscreenStartPosition = nil;
	end
	
	-- For each actor find point on screen or at the edge of the screen
	for _, actor in ipairs(self.actors) do
		
		-- initial 
		local colPoint = nil
		
		if(actor.active) then
			
			local halfSpace = Vector.dot(actor.worldPos, camLook);
			if(halfSpace < 0) then
				actor.dist = 1.0
			else
				local distVec = Vector( Vector.dot(camRight, actor.worldPos), actor.worldPos.dot(camUp, actor.worldPos) )
				actor.dist = math.max(1.0, 2.0*distVec:length()/2150) - 1.0
			end
				
			local worldPos = actor:getGlowCenter():getWorld().position
			local camPosition = self.worldT.position;
			local _startPos;
			local _endPos;
			local _dir;
			_startPos, _endPos, _dir = actorslayer.getLineSegmentToTarget(	camPosition,
																			camLook,
																			camRight,
																			camUp,
																			worldPos,
																			self.projectionDist);
			local segToTarget = { startPos = _startPos, endPos = _endPos, dir = _dir };

			if(not self.allActorsOffscreen) then
				-- point inside view frustum
				colPoint = segToTarget.endPos
			else
				-- re-calculate the camera frustum planes
				local pointR, pointL, pointT, pointB;
				local normalR, normalL, normalT, normalB;
		
				pointR, pointL, pointT, pointB,
				normalR, normalL, normalT, normalB =
				actorslayer.updateCameraFrustum(self.worldT.position,
												self.worldT.rotation,
												self.camForward,
												self.camRight,
												self.camUp,
												self.projectionDist,
												self.xSlope,
												self.ySlope);
				self.frustum = {}
				table.insert(self.frustum, {point = pointR, normal = normalR});-- right plane
				table.insert(self.frustum, {point = pointL, normal = normalL});-- left plane
				table.insert(self.frustum, {point = pointT, normal = normalT});-- top plane
				table.insert(self.frustum, {point = pointB, normal = normalB});-- bottom plane
				
				if(self.debug) then
					RenderDebug.line(segToTarget.startPos, segToTarget.endPos, actor.color);
				end

				colPoint = self:getFrustumCollisionPoint(segToTarget)
			end
		else
			--If actor is inactive, we don't want to consider the actor for offscreen glow
			actor.dist = 5
		end
		
		if(colPoint) then
			actor.position 				= colPoint
			actor.debug 				= self.debug
			actor.scale 				= self.scale
			actor.debugScale 			= self.aspect*self.projectionDist*0.05
			actor.screenRight 			= camRight:copy();
			actor.screenUp 				= camUp:copy();
			actor.fadeTimeScale			= self.fadeTimeScale;
			if(self.debug) then
				actor.points[2].position 	= self.topLeft
				actor.points[3].position 	= self.bottomRight
			end
		end
		
		if(actor.active) then
		
			-- if all actors just went off screen this frame, set alpha to 0 to trigger a fade in
			if(not allActorsWereOffscreen and self.allActorsOffscreen) then
				actor.color.a = 0
			end
			
			-- show off screen glow
			actor:show()
		else
			actor:hide()
		end
		
		actor:update();
	end

	--local timeEnd = duet.hiresTimerGet();
	--local time = timeEnd - timeBegin;
	--Logger:message("ActorsLayer:onUpdate duration " .. time);

end


ActorsLayer.updateScreenEdge = function(self)
	
	self.aspect = (self.width/self.height);
	self.depth = -(Global.nearFar.f - Global.nearFar.n)/2;
	self.persp = 1/(Global.nearFar.f - Global.nearFar.n);
	
	self.xSlope, self.ySlope = Util.frustumSlopes(self.fov, self.aspect);
        
    local rightEdgeT 	= self.rightEdge:getLocal()
    local topEdgeT 		= self.topEdge:getLocal()
    rightEdgeT.position = Vector(-self.depth*self.xSlope, 0, self.depth); 
    topEdgeT.position 	= Vector(0, -self.depth*self.ySlope, self.depth);
    
    self.rightEdge:setLocal(rightEdgeT);
    self.topEdge:setLocal(topEdgeT);
end

ActorsLayer.onReshape = function(self, size)
	self.width = size.x;
	self.height = size.y;
	self.fov = Global.fov*self.fovFudge;
	self:updateScreenEdge()
end

ActorsLayer.segmentPlaying = function(self, name)
	if(ActorTracking and self.fovFudge ~= ActorTracking[name].fovFudge) then
		self.targetFovFudge = ActorTracking[name].fovFudge;
		self.updateFov = true;
	end
end

