


BoundsLayer = createClass({
	-- camera view bounds, a list of transforms that the camera cannot crossover
    boundingActors		= {left = nil, top = nil, right = nil, bottom = nil},
    boundingOffset		= Quaternion.new();
    enabled				= true;
    boundingDirty		= false;
    Left				= 0;
    Right				= 1;
    rotation			= Quaternion.new();
},deepcopy(CameraLayer));

BoundsLayer.name		= "boundsLayer";

function BoundsLayer:onStart()

	self.debug = false;
	self.boundingOffset	= Quaternion.new();
	
	---------------------------
	SceneData.actors.rightSphere  = {
	    model = "episode3:models/unitsphere.mxm",
	    position = { 2050*math.cos(math.pi/2 - math.pi/2.1), 0, -2050*math.sin(math.pi/2 - math.pi/2.1)},
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/debug.rsfx",
	            technique = "color",
	            renderGroup = 2,
	            textures =
	            {
	            },
	            parameters =
	            {
	                u_Color = Color.new( 1.0, 0.5, 1.0, 1.0 ),
	            }
	        },
	    },
	}
	
	SceneData.actors.leftSphere  = {
	    model = "episode3:models/unitsphere.mxm",
	    --position = { -10, 0, -50},
	    
	    position = { 2050*math.cos(math.pi/2 + math.pi/2.1), 0, -2050*math.sin(math.pi/2 + math.pi/2.1)},
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/launcher.rsfx",
	            technique = "transparent_with_color",
	            renderGroup = 2,
	            textures =
	            {
	                u_MainTex = "episode3:textures/white.dds",
	            },
	            parameters =
	            {
	                u_Color = Color.new( 0.5, 0.5, 1.0, 1.0 ),
	            }
	        },
	    },
	}
		
	self.rightSphere = ActorManager:getActor("rightSphere", true)
	self.leftSphere = ActorManager:getActor("leftSphere", true)
	self.boundRoot = ActorManager:getActor("boundRoot", true)
	
	self.boundRoot:setLocal(SRT.new())
	self.boundRoot:addChild(self.rightSphere)
	self.boundRoot:addChild(self.leftSphere)
	
	self.rightSphere:setScale(Vector(100, 100, 100))
	self.leftSphere:setScale(Vector(100, 100, 100))
	
	self.boundingActors.left = self.leftSphere
	self.boundingActors.right = self.rightSphere
	
	self.enabled = false
	-------------------

	 if(self.debug) then
	 
		self.boundRoot:setVisibility(true)
		self.rightSphere:setVisibility(true)
		self.leftSphere:setVisibility(true)
	
		self.width 	= Global.curScreenWidth;
		self.height = Global.curScreenHeight;
		
		self.persp = 1/(Global.nearFar.f - Global.nearFar.n);
		
		self.aspect = Global.curScreenWidth/Global.curScreenHeight;
	    
	    self.ySlope = math.tan(math.rad(Global.fov)*0.5)
	    self.xSlope = self.ySlope*self.aspect;
	    
	end
	
	Events.listen(self, "segmentPlaying")
	Events.listen(self, "onReshape")
end

function BoundsLayer:onReshape(screenSize)
	self.width  	= screenSize.x
	self.height  	= screenSize.y
	self.persp = 1/(Global.nearFar.f - Global.nearFar.n);
end


function BoundsLayer:segmentPlaying(name)
	self.segmentName = name;
--	if(not CameraBounding) then return end
--	self.enabled = CameraBounding[name].enabled;
--	
--	if(not self.enabled) then
--		self.boundingActors.left = nil
--		self.boundingActors.right = nil
--	end
--	
	Log.message("Bounding for "..name.." updated to "..tostring(self.enabled))
end

function BoundsLayer:setBoundsEnabled(enabled)
	
	if(enabled ~= self.enabled) then
		self.boundingDirty = true;
		self.enabled = enabled
		CameraBounding[self.segmentName].boundsEnabled = enabled;
	end
end

function BoundsLayer:onShutdown()

--	-- write out changes to tracking
--	if(self.trackingDataDirty) then
--	
--		local table = "return "..Serpent.dumppretty(CameraBounding)
--	    
--		local file,err = io.open( PersistentPath.."camerabounding.lua", "w" )
--		file:write(table);
--		file:close()
--	end
	
end

function BoundsLayer:setBound(side, position)
	
	local posTable = {
			position.x,
			position.y,
			position.z,
	}
		
	if(side == BoundsLayer.Left) then
		SceneData.actors.LeftBound.position = posTable
		ActorManager:destroyActor(ActorManager:getActor("LeftBound"));
		self.boundingActors.left = ActorManager:getActor("LeftBound", true)
	else
		SceneData.actors.RightBound.position = posTable
		ActorManager:destroyActor(ActorManager:getActor("RightBound"));
		self.boundingActors.left = ActorManager:getActor("RightBound", true)
	end
end

function BoundsLayer:removeBound(side)
	if(side == BoundsLayer.Left) then
		self.boundingActors.left = nil
		ActorManager:destroyActor(ActorManager:getActor("LeftBound"));
	else
		self.boundingActors.right = nil
		ActorManager:destroyActor(ActorManager:getActor("RightBound"));
	end
end

--- updateBounds
-- checks camera view direction against a frustum of 'bounding actors' and forces view angle to stay within its bounds
function BoundsLayer:updateBounds(curWorld, dt)
	if(not self.enabled or self.boundingActors.left == nil and self.boundingActors.right == nil) then return self.boundingOffset; end
	
	local girl = ActorManager:getActor(ActorManager.proxyActors["girl"])
	local boy = ActorManager:getActor(ActorManager.proxyActors["boy"])
	
	if(girl and boy) then
		local angle = math.deg(Vector.angle(girl.targetBone:getWorld().position, boy.targetBone:getWorld().position))
		local mid = Vector.rotateBy(boy.targetBone:getWorld().position, Quaternion.axisAngle(Vector.up, angle/2):inverse())
		
--		local goalRotation = girl.targetBone:getWorld().rotation:inverse()*boy.targetBone:getWorld().rotation
--		angle, goalRotation = goalRotation:getYaw()
--		
--		goalRotation = Quaternion.axisAngle(Vector.up, angle/2)
		
		DrawText:addText("boundsdriver", " angle between %.3f", angle/2)
		
		local rotation = Quaternion.lookAt(mid)
		local localT = self.boundRoot:getLocal()
		
		localT.rotation = rotation
		self.boundRoot:setLocal(localT)
		
	elseif(girl) then
		local localT = self.boundRoot:getLocal()
		local angle = 0
		angle, localT.rotation = Quaternion.lookAt(girl.targetBone:getWorld().position):getYaw()
		self.boundRoot:setLocal(localT)
	elseif(boy) then
		local localT = self.boundRoot:getLocal()
		local angle = 0
		angle, localT.rotation = Quaternion.lookAt(boy.targetBone:getWorld().position):getYaw()
		self.boundRoot:setLocal(localT)
	end
	
	local curRot = curWorld.rotation
	
	-- optimization
	local camLook = self.camForward:copy()
	local camRight = Vector.rotateBy(Vector.right:copy(), curRot);
	local camUp = Vector.rotateBy(Vector.up:copy(), curRot);
	
	local camLookWOffset = Vector.rotateBy(Vector.forward:copy(), self.boundingOffset*curRot);
	
	local distRight = -1;
	local distLeft = 1;
	local toLeft = Vector.zero:copy();
	local toRight = Vector.zero:copy();
	
	if(self.boundingActors.left ~= nil) then
		toLeft = self.boundingActors.left:getWorld().position - curWorld.position;
		local leftHs = Vector.cross(toLeft, Vector.up);	
		distLeft = Vector.dot(camLookWOffset, leftHs);
	end
	
	if(self.boundingActors.right ~= nil) then
		toRight = self.boundingActors.right:getWorld().position - curWorld.position;
		local rightHs = Vector.cross(toRight, Vector.up);
		distRight = Vector.dot(camLookWOffset, rightHs);
	end
	
	local lookRotation = Quaternion.lookAt(camLook);
	
	-- are we outside bounding frustum
	if(distLeft < 0 or distRight > 0) then
		--self.offsetDirty = true;
		local pivotRotation = Quaternion.new();
		local yaw, yawQuat = lookRotation:getYaw();
		
		if(distLeft < 0) then
			yaw, pivotRotation = (yawQuat:inverse()*Quaternion.lookAt(toLeft)):getYaw();
		elseif(distRight > 0) then
			yaw, pivotRotation = (yawQuat:inverse()*Quaternion.lookAt(toRight)):getYaw();
		end
		
		self.boundingOffset = pivotRotation;
	end
	
    --- Do debug rendering
	if(self.debug) then
		self:drawDebug(distLeft, distRight)
    end
	
    return self.boundingOffset;
end

function BoundsLayer:drawDebug(distLeft, distRight)
	
	local yaw, yawQ = MainCamera:getLocal().rotation:getYaw()
	local camLook = Vector.rotateBy(Vector.forward, MainCamera:getLocal().rotation);
	local up = Vector.rotateBy(Vector.forward, yawQ)
	local angle = math.abs(Vector.dot(camLook, up));
	DrawText:addText("AngleBounds", "Angle: %.3f", angle);
	
	DrawText:addText("Left/Right dist", "Left/Right dist <%.3f, %.3f>", distLeft, distRight)
    
	DrawText:addText("BoundsCorrection", "Correction <%.3f>", self.boundingOffset:getYawAngle())
	
	self.boundList = self.boundList or {}
	self.boundList["left"] = self.boundingActors.left
	self.boundList["right"] = self.boundingActors.right
	
	for name, bound in pairs(self.boundList) do
    	local worldT = bound:getWorld()
		local dist = (worldT.position - self.worldT.position)
		
		worldT.scale = Vector(1,1,1)
		
		local scale = dist:length() - Global.nearFar.n
		
		-- cull out results in front of the near plane
		if(scale >= 0) then
			scale = (self.persp*scale)*(self.ySlope*Global.nearFar.f*0.2) + 1.0; 
			local ndc = Util.getWindowPosition(bound:getWorld().position, Global.curScreenWidth, Global.curScreenHeight)
			if(ndc) then
				if(name == "right") then
					
    				RenderDebug.point(bound:getWorld().position, Color(1, 0, 0), scale)
					local textStyle = { color = Color.new(0, 1.0, 1.0), font = "common:fonts/arial_default_14.fnt", scale = 1.0 }
					Renderer.drawText(ndc.x+ 5, ndc.y - 15, textStyle, string.format("%s Right Cam Post", bound.name))
					
    			else
    				RenderDebug.point(bound:getWorld().position, Color(0, 0, 1), scale)
					local textStyle = { color = Color.new(0, 1.0, 1.0), font = "common:fonts/arial_default_14.fnt", scale = 1.0 }
					Renderer.drawText(ndc.x+ 5, ndc.y - 15, textStyle, string.format("%s Left Cam Post", bound.name))
    			end
			end
		end
	end
end
