

---HeadingClamp
--clamps the camera's Y-Axis to a given heading
HeadingClampLayer = createClass({
	name				= "headingClamp";
    yClampStart			= nil;
    yClampFinal			= 0;
    lastYClampFinal		= 0;
    isUp				= true;
    isDown				= false;
    hitQuad				= {x=0, y=0, width=0, height=0};
    hitQuadSize			= .875;
    lastYaw             = 0;
},deepcopy(CameraLayer));

HeadingClampLayer.name	= "headingClamp";

function HeadingClampLayer:onStart()
	local size = math.clamp01(self.hitQuadSize)
	
    local aspectX = Global.screen_size.x/Global.screen_size.y;
    local aspectY = Global.screen_size.y/Global.screen_size.x;
	
	-- setup a quad that represent the touch area on screen ( input on the edges of the screen should ignored )
    self.hitQuad = {
        xMin=Global.screen_size.x*(1 - size),
        yMin=Global.screen_size.y*(1 - size),
        xMax=Global.screen_size.x*(size),
        yMax=Global.screen_size.y*(size)
    };
    
    self:enableTouchControls(true);
    
end

function HeadingClampLayer:onUpdate()
	
	self.lastYaw, self.lastYawQuat = self.worldT.rotation:getYaw()
	
	 if(self.lockHeading) then
         local offset = self:_getHeadingClamp(self.lastYaw);
         if(offset) then
            self.localT.rotation = offset:copy()
         end
    end
end

function HeadingClampLayer:enableTouchControls(enabled)
    self.touchControls = enabled;

    if (self.touchControls) then
        Events.listen(self, "touchDown");
        Events.listen(self, "touchUp");
    else
        Events.stopListening(self, "touchDown");
        Events.stopListening(self, "touchUp");
    end
end

function HeadingClampLayer:touchDown(touchPosition)
	if(self.isDown) then
        Log.message("Touch: UP NOT RECVD");
        self.isUp = true;
        self:resetInput(touchPosition);
    end

    self.isDown = true;

    if(touchPosition.x < self.hitQuad.xMax and touchPosition.y < self.hitQuad.yMax
            and touchPosition.x > self.hitQuad.xMin and touchPosition.y > self.hitQuad.yMin ) then
        self.lockHeading = true;
    end

    self.isUp = false;
end

function HeadingClampLayer:touchUp(touchPosition)
	self.isUp = true;
	self:resetInput(touchPosition);
end

function HeadingClampLayer:resetInput(touchPosition)

    self.lastYClampFinal = self.yClampFinal;
    self.yClampStart = nil;
    self.lockHeading = false;
    self.isDown = false;
end

function HeadingClampLayer:_getHeadingClamp(yaw)
	if(not self.yClampStart) then
	    self.yClampStart = yaw - self.lastYClampFinal;
	end
	
	self.yClampFinal = yaw - self.yClampStart
end