


InputLayer = createClass({
	touchAnchor = nil,
	touchStartTime 		= 0.0;
    touchTime 			= 0.0;
    rotY = 15;
    rotX = 0.0;
    rotationScaleX = 0.14;
    currentRotationScaleX = 0.14;
    rotationScaleY = 0.13;
    quatY = Quaternion.new();
    quatX =  Quaternion.new();
    scrollQuat =  Quaternion.new();
    constrainedTouch	= true;
    isUp				= true;
    isDown				= false;
},deepcopy(CameraLayer));

InputLayer.name	= "gyro";

function InputLayer:onStart()
	if (Platform.win32 or Platform.mac) then
        self.constrainedTouch = false;
    end
    
    self:enableTouchControls(true)
end

--- enableTouchControls
-- if true - start listening for touch events
-- else stop listening to touch events
function InputLayer:enableTouchControls(enabled)
    self.touchControls = enabled;

    if (self.touchControls) then
        Events.listen(self, "touchDown");
        Events.listen(self, "touchUp");
        Events.listen(self, "drag");
    else
        Events.stopListening(self, "touchDown");
        Events.stopListening(self, "touchUp");
        Events.stopListening(self, "drag");
    end
end


--- touchDown
-- event listener - receives the touch down position
function InputLayer:touchDown(touchPosition)
    if(self.isDown) then
        Log.message("Touch: UP NOT RECVD");
        self.isUp = true;
        self:resetInput(touchPosition);
    end

    self.isDown = true;

    self.touchAnchor = touchPosition;
    self.touchStartTime = os.time();
    
    self.isUp = false;
end

--- resetInput
-- at the end of an input session (on 'touchUp') input controls a reset
-- and readied for the next input event
function InputLayer:resetInput(touchPosition)
    if (self.touchAnchor == nil) then return end

    self.touchTime = os.time() - self.touchStartTime;

    local toAnchor = (self.touchAnchor - touchPosition);

    self.dragPosition = nil;
    self.isDown = false;
end

--- touchUP
-- an event listener - receives the touchUp position
function InputLayer:touchUp(touchPosition)
    self.isUp = true;
    self:resetInput(touchPosition);
end

--- drag
-- an event listener - receives the the change in touch position
function InputLayer:drag(touchDelta)

    if(not self.dragPosition) then
        self.dragPosition = Vector(touchDelta.x, touchDelta.y)
    end

    local pos = Vector(touchDelta.x, touchDelta.y);

    self.touchDelta = Vector(0.5*(pos.x - self.dragPosition.x), 0.5*(pos.y - self.dragPosition.y), 0.0, 0.0);

    self.dragPosition = pos;

    local rotation = InputManager:getSensorRotation():copy()
    local rollAngle, rollQuat 	= rotation:getRoll()
    self.touchDelta = self.touchDelta:rotateBy(rollQuat:inverse())
    
    --update rotation
    self.rotX = self.rotX - self.touchDelta.x;
    self.quatY = Quaternion.axisAngle(Vector.up, -self.rotX * self.rotationScaleY)
    
    if(Platform.win32 or Platform.mac) then
    	local yawAngle, yawQ 	= rotation:getYaw()
    	local right 			= Vector.rotateBy(Vector.right, yawQ)
    	self.rotY 				= self.rotY + self.touchDelta.y;
	    self.quatX 				= Quaternion.axisAngle(right, self.rotY * self.currentRotationScaleX)
    end
end
