
OrientationRemapper = 
{
	-- config
	thresholdAngle	= math.rad(80.0),	-- the angle when the remapper kicks in
	startAngle		= math.rad(60.0),	-- the angle the viewer starts at

	-- state
	isCorrecting	= false,
}

-- Estimate a good rotation axis for correction.  When the camera is 
-- looking down, the forward axis doesn't work because forward and 
-- down are near coincident.  We calculate another axis based on 
-- oriented forward and up which gives us a good axis in this case.  
-- The final axis is a blend between them based on the ratio between 
-- down and forward.
function OrientationRemapper:getCorrectionAxis( t, down, forward, up )
	local forwardAxis 	= Vector.cross( down, forward )
	local downAxis 		= Vector.cross( forward, up )

	forwardAxis:normalize()
	downAxis:normalize()

	return Vector.lerp( t, downAxis, forwardAxis ):normalized()
end

-- The correction angle is the difference between where we are 
-- pointing, and where we want to be.
function OrientationRemapper:getCorrectionAngle( t, tiltAngle )
	local desiredAngle = math.lerp( self.startAngle, self.thresholdAngle, t )
	return desiredAngle - tiltAngle
end

function InverseEaseOut(x, a, b)
	return a * (x + math.pow(2.0, b*x-b))
end

-- Skew the orientation when looking down to allow the viewer to see 
-- the interesting content.
function OrientationRemapper:remapOrientation( orientation )
	local down = Vector.new( 0.0, -1.0, 0.0, 0.0 )

	-- forward direction at current orientation
	local forward = Vector.rotateBy( Vector.forward, orientation )
	forward:normalize()

	-- up direction at current orientation
	local up = Vector.rotateBy( Vector.up, orientation )
	up:normalize()

	-- angle between down and forward
	local tiltAngle = math.acos( Vector.dot( down, forward ) )

	--local upsideDown = Vector.dot( up, down ) >= 0.0

	if tiltAngle < self.thresholdAngle then
		self.isCorrecting = true

		-- the blend between straight down and the thresholdAngle
		local t = tiltAngle / self.thresholdAngle

		-- ease the blend factor to smooth transition back to thresholdAngle
		t = InverseEaseOut( t, 0.5, 6.0 )

		-- calculate correction axis, angle
		local angle = self:getCorrectionAngle( t, tiltAngle )
		local axis  = self:getCorrectionAxis( t, down, forward, up )

		-- create the correction quaternion
		local correction = Quaternion.new()
		correction:setAxisAngle( axis, angle )

		-- apply correction
		return correction * orientation, self.isCorrecting
	end

	self.isCorrecting = false	
	return orientation, self.isCorrecting
end
