

--[[
    A component which just puts a colored sphere on the given bone of an actor
--]]

local Logger = TaggedLog("DebugDot", true)

local triggerLabelCounter = 1

mathSqrt = math.sqrt

DebugDot = {
    sphereActor = nil,
    targetBone  = nil,

    radius      = 100.0,
    dotColor    = Color(0.3, 0.1, 0.75, 0.4),
    
    label       = "",    
    boneName    = "",
    
    deleted     = false,
}

DebugDot.new = function(self, data)
    local o = data or {}
    setmetatable(o, self)
    self.__index = self  
    return o
end

DebugDot.onStart = function(self)
	if(self.label == nil) then
		if (self.object ~= nil) then
			self.label = self.object.name
		end
	end

	local bonePath     = self.object:findBonePath(self.boneName)
	if (bonePath and bonePath ~= "") then
        self.targetBone    = self.object:getBone(bonePath)
    end
end

DebugDot.onUpdate = function(self)
--    local posT = self.object[self.boneName]:getWorld()
    if (not self.targetBone) then return end

    local posT = self.targetBone:getWorld()    

    if (not self.sphereActor and not self.deleted) then
        self:createDebugSphere()
    end
    if (self.sphereActor) then
        self.sphereActor:setVisibility(true)

        local transform = self.sphereActor:getLocal()
        transform.position = posT.position;
        transform.scale = Vector(1.0, 1.0, 1.0) * self.radius
        self.sphereActor:setLocal( transform )

        --Global.AppDebugText3 = self.label..string.format(" : %.3f, %.3f, %.3f ", transform.position.x, transform.position.y, transform.position.z)
        --Log.message(Global.AppDebugText3)
    end
	
end

DebugDot.onDestroy = function(self)
    destroySafely(self.sphereActor)
    self.sphereActor = nil
    self.deleted = true
end

DebugDot.createDebugSphere = function(self)
    SceneData.actors["DebugDot"..self.label] = shallowcopy(SceneData.actors.DebugSphere)
    self.sphereActor = ActorManager:getActor("DebugDot"..self.label, true)
    self.sphereActor:setVisibility(true)
    
    if self.sphereActor.materials then
        for name, material in pairs(self.sphereActor.materials) do
            material:setParameters({ u_Color = self.dotColor})
        end
    end
end

