
local Logger = TaggedLog("DogBlender", true)

DogBlender = {
    currentAlpha = 1,
}

function DogBlender:new(params)
    local o = shallowcopy(params) or {}

    Logger:message("table = " .. table.tostring(o))
    setmetatable(o, self)
    self.__index = self

    return o
end

-- Checks if the segment name provided as a param has behaviors defined for it
--   and applies them and calls blend if needed.
function DogBlender:applySegmentBlend(currentSegment)
    self.currentAlpha = 0
end


-- Function used locally to apply the blend 
-- (direction (1)-in, (-1)-out, (0)-none)
function DogBlender:blend()
    if (self.delay > 0) then        
        self.delay = self.delay - Time.dt()
    else
        self.currentAlpha = self.currentAlpha + Time.dt()
        local amount = math.clamp01(self.currentAlpha)
        local matColor = Vector.new(amount, amount, amount, amount)


        -- If the object is a flipbook (flipbook flag has to be set explicitly)
        for name, material in pairs(self.object.materials) do
            if(name == self.materialName) then
                self.material:setParameters({ u_Alpha = amount})
            end
        end                
    end
end

-- Applies blend behavior on creation of the parent actor.
function DogBlender:onStart()

    Events.listen(self, "segmentPlaying")

    self.actor = ActorManager:getActor(self.object.name)

    if(self.materialName) then
        self.material = self.object:findMaterial(self.materialName)
    end


    if(self.segmentData ~= nil) then
        local currentSegment = SegmentManager:currentSegmentName()
        self:applySegmentBlend(currentSegment)
    end
end

function DogBlender:onUpdate()
    
    if (self.currentAlpha < 1) then
        self:blend()
    end
end

-- Process segment transitions
function DogBlender:segmentPlaying(segmentName)
    if(self.blendInSegment == segmentName) then
        self:applySegmentBlend(segmentName)
    end
end


function DogBlender:onDestroy()
    Events.stopListening(self, "segmentPlaying")
end