
DogTest = {
	lookAtDetectorOuter = nil,
	lookAtDetectorInner = nil,
	state = "runstart",
	startPos = Vector.new(),
	yOffsetAngle = 0,
	yOffsetQuat = Quaternion.new(),
	offScreenTime = 0,
	doFlip = false;
	doPosUpdate = false,
	states = {
		runstart = "runstart",
		runloop = "runloop",
		sit = "sit",
	},
	doTrack = false,
	lastTrackPosition,
}

DogTest.new = function(self)
    local o = {};
    setmetatable(o, self);
    self.__index = self;
    return o;
end

function DogTest:onStart()
	local outerTr = Global.scene:createTransform("outerTr");
	local innerTr = Global.scene:createTransform("innerTr");
	
	--self.object:addAttachment("DogRun|planeLocator|mainMover|RootNode", outerTr);
	--self.object:addAttachment("DogRun|planeLocator|mainMover|RootNode", innerTr);
	
	self.lookAtDetectorOuter = self.object:addComponent("LookAtDetectorControllerOuter", AnimatedLookAtDetector:new({ colliderRadius = 2.5 , basePosition = nil, label="outer" }, "DogRun|planeLocator|mainMover|RootNode"));
	self.lookAtDetectorOuter.lookAtCallback = self;
	
	self.lookAtDetectorInner = self.object:addComponent("LookAtDetectorControllerInner", AnimatedLookAtDetector:new({ colliderRadius = 1.0 , basePosition = nil, label="inner" }, "DogRun|planeLocator|mainMover|RootNode"));
	self.lookAtDetectorInner.lookAtCallback = self;
	
	self.object:setVisibility(true);
	local localT = self.object:getLocal();
	self.startPos = self.object.targetBone:getWorld().position;
	self.startPos.y = 0;
	localT.scale.x = 2;
	localT.scale.y = 2;
	self.object:setLocal(localT);
	self:runStart();
end

function DogTest:updatePosition()
	local newPos = self.object.targetBone:getWorld().position;
	newPos.y = 0;
	self.yOffsetAngle = math.deg(Vector.angle(self.startPos, newPos));
	self.startPos = newPos;
	local localT = self.object:getLocal();
	localT.rotation = localT.rotation*Quaternion.axisAngle(Vector.up, math.sign(localT.scale.x)*self.yOffsetAngle);
	self.object:setLocal(localT);
end

local yawDiff = 0
function DogTest:updateCameraTrackOffset()

	local newPos = self.object.targetBone:getWorld().position;
	local cameraPos = Global.scene.cameraRig:getWorld().position;
	
	local lastToMe 	= self.lastTrackPosition - cameraPos;
	local toMe 		= newPos - cameraPos;
	
	local angleBetween = math.deg(Vector.angle(lastToMe, toMe));
	yawDiff = angleBetween;
	if(angleBetween ~= 0) then
		Global.scene.mainCamera.controller:setCruiseOffset(Quaternion.axisAngle(Vector.up, math.sign(self.object:getLocal().scale.x)*angleBetween));
	end
end

function DogTest:onUpdate()
	local textStyle = { color = Color.new(1.0, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = 1.0 }
    Renderer.drawText(10, 130, textStyle, string.format("looking at inner %s", tostring(self.lookAtDetectorInner.lookedAt)))
    Renderer.drawText(10, 160, textStyle, string.format("looking at outer %s", tostring(self.lookAtDetectorOuter.lookedAt)))
    Renderer.drawText(10, 190, textStyle, string.format("state %s", self.state))
    Renderer.drawText(10, 210, textStyle, string.format("yaw dif %.3f", yawDiff))
    
    if(self.doFlip) then
	    -- do flip on random chance
	    local value = math.random(0, 1)
		if(value > 0.5) then
			self.doFlip = false;
			local localT = self.object:getLocal();
			localT.scale.x = -localT.scale.x;
			self.object:setLocal(localT);
		end
    end
    
    if(self.doPosUpdate) then
    	self.doPosUpdate  = false;
    	self:updatePosition();
    end
    
    if(self.state == self.states.runstart) then
    	local duration = self.object.animDriver.playing:getDuration();
    	local time = self.object.animDriver.playing:getTime();
    	if(self.object.animDriver.playing:isDone() or time/duration > 0.985) then
    		self.doPosUpdate  = true;
    		self:runLoop();
    	end
    elseif(self.state == self.states.runloop) then
    	local dt = Time:dt();
    	
    	-- add up off screen time
    	if(not self.lookAtDetectorOuter.lookedAt) then
    		self.offScreenTime = self.offScreenTime + dt;
    	end
    	
    	if(self.offScreenTime > 1.0) then
    		-- off screen for too long and the dog stops
    		self:sit();
    	else
    		-- otherwise the dog runs
    		if(self.object.animDriver:isFinished(true)) then
    			self.object.animDriver:play(0);
    			self.doPosUpdate = true;
    		end
    		
    		--[[
	    	local localT = self.object:getLocal();
	    	localT.rotation = localT.rotation*Quaternion.axisAngle(Vector.up, math.sign(localT.scale.x)*10*dt);
	    	-- save current yaw offset
	    	self.yOffsetAngle, self.yOffsetQuat = localT.rotation:getYaw();
	    	self.object:setLocal(localT);
	    	]]
    	end
    	
    elseif(self.state == self.states.sit) then
    	self.offScreenTime = self.offScreenTime + Time:dt();
    	if(self.lookAtDetectorOuter.lookedAt and self.offScreenTime > 1.0) then
    		self.offScreenTime = 0;
    		self:runStart();
    end
    end
    
    if(self.doTrack) then
    	self:updateCameraTrackOffset();
    end
end

function DogTest:runStart()
	self.state = self.states.runstart;
    self.object.animDriver:init("runStart", self.object.animation, false, self.object.flipbook);
    self.object.animDriver:play();
end

function DogTest:runLoop()
	self.offScreenTime = 0;
	self.state = self.states.runloop;
	self.object.animDriver:init("runLoop", self.object.animation, false, self.object.flipbook);
    self.object.animDriver:play();
end

function DogTest:sit()
	self.state = self.states.sit;
	self.object.animDriver:init("sit", self.object.animation, false, self.object.flipbook);
    self.object.animDriver:play();
    
    -- do flip on random chance
    self.doFlip = true;
	
end

function DogTest:lookedAt(lookAtDetector, didLookAt)

	if(lookAtDetector.label == "outer") then
		if(didLookAt)then
			Log.message("Started looking At")
			self.offScreenTime = 0;
		else
			Log.message("Stopped looking at")
		end
	elseif(lookAtDetector.label == "inner") then
		if(didLookAt)then
			Log.message("Tracking")
			self.lastTrackPosition = self.object.targetBone:getWorld().position;
			self.doTrack = true;
		else
			Log.message("Stopped Tracking")
			Global.scene.mainCamera.controller:releaseCruiseControl();
			self.doTrack = false;
		end
	end
end