

FadeByScale = {}
FadeByScale.__index = FadeByScale

--cache local copies of math functions.
local saturate  = math.clamp01
local clamp     = math.clamp
local inverseLerp = math.inverseLerp

FadeByScale.new = function(scaleMin, scaleMax, materialName, boneName)
    local self = {}
    
    self.scaleMin = scaleMin
    self.scaleMax = scaleMax
    self.trackingBone = boneName
    self.materialName = materialName
        
    return setmetatable(self, FadeByScale)
end

FadeByScale.onStart = function(self)
    if (self.material == nil) then
        self.material = self.object:findMaterial(self.materialName)
    end
end

FadeByScale.onDestroy = function(self)
end

FadeByScale.onUpdate = function(self)
    local currentTransform = self.object:getBoneLocal(self.trackingBone)
    local currentScale = currentTransform.scale.x

	local amount = 0
	local from = self.scaleMin
	local to   = self.scaleMax
	if (from ~= to) then
		local value = clamp(currentScale, from, to)
		amount = saturate( (value - from) / (to - from) )
	end

    if (amount > 0.02) then
        self.object:setVisibility(true)
    end

	local c = 1.0 - amount
    self.material:setParameters({ u_Color = Color(c, c, c, 1) })
end
