
local Logger = TaggedLog("FlipbookDriver", true)

FBWrapMode = {
	-- wrap modes
	loop = 1,
	clamp = 2,
	countedLoop = 3
}

FlipbookDriver = 
{
	frame = 0,
	lastFrame = -1,
	frameOffset = 0,
	frameStartOffset = nil,
	frameRate = 60,
	looping = false,
	animName = nil,
	animation = nil,
	mat = nil,
	material = nil,
	lastTexture = nil,
	bindings = nil,--{"u_FlipbookTex",},
	numBindings = 1,
	uri = nil,
	nativeSampler = nil,
	nativeSetLooping = false,
	lastExposureFrame = -1,
	exposureChangeCallback = nil,
	layerOffset = 0,
	exposureCount = nil,
	
	luaDriven = false,
	
	frameEnd = nil,
	frameStart = nil,
	finished = false,
	frameTranslate = 0,
	elapsed = 0,
	
	loopCountGoal = -1,
	loopCount = 0,
	
	samplerMap = nil,
	
	wrapMode = FBWrapMode.clamp,

	useNative = flipbookdriver and flipbookdriver.onUpdateNative,
}

SamplerMap = {}

FlipbookDriver.new = function(self, animName, data)
    local o = shallowcopy(data);
    setmetatable(o, self);
    self.__index = self;
    o.animName = animName;
	o.sampler = FlipbookSampler.create()
	o.sampler:setFlipbook(o.uri)
	o.nativeSampler = o.sampler

    return o;
end

FlipbookDriver.writeUri = function(self)

	FBDriverPath = "../raw/episode3/scripts/scenedata/persistent/"
	if(not Platform.win32 and not Platform.mac) then FBDriverPath = "episode3/scripts/scenedata/persistent/" end
	
	-- method to read in saved actor tracking settings
	local function load()
		FBDrivers = require("episode3/scripts/scenedata/persistent/fbdrivers")
	end
	
	-- call the load method safely
	pcall(load)
	
	-- Default setup of actor tracking table
	FBDrivers = FBDrivers or {}
		
	FBDrivers[self.uri] = FBDrivers[self.uri] or {uri = self.uri, loaded = self.samplerMap.loaded or false, 
														destroyed = self.samplerMap.destroyed or false,
														loadCount = self.samplerMap.loadCount or 1}
	
	FBDrivers[self.uri].loaded 		= self.samplerMap.loaded or false
	FBDrivers[self.uri].destroyed 	= self.samplerMap.destroyed or false
	FBDrivers[self.uri].loadCount 	= self.samplerMap.loadCount or 1
	
	-- write the table to file
    local fbDriversTable = "return "..Serpent.dumppretty(FBDrivers);
	local file,err = io.open( FBDriverPath.."fbdrivers.lua", "w" )
	if( file ~= nil ) then
		file:write(fbDriversTable)
		file:close()
	end

end

FlipbookDriver.onDestroy = function(self)
	self:stop()
	if(self.nativeSampler) then
		self.sampler:destroy()
		self.sampler = nil
		self.nativeSampler = nil
	end
end

function FlipbookDriver:init(bindings)
	self.bindings = bindings or self.bindings;
	self.animation = self.object.animation[self.animName];
	self.lastTexture = nil;
	self.material = self.object:findMaterial(self.mat)
	self.onUpdate = self.stopUpdate;
	self.numBindings = #self.bindings;
	self.frameStartOffset = self.frameStartOffset or 0

	if self.nativeSampler then
		self.nativeSampler:setMaterial( self.material, self.bindings )
	end
	
	if( (self.nativeSetLooping or self.luaDriven) and self.nativeSampler and self.nativeSampler.setLooping ~= nil) then
		self.nativeSampler:setLooping(true)
	end	

	if(self.frameStart and self.frameEnd and self.exposureCount) then
		self.minRange = self.frameStart;
		self.maxRange = self.frameEnd - (self.wrapMode == FBWrapMode.loop and 1 or 0);
		
--		DrawText:color(0,1,0):addText(self.uri.."_ranges", "%s ranges = %d, %d", self.uri, self.minRange, self.maxRange)
--		self.nativeSampler:setFrameRange(self.minRange, self.maxRange)
	end
end

function FlipbookDriver:play()
--	Log.message("STARTING FB "..self.mat)
	
	local animFrame = self:getNextFrame()
	local frame = math.max(0, animFrame + self.frameStartOffset + self.frameOffset);
	self.nativeSampler:startPlayingFrom(frame)
	
	self.onUpdate = self.playUpdate;
	self.finished = false;
    -- Make sure the first texture is set as the
    -- onUpdate() may have already been called
    -- for this frame
    self:onUpdate()
end

function FlipbookDriver:stop()
--	Log.message("STOPPED FB: "..self.mat)
	self.onUpdate = self.stopUpdate;
	self.finished = true;
	self.elapsed = 0
	self.loopCount = 0
end

function FlipbookDriver:onStart()
end

-- update used when animation is stopped/paused
function FlipbookDriver:stopUpdate()
end

function FlipbookDriver:getNextFrame()
	local animFrame = 0
	
	-- lua will keep track of time and calculate the next frame
	if(self.luaDriven) then
		
		self.elapsed = self.elapsed + Time.dt()
		animFrame = math.max(0, self.frameStart + math.floor(self.elapsed * self.frameRate) + self.frameTranslate );
		
		if(animFrame >= self.frameEnd) then
		
			
			if(self.wrapMode == FBWrapMode.clamp) then -- wrapMode == CLAMP
			
				animFrame = self.frameEnd
				self.finished = true
				
			elseif(self.wrapMode == FBWrapMode.countedLoop) then -- wrapMode == COUNTED LOOP
				
				-- add completed loop
				self.loopCount = self.loopCount + 1
				
				if(self.loopCount < self.loopCountGoal)then
					self.elapsed 	= 0
					animFrame 		= self.frameStart
				else
					animFrame 		= self.frameEnd
					self.finished 	= true
				end
				
			else -- wrapMode == LOOP
			
				self.elapsed = 0
				animFrame = self.frameStart
				
			end
		end
		
	else
		-- frame according to FBX animation time
		animFrame = math.floor(self.animation:getTime() * self.frameRate);
	end
	
	-- verify min and max ranges
--	if(self.minRange and self.maxRange) then
--		local exposure = math.floor(animFrame/self.exposureCount)
--		
--		if(exposure < self.minRange or exposure > self.maxRange) then
--			DrawText:color(1,0,0):addText(self.uri.."_range", "%s ranges = %d, %d EXCEEDED", self.uri, self.minRange, self.maxRange)
--		end
--		
--		DrawText:color(0,1,1):addText(self.uri.."_range", "%s ranges = %d, %d sample at %d", self.uri, self.minRange, self.maxRange, exposure)
--	end
	
	return animFrame
end

function FlipbookDriver:playUpdate()
	flipbookdriver.onUpdateNative(self);
end

FlipbookDriver.isFinished = function(self)
	return self.finished
end
