
IntroEnvControl  = {}
IntroEnvControl.__index = IntroEnvControl

function IntroEnvControl.new(trackingActor)
    local this = {
        trackingActor = trackingActor,
		matName = "Mat_Dome",
		material = nil,
		fadeValue = 1,
		scrollValue = 1,
 		
		dying = false,
		deathTimer = 1,

		manualFade = false,
		manualFadeTimer = 1,
   }
    return setmetatable(this, IntroEnvControl)
end

function IntroEnvControl:onStart()
	self.material = self.object:findMaterial(self.matName)
end

function IntroEnvControl:onUpdate()
	if (not self.dying) then
		if (not self.manualFade) then
			local accelStatus = Sensor.getStatus(ST_ACCELEROMETER)
			if (accelStatus ~= nil) then
				local accel = accelStatus.acceleration
				local angleFromZenith = math.deg(Vector.angle(accel:normalized(), Vector(0.0, 0.0, -1.0)))
				local normalizedZenith = angleFromZenith / 180
				local fadeGoal = math.clamp01(math.inverseLerp(0.35, 0.45, normalizedZenith))  
				local scrollGoal = math.clamp01(math.inverseLerp(0.0, 0.5, normalizedZenith))  

				self.fadeValue = math.lerp(self.fadeValue, fadeGoal, Time.dt() * 3.0)
				self.scrollValue = math.lerp(self.scrollValue, scrollGoal * 1.5, Time.dt() * 2.0)

				self.material:setParameters({ u_Alpha =  1 - self.fadeValue, u_TexOffset = Vector.new(1, 1, 0, -self.scrollValue)})
				domeDirection = Quaternion.lookAt(accel) * Quaternion.fromEulerAngles(Vector(-90,0,0)) 
			end
		else 
			self.manualFadeTimer = self.manualFadeTimer - Time.dt() 
			self.material:setParameters({ u_Alpha = math.clamp01(1 - self.manualFadeTimer)})
			-- if (self.manualFadeTimer < 0) then
				-- self.object:destroy()
				-- return
			-- end		
		end
	else 
		self.deathTimer = self.deathTimer - Time.dt() 
		self.material:setParameters({ u_Alpha =  math.clamp01(self.deathTimer)})
		if (self.deathTimer < 0) then
			self.object:destroy()
			return
		end
	end

end

function IntroEnvControl:startManualFade()
	self.manualFade = true;
end


function IntroEnvControl:getFadeValue()
	return 1.0 - self.fadeValue
end

function IntroEnvControl:killMe()
	self.dying = true
end

