

LockToCamera = {

    lockTimer			    = 0.000,
    lockDuration		    = 0.001,
    
    inDelay                 = 0.000,
    inDuration              = 0.001,
    outDelay                = 0.000,
    outDuration             = 0.001,
    locking                 = true,
    lockToTrackingBone      = true,
    stabilizeFactor         = 0.4,

	blendedWorldRot		    = Quaternion(),
	worldGoalRot		    = Quaternion(),
    modifiedActorRot        = Quaternion(),
	initialYaw			    = 0.0,
    previousLerp            = 0.0,
    previousRotation        = Quaternion(),
    currentRotation         = Quaternion(),
    previousRotationDelta   = Quaternion(),
    currentRotationDelta    = Quaternion(),

    initDone                = false,
	
}

LockToCamera.new = function(self, data)    
    local o = data or {}
    setmetatable(o, self)
    self.__index = self    
    return o
end

LockToCamera.onStart = function(self)
	self.worldGoalRot	        = Global.segmentOffset
	self.initialYawValue        = Global.scene.mainCamera:getWorld().rotation:getYawAngle()
    self.lockTimer              = self.inDelay + self.inDuration
    self.lockDuration           = self.inDuration + 0.001
    self.initDone               = true
end



LockToCamera.stabilizeActor = function(self, updatedTransform) 
    self.previousRotation       = self.currentRotation
    self.previousRotationDelta  = self.currentRotationDelta
    self.currentRotation        = updatedTransform.rotation
    self.currentRotationDelta   = self.currentRotation * Quaternion.inverse(self.previousRotation)
    local alignment             = self.stabilizeFactor
    local output                = Quaternion.slerp(math.clamp01(alignment), self.currentRotationDelta * self.modifiedActorRot, self.modifiedActorRot) 
    return output
end

LockToCamera.updateActor = function(self, updatedTransform, actor)
    
    if (not self.initDone) then
        return nil
    end

    local rotationChange    = Vector.angle(Vector.rotateBy(Vector.forward,updatedTransform.rotation), Vector.rotateBy(Vector.forward, self.previousRotation))
    self.previousRotation   = updatedTransform.rotation

    local actorWorldRot     = self.object:getWorld().rotation
    local actorWorldForward = Vector.rotateBy(Vector.forward, actorWorldRot)
    local actorWorldUp      = Vector.rotateBy(Vector.up, actorWorldRot)
    
    self.modifiedActorRot   = Quaternion.lookAtWithUp(actorWorldForward, actorWorldUp)

    self.lockTimer = self.lockTimer - Time.dt()
    
    if (self.lockTimer < 0) then
        if (self.locking) then
            self.locking = false
            self.lockTimer =  self.outDelay + self.outDuration
            self.lockDuration = self.outDuration + 0.001
        end
    end
    
    
    local finalSRT		        = self.object:getLocal()
    local cameraRotDir          = Vector.rotateBy(Vector.forward, updatedTransform.rotation)
    local cameraRotUp           = Vector.rotateBy(Vector.up, updatedTransform.rotation)    
    local modActorForward       = Vector.rotateBy(Vector.forward, self.modifiedActorRot)
    local angleDiff             = Vector.angle(cameraRotDir, modActorForward)    
    local modifiedCameraRot     = Quaternion.lookAtWithUp(cameraRotDir, Vector.up)
    local boneRotation          = nil 
    local boneRotationforward
    local boneRotationUp
    local bonePosition


    self.modifiedActorRot       = ActorMover.stabilizeActor(self,updatedTransform)

    if (actor.namedBones.targetBone ~= nil) then
        local targetBone = ActorManager:getTargetBone(actor.name)

        if (targetBone) then
            bonePosition = targetBone:getWorld().position
            bonePosition = Vector.normalized(bonePosition)
            boneRotation = Quaternion.lookAtWithUp(bonePosition,Vector.up)
        end

        if (boneRotation and self.lockToTrackingBone) then
            modifiedCameraRot = modifiedCameraRot*Quaternion.inverse(boneRotation)*self.modifiedActorRot
        end
    end

    local lockAmount = math.clamp01(self.lockTimer / self.lockDuration)
    
    if (self.locking) then
        lockAmount = 1 - lockAmount
    end

    finalSRT.rotation =  Quaternion.slerp(lockAmount, self.modifiedActorRot, modifiedCameraRot)
       
    self.object:setLocal(finalSRT)

    -- glow needs to use the actors updated position
	return finalSRT
end


