

TargetData = {
		nameOfPosTarget = nil; -- the name of the object being lit
		nameOfMatTarget = nil; -- then name of the object containing the shared material that needs to be updated/lit
		currentDistanceSqrd = math.floatMax;
		
		lightTargetPos = nil; -- reference to transform of the target (the object being lit)
		lightTargetMat = nil; -- reference to shared material of the object being lit
}

local hashIndex = 1;

OmniLightProbe = {
	-- triggers refresh of links (editor only)
	refreshLinks = false;
	lightTargetData = {}; 		-- list of targets
	lightTargetLookUpMap = {}; 	-- hash to target mapping to quickly locate target that this probe is lighting
	cutOffDist = 30.0;
	cutOffDistSqrd = 0.0;
	lightProbeColor = Vector.one;
	
	currentTarget = nil; -- local cache of current target
	radiusSqrd = 0.0;
}

OmniLightProbe.new = function(self, data)
	local o = data or nil
	setmetatable(o, self);
	self.__index = self;
	return o;
end

function OmniLightProbe:onStart()

	--Events.listen(self, "onSetVisibilityChanged")
	--Events.listen(self, "onActorDestroyed")

	self.cutOffDistSqrd = self.cutOffDist*self.cutOffDist;
	
	self.radiusSqrd = self.scale*2.0;
	
	local localT = self.object:getLocal()
	localT.position = Vector(self.position.x, self.position.y, self.position.z)
	self.object:setLocal(localT)
	
	
	-- create quick look up map
	for i, td in ipairs(self.lightTargetData) do
		if(td ~= nil) then
			td.hashcode = td.targetActorName;
			td.hashname = ""..td.targetActorName.."_"..hashIndex;
			self.lightTargetLookUpMap[td.hashcode] = self.lightTargetData[i];
			hashIndex = hashIndex + 1;	
		end
		self:refreshActorReferences(td)
	end
end

function OmniLightProbe:refreshActorReferences(td)

	local targetdata = self.lightTargetLookUpMap[td.hashcode]
	
	targetdata.lightTargetPos = ActorManager:getTargetBone(targetdata.hashcode)
	
	targetdata.lightTargetMat = { 
		name = "material_"..targetdata.targetActorName,
		actorName = targetdata.targetActorName,
		materialList = targetdata.materials,
		setParameters = function(self, paramTable)
			local actor = ActorManager:getActor(self.actorName)
			if(actor ~= nil) then
				 -- check for specified materials to be lit otherwise light them all
				if(targetdata.materials ~= nil) then
					for _, matName in ipairs(self.materialList) do
						local mat = actor:findMaterial(matName)
						if(mat ~= nil) then
							mat:setParameters(paramTable)
						end
					end
				else
					for name, mat in pairs(actor.materials) do
						mat:setParameters(paramTable)
					end
				end
			end
		end,
	}
	
	return targetdata
end

function OmniLightProbe:onSetVisibilityChanged(name, isVisible)
	if(name == self.setName and not isVisible and not self.isDestroyed) then
		Log.message("Actor: "..name.." Destroying OmniLightProbe")
		self.object:destroy()
		self.isDestroyed = true
	end
end

function OmniLightProbe:onActorDestroyed(name, isVisible)
	if(name == self.setName and not self.isDestroyed) then
		Log.message("Actor: "..name.." Destroying OmniLightProbe")
		self.object:destroy()
		self.isDestroyed = true
		Events.stopListening(self, "onActorDestroyed")
	end
end

function OmniLightProbe:onUpdate()

	 self.currentTarget = nil;
	 
 	--[[local debugSphereName = "DebugSphere_"..self.name;
	local debugSphereActor = ActorManager:getActor(debugSphereName);
	local posActor = ActorManager:getActor(self.setName);
	
	if(posActor ~= nil) then
		local localT = debugSphereActor:getLocal();
		localT.position = posActor:getBone(self.bonePath):getWorld().position;
		
		debugSphereActor:setLocal(localT);
	end]]
	
	--get active target
	 for _, targetData in ipairs(self.lightTargetData) do
		local actor = ActorManager:getActor(targetData.targetActorName)
		-- do a check to make sure the actor still exists
	 	if(actor ~= nil) then
			if(self.active and targetData ~= nil)then
				-- update any material/bone references in case the actor has been destroyed and re-created since the last time we stored a reference
				targetData = self:refreshActorReferences(targetData)
				local setModel = ActorManager:getActor(self.setName)
				 if(targetData.lightTargetPos ~= nil and setModel ~= nil) then
				 	
				 	local parentPos = Vector.new();
				 	 if(self.bonePath ~= nil) then
					 	parentPos = setModel:getBone(self.bonePath):getWorld().position
					 end
					 
					 self.currentTarget = targetData.lightTargetPos;
					 local localT = self.object:getLocal()
					 
					 parentPos 			= parentPos + localT.position;
					 local targetPos 	= self.currentTarget:getWorld().position
					 
					 targetData.currentDistanceSqrd = Vector.lengthSq(parentPos - targetPos);
					 
					 --Log.message("Light model: "..targetData.targetActorName)
					 --Log.message(string.format("TARGET %s: <%.3f, %.3f, %.3f> ", targetData.targetActorName, targetPos.x, targetPos.y, targetPos.z))
					 --Log.message(string.format("Light %s: <%.3f, %.3f, %.3f> ", self.name, parentPos.x, parentPos.y, parentPos.z))
					
					 --Log.message(string.format("? Dist: %.3f < cutoff(%.3f)", targetData.currentDistanceSqrd, (self.cutOffDist * self.cutOffDist)))
					 -- if target is within lighting range add it to the blend manager
					 if(targetData.currentDistanceSqrd < self.cutOffDist * self.cutOffDist) then
					 	--Log.message("WAS LIT")
					 	
						OmniLightBlendManager:addProbe(targetData, self);
					 end
				 end
			 end
	 	end
	 end
	
end

function OmniLightProbe:getInvertedLightProbeColor()
	return Vector.one - self.lightProbeColor;
end

function OmniLightProbe:findTargetInThisProbe (targetData)
	--return self.lightTargetLookUpMap[targetData.hashcode];
	for i, light in ipairs(self.lightTargetData) do
		if(self.lightTargetData[i].targetActorName == targetData.targetActorName) then
			return self.lightTargetData[i]
		end
	end
	
	return nil
end
