
--[[
    Description: Applies the Actor's rotation to OpeningSphere

    Parameters:
    * smoothness - Smaller amounts equal more smoothness
]]

local Logger = TaggedLog("OpeningSphereController", true)

OpeningSphereController = {
    smoothness = 0.2,
    delayedAppearance = 4,
}

function OpeningSphereController:new(params)
    local o = shallowcopy(params) or {}

    Logger:message("table = " .. table.tostring(o))
    setmetatable(o, self)
    self.__index = self

    return o
end

-- Applies blend behavior on creation of the parent actor.
function OpeningSphereController:onStart()
    Events.listen(self, "segmentPlaying")
    self.openingSphere = ActorManager:getActor("OpeningSphere", true)  
    self.openingSphere:setVisibility(true)

end

function OpeningSphereController:onLateUpdate()

   if(self.openingSphere and Global.scene and Global.scene.mainCamera) then

        local rot = self.object:getWorld().rotation
        local openingSphereTransform = self.openingSphere:getWorld()
        if(self.smoothRot == nil) then
            self.smoothRot = rot 
        end
        -- To avoid fuzziness when making strong movements on device
        self.smoothRot = Quaternion.lerp(self.smoothness, self.smoothRot, rot)

        -- Apply actor's rotation to OpeningSphere
        openingSphereTransform.rotation = self.smoothRot
        self.openingSphere:setLocal(openingSphereTransform)
        
    end
end

-- Process segment transitions
function OpeningSphereController:segmentPlaying(segmentName)
    self.openingSphere = ActorManager:getActor("OpeningSphere", false)  
end

function OpeningSphereController:onDestroy()
    Events.stopListening(self, "segmentPlaying")
end