
--[[

    Description:
    Parameters to pass: 

]]

local Logger = TaggedLog("PondWaterBlender", true)
--cache non-local functions
local getDt = Time.dt
local saturate = math.clamp01

PondWaterBlender = {
    currentAlpha = 1,
    delay        = 0,
    
    fadeTime     = 1,
    lerpAlpha    = true,
}

function PondWaterBlender:new(params)
    local o = shallowcopy(params) or {}

    Logger:message("table = " .. table.tostring(o))
    setmetatable(o, self)
    self.__index = self

    return o
end


-- Function used locally to apply the blend 
-- (direction (1)-in, (-1)-out, (0)-none)
function PondWaterBlender:blend(direction)
    self.currentAlpha = self.currentAlpha + (getDt()/self.fadeTime) * direction
    local amount = saturate(self.currentAlpha)
    local matColor = Vector.new(amount, amount, amount, amount)

    if(self.in_out == "in" and amount == 1) then
        self.in_out = "out"
        self.currentAlpha = 1
        self.delay = 1
        self.fadeTime = 2
    end

    if(self.actor) then
    	local materials = self.actor.materials
        if (materials) then      
            for name, material in pairs(materials) do                    
                material:setParameters({ u_Alpha = amount})
            end                
        end
    end
end

-- Applies blend behavior on creation of the parent actor.
function PondWaterBlender:onStart()
    self.actor        = ActorManager:getActor(self.object.name)

    self.currentAlpha = 0
    self.in_out       = "in"
    
    self:blend(0)
end

function PondWaterBlender:onUpdate()
    
    local matColor = Vector.new(1,1,1,1)
    -- Process delay time if needed.    
    if (self.delay > 0) then        
        self.delay = self.delay - Time.dt()
    -- Blend out 
    elseif (self.currentAlpha > 0 and self.in_out == "out") then
        self:blend(-1)
    -- Blend in
    elseif (self.currentAlpha < 1 and self.in_out == "in") then
        self:blend(1)
    -- If blend mode is not defined, do not perform blending.
    end
end


function PondWaterBlender:onDestroy()
end