

SoftTrack = 
{
	lookAtDetectorTarget = nil,
	yOffsetAngle = 0,
	yOffsetQuat = Quaternion.new(),
	doPosUpdate = false,
	doTrack = false,
	lastTrackPosition = Vector(0,0,0,0),
	colliderRadius = 200,
	targetBonePath = nil,
	
	clampAngle 		= 15,
	rayToPlaneParams = nil,
	
	updateTargetBonePosition = nil,
	
	acquiredCruiseControl = false,
	
	-- final tracking offset
	offset = Quaternion.new(),
	
	finalAngle = 0,
	angleBetween = 0,
	angleList = {},
	sampleCount = 16,
	frame = -1,
	frameStep = 4,
	doSteppedUpdate = false,
	debug = false,
	
	trackingLayer = nil,
	lastTrackingStrength = nil,
	useNative = softtrack and softtrack.onStart,
}

function SoftTrack:new(data)
	local o = shallowcopy(data) or {}
    setmetatable(o, self)
    self.__index = self;
    return o
end

SoftTrackerUniqueIndex = 0;

function SoftTrack:onStart()

	self.trackingLayer = MainCamera.controller.trackingLayer
	Events.listen(self, "segmentPlaying");

	if (self.name == nil and self.object) then
		self.name = self.object.name
	end

	if (self.useNative) then
		softtrack.onStart(self)
	end
end

function SoftTrack:onUpdate(cameraTransform)

	if (self.object == nil) then
		return
	end

	local currTimeSecs = Time.seconds();
	if(self.lastUpdateSecs == nil) then
		self.lastUpdateSecs = currTimeSecs;
	end

	local dt = currTimeSecs - self.lastUpdateSecs
    
	local actorName = ActorManager.proxyActors[self.object.name]
	local actor = ActorManager:getActor(actorName);
    
	if(self.doTrack) then
		self:updateCameraTrackOffset(dt, cameraTransform);
	end
end

function SoftTrack:segmentPlaying(name)
	if (self.useNative) then
		softtrack.segmentPlayingNative(self)
	else
		if (self.object == nil) then
			return
		end
		self.lastTrackPosition 		= self.object:getWorld().position:copy()
		self.lastTrackPosition.y 	= 0;
	end
end

function SoftTrack:enableTracking(enable, startPos, doSteppedUpdate, frameStep)
	if(self.debug) then
		DrawText:color(0,1,1):addText("EnableTracking_"..self.object.name, "%s tracking enabled: %s", self.object.name, tostring(enable));
	end
	if(enable)then
		Log.message("Tracking "..self.object.name)
		self.lastTrackPosition 		= self.object:getWorld().position
		self.lastTrackPosition.y 	= 0;
		self.lastPos = nil;
		self.dir = nil;
		self.doTrack = true;
		
		self.angleBetween = 0
		self.finalAngle = 0
		self.angleList = {}
		self.frame = -1
		
		self.doSteppedUpdate = doSteppedUpdate;
		self.frameStep = frameStep
	else
		Log.message("Stopped Tracking "..self.object.name)
		self.doTrack = false;
		self.offset = Quaternion.new()
		self.acquiredCruiseControl = false
	end
end

function SoftTrack:updateCameraTrackOffset(dt, cameraTransform)

	local cameraLocal 	= Global.scene.mainCamera:getLocal()
	local newPos 		= self.object:getWorld().position
	local cameraPos 	= Global.scene.cameraRig:getWorld().position;
	
	local camLook 		= MainCamera.camForward:copy()
	local camRight 		= MainCamera.camRight:copy()
	
	newPos.y = 0;
	cameraPos.y = 0;
	
	local fromStart 	= self.lastTrackPosition - cameraPos;
	local toMe			= newPos - cameraPos
	
	local ninetyDegReset = math.abs(math.deg(Vector.angle(fromStart, toMe)))
	if(ninetyDegReset > 90 or self.lastTrackingStrength ~= self.trackingLayer.strength) then
		
		-- reset start position
		self.lastTrackPosition 		= self.object:getWorld().position
		self.lastTrackPosition.y 	= 0;
		
		-- update tracking offset
		self.trackingLayer:saveAdditiveOffset();
		
		-- reset start vector
		fromStart 	= self.lastTrackPosition - cameraPos;
	end
	
	self.lastTrackingStrength = MainCamera.controller.trackingLayer.strength
	
	local dir = -math.sign(Vector.dot(Vector.cross(fromStart, Vector.up), toMe))
	
	local angle = dir*math.deg(Vector.angle(fromStart, toMe))
--	local angle = (Quaternion.lookAt(fromStart):inverse()*Quaternion.lookAt(toMe)):getYawAngle()
	self.offset = Quaternion.axisAngle(Vector.up, angle);
	
	if(self.debug) then
		DrawText:color(0,1,1):addText("AngleOffset_"..self.object.name, "%s angle between: %.3f", self.object.name, angle);
	end
	
	self.lastPos = newPos;

end

function SoftTrack:onDestroy()
	if(self.planeParent) then
		self.planeParent:destroy()
	elseif(self.lookAtTr) then
		self.lookAtTr:destroy()
	end

	if (self.useNative) then
		softtrack.onDestroy(self)
	end
end
