
local Logger = TaggedLog("StarBlender", true)

local gAttractAccel		= 300.0 
local gAttractMaxSpeed	= 200.0
local gExclusionRadius	= 50.0
local gForceToAttractorScale = 5.0
local gForceUpScale			 = 0.001
local gForceOverallScale	 = 1.5



StarBlender = {
	_attractorPos	        = Vector(0,0,0),
	_targetPos		        = Vector(0,0,0),
	_startPos 		        = nil,
	attractAccel 	        = gAttractAccel,
	attractMaxSpeed         = gAttractMaxSpeed,
	exclusionRadius      	= gExclusionRadius,
	pCurPos			        = Vector(0,0,0),
	pCurVel			        = Vector(0,0,0),
	
	actorDist		        = 0.0,
	actorPos		        = Vector(0,0,0),
	actorRot		        = Quaternion.new(),
	actorFocusDriver		= nil,
	attractorFocusDriver	= nil,

	m2w				        = Matrix.new(),	
	m2wInv			        = Matrix.new(),

	invertY			        = false,
    
    launcherRig             = nil,
    launcherStarBone        = "target_Star|RootNode",
    launcherBlendTimer      = 7.5,
    launcherBlendDuration   = 5.0,
    
	attractorPos	        = Vector(0,0,0),

	attractorPos	        = Vector(0,0,0),
	targetPos		        = Vector(0,0,0),
	startPos		        = Vector(0,0,0),
	vecUp			        = Vector(0,1,0),

	targetReached	        = false,
    findViewerPos			= Vector(0,0,0),

    
    viewerIsLost            = false,
    findingTheViewer        = false,
    headToShowStart         = Vector(0,0,0),
    headToShowEnd           = Vector(0,0,0),
    headToShowTimer         = 0,
    headToShowDuration      = 1,
    
    stupidOffsetAnim        = nil,
    stupidOffsetBone        = "StupidOffset|RootNode",
    stupidOffsetScale       = 1,
	
	lockFocus				= false,
	focusNode				= nil,	
	minProx					= 250.0,
	proxRange 				= 200.0,
	
	evalTime		        = 0.0,
	fullTimeEst				= 9.0, --0.0,
	proxTimeEst				= 4.0,

	startOffset				= Vector(0,0,-1405),

	debugDraw		        = false,
	debugTrail		        = false,
	debugTargets	        = {},
	
	delay					= 2.0,
	
	-- Apparently we call updates BEFORE onStart?! WTF?
	hasStarted					= false,

	--testVector				= Vector(0,0,0),
	--debugCount				= 2,
}


StarBlender.new = function(self, data)
    local o = data or {};
    setmetatable(o, self);
    self.__index = self;
    
    return o;
end


StarBlender.onStart = function(self)
--StarBlender.setUp = function(self)
   	Logger:message( "  == Starting StarBlender == ")
   	
    local actor = ActorManager:getActor(self.targetActor, false)
    
    self.launcherRig = Global.scene.actors.LauncherRig
    self.stupidOffsetAnim = Global.scene.actors.StupidStarOffset;
            
   	if (actor) then
        local bonePath = actor:findBonePath(self.attractorBone)
		local targetTransform = actor:getBoneWorld(bonePath)
	    self._attractorPos	= targetTransform.position:copy()        
        
        bonePath = actor:findBonePath(self.targetBone)
		targetTransform = actor:getBoneWorld(bonePath)
		self._targetPos	= targetTransform.position:copy()		
    end
    
    if (self.object) then
        local bonePath = self.object:findBonePath(self.focusBone)
	    self.focusNode = self.object:getBone(bonePath)

	    if (self.object.components["StarFocusDriver"] or self.object.components["segcomp_StarFocusDriver"]) then
	    	if (self.object.components["StarFocusDriver"]) then
	    		self.actorFocusDriver = self.object.components["StarFocusDriver"]
	    	else
	    		self.actorFocusDriver = self.object.components["segcomp_StarFocusDriver"]
	    	end
	    end	
    end
   	
   	-- We assume a plane normal to z with the depth of the target position
   	-- This basically presumes were going to start with a heading pointed towards -z
   	-- (because we are forcing that to be true at the start)
   	self.actorDist			= self._targetPos:length()
   	local planarDistance	= self._targetPos.z
   	
    self._attractorPos:normalize()
--	self._attractorPos = self._attractorPos * Vector.dot(Vector.forward * (-1.0 * planarDistance), Vector.forward) / Vector.dot(self._attractorPos, Vector.forward)   	
--	self._attractorPos = self._attractorPos * Vector.dot(planarPoint, planarNormal) / Vector.dot(self._attractorPos, planarNormal) 
	self._attractorPos = self._attractorPos * Vector.dot(self._targetPos, self._attractorPos)
	
	local planarPoint  = self._attractorPos:copy()
	
	-- This camera value is not consistent with what we want in onStart()
	-- but is accurate some time later, this needs more debugging
	local cameraWorld   = Global.scene.mainCamera:getWorld()
	local camForward    = cameraWorld:forward():copy() * self.actorDist	
	--self.testVector 	= camForward:copy()
	
	--table.insert(self.debugTargets, { pos = camForward:copy(), size = 150.0} )
	
	local planarNormal = self._attractorPos:copy() * -1.0
	planarNormal:normalize()
	
	-- Position an actor at the attractor location if one is specified
   	if (self.attractorActor) then   	
	    actor = ActorManager:getActor(self.attractorActor, false)
	    if (actor) then
		    local localT = actor:getLocal()
			localT.position = self._attractorPos:copy()
		    actor:setLocal(localT)
		    
		    if (actor.components["StarFocusDriver"] or actor.components["segcomp_StarFocusDriver"]) then
		    	local component = nil
		    	if (actor.components["StarFocusDriver"]) then
		    		self.attractorFocusDriver = actor.components["StarFocusDriver"]
		    	else
		    		self.attractorFocusDriver = actor.components["segcomp_StarFocusDriver"]
		    	end	    	
		    end
		end
	end
   	
   	-- Store debug info for or initial positions	
   	self.invertY = false --(self._startPos.y > self._attractorPos.y)

	local tempUp = self.vecUp:copy()
	local vecToCam = planarNormal:copy()	
	local vecRight = self._targetPos - planarPoint
	vecRight:normalize()
	tempUp = Vector.cross(vecToCam, vecRight)
	tempUp:normalize()
	
	-- Basis vectors
	if (self.invertY) then
		-- Invert up vector to get a matrix that will give us the same up/right 
		-- directions whether we're above or below the star-to-center line
		tempUp = tempUp * -1.0;
	end

	-- Model-to-Worldspace matrix
	self.m2w:setBasisX(vecRight)
	self.m2w:setBasisY(tempUp)
	self.m2w:setBasisZ(vecToCam)
	self.m2w:setTranslation(self._attractorPos)	

	-- Inverse Model-to-Worldspace
	self.m2wInv = self.m2w:rigidInverse()
	
	self.attractorPos	= self.m2wInv:applyTransform(self._attractorPos)
	self.targetPos		= self.m2wInv:applyTransform(self._targetPos)

	local yPos			= -1500.0
	if (self.invertY) then
		yPos = yPos * -1
	end
	self.startPos		= self.attractorPos + Vector(-250.0, yPos, 0.0 )
	self._startPos		= self.m2w:applyTransform(self.startPos)
   	self.pCurPos = self._startPos:copy() 	
	
	table.insert(self.debugTargets, { pos = self._targetPos:copy(), size = 70.0} )
	table.insert(self.debugTargets, { pos = self._attractorPos:copy(), size = 70.0} )
	table.insert(self.debugTargets, { pos = self._startPos:copy(), size = 70.0} )

   	self.evalTime 		= 0.0
   	self.targetReached 	= false
   	
   	if (self.debugTrail) then
   		self:generateDebugTrail()
   	end
   	
   	SegmentManager:blockTransition(true)
   	
   	self.hasStarted = true
end

StarBlender.updateActor = function (self, cameraTransform)
--	if (self.delay <= 0.0 and self.hasStarted == false) then
--		self:setUp(cameraTransform)
--	else
--	   	SegmentManager:blockTransition(true)		
--		self.delay = self.delay - Time.dt()
--	end

	if (self.hasStarted == false) then return end
	
--	if  (self.debugCount > 0) then 
--		local camForward = cameraTransform:forward():copy() * self.actorDist
--		table.insert(self.debugTargets, { pos = camForward:copy(), size = 50.0} )
--		self.debugCount = self.debugCount - 1
--	end

   	--Logger:message( "  == updateActor == ")
    
	if (self.targetReached == false) then
		self:getStarVelocity(0.0) --Time.dt())
	end
	
	-- Set the global camera focus depth based on an animation
	if (self.focusNode) then
		local blendAmount	= 1.0
		-- The "targetReached" check here is potentially far too abrupt, but useful in the short term as a fall-through behavior
		if (self.lockFocus == false and self.targetReached == false) then
			local toCompanion = self._attractorPos - self.pCurPos
			local companionDist = toCompanion:length()

			blendAmount = 1 - math.easeInOutSine(math.clamp01( (companionDist - self.minProx) / self.proxRange ) )
			if (blendAmount >= 1.0) then
				self.lockFocus = true
			end
			
			--Global.AppDebugText1 = string.format("companionDist %.5f", companionDist) 
		end
	
		local animatedDist = math.abs(self.focusNode:getLocal().position.z)
		Global.focalDistance = math.lerp(animatedDist, self.actorDist, blendAmount)	
	end
	
	local normalizedPos = self.pCurPos:copy()
	normalizedPos:normalize()
	self.actorPos = normalizedPos * self.actorDist
	self.actorRot = Quaternion.lookAt(normalizedPos)
    
	local finalSRT = self.object:getLocal();

	finalSRT.position = self.testVector --self.actorPos
	finalSRT.rotation = self.actorRot

	local CameraT = Global.scene.mainCamera:getWorld();


    if (self.targetReached) then

        if (self.findingTheViewer) then
            
            local centerOfCamera = CameraT:forward() * 2000;
            
            self.stupidOffsetScale = math.lerp(self.stupidOffsetScale, 1, Time.dt() * 5);
            self.findViewerPos = Vector.lerp(Time.dt(), self.findViewerPos, centerOfCamera);
            if (Vector.distance(self.findViewerPos, centerOfCamera) < 100) then
                self.findingTheViewer = false;
                self.headToShowStart = self.findViewerPos;
                self.headToShowEnd = self._targetPos;
                self.headToShowTimer = Vector.distance(self.headToShowStart, self.headToShowEnd) / 200;
                self.headToShowDuration =  self.headToShowTimer;
            end
        else 
            self.headToShowTimer = self.headToShowTimer - Time.dt();
            if (self.headToShowTimer < 0) then
                if (self:isOffScreen()) then
                    self.findingTheViewer = true;
                    SegmentManager:blockTransition(true);
                else 
                    SegmentManager:blockTransition(false);
                end
            end    
                
            local amount = math.clamp01(self.headToShowTimer/self.headToShowDuration);
            self.stupidOffsetScale = math.lerp(0, 1, math.easeInOutSine(amount));
            self.findViewerPos = Vector.lerp(math.easeInOutSine(amount), self.headToShowEnd, self.headToShowStart);
            
        end
        
        local dir = (CameraT.position - self.findViewerPos);
        dir:normalize();
        local starRot = Quaternion.lookAtWithUp(CameraT:forward() * -1, Vector.up);

        local stupidOffset = self.stupidOffsetAnim:getBoneLocal(self.stupidOffsetBone);
        local offsetPos = stupidOffset.position;
        
        if (self.findViewerPos:length() < 0.1) then
            self.findViewerPos = finalSRT.position;
        end       
        
        finalSRT.position = self.findViewerPos + (offsetPos * self.stupidOffsetScale);
        finalSRT.rotation = starRot;
        
    else

        if (Global.scene.actors.LauncherRig ~= nil) then
            finalSRT.position = self.actorPos
            finalSRT.rotation = self.actorRot
            
            local launcherSRT =  self.launcherRig:getBoneLocal(self.launcherStarBone)
            launcherSRT.position = launcherSRT.position + self.startOffset;
            if (cameraTransform) then
                launcherSRT =  launcherSRT * cameraTransform 
            end

            if (launcherSRT and launcherSRT.rotation) then
                 launcherSRT.rotation = Quaternion.lookAtWithUp(Vector.rotateBy(Vector.forward, launcherSRT.rotation), Vector.up);
            end
            self.launcherBlendTimer = self.launcherBlendTimer - Time.dt();
            local amount = math.clamp01(self.launcherBlendTimer / self.launcherBlendDuration);

            -- if (amount < 0.1 and self:isOffScreen() ) then
                -- self.viewerIsLost = true;
                -- self.findingTheViewer = true;
            -- end
            
            finalSRT.position = Vector.lerp(amount, finalSRT.position, launcherSRT.position);
            finalSRT.rotation = launcherSRT.rotation;
        end
    end
    
	self.object:setLocal(finalSRT);
	
	-- Set the perceived depths of the stars for focus effects
	if (self.actorFocusDriver) then
		local actorDepth = self.actorDist * (1 - math.clamp01((self.proxTimeEst - self.evalTime) / self.fullTimeEst) )
		self.actorFocusDriver:setDepth(actorDepth)
	end
	if (self.attractorFocusDriver) then
    	self.attractorFocusDriver:setDepth( self.actorDist )	
	end
	
	
	if (self.debugDraw) then	
		self:drawDebug();
	end
	
end

StarBlender.drawDebug = function (self)
   	--Logger:message( "  == drawDebug == ")

    local axesScale = 40.0	
	local worldT = SRT.new()
	
	-- Draw all of our stored target points
	-- Usually the initial points and the trail
	for _ , target in pairs(self.debugTargets) do
    	if (target) then
    		worldT.position = target.pos
			RenderDebug.axes(worldT, target.size)
        end
    end
    
    -- Draw the current result
	--worldT.position = self.pCurPos
    worldT.position = self.actorPos
    worldT.rotation = self.actorRot
	RenderDebug.axes(worldT, axesScale)
										
	Global.AppDebugText1 = string.format("start (%.5f %.5f %.5f)", 
										self._startPos.x, self._startPos.y, self._startPos.z)
	Global.AppDebugText2 = string.format("attr  (%.5f %.5f %.5f)", 
										self._attractorPos.x, self._attractorPos.y, self._attractorPos.z)	
	Global.AppDebugText3 = string.format("targ  (%.5f %.5f %.5f)", 
										self._targetPos.x, self._targetPos.y, self._targetPos.z)
										
--	Global.AppDebugText1 = string.format("currentPos (%.5f, %.5f, %.5f)", self.pCurPos.x, self.pCurPos.y, self.pCurPos.z)
--	Global.AppDebugText2 = string.format("actorPos (%.5f, %.5f, %.5f)", self.actorPos.x, self.actorPos.y, self.actorPos.z)	

--	Global.AppDebugText3 = string.format("actorPos (%.5f, %.5f, %.5f)", self.testVector.x, self.testVector.y, self.testVector.z)	
end


StarBlender.generateDebugTrail = function (self)
	local loopCount	= 0
	local testScale = 1.0

	while (self.targetReached == false) do
		local dtSim = 1/60
		
		self:getStarVelocity(dtSim)
		
		if (loopCount % 10 == 0) then
			table.insert(self.debugTargets, { pos = self.pCurPos:copy(), size = 1.0} )
		end
		
		loopCount = loopCount + 1
	end
	
   	self.targetReached = false
   	
   	self.pCurPos = self._startPos:copy() 
   	self.pCurVel = Vector(0,0,0)
   	Global.AppDebugText3 = ""
end

StarBlender.isOffScreen = function (self) 
    local mySRT = self.object:getWorld();
    local clipSpace = Util.toClipSpace(mySRT.position);
    return (clipSpace.y >= 1.0 or clipSpace.y <= -1.0 or clipSpace.x >= 1.0 or clipSpace.x <= -1.0 or clipSpace.z < 0 or clipSpace.z > 1);
end

StarBlender.getStarVelocity = function (self, dt)
	-- Convert all world-space coordinates to local-space
	local curPos = self.m2wInv:applyTransform(self.pCurPos)
	local curVel = self.m2wInv:applyRotation(self.pCurVel)

	local curDistToAttractor	= (self.attractorPos - curPos):length()
	local curDistToTarget		= (self.targetPos - curPos):length()

	local totalForce			= Vector(0.0, 0.0, 0.0)	
	
	-- Partition into sections and calculate forces
	if (curPos.x < self.startPos.x * 0.3) then

		local attractPt = self.attractorPos:copy()

		if (self.startPos.y < -self.exclusionRadius or curPos.x > self.startPos.x * 0.8) then
			local radiusScale = 1.0
			if(curPos.y > 0.0)	then 
				attractPt = attractPt + (self.vecUp * self.exclusionRadius * radiusScale)
			else
				attractPt = attractPt - (self.vecUp * self.exclusionRadius * radiusScale)
			end
		end

		local forceUpScale = gForceUpScale

		local beginningUpScaleMag = 20.0
		local beginningUpScaleDist = math.abs(self.startPos.y * 0.7)
		if (curPos.y < self.startPos.y + beginningUpScaleDist) then
			local d = 1.0 - ((curPos.y - self.startPos.y) / beginningUpScaleDist)
			local s = 1.0 + d * beginningUpScaleMag
			forceUpScale = forceUpScale * s
		end

		local vecToAttractor = attractPt - curPos
		local distToAttractor = vecToAttractor:length();
		vecToAttractor:normalize()
		local forceUp = 0.0
		if (math.abs(curPos.y) > gExclusionRadius) then
			local dist = math.abs(curPos.y) - gExclusionRadius
			forceUp = dist * forceUpScale;
		end
		-- attractor + up
		local vecForce = (vecToAttractor * gForceToAttractorScale) + (self.vecUp * forceUp)
		vecForce:normalize()
		totalForce = totalForce + vecForce
		
	 elseif (curPos.x < (self.targetPos.x - self.attractorPos.x) * 0.2) then
		local attractPt = self.targetPos * 0.2
		local vecToAttractor = attractPt - curPos
		local distToAttractor = vecToAttractor:length()
		vecToAttractor:normalize()
		totalForce = totalForce + vecToAttractor
		
	 elseif (curPos.x < (self.targetPos.x - self.attractorPos.x) * 0.5) then
	 	 
		local attractPt = self.targetPos * 0.5
		local vecToAttractor = attractPt - curPos
		local distToAttractor = vecToAttractor:length()
		vecToAttractor:normalize()
		totalForce = totalForce + vecToAttractor

	elseif(curPos.x < self.targetPos.x) then
	
		local attractPt = self.targetPos:copy()
		local vecToAttractor = attractPt - curPos
		local distToAttractor = vecToAttractor:length()
		vecToAttractor:normalize()
		totalForce = totalForce + vecToAttractor
	end

	local finalPos = Vector(0,0,0)
	local finalVel = Vector(0,0,0)
	
	-- If non-zero forces, continue to movement calculation
	local epsilon = 0.0001;
	if (totalForce:length() > epsilon) then
		local v = totalForce:copy()
		v:normalize()
		
		v = v * self.attractAccel * dt * gForceOverallScale
		v = v + curVel		

		-- clamp speed
		local speed = v:length()
		if(speed > self.attractMaxSpeed) then
			v:normalize()
			v = v * self.attractMaxSpeed
		end

		-- Clamp position y near end to taper down to final destination point
		local taperDist		= self.targetPos.x * 0.3
		local taperStart	= self.targetPos.x - taperDist
		if (curPos.x > taperStart) then
			-- local taperRatio = (curPos.x - taperStart) / taperDist			
			local taperMult = 1.0 - math.pow((curPos.x - taperStart) / taperDist , 2)

			curPos.y = curPos.y * 1.0 - math.pow((curPos.x - taperStart) / self.targetPos.x, 2)
			--curPos.y *= 1.0f - math.pow(taperRatio, 2);

			dt = math.max(dt * taperMult, dt * 0.1)
		end
		
		-- Convert back to world space
		finalPos = self.m2w:applyTransform(curPos)
		finalVel = self.m2w:applyRotation(v)

		finalPos = finalPos + (finalVel * dt)

		self.evalTime = self.evalTime + dt
	else
		finalVel = Vector(0.0, 0.0, 0.0)
		finalPos = self.pCurPos:copy()
        self.targetReached = true
        self.findViewerPos = self.object:getLocal().position;

        --Global.AppDebugText1 = "PathDonedfklsfadklgdljkdzlfkjhaldkjfvklazlk"
	   	
		--Global.AppDebugText3 = string.format(" DONE %.5f ", self.evalTime)
	end

	self.pCurVel = finalVel:copy()
	self.pCurPos = finalPos:copy()

   --Logger:message( " ===== "..Global.AppDebugText3.." ===== ")
end
