
local Logger = TaggedLog("StarFocusDriver", true)

StarFocusDriver = {
    -- configurable params    
    focusAdjustDist = 1000.0,
    
    maxDepthLayer		= 7,
    currentDepthLayer	= 0,
    
    percievedDepth		= 0.0,
    
    hasStarted			= false,
}

function StarFocusDriver:new(params)
    local newObj = shallowcopy(params) or {}
    setmetatable(newObj, self)
    self.__index = self

    return newObj
end

function StarFocusDriver:onStart()
    self.percievedDepth		= 0.0
    self.hasStarted = true
end

function StarFocusDriver:setDepth(value)
	self.percievedDepth = value
end

function StarFocusDriver:onUpdate()
	if (self.hasStarted == false) then return end

	local distFromFocus = self.percievedDepth - Global.focalDistance
	local blendAmount	= math.easeInOutSine(math.clamp01(distFromFocus / (self.focusAdjustDist)))	
	local newLayer		= math.floor(blendAmount * (self.maxDepthLayer-1))
	
    if ( self.object.animDriver and newLayer ~= self.currentDepthLayer) then
    	self.object.animDriver:setLayer(newLayer)
    	self.currentDepthLayer = newLayer
    end	
    
--	Global.AppDebugText2 = string.format("focal dist %.5f : depth %.5f : dist %.5f",Global.focalDistance, self.percievedDepth, distFromFocus) 
--	Global.AppDebugText3 = string.format("blendAmount %.5f : newLayer %.3d", blendAmount, newLayer) 
end

