
--[[
Description: Controls the animation of the Duet logo and show start

	fadeOutSpeed    : Speed at which logo fades out after loop plays once
]]


TitleAnimator = {}
TitleAnimator.__index = TitleAnimator

TitleAnimator.new = function(fbLoopStart, fbEnd, looping)
    local self = {};
    
    self.currentTime      	= 0
    self.fadeOut 			= 1
    self.fbEnd     		    = fbEnd
    self.dying              = false
    self.loopStart          = fbLoopStart
    self.readyToRockAndRoll = false
    self.alphaAmount        = 0
    self.alphaMax           = 0
    
	
	self.delay              = 1.5
    self.playTime           = 4
	self.titleMatName 		= "Title_Mat"
	self.fadeOutSpeed       = 0.4
	self.looping			= true


    return setmetatable(self, TitleAnimator)
end


function TitleAnimator:onStart()
	
	-- Avoid showing object before fully initialized
	self.object:setVisibility(false)

	self.titleMat = self.object:findMaterial(self.titleMatName)
	self.titleMat:setParameters({ u_Alpha = 0})

	self.acquiredScreen = false
	self.fadeInComplete = false
end

function TitleAnimator:onUpdate()

	local dt = Time.dt()

	-- When self.play is set to true, play the animation
	if(self.play) then

		-- Fetch screen alpha value when show starts
		if(mainView and not self.alphaAmount) then
			self.alphaAmount = mainView:getAlphaRange().min
			self.alphaMax = mainView:getAlphaRange().max
		end

		if(self.delay <= 0) then
			if (not self.acquiredScreen) then
				self.acquiredScreen = true
				Android.acquireScreen()
			end

			self.object:setVisibility(true)		
			self.object.components.DuetLogo_Driver:showTime(self.currentTime)
			self.currentTime = self.currentTime + dt
			self.titleMat:setParameters({ u_Alpha = 1})

			-- Check if show is ready to run
			if(self.currentTime > self.fbEnd) then

				if (self.looping == false) then
					self:killMe()
				elseif ( self.playTime <= 0 ) then
					self:killMe()
				end
				self.currentTime = self.loopStart
				self.readyToRockAndRoll = true			
			end

			if(self.dying) then
				self.fadeOut = self.fadeOut - dt * self.fadeOutSpeed
				self.titleMat:setParameters({ u_Alpha = math.clamp01(self.fadeOut)})	
				if (self.fadeOut <= 0) then
					self.object:destroy()
					return
				end
			end		
			
			self.playTime = self.playTime - dt
		else
			self.delay = self.delay - dt
		end

		-- Background fade in

		-- If show is starting from intro, then wait a short duration 
		--  before fading background in
		if(Global.isFirstLaunch) then
			if(self.delay <= 0) then
				self.alphaAmount = math.clamp01(self.alphaAmount + dt)
				Android.setAlphaRange(self.alphaAmount, 1)	
			end
		-- If show is starting from a checkpoint, start fading in immediately
		else
			self.alphaAmount = math.clamp01(self.alphaAmount + dt)
			Android.setAlphaRange(self.alphaAmount, self.alphaAmount)	
		end

		
	end
end

function TitleAnimator:playLogoIntro(looping)
	self.play = true
end

function TitleAnimator:killMe()
	if (not self.fadeInComplete) then
		self.fadeInComplete = true
		Android.setOpaqueBackground(true)
		mainView:setClipRect(0, Global.screen_size.y - Global.avail_size.y, Global.avail_size.x, Global.avail_size.y)
	end

	self.dying = true	
end

function TitleAnimator:checkReady()
	return self.readyToRockAndRoll
end

function TitleAnimator:onDestroy()
end

