local State = {
	name = nil,
	fbdrivers = nil,
	actor = nil,
	treeAnimator = nil,
	finished = false,
	
	new = function(self, name, data)
        local o = shallowcopy(data) or {};
        setmetatable(o, self);
        o.name = name;
        self.__index = self;

        return o;
	end,
	
	enter = function(self, treeAnimator, actor)
		self.treeAnimator = treeAnimator;
		self.actor = actor;
		self.fbdrivers = actor.fbdrivers[self.name]
		for matName, fb in pairs(self.fbdrivers) do
			Log.message("TreeAnimator state "..self.name.." Animating: "..matName);
			self.fbdrivers[matName]:init();
			self.fbdrivers[matName]:play();
		end
		
	end,
	
	update = function(self) 
		
		local allFinished = true
		
		for matName, fb in pairs(self.fbdrivers) do
			if(allFinished)then
				allFinished = self.fbdrivers[matName]:isFinished()
			else
				break
			end
		end
		self.finished = allFinished
		
	end,
	
	onFinished = function(self)
		for matName, fb in pairs(self.fbdrivers) do
			if(fb.onFinished) then
				fb:onFinished(self.treeAnimator)
			end
		end
	end,
	
	exit = function(self) 
		for name, fb in pairs(self.fbdrivers) do
			fb:stop();
		end
	end,
}


TreeAnimator = {
	states = {
	},
	
	currentState = nil
}


TreeAnimator.new = function(self, data)
    local o = data or {};
    setmetatable(o, self);
    self.__index = self;
    return o;
end

function TreeAnimator:onStart()
	
end

function TreeAnimator:onUpdate()
	if(self.currentState) then
		local wasFinished = self.currentState.finished
		self.currentState:update()
		if(self.currentState.onFinished and (not wasFinished and self.currentState.finished))then
			Log.message("$$$$ ON FISHED ---------------------")
			self.currentState:onFinished(self)
		end
	end
end

function TreeAnimator:changeState(stateID)
	local state = self.states[stateID]
	if(not state) then 
		self.states[stateID] = State:new(stateID) 
		state = self.states[stateID];
	end
	
	if(self.currentState) then
		Log.message("Exit: "..self.currentState.name)
		self.currentState:exit()
	end
	
	Log.message("Enter: "..state.name)
	state:enter(self, self.object)
	
	self.currentState = state
end
