
local Logger = TaggedLog("Delay", false)

Delay = Manager:new {
    name = "Delay",
}

function Delay:onStart()
    self.pending = self.pending or {}
end

function Delay:runAfter(delay, func, name)
    if delay == 0 then
        -- call immediately
--        Logger:message("Running " .. name .. " immediately")
        func()
    else
        name = name or "<function>"
        Logger:message("Queueing " .. name .. " for " .. tostring(delay))
        self.pending = self.pending or {}
        table.insert(self.pending, { func = func, delay = delay, timer = 0, name = name})
    end
end

function Delay:onUpdate()
    local delta = Time.dt()
    local stillPending = {}
    for _, entry in ipairs(self.pending) do
        entry.timer = entry.timer + delta
        if (entry.timer >= entry.delay) then
            Logger:message("Calling: " .. entry.name .. " after " .. entry.delay .. ", with timer = " .. entry.timer)
            entry.func()
        else
            table.insert(stillPending, entry)
        end
    end
    self.pending = stillPending
end
