
require "common/scripts/moxie/moxie"
require "episode3/scripts/fixup/fixup"
require "episode3/scripts/fixup/logger"
require "episode3/scripts/fixup/events"
require "episode3/scripts/fixup/bone"
require "episode3/scripts/util"


-- data
require "episode3/scripts/scenedata/frames/dogtest"
require "episode3/scripts/scenedata/frames/treeclimb01"
require "episode3/scripts/scenedata/frames/treeclimb04_lookup"
require "episode3/scripts/scenedata/actors"
require "episode3/scripts/scenedata/sets"

-- components
require "episode3/scripts/components/animationdriver"
require "episode3/scripts/components/camera/camerarig"
require "episode3/scripts/components/camera/cameragui"
require "episode3/scripts/components/camera/layers/cameralayer"
require "episode3/scripts/components/camera/layers/inputlayer"
require "episode3/scripts/components/camera/layers/headingclamplayer"
require "episode3/scripts/components/camera/layers/actorclamplayer"
require "episode3/scripts/components/camera/layers/trackinglayer"
require "episode3/scripts/components/camera/layers/boundslayer"
require "episode3/scripts/components/camera/layers/boomlayer"
require "episode3/scripts/components/camera/layers/proxylayer"
require "episode3/scripts/components/camera/layers/lookatlayer"
require "episode3/scripts/components/camera/layers/actorslayer"
require "episode3/scripts/components/camera/layers/tiltdampeninglayer"
require "episode3/scripts/components/camera/layers/actormoverlayer"
require "episode3/scripts/components/camera/layers/starblenderlayer"
require "episode3/scripts/components/camera/layers/squashlayer"
require "episode3/scripts/components/camera/cameracontroller"

require "episode3/scripts/components/camera/debugcamera"
require "episode3/scripts/components/camera/headingclamp"
require "episode3/scripts/components/camera/boundsmanager"
require "episode3/scripts/components/lookatdetectorcore"
require "episode3/scripts/components/lookatdetector"
require "episode3/scripts/components/animatedlookatdetector"
require "episode3/scripts/components/particlecontroller"
require "episode3/scripts/components/openingspherecontroller"
require "episode3/scripts/components/omnilightprobe"
require "episode3/scripts/components/actorfader"
require "episode3/scripts/components/actormover"
require "episode3/scripts/components/actorpositionstore"
require "episode3/scripts/components/debugdot"
require "episode3/scripts/components/fbvisualizer"
require "episode3/scripts/components/flipbookdriver"
require "episode3/scripts/components/simplefbdriver"
require "episode3/scripts/components/dogtest"
require "episode3/scripts/components/mover"
require "episode3/scripts/components/moveranim"
require "episode3/scripts/components/softtrack"
require "episode3/scripts/components/starblender"
require "episode3/scripts/components/starfocusdriver"
require "episode3/scripts/components/trackingtarget"
require "episode3/scripts/components/axis"
require "episode3/scripts/components/parenttocamera"
require "episode3/scripts/components/raytoplane"
require "episode3/scripts/components/proxyactor"
require "episode3/scripts/components/waveanimator"
require "episode3/scripts/components/billboard"
require "episode3/scripts/components/masterblender"
require "episode3/scripts/components/showsphereblender"
require "episode3/scripts/components/dogblender"
require "episode3/scripts/components/pondwaterblender"
require "episode3/scripts/components/glowfader"
require "episode3/scripts/components/showender"
require "episode3/scripts/components/actortocameraoffset"
require "episode3/scripts/components/fadebyscale"
require "episode3/scripts/components/locktocamera"
require "episode3/scripts/components/treeanimator"
require "episode3/scripts/components/glowalpha"
require "episode3/scripts/components/titleanimator"
require "episode3/scripts/components/butterflycontroller"
require "episode3/scripts/components/movingstaranimator"
require "episode3/scripts/components/animationdelay"


-- controllers/managers
require "episode3/scripts/manager"
require "episode3/scripts/actormanager"
require "episode3/scripts/bakeactor"
require "episode3/scripts/postprocessactor"
require "episode3/scripts/inputmanager"
require "episode3/scripts/debugmanager"
require "episode3/scripts/delay"
require "episode3/scripts/segment"
require "episode3/scripts/segmentmanager"
require "episode3/scripts/omnilightblendmanager"
require "episode3/scripts/drawtext"
require "episode3/scripts/alignactormanager"
require "episode3/scripts/launcher/analyticsmanager"

-- audio
require "episode3/scripts/audio/audioplayer"
require "episode3/scripts/audio/soundcontrol"
require "episode3/scripts/audio/positionalsound"

-- utils
require "episode3/scripts/util/serpent"

-- stats
require "episode3/scripts/stats" -- please to not require anything after this file.

-- Tools
require "episode3/scripts/tools/findactortool"
require "episode3/scripts/tools/segmentgraph"

local glowhandle = nil

local Logger = TaggedLog("Global", true)

local usePostFlashlight = false;

Global = {
    curScreenWidth =1280, 
    curScreenHeight = 720,
    delayedUpdates = {},
    preUpdates = {},
}

Global.screen_size      = Vector(384.0, 640.0, 0.0, 0.0)
Global.avail_size       = Vector(384.0, 640.0, 0.0, 0.0)
Global.clearColor       = Color(0.0,0.0,0.0,0.0)
Global.nearFar          = { n = 1, f = 50000}
Global.aspect			= 1;
local aperture = 0.945 * 25.4 -- aperture of the camera from Maya * 25.4
local fov = math.deg(2.0 * math.atan2(aperture * 0.5, 35));
Global.fov              = fov
Global.dof              = 670
Global.focalDistance    = 37
Global.isFirstLaunch    = false


Global.introAnimDelay = 5
Global.introAudioIndex = 0

Global.screenGlowRenderGroup = 150
Global.offsetGlowRenderGroup = 250

-- Controls for Glow Fadeoff, opacity and curve flip
Global.opacity          = 0.42

Global.segmentOffset    = Quaternion.new()

Global.renderAnimationInfo = false
Global.renderFlipbookInfo = false
Global.renderAudioInfo = false
Global.renderComponentTree = false

Global.renderFPS = true  -- used for just displaying the FPS in modes where debugRender is off

Global.timeString 		= ""

-- Automatically pause the show on startup (use "-config autopause" to turn this on)
Global.pauseOnStart = false

Global.introStarTouched = true
Global.introStarFinished = false



if (Platform.candidate) then
    Global.debugRender      = false
	Global.enableUnitTest   = false
    Global.renderOverTSB    = true
    Global.renderFPS        = false
end

if (Platform.debug or Platform.release) then
    Global.debugRender      = true
	Global.enableUnitTest   = true
    Global.renderOverTSB    = true
    Global.loadDelay        = 0
end

if (Platform.branch == "R") then
	Global.enableUnitTest   = false
	Global.renderFPS        = false
end

if (Platform.profile) then
    Global.debugRender      = false
	Global.enableUnitTest   = false
    Global.renderOverTSB    = true
end

local function emptyFn() end
if Platform.profile or Platform.candidate then
    -- shut down logging
    Log.enable = false
    Log.message = emptyFn
    Log.warning = emptyFn
    Log.error = emptyFn
    TaggedLog.message = emptyFn
    TaggedLog.warning = emptyFn
    TaggedLog.error = emptyFn

    -- shut down asserts
    assert = emptyFn
end

-- Switch to turn off audio for testing purposes
Global.audioEnabled = true

-- High level profiling display
Global.profileMode       = false
Global.frame             = 0
Global.frameTimesSaved   = false
Global.endShowFrameCount = 0
Global.startShowTime     = nil
Global.endShowTime       = nil

--Lua to C++ conversion helpers.
Global.enableNativeGlow = actorvignette and actorvignette.update

-- Render views
enum( "VF_MAIN_VIEW", 1 )
enum( "RV_OCCLUDER_VIEW", 2 )

local toolboxInitialized = false

Global.init = function()

	mainRenderTargetDescription =
	{
	    surfaces =
	    {
	        {
	            attachment          = RTA_COLOR0,
	            type                = RTF_TEXTURE2D,
	            format              = PF_RGBA8,
	        }
	    }
	}	

	Stat:init();
	
	mainView = Renderer.createView("main")
	mainView:setFlags( VF_MAIN_VIEW )
	mainView:setRenderTarget( mainRenderTargetDescription )
    
	--reduce the size of the mainView render target on certain devices for better performance.  For now,
	--I'll just set it to a default value that looks ok.  Even 0.5 looks acceptible on some devices like MotoG.
	local renderTargetScale = 1.0

	if (DeviceInfo ~= nil) then
		local info = DeviceInfo.getAndroidBuild()
		if (info and info.DEVICE == "peregrine") then
			renderTargetScale = 0.5  -- Moto G-LTE
		end
	end

	if(mainView.setRenderTargetScale) then
		mainView:setRenderTargetScale(renderTargetScale)
	else
		mainView:setSuperSampleRate(renderTargetScale)
	end

    -- don't use time, be deterministic
    if type(jit) == 'table' then
        Log.message("version"..jit.version)  --LuaJIT 2.0.2
    end

    Global.scene = Scene.create()
	if ( duet and duet.setScene ) then
		duet.setScene( Global.scene )
	end
    
    -- initialise Google analytics
   	AnalyticsManager.initialize()

    InputManager:init(Global.scene)


	collectgarbage("setpause",80)	
	collectgarbage("setstepmul",200)

    DebugManager:init(Global.scene)
    Delay:init(Global.scene)
    if Global.audioEnabled then
        Global.audioPlayer = AudioPlayer.new(SceneData.audio)
    end

    Global.scene.cameraRig  = Global.scene:createTransform("cameraRig")
    Global.scene.mainCamera = Global.scene:createCamera("camera001")

    Global.scene.cameraRig:addChild(Global.scene.mainCamera)
    Global.scene.mainCamera:setPerspective(Global.fov, Global.nearFar.n, Global.nearFar.f)
    Global.scene.cameraRig:setPosition(Vector(0, 0, 0))
    Global.scene.mainCamera.controller = Global.scene.mainCamera:addComponent("cameraControl", CameraController:new())
    if(Platform.win32 or Platform.mac) then
        Global.scene.mainCamera.debugcontroller = Global.scene.mainCamera:addComponent("debugcamera", DebugCameraController);
    end
    MainCamera = Global.scene.mainCamera
    
	mainView:setCamera( Global.scene.mainCamera )
	
    -- create a transform for screen space items
    
    local screenSpace = Global.scene:createTransform("ScreenSpace")
    Global.scene.mainCamera:addChild(screenSpace)
    local screenSpaceSRT = screenSpace:getLocal()
    screenSpaceSRT.rotation = Quaternion.fromEulerAngles(Vector(0, 0, 0))
    screenSpaceSRT.position = Vector(0, 0, -1405)
    screenSpace:setLocal(screenSpaceSRT)
    Global.scene.screenSpace = screenSpace
    
    -- show all actor bones
    ActorManager.showBones = false;
    RenderDebug.setEnabled(Global.debugRender);
    if(System.enableOverlayManger ~= nil) then
        System.enableOverlayManger(true);
    end

end

--[[
    Tests to verify if the C++ version of Lua math matches up.
  ]]
Global.unitTest = function()
	if (duet and duet.unitTest_Vector) then
		--test out vector math.
		local vec1 = Vector.new(0.25,1.2,0.4,0.0)
		local vec2 = Vector.new(0.25,1.2,0.4,0.0)
		local vec3 = Vector.new(0.33, 0.87, 0.74, 1.0)
		vec1:normalize()
		vec1:scale(2.73);
		vec1 = vec1 + vec2*0.5 - vec3;
		vec1 = vec1 / vec3
		vec2 = vec2 * vec2
		vec1 = vec1 + vec2 + vec3
		duet.unitTest_Vector(vec1.x, vec1.y, vec1.z, vec1.w)
	end

	if (duet and duet.unitTest_Matrix) then
		local v0 = Vector.new(10.01, 10.02, 1.5, 0.0);	--Vector v0(0.01f, 0.02f, 1.5f, 0.0f);
		local M1 = Matrix.new();						--Matrix M1;
		local v1 = M1:applyTransform(v0);				--Vector v1 = M1.applyTransform(v0);
		
		local M2 = Matrix.new(	1.0, 0.0, 0.0, 0.0,
								0.0, 1.0, 0.0, 0.0,
								0.0, 0.0, 1.0, 0.0,
								0.0, 0.0, 0.0, 1.0);
		local v2 = M2:applyTransform(v1); 					--Vector v2 = M2.applyTransform(v1);
		local M3 = M2:copy();								--Matrix M3 = M2.copy();
		local v3 = M3:applyTransform(v1);					--Vector v3 = M3.applyTransform(v1);
		local v4 = v2 + v3;									--Vector v4 = v2 + v3;
		M3 = Matrix.identity:copy();						--M3 = Matrix::identity;
		M3:setTranslation(v4);								--M3.setTranslation(v4.x, v4.y, v4.z);
		M3:translate(1.0, 2.0, 3.0);						--M3.translate(1.0f, 2.0f, 3.0f);
		local v5 = Vector.new(0.1, 0.2, 0.2, 1.0);			--Vector v5(0.1f, 0.2f, 0.2f, 1.0f);
		local v6 = M3:applyTransform(v5);					--Vector v6 = M3.applyTransform(v5);
		--Matrix MLookAt = Matrix::lookAt(Vector(1.0f, 1.0f, 1.0f), Vector(2.0f, 3.0f, 4.0f), Vector(0.0f, 1.0f, 0.0f));
		local MLookAt = Matrix.lookAt(Vector.new(1.0, 1.0, 1.0), Vector(2.0, 3.0, 4.0), Vector(0.0, 1.0, 0.0));
		local v7 = MLookAt:applyTransform(v6);				--Vector v7 = MLookAt.applyTransform(v6);
		
		local perspX = Matrix.perspectiveFovX(10.0, 3.0, 0.1, 10.0);	--Matrix perspX = Matrix::perspectiveFovX(10.0f, 3.0f, 0.1f, 10.0f);
		local perspY = Matrix.perspectiveFovY(20.0, 4.0, 0.2, 100.0);	--Matrix perspY = Matrix::perspectiveFovY(20.0f, 4.0f, 0.2f, 100.0f);
		local v8 = perspX:applyRotation(v7);				--Vector v8 = perspX.applyRotation(v7);
		local v9 = perspY:applyTransform(v8);				--Vector v9 = perspY.applyTransform(v8);
		perspX:invert();									--perspX.invert();
		perspY:invert();									--perspY.invert();
		local v10 = perspX:applyTransform(v9);				--Vector v10 = perspX.applyTransform(v9);
		local v11 = perspY:applyRotation(v10);				--Vector v11 = perspY.applyRotation(v10);
		local M4 = Matrix.new(); local M5 = Matrix.new();	--Matrix M4, M5;
		local M6 = M4 + M5;									--Matrix M6 = M4 + M5;
		local result = M6:applyTransform(v11);				--Vector result = M6.applyTransform(v11);
		
		duet.unitTest_Matrix(result.x, result.y, result.z, result.w);
	end

	if (duet and duet.unitTest_Quaternion) then
		local v = Vector.new(0.000, 0.001, 0.002, 0.003);--Vector v(1.0f, 1.1f, 1.2f, 1.3f);
		local q = Quaternion.identity:copy();			--Quaternion q = Quaternion::identity;
		q = q + Quaternion.new(1.01, 1.02, 1.03, 1.04);	--q = q + Quaternion(10.0f, 13.0f, 14.0f, 15.0f);
		local x0 = q:length();							--float x0 = q.length();
		local x1 = q:lengthSq();						--float x1 = q.lengthSq();
		local x2 = v:length();							--float x2 = v.length();
		local f = x0 + x1;								--float f = x0 * x1;
		local x1 = q:normalize();						--x1 = q.normalize();
		f = f * x1;										--f = f * x1;
		f = f * x2;										--f = f * x2;
		v.x, v.y, v.z, v.w = Quaternion.unpack(q);		--q.unpack(v.x, v.y, v.z, v.w);
		v:scale(f);										--v.scale(f);
		local q1 = Quaternion.new(2.0, 3.0, 4.0, 5.0);	--Quaternion q1(2.0f, 3.0f, 4.0f, 5.0f);
		local q2 = Quaternion.new(0.1, 0.3, 0.5, 0.7);	--Quaternion q2(6.0f, 7.0f, 8.0f, 9.0f);
		local q3 = Quaternion.new();					--Quaternion q3;
		local q4 = Quaternion.new();					--Quaternion q4;
		q3:fastMul(q1, q2);								--q3.fastMul(q1, q2);
		q4:fastAdd(q1, q3);								--q4.fastAdd(q1, q3);
		q3:fastMadd(q1, q4);							--q3.fastMadd(q1, q4);
		q4 = q3:copy();									--q4 = q3.copy();
		q:copy_inplace(q4);								--q.copy_inplace(q4);
		q3:set(0.1, 0.2, 0.3, 0.4);						--q3.set(0.1f, 0.2f, 0.3f, 0.4f);
		q4:fastMul(q, q3);								--q4.fastMul(q, q3);
		q = q4:scale(2.0);								--q = q4.scale(2.0f);
		q3:set(v.x, v.y, v.z, v.w);						--q3.set(v.x, v.y, v.z, v.w);
		q4:fastAdd(q, q3);								--q4.fastAdd(q, q3);
		q3:set(1.5, 2.6, 3.7, 4.8);						--q3.set(10.0f, 20.0f, 30.0f, 40.0f);
		q:fastAdd(q3, q4);								--q.fastAdd(q3, q4);
		v:set(5.0, 6.0, 7.0);							--v.set(15.0f, 25.0f, 35.0f);
		v = v * f;										--v *= f;
		q3:fastAdd(q, q4);								--q3.fastAdd(q, q4);
 		q = q3:inverse();								--q = q3.inverse();
 		q:invert();										--q.invert();
 		q3 = q:conjugate();								--q3 = q.conjugate();
 		q4:set(0.10, 0.02, 0.03, 0.04);					--q4.set(10.0f, 20.0f, 30.0f, 40.0f);
 		q = Quaternion.fromTo(q3, q4);					--q = Quaternion::fromTo(q3, q4);
		local v2 = Vector.new(1.0, 2.0, 3.0, 4.0);		--Vector v2(1.0f, 2.0f, 3.0f, 4.0f);
 		q1 = Quaternion.lookAt(v2);						--q1 = Quaternion::lookAt(v2, v3);
		local M = Matrix.new();							--Matrix M;
		M = q1:toMatrix();								--M = q1.toMatrix();
 		q1:setMatrix(M);								--q1.setMatrix(M);
 		local dir = Vector.new(1.0, 0.0, 0.0);			--Vector dir(1.0f, 0.0f, 0.0f);
 		local up = Vector.new(0.0, 1.0, 0.0);			--Vector up(0.0f, 1.0f, 0.0f);
 		q2 = Quaternion.lookAtWithUp(dir, up);			--q2 = Quaternion::lookAtWithUp(dir, up);
 		q3 = Quaternion.lookAt(dir);					--q3 = Quaternion::lookAt(dir);
		q4:fastAdd(q, q1);								--q4.fastAdd(q, q1);
 		q:fastAdd(q4, q2);								--q.fastAdd(q4, q2);
 		q2:fastAdd(q, q3);								--q2.fastAdd(q, q3);
 		q3:set(1.0, 2.0, 3.0, 4.0);						--q3.set(1.0f, 2.0f, 3.0f, 4.0f);
 		q4:set(1.0, 3.0, 2.0, 4.0);						--q4.set(1.0f, 3.0f, 2.0f, 4.0f);
 		q = Quaternion.shortestDistance(q3, q4);		--q = Quaternion::shortestDistance(q3, q4);
 		q3:fastAdd(q2, q);								--q3.fastAdd(q2, q); 
 		q = q3:copy();									--q = q3.copy();
		q2:set(1.5, 5.6, 7.3, 0.9);						--q2.set(1.5f, 5.6f, 7.3f, 0.9f);
		q3:set(1.7, 1.8, 1.9, 1.0);						--q3.set(2.8f, 6.8f, 4.9f, 9.5f);
		local l1 = Quaternion.lerp(0.2, q2, q3);		--Quaternion l1 = Quaternion::lerp(0.2f, q2, q3);
		q2:fastAdd(q, l1);								--q2.fastAdd(q, l1);
		q = q2;											--q = q2;
		local result = Vector.new(q.x, q.y, q.z, q.w);	--Vector result(q.x, q.y, q.z, q.w);
		f = Quaternion.dot(q, q3);						--f = Quaternion::dot(q, q3);
		result = result * f;							--result *= f;
		
		duet.unitTest_Quaternion(result.x, result.y, result.z, result.w)
	end
	if (duet and duet.unitTest_SRT) then

		local v = Vector.new(0.0, 0.0, 0.0, 0.0);	--Vector v(1.0f, 2.0f, 3.0f, 4.0f);
		local v1 = Vector.new(1.0, 2.0, 3.0, 4.0);		--Vector v1(1.0f, 2.0f, 3.0f, 4.0f);

		local s = Vector.new(1.5, 2.0, 2.5, 3.0);		--Vector s(1.5f, 2.0f, 2.5f, 3.0f);
		local t = Vector.new(0.11, 0.12, 0.12, 1.0);		--Vector t(11.0f, 12.0f, 13.0f, 1.0f);
		local r = Quaternion.new(0.7, 0.8, 0.9, 1.0);	--Quaternion r(0.7f, 0.8f, 0.9f, 1.0f);
		local srt = SRT.new(t, r, s);					--SRT x(t, r, s);
		local srt2 = srt:copy();						--SRT srt2 = srt.copy();
		srt2:applyTranslation(v1);						--srt2.applyTranslation(v1);
		v:copy_inplace(srt2.position);					--v.copy_inplace(srt2.position);
		v = v + srt2.scale;								--v = v + srt2.scale;
		--v = v + Vector(srt2.rotation.x, srt2.rotation.y, srt2.rotation.z, srt2.rotation.w);
		v = v + Vector.new(srt2.rotation.x, srt2.rotation.y, srt2.rotation.z, srt2.rotation.w);
		srt:reset();									--srt.reset();
		srt:applyScale(v);								--srt.applyScale(v);
		srt:applyScale(1.234);							--srt.applyScale(1.234f);
		v:copy_inplace(srt.scale);						--v.copy_inplace(srt2.scale);
		srt:applyRotation(r);							--srt.applyRotation(r);
		--v1.set(srt.rotation.x, srt.rotation.y, srt.rotation.z, srt.rotation.w);
		v1:set(srt.rotation.x, srt.rotation.y, srt.rotation.z, srt.rotation.w);
		v = v + v1;										--v = v + v1;
		srt.position:set(0.3, 0.2, 0.1);				--srt.position.set(1.0f, 2.0f, 3.0f);
		srt.rotation:set(1.0, 1.1, 1.2, 1.3);			--srt.rotation.set(1.0f, 1.1f, 1.2f, 1.3f);
		srt.scale:set(0.01, 0.02, 0.03);					--srt.scale.set(2.0f, 3.0f, 4.0f);
		srt2.position:set(1.1, 2.1, 3.1);				--srt2.position.set(1.1f, 2.1f, 3.1f);
		srt2.rotation:set(2.0, 2.1, 2.2, 2.3);			--srt2.rotation.set(2.0f, 2.1f, 2.2f, 2.3f);
		srt2.scale:set(0.52, 0.62, 0.72);					--srt2.scale.set(2.2f, 3.2f, 4.2f);
		local srt3 = SRT.new();
		srt3 = srt + srt2;								--SRT srt3 = srt + srt2;
		srt2 = srt * srt3;								--srt2 = srt * srt3;
		srt = srt2 - srt3;								--srt = srt2 - srt3;
		srt.position = srt.position + v;				--srt.position + v;
		local m = srt:getMatrix();						--Matrix m = srt.getMatrix();
		local right = srt:right();						--Vector right = srt.right();
		local forward = srt:forward();					--Vector forward = srt.forward();
		local up = srt:up();							--Vector up = srt.up();
		v = m:applyTransform(v);						--v = m.applyTransform(v);
		local result = v + right + forward + up;		--Vector result = v + right + forward + up;

		duet.unitTest_SRT(result.x, result.y, result.z, result.w)
	end
end
 
Global.initShow = function()
	if (duet and duet.init) then
		duet.init()
	end
	MainCamera.controller:startLayers()
	if (Global.enableUnitTest) then
		Global.unitTest()
	end
end

Global.startShow = function()
    Global.startShowTime = os.clock()
    SegmentManager:onStart()

    AnalyticsManager.sendInitialStats()
    AnalyticsManager:onShowBegan()
end 

Global.showEnded = function()
	if (duet and duet.destroy) then
		duet.destroy();
	end
    Global.endShowTime = os.clock()
	Global.endShowFrameCount = Global.frame
	--Replay.onReplayEnd()
end

Global.onShutdown = function()

    if Global.audioPlayer then
        Global.audioPlayer:destroy()
        Global.audioPlayer = nil
    end

    if (Global.scene) then
        Global.scene:destroy()
        Global.scene = nil
    end

	if (glowhandle) then
		-- Remove reference to glow handle
		glowhandle:destroy()
		glowhandle = nil
	end
	
    if (mainView) then
        mainView:destroy()
        mainView = nil
    end	
    
    lookatdetector:onGlobalShutdown()
    trackingtarget:onGlobalShutdown()
    softtrack:onGlobalShutdown()

	System.gc()
end

Global.onUpdate = function()
	if (duet and duet.updateBegin) then
		duet.updateBegin(Global.frame)
	end

	if (Global.scene == nil) then return end

	    
    InputManager:update()
    
    -- debug clear axis label map
    -- script pre updates
    for _, obj in pairs(Global.preUpdates) do
        if(obj and obj.onPreUpdate) then
            obj:onPreUpdate()
        end
    end

    SegmentManager:onUpdate()
    AnalyticsManager:update()

    if toolboxInitialized then
        Toolbox:update()
    end

    if (Global.scene) then
        Global.scene:update()
    end

    if Global.audioPlayer then
        Global.audioPlayer:update()
    end

    -- script late updates
    for _, obj in pairs(Global.delayedUpdates) do
        if(obj and obj.onDelayedUpdate) then
            obj:onDelayedUpdate()
        end
    end

    if (Input.isKeyDown(KB_F12, true)) then
        mainView:saveScreenshot()
    end

    System.gc()
	collectgarbage("step")
	
	Global.frame = Global.frame + 1;

	if (duet and duet.updateEnd) then
		duet.updateEnd()
	end
end


local KB = 1024
local MB = KB * KB

Global.drawInfo = function()
    local textScale = 1.0
    if (Platform.win32 or Platform.mac) then
        textScale = 1.0 --0.5
    end

    local y = 10
    local yStep = 30 * textScale
    local nextLine = function()
        y = y + yStep
        return y - yStep
    end

	if (Platform.record and Platform.record ~= "" and Global.debugRender == false) then
		DrawText:update()
        local textStyle = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
        Renderer.drawText(10, nextLine(), textStyle, "[record: "..Platform.record.."]")
    end

	if (Platform.replay and Platform.replay ~= "") then
		DrawText:update()

		local textStyleRed = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
		Renderer.drawText(10, nextLine(), textStyleRed, "[replay: "..Platform.replay.."]".."[x"..string.format("%.2f", Time.timescale()).."]")

		local textStylePerf = { color = Color.new(0.8, 0.8, 0.8), font = "common:fonts/arial_default_24.fnt", scale = textScale }
		local fpsString = "["..FPS.string().."]"
		Renderer.drawText(10, nextLine(), textStylePerf, fpsString)

		-- NB: workaround for a bug in the Time.string() function returning bad values on pause
        if (not Time.isPaused()) then
        	Global.timeString = Time.string()
       	end

		if Global.endShowTime ~= nil then
            local time = Global.endShowTime - Global.startShowTime
			Renderer.drawText( 10, nextLine(), textStylePerf, string.format("profile time: %0.2f, frame: %u", time, tostring(Global.endShowFrameCount)) )
        else
            Renderer.drawText(10, nextLine(), textStylePerf, "[show time: "..Global.timeString.."]")
        end
    elseif (Global.debugRender) then
        DrawText:update()

        -- TODO: refactor this segment info out of Global
        local segmentName = SegmentManager:currentSegmentName()
        if (segmentName == "") then
            segmentName = "gwi"
        end
        local title     = "[" .. segmentName .. " | "..Platform.package.."]"
        local fpsString = "["..FPS.string().."]"
        
        -- display memory used by the show so far
        local bytesUsed = 0
        if(System.getCPUMemory) then
            bytesUsed = System.getCPUMemory()
        end
        local mbytes    = math.floor(bytesUsed / MB)
        bytesUsed       = bytesUsed - mbytes * MB
        local kbytes    = math.floor(bytesUsed / KB)
        bytesUsed       = bytesUsed - kbytes * KB
        local memString = "["..mbytes.."m-"..kbytes.."k-"..bytesUsed.."]"
        
        
        local osdEnabled = "No OSD"
        if ( OSD and OSD.enabled ) then
            osdEnabled = "OSD"
        end
        local versionString = "["..System.versionString().." | "..osdEnabled.."]"

        local textStyle = { color = Color.new(0.8, 0.8, 0.8), font = "common:fonts/arial_default_24.fnt", scale = textScale }
        Renderer.drawText(10, nextLine(), textStyle, title.." "..versionString)
        
        if(Global.loadDelay) then
            memString = memString.."[loading delay: "..string.format("%.2f",Global.loadDelay).."s]"
        end

        Renderer.drawText(10, nextLine(), textStyle, fpsString.." "..memString)

        if Memory then
            local kbs = Memory.numBytesAllocated() / KB
            local blocks = Memory.numBlocksAllocated()
            local luaKbs = Memory.getLuaAllocatedBytes() / KB
            local luaAllocs = Memory.getLuaAllocationCount()
            local luaTotalAllocs = Memory.getLuaTotalAllocCount()
            if(Global.lastAllocs == nil)then Global.lastAllocs = luaTotalAllocs end
            local allocDelta = luaTotalAllocs - Global.lastAllocs
            Global.lastAllocs = luaTotalAllocs
            Renderer.drawText(10, nextLine(), textStyle,
                string.format("[m %.2f kb|%2d blk] [lua: %.2fkb|%4d alcs %d dt]",
                                kbs, blocks, luaKbs, luaAllocs, allocDelta))
                                
        end

        Renderer.drawText(10, nextLine(), textStyle, "Pending Resources = "..System.getPendingResourceCount())

        if Global.renderAnimationInfo and Global.scene and Global.scene.actors then
            nextLine()
            for name, actor in pairs(Global.scene.actors) do
                if actor.animDriver and actor.animDriver.playing then
                    local anim = actor.animDriver.playing
                    Renderer.drawText(10, nextLine(), textStyle, name .. ": " .. tostring(anim:getStartFrame() + anim:getFrame()))
                end
            end
        end

        if Global.renderFlipbookInfo and Global.scene and Global.scene.actors then
            nextLine()
            for name, actor in pairs(Global.scene.actors) do
                if actor.fbdrivers then
                    local animTime = actor.animDriver:currentTime() * 60
                    local animFrame = actor.animDriver:currentFrame()
--                    Renderer.drawText(10, nextLine(), textStyle, name.." frame ["..animFrame.."]");
                    for n,f in pairs(actor.fbdrivers) do
                        for m, fb in pairs(f) do
                        	local fbFrame = fb.frame
                        	if(not fb.lastLoggedFrame or (fb.lastLoggedFrame ~= fbFrame)) then
                        		fb.lastLoggedFrame = fbFrame
                            	DrawText:color(1,1,1):addText(actor.actorGroup or "none", name.."["..n.."]["..m.."]:"..(fbFrame).."@"..(fb.nativeSampler:getExposureFrame()).." - fbxTime= "..animTime .. " frame = " .. tostring(animFrame))
                            end
                        end
                    end
                end
            end
        end

        if Global.renderAudioInfo and Global.audioPlayer then
            Global.audioPlayer:renderInfo(nextLine, textStyle)
        end

        if Global.renderComponentTree then
            Renderer.drawText(10, nextLine(), textStyle, "Actor components")
            local smallText = { color = Color.new(0.8, 0.8, 0.8), font = "common:fonts/arial_default_18.fnt", scale = textScale }
            local normalYStep = yStep
            yStep = 20 * textScale
            for name, actor in pairs(Global.scene.actors) do
                Renderer.drawText(30, nextLine(), smallText, name .. ":")
                if actor.components then
                    for componentName, _ in pairs(actor.components) do
                        Renderer.drawText(50, nextLine(), smallText, componentName)
                    end
                end
            end
            yStep = normalYStep
        end

		-- NB: workaround for a bug in the Time.string() function returning bad values on pause
        if (not Time.isPaused()) then
        	Global.timeString = Time.string()
       	end
        Renderer.drawText(10, nextLine(), textStyle, "[show time: "..Global.timeString.."]")
        
                
        if (Platform.record and Platform.record ~= "") then
            textStyle = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
            Renderer.drawText(10, nextLine(), textStyle, "[record: "..Platform.record.."]")
        end

        if DebugManager.renderGroupDebugEnabled then
            nextLine()
            Renderer.drawText(10, nextLine(), textStyle, "Render group debug enabled:")
            Renderer.drawText(10, nextLine(), textStyle,
                "    group filter = (" .. tostring(DebugManager.renderGroupFilterMin) .. ", " .. tostring(DebugManager.renderGroupFilterMax) .. ")     [L/R to change]")
            Renderer.drawText(10, nextLine(), textStyle,
                "    max render models = " .. tostring(DebugManager.renderModelsMax) .. "     [U/D to change]")
        end

        if Global.profileMode then
            if Global.startShowTime ~= nil then
                local time = os.clock() - Global.startShowTime
                if Global.endShowTime ~= nil then
                    time = Global.endShowTime - Global.startShowTime
                else
                    Global.frame = Global.frame + 1
                end
                Renderer.drawText( 10, nextLine(), textStyle, string.format("profile time: %0.2f, frame: %u", time, tostring(Global.frame)) )
            end
        end

        nextLine()
        Renderer.drawText(10, nextLine(), textStyle, Global.AppDebugText1 or "")
        Renderer.drawText(10, nextLine(), textStyle, Global.AppDebugText2 or "")
        Renderer.drawText(10, nextLine(), textStyle, Global.AppDebugText3 or "")

    elseif Global.renderFPS then
        -- Just render the FPS line
        DrawText:update()

        local textStylePerf = { color = Color.new(0.8, 0.8, 0.8), font = "common:fonts/arial_default_24.fnt", scale = textScale }
        local fpsString = "["..FPS.string().."]"
        Renderer.drawText(10, nextLine(), textStylePerf, fpsString)
    end
end

Global.onReshape = function(width, height)
    Log.message("Reshape: "..tostring(width)..", "..tostring(height));
    mainView:resize( width, height )
    Global.curScreenWidth = width
    Global.curScreenHeight = height
    Global.aspect = width/height
    if GUI and not toolboxInitialized then
        Global.initializeToolbox()
    end

    if (Global.screen_size.x ~= width or Global.screen_size.y ~= height) then
	    Global.screen_size.x = width
	    Global.screen_size.y = height
	    Global.avail_size.x = width
	    Global.avail_size.y = height
	    Global.updateScreenSize()
	end

    Events.send("renderer", "onReshape", Vector(width, height, 0, 0))
end

Global.onRender = function( )
    mainView:setClearColor( Global.clearColor )
    Global.scene:render( mainView )
    Global.drawInfo()
    Renderer.display( mainView )
end

Global.initializeToolbox = function()
    Log.message("Initializing toolbox size = " .. tostring(width) .. "x" .. tostring(height))
    local tools = Project and Project.tools or nil
    local options = Project and Project.options or nil
    Toolbox:init(tools, options)
    Toolbox:listen("Player Controls", {
        onPlayerNext = function() SegmentManager:next() end,
        onPlayerPrev = function() SegmentManager:previous() end,
        onPlayerPause = function() SegmentManager:pause() end,
        onPlayerResume = function() SegmentManager:resume() end,
    })

    toolboxInitialized = true
end

Global.updateScreenSize = function()
    Logger:message("updating screen size: " .. Global.screen_size.x .. ", " .. Global.screen_size.y)
    if (Platform.android) then
        if (Global.renderOverTSB) then
            -- Use the whole screen
            mainView:setClipRect(0, 0, Global.screen_size.x, Global.screen_size.y)
        else
            -- Clip out the TSB area
            mainView:setClipRect(0, Global.screen_size.y - Global.avail_size.y, Global.avail_size.x, Global.avail_size.y)
        end
    end
end

Global.setRenderOverTSB = function(render)
    Global.renderOverTSB = render
    Global.updateScreenSize()
end

Global.addDelayedUpdate = function(object)
    Global.delayedUpdates[object] = object;
end

Global.addPreUpdate = function(object)
    Global.preUpdates[object] = object;
end

Global.removeLateUpdate = function(object)
    Global.delayedUpdates[object] = nil;
end
