
-- Component to manage the bug input (swipes, touches, etc)

Launcher.bug = {

    touchDownPos = nil,
    tapAllowed = false,
    tapped = false,
    swiped = false,
    swipeDirection = nil,

    init = function(iconbugActor)
        -- Add ourselves as a component to the bug so we
        -- can track input
        iconbugActor:addComponent("launcherbugmanager", Launcher.bug)
    end,

    wasTapped = function(self) return self.tapped end,
    wasSwiped = function(self) return self.swiped end,

    setTapAllowed = function(self, allowed)
        self.tapAllowed = allowed
    end,

    onStart = function(self)
        self.tapped = false
        self.swiped = false
        self.swipeDirection = nil
        self.touchDownPos = nil
    end,

    onUpdate = function(self)
        -- Handle touch events for windows
        if (Platform.win32 or Platform.mac) then
            if (Input.getMouseButton(MB_L_BUTTON)) then
                local pos = Input.getMousePosition()
                if (mainView:hitTest(self.object,pos) and self.tapAllowed) then
                    local fakeScreen = Global.scene.actors.FakeScreen
                    local screenFader = fakeScreen.components["FakeScreenFader"]
                    screenFader:fadeOut(0, 1, true)
                    self.tapped = true
                end
            elseif (Input.getMouseButton(MB_R_BUTTON)) then
                -- Use right mouse button to simulate the swipe gesture and abort
                self.swipeDirection = Vector(0, 1)
                self.swiped = true
            end
        end
    end,

    onEvent = function(self, event)
        local iconFader = self.object.components["IconFader"]

        if (event.type == "onTouchDown") then
            self.touchDownPos = Vector(event.data.x, event.data.y)
            Scheduler.requestFrame()
            return true

        elseif (event.type == "onTouchUp") then
            if (not self.touchDownPos) then return false end
            local touchDelta = Vector(-(self.touchDownPos.x - event.data.x), (self.touchDownPos.y - event.data.y))
            local touchLength = touchDelta:length()
			Log.error(touchLength)
            if (touchLength < 150) then
				
                -- Tap
                if (self.tapAllowed) then
                    self.tapped = true
--					AnalyticsManager:kickupHappened()
                end
            else
                --Swipe
--                self.swiped = true
--                touchDelta:normalize()
--                self.swipeDirection = touchDelta * 0.5
--				AnalyticsManager:GWIAbandoned()
            end
            Scheduler.requestFrame()
            return true
        elseif (event.type == "onInterceptTouch") then
            local pos = Vector(event.data.windowX + event.data.x, event.data.windowY + event.data.y)
            -- if we hit the bug, we want to intercept the event
            return mainView:hitTest(self.object,pos)
        end
        return false

    end,
}
