
Context = Context or {}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns what caused Avatar to launch
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
enum("START_REASON_NORMAL", 0)    -- cloud service
enum("START_REASON_SHORTCUT")     -- launcher shortcut
enum("START_REASON_NOTIFICATION") -- android notification
enum("START_REASON_RESET")        -- forces a reset of GWI
enum("START_REASON_SPLASH")       -- splash screen start button
enum("START_REASON_BROWSER")      -- browser screen button
enum("START_REASON_LAUNCHER")     -- dev launcher
enum("START_REASON_TEST")         -- stab test

-- PRIVATE
Context._getStartReason = function()
    if (Platform ~= nil) then
        if (Platform.reason == "shortcut") then
            return START_REASON_SHORTCUT
        elseif (Platform.reason == "reset") then
            return START_REASON_RESET
        elseif (Platform.reason == "splash_play") then
            return START_REASON_SPLASH
        elseif (Platform.reason == "browser_play") then
            return START_REASON_BROWSER
        elseif (Platform.reason == "notification") then
            return START_REASON_NOTIFICATION
        elseif (Platform.reason == "launcher") then
            return START_REASON_LAUNCHER
        elseif (Platform.reason == "test") then
            return START_REASON_TEST
        else
            return START_REASON_NORMAL
        end
    end
    return START_REASON_NORMAL
end

Context._getStartReasonString = function()
    if (Platform.reason == nil) then
        return "Unknown"
    else
        return Platform.reason
    end
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns what caused Avatar to exit
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
enum("STOP_REASON_NONE", 0)      -- have not yet seen the show
enum("STOP_REASON_GWO")          -- good way out
enum("STOP_REASON_FWO")          -- fast way out
enum("STOP_REASON_TIMEOUT")      -- long timeout off of TSB
enum("STOP_REASON_HIDDEN")       -- blacklist
enum("STOP_REASON_ABNORMAL")     -- reboot or crash
enum("STOP_REASON_NOTIFICATION") -- notification was posted
enum("STOP_REASON_DONE")         -- show has completed

-- PRIVATE
Context._getStopReason = function()
    if (Persist.episode3.context.stopReason == nil) then
        Persist.episode3.context.stopReason = STOP_REASON_NONE
    end
    return Persist.episode3.context.stopReason
end

Context._getStopReasonString = function()
    if Persist.episode3.context.stopReason == nil or Persist.episode3.context.stopReason == STOP_REASON_NONE then 
        return "None"
    elseif Persist.episode3.context.stopReason == STOP_REASON_GWO then
        return "GWO"
    elseif Persist.episode3.context.stopReason == STOP_REASON_FWO then
        return "FWO"
    elseif Persist.episode3.context.stopReason == STOP_REASON_TIMEOUT then
        return "TimedOut"
    elseif Persist.episode3.context.stopReason == STOP_REASON_HIDDEN then
        return "Hidden"
    elseif Persist.episode3.context.stopReason == STOP_REASON_ABNORMAL then
        return "Abnormal"
    elseif Persist.episode3.context.stopReason == STOP_REASON_NOTIFICATION then
        return "Notification"
    elseif Persist.episode3.context.stopReason == STOP_REASON_DONE then
        return "Done"
    end
    return "Unknown"
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns true if GWI should not play for the current phone state
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
Context.isBlacklisted = function()
    if ((System.getEnv == nil) or Platform.win32 or Platform.mac or System.getEnv("DevMode") == "true") then
        return false
    else
        if (not Platform.android) then
            return false
        end

        -- Check screen is on and keyguard off
        local screenPass = PowerUtils.isScreenOn() and not PlatformUtils.isKeyguardLocked()
        if (not screenPass) then
            Log.message("GWI decision failed: screen is off or the keyguard is visible")
            return true
        end
        
        -- Check TSB style
        if (Context.systemUIViz ~= nil and Context.systemUIViz ~= 0) then
            Log.message("GWI decision failed: TSB is in lights out or low profile mode")
            return true
        end

        return false
    end
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Show events
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

Context.sendStoryEvent = function(what, extra)
    if StoryManager and StoryManager.sendEvent then
        if (extra == nil) then
            extra = ""
        end
        StoryManager:sendEvent(what, extra)
    end
end

-- Call when show completes (iris out)
Context.onGoodWayOut = function()
    Persist.episode3.context.stopReason = STOP_REASON_GWO
    Persist.episode3.context.lastCheckpoint = nil
    Persist.episode3.context.lastRunWasGWO = true
    Persist.episode3.context.notificationCanceled = false
    Persist.episode3.context.abnormalExit = false
    System.flushPersist()

    AnalyticsManager:showComplete()
    Context.sendStoryEvent("STORY_STATE_STOPPED", "GWO")

    Avatar.restart(500, true)
    Scheduler.cancel()
end

-- Call when show is interrupted (TSB key or incoming event)
Context.onFastWayOut = function()
    Persist.episode3.context.stopReason = STOP_REASON_FWO
    Persist.episode3.context.lastRunWasGWO = false
    Persist.episode3.context.notificationCanceled = false
    Persist.episode3.context.abnormalExit = false
    System.flushPersist()

    AnalyticsManager:userLeft()
    Context.sendStoryEvent("STORY_STATE_STOPPED", "FWO")

    Avatar.restart(500, true)
    Scheduler.cancel()
end

-- Call when the star times out (long timeout)
Context.onTimeOut = function()
    Persist.episode3.context.stopReason = STOP_REASON_TIMEOUT
    Persist.episode3.context.abnormalExit = false
    System.flushPersist()

    AnalyticsManager:GWIAbandoned()

    Avatar.restart(500, true)
    Scheduler.cancel()
end

-- Call when the doodle has nothing more to present (i.e. after presenting the splash screen following a GWO)
Context.onDone = function()
    Persist.episode3.context.stopReason = STOP_REASON_DONE
    Persist.episode3.context.abnormalExit = false
    System.flushPersist()
end

-- Call when the hat is hiding from the TSB
Context.onHide = function()
    Persist.episode3.context.stopReason = STOP_REASON_HIDDEN
end

-- Call when the notification is posted
Context.onNotification = function()
    Persist.episode3.context.stopReason = STOP_REASON_NOTIFICATION
end

-- Returns true if "tap to continue" notification is active
Context.isNotificationPosted = function()
    return Persist.episode3.context.stopReason == STOP_REASON_NOTIFICATION
end

-- Call when moxie version changes
Context.onVersionUpdate = function()
    Persist.episode3.context = {}
    Persist.episode3.versionUpdated = true
    System.flushPersist()
end

-- Call when the notification is posted
Context.onNotificationCanceled = function()
    Persist.episode3.context.notificationCanceled = true
    System.flushPersist()
end

-- Call when a show checkpoint is crossed
Context.onCheckpoint = function(name)
    Persist.episode3.context.lastCheckpoint = name
    Context.sendStoryEvent("STORY_STATE_SEGMENT_START", name)
    System.flushPersist()
end

-- Returns the last checkpoint hit before the show was interrupted
Context.getCheckpoint = function()
    if (Persist.episode3.context.lastCheckpoint == "") then
        return nil
    end
    return Persist.episode3.context.lastCheckpoint
end

-- Updates the TSB style
Context.onSystemUIVizUpdate = function(newVisibility)
    Context.systemUIViz = newVisibility
end

-- Returns true if the show can start
Context.canStart = function()
    if ((System.getEnv == nil) or Platform.win32  or Platform.mac or System.getEnv("DevMode") == "true") then
        return true
    else
        if (Context.isUnlocked()) then
            return true
        end

        -- ensure we're past the launch date
        local config = Config.active()
        local diff = Calendar.getDateDifferenceInDays(config.launchDay, config.launchMonth, config.launchYear)
        local passLaunchDate = (diff <= 0)
        return passLaunchDate
    end

    return false;
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Timeout on GWI scenario
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

-- Returns true if the star has appeared to the user at least once
Context.isUnlocked = function()
    return Persist.episode3.context.gwiTimeShown ~= nil
end

-- Call when the star is shown to the user
Context.onLauncherStarShown = function()
    if (not Platform.android) then return end

    -- keep track of the time when the star was shown
    if (Persist.episode3.context.gwiTimeShown == nil) then
        Persist.episode3.context.gwiTimeShown = os.time()
        Persist.episode3.context.gwiTimePeriod = Config.active().showdayTimeout

        System.flushPersist()
    end
end

-- Check to see if the star has been shown for too long
Context.isGWITimedOut = function()
    if (Persist.episode3.context.gwiTimeShown and type(Persist.episode3.context.gwiTimeShown) == "number" and
        Persist.episode3.context.gwiTimePeriod and type(Persist.episode3.context.gwiTimePeriod) == "number") then

        local shownFor = os.time() - Persist.episode3.context.gwiTimeShown
        return shownFor > Persist.episode3.context.gwiTimePeriod
    end
    return false
end

-- Return how much time we have left before we should timeout
Context.getGWIRemaining = function()
    if (Persist.episode3.context.gwiTimeShown and type(Persist.episode3.context.gwiTimeShown) == "number" and
        Persist.episode3.context.gwiTimePeriod and type(Persist.episode3.context.gwiTimePeriod) == "number") then

        local shownFor = os.time() - Persist.episode3.context.gwiTimeShown
        return Persist.episode3.context.gwiTimePeriod - shownFor
    end
    return Config.active().showdayTimeout
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns the current scenario
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
enum("PROMPT_GWI", 0)                     -- GWI based intro
enum("PROMPT_UNHIDE")                     -- star is reappearing after a blacklist condition
enum("PROMPT_FLOURISH_NOTIFICATION")      -- star animates off the top of the screen to post a notification
enum("PROMPT_FLOURISH_NOTIFICATION_IDLE") -- after a screen cycle, star animates off the top of the screen to post a notification
enum("PROMPT_FLOURISH_INTRO_RESTART")     -- star animates into the beginning of the show due to a splash screen start
enum("PROMPT_FLOURISH_INTRO_START_IDLE")  -- star animates into the beginning of the show after user taps notification
enum("PROMPT_FLOURISH_INTRO_MIDDLE_IDLE") -- star animates off the top of the screen after user taps notification
enum("PROMPT_SHOW_LIBRARY")                    -- display the library screen

Context.getPromptType = function()
    local startReason = Context._getStartReason()
    local stopReason = Context._getStopReason()

    --
    -- spaghetti code that determines what scenario should run next based
    -- on context and history
    --
    if (Platform.win32) then
        -- always start the show immediately in Windows
        return PROMPT_FLOURISH_INTRO_RESTART
    elseif (startReason == START_REASON_RESET) then
        -- debug feature - reset GWI
        return PROMPT_GWI
    elseif (startReason == START_REASON_NOTIFICATION or stopReason == STOP_REASON_NOTIFICATION or startReason == START_REASON_TEST) then
        -- we posted a notification and the user just clicked on it
        if (Context.getCheckpoint() == nil) then
            -- FWI to show start - play the stitched animation
            return PROMPT_FLOURISH_INTRO_START_IDLE
        else
            -- FWI to middle of show - play the notification animation
            return PROMPT_FLOURISH_INTRO_MIDDLE_IDLE
        end
    elseif (stopReason == STOP_REASON_HIDDEN) then
        -- bring the star back from a blacklist state
        return PROMPT_UNHIDE
    elseif (stopReason == STOP_REASON_DONE and startReason ~= START_REASON_BROWSER) then
        -- we've already done a GWO and shown the splash, so do nothing
        return nil
    elseif (stopReason == STOP_REASON_GWO) then
        -- show library after GWO
        return PROMPT_SHOW_LIBRARY
    elseif (stopReason == STOP_REASON_TIMEOUT) then
        -- wait for a screen cycle, then fly the star up and post a notification
        return PROMPT_FLOURISH_NOTIFICATION_IDLE
    elseif (stopReason == STOP_REASON_FWO) then
        -- fly the star up and post a notification
        return PROMPT_FLOURISH_NOTIFICATION
    elseif (startReason == START_REASON_SPLASH or startReason == START_REASON_BROWSER or startReason == START_REASON_SHORTCUT) then
        -- splash screen always starts the show immediately
        return PROMPT_FLOURISH_INTRO_RESTART
    elseif (Persist.episode3.context.notificationCanceled) then
        -- user selected X on notification
        return nil
    elseif (not Context.isUnlocked()) then
        -- this is the very first appearance of the star
        return PROMPT_GWI
    elseif (stopReason == STOP_REASON_ABNORMAL) then
        -- a reboot interrupted our last session
        if (Persist.episode3.context.lastRunWasGWO) then
            -- reboot followed a GWO, so do nothing
            return nil
        else
            -- otherwise wait for a screen cycle then post a notification
            return PROMPT_FLOURISH_NOTIFICATION_IDLE
        end
    end

    -- do nothing (unknown reason)
    return nil
end

Context.getPromptReasonString = function()
    reason_int = Context.getPromptType()

    if (reason_int == PROMPT_GWI) then
        return "PROMPT_GWI"
    elseif (reason_int == PROMPT_FLOURISH_NOTIFICATION) then
        return "PROMPT_FLOURISH_NOTIFICATION"
    elseif (reason_int == PROMPT_FLOURISH_NOTIFICATION_IDLE) then
        return "PROMPT_FLOURISH_NOTIFICATION_IDLE"
    elseif (reason_int == PROMPT_FLOURISH_INTRO_RESTART) then
        return "PROMPT_FLOURISH_INTRO_RESTART"
    elseif (reason_int == PROMPT_FLOURISH_NOTIFICATION) then
        return "PROMPT_FLOURISH_NOTIFICATION"
    elseif (reason_int == PROMPT_FLOURISH_INTRO_START_IDLE) then
        return "PROMPT_FLOURISH_INTRO_START_IDLE"
    end

    return "Unknown"
end

Context.bypassIdle = function()
    return Context._getStartReason() == START_REASON_TEST or Context._getStartReason() == START_REASON_NOTIFICATION
end

Context.isBrowserPlay = function()
    return Context._getStartReason() == START_REASON_BROWSER
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Initialize persistent state
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

Persist = Persist or {}
Persist.episode3 = Persist.episode3 or {}
Persist.episode3.context = Persist.episode3.context or {}

-- If we're being reset, clear all persistent context state
if (Context._getStartReason() == START_REASON_RESET) then
    Persist.episode3.context = {}
end

-- If we have upgraded to a new major, minor or build version, clear persist state
if (Context._getStartReason() ~= START_REASON_TEST) then
    if ((Persist.episode3.context.lastPlatformVersionMajor == nil) or (Persist.episode3.context.lastPlatformVersionMajor ~= Platform.versionMajor) or
        (Persist.episode3.context.lastPlatformVersionMinor == nil) or (Persist.episode3.context.lastPlatformVersionMinor ~= Platform.versionMinor) or
        (Persist.episode3.context.lastPlatformVersionBuild == nil) or (Persist.episode3.context.lastPlatformVersionBuild ~= Platform.versionBuild)) then
        Context.onVersionUpdate()
    end
end

Persist.episode3.context.lastPlatformVersionMajor = Platform.versionMajor
Persist.episode3.context.lastPlatformVersionMinor = Platform.versionMinor
Persist.episode3.context.lastPlatformVersionBuild = Platform.versionBuild
Persist.episode3.context.lastPlatformVersionPatch = Platform.versionPatch

-- Detect if the previous run exited due to reboot, battery drain, crash, etc
if (Persist.episode3.context.abnormalExit) then
    Persist.episode3.context.stopReason = STOP_REASON_ABNORMAL
end
Persist.episode3.context.abnormalExit = true

-- Debug info
Persist.episode3.context.lastStopReason = Context._getStopReason()
Persist.episode3.context.lastStopReasonStr = Context._getStopReasonString()
Persist.episode3.context.lastStartReason = Context._getStartReason()
Persist.episode3.context.lastStartReasonStr = Context._getStartReasonString()

-- Flush to disk
System.flushPersist()
