
Launcher.idle = {
    -- where to transition following a screen cycle or notification click
    nextState = nil,

    -- pass true to trigger off notification, false to trigger off screen cycle
    idleOnNotification = false,

    -- whether we've seen a screen on event
    screenCycled = false,

    -- whether we've seen a notification click
    notificationClicked = false,

    onStartup = function(self)
        Android.reset()

        -- init event flags
        self.screenCycled = false
        self.notificationClicked = false

        -- tick just once to (possibly) transition to load state
        Scheduler.cancel()
        Scheduler.requestFrame()

        if (Avatar.setWatchdogPaused) then
            Avatar.setWatchdogPaused(true)
        end

        return true
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onUpdate = function(self)
        if (not Platform.android or Platform.iOS) then
            return self.nextState
        end

        -- if we are in a blacklisted state, go to sleep
        -- we will get ticked upon events such as touches
        if (Context.isBlacklisted()) then
            return
        end

        if ((not self.idleOnNotification and self.screenCycled) or (self.idleOnNotification and self.notificationClicked)) then
            return self.nextState
        end
    end,

    onRender = function(self)
    end,

    onShutdown = function(self)
        if (Avatar.setWatchdogPaused) then
            Avatar.setWatchdogPaused(false)
        end
    end,

    onEvent = function(self, event)
        if (event.type == "onScreenEvent") then
            if (event.data.newState == 1) then
                -- the screen turned on, so it must have cycled
                self.screenCycled = true
            end
        elseif (event.type == "onActivityEvent") then
            if (event.data.eventString == "onStartShow") then
                Avatar.cancelNotification()
                self.notificationClicked = true
            end
        end

        if (self.screenCycled or self.notificationClicked) then
            Scheduler.requestFrame()
        end
    end
}
