
local launcherResourceGroup = System.createResourceGroup("LauncherResources")

Launcher.load1 = {
    -- whether or not we have already passed through this state
    loaded = false,

    -- next state after loading completes
    nextState = nil,

    -- whether or not to wait for a screen cycle
    doIdle = false,
    doIdleOnNotification = false,

    onStartup = function(self)

        Android.reset()

        -- choose the next state
        self.nextState = nil
        self.doIdle = false
        self.doIdleOnNotification = false
        
        local prompt = Context.getPromptType()

        if (prompt == PROMPT_FLOURISH_NOTIFICATION) then
            self.nextState = "launcher_notify"
        elseif (prompt == PROMPT_FLOURISH_NOTIFICATION_IDLE) then
            self.nextState = "launcher_notify"
            self.doIdle = true
        elseif (not Context.bypassIdle() and (prompt == PROMPT_FLOURISH_INTRO_START_IDLE or prompt == PROMPT_FLOURISH_INTRO_MIDDLE_IDLE)) then
            self.nextState = "launcher_load2"
            self.doIdle = true
            self.doIdleOnNotification = true
        else
            self.nextState = "launcher_load2"
        end
        
        -- Initialize the scene & camera and start prefetching of the hat assets
        self:prefetch()

        -- tick continuously
        return true
    end,

    onUpdate = function(self)
        -- go to next state when loading completes
        if (self.loaded or ResourceLoader:prefetchComplete()) then
            self:setupScene()
            self.loaded = true

            if (self.doIdle) then
                Launcher.idle.nextState = self.nextState
                Launcher.idle.idleOnNotification = self.doIdleOnNotification
                return "launcher_idle"
            else
                return self.nextState
            end
        end
    end,

    onRender = function(self)
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    prefetch = function(self)
        if (self.loaded) then return end

        if StoryManager and StoryManager.pauseStory then
            StoryManager.pauseStory()
        end

        launcherResourceGroup:addActor(LaunchData.actors.LauncherToNotification)
        launcherResourceGroup:addActor(LaunchData.actors.GoodWayIn)
        launcherResourceGroup:addActor(LaunchData.actors.GWI_Title)
        launcherResourceGroup:addFlipbook("episode3:flipbooks/title.mfb")
        launcherResourceGroup:addAudio("episode3:audio/mxloop_intro.ogg")
        launcherResourceGroup:prefetch()
    end,

    setupScene = function(self)
        if (self.loaded) then return end

        local goodWayIn = ActorManager:getActor("GoodWayIn", true)
        Global.scene.screenSpace:addChild(goodWayIn)
        -- Fake a current "segment" for the animation and play it
		
        -- Assets used only for GWI
        local gwiTitle = Global.scene:createActor("GWI_Title", LaunchData.actors.GWI_Title)
        gwiTitle:addComponent("DuetLogo_Driver", SimpleFBDriver.new("episode3:flipbooks/title.mfb", gwiTitle:findMaterial("Title_Mat"), true))
        gwiTitle:addComponent("TitleAnimator", TitleAnimator.new(1.1, 2.2))
        Global.scene.screenSpace:addChild(gwiTitle)

        launcherResourceGroup:destroy()
    end
}
