
Launcher.load2 = {
    -- true to bypass screen idle for the initial prompt
    skipFirstIdle = (System.getEnv == nil) or Platform.win32 or Platform.mac or 
                    (System.getEnv("DevMode") == "true"),

    -- whether or not we have already passed through this state
    loaded = false,

    -- next state after loading completes
    nextState = nil,

    -- whether or not to wait for a screen cycle
    doIdle = false,

    onStartup = function(self)

        Android.reset()

        if StoryManager and StoryManager.resumeStory then
            StoryManager.resumeStory()
        end

        -- choose the next state
        self.nextState = nil
        self.doIdle = false
        self.finishedIntro = false
        self.canBeInterrupted = false

        local prompt = Context.getPromptType()

        if (prompt == PROMPT_GWI) then
            self.nextState = "launcher_prompt"
            self.doIdle = not self.skipFirstIdle
        elseif (prompt == PROMPT_UNHIDE) then
            self.nextState = "launcher_fast_prompt"
            self.doIdle = true
        elseif (prompt == PROMPT_FLOURISH_INTRO_RESTART
            or prompt == PROMPT_FLOURISH_INTRO_MIDDLE_IDLE
            or prompt == PROMPT_FLOURISH_INTRO_START_IDLE) then

            self:prefetch()
            Android.acquireScreen()
            InputManager:enableSensors(true)
            -- Play title animation when show is resumed from checkpoint
            Global.scene.actors.GWI_Title.components.TitleAnimator:playLogoIntro(true)
            self.nextState = "launcher_intro"
            self.finishedIntro = false
            self.canBeInterrupted = true

            if (prompt == PROMPT_FLOURISH_INTRO_RESTART) then
                Avatar.cancelNotification()
                Context.onCheckpoint("")
            end
        else
            Log.fatal("Reached launcher_load2 in inapplicable scenario")
        end

        self.interrupted = Context.isBlacklisted()
        self.exiting = false
        self.skipFirstIdle = false

        -- tick continuously
        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()

        if (self.exiting) then
            -- we are done loading and moving that bug off screen
            self.loaded = true

            -- go to next state
            if (self.doIdle) then
                Launcher.idle.nextState = self.nextState
                Launcher.idle.idleOnNotification = false
                Launcher.idle.idleOnScreenCycle = false
                Launcher.idle.idleOnGWI = true
                return "launcher_idle"
            else
                return self.nextState
            end
        elseif (self.finishedIntro) then
            self.exiting = true
        end

        if (self.canBeInterrupted and (self.interrupted or Global.shutdown)) then
            return "launcher_interrupted"
        end
		
        if (self.loaded or ResourceLoader:prefetchComplete()) then
            self.finishedIntro = true
        end
		
    end,

    onRender = function(self)
        if (not (Android.isPaused() or self.interrupted)) then
            Global.onRender()
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onShutdown = function(self)
    end,

    onEvent = function(self, event)
        if(event.type == "onActivityEvent") then
            if (event.data.eventString == "onDummyActivityGainedFocus") then
                Android.onResume()
            elseif (event.data.eventString == "onDummyActivityLostFocus") then
                Android.onPause()
            elseif (event.data.eventString == "onStop") then
                self.interrupted = true
                Android.onResume()
            end
        elseif(event.type == "onChangeDisplaySize") then
            Global.setRenderOverTSB(false)
        end
    end,

    prefetch = function(self)
        local firstSegmentName = Context.getCheckpoint() or SceneData.startSegment
        if Platform.segment ~= nil and Platform.segment ~= "" then
            firstSegmentName = Platform.segment
        end

        if not Platform.scene or Platform.scene == "" then
            ResourceLoader:prefetchInitialSegments()
        end
        ResourceLoader:prefetchSegment(firstSegmentName)
    end,

}
