

OmniLightBlendManager = {
	
	activeLightsMap = {}; -- maps a target (ie 'mouse') to its light data
	exclusivityMap = {}; -- map of targets inside a light probe (deprecated)
	
	litHistory = {}; -- keeps track of any lit objects, object that have had a material change
	
	matTargetToExclusiveLight = {}; --new Dictionary<GameObject, OmniLightProbe>();
	
	numberOfEffectiveLights = 2;
	maxEffectiveLight = 4;
	lightColorUniforms = "_AmbientLightColor";
	
	omniLightWeights = Vector.new(0,0,0,0);
	omniLightColors = Matrix.identity;
	
	scene = nil;
	
	probeList = {};
	activeProbes = {};
}

local compare = function(a, b)
	if(a.hierarchyDepth > b.hierarchyDepth) then
		return true;
	end
	
	return false;
end

local comp = {}
comp[1] = "x"
comp[2] = "y"
comp[3] = "z"
comp[4] = "w"


function OmniLightBlendManager:init(scene)
	self.scene = scene;
	--[[
	local sortedList = {}
	for key, probeData in pairs(SceneData.lights) do
		
		if(self.probeList[probeData.omnilight.setName] == nil) then
			self.probeList[probeData.omnilight.setName] = {}
		end
		table.insert(self.probeList[probeData.omnilight.setName], probeData.omnilight)
	end
	]]
	
end

-- new method for creating light probes
function OmniLightBlendManager:createOmniLight(light)

	if(light == nil) then return end

	self.probeList[light.omnilight.name] = light.omnilight;
	
	-- put light in the scene
	if(self.probeList[light.omnilight.name] ~= nil) then
		
		Log.message("CREATED OMNI LIGHT FOR: "..light.omnilight.setName)
		
		local transform = self.scene:createTransform(light.omnilight.name);
		self.activeProbes[light.omnilight.name] = transform:addComponent(light.omnilight.name, OmniLightProbe:new(light.omnilight))
		--parent.addAttachment(parent, light.omnilight.bonePath, transform)
		
		--[[
		-- create debug sphere
		local debugSphereName = "DebugSphere_"..light.omnilight.name;
		SceneData.actors[debugSphereName] = shallowcopy(SceneData.actors.DebugSphere);
		local debugSphereActor = ActorManager:getActor(debugSphereName, true);
		debugSphereActor:setVisibility(true);
		local localT = debugSphereActor:getLocal();
		localT.scale = Vector(1, 1, 1)
		
		local posActor = ActorManager:getActor(light.omnilight.setName);
		localT.position = posActor:getBone(light.omnilight.bonePath):getWorld().position;
		
		debugSphereActor:setLocal(localT);]]
	end
end

-- new method for destroying light probes
function OmniLightBlendManager:destroyOmniLight(light)

	if(light == nil) then return end
	
	if(self.probeList[light.omnilight.name] ~= nil) then
		if(self.probeList[light.omnilight.name].destroy ~= nil) then
			self.probeList[light.omnilight.name]:destroy()
		end
		self.probeList[light.omnilight.name] = nil
	end
	
	if(self.activeProbes[light.omnilight.name] ~= nil)then
		self.activeProbes[light.omnilight.name] = nil
	end

end
	
function OmniLightBlendManager:onStart ()
	self.numberOfEffectiveLights = math.min(self.numberOfEffectiveLights, self.maxEffectiveLight)
	self:clearWeights();
end
	
function OmniLightBlendManager:clearWeights()
	self.omniLightWeights.x = 0
	self.omniLightWeights.y = 0
	self.omniLightWeights.z = 0
	self.omniLightWeights.w = 0
end
	
function OmniLightBlendManager:clearColor()
	self.omniLightColors = Matrix.new(1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1)
end
	
function OmniLightBlendManager:update()

		-- reset any changes
		for key, material in pairs(self.litHistory) do
			material:setParameters({u_AmbientLightColor = Vector.one});
		end
		
		-- dump history now
		self.litHistory = {}
		
		self:clearWeights();
		self:clearColor();
		
		
		for key, value in pairs(self.activeLightsMap) do
			
			local targetData = value.data;
			local materialTarget = targetData.lightTargetMat;
			local activeLights = value.list;
			local inverseWeight = Vector.zero:copy()
			local numLights = math.min(table.getn(activeLights), self.numberOfEffectiveLights);
			
			-- setup blend weights
			for i = 1, numLights, 1 do
			
				local cutoff2 			 = activeLights[i].cutOffDist;
				cutoff2 		    	 = cutoff2*cutoff2; -- squared cutoff distance
				local dist2 			 = activeLights[i]:findTargetInThisProbe(targetData).currentDistanceSqrd;
				self.omniLightWeights[comp[i]] = math.clamp(1.0 - dist2/cutoff2, 0.0, 1.0)
				inverseWeight[comp[i]] = 1 - self.omniLightWeights[comp[i]];
				
				-- setup inverted colors
				self.omniLightColors:setColumn(i, activeLights[i]:getInvertedLightProbeColor());
				
			end
			
			if(materialTarget ~= nil) then
			
				--Log.message("LIGHT="..materialTarget.name.." WEIGHTS: x ="..self.omniLightWeights.x.." y ="..self.omniLightWeights.y.." z ="..self.omniLightWeights.z.." w ="..self.omniLightWeights.w)
				
				-- multiply inverted colors by weights and then 'un'-invert
				local color = (Vector.one - self.omniLightColors:mulVec(self.omniLightWeights));
				
				color.x = math.clamp01(color.x);
				color.y = math.clamp01(color.y);
				color.z = math.clamp01(color.z);
				color.w = 1;
				--Log.message("Blended Color: r = "..color.x.." g = "..color.y.." b = "..color.z.." a = "..color.w)
				materialTarget:setParameters({ u_AmbientLightColor = color});
				
				-- since we are changing this material keep track of it
				if(not self.litHistory[materialTarget.name]) then
					self.litHistory[materialTarget.name] = materialTarget;
				end
			end
		end
		
		self.activeLightsMap = {}
		self.exclusivityMap = {}
		self.matTargetToExclusiveLight = {}
end
	
-- deprecated
function OmniLightBlendManager:inVolume(target, probe)
		
		local targetName = target.hashname
		if( self.exclusivityMap[targetName] ~= nil and (self.exclusivityMap[targetName]:findTargetInThisProbe(targetName).currentDistanceSqrd > target.currentDistanceSqrd)) then
			self.exclusivityMap[targetName] = probe;
			self.matTargetToExclusiveLight[target.lightTargetMat.name] = probe;
		end
		
end
	
function OmniLightBlendManager:addProbe(target, probe)

	local activeLights = self.activeLightsMap[target.hashname]; -- list of light probes
	
	if(activeLights == nil) then
		activeLights = {list={}, data=nil};
		self.activeLightsMap[target.hashname] = activeLights;
		activeLights.data = target;
	end
	
	table.insert(activeLights.list, probe);
	
	-- insert lights so the nearest light is at the top of the list
	
	-- insertion sort
	local swap = nil;
	local compareProbe = nil;
	local activeLightCount = table.getn(activeLights.list);
	local probeTestIndex = activeLightCount - 1;
	for i = activeLightCount - 1, 1, -1 do
		compareProbe = activeLights.list[i];
		
		if(target.currentDistanceSqrd < compareProbe:findTargetInThisProbe(target).currentDistanceSqrd) then
			
			-- save old
			swap = activeLights.list[i];
			
			-- replace with new
			activeLights.list[i] = activeLights.list[probeTestIndex];
			
			-- put old in new's previous slot
			activeLights.list[probeTestIndex] = swap;
			
			-- update test index
			probeTestIndex = i;
			
		else
			i = 1-1; -- break
		end
	end
end
	
function OmniLightBlendManager:reset(target, probe)
	
	if(target.lightTargetMat ~= nil) then
		local material = target.lightTargetMat;
		--material.setProperty(lightColorUniforms, Vector.one);
		material:setParameters({u_AmbientLightColor = Vector.one});
	end
	
end

function OmniLightBlendManager:shutdown()
	self.activeLightsMap = {}; 				
	self.exclusivityMap = {}; 				
	self.litHistory = {}; 					
	self.matTargetToExclusiveLight = {}; 	
	self.scene = nil;
	self.probeList = {};
	self.activeProbes = {};
end
