
local techniqueToFlipbookSampler = {
    flipbookVideoColor = {"u_FlipbookVideoTex"},
    flipbookVideoColorAlpha = {"u_FlipbookVideoTex", "u_FlipbookVideoAlpha"},
    flipbookColor = {"u_FlipbookTex"},
    flipbookColorAlpha = {"u_FlipbookTex"},
}

local function ProcessActorMaterial(actor, materialName, actorData)
	Log.message("Processing actor: "..actor.name)
    if actorData.dumpMaterials then Log.message("\t"..materialName) end

    -- grab the material already in the actor definition if it exists
    local matData = {}
    local index = nil
    if actorData.materials then
        index = table.find_index(actorData.materials, function(self) return self.name == materialName; end );
        matData = index and actorData.materials[index] or {}
    end

    local matInstance = actor:findMaterial(materialName)
    local technique = actorData.overrideFlipbookTechnique or matInstance:getTechniqueName()
    
    if(actorData.overrideFlipbookTechnique) then
    	technique = actorData.overrideFlipbookTechnique
    end
    
    if(not matData.dontOverrideTechnique) then
		if(technique ~= "default") then
			matData.technique = matData.technique or technique
	
			Log.message("---------------------- >> Technique for "..materialName.." = "..matData.technique)
		end
	end

    -- process material attributes
    local fbMaterial = false
    local props = matInstance:getPropertiesTable()
    for key, value in pairs(props) do
        if actorData.dumpMaterials then Log.message("\t\tproperty:"..tostring(key).."="..tostring(value)) end
    end
	
    matData.renderGroup = matData.renderGroup or props["renderGroup"]
    
     -- fbStart for flipbook frame offsets
    if props["fbStart"] and actorData.fbdrivers then
        fbMaterial = true
        for _, fbDriver in pairs(actorData.fbdrivers) do
        	
        	if(actorData.generateFlipbooks == false) then
        		fbDriver[materialName] = fbDriver[materialName]
        	else
            	fbDriver[materialName] 				= fbDriver[materialName] or {}
            end
			
			if(fbDriver[materialName]) then
                -- we're setting frameOffset to be the inverse of fbStart
                fbDriver[materialName].frameOffset 	= fbDriver[materialName].frameOffset or -(props["fbStart"] or 1) + 1
                fbDriver[materialName].fbStart = props.fbStart
                
                -- setup looping
                if(props["loopFlipbook"]) then  props["loopFlipbook"]=FBWrapMode.loop else  props["loopFlipbook"]=FBWrapMode.clamp end
                
                fbDriver[materialName].wrapMode = fbDriver[materialName].wrapMode or props["loopFlipbook"]
                fbDriver[materialName].frameStart = fbDriver[materialName].frameStart or props["loopStart"]
                fbDriver[materialName].frameEnd = fbDriver[materialName].frameEnd or props["loopEnd"]
                
                local exposureCount = fbDriver[materialName].exposureCount or 1
                if(fbDriver[materialName].frameStart) then
                	fbDriver[materialName].frameStart = math.max(0, fbDriver[materialName].frameStart-1)*exposureCount
                end
                if(fbDriver[materialName].frameEnd) then
                	fbDriver[materialName].frameEnd = math.max(0, fbDriver[materialName].frameEnd-1)*exposureCount
                end
            end
        end
    end
    
    -- process material textures
    local texCount = matInstance:getTextureCount()
    for i = 0, texCount - 1, 1 do
    
        local texName 	= matInstance:getTextureName(i)
		local texType	= matInstance:getTextureType(i)
        local texUri 	= "Invalid URI"
		
		if(texType == MTT_TEXTURE) then
			texUri = string.lower(matInstance:getTextureUri(i))
		elseif(texType == MTT_FLIPBOOK) then	
			texUri = string.lower(matInstance:getFlipbookUri(i))
		else
			Log.message(">>>> BAD Texture URI = "..texUri)
		end

        local set 		= index and matData.textures and matData.textures[texName] -- if the texture is overridden already

        if(		not set and
                not waveMaterial and
                actorData.defaultMaterial and
                actorData.defaultMaterial.textures and
                actorData.defaultMaterial.textures[texName]) then
            matData.textures[texName] = actorData.defaultMaterial.textures[texName]
            set			= true
        end

		local index = i > 0 and i or ""
        local samplerName = (texName == "u_diffuseMap") and {"u_MainTex"} or {"u_MainTex"..index}

        if not set and not waveMaterial then
            if(texType == MTT_TEXTURE) then
                matData.textures = matData.textures or {}
                matData.textures[samplerName[1]] = string.lower(texUri)
            elseif(texType == MTT_FLIPBOOK) then
                matData.flipbooks = matData.flipbooks or {}
				samplerName = techniqueToFlipbookSampler[technique] or {"u_FlipbookTex"}
				for _, sampler in ipairs(samplerName) do
                	matData.flipbooks[sampler] = string.lower(texUri)   
				end           
            end
        end

        if(fbMaterial and actorData.fbdrivers) then
			local fbUri = texUri
            for fbAnim, fbMatTable in pairs(actorData.fbdrivers) do
                if(fbMatTable[materialName]) then
	                fbMatTable[materialName].uri 		= fbMatTable[materialName].uri or fbUri
	                fbMatTable[materialName].bindings 	= fbMatTable[materialName].bindings or   techniqueToFlipbookSampler[technique]  or {"u_FlipbookTex",}
	
	                if actorData.dumpMaterials then Log.message("\t\tflipBook["..fbAnim.."]["..materialName.."]={uri='"..fbUri.."',frameOffset="..fbMatTable[materialName].frameOffset.."}") end
                end
            end
        end

        if actorData.dumpMaterials then
            Log.message("\t\ttexName["..i.."]: "..texName);
            Log.message("\t\ttexUri["..i.."]: "..texUri);

			for _, sampler in ipairs(samplerName) do
				Log.message("\t\tmaterial.textures["..sampler.."]="..texUri);
			end
        end
    end

    -- if the material is not in the actor data already, add it
    if not index then
        actorData.materials = actorData.materials or {}
        matData.name = materialName
        table.insert(actorData.materials, matData)
    end
end

-- Cleanup and fill out the actorData material from defaults
local function ProcessActorDataMaterial(actor, material, actorData)
    -- copy any properties from the default actor material that were not overridden
    if actorData.defaultMaterial then
        for name, value in pairs(actorData.defaultMaterial) do
            material[name] = material[name] or value
        end
    end

    -- set some global shader defaults in case they weren't set yet
    if (not material.parameters or (material.parameters and not material.parameters.u_MainTex_ST)) then
        material.parameters = material.parameters or {}        
        material.parameters.u_MainTex_ST = {1.00,1.00,0.00,0.00}
    end

    -- Flipbook shaders have a u_Alpha uniform. This is set to 1 by default
    if (not material.parameters or (material.parameters and not material.parameters[techniqueToFlipbookSampler[technique]])) then
        material.parameters = material.parameters or {}
        material.parameters["u_Alpha"] = 1
    end

    -- make sure all asset paths are lower-case
    if material.textures then
        for tex, path in pairs(material.textures) do
            material.textures[tex] = string.lower(path)
        end
    end
	
--	REG: this is used for debugging purposes	
--	if(material) then
--		material.technique = "mainDuetDbgLine"
--	end	
end

-- Look up all named bones in the actor and add a path entry to it with
-- the full bone path for each one
local function FindNamedBones(actor, actorData)
    if actorData.namedBones then
        for name, leafBoneName in pairs(actorData.namedBones) do
            local pathKey = name .. "Path"
            actor[pathKey] = actor:findBonePath(leafBoneName)
            if not actor[pathKey] and name == "targetBone" then
                assert(false, "Unable to find targetBone: " .. leafBoneName .. " for actor: " .. actor.name)
            end
            actor[name] = actor:getBone(actor[pathKey])
        end
    end
end


--[[
  Extract material and other data from a freshly loaded actor
  model to fill out and configure the actorData.
]]
local useBakedData = true
function PostProcessActorData(actor, actorData)
--    local startTime = os.clock()

    if not useBakedData then
        -- import all the materials to the actor data so we can override their properties
        if Platform.candidate or Platform.profile then
            actorData.dumpMaterials = false
            actorData.dumpBones = false
        end
        local materialNames = actor:listMaterials()
        if materialNames then
            for _, matName in ipairs(materialNames) do
                ProcessActorMaterial(actor, matName, actorData)
            end
        end

        -- cleanup the actorData's material info
        if actorData.materials then
            for _, matData in ipairs(actorData.materials) do
                ProcessActorDataMaterial(actor, matData, actorData)
            end
        end

        FindNamedBones(actor, actorData)
    else
        local bakedData = SceneData.bakedActorData and SceneData.bakedActorData[actor.name] or {}
        if bakedData then
            if bakedData.materials then
                actorData.materials = bakedData.materials
            end

            if bakedData.fbdrivers then
                actorData.fbdrivers = bakedData.fbdrivers
            end

            if bakedData.attachTo then
                actorData.attachTo = bakedData.attachTo
            end

            if bakedData.bonePaths then
                for name, path in pairs(bakedData.bonePaths) do
                    actor[name .. "Path"] = path
                    actor[name] = actor:getBone(path)
                end
            end

            -- Frame Events are not currently being used, but may be useful in the future
    --        if bakedData.frameEvents then
    --            actor.frameEvents = bakedData.frameEvents
    --        end
        end
    end

--    local postTime = (os.clock() - startTime) * 1000
--    Log.message("Time to post-process actor " .. actor.name .. ": " .. string.format("%.2f", postTime) .. "ms")
end
