
SceneData = SceneData or {}
SceneData.actors = SceneData.actors or {}
SceneData.segments = SceneData.segments or {}


-- Update any fbDrivers' frameOffset to match the fbStart property from its material
local FixFlipbookFrameOffsets = function(actor)
    if actor.fbdrivers then
        for name, driver in pairs(actor.fbdrivers) do
            for matName, driver in pairs(driver) do
                local material = actor.materials[matName]
                if material then
                    local props = material:getPropertiesTable()
                    for key,value in pairs(props) do 
                        Log.message("material("..name.."):"..tostring(key).."="..tostring(value)) 
                    end

                    if props.fbStart then
                        Log.message("Setting frame offset for " .. name .. "." .. matName .. " to " .. tostring(props.fbStart))
                        driver.frameOffset = - (props.fbStart-1)
                    end
                end
            end
        end
    end
end

-- Update the material's render group to match that specified in the properties for the material
local FixMaterialRenderGroups = function(actor)
    if actor.materials then
        for name, material in pairs(actor.materials) do
            local props = material:getPropertiesTable()
            if props.renderGroup then
                Log.message("Setting render group on " .. name .. " to " .. tostring(props.renderGroup))
                material:setRenderGroup(props.renderGroup)
            end
        end
    end
end

local function anim(animDef, animFileName)
    return {
        -- name, startFrame, endFrame, looping
        "episode3:anims/" .. animFileName .. ".mxa", animDef.frames[1], animDef.frames[2], animDef.looping,
    }
end

local function animations(actorName, segments)
    local anims = {}
    for _, segment in ipairs(segments) do
        for animName, animData in pairs(segment.anims) do
            anims[animName] = anim(animData, actorName .. "_" .. segment.name)
        end
    end
    return anims
end

SceneData.actors.DebugSphere = {
    model = "episode3:models/unitsphere.mxm",
    materials = {
        {
            name = "Mat_Sphere",
            shader = "episode3:shaders/unlit.rsfx",
            technique = "textureColorAlpha",
            renderGroup = 7,
            textures ={
            
            },
            parameters =
            {
                u_Color = Color.new( 0, .45, 0, 0.25 ),
            }
        },
    },
}
--[[
SceneData.actors.DebugAxis = {
    model = "episode3:models/axis.mxm",
    materials = {
        { 
            name = "Mat_Axis", shader = "episode3:shaders/launcher.rsfx",
            technique = "vertexcolor",
            renderGroup = 7,
            textures = { },
            parameters = { u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, 
        },
    },
}

SceneData.actors.ScreenQuad = {
    model = "episode3:models/screenquad.mxm",
    materials = {
        { 
            name = "Mat_Quad", shader = "episode3:shaders/launcher.rsfx",
            technique = "texture",
            renderGroup = 3,
            textures = { u_MainTex = "episode3:textures/dogtest/dog_run_test.00001.dds" },
            parameters = { u_Color = Color.new( 1, 1, 1, 1 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, 
        },
    },
}
--]]
-- Add common properties to all of the actors
for _, actor in pairs(SceneData.actors) do

    -- Turn on debug info
    if actor.model then
        if actor.dumpBones == nil then
            actor.dumpBones = true
        end
        if actor.dumpMaterials == nil then
            actor.dumpMaterials = true;
        end
        if(actor.disableInterp == nil) then
            actor.disableInterp = true;
        end
    end

    -- Add post process functions
    actor.postProcess = actor.postProcess or {}
end

