
SceneData = SceneData or {}
SceneData.segments = SceneData.segments or {}
SceneData.actors = SceneData.actors or {}
SceneData.sets = SceneData.sets or {}

SceneData.defaultFrameRate = 60

if (Persist.episode3 and Persist.episode3.lastStartSegment and Persist.episode3.lastStartSegment == "Cell_Girl") then
    SceneData.startSegment = "Cell_Boy"
else 
    SceneData.startSegment = "Cell_Girl"
end

-- List of segments that we want to keep around even though they may appear to be orphaned
--SceneData.keepSegments ={ "Intro", "Star", "Star_Revisit", "End_Sky", "Profile", }

-- List of actors that we want to keep around even though they may appear to be orphaned
SceneData.keepActors = {
    "BoyProxy", "GirlProxy", "DogProxy", "DebugSphere", "DebugAxisCopy",
    "GlowPlane", "HorizonGlow", "Vignette", "Butterfly", "Show_Sphere",
}

-- Manually used fbdrivers on the following actors so don't report them in the orphan list
SceneData.keepFBDriversFor = {
    "BoyTreeLeaves", "BoyTreeTree", "Gazebo", "GirlTreeLeaves", "GirlTreeTree",
}

-- Default shader for drawing flipbook art.
SceneData.defaultMatDef = 
{    
    shader      = "episode3:shaders/unlit.rsfx",
    technique   = "mainDuet",
    parameters  = {u_Color = Vector(1,1,1,1)},
}

SceneData.flipbookMatDef = 
{    
    shader      = "episode3:shaders/unlit.rsfx",
    technique   = "flipbookColorAlpha",
    parameters  = {u_Color = Vector(1,1,1,1), u_Alpha = 1, },
}

SceneData.flipbookBackgroundMatDef = 
{    
    shader      = "episode3:shaders/unlit.rsfx",
    technique   = "flipbookColorMultiTexture",
    parameters  = {u_Color = Vector(1,1,1,1), u_Alpha = 1, },
}

SceneData.nothingMatDef = 
{    
    shader      = "episode3:shaders/unlit.rsfx",
    technique   = "nothing",
}

-- Global resources that are used in ever segment of the show
-- they will not be released until the show shuts down
SceneData.globalResources = 
{
    -- TODO: the system should be prefetching these instead of the app
    { Shader.rtti,  "common:shaders/simple.rsfx" },
    { Shader.rtti,  "common:shaders/particle.rsfx" },
    { Texture.rtti, "common:textures/missingtexture.dds" },
    { Texture.rtti, "common:textures/gui/ldpi/guielements.dds" },
    { Texture.rtti, "common:textures/gui/ldpi/gear.dds" },

    -- Show assets
    { Texture.rtti, "episode3:textures/white.dds" },
    { Model.rtti,   "episode3:models/screenquad.mxm" },
    { Shader.rtti,  "episode3:shaders/launcher.rsfx" },
    { Shader.rtti,  "episode3:shaders/unlit.rsfx" },

    -- Hand code the particle systems for now
    { ParticleSystem.rtti, "episode3:particles/fallingtreeleavesback.mxp"},
    { ParticleSystem.rtti, "episode3:particles/fallingtreeleavesfront.mxp"},
    { ParticleSystem.rtti, "episode3:particles/groundparticles.mxp"},
    { ParticleSystem.rtti, "episode3:particles/treeparticles.mxp"},
    { Texture.rtti, "episode3:textures/particle_leaves.dds" },
    { Texture.rtti, "episode3:textures/particle_c.dds" },
    { Texture.rtti, "episode3:textures/particle_tree.dds" },
}
SceneData.ShowStartActors = { "GlowPlane", "Show_Sphere", "Vignette", "Butterfly", }

-----------------------------------------------------------------------------------------------------------
-----------------------------------------------------------------------------------------------------------
---------------------- ProxyActors -------- -------------------
-- persistent tracking objects stay alive between segments - used to track a single actor through the show
SceneData.actors.GirlProxy =
{
    hideDebug = true,
    components      = 
    {
        { name      = "TrackingTarget", params = { proxyActorName = "girl", bone = "targetBone", useBounds = true, useTracking = true, debug = false} },
    },
}
SceneData.actors.BoyProxy = 
{
    hideDebug = true,
    components      = 
    {
        { name      = "TrackingTarget", params = { proxyActorName = "boy", bone = "targetBone", useBounds = true, useTracking = true, debug = false} },
    }
}
SceneData.actors.DogProxy =
{
    hideDebug = true,
    components      =
    {
        { name      = "TrackingTarget", params = { proxyActorName = "dog", bone = "targetBone", useBounds = false, useTracking = false, debug = false} },
    }
}

-----------------------------------------------------------------------------------------------------------
----------------------Star : Characters---------------------------
SceneData.actors.Origin = 
{
    position        = { 0,0,0},    
    animation       = {},
    defaultMaterial = SceneData.defaultMatDef,
}


SceneData.actors.OpeningSphere = 
{
    model       = "episode3:models/opening_particles_new.mxm",
	
--    overrideFlipbookTechnique = "flipbookColorMultiTexture",
	
    components = 
    {
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 0,
                blending = false,
                segmentData = 
                {
                    { 
                        segment = "Cell_Girl",
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                        fadetime = 1,
                        delay = 6.22,
                    },
                    { 
                        segment = "Cell_Boy",
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                        fadetime = 1,
                        delay = 6.22,
                    },
                    { 
                        segment = "Birth_Girl",
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out", 
                    },
                    { 
                        segment = "Birth_Boy",
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out", 
                    },
                },
            },
        },     
    }, 

    animation   = 
    {
        Cell_Girl          = { "episode3:models/opening_particles_new.mxa", 1, 1073, false, true},
        Cell_Boy           = { "episode3:models/opening_particles_new.mxa", 1, 1073, false, true},
    },  

    fbdrivers = 
    {
        Cell_Boy           = {
            MG_Particles_Mat = {
                bindings = {
                    "u_FlipbookTex"
                },
                frameEnd = 200,
                frameStart = 0,
                frameTranslate = -378,
                loopCountGoal = -1,
                exposureCount = 6,
                luaDriven = true,
                wrapMode = FBWrapMode.clamp,
                uri = "episode3:flipbooks/mg_particles.mfb"
            },
            FG_Particles_Mat = {
                bindings = {
                    "u_FlipbookTex"
                },
                frameEnd = 200,
                frameStart = 0,
                frameTranslate = -378,
                loopCountGoal = -1,
                exposureCount = 6,
                luaDriven = true,
                wrapMode = FBWrapMode.clamp,
                uri = "episode3:flipbooks/fg_particles.mfb"
            },
            BG_environment_Blur_Mat = {
                bindings = {
                    "u_FlipbookTex"
                },
                frameEnd = 200,
                frameStart = 0,
                frameTranslate = -378,
                loopCountGoal = -1,
                exposureCount = 6,
                luaDriven = true,
                wrapMode = FBWrapMode.clamp,
                uri = "episode3:flipbooks/bg_environment_blur.mfb"
            },
        },
        Cell_Girl = {
            MG_Particles_Mat = {
                bindings = {
                    "u_FlipbookTex"
                },
                frameEnd = 200,
                frameStart = 0,
                frameTranslate = -378,
                loopCountGoal = -1,
                exposureCount = 6,
                luaDriven = true,
                wrapMode = FBWrapMode.clamp,
                uri = "episode3:flipbooks/mg_particles.mfb"
            },
            FG_Particles_Mat = {
                bindings = {
                    "u_FlipbookTex"
                },
                frameEnd = 200,
                frameStart = 0,
                frameTranslate = -378,
                loopCountGoal = -1,
                exposureCount = 6,
                luaDriven = true,
                wrapMode = FBWrapMode.clamp,
                uri = "episode3:flipbooks/fg_particles.mfb"
            },
            BG_environment_Blur_Mat = {
                bindings = {
                    "u_FlipbookTex"
                },
                frameEnd = 200,
                frameStart = 0,
                frameTranslate = -378,
                loopCountGoal = -1,
                exposureCount = 6,
                luaDriven = true,
                wrapMode = FBWrapMode.clamp,
                uri = "episode3:flipbooks/bg_environment_blur.mfb"
            },
        },
    },

    defaultMaterial  = SceneData.flipbookBackgroundMatDef,
    
    
    materials  = {
    	{  
			name =  "BG_environment_Blur_Mat",
		    shader      = "episode3:shaders/unlit.rsfx",
		    technique   = "flipbookColorMultiTexture",
		    dontOverrideTechnique = true,
		    textures = 
	        { 
	            u_AlphaTex = "episode3:textures/bg_environment_blur_alpha.dds",
	        },
		    parameters  = {u_Color = Vector(1,1,1,1), u_Alpha = 1, },
		},
    }
	
}    


SceneData.actors.Butterfly = 
{
    actorGroup = "star",
    model		= "episode3:models/butterfly.mxm",

    materials =
    { 
        {
            name = "Butterfly_MAT", 
            shader      = "episode3:shaders/unlit.rsfx",
            technique   = "butterfly",
            renderGroup = 50,            
            parameters  = {u_Color = Vector(1,1,1,1), u_Alpha = 1, },
        },
        {
            name = "Butterfly_Glow_MAT", 
            shader      = "episode3:shaders/unlit.rsfx",
            technique   = "butterflyGlow",
            renderGroup = 51,            
            parameters  = {u_Color = Vector(1,1,1,1), u_Alpha = 1, },
        },
    }
}    

SceneData.actors.Show_Sphere =
{
    model           = "episode3:models/starsphereband.mxm",    
    position        = { 0,0,0},
    animation       = {},
    materials = 
    {
        { 
            name = "starSphereBand_MAT", 
            shader = "episode3:shaders/unlit.rsfx",
            technique = "showSphere",
            textures = { 
                         u_PrimaryTex = "episode3:textures/starsphere.dds",
                         u_SecondaryTex = "episode3:textures/starsphereband.dds",
                       },           
            parameters =  { u_Color = Vector(1,1,1,1), u_blend = 0,}

        },
    },
    components = 
    {
        { 
            name = "MasterBlender",             
            params = 
            { 
                startAlpha = 0,
                blending = true,
                inout = "in",
                segmentData = 
                {   
                    {   
                        segment = "Cell_Girl",
                        startAlpha = 0, 
                        blending = false, 
                    },
                    { 
                        segment = "Cell_Boy",
                        startAlpha = 0, 
                        blending = false, 
                    },                          
                    { 
                        segment = "Birth_Girl",
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in", 
                    },
                    { 
                        segment = "Birth_Boy",
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in", 
                    },                          
                    { 
                        segment = "End_Girl",
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out", 
                        delay = 18 
                    },
                }
            }
        },
        { 
            name = "ShowSphereBlender",             
            params = 
            { 
                startAlpha = 0,
                blending = false,
                segmentData = 
                {
                    { 
                        segment = "First_Meeting", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "Second_Meeting1", 
                        startAlpha = 1, 
                        blending = false, 
                    },
                    { 
                        segment = "Third_Meeting1", 
                        startAlpha = 1, 
                        blending = false, 
                    },
                    { 
                        segment = "girltree_intro_a", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },
                    { 
                        segment = "boytree_intro_a", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },                    
                    { 
                        segment = "girltree_girlblinks", 
                        startAlpha = 0, 
                        blending = false, 
                    },
                    { 
                        segment = "boytree_girlblinks", 
                        startAlpha = 0, 
                        blending = false, 
                    },
                    { 
                        segment = "girltree_girloutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "girltree_boyoutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree_boyoutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree_girloutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "Fifth_Meeting1", 
                        startAlpha = 1, 
                        blending = false, 
                    },  
                    { 
                        segment = "End_Girl",
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out", 
                    },
                }
            }
        },                 
    },
}

SceneData.actors.Tree_Particles_Boy = 
{
    mpxUri          = "episode3:particles/treeparticles.mxp",
    attachTo        = { parentName  = "BoyTreeTree", targetBone = "treeParticleEmitter" },
    components      = 
    { 
        { 
            name = "ParticlesController", 
            params = 
            { 
                emitterName   = "treeParticles", 
                mpxFileName   = "treeparticles", 
                dof           = 4104,
                focalDistance = 373,
                dieOnDetach   = true,
                applyFocus    = true,
            },
        },
        { 
            name = "MasterBlender",             
            params = 
            { 
                startAlpha = 0,
                blending = true,
                inout = "in",        
                segmentData = 
                {
                    { 
                        segment = "girltree_girloutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "girltree_boyoutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree_boyoutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree_girloutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                },
            },
        },
    },

    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/treeparticles.mxp" },
        { Texture.rtti, "episode3:textures/particle_a.dds" },
        { Shader.rtti, "episode3:shaders/particle_focus_plane.rsfx" },
    },
}

SceneData.actors.Tree_Particles_Girl = 
{
    mpxUri          = "episode3:particles/treeparticles.mxp",
    attachTo        = { parentName  = "GirlTreeTree", targetBone = "treeParticleEmitter" },
    components      = 
    { 
        { 
            name = "ParticlesController", 
            params = 
            { 
                emitterName   = "treeParticles", 
                mpxFileName   = "treeparticles", 
                dof           = 4104,
                focalDistance = 373,
                dieOnDetach   = true,
                applyFocus    = true,
            },
        },
        { 
            name = "MasterBlender",             
            params = 
            { 
                startAlpha = 0,
                blending = true,
                inout = "in", 
                segmentData = 
                {
                    { 
                        segment = "girltree_girloutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "girltree_boyoutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree_boyoutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree_girloutro", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                },
            },
        },
    },

    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/treeparticles.mxp" },
        { Texture.rtti, "episode3:textures/particle_a.dds" },
        { Shader.rtti, "episode3:shaders/particle_focus_plane.rsfx" },
    },
}



SceneData.actors.Ground_Particles = 
{
    mpxUri          = "episode3:particles/groundparticles.mxp",
    attachTo        = { parentName  = "Origin" },
    components      = 
    { 
        { 
            name = "ParticlesController", 
            params = 
            {                 
                dieOnDetach    = true,
                dof            = 5522,
                focalDistance  = 3675,
                position       = Vector(0,0,0,0), 
                applyDepthY    = true,
                applyFocus     = true,
            },
        },
        { 
            name = "MasterBlender",             
            params = 
            { 
                startAlpha = 0,
                blending = true,
                inout = "in",       
                segmentData = 
                {                    
                    { 
                        segment = "girltree_intro_a", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },
                    { 
                        segment = "boytree_intro_a", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },                   
                    {
                        segment = "End_Girl", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },
                },
            },
        },
    },

    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/groundparticles.mxp" },
        { Texture.rtti, "episode3:textures/particle_a.dds" },
        { Shader.rtti, "episode3:shaders/particle_focus_plane.rsfx" },
    },
}


-----------------------------------------------------------------------------------------------------------
----------------------Cell : Characters---------------------------
-- The "Cell" and "Birth" sequences are now separate for the girl and boy, unlike our other sequences
SceneData.actors.Girl_Intro_Alt = 
{
    actorGroup      = "girl",
    model           = "episode3:models/girl_birth_alt.mxm",
    
    namedBones      = 
    {
        targetBone       = "targetGirl",
        offsetBone       = "offsetGirl",
        leftBound        = "leftGirl",
        rightBound       = "rightGirl",
        boyGlowTop       = "glowTLBoy",
        boyGlowBottom    = "glowBRBoy",
        boyGlowBone      = "glowBoy",
        boyGlowUp        = "glowTopBoy",
        boyGlowIntensity = "glowAlphaBoy",
    },
    animation       = 
    {
        Cell_Girl    = { "episode3:models/girl_birth_alt.mxa", 0, 551+505, false,  true}, 
        Birth_Girl   = { "episode3:models/girl_birth_alt.mxa", 551+506, -1, false, true},
    },
    components = 
    { 
        {
            name = "ActorMover",
            params = 
            {
                blendTimeArray      = { 0.0, 6.0, 6.1, 8.0,  15.0,  16.0,  17.0, 23.0,   35.18 },
                lockArray           = { 0.0, 0.0, 0.0, 0.0,   0.0,   0.0,   0.0,  0.0,   180.0 },
                innerLockArray      = { 0.0, 0.0, 0.0, 0.0,   0.0,   0.0,   0.0,  0.0,   0.0   },
                samples             = { 0.0, 0.0, 1.0, 4.0,   4.0,    0.0,   0.0,  4.0,   4.0, },
                lerpSpeeds          = { 1600,1600, 32, 2,    2,     10,    1600,   2,    0.0,  },
                
                lerpSpeed           = 0.05,
                startAtCamera       = false,

                usePolarSmoothing   = true,                
                
                useRepositionTable  = true,
                repositionKey       = "introStar",
                
            },             
        },         
        {
            name = "OpeningSphereController",
            params = 
            {

            },
        },
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 0,
                blending = true,
                flipbook = true,
                inout = "in",
                delay = 2,
            },
        },
    }, 

    fbdrivers       = { 
    	Cell_Girl    = {}, 
    	Birth_Girl  = {},
    },

    defaultMaterial = SceneData.flipbookMatDef,
    
    materials = { 
            {
            name = "starGlow_Mat", 
            shader      = "episode3:shaders/unlit.rsfx",
            technique   = "mainDuet",
            renderGroup = 1, --- some high number to render on top of the horizon
            textures = { u_MainTex = "episode3:textures/starglow.dds", },           
            parameters  = {u_Color = Vector(1,1,1,1)},
        },
    }
}


SceneData.actors.Boy_Intro_Alt = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_cell_birth_alt.mxm",
    namedBones      = 
    {
        targetBone        = "targetBoy",
        offsetBone        = "offsetBoy",
        leftBound         = "leftBoy",
        rightBound        = "rightBoy",
        girlGlowTop       = "glowTLGirl",
        girlGlowBottom    = "glowBRGirl",
        girlGlowBone      = "glowGirl",
        girlGlowUp        = "glowTopGirl",
        girlGlowIntensity = "glowAlphaGirl",
    },
    animation       = 
    {
        Cell_Boy    = { "episode3:models/boy_cell_birth_alt.mxa", 0, 551+554, false, true}, 
        Birth_Boy   = { "episode3:models/boy_cell_birth_alt.mxa", 551+555, -1, false, true}, 
	   	
    },
    components = 
    { 
        {
            name = "ActorMover",
            params = 
            {                                      
                blendTimeArray      = { 0.0, 6.0, 6.1, 8.0,  16.0,  17.0,  21.0, 22.0,  34.18 },
                lockArray           = { 0.0, 0.0, 0.0, 0.0,   0.0,   0.0,   0.0,  0.0,   180.0 },
                innerLockArray      = { 0.0, 0.0, 0.0, 0.0,   0.0,   0.0,   0.0,  0.0,   0.0   },
                samples      		= { 0.0, 0.0, 1.0, 4.0,   4.0,    0.0,   0.0,  4.0,   4.0, },
                lerpSpeeds			= { 1600,1600, 32, 2,    2,     10,    1600,   2,    0.0,  },
                
                frameStep			= 5,           

                lerpSpeed           = 0.05,
                startAtCamera       = false,

                usePolarSmoothing   = true,                
                
                useRepositionTable  = true,
                repositionKey       = "introStar",
            }, 
        }, 
        {
            name = "OpeningSphereController",
            params = 
            {

            },
        },
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 0,
                blending = true,
                flipbook = true,
                inout = "in",
                delay = 2,
            },
        },
    },

    fbdrivers       = { 
    	Cell_Boy    = {}, 
        Birth_Boy  = {}, 
   	},

    defaultMaterial = SceneData.flipbookMatDef,
    
    materials = { 
            {
            name = "starGlow_Mat", 
            shader      = "episode3:shaders/unlit.rsfx",
            technique   = "mainDuet",
            renderGroup = 1, --- some high number to render on top of the horizon
            textures = { u_MainTex = "episode3:textures/starglow.dds", },           
            parameters  = {u_Color = Vector(1,1,1,1)},
        },
    }
}


table.insert(SceneData.segments,
    {   
        name            = "Cell_Girl", 
        needsFocus      = false,
        animated        = { "Girl_Intro_Alt", "OpeningSphere" },
        active          = { "Girl_Intro_Alt", "OpeningSphere" },
        prefetch        = { SceneData.ShowStartActors },
        transitions     = 
        {
            default = { nextSegment = "Birth_Girl", leadActor = "Girl_Intro_Alt",}
        },

	    methods			= 
        {
	    	NoActor		= 
            { 
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end
            }

	    },
	    components = 
        {
			Girl_Intro_Alt = { 
				{ name = "GlowAlpha", params = {boneName = "glowIntensity"} },
				{ name = "AnimationDelay", params = { animation = "Cell_Girl"} }
			},
      	},
        disableBGStar  = true,        
        disableGroundParticles = true,
	}
)


table.insert(SceneData.segments,
    {   
        name            = "Cell_Boy", 
        needsFocus      = false,
        animated        = { "Boy_Intro_Alt", "OpeningSphere" },
        active          = { "Boy_Intro_Alt", "OpeningSphere" },
        prefetch        = { SceneData.ShowStartActors },
        transitions     =
        {
            default = { nextSegment = "Birth_Boy", leadActor = "Boy_Intro_Alt",}
        },

	    methods			= 
        {
	    	NoActor		= 
            { 
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end
            },
	    },
	    components = 
        {

			Boy_Intro_Alt = { 
				{ name = "GlowAlpha", params = {boneName = "glowIntensity"} },
				{ name = "AnimationDelay", params = {  animation = "Cell_Boy"} }
 			},
      	},
        disableBGStar  = true,
        disableGroundParticles = true,
	}
)


-----------------------------------------------------------------------------------------------------------
----------------------Birth : Targets---------------------------
SceneData.actors.Girl_To_1st_Meeting = 
{
    keyActor        = "Girl_Intro_Alt",
    attachTo        = { parentName  = "Girl_Intro_Alt", targetBone = "lookGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false, offset = Vector(-50, 0, 0) }, }, },
}

SceneData.actors.Girl_Alt_To_1st_Meeting = 
{
    keyActor        = "Boy_Intro_Alt",
    attachTo        = { parentName  = "Boy_Intro_Alt", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector" , params = {colliderRadius = 700, debugMode = false, offset = Vector(-50, 0, 0) }, }, },
}


SceneData.actors.Boy_To_1st_Meeting = 
{
    keyActor        = "Boy_Intro_Alt",
    attachTo        = { parentName  = "Boy_Intro_Alt", targetBone = "lookBoy" },
    components      = { { name      = "LookAtDetector" , params = {colliderRadius = 700, debugMode = false, offset = Vector(50, 0, 0) }, }, },
}

SceneData.actors.Boy_Alt_To_1st_Meeting = 
{
    keyActor        = "Girl_Intro_Alt",
    attachTo        = { parentName  = "Girl_Intro_Alt", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector" , params = {colliderRadius = 700, debugMode = false, offset = Vector(50, 0, 0) }, }, },
}

----------------------Birth : Segment---------------------------
table.insert(SceneData.segments,
    {   
        name            = "Birth_Girl", 
        needsFocus      = true,
        animated        = { "Girl_Intro_Alt", "OpeningSphere"},
        active          = { "Girl_Intro_Alt", "Girl_To_1st_Meeting", "Boy_Alt_To_1st_Meeting", "OpeningSphere" },
        
        transitions     = 
        { 
            Girl_To_1st_Meeting    = { nextSegment = "First_Meeting", leadActor = "Girl_Intro_Alt", },
            Boy_Alt_To_1st_Meeting = { nextSegment = "First_Meeting", leadActor = "Girl_Intro_Alt", },            
        },
        components = 
        {
			Girl_Intro = { { name = "GlowAlpha", paramHeadingClamps = {boneName = "glowIntensity"} }, },
      	},
		methods = 
        {
    		NoActor		= {
                disable90DegOffset = function(segment, actor) MainCamera.controller.enable90DegOffsets = false end,
                cleanupGWI = function() destroySafely(Global.scene.actors.GoodWayIn) end
            }
    	},
        disableBGStar   = true,
        ignoreOffset	= true,
        disableGroundParticles = true,
	}
)

table.insert(SceneData.segments,
    {   
        name            = "Birth_Boy", 
		needsFocus      = true,
        animated        = { "Boy_Intro_Alt", "OpeningSphere"},
        active          = { "Boy_Intro_Alt", "Boy_To_1st_Meeting", "Girl_Alt_To_1st_Meeting", "OpeningSphere" },
        
        transitions     = 
        {
            Girl_Alt_To_1st_Meeting = { nextSegment = "First_Meeting", leadActor = "Boy_Intro_Alt", },        
            Boy_To_1st_Meeting      = { nextSegment = "First_Meeting", leadActor = "Boy_Intro_Alt", },
        },
        components = 
        {
			Boy_Intro_Alt = { { name = "GlowAlpha", params = {boneName = "glowIntensity"} }, },
		},
		methods = 
        {
    		NoActor		= {
                disable90DegOffset = function(segment, actor) MainCamera.controller.enable90DegOffsets = false end,
                cleanupGWI = function() destroySafely(Global.scene.actors.GoodWayIn) end
            }
    	},
        disableBGStar   = true,
        ignoreOffset	= true,
        disableGroundParticles = true,        
	}
)


-----------------------------------------------------------------------------------------------------------
----------------------First Meeting : Characters---------------------------
SceneData.actors.Girl_1st_Meeting = 
{
    actorGroup      = "girl",
    model           = "episode3:models/girl_1st_meeting.mxm",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",
        leftBound   = "leftGirl",
        rightBound  = "rightGirl",
    },
    animation       = 
    {
        First_Meeting              = { "episode3:models/girl_1st_meeting.mxa", 0, 591, false, true},
        First_Meeting_Dog_To_Boy   = { "episode3:models/girl_1st_meeting.mxa", 592, -1, false, true},
        
        First_Meeting_Dog_To_Boy_B = { "episode3:models/girl_1st_meeting.mxa", 670, -1, false, true},
        First_Meeting_Boy_Solo     = { "episode3:models/girl_1st_meeting.mxa", 624, -1, false, true}, 
        First_Meeting_Jump_To_Boy  = { "episode3:models/girl_1st_meeting.mxa", 670, -1, false, true},               
    },
    fbdrivers       = 
    { 
    	First_Meeting = {}, 
    	First_Meeting_Dog_To_Boy = {},
    	First_Meeting_Dog_To_Boy_B = {},
        First_Meeting_Boy_Solo = {},
        First_Meeting_Jump_To_Boy = {},
    },
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Boy_1st_Meeting = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_1st_meeting.mxm",
    namedBones      = 
    {
        targetBone = "targetBoy",
        offsetBone = "offsetBoy",
        leftBound   = "leftBoy",
        rightBound  = "rightBoy",
    },
    animation       = 
    {
        First_Meeting   = { "episode3:models/boy_1st_meeting.mxa", 0, 591, false, true},
        First_Meeting_Boy_Solo = { "episode3:models/boy_1st_meeting.mxa", 592, -1, false, true},
    },
    fbdrivers       = 
    { 
        First_Meeting = {},
        First_Meeting_Boy_Solo = {},
    },

        
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Dog_1st_Meeting = 
{
    actorGroup      = "dog",
    model           = "episode3:models/dog_1st_meeting.mxm",
    namedBones      = 
    {
        targetBone = "targetDog",
        offsetBone = "offsetDog",
    },
    animation       = 
    {
        First_Meeting   = { "episode3:models/dog_1st_meeting.mxa", 0, 591, false, true},         
        First_Meeting_Dog_To_Boy = { "episode3:models/dog_1st_meeting.mxa", 596, 669, false, true},
        First_Meeting_Dog_To_Boy_B   = { "episode3:models/dog_1st_meeting.mxa", 670, -1, false, true},
        First_Meeting_Jump_To_Boy = { "episode3:models/dog_1st_meeting.mxa", 670, -1, false, true},
    },
    fbdrivers       = 
    { 
        First_Meeting   = {}, 
        First_Meeting_Dog_To_Boy = {},
        First_Meeting_Dog_To_Boy_B = {},
        First_Meeting_Jump_To_Boy = {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}


SceneData.actors.Boy_1st_Meeting_Alt = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_1st_meeting_alt.mxm",
    namedBones      = 
    {
        targetBone = "targetBoy",
        offsetBone = "offsetBoy",
        leftBound   = "leftBoy",
        rightBound  = "rightBoy",
    },
    animation       = 
    {
        First_Meeting_Dog_To_Boy = { "episode3:models/boy_1st_meeting_alt.mxa", 669, 669, true, true },
        First_Meeting_Dog_To_Boy_B = { "episode3:models/boy_1st_meeting_alt.mxa", 670, -1, false, true },
        First_Meeting_Jump_To_Boy   = { "episode3:models/boy_1st_meeting_alt.mxa", 670, -1, false, true},
    },
    fbdrivers       = 
    { 
        First_Meeting_Dog_To_Boy = {},
        First_Meeting_Dog_To_Boy_B = {},
        First_Meeting_Jump_To_Boy = {}
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Dog_1st_Meeting_Alt = 
{
    actorGroup      = "dog",
    model           = "episode3:models/dog_1st_meeting_alt.mxm",
    namedBones      =
    {
        targetBone = "targetDog",
        offsetBone = "offsetDog",
    },
    animation       = 
    {
        First_Meeting_Boy_Solo = { "episode3:models/dog_1st_meeting_alt.mxa", 592, -1, false, true},
    },
    fbdrivers       = 
    { 
        First_Meeting_Boy_Solo = {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

----------------------First Meeting : Targets---------------------------

SceneData.actors.Girl_1st_Meeting_Target = 
{
    keyActor        = "Girl_1st_Meeting",
    attachTo        = { parentName  = "Girl_1st_Meeting", targetBone = "targetGirl"},
    components      = { { name      = "LookAtDetector", params = { colliderRadius = 1200, debugMode = false }  },},
}

SceneData.actors.Boy_1st_Meeting_Target = 
{
    keyActor        = "Boy_1st_Meeting",
    attachTo        = { parentName  = "Boy_1st_Meeting", targetBone = "targetBoy"},
    components      = { { name      = "LookAtDetector", params = { colliderRadius = 900, debugMode = false }  },},
}

SceneData.actors.Dog_1st_Meeting_Target =
{
    keyActor        = "Dog_1st_Meeting",
    attachTo        = { parentName  = "Dog_1st_Meeting", targetBone = "targetDog"},
    components      = { { name      = "LookAtDetector", params = { colliderRadius = 900, debugMode = false }  },},
}

SceneData.actors.Dog_1st_Meeting_Target_Alt =
{
    keyActor        = "Dog_1st_Meeting_Alt",
    attachTo        = { parentName  = "Dog_1st_Meeting_Alt", targetBone = "targetDog"},
    components      = { { name      = "LookAtDetector", params = { colliderRadius = 900, debugMode = false }  },},
}

SceneData.actors.Boy_1st_Meeting_Alt_Target =
{
    keyActor        = "Boy_1st_Meeting_Alt",
    attachTo        = { parentName  = "Boy_1st_Meeting_Alt", targetBone = "targetBoy"},
    components      = { { name      = "LookAtDetector", params = { colliderRadius = 800, debugMode = false }  },},
}


----------------------First Meeting : Segment---------------------------
table.insert(SceneData.segments,
    {   
        name            = "First_Meeting", 
        needsFocus      = true,

        animated        = { "Girl_1st_Meeting", "Boy_1st_Meeting", "Dog_1st_Meeting",},
        active          = { "Girl_1st_Meeting", "Boy_1st_Meeting", "Dog_1st_Meeting", 
                            "Girl_1st_Meeting_Target", "Boy_1st_Meeting_Target", "Dog_1st_Meeting_Target",},

        transitions     = 
        {
            Boy_1st_Meeting_Target	= { nextSegment = "First_Meeting_Boy_Solo", leadActor = "Boy_1st_Meeting",},
            Girl_1st_Meeting_Target = { nextSegment = "First_Meeting_Dog_To_Boy", leadActor = "Girl_1st_Meeting",},      
        },
              
	    methods			= 
        {
	    	NoActor		= 
            { 
	    		disable90DegOffset = function(segment, actor)
                    MainCamera.controller.enable90DegOffsets = false 
                end 
            }
	    },
        checkpoint = true,
        checkpointLocation = { x = 0, y = -76.7, z = -2100.9, w = 0 },
    }
)


table.insert(SceneData.segments,
    {   
        name            = "First_Meeting_Dog_To_Boy", 
        needsFocus      = true,

        animated        = { "Boy_1st_Meeting_Alt", "Girl_1st_Meeting", "Dog_1st_Meeting", },
        active          = { "Boy_1st_Meeting_Alt", "Girl_1st_Meeting", "Dog_1st_Meeting",
        					"Boy_1st_Meeting_Alt_Target", "Dog_1st_Meeting_Target", "Girl_1st_Meeting_Target"  },

        transitions     = 
        {
            Dog_1st_Meeting_Target  = { nextSegment = "First_Meeting_Dog_To_Boy_B", leadActor = "Dog_1st_Meeting",},
            Boy_1st_Meeting_Alt_Target  = { nextSegment = "First_Meeting_Jump_To_Boy", leadActor = "Boy_1st_Meeting_Alt",},
            Girl_1st_Meeting_Target  = { nextSegment = "Second_Meeting1", leadActor = "Girl_1st_Meeting",},            
        },
        
        ignoreOffset    = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "First_Meeting_Dog_To_Boy_B", 
        needsFocus      = true,

        animated        = { "Boy_1st_Meeting_Alt", "Dog_1st_Meeting", "Girl_1st_Meeting"},
        active          = { "Boy_1st_Meeting_Alt", "Dog_1st_Meeting", "Girl_1st_Meeting", "Boy_1st_Meeting_Alt_Target",  "Girl_1st_Meeting_Target", "Dog_1st_Meeting_Target" },

        transitions     = 
        {
            Boy_1st_Meeting_Alt_Target  = { nextSegment = "Second_Meeting1", leadActor = "Boy_1st_Meeting_Alt",},
            Dog_1st_Meeting_Target  = { nextSegment = "Second_Meeting1", leadActor = "Boy_1st_Meeting_Alt",},            
            Girl_1st_Meeting_Target  = { nextSegment = "Second_Meeting1", leadActor = "Girl_1st_Meeting",},
        },
        
        ignoreOffset    = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "First_Meeting_Jump_To_Boy", 
        needsFocus      = true,

        animated        = { "Boy_1st_Meeting_Alt", "Dog_1st_Meeting", "Girl_1st_Meeting", },
        active          = { "Boy_1st_Meeting_Alt", "Dog_1st_Meeting", "Girl_1st_Meeting", "Boy_1st_Meeting_Alt_Target", "Girl_1st_Meeting_Target", },

        transitions     = 
        {
            Boy_1st_Meeting_Alt_Target  = { nextSegment = "Second_Meeting1", leadActor = "Boy_1st_Meeting_Alt",},
            Girl_1st_Meeting_Target  = { nextSegment = "Second_Meeting1", leadActor = "Girl_1st_Meeting",},                            
        },
        
        ignoreOffset    = true,
    }
)


table.insert(SceneData.segments,
    {   
        name            = "First_Meeting_Boy_Solo", 
        needsFocus      = true,

        animated        = { "Boy_1st_Meeting", "Dog_1st_Meeting_Alt", "Girl_1st_Meeting",  },
        active          = { "Boy_1st_Meeting", "Dog_1st_Meeting_Alt", "Girl_1st_Meeting", "Boy_1st_Meeting_Target", "Dog_1st_Meeting_Target_Alt", "Girl_1st_Meeting_Target", },

        transitions     = 
        {
            Boy_1st_Meeting_Target  = { nextSegment = "Second_Meeting1", leadActor = "Boy_1st_Meeting",},
            Dog_1st_Meeting_Target_Alt  = { nextSegment = "Second_Meeting1", leadActor = "Boy_1st_Meeting",},
            Girl_1st_Meeting_Target  = { nextSegment = "Second_Meeting1", leadActor = "Girl_1st_Meeting",},                    
        },
        
        ignoreOffset    = true,
    }
)

-----------------------------------------------------------------------------------------------------------
----------------------Second Meeting : Characters---------------------------
SceneData.actors.Girl_2nd_Meeting = 
{
    actorGroup      = "girl",
    model           = "episode3:models/girl_2nd_meeting.mxm",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",
        leftBound   = "leftGirl",
        rightBound  = "rightGirl",
    },
    animation       = 
    { 
        Second_Meeting1             = { "episode3:models/girl_2nd_meeting.mxa", 0, 232, false, true},
        Second_Meeting2             = { "episode3:models/girl_2nd_meeting.mxa", 233, 467, false, true},
        Second_Meeting3             = { "episode3:models/girl_2nd_meeting.mxa", 468, 674, false, true},
    },
    fbdrivers       =   
    {
        Second_Meeting1 = {},
		Second_Meeting2 = {},
		Second_Meeting3 = {},
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Boy_2nd_Meeting = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_2nd_meeting.mxm",
    namedBones      = 
    {
        targetBone  = "targetBoy",
        offsetBone  = "offsetBoy",
        leftBound   = "leftBoy",
        rightBound  = "rightBoy",
        dogGlowTop = "glowTLDog",
        dogGlowBottom = "glowBRDog",
        dogGlowBone = "glowDog",
        dogGlowUp = "glowTopDog",
        dogGlowIntensity = "glowAlphaDog",
    },
    animation       = 
    {
        Second_Meeting1             = { "episode3:models/boy_2nd_meeting.mxa", 0, 232, false, true},
        Second_Meeting2             = { "episode3:models/boy_2nd_meeting.mxa", 233, 467, false, true},
        Second_Meeting3             = { "episode3:models/boy_2nd_meeting.mxa", 468, 625, false, true},
    },
    fbdrivers       = 
    { 
        Second_Meeting1 = {},
		Second_Meeting2 = {},
		Second_Meeting3 = {},
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

----------------------Second Meeting : Targets---------------------------
    --[[ Second Girl Solo : Targets --]]
SceneData.actors.Girl_To_3rd_Meeting = 
{
    keyActor        = "Girl_2nd_Meeting",
    attachTo        = { parentName  = "Girl_2nd_Meeting", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

    --[[ Second Boy Solo : Targets --]]
SceneData.actors.Boy_To_3rd_Meeting = 
{
    keyActor        = "Boy_2nd_Meeting",
    attachTo        = { parentName  = "Boy_2nd_Meeting", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

SceneData.actors.Dog_To_3rd_Meeting = 
{
    keyActor        = "Boy_2nd_Meeting",
    attachTo        = { parentName  = "Boy_2nd_Meeting", targetBone = "targetDog" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

----------------------Second Meeting : Segment---------------------------

table.insert(SceneData.segments,
    {   
        name            = "Second_Meeting1", 
        needsFocus = true,
        animated        = { "Boy_2nd_Meeting", "Girl_2nd_Meeting",  },
        active          = { "Boy_2nd_Meeting", "Girl_2nd_Meeting",  "Boy_To_3rd_Meeting", "Girl_To_3rd_Meeting", },

        transitions     = 
        {
            Boy_To_3rd_Meeting = { nextSegment = "Second_Meeting2", leadActor = "Boy_2nd_Meeting",},
            Girl_To_3rd_Meeting  = { nextSegment = "Second_Meeting2", leadActor = "Girl_2nd_Meeting",}
        },
		
	    methods			= 
        {
	    	NoActor		= 
            {
	    		disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = true 
                end 
	    	}
	    },
        checkpoint = true,
        checkpointLocation = { x = 500, y = 1.09, z = -2106, w = 0 },
    }
)
table.insert(SceneData.segments,
    {   name            = "Second_Meeting2", needsFocus = true,
        animated        = { "Boy_2nd_Meeting", "Girl_2nd_Meeting", },
        active          = { "Boy_2nd_Meeting", "Girl_2nd_Meeting", "Boy_To_3rd_Meeting", "Girl_To_3rd_Meeting","Dog_To_3rd_Meeting", },
		ignoreOffset    = true,
        transitions     = 
        {
            Boy_To_3rd_Meeting  = { nextSegment = "Second_Meeting3", leadActor = "Boy_2nd_Meeting",},
            Dog_To_3rd_Meeting  = { nextSegment = "Second_Meeting3", leadActor = "Boy_2nd_Meeting",},
            Girl_To_3rd_Meeting = { nextSegment = "Second_Meeting3", leadActor = "Girl_2nd_Meeting",}
        },

    }
)
table.insert(SceneData.segments,
    {   
        name            = "Second_Meeting3", 
        needsFocus      = true,
        animated        = { "Boy_2nd_Meeting", "Girl_2nd_Meeting", },
        active          = { "Boy_2nd_Meeting", "Girl_2nd_Meeting", "Boy_To_3rd_Meeting", "Girl_To_3rd_Meeting"},
		ignoreOffset    = true,
        transitions     = 
        {
            Boy_To_3rd_Meeting  = { nextSegment = "Third_Meeting1", leadActor = "Boy_2nd_Meeting",},
            Girl_To_3rd_Meeting = { nextSegment = "Third_Meeting1", leadActor = "Girl_2nd_Meeting",}
        },

    }
)

-----------------------------------------------------------------------------------------------------------
----------------------Third Meeting : Characters---------------------------
SceneData.actors.Girl_3rd_Meeting =
{
    actorGroup      = "girl",
    frameStep       = 2,
    model           = "episode3:models/girl_3rd_meeting.mxm",
    namedBones      =
    {
        targetBone  = "targetGirl",
        offsetBone  = "offsetGirl",
        leftBound   = "leftGirl",
        rightBound  = "rightGirl",
        -- for by dog actorvignette
        dogGlowTop       = "glowTLGirlDog",
        dogGlowBottom    = "glowBRGirlDog",
        dogGlowBone      = "glowGirlDog",
    },
    animation       =
    {
        Third_Meeting1            = { "episode3:models/girl_3rd_meeting.mxa", 1, 272, false, true },
        Third_Meeting2           = { "episode3:models/girl_3rd_meeting.mxa", 273, 558, false, true },
        Third_Meeting3            = { "episode3:models/girl_3rd_meeting.mxa", 535, 545, false, true },
        Third_Meeting3_Alt = { "episode3:models/girl_3rd_meeting.mxa", 559, 600, false, true },
        Third_Meeting4            = { "episode3:models/girl_3rd_meeting.mxa", 546, 699, false, true },
        Third_Meeting4_Alt	  = { "episode3:models/girl_3rd_meeting.mxa", 601, 699, false, true },
        Third_Meeting_End_Girl	  = {  "episode3:models/girl_3rd_meeting.mxa", 700, -1, false, true },
    },
    fbdrivers       =
    {
        Third_Meeting1 = {},
        Third_Meeting2 = {},
        Third_Meeting3 = {},
        Third_Meeting3_Alt = {},
        Third_Meeting4 = {},
        Third_Meeting4_Alt = {},
        Third_Meeting_End_Girl = {},
    },

    defaultMaterial = SceneData.flipbookMatDef,
}


SceneData.actors.Boy_3rd_Meeting = 
{
    actorGroup      = "boy",
    frameStep       = 4,
    model           = "episode3:models/boy_3rd_meeting_to_tree.mxm",
    materials     = {},
    namedBones      = 
    {
        targetBone  = "targetBoy",
        offsetBone  = "offsetBoy",
        leftBound   = "leftBoy",
        rightBound  = "rightBoy",
        targetDog   = "targetDog",
        
        -- for by dog actorvignette
        dogGlowTop       = "glowTLDog",
        dogGlowBottom    = "glowBRDog",
        dogGlowBone      = "glowDog",
        dogGlowUp        = "glowTopDog",
        dogGlowIntensity = "glowAlphaDog",
    },
    animation       = 
    {
        Third_Meeting1               = { "episode3:models/boy_3rd_meeting_to_tree.mxa", 1, 272, false, true },
        Third_Meeting2               = { "episode3:models/boy_3rd_meeting_to_tree.mxa", 273, 558, false, true },
        Third_Meeting3               = { "episode3:models/boy_3rd_meeting_to_tree.mxa", 559, 600, false, true },
        Third_Meeting4               = { "episode3:models/boy_3rd_meeting_to_tree.mxa", 601, 699, false, true },
        Third_Meeting_End_Boy        = { "episode3:models/boy_3rd_meeting_to_tree.mxa", 700, -1, false, true },
    },
    fbdrivers       = 
    { 
        Third_Meeting1 = {},
        Third_Meeting2 = {},
        Third_Meeting3 = {}, 
        Third_Meeting4 = {}, 
        Third_Meeting_End_Boy = {}
    },
    components      =
    {
        {
            name = "DogBlender",
            params = 
            { 
                delay = 0.8,
                materialName = "Dog_to_Boy_tree_Mat",                                
                blendInSegment = "Third_Meeting_End_Boy",
            }       
        }
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Boy_3rd_Meeting_Alt = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_3rd_meeting_to_tree_alt.mxm",
    namedBones      = 
    {
        targetBone  = "targetBoy",
        offsetBone  = "offsetBoy",
        leftBound   = "leftBoy",
        rightBound  = "rightBoy",
        targetDog   = "targetDog",
        
        -- for by dog actorvignette
        dogGlowTop       = "glowTLDog",
        dogGlowBottom    = "glowBRDog",
        dogGlowBone      = "glowDog",
        dogGlowUp        = "glowTopDog",
        dogGlowIntensity = "glowAlphaDog",
    },
    animation       = 
    {
        Third_Meeting3_Alt	 		= { "episode3:models/boy_3rd_meeting_to_tree_alt.mxa", 503, 600, false, true },
        Third_Meeting4_Alt	 		= { "episode3:models/boy_3rd_meeting_to_tree_alt.mxa", 601, 699, false, true },
        Third_Meeting4_Alt_FromDog	= { "episode3:models/boy_3rd_meeting_to_tree_alt.mxa", 601, 699, false, true },
        Third_Meeting_End_Boy_Alt   = { "episode3:models/boy_3rd_meeting_to_tree_alt.mxa", 700, -1, false, true  },
    },
    fbdrivers       = 
    { 
        Third_Meeting3_Alt = {},
        Third_Meeting4_Alt = {},
        Third_Meeting4_Alt_FromDog = {},
        Third_Meeting_End_Boy_Alt = {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

----------------------Third Meeting : Targets---------------------------
SceneData.actors.Girl_Third_Meeting_Target =
{
    keyActor        = "Girl_3rd_Meeting",
    attachTo        = { parentName  = "Girl_3rd_Meeting", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 900, debugMode = false} }, },    
}

SceneData.actors.Boy_Third_Meeting_Target_Alt = 
{
    keyActor        = "Boy_3rd_Meeting_Alt",
    attachTo        = { parentName  = "Boy_3rd_Meeting_Alt", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1200, debugMode = false} }, },
}

SceneData.actors.Dog_Third_Meeting_Target = 
{
    keyActor        = "Boy_3rd_Meeting",
    attachTo        = { parentName  = "Boy_3rd_Meeting", targetBone = "targetDog" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 900, debugMode = false} }, },
}

SceneData.actors.Dog_Third_Meeting_Target_Offset = 
{
    keyActor        = "Boy_3rd_Meeting",
    attachTo        = { parentName  = "Boy_3rd_Meeting", targetBone = "targetDog" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false, offset = Vector(-450, 0, 0)} }, },
}

SceneData.actors.Dog_Third_Meeting_Target_Alt = 
{
    keyActor        = "Boy_3rd_Meeting_Alt",
    attachTo        = { parentName  = "Boy_3rd_Meeting_Alt", targetBone = "targetDog" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 900, debugMode = false} }, },
}

SceneData.actors.Boy_To_Third_Meeting_Boy = 
{
    keyActor        = "Boy_3rd_Meeting",
    attachTo        = { parentName  = "Boy_3rd_Meeting", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1100, debugMode = false} }, },
}

SceneData.actors.Boy_End_Target = 
{
    keyActor        = "Boy_3rd_Meeting",
    attachTo        = { parentName  = "Boy_3rd_Meeting", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

SceneData.actors.Boy_End_Target_Alt = 
{
    keyActor        = "Boy_3rd_Meeting_Alt",
    attachTo        = { parentName  = "Boy_3rd_Meeting_Alt", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

----------------------Third Meeting : Segment---------------------------
table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting1", 
        needsFocus      = true,

        animated        = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", },
        active          = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", "Girl_Third_Meeting_Target", "Boy_To_Third_Meeting_Boy", "Dog_Third_Meeting_Target_Offset"  },
        
        transitions     = 
        {
            Girl_Third_Meeting_Target       = { nextSegment = "Third_Meeting2",    leadActor = "Girl_3rd_Meeting",},
            Boy_To_Third_Meeting_Boy   		= { nextSegment = "Third_Meeting2",     leadActor = "Boy_3rd_Meeting",},
            Dog_Third_Meeting_Target_Offset = { nextSegment = "Third_Meeting2",     leadActor = "Boy_3rd_Meeting",},
        },
        
        methods			= 
        {
        	Boy_3rd_Meeting = 
            { 
                restoreHitRectSize = function(segment, actor) 
                	if(MainCamera.controller.proxyLayer) then
	        			local boyTracker = MainCamera.controller.proxyLayer.boyTracker
	                    boyTracker.lookAtDetector.colliderRadius = 1000
                    end
        		end
            },
        },
        checkpoint = true,
        checkpointLocation = { x = -993, y = 126, z = -1915, w = 0 },
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting2", 
        needsFocus      = true,

        animated        = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", },
        active          = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", "Girl_Third_Meeting_Target", "Boy_To_Third_Meeting_Boy", "Dog_Third_Meeting_Target"  },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Girl_Third_Meeting_Target       = { nextSegment = "Third_Meeting3_Alt",    leadActor = "Girl_3rd_Meeting",},
            Boy_To_Third_Meeting_Boy   		= { nextSegment = "Third_Meeting3",     leadActor = "Boy_3rd_Meeting",},
            Dog_Third_Meeting_Target        = { nextSegment = "Third_Meeting3_Alt",   leadActor = "Boy_3rd_Meeting",},
        },
        
    }
)

table.insert(SceneData.segments,
    {   name            = "Third_Meeting3", 
        needsFocus      = true,

        animated        = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", },
        active          = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", "Girl_Third_Meeting_Target", "Boy_To_Third_Meeting_Boy", "Dog_Third_Meeting_Target"  },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Girl_Third_Meeting_Target  = { nextSegment = "Third_Meeting4",    leadActor = "Girl_3rd_Meeting",},
            Boy_To_Third_Meeting_Boy   = { nextSegment = "Third_Meeting4",     leadActor = "Boy_3rd_Meeting",},
        },
        
        methods			= 
        {
        	NoActor		= 
            {
        		-- disable 90deg rotations while girl and dog are interacting (otherwise the dog gets pulled off screen)
				disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end 
			}
        },
    }
)

table.insert(SceneData.segments,
    {   name            = "Third_Meeting3_Alt", 
    	needsFocus      = true,

        animated        = { "Girl_3rd_Meeting", "Boy_3rd_Meeting_Alt", },
        active          = { "Girl_3rd_Meeting", "Boy_3rd_Meeting_Alt", "Girl_Third_Meeting_Target", "Boy_Third_Meeting_Target_Alt", "Dog_Third_Meeting_Target_Alt"},
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Girl_Third_Meeting_Target    = { nextSegment = "Third_Meeting4_Alt", leadActor = "Girl_3rd_Meeting",},
            Boy_Third_Meeting_Target_Alt = { nextSegment = "Third_Meeting4_Alt", leadActor = "Boy_3rd_Meeting",},
            Dog_Third_Meeting_Target_Alt = { nextSegment = "Third_Meeting4_Alt_FromDog", leadActor = "Boy_3rd_Meeting",},
        },
        
        methods			= 
        {
        	NoActor		= 
            {
        		-- disable 90deg rotations while girl and dog are interacting (otherwise the dog gets pulled off screen)
				disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end 
			}
        }, 
    }
)
table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting4", 
        needsFocus      = true,
        animated        = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", },
        active          = { "Girl_3rd_Meeting", "Boy_3rd_Meeting", "Girl_Third_Meeting_Target", "Boy_To_Third_Meeting_Boy"},
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Girl_Third_Meeting_Target = { nextSegment = "Third_Meeting_End_Girl", leadActor = "Girl_3rd_Meeting",},
            Boy_To_Third_Meeting_Boy  = { nextSegment = "Third_Meeting_End_Boy", leadActor = "Boy_3rd_Meeting",},
        },
        
        methods			= 
        {
        	NoActor		= 
            {
                disable90DegOffset = function(segment, actor) MainCamera.controller.enable90DegOffsets = true end 
	    	} 
        },
        
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting4_Alt", 
        needsFocus      = true,

        animated        = { "Girl_3rd_Meeting", "Boy_3rd_Meeting_Alt", },
        active          = { "Girl_3rd_Meeting", "Boy_3rd_Meeting_Alt", "Girl_Third_Meeting_Target", "Boy_Third_Meeting_Target_Alt" },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Girl_Third_Meeting_Target    = { nextSegment = "Third_Meeting_End_Girl", leadActor = "Girl_3rd_Meeting",},
            Boy_Third_Meeting_Target_Alt = { nextSegment = "Third_Meeting_End_Boy_Alt", leadActor = "Boy_3rd_Meeting_Alt",},
        },
        
        methods			= 
        {
        	NoActor		=
            {
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = true 
                end 
	    	} 
        },        
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting4_Alt_FromDog", 
        animated        = { "Boy_3rd_Meeting_Alt", },
        active          = { "Boy_3rd_Meeting_Alt", "Boy_Third_Meeting_Target_Alt" },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            default = { nextSegment = "Third_Meeting_End_Boy_Alt", leadActor = "Boy_3rd_Meeting_Alt",},
        },
        
        methods			= 
        {
        	NoActor		=
            {
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = true 
                end 
	    	} 
        },        
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting_End_Girl", 
        needsFocus      = true,

        animated        = { "Girl_3rd_Meeting", },
        active          = { "Girl_3rd_Meeting", "Girl_Third_Meeting_Target", },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Girl_Third_Meeting_Target       = { nextSegment = "girltree_intro_a",    leadActor = "Girl_3rd_Meeting",},
        },
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting_End_Boy", 
        needsFocus      = true,

        animated        = { "Boy_3rd_Meeting", },
        active          = { "Boy_3rd_Meeting", "Boy_End_Target", "Dog_Third_Meeting_Target" },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Boy_End_Target           = { nextSegment = "boytree_intro_a",     leadActor = "Boy_3rd_Meeting",},
            Dog_Third_Meeting_Target = { nextSegment = "boytree_intro_a",     leadActor = "Boy_3rd_Meeting",},
        },
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Third_Meeting_End_Boy_Alt", 
        needsFocus      = true,

        animated        = { "Boy_3rd_Meeting_Alt", },
        active          = { "Boy_3rd_Meeting_Alt", "Boy_End_Target_Alt", "Dog_Third_Meeting_Target_Alt" },
        
        ignoreOffset    = true, 
        transitions     = 
        {
            Boy_End_Target_Alt           = { nextSegment = "boytree_intro_a",     leadActor = "Boy_3rd_Meeting_Alt",},
            Dog_Third_Meeting_Target_Alt = { nextSegment = "boytree_intro_a",     leadActor = "Boy_3rd_Meeting_Alt",},            
        },
    }
)

-----------------------------------------------------------------------------------------------------------
-- Girl tree: Characters

-- TODO GirlTreeGirl

SceneData.actors.GirlTreeGirl = 
{
    actorGroup      = "girl",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",
    },

    model           = "episode3:models/girl_4th_meeting.mxm",

    animation       = 
    {
        girltree_intro_a    	= { "episode3:models/girl_4th_meeting.mxa", 1, 175, false, true},
        girltree_intro_b    	= { "episode3:models/girl_4th_meeting.mxa", 176, 360, false, true},
        girltree            	= { "episode3:models/girl_4th_meeting.mxa", 361, 779, false, true},
        girltree_dance      	= { "episode3:models/girl_4th_meeting.mxa", 780, 1001, false, true},
        
        girltree_girlblinks   	= { "episode3:models/girl_4th_meeting.mxa", 1002, 1100, false, true},
        girltree_girlwalks_a   	= { "episode3:models/girl_4th_meeting.mxa", 1101, 1202, false, true},
        girltree_girlwalks_b   	= { "episode3:models/girl_4th_meeting.mxa", 1203, 1458, false, true},
        
		girltree_girljumps		= { "episode3:models/girl_4th_meeting.mxa", 1735, 1751, false, true},
		girltree_girloutro		= { "episode3:models/girl_4th_meeting.mxa", 1752, -1, false, true},
		
    },
    
    fbdrivers       = 
    { 
        girltree_intro_a    = {},
        girltree_intro_b    = {},
        girltree            = {},
        girltree_dance      = {},
        
        girltree_girlblinks = {},
        girltree_girlwalks_a = {},
        girltree_girlwalks_b = {},
        
        girltree_girljumps	= {},
        girltree_girloutro	= {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.GirlTreeDog = 
{
    actorGroup      = "dog",

    model           = "episode3:models/girl_4th_meeting_dog.mxm",

    namedBones      =
    {
        targetBone = "targetDog",
    },

    animation       = 
    {
        girltree_intro_a    = { "episode3:models/girl_4th_meeting_dog.mxa", 1, 175, false, true},
        girltree_intro_b    = { "episode3:models/girl_4th_meeting_dog.mxa", 176, 360, false, true},
        girltree_boyoutro 	= { "episode3:models/girl_4th_meeting_dog.mxa", 1450, -1, false, true},
    },
    
    fbdrivers       = 
    { 
        girltree_intro_a    = {},
        girltree_intro_b    = {},
        girltree            = {},
        girltree_boyoutro 	= {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.GirlPond =
{
    actorGroup      = "girl",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",        
    },    

    model           = "episode3:models/girl_pond.mxm",

    animation       = 
    {
        Girl_4th_Solo_B     		= { "episode3:models/girl_pond.mxa", 369, 886, false, true},
        Girl_4th_Solo_B2     		= { "episode3:models/girl_pond.mxa", 887, -1, false, true},
        
        Boy_4th_Solo_B     = { "episode3:models/girl_pond.mxa", 300+201+1, -1, false, true	},
        
    },

    fbdrivers       = 
    {
        Girl_4th_Solo_B     = {},
        Girl_4th_Solo_B2     = {},
        Boy_4th_Solo_B     = {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.GirlPondWater =
{
    model           = "episode3:models/girl_pond_water.mxm",

    animation       = 
    {
        Girl_4th_Solo_B             = { "episode3:models/girl_pond_water.mxa", 1, -1, true, true},
        Boy_4th_Solo_B              = { "episode3:models/girl_pond_water.mxa", 17, -1, true, true},          
    },

    fbdrivers       = 
    {
        Girl_4th_Solo_B     = {},
        Boy_4th_Solo_B      = {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Girl_Pond_Park = 
{
    model           = "episode3:models/girl_pond_park.mxm",
    animation       = 
    {
        Girl_4th_Solo_B 		= { "episode3:models/girl_pond_park.mxa", 351, 886, false, true},
        Girl_4th_Solo_B_Idle 	= { "episode3:models/girl_pond_park.mxa", 886, 886, true, true},
        Girl_4th_Solo_B2 		= { "episode3:models/girl_pond_park.mxa", 887, 911, false, true},
        Girl_4th_Solo_C 		= { "episode3:models/girl_pond_park.mxa", 912, -1, false, true},
        
        Boy_4th_Solo_B          = { "episode3:models/girl_pond_park.mxa", 484, 925, false, true},      
    },
    fbdrivers       = { 
        Girl_4th_Solo_B = {},
        Girl_4th_Solo_B_Idle = {
        	Girl_jumps_in_Pond_City_BG_Mat = {
				bindings = {
					"u_FlipbookTex"
				},
				
				wrapMode = FBWrapMode.loop,
				luaDriven = true,
				fbStart = 1,
				frameOffset = 0,
				frameStartOffset = 0,
				frameStart = 1,
				frameEnd = 30,
				exposureCount = 4,
				
				uri = "episode3:flipbooks/city_bg_cycle.mfb"
			},
        },
        Girl_4th_Solo_B2 = {},
        Girl_4th_Solo_C = {},
        Boy_4th_Solo_B   = {},
    },
    
	-- this actor is always on behind of the girl
    attachTo        = { parentName  = "GirlPond", targetBone = "RootNode"},
    
    defaultMaterial = SceneData.flipbookMatDef,
}


SceneData.actors.GirlTreeBoy = 
{
    actorGroup      = "boy",
    namedBones      = 
    {
        targetBone = "targetBoy",
        offsetBone = "offsetBoy",
    },

    model           = "episode3:models/girl_4th_meeting_boy.mxm",

    animation       = 
    {
        girltree            		= { "episode3:models/girl_4th_meeting_boy.mxa", 361, 779, false, true},
        girltree_dance      		= { "episode3:models/girl_4th_meeting_boy.mxa", 780, 1002, false, true},
        
        girltree_girlblinks  	 	= { "episode3:models/girl_4th_meeting_boy.mxa", 1003, 1102, false, true},
        girltree_girlwalks_a   		= { "episode3:models/girl_4th_meeting_boy.mxa", 1103, 1204, false, true},
        girltree_girlwalks_b   		= { "episode3:models/girl_4th_meeting_boy.mxa", 1205, 1448, false, true},
        girltree_girlwalks_b_Idle   = { "episode3:models/girl_4th_meeting_boy.mxa", 1448, 1448, true, true},
        
        girltree_boyjumps 			= { "episode3:models/girl_4th_meeting_boy.mxa", 1448, 1449, false, true},
        girltree_boyoutro 			= { "episode3:models/girl_4th_meeting_boy.mxa", 1450, -1, false, true},
    },
    
    fbdrivers       = 
    { 
        girltree        	= {},
        girltree_dance  	= {},
        
        girltree_girlblinks = {},
        girltree_girlwalks_a = {},
        girltree_girlwalks_b = {},
        girltree_girlwalks_b_Idle = {
        	Boy_4thMeeting_Boy_Part_02_Mat = {
				bindings = {
					"u_FlipbookTex"
				},
				
				wrapMode = FBWrapMode.loop,
				luaDriven = true,
				fbStart = 1,
				frameOffset = 0,
				frameStartOffset = 0,
				frameStart = 1,
				frameEnd = 4,
				exposureCount = 4,
				
				uri = "episode3:flipbooks/boy_4thmeeting_lookback_idle.mfb"
			},
        },
        
        girltree_boyjumps 	= {},
        girltree_boyoutro 	= {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.GirlTreeBirds = 
{

    model           = "episode3:models/girl_4th_meeting_birds.mxm",

    animation       = 
    {
        girltree    		= { "episode3:models/girl_4th_meeting_birds.mxa", 361, 779, false, true},
        girltree_dance      = { "episode3:models/girl_4th_meeting_birds.mxa", 780, -1, false, true},
    },
    
    fbdrivers       = 
    { 
        girltree    = {},
        girltree_dance  = {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

-----------------------------------------------------------------------------------------------------------
-- Girl tree: Targets


SceneData.actors.GirlTree_Girl_Target = 
{
    keyActor        = "GirlTreeGirl",
    attachTo        = { parentName  = "GirlTreeGirl", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false} }, },
}

SceneData.actors.GirlTree_GirlOutro_Target = 
{
    keyActor        = "GirlTreeGirl",
    attachTo        = { parentName  = "GirlTreeGirl", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

SceneData.actors.GirlTreeBoy_To_4th_Solo = 
{
    keyActor        = "GirlTreeBoy",
    attachTo        = { parentName  = "GirlTreeBoy", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false} }, },
}

SceneData.actors.GirlTreeBoy_To_4th_Solo_Big = 
{
    keyActor        = "GirlTreeBoy",
    attachTo        = { parentName  = "GirlTreeBoy", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

SceneData.actors.Girl_TreeLeaves_Particle1 = 
{
    mpxUri          = "episode3:particles/fallingtreeleavesfront.mxp",
    keyActor        = "GirlTreeTree",
    attachTo        = { parentName  = "GirlTreeTree", targetBone = "leafEmitter01" },
    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/fallingtreeleavesfront.mxp" },
    },
}

SceneData.actors.Girl_TreeLeaves_Particle2 = 
{
    mpxUri          = "episode3:particles/fallingtreeleavesback.mxp",
    keyActor        = "GirlTreeTree",
    attachTo        = { parentName  = "GirlTreeTree", targetBone = "leafEmitter02" },
    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/fallingtreeleavesback.mxp" },
    },
}


-----------------------------------------------------------------------------------------------------------
-- Girl tree: Segments

-- TODO girltree_intro_a

SceneData.disableBounds = false

table.insert(SceneData.segments,
    {   
        name            = "girltree_intro_a", 

        animated        = { "GirlTreeGirl", "GirlTreeDog", },
        active          = { "GirlTreeGirl", "GirlTreeDog", "GirlTree_Girl_Target", },
 
        transitions     = 
        {
            default  =  { nextSegment = "girltree_intro_b", leadActor = "GirlTreeGirl", },
        },
        methods			= 
        {
        	NoActor		= 
            {   
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end,
                disableBounds = function(segment, actor) 
                	if(SceneData.disableBounds) then
                    	MainCamera.controller.boundsLayer.enabled = false
                    end
                end 
	    	},
        }, 
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_intro_b", 
        needsFocus      = true,
        animated        = { "GirlTreeGirl", "GirlTreeDog", "GirlTreeTree", "GirlTreeLeaves"},
        active          = { "GirlTreeGirl", "GirlTree_Girl_Target", "GirlTreeDog", "GirlTreeTree", "GirlTreeLeaves"},
 
        transitions     = 
        {
            GirlTree_Girl_Target  =  { nextSegment = "girltree", leadActor = "GirlTreeGirl", },
        },
        methods			= 
        {
	    	GirlTreeTree = {
	    		show = function(segment, actor)
					actor.components["GirlTreeAnimator"]:changeState("loop");
					actor.components["GirlTreeGodRayAnimator"]:changeState("godrays");
					actor.components["BoyTreeTracebackAnimator"]:changeState("traceback");
				end,
				doIntro = function(segment, actor)
					actor.components["GirlAlphaTreeAnimator"]:changeState("intro");
				end
			},
			GirlTreeLeaves = {
				show = function(segment, actor)
					actor:setVisibility(true);
					actor.components["GirlTreeLeavesAnimator"]:changeState("loop");
					actor.components["GirlAlphaTreeLeavesAnimator"]:changeState("intro");
				end
			}
        }, 
        
        components = {
        	GirlTreeTree = {
    			{ name = "Fader", params = {delay = 0, fadeTime = 1.9, material = "GirlTree_Traceback_Mat"}},
    			{ name = "FrameRateTween", params = {delay=0, duration=1.5, from=60, to=8, mat="traceback", flipbookdriver="GirlTree_Traceback_Mat"} }
        	}
        },
        
        ignoreOffset = true,
        disableGroundParticles = true,
        disableHorizonStars = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree", 
        animated        = { "GirlTreeGirl", "GirlTreeTree", "GirlTreeBoy", "GirlTreeBirds", "GirlTreeLeaves"   },
        active          = { "GirlTreeGirl", "GirlTreeTree", "GirlTreeBoy", "GirlTreeBirds", "GirlTreeLeaves", "GirlTree_Girl_Target", "GirlTreeBoy_To_4th_Solo", "GirlTreeTree", "Tree_Particles_Girl",   },

        transitions     = 
        {
            default  =  { nextSegment = "girltree_dance", leadActor = "GirlTreeGirl", },
        },

        methods			= 
        {
            NoActor		= 
            { 
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end 
            },
            
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_dance", 
        needsFocus		= true,
        animated        = { "GirlTreeGirl", "GirlTreeTree", "GirlTreeBoy",  "GirlTreeBirds", "GirlTreeLeaves" },
        active          = { "GirlTreeGirl", "GirlTreeTree", "GirlTreeBoy", "GirlTreeBirds",  "GirlTreeLeaves", "GirlTree_Girl_Target", "GirlTreeBoy_To_4th_Solo", "Tree_Particles_Girl", "Girl_TreeLeaves_Particle1", "Girl_TreeLeaves_Particle2"  },

        transitions     = 
        {
            GirlTree_Girl_Target  =  { nextSegment = "girltree_girlblinks", leadActor = "GirlTreeGirl", },
            GirlTreeBoy_To_4th_Solo  =  { nextSegment = "girltree_girlblinks", leadActor = "GirlTreeBoy", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)


table.insert(SceneData.segments,
    {   
        name            = "girltree_girlblinks", 
        needsFocus		= true,
        animated        = { "GirlTreeGirl", "GirlTreeBoy", "GirlTreeTree", "GirlTreeLeaves",  },
        active          = { "GirlTreeGirl", "GirlTreeBoy", "GirlTreeTree", "GirlTreeLeaves", "GirlTree_Girl_Target", "GirlTreeBoy_To_4th_Solo", "Tree_Particles_Girl", "Girl_TreeLeaves_Particle1", "Girl_TreeLeaves_Particle2" },
        transitions     = 
        {
            GirlTree_Girl_Target  =  { nextSegment = "girltree_girlwalks_a", leadActor = "GirlTreeGirl", },
            GirlTreeBoy_To_4th_Solo  =  { nextSegment = "girltree_girlwalks_a", leadActor = "GirlTreeBoy", },
        },
        methods			=
        {
            GirlTreeTree = {
                show = function(segment, actor)
                    if(segment.checkpointTriggered) then
                        actor.components["GirlTreeAnimator"]:changeState("loop");
                        actor.components["GirlTreeGodRayAnimator"]:changeState("godrays");
                    end
                end
            },
            GirlTreeLeaves = {
                show = function(segment, actor)
                    if(segment.checkpointTriggered) then
                        actor.components["GirlTreeLeavesAnimator"]:changeState("loop");
                    end
                end
            }
        },
        ignoreOffset = true,
        checkpoint = true,
        disableGroundParticles = true,
        disableHorizonStars = true,
        checkpointLocation = { x = 4000, y = 2500, z = -2000, w = 0 },
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_girlwalks_a", 
        animated        = { "GirlTreeGirl", "GirlTreeBoy", "GirlTreeTree", "GirlTreeLeaves", },
        active          = { "GirlTreeGirl", "GirlTreeBoy", "GirlTreeTree", "GirlTreeLeaves", "GirlTree_Girl_Target", "GirlTreeBoy_To_4th_Solo", "Tree_Particles_Girl", "Girl_TreeLeaves_Particle1", "Girl_TreeLeaves_Particle2" },
        transitions     = 
        {
            default  =  { nextSegment = "girltree_girlwalks_b", leadActor = "GirlTreeGirl", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_girlwalks_b", 
        needsFocus		= true,
        animated        = { "GirlTreeGirl", "GirlTreeBoy", "GirlTreeTree", "GirlTreeLeaves", },
        active          = { "GirlTreeGirl", "GirlTreeBoy", "GirlTreeTree", "GirlTreeLeaves", "GirlTree_Girl_Target", "GirlTreeBoy_To_4th_Solo", "Tree_Particles_Girl", "Girl_TreeLeaves_Particle1", "Girl_TreeLeaves_Particle2" },
        transitions     = 
        {
            GirlTree_Girl_Target  =  { nextSegment = "girltree_girljumps", leadActor = "GirlTreeGirl", },
            GirlTreeBoy_To_4th_Solo  	=  { nextSegment = "girltree_boyjumps", leadActor = "GirlTreeBoy", },
        },
        
        methods = {
        	GirlTreeBoy_To_4th_Solo = {
        		doOffsetTarget = function(segment, actor)
        			actor.components["LookAtDetector"].offset = Vector(0,-450,0)
        		end
        	}
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_girljumps", 

        animated        = { "GirlTreeGirl", "GirlTreeTree", "GirlTreeLeaves", },
        active          = { "GirlTreeGirl", "GirlTreeTree", "GirlTreeLeaves", "GirlTree_Girl_Target", "Tree_Particles_Girl",   },

        transitions     = 
        {
            default  =  { nextSegment = "girltree_girloutro", leadActor = "GirlTreeGirl", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
        prefetchTransitions = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_girloutro", 
        needsFocus      = true,

        animated        = { "GirlTreeGirl", "GirlTreeTree"},
        active          = { "GirlTreeGirl", "GirlTreeTree", "GirlTree_GirlOutro_Target", "Tree_Particles_Girl" },

        transitions     = 
        {
            GirlTree_GirlOutro_Target  =  { nextSegment = "Girl_4th_Solo_B", leadActor = "GirlTreeGirl", },
        },
        
        methods = {
          GirlTreeTree = {
              doOutro = function(segment, actor)
                  actor.components["GirlAlphaTreeAnimator"]:changeState("outro_girl");
              end
          },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_boyjumps", 
		needsFocus      = true,
        animated        = { "GirlTreeTree", "GirlTreeBoy", "GirlTreeLeaves", },
        active          = { "GirlTreeTree", "GirlTreeBoy", "GirlTreeLeaves", "GirlTreeBoy_To_4th_Solo", "Tree_Particles_Girl" },

        transitions     = 
        {
            GirlTreeBoy_To_4th_Solo  =  { nextSegment = "girltree_boyoutro", leadActor = "GirlTreeGirl", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "girltree_boyoutro", 
        needsFocus      = true,

  		animated        = { "GirlTreeTree", "GirlTreeBoy", "GirlTreeDog", "GirlTreeLeaves", },
        active          = { "GirlTreeTree", "GirlTreeBoy", "GirlTreeDog", "GirlTreeLeaves", "GirlTreeBoy_To_4th_Solo_Big", "Tree_Particles_Girl" },

        transitions     = 
        {
            GirlTreeBoy_To_4th_Solo_Big  =  { nextSegment = "Boy_4th_Solo_B", leadActor = "GirlTreeBoy", },
        },
        
        methods = {
	        GirlTreeTree = {
				doOutro = function(segment, actor)
					actor.components["GirlAlphaTreeAnimator"]:changeState("outro_boy");
				end
			},
			GirlTreeLeaves = {
				doOutro = function(segment, actor)
					actor.components["GirlAlphaTreeLeavesAnimator"]:changeState("outro_boy");
				end
			},
		},
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)


-----------------------------------------------------------------------------------------------------------
--
-- Girl 4th Solo
-- 
-----------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------------------------
-- Girl 4th Solo: Characters

SceneData.actors.Girl_4th_Solo_C = 
{
    actorGroup      = "girl",
    model           = "episode3:models/girl_dance.mxm",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",
        leftBound   = "leftGirl",
        rightBound  = "rightGirl",
    },
    animation       = 
    {
        Girl_4th_Solo_C = { "episode3:models/girl_dance.mxa", 0, -1, false, true},
    },
    fbdrivers       = { 
        Girl_4th_Solo_C = {},
    },
    tracking = {
        Girl_4th_Solo_C = false,
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

-----------------------------------------------------------------------------------------------------------
-- Girl 4th Solo: Targets

SceneData.actors.Girl_4th_Meeting_B_to_C = 
{
    keyActor        = "GirlPond",
    attachTo        = { parentName  = "GirlPond", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false} }, },
}

SceneData.actors.Girl_To_5th_Meeting = 
{
    keyActor        = "Girl_4th_Solo_C",
    attachTo        = { parentName  = "Girl_4th_Solo_C", targetBone = "lookGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 750, debugMode = false} }, },
}

-----------------------------------------------------------------------------------------------------------
-- Girl 4th Solo: Segments
-- 

table.insert(SceneData.segments,
    {   
        name            = "Girl_4th_Solo_B", 
        needsFocus      = true,
        animated        = { "GirlPond", "Girl_Pond_Park", "Boy_4th_Solo_B", "GirlPondWater" },
        active          = { "GirlPond", "Girl_Pond_Park", "Boy_4th_Solo_B",  "Girl_4th_Meeting_B_to_C", "Boy_To_5th_Meeting", "Dog_To_5th_Meeting", "GirlPondWater" },
        transitions     = 
        {
            Girl_4th_Meeting_B_to_C  =  { nextSegment = "Girl_4th_Solo_B2", leadActor = "GirlPond", },
           	Boy_To_5th_Meeting  =  { nextSegment = "Fifth_Meeting1", leadActor = "Boy_4th_Solo_B", },	
           	Dog_To_5th_Meeting = {nextSegment = "Fifth_Meeting1", leadActor = "Boy_4th_Solo_B"},
        },
        offsetBy90 = 
        {
	    	Boy_4th_Solo_B = "left"
	    },
        methods			= 
        {
    		NoActor	= 
            {
    			disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = true 
                end,
                enableBounds = function(segment, actor) 
                	if(SceneData.disableBounds) then
                    	MainCamera.controller.boundsLayer.enabled = true
                    end
                end 
    		}
        },
        
        components = {
            GirlPondWater = {
                { name = "PondWaterBlender", params = { fadeTime = 3.0 } },
            }
        },
        disableGroundParticles = true,        
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Girl_4th_Solo_B2", 
        needsFocus      = true,
        animated        = { "GirlPond", "Girl_Pond_Park",},
        active          = { "GirlPond", "Girl_Pond_Park", "Girl_4th_Meeting_B_to_C", "Boy_To_5th_Meeting"  },
        transitions     = 
        {
            Girl_4th_Meeting_B_to_C  =  { nextSegment = "Girl_4th_Solo_C", leadActor = "GirlPond", },
           	Boy_To_5th_Meeting  =  { nextSegment = "Fifth_Meeting1", leadActor = "Boy_4th_Solo_B", },
        },
        ignoreOffset    = true,
        
    }
)

table.insert(SceneData.segments,
    {   
        name            = "Girl_4th_Solo_C", 
        needsFocus = true,
        animated        = { "Girl_4th_Solo_C", "Girl_Pond_Park"},
        active          = { "Girl_4th_Solo_C", "Girl_Pond_Park", "Girl_To_5th_Meeting" },
    
        transitions     = 
        {
           Girl_To_5th_Meeting  =  { nextSegment = "Fifth_Meeting1", leadActor = "Girl_4th_Solo_C", },
        },
        
        components      = 
	    { 
	        Girl_Pond_Park =
            { 
	            {
		            name = "MasterBlender",
		            params = 
		            { 
		                startAlpha = 1,
		                blending = true,
		                inout = "out",                                        
		                flipbook = true,
		                delay    = 0,
		                fadetime = 0.1,
		            },
		        },
	        },
	    },
        
        
    }
)

-----------------------------------------------------------------------------------------------------------
-- Boy tree: Characters
SceneData.actors.BoyTreeGirl = 
{
    actorGroup      = "girl",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",
    },

    model           = "episode3:models/boy_4th_meeting_girl.mxm",

    animation       = 
    {
		boytree_intro_a          = { "episode3:models/boy_4th_meeting_girl.mxa", 1, 263, false, true},
        boytree                  = { "episode3:models/boy_4th_meeting_girl.mxa", 265, 580, false, true},
        
        boytree_dance            = { "episode3:models/boy_4th_meeting_girl.mxa", 581, 780, false, true},
        
        boytree_girlblinks   	= { "episode3:models/boy_4th_meeting_girl.mxa", 781, 880, false, true},
        boytree_girlwalks_a   	= { "episode3:models/boy_4th_meeting_girl.mxa", 881, 1002, false, true},
        boytree_girlwalks_b   	= { "episode3:models/boy_4th_meeting_girl.mxa", 1003, 1259, false, true},
        
		boytree_girljumps		=  { "episode3:models/boy_4th_meeting_girl.mxa", 1535, 1585, false, true},
		boytree_girloutro		=  { "episode3:models/boy_4th_meeting_girl.mxa", 1586, 1749, false, true},
    },
    
    fbdrivers       = 
    { 
    	boytree_intro_a = {},
        boytree         = {},
        boytree_dance   = { },
        boytree_girlblinks = {},
        boytree_girlwalks_a = {},
        boytree_girlwalks_b = {},
        boytree_girljumps = {},		
        boytree_girloutro = {},	
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.BoyTreeBoy = 
{
    actorGroup      = "boy",
    namedBones      = 
    {
        targetBone = "targetBoy",
        offsetBone = "offsetBoy",
        dogGlowTop = "glowTLDog",
        dogGlowBottom = "glowBRDog",
        dogGlowBone = "glowDog",
    },

    model           = "episode3:models/boy_4th_meeting.mxm",

    animation       = 
    {
        boytree_intro_a     = { "episode3:models/boy_4th_meeting.mxa", 1, 263, false, true},
        boytree             = { "episode3:models/boy_4th_meeting.mxa", 264, 580, false, true},
        boytree_dance       = { "episode3:models/boy_4th_meeting.mxa", 581, 780, false, true},
        
        boytree_girlblinks   = { "episode3:models/boy_4th_meeting.mxa", 781, 880, false, true},
        boytree_girlwalks_a   = { "episode3:models/boy_4th_meeting.mxa", 881, 980, false, true},
        boytree_girlwalks_b   = { "episode3:models/boy_4th_meeting.mxa", 981, 1241, false, true},
        boytree_girlwalks_b_Idle   = { "episode3:models/boy_4th_meeting.mxa", 1241, 1241, true, true},
        
        boytree_boyjumps 	= { "episode3:models/boy_4th_meeting.mxa", 1241, 1262, false, true},
        boytree_boyoutro 	= { "episode3:models/boy_4th_meeting.mxa", 1263, 1507, false, true},
    },
    
    fbdrivers       = 
    {
        boytree_intro_a = {},
        boytree         = {},
        boytree_dance   = {},
        boytree_girlblinks = {},
        boytree_girlwalks_a = {},
        boytree_girlwalks_b = {},
        boytree_girlwalks_b_Idle = {
        	Boy_4thMeeting_Boy_Part_02_Mat = {
				bindings = {
					"u_FlipbookTex"
				},
				
				wrapMode = FBWrapMode.loop,
				luaDriven = true,
				fbStart = 1,
				frameOffset = 0,
				frameStartOffset = 0,
				frameStart = 1,
				frameEnd = 4,
				exposureCount = 4,
				
				uri = "episode3:flipbooks/boy_4thmeeting_lookback_idle.mfb"
			},
        },
        boytree_boyjumps = {},
        boytree_boyoutro = {}
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}


SceneData.actors.BoyTreeBirds = 
{

    model           = "episode3:models/boy_4th_meeting_birds.mxm",

    animation       = 
    {
        boytree     	   = { "episode3:models/boy_4th_meeting_birds.mxa", 264, 580, false, true},
        boytree_dance      = { "episode3:models/boy_4th_meeting_birds.mxa", 581, -1, false, true},
    },
    
    fbdrivers       = 
    { 
        boytree        	= {},
        boytree_dance  	= {},
    },
    
    defaultMaterial = SceneData.flipbookMatDef,
}

-----------------------------------------------------------------------------------------------------------
-- Boy tree: Targets
SceneData.actors.Boy_To_Next_Tree =
{
    keyActor        = "BoyTreeBoy",
    attachTo        = { parentName  = "BoyTreeBoy", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 500, debugMode = false} }, },
}

SceneData.actors.Boy_To_Next_Tree_Big = 
{
    keyActor        = "BoyTreeBoy",
    attachTo        = { parentName  = "BoyTreeBoy", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 800, debugMode = false} }, },
}

SceneData.actors.BoyTree_Girl_Target = 
{
    keyActor        = "BoyTreeGirl",
    attachTo        = { parentName  = "BoyTreeGirl", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false} }, },
}

SceneData.actors.BoyTree_GirlOutro_Target = 
{
    keyActor        = "BoyTreeGirl",
    attachTo        = { parentName  = "BoyTreeGirl", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

SceneData.actors.Boy_TreeLeaves_Particle1 = 
{
	mpxUri			= "episode3:particles/fallingtreeleavesfront.mxp",
    keyActor        = "BoyTreeTree",
    attachTo        = { parentName  = "BoyTreeTree", targetBone = "leafEmitter01" },
    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/fallingtreeleavesfront.mxp" },
    },
}

SceneData.actors.Boy_TreeLeaves_Particle2 = 
{
	mpxUri			= "episode3:particles/fallingtreeleavesback.mxp",
    keyActor        = "BoyTreeTree",
    attachTo        = { parentName  = "BoyTreeTree", targetBone = "leafEmitter02" },
    prefetch = {
        { ParticleSystem.rtti, "episode3:particles/fallingtreeleavesback.mxp" },
    },
}



-----------------------------------------------------------------------------------------------------------
-- Boy tree: Segments

-- TODO boytree_intro_a

table.insert(SceneData.segments,
    {   
        name            = "boytree_intro_a", 
        animated        = {"BoyTreeBoy",  "BoyTreeGirl", },
        active          = {"BoyTreeBoy", "BoyTreeTree", "BoyTreeGirl", "Boy_To_Next_Tree_Big",   }, 
        transitions     = 
        {
           default  =  { nextSegment = "boytree", leadActor = "BoyTreeBoy", },
        },
        
        methods			= 
        {
        	NoActor		= 
            { 
				disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = false 
                end,
                disableBounds = function(segment, actor) 
                	if(SceneData.disableBounds) then
                    	MainCamera.controller.boundsLayer.enabled = false
                    end
                end 
			},
			BoyTreeTree = {
				hide = function(segment, actor)
					actor:setVisibility(false)
				end
			}
        },
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree",
        animated        = { "BoyTreeGirl",  "BoyTreeBoy", "BoyTreeLeaves", "BoyTreeTree", "BoyTreeBirds"},
        active          = { "BoyTreeGirl",  "BoyTreeBoy", "BoyTreeLeaves","BoyTreeTree", "BoyTreeBirds", "Boy_To_Next_Tree", "BoyTree_Girl_Target", "Tree_Particles_Boy"},
		
        transitions     = 
        {
            default     =  { nextSegment = "boytree_dance", leadActor = "BoyTreeBoy", }, 
        },
        
        methods			= 
        {
			BoyTreeTree = {
				show = function(segment, actor)
					actor:setVisibility(true);
					actor.components["BoyTreeAnimator"]:changeState("loop");
					actor.components["BoyTreeTracebackAnimator"]:changeState("traceback");
					actor.components["BoyTreeGodRayAnimator"]:changeState("godrays");
					actor.components["BoyAlphaTreeAnimator"]:changeState("intro");
				end
			},
			BoyTreeLeaves = {
				show = function(segment, actor)
					actor:setVisibility(true);
					actor.components["BoyTreeLeavesAnimator"]:changeState("loop");
					actor.components["BoyAlphaTreeLeavesAnimator"]:changeState("intro");
				end
			}
        },
        
        components = {
        	BoyTreeTree = {
    			{ name = "Fader", params = {delay = 0, fadeTime = 1.45, material = "boyTree_Traceback_MAT"}},
    			{ name = "FrameRateTween", params = {delay=0, duration=1.4, from=60, to=8, mat = "traceback", flipbookdriver="boyTree_Traceback_MAT"} }
        	}
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_dance", 
        needsFocus		= true,
        animated        = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "BoyTreeBirds", },
        active          = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "BoyTreeBirds", "BoyTree_Girl_Target", "Boy_To_Next_Tree", "Tree_Particles_Boy", "Boy_TreeLeaves_Particle1", "Boy_TreeLeaves_Particle2" },
        transitions     = 
        {
        	BoyTree_Girl_Target  =  { nextSegment = "boytree_girlblinks", leadActor = "BoyTreeGirl", },
            Boy_To_Next_Tree  =  { nextSegment = "boytree_girlblinks", leadActor = "BoyTreeGirl", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_girlblinks", 
        needsFocus		= true,
        animated        = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", },
        active          = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "BoyTree_Girl_Target", "Boy_To_Next_Tree", "Tree_Particles_Boy", "Boy_TreeLeaves_Particle1", "Boy_TreeLeaves_Particle2"  },
        transitions     = 
        {
            BoyTree_Girl_Target  =  { nextSegment = "boytree_girlwalks_a", leadActor = "BoyTreeGirl", },
            Boy_To_Next_Tree  =  { nextSegment = "boytree_girlwalks_a", leadActor = "BoyTreeGirl", },
        },
        methods			=
        {
            BoyTreeTree = {
                show = function(segment, actor)
                    if(segment.checkpointTriggered) then
                        actor.components["BoyTreeAnimator"]:changeState("loop");
                        actor.components["BoyTreeGodRayAnimator"]:changeState("godrays");
                    end
                end
            },
            BoyTreeLeaves = {
                show = function(segment, actor)
                    if(segment.checkpointTriggered) then
                        actor.components["BoyTreeLeavesAnimator"]:changeState("loop");
                    end
                end
            }
        },
        
--        components = {
--        	BoyTreeTree = {
--    				{ name = "Fader", params = {delay = 0, fadeTime = 0, material = "boyTree_Traceback_MAT"}},
--    		}
--        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
        checkpoint = true,
        checkpointLocation = { x = -648, y = 1306, z = -2123, w = 0 },
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_girlwalks_a", 
        animated        = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", },
        active          = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "BoyTree_Girl_Target", "Boy_To_Next_Tree", "Tree_Particles_Boy", "Boy_TreeLeaves_Particle1", "Boy_TreeLeaves_Particle2" },
        transitions     = 
        {
            default  =  { nextSegment = "boytree_girlwalks_b", leadActor = "BoyTreeGirl", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_girlwalks_b", 
        needsFocus		= true,
        animated        = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", },
        active          = { "BoyTreeGirl", "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "BoyTree_Girl_Target", "Boy_To_Next_Tree", "Tree_Particles_Boy", "Boy_TreeLeaves_Particle1", "Boy_TreeLeaves_Particle2" },
        transitions     = 
        {
            BoyTree_Girl_Target  =  { nextSegment = "boytree_girljumps", leadActor = "BoyTreeGirl", },
            Boy_To_Next_Tree  	=  { nextSegment = "boytree_boyjumps", leadActor = "BoyTreeBoy", },
        },
        
        methods = {
        	Boy_To_Next_Tree = {
        		doOffsetTarget = function(segment, actor)
        			actor.components["LookAtDetector"].offset = Vector(0,-450,0)
        		end
        	}
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_boyjumps", 
        animated        = { "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", },
        active          = { "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "Boy_To_Next_Tree", "Tree_Particles_Boy" },
        transitions     = 
        {
            default  =  { nextSegment = "boytree_boyoutro", leadActor = "BoyTreeBoy", },
        },
        
         methods = {
        	Boy_To_Next_Tree = {
        		doOffsetTarget = function(segment, actor)
        			actor.components["LookAtDetector"].offset = Vector(0,0,0)
        		end
        	}
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_boyoutro", 
        needsFocus		= true,
        animated        = { "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", },
        active          = { "BoyTreeBoy", "BoyTreeTree", "BoyTreeLeaves", "Boy_To_Next_Tree_Big", "Tree_Particles_Boy" },
        transitions     = 
        {
            Boy_To_Next_Tree_Big  =  { nextSegment = "Boy_4th_Solo_B", leadActor = "BoyTreeBoy", },
        },
        
        methods			= 
        {
			BoyTreeTree = {
				hide = function(segment, actor)
					actor.components["BoyAlphaTreeAnimator"]:changeState("outro_boy");
				end
			},
			BoyTreeLeaves = {
				hide = function(segment, actor)
					actor.components["BoyAlphaTreeLeavesAnimator"]:changeState("outro_boy");
				end
			}
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)


table.insert(SceneData.segments,
    {   
        name            = "boytree_girljumps", 
        animated        = { "BoyTreeGirl", "BoyTreeTree", },
        active          = { "BoyTreeGirl", "BoyTreeTree", "BoyTree_Girl_Target", "BoyTree_Girl_Target", "Tree_Particles_Boy" },
        transitions     = 
        {
            default  =  { nextSegment = "boytree_girloutro", leadActor = "BoyTreeGirl", },
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "boytree_girloutro", 
        needsFocus		= true,
        animated        = { "BoyTreeGirl", "BoyTreeTree", },
        active          = { "BoyTreeGirl", "BoyTreeTree",  "BoyTree_GirlOutro_Target", "Tree_Particles_Boy" },
        transitions     = 
        {
            BoyTree_GirlOutro_Target  =  { nextSegment = "Girl_4th_Solo_B", leadActor = "BoyTreeGirl", },
        },
        
        methods			= 
        {
			BoyTreeTree = {
				hide = function(segment, actor)
					actor.components["BoyAlphaTreeAnimator"]:changeState("outro_girl");
				end
			},
        },
        disableGroundParticles = true,
        disableHorizonStars = true,
        prefetchTransitions = true,
        ignoreOffset = true,
    }
)


-----------------------------------------------------------------------------------------------------------
--
-- Boy 4th Solo
-- 
-----------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------------------------
-- Boy 4th Solo: Characters

SceneData.actors.Boy_4th_Solo_B = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_surfs.mxm",
    namedBones      = 
    {
        targetBone = "targetBoy",
        offsetBone = "offsetBoy",
        -- for by dog actorvignette
        dogGlowTop = "glowTLDog",
        dogGlowBottom = "glowBRDog",
        dogGlowBone = "glowDog",
        dogGlowUp = "glowTopDog",
        dogGlowIntensity = "glowAlphaDog",
    },
    
    animation       = 
    {
        Boy_4th_Solo_B  = { "episode3:models/boy_surfs.mxa", 0, -1, false, true},
        Girl_4th_Solo_B     = { "episode3:models/boy_surfs.mxa", 0, -1, false, true},
    },
    
    fbdrivers       = 
    { 
        Boy_4th_Solo_B = {}, 
        Girl_4th_Solo_B = {}, 
    },
    
    overrideFlipbookTechnique = "flipbookColorAlpha",
    
    defaultMaterial = SceneData.flipbookMatDef,
}


-----------------------------------------------------------------------------------------------------------
-- Boy 4th Solo: Targets

SceneData.actors.Boy_To_5th_Meeting = 
{
    keyActor        = "Boy_4th_Solo_B",
    attachTo        = { parentName  = "Boy_4th_Solo_B", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 800, debugMode = false} }, },
}

SceneData.actors.Dog_To_5th_Meeting = 
{
    keyActor        = "Boy_4th_Solo_B",
    attachTo        = { parentName  = "Boy_4th_Solo_B", targetBone = "targetDog" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false} }, },
}

-----------------------------------------------------------------------------------------------------------
-- Boy 4th Solo: Segments
table.insert(SceneData.segments,
    {   
        name            = "Boy_4th_Solo_B", needsFocus = true,
        animated        = { "Boy_4th_Solo_B", "GirlPond", "GirlPondWater", "Girl_Pond_Park"},
        active          = { "Boy_4th_Solo_B", "GirlPond", "GirlPondWater", "Girl_Pond_Park", "Boy_To_5th_Meeting",  "Girl_4th_Meeting_B_to_C", "Dog_To_5th_Meeting" },
   		offsetBy90      = 
        {
	    	GirlPond = "right",
	    	GirlPondWater = "right",
	    },
        transitions     = {
           Dog_To_5th_Meeting = {nextSegment = "Fifth_Meeting1", leadActor = "Boy_4th_Solo_B"},
           Boy_To_5th_Meeting  =  { nextSegment = "Fifth_Meeting1", leadActor = "Boy_4th_Solo_B", },
           Girl_4th_Meeting_B_to_C  =  { nextSegment = "Girl_4th_Solo_C", leadActor = "GirlPond", },
        },
        methods			= 
        {
            NoActor		= 
            {
                disable90DegOffset = function(segment, actor) 
                    MainCamera.controller.enable90DegOffsets = true 
                end,
                enableBounds = function(segment, actor) 
                	if(SceneData.disableBounds) then
                		MainCamera.controller.boundsLayer.enabled = true
                	end
            	end 
        	},
        },
        components = {
            GirlPondWater = {
                { name = "PondWaterBlender", params = { fadeTime = 0.8 } },
            }
        },
        disableGroundParticles = true,        
    }
)


-----------------------------------------------------------------------------------------------------------
--
-- Fifth Meeting
-- 
-----------------------------------------------------------------------------------------------------------


-----------------------------------------------------------------------------------------------------------
-- Fifth Meeting: Characters
local FifthBStart = 420
local FifthBEnd   = 485

SceneData.actors.Girl_5th_Meeting = 
{
    actorGroup      = "girl",
    model           = "episode3:models/girl_5th_meeting.mxm",
    namedBones      = 
    {
        targetBone = "targetGirl",
        offsetBone = "offsetGirl",
    },
    animation       = 
    {
        Fifth_Meeting1   = { "episode3:models/girl_5th_meeting.mxa", 0, 700, false, true},
        Fifth_Meeting2   = { "episode3:models/girl_5th_meeting.mxa", 701, 1400, false, true},
        Fifth_Meeting3   = { "episode3:models/girl_5th_meeting.mxa", 1401, 1862, false, true},
    },
    fbdrivers       = { 
        Fifth_Meeting1 = {},
        Fifth_Meeting2 = {},
        Fifth_Meeting3 = {},
        
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.Boy_5th_Meeting = 
{
    actorGroup      = "boy",
    model           = "episode3:models/boy_5th_meeting.mxm",
    namedBones      = 
    {
        targetBone = "targetBoy",
        offsetBone = "offsetBoy",
        dogGlowTop = "glowTLDog",
        dogGlowBottom = "glowBRDog",
        dogGlowBone = "glowDog",
        dogGlowUp = "glowTopDog",
        dogGlowIntensity = "glowAlphaDog",
    },
    animation       = 
    {
        Fifth_Meeting1   = { "episode3:models/boy_5th_meeting.mxa", 0, 700, false, true},
        Fifth_Meeting2   = { "episode3:models/boy_5th_meeting.mxa", 701, 1400, false, true},
        Fifth_Meeting3   = { "episode3:models/boy_5th_meeting.mxa", 1401, 1915, false, true},   
    },
    fbdrivers       = 
    {       
        Fifth_Meeting1 = {},
        Fifth_Meeting2 = {},
        Fifth_Meeting3 = {},
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

-----------------------------------------------------------------------------------------------------------
-- Fifth Meeting: Targets
SceneData.actors.Girl_5th_Solo= 
{
    keyActor        = "Girl_5th_Meeting",
    attachTo        = { parentName  = "Girl_5th_Meeting", targetBone = "targetGirl" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 1000, debugMode = false} }, },
}

SceneData.actors.Boy_5th_Solo = 
{
    keyActor        = "Boy_5th_Meeting",
    attachTo        = { parentName  = "Boy_5th_Meeting", targetBone = "targetBoy" },
    components      = { { name      = "LookAtDetector", params = {colliderRadius = 700, debugMode = false} }, },
}

-----------------------------------------------------------------------------------------------------------
-- Fifth Meeting: Segments
table.insert(SceneData.segments,
    {   
        name            = "Fifth_Meeting1", 
        needsFocus      = false, 
        animated        = { "Girl_5th_Meeting", "Boy_5th_Meeting", "Gazebo"},
        active          = { "Girl_5th_Meeting", "Boy_5th_Meeting", "Girl_5th_Solo", "Boy_5th_Solo", "Gazebo"},
        transitions     = 
        {
            default     =  
            { 
                nextSegment = "Fifth_Meeting2", 
                leadActor = "Boy_5th_Meeting", 
            },
        },
        checkpoint = true,
        checkpointLocation = { x = 788, y = -32, z = -1960, w = 0 },
        
        methods			= 
        {
			Gazebo = {
				show = function(segment, actor)
					actor.components["GazeboAnimator"]:changeState("intro");
				end
			},
        },
    }
)

table.insert(SceneData.segments,
    {   
        name                = "Fifth_Meeting2", 
        needsFocus          = false, 
        animated            = { "Girl_5th_Meeting", "Boy_5th_Meeting", "Gazebo" },
        active              = { "Girl_5th_Meeting", "Boy_5th_Meeting", "Gazebo", "Girl_5th_Solo", "Boy_5th_Solo", },
        ignoreOffset        = true,
        disableHorizonStars = true,
        transitions         = 
        {
            default         =  
            { 
                nextSegment = "Fifth_Meeting3", 
                leadActor = "Boy_5th_Meeting", 
            },
        },
    }
)
table.insert(SceneData.segments,
    {   
        name                = "Fifth_Meeting3", 
        needsFocus          = false, 
        disableHorizonStars = true,
        animated            = { "Boy_5th_Meeting", "Girl_5th_Meeting", },
        active              = { "Boy_5th_Meeting", "Girl_5th_Meeting", "Girl_5th_Solo", "Boy_5th_Solo" },
        ignoreOffset        = true,
        transitions         = 
        {
            Boy_5th_Solo     =  
            { 
                nextSegment = "End_Boy", 
                leadActor   = "Boy_5th_Meeting", 
            },
            Girl_5th_Solo    =  
            { 
                nextSegment = "End_Girl", 
                leadActor   = "Girl_5th_Meeting", 
            },
            default     =  
            { 
                nextSegment = "End_Boy", 
                leadActor = "Boy_5th_Meeting", 
            },
        }
    }
)

-----------------------------------------------------------------------------------------------------------
--
-- End
-- 
-----------------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------------------
-- End : Characters
SceneData.actors.End_Girl = 
{
    actorGroup           = "girl",
    model                = "episode3:models/girl_end.mxm",
    namedBones           = 
    {
        targetBone       = "targetGirl",
    },
    animation       = 
    {
        End_Girl        = { "episode3:models/girl_end.mxa", 0, 1537, false, true},
    },
    fbdrivers       = 
    { 
        End_Girl    = {},
    },

    components      = 
    { 
        {   
            name    = "ActorMover", 
            params  = 
            {
                blendTimeArray      = { 0.0,     4.0,    13.0,    17.0 },
                lockArray           = { 0.0,     0.0,    0.0,     0.0 },
                innerLockArray      = { 0.0,     0.0,    0.0,     0.0 },
                samples      		= { 0.0,     4.0,    4.0,     0.0, },
                lerpSpeeds			= { 0.0,     32.0,    2.0,     2.0,  },

                lockAngle           = 2.0, 
                lockToTrackingBone  = true, 
                startAtCamera       = true,
            },
        },
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 1,
                blending = true,
                inout = "out",                                        
                flipbook = true,
                delay    = 26,
                fadetime = 0.1,
            },
        },
        {
            name = "GlowFader",
            params = 
            { 
                delay    = 25
            },
        },
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

SceneData.actors.End_Boy = 
{
    actorGroup           = "boy",
    model                = "episode3:models/boy_end.mxm",
    namedBones           = 
    {
        targetBone       = "targetBoy",
    },
    animation       = 
    {
--        End_Girl        = { "episode3:models/girl_end.mxa", 0, 1159, false, true},
        End_Boy        = { "episode3:models/boy_end.mxa", 0, 1523, false, true},
    },
    fbdrivers       = 
    { 
        End_Boy    = {},
    },

    components      = 
    { 
        {   
            name    = "ActorMover", 
            params  = 
            {
                blendTimeArray      = { 0.0,     4.0,    13.0,    17.0 },
                lockArray           = { 0.0,     0.0,    0.0,     0.0 },
                innerLockArray      = { 0.0,     0.0,    0.0,     0.0 },
                samples      		= { 0.0,     4.0,    4.0,     0.0, },
                lerpSpeeds			= { 0.0,     32.0,    2.0,     2.0,  },

                lockAngle           = 2.0, 
                lockToTrackingBone  = true, 
                startAtCamera       = true,
            },
        },
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 1,
                blending = true,
                inout = "out",                                        
                flipbook = true,
                delay    = 25.5,
                fadetime = 0.1,
            },
        },
        {
            name = "GlowFader",
            params = 
            { 
                delay    = 24.9
            },
        },
    },

    defaultMaterial = SceneData.flipbookMatDef,
}

-----------------------------------------------------------------------------------------------------------
-- End: Targets
SceneData.actors.End_Target_Girl = 
{
    keyActor        = "End_Girl",
    attachTo        = 
    { 
        parentName  = "End_Girl", 
        targetBone  = "targetGirl" 
    },
    components      =   
    { 
        { 
            name    = "LookAtDetector", 
            params  = 
            {
                colliderRadius = 500, 
                debugMode = false
            } 
        }, 
    },
}

SceneData.actors.End_Target_Choice1 = 
{
    keyActor        = "End_Girl",
    attachTo        = 
    { 
        parentName  = "End_Girl", 
        targetBone  = "targetGirl" 
    },
    components      =   
    { 
        { 
            name    = "LookAtDetector", 
            params  = 
            {
                colliderRadius = 5000, 
                debugMode = false
            } 
        }, 
    },
}

SceneData.actors.End_Target_Choice2 = 
{
    keyActor        = "End_Girl",
    attachTo        = 
    { 
        parentName  = "End_Girl", 
        targetBone  = "targetGirl" 
    },
    components      =   
    { 
        { 
            name    = "LookAtDetector", 
            params  = 
            {
                colliderRadius = 5000, 
                debugMode = false
            } 
        }, 
    },
}

-----------------------------------------------------------------------------------------------------------
-- End: Segments
table.insert(SceneData.segments,
    {   
        name            = "End_Girl", 

        animated        = { "End_Girl", },
        active          = { "End_Girl", "End_Target_Girl", "End_Target_Choice1", "End_Target_Choice2"},
 
        transitions     = {

        },

        components = 
        {
			End_Girl =
            { 
                {   
                    name = "ShowEnder",  
                    params = 
                    { 
                        timer = 26,
                    }, 
                }, 
            },      
        },        

        disableGroundParticles = true,
        disableHorizonStars = true,
    }
)

table.insert(SceneData.segments,
    {   
        name            = "End_Boy", 

        animated        = { "End_Boy", },
        active          = { "End_Boy" },
 
        transitions     = {

        },

        components =
        {
            End_Boy =
            { 
                {   
                    name = "ShowEnder",  
                    params = 
                    { 
                        timer = 26,
                    }, 
                }, 
            },      
        },        

        disableGroundParticles = true,
        disableHorizonStars = true,
    }
)

-----------------------------------------------------------------------------------------------------------


-- Used to draw the glow for the actors
SceneData.actors.GlowPlane = 
{
    model = "episode3:models/screenquad.mxm",
    position = { 0.0,0.0, -2.0},
    materials = 
    {
        { 
            name = "Mat_Quad", shader = "episode3:shaders/unlit.rsfx",
            technique = "glow",
            renderGroup = 150,            
        },
    },    
}

SceneData.actors.Vignette = 
{
    model = "episode3:models/screenquad.mxm",
    position = { 0.0,0.0, -1.0},
    materials = 
    {
        { 
            name = "Mat_Quad", shader = "episode3:shaders/unlit.rsfx",
            technique = "vignette",
            renderGroup = 151,
            textures = 
            { 
                u_MainTex = "episode3:textures/vignettev2.dds"
            },
            parameters = {u_Texture_ST = Vector(1,1,0,0), u_VignetteColor = Vector(0,0.0078,0.1725,1), u_Alpha = 1},
        },        
    },
    components =
    {
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 0,
                blending = false,
                segmentData = 
                {
                    { 
                        segment = "girltree", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "boytree", 
                        startAlpha = 0, 
                        blending = true, 
                        inout = "in",
                    },
                    { 
                        segment = "girltree_girlblinks", 
                        startAlpha = 1, 
                        blending = false, 
                    },
                    { 
                        segment = "boytree_girlblinks", 
                        startAlpha = 1, 
                        blending = false, 
                    },
                    { 
                        segment = "girltree_girloutro", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },
                    { 
                        segment = "girltree_boyoutro", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },
                    { 
                        segment = "boytree_boyoutro", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },
                    { 
                        segment = "boytree_girloutro", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                    },                    
                }
            }
        }        
    }
}     

-- Stars in the sky
SceneData.actors.HorizonStars =
{
    model         = "episode3:models/horizonstars.mxm",
    dumpMaterials = true,
    scale         = {1,1,1},
    animation     = {},
    flipbooks     = {},
    ignoreSegmentOffsets = true,
    materials     = 
    {
        { 
            name        = "stars_Mat", 
            shader      = "episode3:shaders/waves.rsfx",
            technique   = "waves",
            textures    = { u_diffuseMap1 = "episode3:textures/cloudstars_sky/cloudsstars_sky.0001.dds",
            				u_diffuseMap2 = "episode3:textures/cloudstars_sky/cloudsstars_sky.0002.dds",
            				u_diffuseMap3 = "episode3:textures/cloudstars_sky/cloudsstars_sky.0003.dds",
            				u_diffuseMap4 = "episode3:textures/cloudstars_sky/cloudsstars_sky.0004.dds",
            				u_diffuseMap5 = "episode3:textures/cloudstars_sky/cloudsstars_sky.0005.dds"
            			  },
            renderGroup = 20,  
            parameters  = { u_WaveColor = Vector(1,1,1,1), u_duration = 1.0 }
        },
        { 
            name        = "cloud_A_Mat", 
            shader      = "episode3:shaders/waves.rsfx",
            technique   = "waves",
            textures    = { u_diffuseMap1 = "episode3:textures/cloud_a/clouds_a.0001.dds",
            				u_diffuseMap2 = "episode3:textures/cloud_a/clouds_a.0002.dds",
            				u_diffuseMap3 = "episode3:textures/cloud_a/clouds_a.0003.dds",
            				u_diffuseMap4 = "episode3:textures/cloud_a/clouds_a.0004.dds",
            				u_diffuseMap5 = "episode3:textures/cloud_a/clouds_a.0005.dds"
            			  },
            renderGroup = 20,  
            parameters  = { u_WaveColor = Vector(1,1,1,0.15), u_duration = 2.8 }
        },
        { 
            name        = "Cloud_B_Mat", 
            shader      = "episode3:shaders/waves.rsfx",
            technique   = "waves",
            textures    = { u_diffuseMap1 = "episode3:textures/cloud_b/clouds_b.0001.dds",
            				u_diffuseMap2 = "episode3:textures/cloud_b/clouds_b.0002.dds",
            				u_diffuseMap3 = "episode3:textures/cloud_b/clouds_b.0003.dds",
            				u_diffuseMap4 = "episode3:textures/cloud_b/clouds_b.0004.dds",
            				u_diffuseMap5 = "episode3:textures/cloud_b/clouds_b.0005.dds"
            			  },
            renderGroup = 20,  
            parameters  = { u_WaveColor = Vector(1,1,1,0.15),  u_duration = 3.2 }
        },
        { 
            name        = "Cloud_C_Mat", 
            shader      = "episode3:shaders/waves.rsfx",
            technique   = "waves",
            textures    = { u_diffuseMap1 = "episode3:textures/cloud_c/clouds_c.0001.dds",
            				u_diffuseMap2 = "episode3:textures/cloud_c/clouds_c.0002.dds",
            				u_diffuseMap3 = "episode3:textures/cloud_c/clouds_c.0003.dds",
            				u_diffuseMap4 = "episode3:textures/cloud_c/clouds_c.0004.dds",
            				u_diffuseMap5 = "episode3:textures/cloud_c/clouds_c.0005.dds"
            			  },
            renderGroup = 20,  
            parameters  = { u_WaveColor = Vector(1,1,1,0.15),  u_duration = 3.0 }
        },
    },

    components = 
    {
        {
            name = "MasterBlender",
            params = 
            { 
                startAlpha = 0,
                blending = true,
                inout = "in",                    
                segmentData = 
                {
                    { 
                        segment = "Third_Meeting_End_Girl", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                        delay = 1,
                    },
                    { 
                        segment = "Third_Meeting_End_Boy", 
                        startAlpha = 1, 
                        blending = true, 
                        inout = "out",
                        delay = 1,
                    },
                    { 
                        segment = "Third_Meeting_End_Boy_Alt", 
                        startAlpha = 1, 
                        blending = true, 
                        fadetime = 0.5, 
                        inout = "out",
                    },                          
                    { 
                        segment = "Fifth_Meeting1", 
                        startAlpha = 1,                         
                        blending = true,
                        fadetime = 2, 
                        inout = "out",
                        delay = 10,
                    }, 
                }
            }
        },
    },
}


require "episode3/scripts/scenedata/scenes/duet/boytree"
require "episode3/scripts/scenedata/scenes/duet/boyleaves"
require "episode3/scripts/scenedata/scenes/duet/girlleaves"
require "episode3/scripts/scenedata/scenes/duet/girltree"
require "episode3/scripts/scenedata/scenes/duet/gazebo"

-- Return a table of Flipbook resources needed for the given actor def
local function fbResourcesNeeded(actorDef)
    local resources = {}
    if actorDef.fbdrivers then
        for _, stateMats in pairs(actorDef.fbdrivers) do
            for _, matDef in pairs(stateMats) do
                if matDef.uri then
                    table.insert(resources, { Flipbook.rtti, matDef.uri })
                end
            end
        end
    end
    return resources
end

-- TODO: we really need to make the segment table a hash table instead of an array
local segmentByName = {}
for _, segmentData in ipairs(SceneData.segments) do
    segmentByName[segmentData.name] = segmentData
end

local function addSegmentMethod(segmentName, methodName, fn)
    local segmentData = segmentByName[segmentName]
    if segmentData then
        segmentData.methods = segmentData.methods or {}
        segmentData.methods.methodName = { methodName = fn }
    end
end

-- Patch in prefetching the tree flipbooks early, so they are there in time when we hit the tree segments
-- This is just a stop-gap until we can find a better way to handle these resources
local secondMeeting3Segment = segmentByName.Second_Meeting3
local thirdMeeting1Segment = segmentByName.Third_Meeting1

local function patchPrefetchGroups(segmentName, resources)
    local segment = segmentByName[segmentName]
    if segment then
        segment.prefetchGroups = segment.prefetchGroups or {}
        for name, resource in pairs(resources) do
            segment.prefetchGroups[name] = resource
        end
    end
end

local boytreeResources ={
    BoyTreeTreeFBs = fbResourcesNeeded(SceneData.actors.BoyTreeTree),
}
local girltreeResources = {
    GirlTreeTreeFBs = fbResourcesNeeded(SceneData.actors.GirlTreeTree)
}

local postTreeResources = {
    GirlPondFB1 = fbResourcesNeeded(SceneData.actors.Girl_4th_Meeting_B_to_C),
    GirlPondFB2 = fbResourcesNeeded(SceneData.actors.GirlPond),
    GirlPondFB3 = fbResourcesNeeded(SceneData.actors.Dog_To_5th_Meeting),
    GirlPondFB4 = fbResourcesNeeded(SceneData.actors.Girl_Pond_Park),
    GirlPondFB5 = fbResourcesNeeded(SceneData.actors.Boy_4th_Solo_B),
    GirlPondFB6 = fbResourcesNeeded(SceneData.actors.Boy_To_5th_Meeting),
    GirlPondFB7 = fbResourcesNeeded(SceneData.actors.GirlPondWater),
}

patchPrefetchGroups("Second_Meeting3", boytreeResources)
patchPrefetchGroups("Second_Meeting3", girltreeResources)
-- Also include the following, as they are checkpoints and will need to have these loaded as well
patchPrefetchGroups("Third_Meeting1", boytreeResources)
patchPrefetchGroups("Third_Meeting1", girltreeResources)
patchPrefetchGroups("boytree_girlblinks", boytreeResources)
patchPrefetchGroups("girltree_girlblinks", girltreeResources)
patchPrefetchGroups("girltree_girlblinks", postTreeResources)
patchPrefetchGroups("boytree_girlblinks", postTreeResources)



-- Make sure we release the boy tree assets at the right segments
local function ReleaseBoyTreeFBs()
    ResourceLoader:releaseGroup("BoyTreeTreeFBs")
end
addSegmentMethod("Third_Meeting_End_Girl", "ReleaseBoyTreeFBs", ReleaseBoyTreeFBs)
addSegmentMethod("Boy_4th_Solo_B", "ReleaseBoyTreeFBs", ReleaseBoyTreeFBs)
addSegmentMethod("Girl_4th_Solo_B", "ReleaseBoyTreeFBs", ReleaseBoyTreeFBs)

-- Make sure we release the girl tree assets at the right segments
local function ReleaseGirlTreeFBs()
    ResourceLoader:releaseGroup("GirlTreeTreeFBs")
end
addSegmentMethod("Third_Meeting_End_Boy", "ReleaseGirlTreeFBs", ReleaseGirlTreeFBs)
addSegmentMethod("Third_Meeting_End_Boy_Alt", "ReleaseGirlTreeFBs", ReleaseGirlTreeFBs)
addSegmentMethod("Boy_4th_Solo_B", "ReleaseGirlTreeFBs", ReleaseGirlTreeFBs)
addSegmentMethod("Girl_4th_Solo_B", "ReleaseGirlTreeFBs", ReleaseGirlTreeFBs)


-- Add default actors to all segments
for _, segment in pairs(SceneData.segments) do
	if (not segment.disableAllAutoActors) then

        table.insert(segment.active, "Origin")
	    if (not segment.disableBGStar) then
            if (not segment.disableHorizonStars) then
    			table.insert(segment.active, "HorizonStars")
            end
		end        

		if ActorManager.enableParticles then
			 if (not segment.disableGroundParticles) then
			 	table.insert(segment.active, "Ground_Particles" )
			 end             
		end
     end
    if segment.checkpoint then
        segment.prefetch = segment.prefetch or {}
        table.insert(segment.prefetch, SceneData.ShowStartActors)
    end
end

-- Add common properties to all of the actors
for _, actor in pairs(SceneData.actors) do

    if(actor.actorGroup == "dog" or actor.actorGroup == "boy" or actor.actorGroup == "girl") then
        actor.components = actor.components or {}
    end

     -- Auto assign glow bones
    if(actor.actorGroup == "boy" and (actor.disableGlow == nil or false)) then            
        actor.namedBones = actor.namedBones or {}
        actor.namedBones.glowTop        = "glowTLBoy"
        actor.namedBones.glowBottom     = "glowBRBoy"
        actor.namedBones.glowBone       = "glowBoy"
        actor.namedBones.glowUp         = "glowTopBoy"
        actor.namedBones.glowIntensity  = "glowAlphaBoy"
        actor.namedBones.depthY         = "depthYBoy"
    elseif(actor.actorGroup == "girl" and (actor.disableGlow == nil or false)) then
        actor.namedBones = actor.namedBones or {}
        actor.namedBones.glowTop        = "glowTLGirl"
        actor.namedBones.glowBottom     = "glowBRGirl"
        actor.namedBones.glowBone       = "glowGirl"
        actor.namedBones.glowUp         = "glowTopGirl"
        actor.namedBones.glowIntensity  = "glowAlphaGirl"
        actor.namedBones.depthY         = "depthYGirl"
    elseif(actor.actorGroup == "dog" and (actor.disableGlow == nil or false)) then
        actor.namedBones = actor.namedBones or {}
        actor.namedBones.glowTop        = "glowTLDog"
        actor.namedBones.glowBottom     = "glowBRDog"
        actor.namedBones.glowBone       = "glowDog"
        actor.namedBones.glowUp         = "glowTopDog"
        actor.namedBones.glowIntensity  = "glowAlphaDog"
    end 
    
    -- Turn on debug info
    if actor.model then
        actor.model = string.lower(actor.model)
        if actor.dumpBones == nil then
            actor.dumpBones = true
        end
        if actor.dumpMaterials == nil then
            actor.dumpMaterials = true;
        end
        if(actor.disableInterp == nil) then
            actor.disableInterp = true;
        end
    end
    
    -- Add post process functions
    actor.postProcess = actor.postProcess or {}
    -- Add render view flags
    if(not actor.viewFlags) then
   		actor.viewFlags = bit.bor( VF_MAIN_VIEW );
    end

end


-- Load the baked data and inject it here
SceneData.bakedActorData = {}
pcall(function() SceneData.bakedActorData = require(Project.persistSceneDir .. "baked_actor") end)
SceneData.bakedActorData = SceneData.bakedActorData or {}
