
SceneData = SceneData or {}
SceneData.audio = {}

local function uri(basename)
    return "episode3:audio/" .. basename .. ".ogg"
end

local audio = SceneData.audio

audio.frameRate = 60
audio.frameLatency = 10

local DefaultHeadingCurve = {
    { heading =   0, volume = 1, pan = 0, },
    { heading = 0.1, volume = 1, pan = 0, },
    { heading = 0.3, volume = 0, pan = 0, },

    speed = 1.5,
    symmetric = true,
}

local StabHeadingCurve = {
    { heading =   0, volume = 1, pan = 0, },
    { heading = 0.1, volume = 1, pan = 0, },
    { heading = 0.3, volume = 0, pan = 0, },

    speed = 2,
    symmetric = true,
}

local dogTarget = { { actor = "dog" }, { actor = "boy", bone = "dogGlowBone"} }
local DogStemHeadingCurve = {
    { heading =   0, volume = 0.4, pan = 0, },
    { heading = 0.1, volume = 0.4, pan = 0, },
    { heading = 0.3, volume = 0.1, pan = 0, },

    speed = 1.5,
    symmetric = true,
}

-- Special fades for tracking the characters in the tree
local TreeTrackingTargets = { "boy", "girl", closest = true }
local TreeTrackingHeadingCurve = {
    { heading =     0, volume = 1, pan = 0, },
    { heading = 0.125, volume = 1, pan = 0, },
    { heading = 0.275, volume = 0.1463625, pan = 0, },

    speed = 1,
    symmetric = true,
}

local AmbientTreeTrackingHeadingCurve = {
    { heading =    0, volume = 1, pan = 0, },
    { heading = 0.75, volume = 1, pan = 0, },
    { heading =    1, volume = 0, pan = 0, },

    speed = 2,
    symmetric = true,
}

local OnScreenTrackingTargets = { "boy", "girl", "dog", closest = true }
local OnScreenTrackingHeadingCurve = {
    { heading =    0, volume =   1, pan = 0, },
    { heading = 0.35, volume =   1, pan = 0 },
    { heading = 0.48, volume =   0, pan = 0 },

    speed = 2,
    symmetric = true,
}

local AmbientVerticalCurve = {
    { pitch = 0.10, volume = 0,   },
    { pitch = 0.20, volume = 0.3, },

    speed = 1,
}

local CheckpointFadeDuration = 1

audio.types = {
    Bed =           { enabled = true, masterVolume = 1, looping = true, autorelease = true, masterTrackingTargets = OnScreenTrackingTargets, masterTrackingCurve = OnScreenTrackingHeadingCurve },
    Stem =          { enabled = true, masterVolume = 1, looping = true, trackingCurve = DefaultHeadingCurve,  masterTrackingTargets = OnScreenTrackingTargets, masterTrackingCurve = OnScreenTrackingHeadingCurve },
    Stab =          { enabled = true, masterVolume = 1, autorelease = true, volume = 0.8, trackingCurve = StabHeadingCurve },
    AmbientTree =   { enabled = true, masterVolume = 1, looping = true, },
    AmbientStem =   { enabled = true, masterVolume = 1, looping = true, verticalAttenuationCurve = AmbientVerticalCurve, },
}

audio.sounds = {
    mxstab_intro = { uri = uri("mxloop_intro"), type = "Stab" },

    boy_cell_birth_alt_57 = { uri = uri("boy_cell_birth_alt_57"), type = "Stab" },
    girl_birth_alt_47 =     { uri = uri("girl_birth_alt_47"), type = "Stab" },

    a02_491_stem_dog =      { uri = uri("a02_491_stem_dog"), type = "Stem", trackingTargets = { dogTarget }, trackingCurve = DogStemHeadingCurve },
    a02_491_stem_boy =      { uri = uri("a02_491_stem_boy"), type = "Stem", trackingTargets = { "boy" }, },
    a02_491_stem_meddry =   { uri = uri("a02_491_stem_meddry"), type = "Stem", },
    a02_491_stem_piano =    { uri = uri("a02_491_stem_piano"), type = "Stem", quantizedBed = true, },
    a02_491_stem_skipping = { uri = uri("a02_491_stem_skipping"), type = "Stem", trackingTargets = { "girl" }, },
    a02_491_stem_slowwet =  { uri = uri("a02_491_stem_slowwet"), type = "Stem", },
    a02_491_stem_vocals =   { uri = uri("a02_491_stem_vocals"), type = "Stem", }, -- minVolume = 0.2, },
    birth_stem_stars =      { uri = uri("birth_stem_stars"), type = "AmbientStem", volume = 1, },

    a03_1088_stab_girldrop = { uri = uri("a03_1088_stab_girldrop"), type = "Stab", trackingTargets = { "girl" }, },
    a20_676_stab_gaze =      { uri = uri("a20_676_stab_gaze"), type = "Stab", },
    a04_45_stab_somersault = { uri = uri("a04_45_stab_somersault"), type = "Stab", trackingTargets = { "girl" }, },

    a11_489_stab_glidevox =  { uri = uri("a11_489_stab_glidevox"), type = "Stab", trackingTargets = { "girl" }, },

    a12_900q_stab_voxrisers =  { uri = uri("a12_900q_stab_voxrisers"), type = "Stab", quantized = 1.25, trackingTargets = { "girl" }, },
    a12_664_stab_leaveskick =  { uri = uri("a12_664_stab_leaveskick"), type = "Stab", trackingTargets = { "girl" }, },

    a20_188_bed_canopy1 =  { uri = uri("a20_188_bed_canopy1"), type = "Bed", },
    a20_276_bed_canopy2 =  { uri = uri("a20_276_bed_canopy2"), type = "Bed", },
    a31_195_bed_canopy1 =  { uri = uri("a31_195_bed_canopy1"), type = "Bed", },

    a15_933_stab_woosh =  { uri = uri("a15_933_stab_woosh"), type = "Stab", trackingTargets = { "boy" }, },

    a15_979_stem_boy =           { uri = uri("a15_979_stem_boy"), type = "Stem", trackingTargets = { "boy" }, },
    a15_979_stem_boypercussion = { uri = uri("a15_979_stem_boypercussion"), type = "Stem", trackingTargets = { "boy" }, },
    a15_979_stem_ahvox =         { uri = uri("a15_979_stem_ahvox"), type = "Stem", trackingTargets = { "boy" }, },

    forestambience = { uri = uri("forestambience"), type = "AmbientTree", },

    girlwalks_1_girlstands = { uri = uri("girlwalks_1_girlstands"), type = "Stab", trackingTargets = { "girl" }, },

    a25_228_stem_strings = { uri = uri("a25_228_stem_strings"), type = "Stem", trackingTargets = { "girl" }, },
    a25_228_stem_main =    { uri = uri("a25_228_stem_main"), type = "Stem", trackingTargets = { "girl" }, },
    a25_228_stem_wasser =  { uri = uri("a25_228_stem_wasser"), type = "Stem", trackingTargets = { "girl" }, },
    tree_stem_stars =      { uri = uri("tree_stem_stars"), type = "AmbientStem", volume = 1, },

    a25_230_pirouette_jump =  { uri = uri("a25_230_pirouette_jump"), type = "Stab", trackingTargets = { "girl" }, },

    girl_4th_757_birds =      { uri = uri("girl_4th_757_birds"), type = "Stab", trackingTargets = { "girl" }, },
    girl_4th_1753_limb_leap = { uri = uri("girl_4th_1753_limb_leap"), type = "Stab", trackingTargets = { "girl" }, },
    girl_pond_365_splash =    { uri = uri("girl_pond_365_splash"), type = "Stab", trackingTargets = { "girl" }, },

    a28_675_stab_turn =       { uri = uri("a28_675_stab_turn"), type = "Stab", },
    a28_1958_stab_end =       { uri = uri("a28_1958_stab_end"), type = "Stab", },
    a29_533_stab_dogpullboy = { uri = uri("a29_533_stab_dogpullboy"), type = "Stab", },
    a28_21_stab_tripcatch =   { uri = uri("a28_21_stab_tripcatch"), type = "Stab", trackingTargets = { "girl" }, },
    a28_945_stab_glidevox =   { uri = uri("a28_945_stab_glidevox"), type = "Stab", trackingTargets = { "girl" }, },

    a29_701_stem_boy =         { uri = uri("a29_701_stem_boy"), type = "Stem", trackingTargets = { "boy" }, },

    a28_701_stem_ostinato =    { uri = uri("a28_701_stem_ostinato"), type = "Stem", trackingTargets = { "boy", "girl" }, },
    a28_701_stem_pirouette =   { uri = uri("a28_701_stem_pirouette"), type = "Stem", trackingTargets = { "boy", "girl" }, },
    a28_701_stem_celloyearn =  { uri = uri("a28_701_stem_celloyearn"), type = "Stem", trackingTargets = { "boy", "girl" }, },
    a28_701_stem_melody =      { uri = uri("a28_701_stem_melody"), type = "Stem", trackingTargets = { "boy", "girl" }, },
    a28_701_stem_arcobass =    { uri = uri("a28_701_stem_arcobass"), type = "Stem", },
    a28_701_stem_piano =       { uri = uri("a28_701_stem_piano"), type = "Stem", },

}

audio.segments = {

    First_Meeting = {
        checkpoint = {
            { play = "birth_stem_stars", volume = 0, },
            { play = "a02_491_stem_dog", volume = 0, },
            { play = "a02_491_stem_boy", volume = 0, },
            { play = "a02_491_stem_meddry", volume = 0, },
            { play = "a02_491_stem_piano", volume = 0, },
            { play = "a02_491_stem_skipping", volume = 0, },
            { fade = "a02_491_stem_slowwet", duration = CheckpointFadeDuration, to = 0.8, },
            { fade = "a02_491_stem_vocals", duration = CheckpointFadeDuration , to = 0.7, },
        },
    },

    Second_Meeting1 = {
        checkpoint = {
            { fade = "birth_stem_stars", duration = CheckpointFadeDuration, to = 1, },
            { fade = "a02_491_stem_dog", duration = CheckpointFadeDuration, to = 1, },
            { play = "a02_491_stem_boy", volume = 0, },
            { fade = "a02_491_stem_meddry", duration = CheckpointFadeDuration, to = 0.8, },
            { play = "a02_491_stem_piano", volume = 0, },
            { play = "a02_491_stem_skipping", volume = 0, },
            { play = "a02_491_stem_slowwet", volume = 0, },
            { fade = "a02_491_stem_vocals", duration = CheckpointFadeDuration , to = 0.7, },
        },
    },

    Third_Meeting1 = {
        checkpoint = {
            { fade = "birth_stem_stars", duration = CheckpointFadeDuration, to = 1, },
            { fade = "a02_491_stem_dog", duration = CheckpointFadeDuration, to = 1, },
            { fade = "a02_491_stem_boy", duration = CheckpointFadeDuration, to = 0.8, },
            { fade = "a02_491_stem_meddry", duration = CheckpointFadeDuration, to = 0.8, },
            { fade = "a02_491_stem_piano", duration = CheckpointFadeDuration, to = 0.8, },
            { play = "a02_491_stem_skipping", volume = 0, },
            { fade = "a02_491_stem_slowwet", duration = CheckpointFadeDuration, to = 0.4, },
            { fade = "a02_491_stem_vocals", duration = CheckpointFadeDuration, to = 0.7, },
        },
    },

    boytree_girlblinks = {
        checkpoint = {
            { fade = "forestambience", duration = CheckpointFadeDuration, to = 0.2, },
            { fade = "a20_188_bed_canopy1", duration = CheckpointFadeDuration, to = 0.8, },
            { fade = "a20_188_bed_canopy2", frameOffset = 350, duration = CheckpointFadeDuration, to = 0.8, },
        },
    },

    girltree_girlblinks = {
        checkpoint = {
            { fade = "forestambience", duration = CheckpointFadeDuration, to = 0.2, },
            { fade = "a20_188_bed_canopy1", duration = CheckpointFadeDuration, to = 0.8, },
            { fade = "a20_188_bed_canopy2", frameOffset = 350, duration = CheckpointFadeDuration, to = 0.8, },
        },
    },

    girltree_girlwalks_a = {
        events = {
            { masterVolume = "Bed", masterTrackingTargets = OnScreenTrackingTargets, masterTrackingCurve = OnScreenTrackingHeadingCurve },
            { masterVolume = "Stem", masterTrackingTargets = OnScreenTrackingTargets, masterTrackingCurve = OnScreenTrackingHeadingCurve },
        },
    },

    girltree_girloutro = {
        events = {
            { fade = "forestambience", duration = 1, to = 0, autorelease = true, },
            { masterVolume = "AmbientTree", masterTrackingTargets = nil, masterTrackingCurve = nil },
        },
    },

    girltree_boyoutro = {
        events = {
            { fade = "forestambience", duration = 1, to = 0, autorelease = true, },
            { masterVolume = "AmbientTree", masterTrackingTargets = nil, masterTrackingCurve = nil },
        },
    },

    boytree = {
        checkpoint = {
            { play = "a20_188_bed_canopy1", volume = 0.8, },
            { play = "a20_276_bed_canopy2", frameOffset = 350, volume = 0.8, },
        },
    },

    boytree_girlwalks_a = {
        events = {
            { masterVolume = "Bed", masterTrackingTargets = OnScreenTrackingTargets, masterTrackingCurve = OnScreenTrackingHeadingCurve },
            { masterVolume = "Stem", masterTrackingTargets = OnScreenTrackingTargets, masterTrackingCurve = OnScreenTrackingHeadingCurve },
        },
    },

    boytree_girloutro = {
        events = {
            { fade = "forestambience", duration = 1, to = 0, autorelease = true, },
            { masterVolume = "AmbientTree", masterTrackingTargets = nil, masterTrackingCurve = nil },
        },
    },

    boytree_boyoutro = {
        events = {
            { fade = "forestambience", duration = 1, to = 0, autorelease = true, },
            { masterVolume = "AmbientTree", masterTrackingTargets = nil, masterTrackingCurve = nil },
        },
    },

    Fifth_Meeting1 = {
        checkpoint = {
            { fade = "a31_195_bed_canopy1", duration = CheckpointFadeDuration, to = 0.4, },
            { fade = "tree_stem_stars", duration = CheckpointFadeDuration, to = 1, },
        }
    },
}

audio.animation = {
    boy_cell_birth_alt = {
        { play = "boy_cell_birth_alt_57", frame = 57 + 551 - 50, volume = 0.8, includeGlobalIntroAnimDelay = true },
        { play = "a02_491_stem_dog",      frame = 613 + 551, },
        { play = "a02_491_stem_boy",      frame = 613 + 551, },
        { play = "a02_491_stem_meddry",   frame = 613 + 551, },
        { play = "a02_491_stem_piano",    frame = 613 + 551, },
        { play = "a02_491_stem_skipping", frame = 613 + 551, },
        { play = "a02_491_stem_slowwet",  frame = 613 + 551, volume = 0.8, },
        { play = "a02_491_stem_vocals",   frame = 613 + 551, volume = 0.7, },
        { play = "birth_stem_stars",      frame = 613 + 551, volume = 0, },

        { play = "a20_676_stab_gaze",      frame = 881 + 551, volume = 0.8, },
        { play = "a03_1088_stab_girldrop", frame = 1210 + 551, volume = 1, trackingTargets = { "boy" } },
    },

    girl_birth_alt = {
        { play = "girl_birth_alt_47", frame = 47 + 551 + 3, volume = 0.8, includeGlobalIntroAnimDelay = true },

        { play = "a02_491_stem_dog",      frame = 653 + 551, },
        { play = "a02_491_stem_boy",      frame = 653 + 551, },
        { play = "a02_491_stem_meddry",   frame = 653 + 551, },
        { play = "a02_491_stem_piano",    frame = 653 + 551, },
        { play = "a02_491_stem_skipping", frame = 653 + 551, },
        { play = "a02_491_stem_slowwet",  frame = 653 + 551, volume = 0.8, },
        { play = "a02_491_stem_vocals",   frame = 653 + 551, volume = 0.7, },
        { play = "birth_stem_stars",      frame = 653 + 551, volume = 0, },

        { play = "a20_676_stab_gaze",      frame = 939 + 551, volume = 0.8, },
        { play = "a03_1088_stab_girldrop", frame = 1247 + 551, volume = 1, },
    },

    boy_1st_meeting = {
        { fade = "a02_491_stem_meddry", frame = 785, duration = 1, to = 0.8, },
        { fade = "a02_491_stem_slowwet", frame = 805, duration = 1.5, to = 0, autorelease = true, },
    },

    girl_1st_meeting = {
        { fade = "birth_stem_stars", frame = 0, duration = 1, to = 1, },

        { play = "a04_45_stab_somersault", type = "Stab", frame = 37, volume = 0.8, trackingTargets = {"girl"} },

        { fade = "a02_491_stem_dog",     frame = 261, duration = 1, to = 1 },
        { fade = "a02_491_stem_meddry",  frame = 779, duration = 1, to = 0.8, },
        { fade = "a02_491_stem_slowwet", frame = 800, duration = 1.5, to = 0, autorelease = true, },
    },

    girl_2nd_meeting = {
        { play = "a11_489_stab_glidevox", frame = 495, volume = 0.8, },

        { fade = "a02_491_stem_slowwet", frame = 500, duration = 0.25, to = 0.4, },
        { fade = "a02_491_stem_meddry",  frame = 509, duration = 0.5, to = 0.2, },
        { fade = "a02_491_stem_meddry",  frame = 625, duration = 0.6, to = 0.8, },
    },

    boy_2nd_meeting = {
        { fade = "a02_491_stem_boy",   frame = 323, duration = 3, to = 0.8, },
        { fade = "a02_491_stem_piano", frame = 341, duration = 1.5, to = 0.8, },
    },

    boy_3rd_meeting_to_tree = {
        { play = "a20_676_stab_gaze", type = "Stab", frame = 42, volume = 0.8, },

        { fade = "a02_491_stem_boy", frame = 54, duration = 4, to = 0, autorelease = false },
        { fade = "a02_491_stem_boy", frame = 402, duration = 2, to = 0.8, },
    },

    girl_3rd_meeting = {
        { fade = "a02_491_stem_piano", frame = 1, duration = 1.5, to = 0.8, },
        { play = "a20_676_stab_gaze", frame = 30, volume = 0.8, },
        { fade = "a02_491_stem_slowwet", frame = 30, duration = 1, to = 0, autorelease = true, },

        { fade = "a02_491_stem_skipping", frame = 468, duration = 1, to = 0.7, },

        { play = "a12_664_stab_leaveskick", frame = 719, volume = 0.8, },

        { fade = "a02_491_stem_skipping", frame = 800, duration = 2, to = 0, autorelease = true, },
        { fade = "a12_900q_stab_voxrisers", frame = 900, duration = 4, to = 0.5, },
    },

    boy_4th_meeting = {
        { fade = "a02_491_stem_boy", frame = 1, duration = 3, to = 0, autorelease = true },
        { fade = "a02_491_stem_dog", frame = 1, duration = 2, to = 0, autorelease = true, },
        { fade = "birth_stem_stars", frame = 1, duration = 2, to = 0, autorelease = true, },

        { fade = "a12_900q_stab_voxrisers", frame = 19, duration = 4, to = 0.5, trackingTargets = { "boy" }, },

        { play = "a20_188_bed_canopy1", frame = 49, volume = 0.8, },

        { fade = "forestambience",      frame = 112, duration = 1, to = 0.2, },
        { masterVolume = "Bed",         frame = 112, masterTrackingTargets = TreeTrackingTargets, masterTrackingCurve = TreeTrackingHeadingCurve },
        { masterVolume = "Stem",        frame = 112, masterTrackingTargets = TreeTrackingTargets, masterTrackingCurve = TreeTrackingHeadingCurve },
        { masterVolume = "AmbientTree", frame = 112, masterTrackingTargets = TreeTrackingTargets, masterTrackingCurve = AmbientTreeTrackingHeadingCurve },

        { fade = "a02_491_stem_meddry",   frame = 300, duration = 4, to = 0, autorelease = true },
        { fade = "a02_491_stem_piano",    frame = 300, duration = 4, to = 0, autorelease = true, },
        { fade = "a02_491_stem_skipping", frame = 300, duration = 4, to = 0, autorelease = true, },

        { play = "a20_276_bed_canopy2", frame = 399, volume = 0.8, },

        { fade = "a02_491_stem_vocals", frame = 429, duration = 4, to = 0, autorelease = true, },

        { play = "a20_676_stab_gaze", frame = 559, volume = 0.8, trackingTargets = { "girl", } },

        { play = "a15_933_stab_woosh", frame = 1246, volume = 0.4, },

        { play = "a15_979_stem_boy", frame = 1292, volume = 0.8, },
        { play = "a15_979_stem_boypercussion", frame = 1292, volume = 0.2, },
        { play = "a15_979_stem_ahvox", frame = 1292, volume = 0.8, },
        { play = "a25_228_stem_strings", frame = 1292, volume = 0, },
        { fade = "a25_228_stem_main",    frame = 1292, duration = 1, to = 0.8, },
        { play = "a25_228_stem_wasser",  frame = 1292, volume = 0, },
        { play = "tree_stem_stars",      frame = 1292, volume = 0, },
    },

    boy_4th_meeting_girl = {
        { play = "girl_4th_757_birds", frame = 537, volume = 0.8, },
        { play = "girlwalks_1_girlstands", frame = 881, volume = 0.4, },

        { play = "a25_228_stem_strings",       frame = 1624, volume = 0, },
        { fade = "a25_228_stem_main",          frame = 1624, duration = 1, to = 0.8, },
        { play = "a25_228_stem_wasser",        frame = 1624, volume = 0, },
        { play = "tree_stem_stars",            frame = 1624, volume = 0, },
        { play = "a15_979_stem_boy",           frame = 1624, volume = 0.8, },
        { play = "a15_979_stem_boypercussion", frame = 1624, volume = 0.2, },
        { play = "a15_979_stem_ahvox",         frame = 1624, volume = 0.8, },

        { play = "a25_230_pirouette_jump", frame = 1641, volume = 0.4, },

        { fade = "a25_228_stem_wasser", frame = 1709, duration = 1, to = 0.8, autorelease = true, },
    },

    girl_4th_meeting = {
        { fade = "a02_491_stem_dog", frame = 1, duration = 2, to = 0, autorelease = true, },
        { fade = "birth_stem_stars", frame = 1, duration = 2, to = 0, autorelease = true, },

        { fade = "a02_491_stem_boy",    frame = 142, duration = 4, to = 0, autorelease = true, },
        { fade = "a02_491_stem_meddry", frame = 142, duration = 4, to = 0, autorelease = true, },
        { fade = "a02_491_stem_piano",  frame = 142, duration = 4, to = 0, autorelease = true, },
        { fade = "a02_491_stem_vocals", frame = 142, duration = 4, to = 0, autorelease = true, },

        { play = "a20_188_bed_canopy1", frame = 188, volume = 0.8, },

        { fade = "forestambience",      frame = 200, duration = 1, to = 0.2, },
        { masterVolume = "Bed",         frame = 200, masterTrackingTargets = TreeTrackingTargets, masterTrackingCurve = TreeTrackingHeadingCurve },
        { masterVolume = "Stem",        frame = 112, masterTrackingTargets = TreeTrackingTargets, masterTrackingCurve = TreeTrackingHeadingCurve },
        { masterVolume = "AmbientTree", frame = 112, masterTrackingTargets = TreeTrackingTargets, masterTrackingCurve = AmbientTreeTrackingHeadingCurve },

        { play = "a20_276_bed_canopy2", frame = 276, volume = 0.8, },
        { play = "a20_676_stab_gaze", frame = 676, volume = 0.8, trackingTargets = {"girl"}, },
        { play = "girl_4th_757_birds", frame = 757, volume = 0.8, },
        { play = "girlwalks_1_girlstands", frame = 1101, volume = 0.8, },

        { play = "girl_4th_1753_limb_leap", frame = 1753, volume = 0.4, },

        { play = "a25_228_stem_strings",       frame = 1824, volume = 0, },
        { fade = "a25_228_stem_main",          frame = 1824, duration = 1, to = 0.8, },
        { play = "a25_228_stem_wasser",        frame = 1824, volume = 0, },
        { play = "tree_stem_stars",            frame = 1824, volume = 0, },
        { play = "a15_979_stem_boy",           frame = 1824, volume = 0.8, },
        { play = "a15_979_stem_boypercussion", frame = 1824, volume = 0.2, },
        { play = "a15_979_stem_ahvox",         frame = 1824, volume = 0.8, },

        { fade = "a25_228_stem_wasser", frame = 1923, duration = 1, to = 0.8, },
    },

    girl_4th_meeting_boy = {
        { play = "a15_933_stab_woosh", frame = 1451, volume = 0.4, },

        { play = "a15_979_stem_boy",           frame = 1495, volume = 0.8, },
        { play = "a15_979_stem_boypercussion", frame = 1495, volume = 0.2, },
        { play = "a15_979_stem_ahvox",         frame = 1495, volume = 0.8, },
        { play = "a25_228_stem_strings",       frame = 1495, volume = 0, },
        { fade = "a25_228_stem_main",          frame = 1495, duration = 1, to = 0.8, },
        { play = "a25_228_stem_wasser",        frame = 1495, volume = 0, },
        { play = "tree_stem_stars",            frame = 1495, volume = 0, },
    },

    boy_leaves = {
        { fade = "a20_188_bed_canopy1", frame = 2, duration = 4, to = 0, autorelease = true, },
        { fade = "a20_276_bed_canopy2", frame = 2, duration = 4, to = 0, autorelease = true, },

        { fade = "a15_979_stem_boypercussion", frame = 106, duration = 3, to = 0.8, },
    },

    girl_pond = {
        { play = "girl_pond_365_splash", frame = 369, volume = 0.6, },
        { fade = "tree_stem_stars", frame = 369, duration = 1, to = 1, },
        { fade = "a25_228_stem_wasser", frame = 806, duration = 2.5, to = 0, autorelease = true, },
    },

    girl_dance = {
        { fade = "a25_228_stem_strings", frame = 67, duration = 0.5, to = 0.3, },

        { play = "a31_195_bed_canopy1", frame = 195, volume = 0.4, },

        { fade = "a25_228_stem_strings", frame = 241, duration = 1, to = 0, autorelease = true, },

        { fade = "a25_228_stem_main", frame = 251, duration = 1, to = 0, autorelease = true, },
        { fade = "a25_228_stem_strings", frame = 251, duration = 1, to = 0, autorelease = true, },

        { play = "a28_21_stab_tripcatch", frame = 255, volume = 0.8, },
    },

    boy_surfs = {
        { fade = "a20_188_bed_canopy1", frame = 0, duration = 4, to = 0, autorelease = true, },
        { fade = "a20_276_bed_canopy2", frame = 0, duration = 4, to = 0, autorelease = true, },

        { fade = "a15_979_stem_boypercussion", frame = 285, duration = 6, to = 0, autorelease = true, },

        { fade = "a15_979_stem_boy", frame = 407, duration = 3.5, to = 0, autorelease = true, },
        { fade = "a31_195_bed_canopy1", frame = 407, duration = 2, to = 0.4, },

        { play = "a28_21_stab_tripcatch", frame = 541, volume = 0.8, },
    },

    boy_5th_meeting = {
        { fade = "a15_979_stem_ahvox", frame = 0, duration = 3, to = 0, autorelease = true, },

        { play = "a29_533_stab_dogpullboy", frame = 533, volume = 0.1 },
        { play = "a28_675_stab_turn", frame = 657, volume = 0.8, },

        { fade = "a29_701_stem_boy", frame = 701, duration = 4, to = 0.8, },

        { play = "a15_933_stab_woosh", frame = 1174, volume = 0.4, },

        { fade = "a28_701_stem_pirouette", frame = 1582, duration = 8, to = { ["boy"] = 0.6 }, },
    },

    girl_5th_meeting = {
        { fade = "a25_228_stem_strings", frame = 0, duration = 1, to = 0, autorelease = true, },
        { fade = "a25_228_stem_main",    frame = 0, duration = 1, to = 0, autorelease = true, },

        { play = "a20_676_stab_gaze",     frame = 77, volume = 0.8 },

        { play = "a28_701_stem_ostinato",   frame = 701, volume = { ["boy"] = 0, ["girl"] = 0 }, },
        { play = "a28_701_stem_pirouette",  frame = 701, volume = { ["boy"] = 0, ["girl"] = 0 }, },
        { play = "a28_701_stem_celloyearn", frame = 701, volume = { ["boy"] = 0, ["girl"] = 0 }, },
        { play = "a28_701_stem_melody",     frame = 701, volume = { ["boy"] = 0, ["girl"] = 0 }, },
        { play = "a28_701_stem_arcobass",   frame = 701, volume = 0,},
        { play = "a28_701_stem_piano",      frame = 701, volume = 0, trackingTargets = { "girl" }, },
        { fade = "tree_stem_stars",         frame = 701, duration = 1, to = 0, autorelease = true, },

        { fade = "a28_701_stem_arcobass",   frame = 705, duration = 4, to = 0.8, },
        { fade = "a28_701_stem_celloyearn", frame = 705, duration = 4, to = { ["girl"] = 0.8, ["boy"] = 1 }, },
        { fade = "a28_701_stem_melody",     frame = 705, duration = 4, to = { ["girl"] = 0.8, ["boy"] = 0.4 }, },
        { fade = "a28_701_stem_ostinato",   frame = 705, duration = 4, to = { ["girl"] = 0.8, ["boy"] = 0.4 }, },
        { fade = "a28_701_stem_piano",      frame = 705, duration = 4, to = 0.8, },

        { fade = "a31_195_bed_canopy1",     frame = 747, duration = 3, to = 0, autorelease = true,  },

        { play = "a28_945_stab_glidevox",  frame = 945, volume = 0.8, },

        { fade = "a28_701_stem_pirouette", frame = 1266, duration = 0.25, to = { ["girl"] = 0.3 }, },
        { fade = "a28_701_stem_pirouette", frame = 1320, duration = 5, to = { ["girl"] = 0.6 }, },
    },

    boy_end = {
        { fade = "a28_701_stem_ostinato",   frame = 9, duration = 1.5, to = { ["boy"] = 0, ["girl"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_pirouette",  frame = 9, duration = 1.5, to = { ["boy"] = 0, ["girl"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_celloyearn", frame = 9, duration = 1.5, to = { ["boy"] = 0, ["girl"] = 0 }, autorelease = true, },

        { play = "a28_1958_stab_end", frame = 59, volume = 0.8, },

        { fade = "a29_701_stem_boy", frame = 116, duration = 1.5, to = 0, autorelease = true, },

        { fade = "a28_701_stem_melody",     frame = 134, duration = 1.5, to = { ["boy"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_arcobass",   frame = 134, duration = 1.5, to = 0, autorelease = true, },
        { fade = "a28_701_stem_piano",      frame = 134, duration = 1.5, to = 0, autorelease = true },

        { fade = "a28_701_stem_melody", frame = 171, duration = 2, to = { ["girl"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_piano",  frame = 171, duration = 2, to = 0, autorelease = true, },

        { play = "a20_676_stab_gaze", frame = 603, volume = 0.8 },
    },

    girl_end = {
        { fade = "a29_701_stem_boy",       frame = 1, duration = 1.5, to = 0, autorelease = true, },

        { fade = "a28_701_stem_arcobass",   frame = 64, duration = 1.5, to = 0, autorelease = true, },
        { fade = "a28_701_stem_ostinato",   frame = 64, duration = 1.5, to = { ["boy"] = 0, ["girl"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_celloyearn", frame = 64, duration = 1.5, to = { ["boy"] = 0, ["girl"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_pirouette",  frame = 64, duration = 1.5, to = { ["boy"] = 0, ["girl"] = 0 }, autorelease = true, },

        { play = "a28_1958_stab_end",      frame = 96, volume = 0.8, },

        { fade = "a28_701_stem_melody",     frame = 151, duration = 1.5, to = { ["boy"] = 0 }, autorelease = true, },

        { fade = "a28_701_stem_melody", frame = 1, duration = 2, to = { ["girl"] = 0 }, autorelease = true, },
        { fade = "a28_701_stem_piano",  frame = 1, duration = 2, to = 0, autorelease = true, },

        { play = "a20_676_stab_gaze", frame = 617, volume = 0.8 },
    },
}


local function animDataFor(actorName, segmentName)
    local actorData = SceneData.actors[actorName]
    local animData = actorData and actorData.animation and actorData.animation[segmentName]
    if animData then
        local startFrame = animData[2]
        local endFrame = animData[3]
        if endFrame == -1 then
            -- load the actor so we can get the animation's real end frame
            local scene = Scene.create()
            local actor = scene:createActor(actorName, actorData)
            if actor and actor.animation and actor.animation[segmentName] then
                endFrame = actor.animation[segmentName]:getStopFrame()
                actor:destroy()
                scene:destroy()
                Util.fullGC()
            end
        end
        return { animData[1], startFrame, endFrame }
    end
    return nil
end

local function segmentsByName()
    -- create a hash table for segment data
    local segments = {}
    if SceneData.segments then
        for _, segmentData in ipairs(SceneData.segments) do
            segments[segmentData.name] = segmentData
        end
    end
    return segments
end


local function animDataBySegment(segmentData)
    -- Find all animations and record what segments they play in
    local anims = {}
    if SceneData.actors then
        for actorName, actorData in pairs(SceneData.actors) do
            if actorData.animation then
                for animName, animData in pairs(actorData.animation) do
                    local animURI, startFrame, endFrame = animData[1], animData[2], animData[3]
                    local animFile = string.match(animURI, ".*/(.*)%.mxa")
                    local segment = segmentData[animName]
                    if segment and segment.animated then
                        for _, animatedActor in ipairs(segment.animated) do
                            if animatedActor == actorName then
                                anims[animFile] = anims[animFile] or {}
                                anims[animFile][animName] = {startFrame, endFrame }
                                break
                            end
                        end
                    end
                end
            end
        end
    end
    return anims

end


-- The AudioPlayer drives the show from the segment data, so we need to translate
-- the animation based events to segment based events
local function insertAnimationSoundsIntoSegments()
    local segments = segmentsByName()
    local anims = animDataBySegment(segments)

    -- process all animation timed audio and convert them into segment audio entries
    if audio.animation then
        for animName, events in pairs(audio.animation) do
            local anim = anims[animName]
            if anim and #events > 0 then
                for _, event in ipairs(events) do
                    local referenced = false
                    local eventStart = (event.frame or 0) + (event.frameOffset or 0)
                    for segmentName, frames in pairs(anim) do
                        local frameStart, frameEnd = frames[1], frames[2]
                        if eventStart >= frameStart then
                            if eventStart <= frameEnd or frameEnd == -1 then
                                local segmentEvent = deepcopy(event)
                                segmentEvent.frame = nil
                                segmentEvent.frameOffset = eventStart - frameStart
                                audio.segments[segmentName] = audio.segments[segmentName] or {}
                                audio.segments[segmentName].events = audio.segments[segmentName].events or {}
                                table.insert(audio.segments[segmentName].events, segmentEvent)
                                referenced = true
--                                Log.message("adding audio for segment:" .. segmentName .. " anim = " .. animName .. " sound = " .. table.tostring(segmentEvent))
                            end
                        end
                    end
                    if not referenced then
                        local name = event.play or event.fade or event.masterVolume
                        Log.error("Sound will not play " .. name .. " starting during " .. animName .. "[" .. tostring(eventStart) .. "] : " .. table.tostring(event))
                    end
                end
            end
        end
    end

end

local function sanityCheckAudio()
    for segmentName, segmentData in pairs(audio.segments) do
        if segmentData.events then
            for _, event in ipairs(segmentData.events) do
                local soundName = event.play or event.fade
                if not soundName then
                    if not event.masterVolume then
                        Log.error("Unknown sound event in segment " .. segmentName .. ": " .. table.tostring(event))
                        assert(false)
                    end
                else
                    if event.play and event.duration then
                        Log.error("Specifying a duration for a play event: " .. table.tostring(event))
                        assert(false)
                    end
                    local soundDef = audio.sounds[soundName]
                    if not soundDef then
                        Log.error("Undefined sound: " .. tostring(soundName))
                        assert(false)
                    end
                    local uri = soundDef.uri
                    if not uri then
                        Log.error("Unspecified sound uri: " .. tostring(soundName))
                        assert(false)
                    end
                    local basefile = string.match(uri, ".*/(.*).ogg")
                    local fileName = "episode3/audio/" .. basefile .. ".ogg"
                    local file, error = io.open(fileName)
                    if not file or error then
                        Log.error("Missing sound file for: " .. tostring(soundName) .. " - " .. fileName)
                        assert(false)
                    end
                end
            end
        end
    end
end

local function segmentTransitions()
    -- generate a table of segmentName -> list of transitions for every segment in the show
    local transitions = {}
    if SceneData.segments then
        for _, segmentData in ipairs(SceneData.segments) do
            local segmentTransitions
            if segmentData.transitions then
                for _, transition in pairs(segmentData.transitions) do
                    segmentTransitions = segmentTransitions or {}
                    if type(transition) == "table" then
                        segmentTransitions[transition.nextSegment] = true
                    else
                        segmentTransitions[transition] = true
                    end
                end
            end
            transitions[segmentData.name] = segmentTransitions
        end
    end
    return transitions
end

local function loopingSounds()
    local loopingSounds = {}
    for soundName, soundDef in pairs(audio.sounds) do
        if soundDef.type then
            local type = audio.types[soundDef.type]
            if type then
                local looping = soundDef.looping or type.looping
                if looping then
                    table.insert(loopingSounds, soundName)
                end
            end
        end
    end
    return loopingSounds
end

local function pathStillPlaying(startSegmentName, endSegments, segmentTransitions, path)
    -- Walk all the start positions and see if it always leads to an end segment
    -- if not, return the first path that doesn't end
    if not endSegments[startSegmentName] then
        local transitions = segmentTransitions[startSegmentName]
        local newPath = deepcopy(path)
        table.insert(newPath, startSegmentName)
        if transitions then
            for newStart, _ in pairs(transitions) do
                local nonEndingPath = pathStillPlaying(newStart, endSegments, segmentTransitions, newPath)
                if nonEndingPath then
                    return nonEndingPath
                end
            end
        else
            return newPath
        end
    end
    return nil
end

local function isStillPlayingAtEnd(soundName, transitions)
    local startSegments = {}
    local endSegments = {}

    local soundDef = audio.sounds[soundName]
    local type = audio.types[soundDef.type]

    for segmentName, segmentData in pairs(audio.segments) do
        if segmentData.events then
            for _, event in ipairs(segmentData.events) do
                local eventSoundName = event.play or event.fade
                if eventSoundName == soundName then
                    local autorelease = event.autorelease or soundDef.autorelease or type.autorelease
                    if event.play or (event.fade and not autorelease) then
                        startSegments[segmentName] = true
                    else
                        endSegments[segmentName] = true
                    end
                end
            end
        end
    end

    for startSegmentName, _ in pairs(startSegments) do
        local nonEndingPath = pathStillPlaying(startSegmentName, endSegments, transitions, {})
        if nonEndingPath then
            nonEndingPath.startSegments = startSegments
            nonEndingPath.endSegments = endSegments
            return nonEndingPath
        end
    end
    return nil
end

local function checkEndings()
    -- map all the segment transitions
    local transitions = segmentTransitions()

    -- find looping sounds
    local looping = loopingSounds()

    for _, soundName in ipairs(looping) do
        local nonEndingPath = isStillPlayingAtEnd(soundName, transitions)
        if nonEndingPath then
            Log.error("Still playing sound " .. soundName .. " from: " .. Serpent.dumppretty(nonEndingPath))
        end
    end
end

local function checkForMissingEvents()
    local segments = segmentsByName()

    for segmentName, segmentData in pairs(segments) do
        local audioSegmentEvents = audio.segments[segmentName] and audio.segments[segmentName].events
        if audioSegmentEvents then
            -- Find the minimum frame length of the segment to see if there are audio event that may not fire
            local minLength

            if segmentData.needsFocus then
                -- Look through the transition triggers
                if segmentData.transitions then
                    for targetName, transition in pairs(segmentData.transitions) do
                        local targetActorName = SceneData.actors[targetName] and SceneData.actors[targetName].keyActor
                        if targetActorName then
                            local animData = animDataFor(targetActorName, segmentName)
                            if animData then
                                local animLength = animData[3] - animData[2]
                                if animLength == 0 then
                                    Log.warning(targetActorName .. " in " .. segmentName .. " anim = " .. table.tostring(animData))
                                end
                                if minLength then
                                    minLength = math.min(minLength, animLength)
                                else
                                    minLength = animLength
                                end
                            else
                                Log.warning("Unable to get anim data for " .. targetActorName .. " in " .. segmentName)
                            end
                        else
                            Log.warning("Unable to find target actor for " .. segmentName .. " transition = " .. table.tostring(transition))
                        end
                    end
                end
            else
                -- just use the animation length of the first animated actor
                local firstActor = segmentData.animated and segmentData.animated[1]
                if firstActor then
                    local animData = animDataFor(firstActor, segmentName)
                    if animData then
                        minLength = animData[3] - animData[2]
                    end
                end
            end

            if minLength then
                -- See if any audio events are not in the minimum frame range for the segment
                for _, event in ipairs(audioSegmentEvents) do
                    local frame = (event.frame or 0) + (event.frameOffset or 0)
                    if frame > minLength then
                        Log.error("Audio event may not fire in " .. segmentName .. ": " .. table.tostring(event) .. ", min segment length = " .. tostring(minLength))
                    end
                end
            end
        end
    end
end

insertAnimationSoundsIntoSegments()
--sanityCheckAudio()
--checkEndings()
--checkForMissingEvents()
