

local Logger = TaggedLog("Duet", true)


Duet = 
{
	interrupted 			= false,
	fadeAmount              = 0,
	fadeBackgroundTimer 	= 0,
	fadeBackgroundDuration  = 0,
	state 					= "none",
	lostTimer 				= 2
}


--[[
	Function used to reset the timer for a segment. Takes the duration as an argument
]]
Duet.resetTimer = function (self,value)
	self.fadeBackgroundTimer = 1
	self.fadeBackgroundDuration = value
end
 
Duet.onStartup = function(self)

	-- make a logo if there isn't one
	local titleAnim = Global.scene.actors.TitleAnim
	if (titleAnim ~= nil) then
		titleAnim.components.TitleAnimator:autoPlayIntro()
	end
	
	-- Set fade info for Intro state
	self.state = "Intro"
	Duet:resetTimer(3)
	
	InputManager:enableSensors(true)

    -- Background fade-in timer for the start of the show
    Global.initShow()
    Global.startShow()

    self:loadActorTrackingTable()

    Scheduler.schedule(true, true, 0, false, 0, 0)

    if(Global.scene.mainCamera.controller.boomLayer) then
        Global.scene.mainCamera.controller.boomLayer.boomLength = 1000
    end	

    -- Attach GlowQuad to MainCamera
    self.glowQuad = ActorManager:getActor("GlowPlane", true)
    if (self.glowQuad ~= nil) then
        self.glowQuad:setVisibility(true)
        MainCamera:addChild(self.glowQuad)
    end

    -- Create the Sphere with the horizon and background
    self.showSphere = ActorManager:getActor("Show_Sphere", true)
    if(self.showSphere ~= nil) then
        self.showSphere:setVisibility(true)
    end

    -- Attach Vignette to MainCamera
    self.vignetteQuad = ActorManager:getActor("Vignette", true)
    if (self.vignetteQuad ~= nil) then
        self.vignetteQuad:setVisibility(true)
        MainCamera:addChild(self.vignetteQuad)
    end

    --clear out the launcher actors
    destroySafely(Global.scene.actors.LauncherToNotification)

    Replay:init();

	local butterfly = Global.scene:createActor("Butterfly", SceneData.actors.Butterfly)
	butterfly:addComponent("ButterflyController", ButterflyController:new())
	
    return true
end

Duet.onShutdown = function(self)	
	Events.send(self, "onShutdown")
end

Duet.loadActorTrackingTable = function(self)
	ActorTracking = require("episode3/scripts/scenedata/persistent/actortracking")
end

Duet.configureActorTrackingTable = function(self)

	ActorTrackingPath = "../raw/episode3/scripts/scenedata/persistent/"
	if(not Platform.win32) then ActorTrackingPath = "episode3/scripts/scenedata/persistent/" end
	
	-- method to read in saved actor tracking settings
	local function load()
		ActorTracking = require("episode3/scripts/scenedata/persistent/actortracking")
	end
	
	-- call the load method safely
	pcall(load)
	
	-- Default setup of actor tracking table
	ActorTracking = ActorTracking or {}
	for _, segment in pairs(SceneData.segments) do
		ActorTracking[segment.name] = ActorTracking[segment.name] or {
				strength = 0.5, fovFudge = 1.1, 
				cameraSquash = 0.25, characterOffset = 60
			}
			
	end
	
	-- clean out dead segments
	for name, value in pairs(ActorTracking) do
	
		local found = false
		for _, segment in pairs(SceneData.segments) do
			found = false
			if( segment.name == name ) then
				found = true
				break
			end
		end
		
		if(not found) then
			ActorTracking[name] = nil
		end
	end
	
	-- write the table to file
    local actorTrackingTable = "return "..Serpent.dumppretty(ActorTracking);
	local file,err = io.open( ActorTrackingPath.."actortracking.lua", "w" )
	if( file ~= nil ) then
		file:write(actorTrackingTable)
		file:close()
	end
end

Duet.configureActorBoundingTable = function(self)

	CameraBounding = require("episode3/scripts/scenedata/persistent/camerabounding");
	return
	
end

Duet.onUpdate = function(self)

    if (self.interrupted or Global.shutdown) then
        return "launcher_interrupted"        
    end
	
    Global.onUpdate()

    local dt = Time.dt()

    -- When show exits after End segment
    if (self.state == "Outro") then
    	
    	if (self.fadeBackgroundTimer <= 0) then
    		return "endofshow"
    	end

    	-- Fade show out
     	self.fadeBackgroundTimer = self.fadeBackgroundTimer - dt 
        self.fadeAmount = math.clamp01(self.fadeBackgroundTimer)	                
		Android.setAlphaRange(self.fadeAmount, self.fadeAmount)
    end

end

-- Called externally when the show is ready to end.
Duet.endShow = function (self)
    Duet:resetTimer(1)
    self.state = "Outro"
    Android.setOpaqueBackground(false)
end

Duet.onReshape = function(self, width, height)	
    Global.onReshape(width, height)    
end

Duet.onRender = function(self)		
	
    if (not Android.isPaused()) then
        Global.onRender()
    end
end

Duet.onEvent = function(self, event)
    if(event.type == "onActivityEvent") then
        if (event.data.eventString == "onDummyActivityGainedFocus") then
            Logger:message("Resuming")
            Android.onResume()
        elseif (event.data.eventString == "onDummyActivityLostFocus") then
            Logger:message("Pausing")
            Android.onPause()
        elseif (event.data.eventString == "onStop") then
            self.interrupted = true
            Android.onResume()
        end
    elseif(event.type == "onChangeDisplaySize") then
        Global.setRenderOverTSB(false)
    elseif(event.type == "onBatteryEvent") then
        if (event.data.newState == 0) then
            self.interrupted = true
            Android.onResume()
        end
    end
    return false;
end

Shell.attach("duet", Duet)

