
local Logger = TaggedLog("Editor", false)

Editor = {}

Editor.interrupted = false
Editor.endOfShow = false

Editor.onStartup = function(self)
	require "episode3/scripts/scenedata/scenes/editor/segments"
	require "episode3/scripts/scenedata/scenes/editor/actors"
	require "episode3/scripts/scenedata/scenes/editor/exportmaterials"
	require "episode3/scripts/scenedata/scenes/editor/util/directory"
	
	GUI.loadTheme()
	GUI.show()
	
	-- data views
	require "episode3/scripts/scenedata/scenes/editor/view/viewmanager"
	require "episode3/scripts/scenedata/scenes/editor/view/viewbase"
	require "episode3/scripts/scenedata/scenes/editor/view/modelbrowser"
	require "episode3/scripts/scenedata/scenes/editor/view/materialbrowser"
	
	ViewManager:init()
	
	local mb = ModelBrowser:new();
	ViewManager:pushView(mb)
	
	
	--ExportMaterials.export("episode3:models/treetestboy.mxm")
	
	
	
	-- GUI EXAMPLE
	
	--[[
	local frame = GUI.frame(nil, GUIAligned(ALIGN_CENTER), GUISize(256, 256), "red"):post();	
	frame:draggable();
	
	local btn = GUI.button(frame, GUIAligned(ALIGN_CENTER), GUISize(256, 32), "PressMe")
	
	-- GUI.slider = function(parent, pos, width, caption, initialValue, minValue, maxValue, numTicks)
	local slider = GUI.slider(frame, GUIPos(0, 0), 256, "Slider0", 0.5, 0, 1, 10);
	
	function btn:onButtonDown() 
		self:move(32, 0)
	end
	]]
		
    return true
end

Editor.onShutdown = function(self)
end

Editor.onUpdate = function(self)
	ViewManager:update();
    Global.onUpdate()
    if (Editor.interrupted or Global.shutdown) then
        -- For now just exit the show
        Avatar.exitDoodle()

--        return "launcher_interrupted"
    elseif (Editor.endOfShow) then
        return "endofshow"
    end
end

Editor.onReshape = function(self, width, height)
    Global.onReshape(width, height)
end

Editor.onRender = function(self)
	Global.onRender()
end

Editor.onEvent = function(self, event)
    if(event.type == "onActivityEvent") then
        if (event.data.eventString == "onDummyActivityGainedFocus") then
            Logger:message("Resuming")
            Android.onResume()
        elseif (event.data.eventString == "onDummyActivityLostFocus") then
            Logger:message("Pausing")
            Android.onPause()
        elseif (event.data.eventString == "onStop") then
            Editor.interrupted = true
            Android.onResume()
        end
    elseif(event.type == "onChangeDisplaySize") then
        Global.setRenderOverTSB(false)
    elseif(event.type == "onBatteryEvent") then
        if (event.data.newState == 0) then
            Editor.interrupted = true
            Android.onResume()
        end
    end
    return false;
end

Shell.attach("editor", Editor)
