
local Logger = TaggedLog("CamBounds", false)

CamBounds = {
	tree = nil,
	aspect = 1,
	xSlope = 1,
	ySlope = 1,
	depth = 10000,
	
	Plane = {
		point = Vector(),
		normal = Vector.forward:copy()
	},
	
	frustum = {}
 }
 

CamBounds.interrupted = false
CamBounds.endOfShow = false

local rightEdge = nil
local topEdge = nil
local aspect = 1;

CamBounds.onStartup = function(self)
	
	
	MainCamera.controller.boundsLayer.debug = true
	MainCamera.controller.boundsLayer.enabled = true
	
	MainCamera.controller:startLayers()
	
	
	rightEdge = Global.scene:createTransform("rightEdge")
	topEdge = Global.scene:createTransform("topEdge")
	
	Global.scene.mainCamera:addChild(rightEdge)
	Global.scene.mainCamera:addChild(topEdge)
	
	local radius = 2050
	local scale = 100
	
	SceneData.actors.rightSphere  = {
	    model = "episode3:models/unitsphere.mxm",
	    position = { radius*math.cos(math.pi/2 - math.pi/3), 0, -radius*math.sin(math.pi/2 - math.pi/3)},
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/debug.rsfx",
	            technique = "color",
	            renderGroup = 2,
	            textures =
	            {
	            },
	            parameters =
	            {
	                u_Color = Color.new( 1.0, 0.5, 1.0, 1.0 ),
	            }
	        },
	    },
	}
	
	SceneData.actors.leftSphere  = {
	    model = "episode3:models/unitsphere.mxm",
	    --position = { -10, 0, -50},
	    
	    position = { radius*math.cos(math.pi/2 + math.pi/3), 0, -radius*math.sin(math.pi/2 + math.pi/3)},
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/launcher.rsfx",
	            technique = "transparent_with_color",
	            renderGroup = 2,
	            textures =
	            {
	                u_MainTex = "episode3:textures/white.dds",
	            },
	            parameters =
	            {
	                u_Color = Color.new( 0.5, 0.5, 1.0, 1.0 ),
	            }
	        },
	    },
	}
		
	self.tree = {
    	model = "episode3:models/treetestboy.mxm",
    	materials = {
		},
		animation = {
		}
	}
	
	--self.tree = Global.scene:createActor("tree", self.tree)
	self.rightSphere = ActorManager:getActor("rightSphere", true)
	self.leftSphere = ActorManager:getActor("leftSphere", true)
	self.rightSphere:setScale(Vector(scale, scale, scale))
	self.leftSphere:setScale(Vector(scale, scale, scale))
	self.rightSphere:setVisibility(true)
	self.leftSphere:setVisibility(true)
	
    return true
end

CamBounds.collided = function(self, didCollide)
	Log.message("COLLISION "..tostring(didCollide))
end

CamBounds.onShutdown = function(self)
end

local done = false
local maxDist = 100
local minDist = 100

camRightRay ={x=0,y=0,z=0, w=0}
	
CamBounds.onUpdate = function(self)
	
   Global.onUpdate()
    if (CamBounds.interrupted or Global.shutdown) then
        -- For now just exit the show
        Avatar.exitDoodle()

--        return "launcher_interrupted"
    elseif (CamBounds.endOfShow) then
        return "endofshow"
    end
   
   --RenderDebug.point(rightEdge:getWorld().position, Color(1, 0, 0), 500)
   --RenderDebug.point(topEdge:getWorld().position, Color(0, 1, 0), 500)
   
	
   
	if(not done and MainCamera.controller.boundsLayer) then
		done = true
		MainCamera.controller.boundsLayer.boundingActors.left = self.leftSphere
		MainCamera.controller.boundsLayer.boundingActors.right = self.rightSphere
	end
   
   local camT = Global.scene.mainCamera:getLocal()
   local camLook = Vector.rotateBy(Vector.forward:copy(), camT.rotation);
   local camUp = Vector.rotateBy(Vector.up:copy(), camT.rotation);
   local camRight = Vector.rotateBy(Vector.right:copy(), camT.rotation);
   
   local camLookNoRoll = Quaternion.lookAt(camLook)
   
   local camRightNoRoll = Vector.rotateBy(Vector.right:copy(), camLookNoRoll);
   local right = Vector.rotateBy(Vector.right*self.xSlope, camLookNoRoll)
   local left = Vector.rotateBy(Vector.right*self.xSlope*-1, camLookNoRoll)
   
   local edgePointR = camT.position + (camLook + right)*maxDist
   local edgePointL = camT.position + (camLook + left)*maxDist
   
   local rightPlaneN = Vector.cross(Vector.up, edgePointR):normalized();
   local leftPlaneN = Vector.cross(edgePointL, Vector.up):normalized();
   
   local rightPlaneDist = Vector.dot(rightPlaneN, edgePointR);
   local leftPlaneDist = Vector.dot(leftPlaneN, edgePointL);
   
   local rayStart = camT.position + camLook
   
--	RenderDebug.point(edgePointR, Color(1, 0, 1), 2)
--	RenderDebug.point(edgePointL, Color(0, 1, 0), 2)
	
	local segToRight = {
		startPos = (camT.position + camLook*minDist),
		endPos = (camT.position + camLook*minDist + camRightNoRoll*maxDist),
		dir = ((camT.position + camLook*minDist + camRightNoRoll*maxDist)-(camT.position + camLook*minDist)):normalized(),
	}
	
	RenderDebug.line(segToRight.startPos, segToRight.endPos, Color(0,1,0));
	
	camRightRay = (segToRight.endPos - segToRight.startPos):normalized()
	
	self:updateCameraFrustum()
	
	local rightRayLen = math.floatMax
	
	for _, plane in ipairs(self.frustum) do
	
		if(Vector.dot(segToRight.dir, plane.normal) > 0) then
			RenderDebug.point(plane.point, Color(0, 1, 0), 2)
		else
			RenderDebug.point(plane.point, Color(0, 1, 1), 2)
			
			-- determine collision point
			
			-- dist to end of segment wrt plane
			local endDist = Vector.dot(segToRight.endPos, plane.normal)
			local begDist = Vector.dot(segToRight.startPos, plane.normal)
			local planeDist = Vector.dot(plane.point, plane.normal)
			local segLength = (endDist - begDist)
			local toPlaneDist = (planeDist - begDist)/segLength;
			
			local toColPoint = (segToRight.endPos - segToRight.startPos)*toPlaneDist
			
			local colPoint = segToRight.startPos + toColPoint
			
			local curLength = colPoint:lengthSq()
			
--			if(curLength < rightRayLen) then
--				curLength = rightRayLen;
--				MainCamera.controller.boundsLayer.rightRay = 
--			end
			
			RenderDebug.point(colPoint, Color(1, 1, 1), 10)
		end
	end

end

CamBounds.updateScreenEdge = function(self, width, height)

	aspect = width/height;
	
	self.ySlope = math.tan(math.rad(Global.fov)*0.5)
    self.xSlope = self.ySlope*aspect;
    self.depth = -(Global.nearFar.f - Global.nearFar.n)/2;
    
    local rightEdgeT = rightEdge:getLocal()
    local topEdgeT = topEdge:getLocal()
    rightEdgeT.position = Vector(-self.depth*self.xSlope, 0, self.depth); 
    topEdgeT.position = Vector(0, -self.depth*self.ySlope, self.depth);
    
    rightEdge:setLocal(rightEdgeT);
    topEdge:setLocal(topEdgeT);
end

CamBounds.updateCameraFrustum = function(self)
	
	-- only interested in left, right, top, and bottom planes
	
	local camT 		= Global.scene.mainCamera:getLocal()
   	local camLook 	= Vector.rotateBy(Vector.forward:copy(), camT.rotation);
   	local camUp 	= Vector.rotateBy(Vector.up:copy(), camT.rotation);
   	local camRight 	= Vector.rotateBy(Vector.right:copy(), camT.rotation);
   
	local identityRotation = camT.rotation
	local right 	= Vector.rotateBy(Vector.right*self.xSlope, identityRotation)
	local left 		= Vector.rotateBy(Vector.right*self.xSlope*-1, identityRotation)
	
	local top 		= Vector.rotateBy(Vector.up*self.ySlope, identityRotation)
	local bottom 	= Vector.rotateBy(Vector.up*self.ySlope*-1, identityRotation)
			
	local edgePointR = camT.position + (camLook + right)*maxDist
	local edgePointL = camT.position + (camLook + left)*maxDist
	local edgePointT = camT.position + (camLook + top)*maxDist
	local edgePointB = camT.position + (camLook + bottom)*maxDist
	
	self.frustum = {}
	
	-- right plane
	table.insert(self.frustum, {point = edgePointR, normal = Vector.cross(camUp, edgePointR)});
	-- left plane
	table.insert(self.frustum, {point = edgePointL, normal = Vector.cross(edgePointL, camUp)});
	-- top plane
	table.insert(self.frustum, {point = edgePointT, normal = Vector.cross(edgePointT, camRight)});
	-- bottom plane
	table.insert(self.frustum, {point = edgePointB, normal = Vector.cross(camRight, edgePointB)});
	
end

CamBounds.onReshape = function(self, width, height)
	self:updateScreenEdge(width, height)
    Global.onReshape(width, height)
    
end

CamBounds.onRender = function(self)
	Global.onRender()
end

CamBounds.onEvent = function(self, event)
    if(event.type == "onActivityEvent") then
        if (event.data.eventString == "onDummyActivityGainedFocus") then
            Logger:message("Resuming")
            Android.onResume()
        elseif (event.data.eventString == "onDummyActivityLostFocus") then
            Logger:message("Pausing")
            Android.onPause()
        elseif (event.data.eventString == "onStop") then
            CamBounds.interrupted = true
            Android.onResume()
        end
    elseif(event.type == "onChangeDisplaySize") then
        Global.setRenderOverTSB(false)
    elseif(event.type == "onBatteryEvent") then
        if (event.data.newState == 0) then
            CamBounds.interrupted = true
            Android.onResume()
        end
    end
    return false;
end

Shell.attach("cambounds", CamBounds)
