
local Logger = TaggedLog("LookAtTest", false)

LookAtTest = {
	tree = nil,
	aspect = 1,
	xSlope = 1,
	ySlope = 1,
	depth = 10000,
	rightSphere = nil,
	leftSphere = nil
 }
 

LookAtTest.interrupted = false
LookAtTest.endOfShow = false

local rightEdge = nil
local topEdge = nil
local aspect = 1;

LookAtTest.onStartup = function(self)
	MainCamera.controller:startLayers()
	if(Global.scene.mainCamera.controller.boomLayer) then
   		Global.scene.mainCamera.controller.boomLayer.boomLength = 1000;
    end

	rightEdge = Global.scene:createTransform("rightEdge")
	topEdge = Global.scene:createTransform("topEdge")
	
	Global.scene.mainCamera:addChild(rightEdge)
	Global.scene.mainCamera:addChild(topEdge)
	
	SceneData.actors.rightSphere  = {
	    model = "episode3:models/unitsphere.mxm",
	    
	    position = { 2050*math.cos(math.pi/2 - math.pi/3), 0, -2050*math.sin(math.pi/2 - math.pi/3)},
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/debug.rsfx",
	            technique = "color",
	            renderGroup = 2,
	            textures =
	            {
	            },
	            parameters =
	            {
	                u_Color = Color.new( 1.0, 0.5, 1.0, 1.0 ),
	            }
	        },
	    },
	}
	
	SceneData.actors.leftSphere  = {
	    model = "episode3:models/unitsphere.mxm",
	    
	    position = { 2050*math.cos(math.pi/2 + math.pi/3), 0, -2050*math.sin(math.pi/2 + math.pi/3)},
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/launcher.rsfx",
	            technique = "transparent_with_color",
	            renderGroup = 2,
	            textures =
	            {
	                u_MainTex = "episode3:textures/white.dds",
	            },
	            parameters =
	            {
	                u_Color = Color.new( 0.5, 0.5, 1.0, 1.0 ),
	            }
	        },
	    },
	}
	
	SceneData.actors.dome  = {
	    model = "episode3:models/unitsphere.mxm",
	    
	    materials = {
	        {
	            name = "Mat_Sphere",
	            shader = "episode3:shaders/launcher.rsfx",
	            technique = "transparent_with_color",
	            renderGroup = 0,
	            textures =
	            {
	                u_MainTex = "episode3:textures/white.dds",
	            },
	            parameters =
	            {
	                u_Color = Color.new( 0.5, 0.5, 1.0, 0.5 ),
	            }
	        },
	    },
	}
		
	
	self.rightSphere = ActorManager:getActor("rightSphere", true)
	self.leftSphere = ActorManager:getActor("leftSphere", true)
	self.dome = ActorManager:getActor("dome", true)
	self.rightSphere:setScale(Vector(100, 100, 100))
	self.leftSphere:setScale(Vector(100, 100, 100))
	self.dome:setScale(Vector(8000, 8000, 8000))
	self.rightSphere:setVisibility(true)
	self.leftSphere:setVisibility(true)
	self.dome:setVisibility(true)
	
    return true
end

LookAtTest.collided = function(self, didCollide)
	Log.message("COLLISION "..tostring(didCollide))
end

LookAtTest.onShutdown = function(self)
end

local lookAtToggle = false
local keyDown = false
LookAtTest.onUpdate = function(self)
	
   Global.onUpdate()
    if (LookAtTest.interrupted or Global.shutdown) then
        -- For now just exit the show
        Avatar.exitDoodle()

--        return "launcher_interrupted"
    elseif (LookAtTest.endOfShow) then
        return "endofshow"
    end
    
    local camWorld = MainCamera:getWorld().rotation
    DrawText:color(0,1,0):addText("Camera rot", "Cam yaw %.3f, pitch %.3f, roll %.3f", camWorld:getYaw(), camWorld:getPitch(), camWorld:getRoll())
    
    if (Input.isKeyDown(KB_N) and not keyDown) then
    	keyDown = true
    	lookAtToggle = not lookAtToggle
        if(lookAtToggle) then
        	DrawText:color(0,1,1):addText("Camera target", "Lookat 1")
        	MainCamera.controller:lookAt(self.rightSphere:getWorld().position)
        else
        	DrawText:color(1,0,1):addText("Camera target", "Lookat 2")
        	MainCamera.controller:lookAt(self.leftSphere:getWorld().position)
        end
    end
    
    if(Input.isKeyUp(KB_N)) then
    	keyDown = false
    end
   
end


LookAtTest.onReshape = function(self, width, height)
    Global.onReshape(width, height)
end

LookAtTest.onRender = function(self)
	Global.onRender()
end

LookAtTest.onEvent = function(self, event)
    if(event.type == "onActivityEvent") then
        if (event.data.eventString == "onDummyActivityGainedFocus") then
            Logger:message("Resuming")
            Android.onResume()
        elseif (event.data.eventString == "onDummyActivityLostFocus") then
            Logger:message("Pausing")
            Android.onPause()
        elseif (event.data.eventString == "onStop") then
            LookAtTest.interrupted = true
            Android.onResume()
        end
    elseif(event.type == "onChangeDisplaySize") then
        Global.setRenderOverTSB(false)
    elseif(event.type == "onBatteryEvent") then
        if (event.data.newState == 0) then
            LookAtTest.interrupted = true
            Android.onResume()
        end
    end
    return false;
end

Shell.attach("lookattest", LookAtTest)
