
local Logger = TaggedLog("PerfTests", false)
PerfTests = {
	results = {},
	frame = nil,
	framePos = Vector(400,100),
	totaleTest = 0,
	
	baseTimes = {},
}
PerfTests.interrupted = false
PerfTests.endOfShow = false

-- keeping track of frames for simple timing
local frameCount = 0

PerfTests.onStartup = function(self)

	self:testSetup()

	Global.clearColor       = Color(0.25,0.25, 0.25,1.0)
    if (Avatar.setWatchdogPaused) then
        Avatar.setWatchdogPaused(true)
    end
    frameCount = 0
    
    -- gui setup
	self.frame = GUI.frame(nil, GUIPos(self.framePos.x, self.framePos.y), GUISize(700, #Tests*64*4), "blue"):post();	
	self.frame:draggable();
	self.totaleTest = #Tests;
    return true
end

--- Test list ---
-- test are run
PerfTests.testSetup = function(self)
TestLoopCount = 300000

Tests = {
	{
		group = "callNative",
		isBase = true,
		name = "Call Native baseline",
		loopcount = TestLoopCount,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				System.benchmarkMethodCall()
			end
		end
	},
	{
		group = "toNative",
		isBase = true,
		name = "To Native baseline",
		loopcount = TestLoopCount,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				System.benchmarkMethodSend(1)
			end
		end
	},
	{
		group = "toFromNative",
		isBase = true,
		name = "To From Native baseline",
		loopcount = TestLoopCount,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				local v = System.benchmarkMethodRecv()
			end
		end
	},
	{
		group = "toClipSpace",
		isBase = true,
		name = "toClipSpace using native",
		loopcount = TestLoopCount,
		position = Vector(1,1,1,1),
		cam = SRT.new(),
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				mainView:project(self.position)
			end
		end
	},
	{
		group = "toClipSpace",
		name = "toClipSpace using lua",
		loopcount = TestLoopCount,
		position = Vector(1,1,1,1),
		cam = SRT.new(),
		toClipSpace = function(self, position)
			local camT = self.cam
			local camLook = Vector.forward:copy();
			local camRight = Vector.right:copy();
			local camUp = Vector.up:copy();
			
		--	local ySlope = math.tan(math.rad(fov or Global.fov)*0.5)
		--	local xSlope = ySlope*Global.aspect;
			local xSlope, ySlope = Util.frustumSlopes();
			
			-- get vector pointing at the boy (from camera)
			local toTarget = (position - camT.position)
			
			-- project vector to boy against the camera 'look' - provides distance from camera plane
			local depth = (camLook:dot(toTarget))
			depth = math.sign(depth)*math.max(1.0, math.abs(depth))
			
			-- gets the vector that points at the target and is perpendicular to the camera look ray (on the camera plane)
			local toTarget = toTarget - camLook*depth
			
			local x = Vector.dot(toTarget, camRight)
			local y = Vector.dot(toTarget, camUp)
		
			local screenX = xSlope*depth;
			local screenY = ySlope*depth;
			local ndc = Vector(x/screenX, y/screenY, depth/(Global.nearFar.f - Global.nearFar.n), depth)
		end,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				 self:toClipSpace(self.position)
			end
		end,
	},
	{
		group = "rotateBy",
		isBase = true,
		name = "Rotate by with cross",
		loopcount = TestLoopCount,
		a = Vector(1,1,1,1),
		b = Quaternion.axisAngle(Vector(1,1,1), 30),
		c = nil,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				self.c = Vector.rotateBy(self.a, self.b)
			end
		end
	},
	{
		group = "rotateBy",
		name = "Rotate by with q*v*q-inv",
		loopcount = TestLoopCount,
		a = Vector(1,1,1,1),
		b = Quaternion.axisAngle(Vector(1,1,1), 30),
		c = nil,
		enter = function(self)
			-- self old vector function
			self.oldVectorrotateBy = Vector.rotateBy
			
			-- create new vector function
			Vector.rotateBy = function(a, q)
			    local out = q * Quaternion(self.x,self.y,self.z,0.0) * q:inverse();
				return Vector(out.x,out.y,out.z); 
			end
		end,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				self.c = Vector.rotateBy(self.a, self.b)
			end
		end,
		exit = function(self)
			-- restore old vector function
			Vector.rotateBy = self.oldVectorrotateBy
		end
	},
	{
		group = "vectorMul",
		isBase = true,
		name = "Vector.__mul as is",
		loopcount = TestLoopCount,
		a = Vector(2,2,2,2),
		b = Vector(2,2,2,2),
		c = nil,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				self.c = self.a*self.b
			end
		end
	},
	{
		group = "vectorMul",
		name = "Vector.__mul without assert",
		loopcount = TestLoopCount,
		a = Vector(1,1,1,1),
		b = Vector(2,2,2,2),
		c = nil,
		enter = function(self)
			-- self old vector function
			self.oldMul = Vector.__mul
			
			-- create new vector function
			Vector.__mul = function(a, b)
			    if (type(a)=="number") then
			        local c = a; 
			        a = b;
			        b = c;
			    end
				if (type(b)=="number") then
					return Vector.new(a.x * b, a.y * b, a.z * b, a.w * b);
				elseif (type(b=="table")) then
					return Vector.new(a.x * b.x, a.y * b.y, a.z * b.z, a.w * b.w);
				end
			end
		end,
		doTest = function(self)
			for i = 1, self.loopcount, 1 do
				self.c = self.a*self.b
			end
		end,
		exit = function(self)
			-- restore old vector function
			Vector.__mul = self.oldMul
		end
	},
	
	Util.fullGC()
}
	
end

PerfTests.onShutdown = function(self)
end

local offset = 0
local doneCount = 0
PerfTests.onUpdate = function(self)

	if (Avatar.setWatchdogPaused) then
		Avatar.setWatchdogPaused(true)
	end
	if (PerfTests.interrupted or Global.shutdown) then
        -- For now just exit the show
        Avatar.exitDoodle()
    end

	frameCount = frameCount + 1;
	
	Util.fullGC()
	
	-- wait 5 seconds and start tests
	if(frameCount*0.016 > 4 and #Tests > 0) then
		local test = table.remove(Tests, 1)
		
		if(test.enter) then test:enter() end
		
		local totalTime = 0
		local totalCount = 0
		local totalBytes = 0
		
		local reTestCount = 3
		
		for i = 1, reTestCount, 1 do
		
			local startCount = Memory.getLuaTotalAllocCount();
			local startBytes = Memory.getLuaAllocatedBytes();
			Log.message("Start: "..test.name.." byte# ".. startCount.." bytes ".. startBytes)
			local startTime = os.clock();
			test:doTest()
			totalTime = totalTime + os.clock() - startTime;
			local countDiff = Memory.getLuaTotalAllocCount() - startCount
			local byteDiff = Memory.getLuaAllocatedBytes() - startBytes
			totalCount = totalCount + countDiff
			totalBytes = totalBytes + byteDiff
			
		end
		
		totalTime = totalTime/reTestCount
		totalCount = totalCount/reTestCount
		totalBytes = totalBytes/reTestCount
		
		if(test.exit) then test:exit() end
		
		PerfTests.results = PerfTests.results or {}
		
		local changeInTime = 0
		
		if(test.group) then
			if(test.isBase) then
				self.baseTimes[test.group] = totalTime
			else
				changeInTime = self.baseTimes[test.group]/totalTime
			end
		end
		
		local result = {
			name = test.name,
			loopcount = test.loopcount,
			time = totalTime,
			change = changeInTime,
			allocCount = totalCount,
			allocBytes = totalBytes,
		}
		
		-- reset frame time
		frameCount = 0
		
		offset = 64*4*doneCount
		
		doneCount = doneCount + 1
		
		GUI.button(self.frame, GUIPos(0, offset + 0), 	 GUISize(512, 64), "name: "..result.name)
		GUI.button(self.frame, GUIPos(0, offset + 64), 	 GUISize(512, 64), "count: "..result.loopcount)
		GUI.button(self.frame, GUIPos(0, offset + 64*2), GUISize(512, 64), string.format("alloc Count: %d, %d per call - alloc Bytes: %d", result.allocCount, result.allocCount/TestLoopCount, result.allocBytes))
		GUI.button(self.frame, GUIPos(0, offset + 64*3), GUISize(512, 64),
					string.format("time: %.6f, change: %s%.4f | %.4f%%", 
					result.time
					,(result.change > 1 and "+" or "-")
					,result.change
					,(result.change - 1)*100)
					)
		
		table.insert(PerfTests.results, result)
		
		
	end
	
end

PerfTests.onReshape = function(self, width, height)
    Global.onReshape(width, height)
    
end

PerfTests.onRender = function(self)
	local textStyle = { color = Color.new(1, 1, 1), font = "common:fonts/arial_default_14.fnt", scale = 2 }
    local fpsString = "[".."Test "..(#Tests > 0 and "running" or "done").."] frame "..frameCount
    
    local yOffset = 140
    Renderer.drawText(10, yOffset, textStyle, fpsString)
    
    if(#PerfTests.results > 0) then
	    local result = PerfTests.results[#PerfTests.results]
	    
		Renderer.drawText(10, yOffset + 40, textStyle, string.format("last test: %s", result.name))
		Renderer.drawText(20, yOffset + 40*2, textStyle, string.format("loops: %d", result.loopcount))
		Renderer.drawText(20, yOffset + 40*3, textStyle, string.format("alloc Count: %d alloc Bytes: %d", result.allocCount, result.allocBytes))
		Renderer.drawText(20, yOffset + 40*4, textStyle, string.format("time: %f, change: %s%.4f | %.4f%%"
					,result.time
					,(result.change > 1 and "+" or "-")
					,result.change
					,(result.change - 1)*100))
    end
    
    mainView:setClearColor( Global.clearColor )
    Global.scene:render( mainView )
    Renderer.display( mainView )
end

PerfTests.onEvent = function(self, event)
    if(event.type == "onActivityEvent") then
        if (event.data.eventString == "onDummyActivityGainedFocus") then
            Logger:message("Resuming")
            Android.onResume()
        elseif (event.data.eventString == "onDummyActivityLostFocus") then
            Logger:message("Pausing")
            Android.onPause()
        elseif (event.data.eventString == "onStop") then
            PerfTests.interrupted = true
            Android.onResume()
        end
    elseif(event.type == "onChangeDisplaySize") then
        Global.setRenderOverTSB(false)
    elseif(event.type == "onBatteryEvent") then
        if (event.data.newState == 0) then
            PerfTests.interrupted = true
            Android.onResume()
        end
    end
    return false;
end

Shell.attach("perftests", PerfTests)
