
local Logger = TaggedLog("TrackTest", false)

TrackTest = {
	tr = nil,
 	parent = nil,
 }

TrackTest.interrupted = false
TrackTest.endOfShow = false


TrackTest.onStartup = function(self)
	
	self.parent = Global.scene:createTransform("parent");
	self.tr = Global.scene:createTransform("point");
	local localT = self.tr:getLocal()
	local parentT = self.tr:getLocal();
	
	--localT.position = Vector( .1*math.cos(.707), .1*math.sin(.707), -1);
	localT.position = Vector( 0, 0, -Global.nearFar.n - 1000);
	--parentT.rotation = localT.rotation*Quaternion.axisAngle(Vector.up, 180);
	--local rot = Quaternion.axisAngle(Vector.forward, -45);
	--local m = rot:toMatrix()
	
	--localT.position = m:applyTransform(localT.position);
	
	self.tr:setLocal(localT);
	
	self.tr:addComponent("track", RayToPlane:new({raySource=Global.scene.mainCamera, callback = { target = self, method="collided"}, debug = true }))
	self.tr:addComponent("billboard", Billboard:new({debug = true}))
	
	self.parent:addChild(self.tr);
	self.parent:setLocal(parentT);
	
	
    return true
end

TrackTest.collided = function(self, didCollide)
	Log.message("COLLISION "..tostring(didCollide))
end

TrackTest.onShutdown = function(self)
end

TrackTest.onUpdate = function(self)
    Global.onUpdate()
    if (TrackTest.interrupted or Global.shutdown) then
        -- For now just exit the show
        Avatar.exitDoodle()

--        return "launcher_interrupted"
    elseif (TrackTest.endOfShow) then
        return "endofshow"
    end
    
    local localT = self.parent:getLocal();
    
   -- localT.rotation = localT.rotation*Quaternion.axisAngle(Vector.right, 5*Time.dt())*Quaternion.axisAngle(Vector.forward, 5*Time.dt())
    local tr = self.tr:getLocal()
    local rot = Quaternion.axisAngle(Vector.up, 15*Time.dt());
	tr.position = tr.position:rotateBy(rot)
	self.tr:setLocal(tr);
	
	local rigT = Global.scene.cameraRig:getLocal()
	rigT.rotation = Quaternion.slerp(50*Time.dt(), rigT.rotation, rigT.rotation*rot);
	Global.scene.cameraRig:setLocal(rigT)
	
    self.parent:setLocal(localT);
    
    RenderDebug.axes(self.tr:getWorld(), .5)
end

TrackTest.onReshape = function(self, width, height)
    Global.onReshape(width, height)
end

TrackTest.onRender = function(self)
	Global.onRender()
end

TrackTest.onEvent = function(self, event)
    if(event.type == "onActivityEvent") then
        if (event.data.eventString == "onDummyActivityGainedFocus") then
            Logger:message("Resuming")
            Android.onResume()
        elseif (event.data.eventString == "onDummyActivityLostFocus") then
            Logger:message("Pausing")
            Android.onPause()
        elseif (event.data.eventString == "onStop") then
            TrackTest.interrupted = true
            Android.onResume()
        end
    elseif(event.type == "onChangeDisplaySize") then
        Global.setRenderOverTSB(false)
    elseif(event.type == "onBatteryEvent") then
        if (event.data.newState == 0) then
            TrackTest.interrupted = true
            Android.onResume()
        end
    end
    return false;
end

Shell.attach("tracktest", TrackTest)
