
local LastID = "__lastID"

StableIDs = {
    name = "",
    prefix = "i",
    fileName = "ids",
}

function StableIDs:new(params)
    local instance = shallowcopy(params)

    setmetatable(instance, self)
    self.__index = self

    instance:load()

    return instance
end

function StableIDs:load()
    local ids = nil
    pcall(function() ids = require(Project.persistSceneDir .. self.fileName) end)
    if type(ids) == "table" then
        self.ids = ids
    else
        self.ids = {}
    end
end

function StableIDs:save()
    local luaValue = "return " .. Serpent.dumppretty(self.ids)
    local file, err = io.open(Project.rawPersistSceneDir .. self.fileName .. ".lua", "w" )
    if not err and file then
        file:write(luaValue)
        file:close()
    end
end

function StableIDs:getId(name)
    local ids = self.ids
    if not ids[name] then
        ids[LastID] = (ids[LastID] or 0) + 1
        ids[name] = ids[LastID]
    end
    return self.prefix .. string.format("%02d", ids[name])
end

function StableIDs:getName(id)
    local idNum = tonumber(string.match(id, self.prefix .. "(%d+)"))
    for name, testId in pairs(self.ids) do
        if testId == idNum and name ~= LastID then
            return name
        end
    end
    return nil
end

function StableIDs:writeCSV(fileName)
    local file, error = io.open(fileName .. ".csv", "w")
    if not error and file then
        file:write("ID, " .. self.name .. "\n")
        for name, _ in pairs(self.ids) do
            if name ~= LastID then
                file:write(self:getId(name) .. ", " .. name .. "\n")
            end
        end
        file:close()
    end
end
