-- default button control
-- NOTE: For internal use only!
__button = function(parent, pos, size, caption, style)
    if(pos == DEFAULT) then
        pos = px(0,0)
    end

    if(size == DEFAULT) then
        size = _T[style].default.image.size
    end

    local button = GUI.frame(parent, pos, size, style..".default")
    button.style = style

    button.caption = caption

    local buttonClientRect = button:getClientLocalRect()

    button.selectStyle = function(self)
        if(self.disabled) then
            self:setStyle(self.style..".disabled")
        elseif(self.pressed) then
            self:setStyle(self.style..".pressed")
        elseif(self.hovered) then
            self:setStyle(self.style..".hovered")
        else
            self:setStyle(self.style..".default")
        end
    end

    button.enable = function(self)
        self.disabled = false
        self:selectStyle()
    end

    button.disable = function(self)
        self.disabled = true
        self:selectStyle()
    end

    local label = nil
    button.setCaption = function(self, text)
        if(label==nil) then
            label = GUI.label(button, px(0,0), nx(1,1), caption, style..".text")
            label:anchor(ANCHOR_ALL)
        end
        label:setText(text)
    end

    if(caption and caption ~= "") then
        button:setCaption(caption)
        --		label = GUI.label(button, px(0,0), nx(1,1), caption, style..".text")
    end

    local handler = GUI.defaultButtonHandler
    button.spoof = function(self)
        handler.onMouseDown(self)
    end
    button:addEventHandler(handler)

    return button
end

-- generic button with text label
GUI.button = function(parent, pos, size, caption)
    return __button(parent, pos, size, caption, "button")
end

-- [+] button
GUI.plus = function(parent, pos)
    return __button(parent, pos, px(32, 32), nil, "plus")
end

-- [-] button
GUI.minus = function(parent, pos)
    return __button(parent, pos, px(32, 32), nil, "minus")
end

-- GUI.minimizer
-- This is a specialized button that can be used to toggle between
-- minimized and maximized states. The user may implement
-- onMinimize and onMaximize to extend its behavior. Visually, this
-- button appears as a [+] button when minimized and a [-] button when
-- maximized.
GUI.minimizer = function(parent, pos, state)
    local button = __button(parent, pos, px(32, 32), nil, "plus")

    function button:changeState(newstate)
        if(newstate == self.state) then
            return
        end
        if(newstate) then
            self.style = "minus"
            self:setStyle(self.style..".default")
            if(self.onMaximize~=nil) then
                self:onMaximize()
            end
        else
            self.style = "plus"
            self:setStyle(self.style..".default")
            if(self.onMinimize~=nil) then
                self:onMinimize()
            end
        end
        self.state = newstate
    end

    button:changeState(state)

    function button:onButtonDown()
        -- toggle
        self:changeState(not(self.state))
    end

    function button:minimize()
        self:changeState(false)
    end

    function button:maximize()
        self:changeState(true)
    end

    return button
end
