
--[[
	__checkbox:
		- creates a left-aligned checkbox ([] label)
		- TODO: add support for right-aligned checkboxes (label [])
]]
local __checkbox = function(parent, pos, size, checked, caption, style)
    local textSize = GUI.textSize(caption, style..".text");
    local checkbox = GUI.frame(parent, pos, size, "");

    checkbox.checked = checked;
    local imageState = ".state.default";
    if checkbox.checked then
        imageState = ".state.checked";
    end

    -- TODO: gross hack to make the gear icon the right size.  Need to fix this in the image.
    local imageSize = GUIAutoSize()
    if style == "gear" then
        imageSize = px(32, 32)
    end
    local image = GUI.frame(checkbox, px(0, 0), imageSize, style..imageState);
    local label = GUI.label(checkbox, px(image:getWidth(), 0), px(textSize.x, textSize.y), caption, style..".text");
    label:setTextAlign(ALIGN_CENTER_LEFT);

    if size.hmode == WRAP_CONTENT then
        size.x = image:getWidth() + textSize.x
    end
    if size.vmode == WRAP_CONTENT then
        size.y = math.max(image:getHeight(), textSize.y)
    end
    checkbox:resize(size.x, size.y)

    checkbox.image = image
    checkbox.label = label

    local __toggle = function(self)
    -- group logic...
        if(self.__group) then
            if(status(self.__group.active) ~= nil) then
                if(status(self.__group.active) == self) then
                    return;
                end
            end

            local oldctrl = self.__group.active;
            if not self.checked then
                self.__group.active = self;
                -- oldctrl can be nil.
                if oldctrl then
                    oldctrl:uncheck();
                end
            end
        end

        -- update state...
        self.checked = not self.checked

        -- update image style
        if self.checked then
            image:setStyle(style..".state.checked");
        else
            image:setStyle(style..".state.default");
        end

        -- onclick callback
        -- add an onclick function to receive a callback for this
        -- checkbox.
        if(self.onclick) then
            self:onclick();
        end
    end

    local checkBoxHandler = {
        onMouseDown = function(self)
            __toggle(checkbox);
            return true;
        end,
    };
    -- attach event handler
    checkbox:addEventHandler(checkBoxHandler);

    -- external API
    -- use these functions for programmatic control
    checkbox.check = function(self)
        if not self.checked then
            checkBoxHandler.onMouseDown(self);
        end
    end

    checkbox.uncheck = function(self)
        if self.checked then
            checkBoxHandler.onMouseDown(self);
        end
    end

    -- checkbox groups
    -- only one checkbox will be enabled at a time.
    checkbox.group = function(self, ...)
        self.__group = self.__group or {};
        self:check();
        for _, ctrl in ipairs({...}) do
            ctrl:uncheck();
            ctrl.__group = self.__group;
        end
    end

    return checkbox;
end

-- default checkbox
GUI.checkbox = function(parent, pos, size, state, caption)
    return __checkbox(parent, pos, size, state, caption, "checkbox");
end

-- default radio control
GUI.radio = function(parent, pos, size, state, caption)
    return __checkbox(parent, pos, size, state, caption, "radio");
end

GUI.gear = function(parent, pos, size, state, caption)
    return __checkbox(parent, pos, size, state, caption, "gear");
end
