
style = {}
style.defaultTexture = { "common:textures/gui/ldpi/guielements.dds", 640, 320 }
style.defaultFont = "common:fonts/arial_default_18.fnt"

style.new = function(table)
    table.__isstyle = true
    table.font = table.font or style.defaultFont
    if table.image then
        -- store some useful information
        if not table.image.size then
            if table.image.coords then
                table.image.width = table.image.coords[3] - table.image.coords[1]
                table.image.height = table.image.coords[4] - table.image.coords[2]
                table.image.size = px(table.image.width, table.image.height)
            else
                table.image.size = nx(1, 1)
            end
        end

        table.image.color = table.image.color or Color.white
        table.image.texture = table.image.texture or style.defaultTexture
    end
    return table
end

--[[
	imagelist:
		This function automates the process of generating states for a sequence of images.
		Ideal for generating styles from images organized horizontally or vertically.
		parameters:
			state: a list of state names, or a number (states will be named ["state"..index])
			listdesc: a table containing the following parameters:
				texture: the texture for this imagelist.
				coords: the pixel coordinates (left, top, right, bottom) of the first state of the list.
				ninepatch: (optional) ninepatch offsets, all states will use the same offsets.
				vertical: (optional) images are aligned vertically, horizontal alignment is assumed by default.
				stride: (optional) the stride length or padding between each image state.
]]
imagelist = {}
imagelist.new = function(states, listdesc, ob)
    local list = ob or {}
    local ox = 0
    local oy = 0

    local stepx = 0
    local stepy = 0
    if(not(listdesc.vertical)) then
        if(listdesc.stride == nil) then
            stepx = listdesc.coords[3] - listdesc.coords[1]
        else
            stepx = listdesc.stride
        end
    else
        if(listdesc.stride == nil) then
            stepy = listdesc.coords[4] - listdesc.coords[2]
        else
            stepy = listdesc.stride
        end
    end

    local newstyle = function(ox, oy, listdesc)
        local styledesc = {}
        styledesc.image = {}
        styledesc.image.texture = listdesc.texture
        styledesc.image.color = listdesc.color
        styledesc.image.ambient = listdesc.ambient
        styledesc.image.coords = {}
        styledesc.image.coords[1] = listdesc.coords[1] + ox
        styledesc.image.coords[2] = listdesc.coords[2] + oy
        styledesc.image.coords[3] = listdesc.coords[3] + ox
        styledesc.image.coords[4] = listdesc.coords[4] + oy
        styledesc.image.ninepatch = listdesc.ninepatch
        return style.new(styledesc)
    end

    if(type(states) == "table") then
        for _, name in ipairs(states) do
            list[name] = newstyle(ox, oy, listdesc)
            ox = ox + stepx
            oy = oy + stepy
        end
    elseif(type(states) == "number") then
        for index = 0, states, 1 do
            list["state"..index] = newstyle(ox, oy, listdesc)
            ox = ox + stepx
            oy = oy + stepy
        end
    end
    return list
end
