
--_enumerator = 0;
Log.enable = true;
Log.warnings = true;
Log.errors = true;
Log.messages = true;

Log.super = {}
Log.super.message = Log.message;
Log.super.error = Log.error;
Log.super.warning = Log.warning;

Log.message = function(msg)
	if(Log.enable) then
		Log.super.message(msg)
	end
end

Log.error = function(msg)
	if(Log.enable and Log.errors) then
		Log.super.error(msg)
	end
end

Log.warning = function(msg)
	if(Log.enable and Log.warnings) then
		Log.super.warning(msg)
	end
end

Log.debug = function(msg)
	if(Log.debug) then
--		Log.super.error(msg);
--		Log.super.message(debug.traceback());
		Log.fatal(msg.."\n"..debug.traceback());
	end
end

local debugInfo = { caller = nil, callee = nil }
function hook()
	local info = debug.getinfo(2);
	
	if(info.what == "C") then
		if(info.name ~= nil) then
			Log.message("["..info.what.."] : "..info.name);
		end
	end
--    local info = debug.getinfo(2)
 --   if info == nil then
 --       debugInfo.callee = nil
  --      return
  --  end

    -- we only want to watch lua function calls (not C functions)
--    if info.what ~= "Lua" then
--        debugInfo.callee = "C function"
		
--        return
--    end

--    debugInfo.caller = debugInfo.callee
--   debugInfo.callee = info.name
	
end
--debug.sethook(hook, "c");

enum = function(e, c)
	if (c ~= nil) then
		_enumerator = c;
	end
	_G[e] = _enumerator;
	_enumerator = _enumerator + 1;
end

package = "*";
uri = function(path) 
	return package..":"..path;
end



-- makes table readonly (including sub tables)
function readonly (orig)
	
	-- recursivly apply readonly-ness to nested tables
	for key,value in pairs(orig) do
		local typeName = type(value);
        if typeName == 'table' then
			orig[key] = readonly(orig[key]);
        end
    end
    
	local proxy = {}
	local mt = {       -- create metatable
		__index = orig,
		__newindex = function (orig,k,v)
				error("\nattempt to update a read-only table\n", 2);
			end
	 }
	setmetatable(proxy, mt)
  
  return proxy
end


-- shallow copy a lua table
function shallowcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in pairs(orig) do
            copy[orig_key] = orig_value
        end
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end

-- deep copy a lua table
function deepcopy(t)
	if type(t) ~= 'table' then return t end
	local mt = getmetatable(t)
	local res = {}
	for k,v in pairs(t) do
		if type(v) == 'table' then
			v = deepcopy(v)
		end
		res[k] = v
	end
	setmetatable(res,mt)
	return res
end

-- look up for `k' in list of tables `plist'
local function search (k, plist)
  for i=1, table.getn(plist) do
    local v = plist[i][k]     -- try `i'-th superclass
    if v then return v end
  end
end

function createClass (...)
  local c = {}        -- new class

  -- class will search for each method in the list of its
  -- parents (`arg' is the list of parents)
  local arg = {...}
  setmetatable(c, {__index = function (t, k)
   	local v = search(k, arg)
    t[k] = v       -- save for next access
    return v
  end})

  -- prepare `c' to be the metatable of its instances
  c.__index = c

  -- define a new constructor for this new class
  function c:new (o)
    o = o or {}
    setmetatable(o, c)
    return o
  end

  -- return new class
  return c
end

function table.val_to_str ( v )
	if "string" == type( v ) then
		v = string.gsub( v, "\n", "\\n" )
		if string.match( string.gsub(v,"[^'\"]",""), '^"+$' ) then
			return "'" .. v .. "'"
	end
		return '"' .. string.gsub(v,'"', '\\"' ) .. '"'
	else
		return "table" == type( v ) and table.tostring( v ) or
	tostring( v )
	end
end

function table.key_to_str ( k )
	if "string" == type( k ) and string.match( k, "^[_%a][_%a%d]*$" ) then
		return k
	else
		return "[" .. table.val_to_str( k ) .. "]"
	end
end

function table.tostring( tbl )
	if tbl == nil then
		return "nil"
	end

	local result, done = {}, {}
	for k, v in ipairs( tbl ) do
		table.insert( result, table.val_to_str( v ) )
		done[ k ] = true
	end
	for k, v in pairs( tbl ) do
		if not done[ k ] then
			table.insert( result,
			table.key_to_str( k ) .. "=" .. table.val_to_str( v ) )
		end
	end
	return "{" .. table.concat( result, "," ) .. "}"
end

function serialize(tbl, filename)
	local str = table.tostring(tbl);
	local file = io.open(filename, "w");
	if(file) then
		file:write(str);
	end
end

status = function(ob)	
	if(type(ob) == "table" and ob.__object ~= nil) then
		ob = __status(ob);		
	end
	return ob;
end