

require "common/scripts/moxie/moxie"
require "common/scripts/moxie/shell"

require "help/scripts/help"

local Logger = TaggedLog("Application", true)

local app = HelpApp.create()

Shell.start("help")

Application = 
{
	-- config
	debug			= true,
	fullscreen		= false,

	-- state
	shutdown		= false,
	paused			= false,
}

function Application.onInitialize()
	if Application.debug then
		Avatar.setCanPreempt( false )		-- prevent cloud service from killing us
	end

	-- enable audio
	JavaAudio.requestFocus( 1 )

	-- lock orientation on the kickup
	Avatar.setWindowLockOrientation( true )

	-- keep screen on
	Avatar.setWindowKeepScreenOn( true )

	-- launch dummy activity
	local enableActivity 		= true
	local excludeFromRecents	= true
	local systemUIVisible		= 0
	local backgroundColor		= 0x0
	local transparent			= false
	local opaqueTSB				= true
	local fullscreen			= true
	Avatar.setEnableActivity( enableActivity, excludeFromRecents, systemUIVisible, backgroundColor, transparent, opaqueTSB, fullscreen )

	-- render continuously
	local onVsync 				= true
	local renderContinuous		= true
	local alarmType				= 0
	local alarmLazy				= false
	local alarmTimeMS			= 0
	local alarmRepeatMS			= 0
	Scheduler.schedule( onVsync, renderContinuous, alarmType, alarmLazy, alarmTimeMS, alarmRepeatMS )

	return app:onInitialize()
end

function Application.onTerminate()
	app:onShutdown()
end

function Application.onUpdate()
	if Application.shutdown then
		Avatar.setWindowLockOrientation( false )
		Avatar.setWindowKeepScreenOn( false )

		-- tick once
		Scheduler.cancel()
		Scheduler.requestFrame()
		
		Avatar.restart( 500, false )
		Scheduler.cancel()
		return
	end
	
	app:onUpdate()
end

function Application.onReshape( width, height )
	app:onReshape( width, height )
end

function Application.onRender()
	if not Application.paused then
	   app:onRender()
	end
end

function Application.onActivityEvent( event )
	if event == "onStop" then
		Application.shutdown = true
	elseif event == "onDummyActivityGainedFocus" then
		Application.onResume()
	elseif event == "onDummyActivityLostFocus" then
		Application.onPause()
	end
end

function Application.onChangeDisplaySize( screenWidth, screenHeight, windowWidth, windowHeight )
	local x = 0
	local y = 0
	local w = screenWidth
	local h = screenHeight

	if not Application.fullscreen then
		x = 0
		y = screenHeight - windowHeight
		w = windowWidth
		h = windowHeight
	end

	app.view:setClipRect( x, y, w, h )
end

function Application.onBatteryEvent( newState )
	if newState == 0 then
		Application.shutdown = true
		Application.onResume()
	end
end

function Application.onPause()
	Application.paused = true
    if GUI then
        GUI.hide()
    end
	Sensor.pause()

	Avatar.setWindowLockOrientation( false )
	Avatar.setWindowKeepScreenOn( false )

	Scheduler.cancel()
	Scheduler.requestFrame()
end

function Application.onResume()
	Application.paused = false
    if GUI then
	    GUI.show()
    end
	Sensor.resume()

	Avatar.setWindowLockOrientation( true )
	Avatar.setWindowKeepScreenOn( true )

	-- render continuously
	local onVsync 			= true
	local renderContinuous	= true
	local alarmType			= 0
	local alarmLazy			= false
	local alarmTimeMS		= 0
	local alarmRepeatMS		= 0
	Scheduler.schedule( onVsync, renderContinuous, alarmType, alarmLazy, alarmTimeMS, alarmRepeatMS )
end

function Application.onRotation( deviceRotation )
end

function Application.onInterceptTouch( windowX, windowY, x, y, eventTime, onTSB )
end

function Application.onPPStateChange( window )
end

function Application.onPPLocSizeChanged( appBecameFullScreen, window )
end

function Application.onSystemUIVizUpdate( newVisibility )
end

function Application.onVolumeUp( newVolume )
end

function Application.onVolumeDown( newVolume )
end

function Application.onScreenEvent( newState )
end

function Application.onHeadsetEvent( newState )
end

function Application.onBluetoothEvent( newState )
end

function Application.onKeyPress( keyCode, action, isLongPress, downTime )
end

function Application.onTouchDown( pointerId, x, y, eventTime, touchPhase )
    Logger:message("Touch down " .. pointerId .. ", " .. x .. ", " .. y .. ", " .. eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchDown", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    if(pointerId == 0) then
        Events.send(Application, "touchDown", Vector(x, y, 0, 0))
    end

    return true
end

function Application.onTouchUp( pointerId, x, y, eventTime, touchPhase )
    Logger:message("Touch up " .. pointerId .. ", " .. x .. ", " .. y .. ", " ..eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchUp", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    if(pointerId == 0) then
        Events.send(Application, "touchUp", Vector(x,y, 0, 0))
    end

	return true
end

function Application.onTouchMove( pointerId, x, y, eventTime, touchPhase )
    Logger:message("Touch move " .. pointerId .. ", " .. x .. ", " .. y .. ", " .. eventTime .. ", " .. touchPhase)

    -- dispatch to current state
    Shell.dispatch("onTouchMove", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase})
    if(pointerId == 0) then
        Events.send(Application, "drag", Vector(x, y, 0, 0))
    end

	return true
end

function Application.onRequestExit()
	return true
end



