

local headingCache = {}

HeadingVolume = {
}

function HeadingVolume:clearCache()
    headingCache = {}
end

function HeadingVolume:getVolumeFor(actorName, headingVolumeCurve)
    local heading = self:_getCameraHeadingFor(actorName)
    if not heading then return nil, nil end

    return self:_evaluateHeadingVolumeCurve(headingVolumeCurve, heading)
end

function HeadingVolume:_getCameraHeadingFor(actorName)
    local heading = headingCache[actorName]
    if heading then
        return heading
    end

    local position = self:_getTargetActorPosition(actorName)
    if not position then
        --        Log.error("Unable to get tracking position for actor: " .. actorName)
        return nil, false
    end

    local camera        = Global.scene.cameras.main
    local cameraParent  = camera.parent or camera
    local cameraWorld 	= camera:getWorld()
    local cameraForward = cameraWorld:forward()

    -- Direction to the actor
    local targetForward = position - cameraParent:getWorld().position
    targetForward.w 	= 0
    targetForward:normalize()

    -- Camera right without roll
    local cameraRight 	= Vector.cross(cameraForward, Vector.up)
    cameraRight.w		= 0
    cameraRight:normalize()

    local forwardDot 	= Vector.dot(targetForward, cameraForward)
    local rightDot 		= Vector.dot(targetForward, cameraRight)

    -- Compute the heading with:
    -- heading =   0 -> camera is looking right at the target
    -- heading = 0.5 -> target is directly to the left of the camera
    -- heading =   1 -> target is directly behind the camera
    -- negative of these are on the right side
    heading = (1 - forwardDot) * 0.5
    if rightDot < 0 then heading = -heading end

    headingCache[actorName] = heading
    return heading
end

function HeadingVolume:_getTargetActorPosition(actorName)
    local actor = Global.scene.actors[actorName]
    if not actor or not actor.targetLocator then return nil end

    return actor:getBoneWorld(actor.targetLocator).position
end

function HeadingVolume:_evaluateHeadingVolumeCurve(curve, heading)
    if not curve or #curve == 0 then return end

    local volume, pan
    local lookupHeading = heading
    if curve.symmetric then
        lookupHeading = math.abs(heading)
    end

    -- walk the curve data table and find where our heading fits
    local upperEntry, lowerEntry
    for _, entry in ipairs(curve) do
        if lookupHeading < entry.heading then
            upperEntry = upperEntry or entry
        else
            lowerEntry = entry
        end
    end

    -- If we are lower than the lowest or higher than the highest use the same for both
    lowerEntry = lowerEntry or upperEntry
    upperEntry = upperEntry or lowerEntry

    -- lerp between the upper and lower to get the volume and panning balance
    local t = (lookupHeading - lowerEntry.heading) / (upperEntry.heading - lowerEntry.heading)
    pan = math.lerp(t, lowerEntry.pan, upperEntry.pan)
    if heading < 0 then pan = -pan end
    volume = math.lerp(t, lowerEntry.volume, upperEntry.volume)

    return volume, pan
end
