

ActorTracker_Controller = 
{
	previousTrackedLabel = nil,
	previousTrackedRotation = nil,
	currentTrackedLabel = nil,
	currentTrackedRotation = nil,

}

function ActorTracker_Controller:onStart() 
end

local function getWindowPosition(worldPosition, screenWidth, screenHeight)
	local ndc = Global.mainView:project(worldPosition)
	if(ndc.y > 1.1 or ndc.y < -1.1 or ndc.x > 1.2 or ndc.x < -1.2 or ndc.z < 0 or ndc.z > 1) then
		return nil;
	end
	ndc.x = 0.5*screenWidth*(ndc.x + 1)
	ndc.y = 0.5*screenHeight*(1 - ndc.y)
	
	return ndc;
end

local function renderAxes (inputRotation, inputLabel)
    local mySRT = {
        rotation=inputRotation,
        position= Vector.new ( 0, 0, -.002),
        scale= {1,1,1},
    }

    mySRT.position = Vector.rotateBy(mySRT.position, inputRotation);
   -- Log.message("rendering " .. inputLabel .. " position " .. printVect(mySRT.position) .. " rotation " .. printQuat(mySRT.rotation) .. " euler " .. printVect(Quaternion.getEuler(mySRT.rotation)));
    --local labelposition = getWindowPosition(mySRT.position, Global.curScreenWidth, Global.curScreenHeight)
    
    if (labelposition == nil) then 
        labelposition = {}
        labelposition.x = 0
        labelposition.y = 0
    end
    Renderer.drawText(labelposition.x, labelposition.y, textStyle, inputLabel)
    
    RenderDebug.axes(mySRT, 100.0)
end


local function printQuat(q)
    return tostring( math.floor(q.x*100)/100) .. " " .. tostring( math.floor(q.y*100)/100) .. " " ..  tostring(math.floor(q.z*100)/100) .. " " .. tostring(math.floor(q.w*100)/100)
end

function ActorTracker_Controller:onUpdate()
	if GUI and GUI.isBlocking() then
		return
	end

	local rotation			= self.object:getRotation()
	local position			= self.object:getWorld().position
	local currentRotation   = Quaternion()

	local cameraPosition 
	if ( Global.scene.cameras.main ) then
		cameraPosition = Global.scene.cameras.main:getWorld().position
	else
		cameraPosition = Vector.new(0,0,0)
	end
 	for _, actor in pairs(Global.scene.actors) do

 		if (actor.components and actor.components.LookAtDetector and self.previousTrackedLabel and actor.components.LookAtDetector.label == self.previousTrackedLabel)  then
 			local transform = actor:getBoneLocal(self.previousTrackedLabel)
			if (transform) then

				currentRotation = Quaternion.lookAtWithUp(transform.position - cameraPosition, Vector.up)
				local diffRotation =  currentRotation * Quaternion.inverse(self.previousTrackedRotation)
				local weightedTrackingRotation = Quaternion.slerp(actor.components.LookAtDetector.trackingWeight,Quaternion.new(), diffRotation)
				rotation = rotation *currentRotation * Quaternion.inverse(self.previousTrackedRotation)

				rotation = Quaternion.lookAtWithUp(Vector.forward:rotateBy(rotation), Vector.up)


				renderAxes(rotation, "final")
				renderAxes(self.object:getRotation(), "original")
				renderAxes(currentRotation,"current")



			end
			self.previousTrackedLabel = nil

 		end
 		if (actor.components and actor.components.LookAtDetector and Global.currentTrackedActor and actor.components.LookAtDetector.label == Global.currentTrackedActor)  then

			local transform = actor:getBoneLocal(Global.currentTrackedActor)
			if (transform) then
				self.previousTrackedLabel = Global.currentTrackedActor

				self.previousTrackedRotation = Quaternion.lookAtWithUp(transform.position - cameraPosition, Vector.up)

			end


		end

	end

	Global.AppDebugText3 = Global.currentTrackedActor
	Global.currentTrackedActor = nil






	--Log.message("Rotation " .. printQuat(rotation))
	--Log.message("camera position " .. cameraPosition.x .. " y " .. cameraPosition.y .. " z " .. cameraPosition.z)
	--Log.message("Position x " .. position.x .. " y " .. position.y .. " z " .. position.z)
	
	--For right now disable this functionality
	self.object:setRotation( rotation )
end