
local Logger = TaggedLog("CameraController", true)

CameraController = 
{
	cameraRig = nil,
	
--	smoothCamera		= true,
--	cameraSpeed			= 16,
--	boomLength			= 1,
--	cameraCrane			= Vector(),
--	lookAtOffset		= Quaternion(),
    
    layersStarted       = false,
}

function CameraController:new(data)
	local obj = shallowcopy(data) or {}
    setmetatable(obj, self)
    self.__index = self
    
    obj:init()
    return obj
end

function CameraController:init()
	self.cameraRig          = CameraRig:new()

	-- Camera layers setup
    self.inputLayer         = self.cameraRig:addLayer(InputLayer:new())
    --self.trackingLayer      = self.cameraRig:addLayer(TrackingLayer:new())
    --self.targetWarpLayer    = self.cameraRig:addLayer(TargetWarpLayer:new())
end

function CameraController:attachCamera(camera)

end

function CameraController:lookAt(pos)
    -- Work around for trying to look at the origin (which sometimes happens with incomplete data)
	if(not self.lookAtLayer or (pos.x == 0 and pos.y == 0 and pos.z == 0) ) then
		return;
	end

	self.lookAtLayer:setLookAt(lookAt)
end

function CameraController:start()
    MainCamera.camForward    = Vector.forward:copy()
    MainCamera.camRight      = Vector.right:copy()
    MainCamera.camUp         = Vector.up:copy()
    
    self.cameraRig:startLayers()
    self.layersStarted = true
end

function CameraController:onLateUpdate()
    if(self.layersStarted) then
        self.cameraRig:updateLayers()

        -- HACK to avoid crash when show is interrupted. Ideally, the layers in cameraRig should not get populated with bogus values upon interruption
        -- This check was moved as part of a code port and we should investigate if it's still needed
        if (self.cameraRig.worldT.rotation:lengthSq() > 0 and self.cameraRig.worldT.rotation:lengthSq() < 2) then        
            self.object:setLocal(self.cameraRig.worldT)
        end
        
        MainCamera.camForward  = self.cameraRig.camForward:copy()
        MainCamera.camRight    = self.cameraRig.camRight:copy()
        MainCamera.camUp       = self.cameraRig.camUp:copy()
    end
end

function CameraController:onDestroy()
    self.cameraRig.destroyLayers()
end
