

local Logger = TaggedLog("TrackingLayer", true)

TrackingLayer = createClass({
    --targetOffset        = Quaternion.new(),
    --localT              = SRT.new(),
    
    driverName          = "",
    
    actorName           = "",
    trackingRange       = 2.0,
    prevDeltaRot        = Quaternion.new(),
	
}, deepcopy(CameraLayer))

TrackingLayer.name = "TrackingLayer"
	
function TrackingLayer:onStart()
    self.actorName = "TrackGirl"
end

--function TrackingLayer:onUpdate(dt)
function TrackingLayer:onPostUpdate(worldT)
    local actor = Global.scene.actors[self.actorName]

    if(actor and actor.targetLocator) then

        local targetTransform = actor:getBoneWorld(actor.targetLocator)
        local actorPos = targetTransform.position
        actorPos.w = 0.0
        actorPos:normalize()
      
        local targetRot     = Quaternion.lookAt(actorPos)
        local targetOffset  = self.targetRot

        local prevRot       = worldT.rotation * self.prevDeltaRot
        local fwd           = Vector.rotateBy(Vector.forward, prevRot)
        
        -- A 0-1 value that should map to 0-180 degrees of angle (possibly scaled for tuning)
        local range = math.acos( Vector.dot( actorPos, fwd )) / math.pi
        range = range * 0.9
        
        local blend = math.clamp(1 + range/((range*range)-1), 0, 0.8)

        worldT.rotation = Quaternion.lerp(blend, worldT.rotation, self.targetOffset) 
        
        self.prevDeltaRot = prevRot:inverse() * worldT.rotation 
        
        --Global.AppDebugText2 = string.format("actorPos %.3f %.3f %.3f : camFwd %.3f %.3f %.3f", actorPos.x, actorPos.y, actorPos.z, fwd.x, fwd.y, fwd.z)
        --Global.AppDebugText2 = string.format("str %.3f : targetRot %.3f %.3f %.3f %.3f : prev %.3f %.3f %.3f %.3f", self.strength, self.targetRot.x, self.targetRot.y, self.targetRot.z, self.targetRot.w, worldT.rotation.x, worldT.rotation.y, worldT.rotation.z, worldT.rotation.w)
        --Logger:message(Global.AppDebugText3)
	end
end
