

CameraLock = {
}


CameraLock.new = function(self, params)
    local o = shallowcopy(params) or {}

    Log.message("new : table = " .. table.tostring(o))

    setmetatable(o, self)
    self.__index = self

    return o
end


CameraLock.onStart = function(self)
    -- Record initial target values, since these should be static
    local targetWorld       = self.object:getBoneWorld(self.targetBone)

    self.targetStartDir     = Vector.normalized(targetWorld.position)   
    self.targetStartRot     = targetWorld.rotation
    --self.targetStartQuat    = Quaternion.lookAtWithUp(self.targetStartDir, Vector.up)
    self.targetStartQuat    = Quaternion.lookAtWithUp(self.targetStartDir, targetWorld:up())
    self.targetUp           = targetWorld:up()
    
    local camWorld          = MainCamera:getWorld()    
    self.dotCamUp          = Vector.dot(Vector.up, camWorld:up())
    self.dotCamRight       = Vector.dot(Vector.up, camWorld:right())
end


CameraLock.onDestroy = function(self)

end


CameraLock.onLateUpdate = function(self)
    -- Current camera transform
    local camWorld  = MainCamera:getWorld()
    local camFwd    = camWorld:forward()
    local camUp     = self.targetUp

    -- Use the world "Up" when horizontal, use the actor's up when pointed stright down, and some blend when in-between
    -- This prevents us from the having really ugly flipping at the poles as we switch between "up"s
    local targetWorld   = self.object:getBoneWorld(self.targetBone)
    local dotFwd        = Vector.dot(Vector.up, targetWorld:forward())
    local upBlend       = 1.0 - (math.asin(math.abs(dotFwd)) / (math.pi * 0.5))
    
    local currentUp     = nil
    
    -- Pick an "up" that changes based on how far off the camera is from world up
    --currentUp     = Vector.normalized(Vector.lerp(upBlend, targetWorld:up(), camUp))

    -- Hard-lock to a camera orientation based on 
    if (math.abs(self.dotCamUp) > math.abs(self.dotCamRight)) then
        currentUp = camWorld:up() * math.sign(self.dotCamUp)
    else
        currentUp = camWorld:right() * math.sign(self.dotCamRight)
    end

    local destQuat  = Quaternion.lookAtWithUp(camFwd, currentUp)
    local diffQuat  = destQuat * self.targetStartQuat:inverse()
    
    local finalQuat = nil
    
    if (self.hardLock == false) then
        local diffDot   = Vector.dot(camFwd, self.targetStartDir)
        local rotBlend  = ((math.asin(diffDot) / (math.pi * 0.5)) + 1) * 0.5
        rotBlend        = math.pow( 2, 10 * (rotBlend - 1))
        rotBlend        = 0.7 + rotBlend * 0.3
        
        Global.AppDebugText2 = string.format("diffDot %4.4f : blendAmount %4.4f", diffDot, rotBlend )
        finalQuat            = Quaternion.slerp( rotBlend, Quaternion.identity, diffQuat )
    else
        finalQuat            = diffQuat
    end
    
    -- Create a new orientation that rotates the target to be in front of the cam
    local localT    = self.object:getLocal()
    localT.rotation = self.targetStartRot * finalQuat    
    
    -- Set the new orientation
    self.object:setLocal(localT) 
end