

FBDriver = {
    sampler         = nil,
    flipbookURI     = nil,
    material        = nil,
    animation       = nil,
    
    looping         = false,
    playing         = false,
    currentFrame    = 0,
    timer           = 0,
    elapsed         = 0,
    
    frameOffset     = 0,
    frameStart      = 0,
    frameStartOffset= 0,
    frameEnd        = 1000,
    frameRate       = 60,
    frameTranslate  = 0,
    
    animStartFrame  = 0,
    animFPS         = 0,
    
    luaDriven       = false,
}


FBDriver.new = function(self, params)
    local o = shallowcopy(params) or {}

    Log.message("new : table = " .. table.tostring(o))
    
    setmetatable(o, self)
    self.__index = self
        
    return o    
end


FBDriver.onStart = function(self)
    Log.message("Starting flipbook " .. tostring(self.flipbookURI))

    self.sampler = FlipbookSampler.create()
    self.sampler:setFlipbook(self.flipbookURI)
    self.sampler:setLooping(self.looping)
    
    local mat = self.object:findMaterial(self.material)
    self.sampler:setMaterial( mat, { "u_FlipbookTex" } )

    local matProps = mat:getPropertiesTable()
     -- fbStart for flipbook frame offsets
    if matProps["fbStart"] then
        self.frameOffset = matProps["fbStart"]
    end

    --self.animation      = nil
    self.animation      = self.object.animation[self.anim]
    self.animStartFrame = self.animation:getStartFrame()
    self.animFPS        = self.animation:getFramesPerSecond()
    
    self.frameStart     = (self.animStartFrame / self.animFPS) * self.frameRate
    
    self.currentFrame   = 0
    self.playing        = false
    self.timer          = 0
    self.elapsed        = 0
    
    self:play()

    --Global.AppDebugText1 = "Started "..mat:getTechniqueName()
end


FBDriver.onDestroy = function(self)
    self:stop()
    if(self.sampler) then
        Log.message("Destroying flipbook " .. tostring(self.material))
        self.sampler:destroy();
        self.sampler = nil
    end
end


FBDriver.play = function(self) 
    self.playing = true;
end


FBDriver.stop = function(self) 
    self.playing = false;
end


FBDriver.showFrame = function(self, frameNumber)
	self.sampler:setFrame(frameNumber)
end


FBDriver.showTime = function(self, currentTime)
	self.sampler:setTime(currentTime)
end


FBDriver.onLateUpdate = function(self)
	if (self.playing) then
        self.currentFrame = self:getNextFrame()
        local animTime = self.animation:getTime()
        --Global.AppDebugText2 = string.format("%4d (%4d) currentFrame : %4f animTime : %3.3f frameRate", self.currentFrame, self.frameStart + self.currentFrame, animTime, self.animFPS )        

		self.sampler:setFrame(self.frameStart + self.currentFrame)
	end
end


FBDriver.getNextFrame = function(self)
    local animFrame = 0
    
    if (self.luaDriven) then
        -- lua will keep track of time and calculate the next frame
--        self.elapsed = self.elapsed + Time.dt()
        self.elapsed = self.animation:getTime()
        
        animFrame = math.max(0, self.frameStart + math.floor(self.elapsed * self.frameRate) + self.frameTranslate );
        
        if(animFrame >= self.frameEnd) then 
            -- Unused in Help project... for now
            --[[
            if(self.wrapMode == FBWrapMode.clamp) then -- wrapMode == CLAMP
            
                animFrame = self.frameEnd
                self.finished = true
                
            elseif(self.wrapMode == FBWrapMode.countedLoop) then -- wrapMode == COUNTED LOOP
                
                -- add completed loop
                self.loopCount = self.loopCount + 1
                
                if(self.loopCount < self.loopCountGoal)then
                    self.elapsed    = 0
                    animFrame       = self.frameStart
                else
                    animFrame       = self.frameEnd
                    self.finished   = true
                end
                
            else -- wrapMode == LOOP
            
                self.elapsed = 0
                animFrame = self.frameStart
                
            end
            --]]
        end
    else
        -- frame according to FBX animation time
        animFrame = math.floor(self.animation:getTime() * self.frameRate);
    end

    return animFrame
end
