

local debugMode = false

local function printQuat(q)
    return tostring( math.floor(q.x*100)/100) .. " " .. tostring( math.floor(q.y*100)/100) .. " " ..  tostring(math.floor(q.z*100)/100) .. " " .. tostring(math.floor(q.w*100)/100)
end
LookAtDetector = {
    worldPosT = nil,
    isTriggered = false,
    triggeredOnce = false,
    label = nil,

    colliderRadius = 0,
    previousFrameTriggered = false,
    trackingWeight = 1.0,
    hideActor = false,

	isDebugDrawOn = false,
    debugSphere = nil,
}

LookAtDetector.new = function(self, params)
    local o = shallowcopy(params) or {}

    Log.message("new : table = " .. table.tostring(o))
    if (o.label == nil) then
        o.label = "LookAtDetector"
    end

    setmetatable(o, self)
    self.__index = self

    return o
end

LookAtDetector.onStart = function(self)

end

LookAtDetector.getTransform = function(self)
    return self.worldPosT
end

LookAtDetector.calculateAngleToCamera = function(self)
    if (MainCamera ~= nil) then
        local cameraRotation = MainCamera:getWorld().rotation
        local rotT = Quaternion()
        if (self.label) then
          local posT = self.object:getBoneWorld(self.label).position
          rotT = Quaternion.lookAtWithUp(posT, Vector.up)
          local negRot = Quaternion.new (-rotT.x, -rotT.y, -rotT.z, -rotT.w)
          local angle = math.min( Quaternion.angle(rotT, cameraRotation), Quaternion.angle(negRot, cameraRotation))

          if (angle < self.colliderRadius or self.previousFrameTriggered) then
            self.hideActor = false
            Global.currentTrackedActor = self.label
          else
            --self.hideActor = true
          end
          
		  
		  Global.AppDebugText2 = "angle = " .. self:getAngleOffset() .. " and radius = " .. self.lookAtThreshold 
          if (self:getAngleOffset() >= self.lookAtThreshold) then
            self.previousFrameTriggered = true
          else
            self.previousFrameTriggered = false
          end

        end
    end

end

LookAtDetector.getAngleOffset = function(self)

    local bone = self.object:getBoneWorld(self.label)
	local posT = bone.position

	

    -- Don't use full camera pos because it leads to parallax error (since the cam is on a boom)
    -- Do account for the camera height however, so get the rig
    if (Global.scene.cameras.main == nil) then
        return
    end

    local toTarget = posT - Global.scene.cameras.main:getWorld().position

    -- If the camera to target vector has changed then something moved, recalculate or goal value
    if (self.toTarget == nil or self.toTarget ~= toTarget) then
        self.toTarget = toTarget:copy()
        self.lookAtThreshold = self:calculateGoal()
    end

    toTarget:normalize()

    local look = Global.scene.cameras.main:getWorld():forward()

    self.dotDifferential = Vector.dot(look, toTarget)
    local dotDifferential = self.dotDifferential
	
	return dotDifferential
end 


LookAtDetector.onUpdate = function(self)

    if (MainCamera == nil) then
        return
    end

   -- self:calculateAngleToCamera()
	
	if (self:getAngleOffset() >= self.lookAtThreshold) then
		self.isLookedAt = true
	else
		self.isLookedAt = false
	end
	
	if (self.isLookedAt and not self.triggeredOnce) then
		self.triggeredOnce = true
		Events.send(self, self.eventName, self.eventData);
	end

    self.isTriggered = (Global.scene.useDebugTargets ~= nil and Global.scene.useDebugTargets) and (not self.hideActor)

    if (self.enableHiding and self.isTriggered ~= self.object:isVisible()) then 
        self.object:setVisibility(self.isTriggered)
    end
	
	if(self.isDebugDrawOn) then
		local bone = self.object:getBoneWorld(self.label)
	    RenderDebug.sphere(bone.position, self.colliderRadius*0.25)
    end
    
    --local posT = self.object:getBoneWorld(self.label).position  
    --Global.AppDebugText1 = string.format("%s target %.3f, %.3f, %.3f : offset %.3f : %s",self.object.name, posT.x, posT.y, posT.z, self:getAngleOffset(), tostring(self.isLookedAt))
end

LookAtDetector.lookedAt = function(self)
	return self.isLookedAt --self.isTriggered
end

LookAtDetector.calculateGoal = function(self)
    -- Calculate the minimum result of our dot product for the location and radius
    -- originally generated in the prototype.

    local targetLen	= self.toTarget:length()
    local maxAngle = math.atan2( self.colliderRadius, targetLen )

    -- Minimum dot-product we should need in our transition check
    local dotGoal = math.cos(maxAngle)

    return dotGoal
end


LookAtDetector.onDestroy = function(self)

end



