

def = def or {}
def.audio = def.audio or {}

def.audio.FrameLatency = 10  -- In frames

-- Heading based volume curves
-- heading =   0 -> camera is looking right at the target
-- heading = 0.5 -> target is directly to the left of the camera
-- heading =   1 -> target is directly behind the camera
-- negative of these are on the right side
-- if symmetric is true then you only need to define one side


 local GirlHeadingCurve = {
    { heading =    0, volume =   1, pan = 0, },
    { heading =  0.5, volume =   1, pan = 1, },
    { heading =  0.6, volume =  0.7, pan = 1, },
    { heading =    1, volume =  0.2, pan = 0, },

     symmetric = true,
 }
 
 local HeliHeadingCurve = {
    { heading =    0, volume = 1, pan = 0, },
    { heading =  0.5, volume = 1, pan = 0, },
    { heading =  0.6, volume = 0.7, pan = 0, },
    { heading =    1, volume = 0.2, pan = 0, },

     symmetric = true,
 }
 
 local HeliRotorHeadingCurve = {
    { heading =     0, volume = 1, pan = 0, },
    { heading =  0.25, volume = 1, pan = 0, },
    { heading =   0.5, volume = 0.5, pan = 0, },
    { heading =     1, volume = 0, pan = 0, },

     symmetric = true,
 }
 
  local InteriorExplosionCurve = {
    { heading =    0, volume = 1, pan = 0, },
    { heading =  0.5, volume = 1, pan = 0, },
    { heading =  0.6, volume = 1, pan = 0, },
    { heading =    1, volume = 1, pan = 0, },

     symmetric = true,
 }
 
 local AlienHeadingCurve = {
    { heading =    0, volume = 1, pan = 0, },
    { heading =  0.5, volume = 1, pan = 1, },
    { heading =  0.6, volume = 0.7, pan = 1, },
    { heading =    1, volume = 0.2, pan = 0, },

     symmetric = true,
 }

-- The audio track the Help video will be synced to
def.audio.primary = "sfx_mainshow"

def.audio.sounds = {
    -- Order may matter here for preloading, so listed in order of usage
    ["sfx_title"]      = { uri = "help:audio/help_sfx_bgloop.ogg" },
    ["sfx_title_in"]   = { uri = "help:audio/help_sfx_evolveloop.ogg" },
    ["music_title"]    = { uri = "help:audio/help_mx_openloop.ogg" },
    ["sfx_title_out"]  = { uri = "help:audio/help_sfx_devolve.ogg" },
    ["sfx_mainshow"]   = { uri = "help:audio/help_sfx.ogg", binaural = true },
    ["music_mainshow"] = { uri = "help:audio/help_mx.ogg" },
    ["music_credits"]  = { uri = "help:audio/help_mx_credits.ogg" },
}

def.audio.segments = {
    SegmentFadeToShow = {
        { play = "music_title", at = 0, volume = 0.7, startTime = 0, fadeTime = 3.0, loop = true },
    },
    SegmentTitle = {
        { play = "sfx_title",   at = 0, volume = 1.0, startTime = 0, fadeTime = 2.0, loop = true },
        { play = "sfx_title_in",at = 0, volume = 1.0, startTime = 0, fadeTime = 2.0, loop = true },
    },
    SegmentMainShow = {
        { stop = "music_title", at = 0, fadeTime = 3.0 },
        { stop = "sfx_title",   at = 0, fadeTime = 3.0 },
        { stop = "sfx_title_in",at = 0, fadeTime = 1.5 },
    
        { play = "sfx_title_out",  at = 0, volume = 1.0, startTime = 0, loop = false },

        { play = "sfx_mainshow",   at = 0, volume = 1.0, startTime = 0, fadeTime = 2.0 },
        { play = "music_mainshow", at = 0, volume = 0.7, startTime = 0, fadeTime = 2.0 },
        
        { stop = "sfx_title_out",  at = 60, fadeTime = 0.0 },
    },
    SegmentCredits = {
        { stop = "sfx_mainshow",   at = 0, fadeTime = 3.0 },
        { stop = "music_mainshow", at = 0, fadeTime = 3.0 },
        
        { play = "music_credits",  at = 0, volume = 1.0, startTime = 0 },
        
    },
    SegmentCreditsFade = {
        { stop = "music_credits",  at = 0, fadeTime = 3.0 },
    }
}
