

Settings = 
{
    scene       = "video",
    paused      = false,
    resolution  = 2048, 
    lut_enabled = false,
    qual_slider         = nil,
    adaptive_toggle     = nil,
    tracking_debug = nil,
    target_centering = nil,
    trackingStrengthAnimation = nil,
    trackingStrength = nil,
    tracking_comp = nil,
    tracking_hero = nil,
    onInitialized = nil,
    tclEnabled = true,
    tclDebug = true,
    trackingAnalytics = false,
    precisionMin = 1.25,
    precisionMax = 7,
}
Settings.__index = Settings

function Settings.create( app )
    local self          = {}
    setmetatable( self, Settings )

    self.app                  = app
    self.window               = GUI.window( GUIPos(22,22), GUISize(320, 900), "Settings" )

	local version_id          = GUI.checkbox(self.window, GUIPos(0,  5), GUISize(200, 30), true, "V4 - updated pitch clamps")
	Settings.fps          	  = GUI.checkbox(self.window, GUIPos(0,  35), GUISize(200, 30), false, "Show FPS")
    Settings.qual_slider         = GUI.slider(self.window,   GUIPos(0,  65), 300, "Quality", 10, 0, 10 )
    Settings.adaptive_toggle     = GUI.checkbox(self.window, GUIPos(0,  105), GUISize(200, 30), true, "Adaptive Quality")

    -- Targets Toggle
--    local target_toggle = GUI.checkbox(self.window, GUIPos(0,90+65), GUISize(200, 30), false, "LookAt Target Debug")
    Settings.tracking_debug   = GUI.checkbox(self.window, GUIPos(0,  130), GUISize(200, 30), false, "Tracking Debug")
    Settings.target_centering = GUI.slider(self.window,   GUIPos(0,  180), 300, "Target Centering", 0.5, 0.0, 1)
    Settings.trackingStrengthAnimation = GUI.checkbox(self.window, GUIPos(0, 240), GUISize(200, 30), false, "Use Global Track Animation")
    Settings.trackingStrength = GUI.slider(self.window,   GUIPos(0,  270), 300, "Global Tracking Strength", 1, 0.0, 1)
    Settings.tracking_comp    = GUI.checkbox(self.window, GUIPos(20, 330), GUISize(200, 30), false, "Composition Tracking")
    Settings.tracking_hero    = GUI.checkbox(self.window, GUIPos(20, 360), GUISize(200, 30), false, "Hero Tracking")
    Settings.tclEnabled       = GUI.checkbox(self.window, GUIPos(0,  390), GUISize(200, 30), false, "TCL Tracking")
    Settings.tclDebug         = GUI.checkbox(self.window, GUIPos(0,  410), GUISize(200, 30), false, "TCL Debug")

    -- Color
    local lut_toggle          = GUI.checkbox(self.window, GUIPos(0,  440), GUISize(200, 30), true, "Enable Color LUT")

    -- Performance
    local debug_toggle        = GUI.checkbox(self.window, GUIPos(0,  470), GUISize(200, 30), false, "Debug Timeline")
    local debug_mode_slider   = GUI.slider(self.window,   GUIPos(0,  500), 300, "Debug Mode", 0, 0, 4 )
    
    -- analytics
    Settings.trackingAnalytics  = GUI.checkbox(self.window, GUIPos(0,  560), GUISize(200, 30), false, "Tracking Analytics")

    -- Yaw-Multiplier Toggle
    --local yawmult_slider   = GUI.slider(self.window, GUIPos(0,120), 300, "Yaw Multiplier", 1.0, 0.0, 4.0 )    

    --[[
    local frame_label   = GUI.label( self.window, GUIPos(0,120), GUISize(50,20), "Frames:", "text" )
    local frame_menu    = GUI.dropdownmenu( self.window, GUIPos(0,140), 0, 
    {
        "ft05fps30v01_cubemap3x2_1024",
        "ft10fps30v01_cubemap3x2_1024",
        "ft15fps30v01_cubemap3x2_1024",
        "intaptv01f24_cubemap3x2_1024",
        "runningtest02v02f30_cubemap3x2_1024",
        "shot04v01_cubemap3x2_1024",
    } )
    --]]
    
    Settings.precisionMin = GUI.slider(self.window,   GUIPos(0,  620), 300, "PrecisionMin", 1.25, 0.01, 5)
    Settings.precisionMax = GUI.slider(self.window,   GUIPos(0,  680), 300, "PrecisionMax", 7, 5, 10)
    
    local video_frame   = GUI.frame( self.window, GUIPos(0,740), GUISize(320,500), "frame" )
    local pause_button  = GUI.button( video_frame, GUIPos(100,10), GUISize(100,40), "Pause" )
    pause_button.paused = false;

    function pause_button:onButtonDown()
        if self.paused then
            self.paused = false
            self:setCaption( "Pause" )
        else
            self.paused = true
            self:setCaption( "Play" )
        end

        app:pause( self.paused )
    end

	function Settings.fps:onclick()
        app:enableFps(self.checked)
    end
    
    function Settings.qual_slider:onValueChanged( value, old )
        app:setVideoQualityLevel( math.floor(value) )
    end

    function debug_mode_slider:onValueChanged( value, old )
        app:setVideoDebugMode( math.floor(value) )
    end

    function debug_toggle:onclick()
        app:setDebugTimeline(self.checked)
    end

    --function target_toggle:onclick()    
    --    app:enableDebugTargets(self.checked) 
    --end
    
    function Settings.tracking_debug:onclick()    
        app:enableTrackingDebug(self.checked) 
    end
    
    function Settings.target_centering:onValueChanged(value, old)    
        app:setTargetCenteringSpeed(value) 
    end
    
    function Settings.trackingStrengthAnimation:onclick()    
        app:enableGlobalTrackScaleAnimation(self.checked) 
    end
    
    function Settings.trackingStrength:onValueChanged(value, old)    
        app:updateGlobalTrackingStrength(value) 
    end
    
    function Settings.tracking_comp:onclick()   
        app:enableTrackingGroup("composition", self.checked)
    end
    
    function Settings.tracking_hero:onclick()  
        app:enableTrackingGroup("hero",  self.checked)
    end
    
    function Settings.adaptive_toggle:onclick()
        app:setAdaptiveQuality(self.checked)
    end

    function lut_toggle:onclick()
        app:setVideoCubeShader(self.checked)
    end
    
    function Settings.tclEnabled:onclick()    
        app:isTCLEnabled(self.checked) 
    end
    
    function Settings.tclDebug:onclick()    
        app:setTCLDebug(self.checked) 
    end
    
    function Settings.trackingAnalytics:onclick()    
        app:enableTrackingAnalytics(self.checked) 
    end
    
    function Settings.precisionMin:onValueChanged(value, old)    
        app:setCameraPrecisionFilterMinMax(value, Settings.precisionMax.value) 
    end
    
    function Settings.precisionMax:onValueChanged(value, old)    
        app:setCameraPrecisionFilterMinMax(Settings.precisionMin.value, value) 
    end

    --[[
    function yawmult_slider:onValueChanged( value, old )
        Global.scene.yawMult = value
    end
    --]]

    --[[
    function frame_menu:onItemSelected( item )
        app:selectFrame( item )
    end
    --]]
    
    if(self.onInitialized)then
        self:onInitialized()
    end

    return self
end


