

local Logger = TaggedLog("InputManager", true)

InputManager = {
    scene       = nil,
    
    rotation    = Quaternion.new(),
    calib       = Quaternion.new(),
    orient      = Quaternion.new(),

    isMouseDown = false,
    
    orientationAngle    = 0,
    orientations        = {0, 90, 180, 270},

    -- For win32 simulation
    axisAngles  = Vector.new(0,0,0),
    rotAngles   = Vector.new(0,0,0),

    rotationScaleX  = 0.1,
    rotationScaleY  = 0.1,

    step            = 800,
    orientation     = DEVICE_ORIENATION_0,
}

InputManager.getSensorRotation = function(self)
    return self.rotation
end

InputManager.init = function(self, scene)
    self.scene = scene;
    -- for win32 simulation
    if(Platform.win32) then
        self.onSensorUpdate = self.onSensorUpdateWindows;
        self.calib:setAxisAngle(Vector.right, math.rad(-90))
    elseif(Platform.android) then
        self.calib:setAxisAngle(Vector.right, math.rad(-90))
        self.onSensorUpdate = self.onSensorUpdateAndroid;
    elseif(Platform.iOS) then
        self.calib:setAxisAngle(Vector.right, math.rad(-90))
        self.onSensorUpdate = self.onSensorUpdateIOS;
    end
    self:enableSensors(true)
    return true
end


InputManager.paused = false
InputManager.pauseKeyUp = true
InputManager.lastGUIBlockingState = false;

InputManager.update = function(self)

    if GUI then
		local isGuiBlocking = GUI.isBlocking();
    	if isGuiBlocking ~= InputManager.lastGUIBlockingState then
	    	InputManager.lastGUIBlockingState = isGuiBlocking
	        MainCamera.controller:enableInput(not isGuiBlocking);
        end
    end

    self:onSensorUpdate()

    if (Input.isKeyDown(KB_F1)) then
        --        toggleRenderDebug();
        --      if(GUI.isVisible()) then
        --          GUI.hide();
        --      else
        --          GUI.show();
        --      end
    end

    -- enable debug camera
    if (Input.isKeyDown(KB_C)) then
        Input.KB_C_wasDown = true
    elseif(Input.KB_C_wasDown) then
        Input.KB_C_wasDown = false
        Global.scene.cameras.main.controller.active = not Global.scene.cameras.main.controller.active;
        Global.scene.cameras.main.debugcontroller.active = not Global.scene.cameras.main.debugcontroller.active;
    end

    Input.isM_Up = Input.isM_Up or true
    if (Input.isKeyDown(KB_M) and Input.isM_Up) then
    	Input.isM_Up = false
    	MainCamera.controller:startSensorClient();
    elseif (Input.isKeyUp(KB_M)) then
    	Input.isM_Up = true
    end

end

InputManager.onSensorUpdateAndroid = function(self)
    local sensorRotation    = Sensor.getSensorFusionOrientation()
    local orientation       = Input.getDeviceRotation()

    local angles = sensorRotation:getEulerAngles()
    --Global.AppDebugText1 = string.format("sensor %.3f, %.3f, %.3f", angles.x, angles.y, angles.z)

    local angle = self.orientations[orientation + 1]

    if(self.orientationAngle ~= angle) then
        self.orientationAngle = angle;
        self.orient:setAxisAngle(Vector(0, 0, -1), math.rad(angle))
    end

    if(sensorRotation:lengthSq() > 0.0) then
        self.rotation = self.calib * sensorRotation * self.orient
    end

    angles = self.rotation:getEulerAngles()
    --Global.AppDebugText2 = string.format("sensor %.3f, %.3f, %.3f", angles.x, angles.y, angles.z)
    
    -- Multiplier hack - Very simple attempt to "boost" the yaw and pitch movements
    --[[
    local yawMult   = Global.scene.yawMult or 1.0
    local pitchMult = Global.scene.pitchMult or 1.0
    
    local yaw   = Quaternion.axisAngle(Vector(0.0, 1.0, 0.0, 0.0), angles.y * yawMult)
    local pitch = Quaternion.axisAngle(Vector(1.0, 0.0, 0.0, 0.0), angles.x * pitchMult)
    local roll  = Quaternion.axisAngle(Vector(0.0, 0.0, 1.0, 0.0), angles.z)
    
    if(not self.lastYaw) then
        self.lastYaw = yaw:copy()
    end
    if(not self.lastPitch) then
        self.lastPitch = pitch:copy()
    end
    if(not self.lastRoll) then
        self.lastRoll = roll:copy()
    end

    yaw     = Quaternion.shortestDistance(yaw,   self.lastYaw)
    pitch   = Quaternion.shortestDistance(pitch, self.lastPitch)
    roll    = Quaternion.shortestDistance(roll,  self.lastRoll)

    --self.rotation = yaw * self.rotation
    self.rotation = yaw*pitch*roll
    --]]

    return self.rotation
end

InputManager.onSensorUpdateIOS = function(self)
    local sensorRotation    = Sensor.getSensorFusionOrientation()
    local orientation       = Input.getDeviceRotation()

    local angle = self.orientations[orientation + 1]

    if(self.orientationAngle ~= angle) then
        self.orientationAngle = angle;
        self.orient:setAxisAngle(Vector(0, 0, -1), math.rad(angle))
    end

    if(sensorRotation:lengthSq() > 0.0) then
        self.rotation = self.calib * sensorRotation * self.orient
    end

    return self.rotation
end

InputManager.onSensorUpdateWindows = function(self)

	-- Calculating our DT so system dt pause does not effect the camera
	local currTimeSecs = os.clock();
	if(self.lastUpdateSecs == nil) then
		self.lastUpdateSecs = currTimeSecs;
	end
	
	local dt = currTimeSecs - self.lastUpdateSecs;
	self.lastUpdateSecs = currTimeSecs;
	
    local scaledStep = self.step * dt

    if( (Input.getMouseButton(MB_L_BUTTON) == true) and (self.isMouseDown == false))then
        self.isMouseDown = true;
        Events.send("app", "touchDown", Input.getMousePosition());
    end
    if( (Input.getMouseButton(MB_L_BUTTON) ~= true) and (self.isMouseDown == true))then
        self.isMouseDown = false
        Events.send("app", "touchUp", Input.getMousePosition());
    end
    if(Input.getMouseButton(MB_L_BUTTON) and Input.isMouseMoving())then
        Events.send("app", "drag", Input.getMousePosition());
    end
    
    
    if(Global.scene.cameras.main.controller or Global.scene.cameras.main.controller.active)then
        self:updateLuaCamera()
    end

   
end

InputManager.enableSensors = function(self, enable)
	if(Platform.android) then
	    Sensor.setSensorFusionEnabled(enable)
	elseif(Platform.iOS) then
	    Sensor.setSensorFusionEnabled(enable)
	end
end

InputManager.updateLuaCamera = function(self)
	 if (Input.isKeyDown(KB_W)) then
        self.axisAngles.x = self.axisAngles.x + self.rotationScaleX*scaledStep;
    end

    if (Input.isKeyDown(KB_S)) then
        self.axisAngles.x = self.axisAngles.x - self.rotationScaleX*scaledStep;
    end

    if (Input.isKeyDown(KB_A)) then
        self.axisAngles.y = self.axisAngles.y + self.rotationScaleY*scaledStep;
    end

    if (Input.isKeyDown(KB_D)) then
        self.axisAngles.y = self.axisAngles.y - self.rotationScaleY*scaledStep;
    end

    if (Input.isKeyDown(KB_Z)) then
        self.axisAngles.z = self.axisAngles.z + self.rotationScaleY*scaledStep;
    end

    if (Input.isKeyDown(KB_X)) then
        self.axisAngles.z = self.axisAngles.z - self.rotationScaleY*scaledStep;
    end
    
    if(math.abs(self.axisAngles.x) > 180)then
        self.axisAngles.x = -1*math.sign(self.axisAngles.x)*180
    end
    if(math.abs(self.axisAngles.y) > 180)then
        self.axisAngles.y = -1*math.sign(self.axisAngles.y)*180
    end
    if(math.abs(self.axisAngles.z) > 180)then
        self.axisAngles.z = -1*math.sign(self.axisAngles.z)*180
    end

    local yaw 	= Quaternion.axisAngle(Vector(0.0, 1.0, 0.0, 0.0), 	self.axisAngles.y)
    local pitch = Quaternion.axisAngle(Vector(1.0, 0.0, 0.0, 0.0), 	self.axisAngles.x)
    local roll 	= Quaternion.axisAngle(Vector(0.0, 0.0, -1.0, 0.0), self.axisAngles.z)

    if(not self.lastYaw) then
        self.lastYaw = yaw:copy()
    end
    if(not self.lastPitch) then
        self.lastPitch = pitch:copy()
    end
    if(not self.lastRoll) then
        self.lastRoll = roll:copy()
    end

    yaw 	= Quaternion.shortestDistance(yaw,   self.lastYaw)
    pitch 	= Quaternion.shortestDistance(pitch, self.lastPitch)
    roll 	= Quaternion.shortestDistance(roll,  self.lastRoll)

    self.lastYaw 	= yaw:copy();
    self.lastPitch 	= pitch:copy();
    self.lastRoll 	= roll:copy();

    --local angles = self.lastRoll:getEulerAngles()
    --local textStyle = { color = Color.new(1.0, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = 1.0 }
    --Renderer.drawText(10, 100, textStyle, string.format("orig rpy <%.3f,%.3f,%.3f>", self.axisAngles.x, self.axisAngles.y, angles.z))

    local rotation = yaw*pitch*roll;

    if (Input.isKeyDown(KB_1)) then
        self.orientation = 0
    end
    if (Input.isKeyDown(KB_2)) then
        self.orientation = 1
    end
    if (Input.isKeyDown(KB_3)) then
        self.orientation = 2
    end
    if (Input.isKeyDown(KB_4)) then
        self.orientation = 3
    end

    local angle = self.orientations[self.orientation + 1];

    self.orient:setAxisAngle(Vector(0,0,-1), math.rad(angle))

    local offset = Quaternion.axisAngle(Vector.right, 90)
    -- Simulates the android data
    self.rotation = self.calib*(offset*self.orient:inverse()*rotation)*self.orient
    
    if inputlayer then
        inputlayer.setSensorRotation(self.rotation)
    end
    
end
