
SegmentManager = {};
SegmentManager.__index = SegmentManager;

function SegmentManager.new(theScene)
	local ob = {};
	
	ob.scene           = theScene;
	ob.activeSegments  = {};
	
	setmetatable(ob, SegmentManager);
	
	local segmentDef   = def[def.firstSegment];
	ob:playSegment(segmentDef);

    Log.message("first scene: " .. segmentDef.name);
	
	Events.listen(ob, "segmentFinished");
	Events.listen(ob, "SegmentTransition");

	return ob;
end


function SegmentManager:playSegment(segmentDef)
	local segment = Segment.new(segmentDef, self.scene);

	segment:startPlaying(); 
	self:addSegment(segment);
	
	self.currentSegment = segment.name
end


function SegmentManager:stopSegment(segmentDef)
    -- TODO create functionality
end


function SegmentManager:changeSegment(from, to)
    -- TODO create functionality
end


function SegmentManager:segmentFinished(finishedSegment)
	local transitions      = finishedSegment.def.transitions;
	local target           = finishedSegment.target;
	
	if (transitions) then
    	for _, element in ipairs(transitions) do
    		if (element.event and element.event == "SegmentFinished") then
    			--Log.message("found transition to " .. element.);

                local targetMatched = true

                if (element.targets) then
                    targetMatched = false
                    for _, targetName in ipairs(element.targets) do
                        if (finishedSegment.target and targetName == finishedSegment.target.name) then
                            targetMatched = true
                        end
                    end
                end
                
                if (targetMatched) then
                    self:removeSegment(finishedSegment);                
                
        			local segment = Segment.new(def[element.goTo], self.scene);
        			segment:startPlaying();
					self.currentSegment = segment.name
        			self:addSegment(segment);
                    
                    -- HACK: handle this correctly
                    finishedSegment.def.transitions = nil
                end
    		end
    	end
    end
	
end

function SegmentManager:SegmentTransition(gotoSegment) 
	local segment = Segment.new(def[gotoSegment], self.scene);
	segment:startPlaying();
	self.currentSegment = segment.name
	self:addSegment(segment);
end


function SegmentManager:addSegment(segment)
	self.activeSegments[segment.name] = segment;
end


function SegmentManager:clearActiveSegments()
    for _, segment in pairs(self.activeSegments) do
        self:removeSegment(segment)
    end
end


function SegmentManager:removeSegment(segment)
    if (not segment) then
        return
    end
--	Log.message ("segment finished " ..segment:getName());
--	Log.message ("active segements " .. table.tostring(self.activeSegments));
--	Log.message ("the segement " .. table.tostring(segment));
--	table.remove(self.activeSegments, segment:getName());
	
	self.activeSegments[segment.name] = nil
end


function SegmentManager:onUpdate()
    for _, segment in pairs(self.activeSegments) do
		segment:onUpdate();
	end
end
