

local Logger = TaggedLog("Util", false)

Util = {}

function Util.getWindowPosition(worldPosition, screenWidth, screenHeight)
	local ndc = mainView:project(worldPosition)
	if(ndc.y > 1.1 or ndc.y < -1.1 or ndc.x > 1.2 or ndc.x < -1.2 or ndc.z < 0 or ndc.z > 1) then
		return nil;
	end
	ndc.x = 0.5*screenWidth*(ndc.x + 1)
	ndc.y = 0.5*screenHeight*(1 - ndc.y)
	
	return ndc;
end

function Util.frustumSlopes(fov, aspect)

	local aratio = aspect or MainCamera.aspect
	if(aratio < 1.0) then
		local xSlope = math.tan(math.rad(fov or MainCamera.fov)*0.5)
		local ySlope = xSlope/aratio;
		return xSlope, ySlope
	else
		local ySlope = math.tan(math.rad(fov or MainCamera.fov)*0.5)
		local xSlope = ySlope*aratio;
		return xSlope, ySlope
	end
end

function Util.toClipSpace(worldPosition, fov)
	local camT 		= MainCamera:getWorld()
	local camLook 	= MainCamera.camForward:copy()
	local camRight 	= MainCamera.camRight:copy()
	local camUp 	= MainCamera.camUp:copy()
	
--	local ySlope = math.tan(math.rad(fov or Global.fov)*0.5)
--	local xSlope = ySlope*Global.aspect;
	local xSlope, ySlope = Util.frustumSlopes(fov);
	
	-- get vector pointing at the boy (from camera)
	local toTarget = (worldPosition - camT.position)
	
	-- project vector to boy against the camera 'look' - provides distance from camera plane
	local depth = (camLook:dot(toTarget))
	depth = math.sign(depth)*math.max(1.0, math.abs(depth))
	
	-- gets the vector that points at the target and is perpendicular to the camera look ray (on the camera plane)
	local toTarget = toTarget - camLook*depth
	
	local x = Vector.dot(toTarget, camRight)
	local y = Vector.dot(toTarget, camUp)

	local screenX = xSlope*depth;
	local screenY = ySlope*depth;
	
	local ndc = Vector(x/screenX, y/screenY, depth/(MainCamera.far - MainCamera.near), depth)
	
	-- DEBUG VISUALIZATION
--	local rightEdge = Vector(math.abs(xDist), 0, math.min(-1.001, -math.abs(depth))):rotateBy(camT.rotation);
--	rightEdge = rightEdge + camT.position
--	
--	local topEdge = Vector(0, math.abs(yDist), math.min(-1.001, -math.abs(depth))):rotateBy(camT.rotation);
--	topEdge = topEdge + camT.position
--	
--  RenderDebug.point(rightEdge, Color(1, 0, 0), math.max(1.0, math.abs(depth)*0.25))
--  RenderDebug.point(topEdge, Color(0, 1, 0), math.max(1.0, math.abs(depth)*0.25))
    
--	local style = { color = Color.new(1.0, 0.0, 1.0), font = "common:fonts/arial_default_14.fnt", scale = 2.0 }
--	Renderer.drawText(((ndc.x)*0.5 + 0.5)*Global.curScreenWidth, (-(ndc.y)*0.5 + 0.5)*Global.curScreenHeight, style, string.format("WINDOW_POS"))
	
	return ndc
end

-- Perform a shallow copy of the given table
Util.copy = function(t)
    if (t) then
        local newT = {}
        for key, value in pairs(t) do
            newT[key] = value
        end
        return newT
    end
    return nil
end

-- Randomly shuffles the given table in place
Util.shuffle = function(tab)
    local n = #tab
    while n >= 2 do
        local i = math.random(n)
        tab[n], tab[i] = tab[i], tab[n]
        n = n - 1
    end
end

Util.hash = function(str)
	local hash = 2166136261
	for i = 1, #str do
		local c = str:sub(i,i)
		hash = bit.bxor(hash, string.byte(c, 1)) * 16777619
	end
	return hash
end

Util.numKeys = function(tbl)
	local numItems = 0
	for k,v in pairs(tbl) do
	    numItems = numItems + 1
	end
	return numItems
end

Util.getKeyIdx = function(tbl, idx)
	for k,v in pairs(tbl) do
		if (idx == 1) then return v end
		if (idx < 1) then break end
		idx = idx - 1
	end
	return nil
end

math.clamp = function(value, min, max)
	if (value < min) then
		value = min
	elseif (value > max) then
        value = max
	end
	return value
end

math.clamp01 = function(value)
    return math.clamp(value, 0, 1)
end

math.sign = function(value)
	if( value < 0.0) then
		return -1.0
	end
	
	return 1.0
end

math.easeOutExpo = function (t) 
	return -2^(-10 * t) + 1
end

math.easeInOutSine = function(t) 
	return -0.5 * (math.cos(math.pi*t) - 1)
end

math.easeInOutQuart = function (t) 
	t = t / 0.5
	if (t < 1) then
		return 1/2*t*t*t*t
	end
	t = t - 2
	return -0.5 * (t*t*t*t - 2)
end

math.easeInOutQuad = function ( t,  start,  scale,  period)
	start = start or 0
	scale = scale or 1
	period = period or 1
	t = t/(period*0.5);
	if (t < 1) then return scale*(0.5*t*t) + start; end
	t = t - 1;
	return -scale*(0.5 * (t*(t-2) - 1)) + start;
end

math.easeOutQuad = function (t) 

	return -1 * t *(t-2)
end

math.easeInQuad = function(t, start, scale, period)
	t = t/period
	return scale*t*t + start
end

math.easeInQuart = function (t, start, scale, period) 
	t = t/period
	return scale*t*t*t*t + start
end

math.lerpEaseIn = function(from, to, value)

	local t = math.clamp01(value)
	local _value = t*t*t*2.5
	return from + (to - from) * math.clamp01(_value)
end


math.inverseLerp = function(from, to, value)
    if (from == to) then return 0 end
    value = math.clamp(value, from, to)
    return (value - from) / (to - from)
end

math.minArr = function( arr )
	local min = nil
	if(type(arr) == "table") then
		for i, value in ipairs(arr) do
			if(value < min or min == nil) then
				min = value
			end
		end
	end
	return min
end

math.maxArr = function( arr )
	local max = nil
	if(type(arr) == "table") then
		for i, value in ipairs(arr) do
			if(value > max or max == nil) then
				max = value
			end
		end
	end
	return max
end

math.floatMax = 5.6e300
math.floatMin = 5.6e-300

math.spline = function(sampleTime, knots, knotCount)
	if(knotCount >= 4) then
		-- find the appropriate 4-point span of the spline
		knotCount				= knotCount - 3
		sampleTime				= math.clamp(sampleTime, .0, 1.0) * knotCount
	
		local s					= math.floor(sampleTime)
		if(s >= knotCount)then
			s = knotCount - 1
		end
	
		sampleTime		   	= sampleTime - s
		local index			= s
	
		local k0			= knots[index+1]
		local k1			= knots[index+2]
		local k2			= knots[index+3]
		local k3			= knots[index+4]
	
		-- evaluate the span cubic at sampleTime using Horner's rule
		local c3			= -0.5 * k0 +  1.5 * k1 + -1.5 * k2 +  0.5 * k3
		local c2			=  1.0 * k0 + -2.5 * k1 +  2.0 * k2 + -0.5 * k3
		local c1			= -0.5 * k0 + 0.5 * k2
		local c0			= k1
	
		return ((c3 * sampleTime + c2) * sampleTime + c1) * sampleTime + c0
	end
	
	-- if you did it wrong you will blow up
	return nil
end

-- not ready for fixemups yet
Quaternion.rotatePoint = function(self, point)
	local num = self.x * 2.0
	local num2 = self.y * 2.0
	local num3 = self.z * 2.0
	local num4 = self.x * num
	local num5 = self.y * num2
	local num6 = self.z * num3
	local num7 = self.x * num2
	local num8 = self.x * num3
	local num9 = self.y * num3
	local num10 = self.w * num
	local num11 = self.w * num2
	local num12 = self.w * num3
	local result = Vector(0,0,0,1)
	
	result.x = (1 - (num5 + num6)) * point.x + (num7 - num12) * point.y + (num8 + num11) * point.z
	result.y = (num7 + num12) * point.x + (1 - (num4 + num6)) * point.y + (num9 - num10) * point.z
	result.z = (num8 - num11) * point.x + (num9 + num10) * point.y + (1 - (num4 + num5)) * point.z
	return result
end
table.insert_sort = function(list, item, compareFunc)
	-- insertion sort
	local swap = nil
	local compareProbe = nil
	
	table.insert(list, item)
	
	local count = table.getn(list)
	
	local test = count
	
	for i = count - 1, 1, -1 do
		compareProbe = list[i]
		
		if(compareFunc(item, compareProbe)) then
			
			-- save old
			swap = list[i]
			
			-- replace with new
			list[i] = list[test]
			
			-- put old in new's previous slot
			list[test] = swap
			
			-- update test index
			test = i
			
		else
			i = 0
		end
	end
	
end

table.find_index = function(list, search_delegate)
	if(list and type(list) == "table") then
		for i, value in ipairs(list) do
			if search_delegate(value) then
				return i
			end
		end
	end
	
	return nil
end

function table.length(t, excludeNil)
  local count = 0
  for _, value in pairs(t) do
  	if(not excludeNil or value ~= nil) then
  		count = count + 1 
  	end
  end
  return count
end

-- Given a table of values that may contain embedded tables of values,
-- this function will return a flattened list of all the values from all
-- the nested lists
table.flatten = function(tbl, flattened)
    local flattened = flattened or {}
    if tbl then
        for _, value in ipairs(tbl) do
            if type(value) == "table" then
                flattened = table.flatten(value, flattened)
            else
                table.insert(flattened, value)
            end
        end
    end
    return flattened
end

-- Given an array table, create a hash table with the keys being the set
-- of values from the original table.  Useful for checking if an entry is
-- in the array
table.create_set = function(tbl)
    if tbl ~= nil then
        local set = {}
        for _, value in ipairs(tbl) do
            set[value] = true
        end
        return set
    end
    return nil
end

string.startsWith = function(s, prefix)
    return string.sub(s, 1, string.len(prefix)) == prefix
end

string.endsWith = function(s, suffix)
    return suffix == '' or string.sub(s, -string.len(suffix)) == suffix
end

-- split a string at matches to the given pattern regular expression,
-- returning a string array table of the results.  If no pattern
-- is provided, it will just split at spaces
string.split = function(self, seperatorPattern)
    seperatorPattern = seperatorPattern or '%s+'
    local result = {}
    local strStart = 0
    for sepStart, seperator in self:gmatch("()(" .. seperatorPattern .. ")") do
        table.insert(result, self:sub(strStart, sepStart - 1))
        strStart = sepStart + #seperator
    end
    table.insert(result, self:sub(strStart))
    return result
end

-- recursively compare two values and return true if they are equal
function deepcompare(t1,t2,ignore_mt)
    local ty1 = type(t1)
    local ty2 = type(t2)
    if ty1 ~= ty2 then return false end
    -- non-table types can be directly compared
    if ty1 ~= 'table' and ty2 ~= 'table' then return t1 == t2 end
    -- as well as tables which have the metamethod __eq
    local mt = getmetatable(t1)
    if not ignore_mt and mt and mt.__eq then return t1 == t2 end
    for k1,v1 in pairs(t1) do
        local v2 = t2[k1]
        if v2 == nil or not deepcompare(v1,v2) then return false end
    end
    for k2,v2 in pairs(t2) do
        local v1 = t1[k2]
        if v1 == nil or not deepcompare(v1,v2) then return false end
    end
    return true
end



-- if the given object is a valid handle, it returns the object.
-- if the given object is nil or invalid, returns nil.
-- this can be used to determine the status of a handle before
-- attempting to use it.
status = function(ob)
    if type(ob) == "table" and ob.__object then
        ob = __status(ob)
    end
    return ob
end

-- Safely destroy a moxie object if it is still valid
function destroySafely(obj)
    if obj and status(obj) then
        obj:destroy()
    end
end