
--local Logger = TaggedLog("UpdateTester", true)

local clearcolor			= Color.new( 0.0, 0.0, 0.0, 1.0 )

Story = {}
Story.__index = Story

function Story.onInitialize()
	Font.prefetch( "common:fonts/arial_default_24.fnt" )
	local ob = {};
	ob.scene = Story.createScene()
	ob.main_view 	= createMainView()
	ob.segmentManager = SegmentManager.new(scene);
	assert(ob.main_view);
	ob.main_view:setCamera( ob.scene.cameras.main )
	ob.main_view:setClearColor(clearcolor);
    
    --- just here for test, probabaly should be moved into a segment or something
    ob.mainLight = scene:createLight("MainLight", def.Light_Sun)
    
    local sunRotation = Quaternion.new()
	sunRotation:setAxisAngle( Vector.new( 1.0, 0.0, 0.0 ), math.pi*-0.4 ) 
	ob.mainLight:setLocal({ rotation = sunRotation })
    --ob.mainLight:setDiffuse( Vector.new( 0.0, 0.0, 1.0 ))

	return setmetatable(ob, Story);
end

function Story:onTerminate()
	if self.scene then
		self.scene:destroy()
	end

	if self.main_view then
		self.main_view:destroy()
	end
end

function Story:onUpdate()
	if self.scene then
		self.scene:update()
	end
	if self.segmentManager then
		self.segmentManager:onUpdate();
	end
end


function Story:onRender()

	self.scene:render( self.main_view )

	Renderer.display( self.main_view )
end


function Story.createScene( )
	scene = Scene.create()

	local input		= MouseController
	local fov		= 60.0

	if Platform.android or Platform.iOS then
		input		= SensorController
		fov			= 40.0
	end

	camera		= scene:createCamera( "main", persp( fov, 0.01, 1000.0 ) )
    local cameraPosition = Vector.new(0, 2, 0);
    camera:setLocal({ position = cameraPosition })

	controller 	= camera:addComponent( "controller", input )
	

	return scene;
end