
-- EPlayState
enum("PS_STOPPED", 0)
enum("PS_PAUSED")
enum("PS_PLAYING")


-- Make the sound API a little easier to use
Sound.stop = Sound.stop or function(self)
    self:setPlayState(PS_STOPPED)
end

Sound.pause = Sound.pause or function(self)
    self:setPlayState(PS_PAUSED)
end

Sound.play = Sound.play or function(self)
    self:setPlayState(PS_PLAYING)
end

Sound.isPlaying = Sound.isPlaying or function(self)
    return (self:getPlayState() == PS_PLAYING)
end

Sound.getDurationSeconds = Sound.getDurationSeconds or function(self)
    return self:getDuration() / 1000
end

Sound.getSeekPositionSeconds = Sound.getSeekPositionSeconds or function(self)
    return self:getSeekPosition() / 1000
end

-- camera: 			camera object
-- position: 		sound position: Vector3
-- headOcclusion:	how much to dampen volume when audio is behind listener
Sound.getPanVolume3D = function( self, camera, position, headOcclusion )
    local middleVolume = 1
    local extentVolume = 0.3
    local cameraWorld 	= camera:getWorld()
    local cameraForward = cameraWorld:forward()

    local cameraParent = camera.parent or camera

    local targetForward = position - cameraParent:getWorld().position
    targetForward.w 	= 0.0
    targetForward:normalize()

    -- Camera right without roll
    local cameraRight 	= Vector.cross( cameraForward, Vector.up )
    cameraRight.w		= 0.0
    cameraRight:normalize()

    local forwardDot 	= Vector.dot( targetForward, cameraForward )
    local rightDot 		= Vector.dot( targetForward, cameraRight )

    local pan = (1 - math.abs(forwardDot)) * math.sign(rightDot)
    local volume = 1

    local middle = middleVolume
    local extent = extentVolume
    if forwardDot < 0 then
        -- dampen volume if position is behind head
        middle = headOcclusion
    end
    volume = volume * math.lerp(middle, extent, math.abs(pan))

    local compressedPan = math.clamp(pan * 1.42, -1, 1)

    return compressedPan, volume
end
