
enum("LAYER0", 0);
enum("LAYER1", 1);
enum("LAYER2", 2);
enum("LAYER3", 3);

-- the default moxie GUI theme
enum("ALIGN_DEFAULT", 0); -- not supported yet
enum("ALIGN_TOP_LEFT");
enum("ALIGN_TOP_CENTER");
enum("ALIGN_TOP_RIGHT");
enum("ALIGN_CENTER_LEFT");
enum("ALIGN_CENTER");
enum("ALIGN_CENTER_RIGHT");
enum("ALIGN_BOTTOM_LEFT");
enum("ALIGN_BOTTOM_CENTER");
enum("ALIGN_BOTTOM_RIGHT");
enum("ALIGN_MAX");

enum("DOCK_NONE", 0);
enum("DOCK_LEFT");
enum("DOCK_TOP");
enum("DOCK_RIGHT");
enum("DOCK_BOTTOM");
enum("DOCK_FILL");

enum("ABSOLUTE", 0);
enum("RELATIVE");
enum("WRAP_CONTENT");
enum("FILL_PARENT");


-- only the most common cases are provided here.
ANCHOR_TOP = { false, true, false, false }
ANCHOR_TOP_LEFT = { true, true, false, false }
ANCHOR_BOTTOM = { false, false, false, true }
ANCHOR_BOTTOM_RIGHT = { false, false, true, true }
ANCHOR_LEFT = { true, false, false, false }
ANCHOR_RIGHT = { false, true, false, false }
ANCHOR_LEFT_RIGHT = { true, false, true, false }
ANCHOR_TOP_BOTTOM = { false, true, false, true }
ANCHOR_ALL = { true, true, true, true }
ANCHOR_NONE = { false, false, false, false }
local __anchor = GUIFrame.anchor;
GUIFrame.anchor = function(self, mode, ...)
	if(type(mode) == 'table') then
		__anchor(self, mode[1], mode[2], mode[3], mode[4]);
	else
		__anchor(self, mode, ...);
	end
end

local __destroy = GUIFrame.destroy
GUIFrame.destroy = function(self)
	local destroyHandler = {
		onPassiveUpdate = function(self)
			if(self.counter > 0) then
				self.counter = self.counter - 1;
				return;
			end
			__destroy(self);
		end
	}
	self:hide();
	-- wait for the next frame to destroy.
	self.counter = 1;
	self:addEventHandler(destroyHandler);
end


local __frame = GUI.frame
GUI.frame = function(parent, pos, size, style)
	assert(pos);
	assert(size);
	assert(style);

	if(parent and parent.proxy) then
		parent = parent.proxy;
	end

	return __frame(parent, pos, size, style);
end

local __label = GUI.label
GUI.label = function(parent, pos, size, caption, style)
	assert(pos);
	assert(size);
	assert(caption);
	assert(style);

	if(parent.proxy) then
		parent = parent.proxy;
	end

	return __label(parent, pos, size, caption, style);
end

local __text = GUI.text
GUI.text = function(parent, pos, size, text, style)
	assert(pos);
	assert(size);
	assert(text);
	assert(style);

	if(parent.proxy) then
		parent = parent.proxy;
	end

	return __text(parent, pos, size, text, style);
end

local __list = GUI.list
GUI.list = function(parent, pos, size, style, orient)
	assert(pos);
	assert(size);
	assert(style);

	if(parent.proxy) then
		parent = parent.proxy;
	end

	return __list(parent, pos, size, style, orient);
end

GUIFrame.setProxy = function(self, frame)
	self.proxy = frame;
end

-- extend GUIFrame
-- Frames can dispatch events to anyone who wants to listen for them.
GUIFrame.addListener = function(self, listener)
	self.listeners = self.listeners or {};
	table.insert(self.listeners, listener);
end

GUIFrame.removeListener = function(self, listener)
	table.remove(self.listeners, listener);
end

GUIFrame.dispatch = function(self, func)
	if(self.listeners~=nil) then
		for _, receiver in ipairs(self.listeners) do
			-- call the function that matches the event
			if(type(receiver[func])=="function") then
				receiver[func](receiver, self);
			elseif(type(receiver.onEvent)=="function") then
				receiver:onEvent(func, self);
			end
		end
	end
end

local __px = function(x, y, mode, align)
	local out = Vector.new(x, y, 0, 0)

    mode = mode or ABSOLUTE
    if type(mode) == "table" then
		out.hmode = mode[1]
		out.vmode = mode[2]
	else
		out.mode = mode
		out.vmode = mode
		out.hmode = mode
	end
    out.align = align

	return out
end

nx = function(x, y)
	return __px(x, y, RELATIVE);
end

-- pixel units
px = function(x, y)
	return __px(x, y, ABSOLUTE);
end

-- positioning
GUIRelative = function(x, y)
	return __px(x, y, RELATIVE);
end

GUIPos = function(x, y)
	return __px(x, y, ABSOLUTE);
end

GUIAligned = function(align)
	return __px(0, 0, ABSOLUTE, align);
end

-- sizing
SZ_WRAP = "wrap";
SZ_FILL = "fill";
GUISize = function(x, y)
	local hmode = nil;
	local vmode = nil;
	if(type(x)=='string') then
		if(x == SZ_FILL) then
			x = 0;
			hmode = FILL_PARENT;
		end
		if(x == SZ_WRAP) then
			x = 0;
			hmode = WRAP_CONTENT;
		end
	end
	if(type(y)=='string') then
		if(y == SZ_FILL) then
			y = 0;
			vmode = FILL_PARENT;
		end
		if(y == SZ_WRAP) then
			y = 0;
			vmode = WRAP_CONTENT;
		end
	end
	return __px(x, y, { hmode, vmode });
end

GUIAutoSize = function(hmode, vmode)
    hmode = hmode or WRAP_CONTENT
    vmode = vmode or WRAP_CONTENT
	return __px(0,0, {hmode, vmode} )
end

GUI.tint = function(theme, addcolor, mulcolor)
	local traverse;
	traverse = function(theme, func)
		for k, v in pairs(theme) do
			if(type(v) == "table") then
				if(v.__isstyle) then
					func(v);
				else
					traverse(v, func);
				end
			end
		end
	end

	local tint = function(color, add, mul)
		color.r = (color.r+add.r)*mul.r;
		color.g = (color.g+add.g)*mul.g;
		color.b = (color.b+add.b)*mul.b;
		color.a = (color.a+add.a)*mul.a;
	end

	local tintfunc = function(style)
		if(style.image ~= nil) then
			if(style.image.color==nil) then
				style.image.color=Color(1,1,1,1);
			end
			tint(style.image.color, addcolor, mulcolor);
		end
--		if(style.textcolor ~= nil) then
--			tint(style.textcolor, addcolor, mulcolor);
--		end
	end

	traverse(theme, tintfunc);
end

GUI.loadTheme = function(name, theme)
	if name == nil then
	    name = "blackbox"
		theme = blackbox
	end

	GUI.addTheme(name, theme);
	GUI.setTheme(name);

	GUI.theme = theme;
end

local hoveredHandler = function(name)
	return {
		onMouseEnter = function(self)
			self:setStyle(name .. ".hovered")
			self.hovered = true
			return false
		end,
		onMouseLeave = function(self)
			self:setStyle(name .. ".default")
			self.hovered = false
			return false
		end,
	}
end

GUI.callback = function(obj, callbackName, ...)
    if obj[callbackName] then
        obj[callbackName](obj, ...)
    end
end

-- Generic Button Handler
-- Callbacks: onButtonDown, onButtonUp, onButtonPressed
GUI.defaultButtonHandler = {
    onMouseEnter = function(self)
        if self.disabled then return end
        self.hovered = true
        GUI.callback(self, "selectStyle")
        return true
    end,

    onMouseLeave = function(self)
        if (self.disabled) then return end
        self.hovered = false
        self.pressed = false
        GUI.callback(self, "selectStyle")
        return true
    end,

    onMouseDown = function(self)
        if self.disabled then return end
        self.pressed = true
        GUI.callback(self, "selectStyle")
        GUI.callback(self, "onButtonDown")
        return true
    end,

    onMouseUp = function(self)
        if self.disabled then return end
        self.pressed = false
        GUI.callback(self, "selectStyle")
        GUI.callback(self, "onButtonUp")
        return true
    end,

    onUpdate = function(self)
        if self.disabled then return end
        GUI.callback(self, "selectStyle")
        GUI.callback(self, "onButtonPressed") -- TODO: rename this
        return true
    end,
}

dragHandler =
{
	onMouseDown = function(self)
		if(self.nodrag) then return false end;
		self.drag = true;
		self.dragoffset = GUI.getCursorPosition() - self:getWorldPosition();
		return false;
	end,

	onMouseUp = function(self)
		self.drag = false;
		return false;
	end,

	onUpdate = function(self)
		if(self.nodrag) then return end;
		if(self.drag) then
			local newpos = GUI.getCursorPosition() - self.dragoffset;
			self:moveTo(newpos.x, newpos.y);
		end
		return false;
	end
}

local resizeGripSize = 4;
resizeHandler = {
	onMouseDown = function(self)
		--self.__resizeable = self.__resizeable or true;
		if(self.noresize) then return false end;

		local mpos = GUI.getCursorPosition();
		local size = self:getSize();
		local pos = self:getWorldPosition();

		self.resizeLeft = false;
		self.resizeTop = false;
		self.resizeRight = false;
		self.resizeBottom = false;

		if(mpos.x < (pos.x + resizeGripSize)) then
			self.resizeLeft = true;
		end

		if(mpos.y < (pos.y + resizeGripSize)) then
			self.resizeTop = true;
		end

		if(mpos.x > (pos.x + size.x)-resizeGripSize) then
			self.resizeRight = true;
		end

		if(mpos.y > (pos.y + size.y)-resizeGripSize) then
			self.resizeBottom = true;
		end

		return false;
	end,

	onMouseUp = function(self)
		self.resizeLeft = false;
		self.resizeTop = false;
		self.resizeRight = false;
		self.resizeBottom = false;
		return false;
	end,

	onUpdate = function(self)
		if(self.noresize) then return false end;

		local delta = GUI.getCursorDelta();

		if(self.resizeLeft) then
			self.drag = false;
			self:move(delta.x, 0);
			self:resizeBy(-delta.x, 0);
		end

		if(self.resizeTop) then
			self.drag = false;
			self:move(0, delta.y);
			self:resizeBy(0, -delta.y);
		end

		if(self.resizeRight) then
			self.drag = false;
			self:resizeBy(delta.x, 0);
		end

		if(self.resizeBottom) then
			self.drag = false;
			self:resizeBy(0, delta.y);
		end

		return false;
	end,
};

GUIFrame.draggable = function(self)
	self:addEventHandler(dragHandler);
end

GUIFrame.resizeable = function(self)
	self:addEventHandler(resizeHandler);
end

local animHandler = {
	onPassiveUpdate = function(self)
		local currTimeSecs = os.clock();
		if(self.lastUpdateSecs == nil) then
			self.lastUpdateSecs = currTimeSecs;
		end

		local dt = currTimeSecs - self.lastUpdateSecs;
		self.lastUpdateSecs = currTimeSecs;

		local currPos = self:getWorldPosition();
		if(self.invduration == nil) then
			self.invduration = 0;
		end
		if(self.startPos == nil) then
			self.startPos = currPos;
		end
		if(self.goalPos == nil) then
			self.goalPos = currPos;
		end

		local diff = self.goalPos - currPos;
		local difflen = diff:length();
		if(difflen > 0) then
			local delta = (self.goalPos - self.startPos);
			if(self.invduration > 0) then
				delta = delta*dt*self.invduration;
			end
			if(delta:length() > difflen) then
				delta = diff;
				self.invduration = 0;
				if(self.onDone) then
					self:onDone();
					self.onDone = nil;
				end
			end
			self:move(delta.x, delta.y);
		end
	end
}

-- need to move this into native updates.
GUIFrame.animatable = function(self)
	self:addEventHandler(animHandler);

	self.slideOut = function(self, duration, dir)
		if(duration == nil) then
			duration = 0.25;
		end

		local vp = GUI.getViewport();
		local size = self:getSize();
		local pos = self:getLocalPosition();

		-- find the shortest path to being off screen.
		local dleft = -(pos.x + size.x);
		local dright = (vp.width - pos.x);
		local dup = -(pos.y + size.y);
		local ddown = (vp.height - pos.y);
		local xmin = math.min(dleft*dleft, dright*dright);
		local ymin = math.min(dup*dup, ddown*ddown);

		self.slidePos = pos;

		local dest = pos;
		if(dir == nil) then
			-- shortest distance to travel offscreen.
			if( xmin < ymin ) then
				if(dleft*dleft < dright*dright) then
					dest.x = dleft;
				else
					dest.x = dright;
				end
			else
				if(dup*dup < ddown*ddown) then
					dest.y = dup;
				else
					dest.y = ddown;
				end
			end
		else
			-- user provided direction
			if(dir == "left") then
				dest.x = dest.x + dleft;
			end
			if(dir == "right") then
				dest.x = dest.x + dright;
			end
			if(dir == "up") then
				dest.y = dest.y + dup;
			end
			if(dir == "down") then
				dest.y = dest.y + ddown;
			end
		end

		self:animMove(dest.x, dest.y, duration );
	end

	self.slideIn = function(self, duration, show)
		self:show();

		if(duration == nil) then
			duration = 0.25;
		end

		if(self.slidePos ~= nil) then
			self:animMove(self.slidePos.x, self.slidePos.y, duration);
			self.slidePos = nil;
		end
	end

	self.animMove = function(self, x, y, duration, onDone)
		self.startPos = self:getWorldPosition();
		self.goalPos = self.startPos + Vector(x, y);
		self.onDone = onDone;
		if(duration==0) then
			self.invduration = 0.0;
		else
			self.invduration = 1.0/duration;
		end
	end
end

GUI.widget = function(pos, size)
	local widget = GUI.frame(nil, pos, size, "widget.default");
	local clientRect = widget:getClientLocalRect();
	size.x = clientRect.width;
	size.y = clientRect.height;

	local content = GUI.frame(widget, px(0,0), size, "");
	content:anchor(true, true, true, true);

	widget:addEventHandler(hoveredHandler("widget"));
	widget:addEventHandler(dragHandler);

	return content;
end

-- media buttons
GUI.playbutton = function(parent, pos, size, isplaying)
	isplaying = isplaying or false;
	local playButtonHandler = {
		onMouseEnter = function(self)
			self.hovered = true;
			if(self.pressed == nil or self.pressed == false) then
				if(self.isPlaying == false) then
					self:setStyle("controlWidget.play.hovered");
				else
					self:setStyle("controlWidget.pause.hovered");
				end
			end
			return true;
		end,
		onMouseLeave = function(self)
			self.hovered = false;
			self.pressed = false;
			if(self.isPlaying == false) then
				self:setStyle("controlWidget.play.default");
			else
				self:setStyle("controlWidget.pause.default");
			end
			return true;
		end,
		onMouseDown = function(self)
			self.pressed = true;
			if(self.isPlaying == false) then
				self:setStyle("controlWidget.play.pressed");
			else
				self:setStyle("controlWidget.pause.pressed");
			end
			return true;
		end,
		onMouseUp = function(self)
			self.pressed = false;
			-- toggle is playing
			if(self.isPlaying == true) then
				self.isPlaying = false;
				if(self.onPause ~= nil) then
					self:onPause();
				end
			else
				if(self.onPlay ~= nil) then
					self:onPlay();
				end
				self.isPlaying = true;
			end

			if(self.hovered == true) then
				if(self.isPlaying == false) then
					self:setStyle("controlWidget.play.hovered");
				else
					self:setStyle("controlWidget.pause.hovered");
				end
			else
				if(self.isPlaying == false) then
					self:setStyle("controlWidget.play.default");
				else
					self:setStyle("controlWidget.pause.default");
				end
			end
			return true;
		end,
	};

	local initialstyle = "controlWidget.play.default";
	if(isplaying) then
		initialstyle = "controlWidget.pause.default";
	end

	if(size == DEFAULT) then
		size = GUI.theme.controlWidget.play.default.image.size;
	end

	local out = GUI.frame(parent, pos, size, initialstyle);
	out.isPlaying = isplaying;
	out:addEventHandler(playButtonHandler);

	return out;
end

GUI.controlWidget = function(pos, size, caption, isplaying)
	if(size == DEFAULT) then
		size = px(224,155);
	end

	local widget = GUI.widget(pos, size);

	local textSize = GUI.textSize(caption, "controlWidget.text");
	local label = GUI.label(widget, px(0, 0), px(size.x, textSize.y), caption, "controlWidget.text");
	label:setTextAlign(ALIGN_TOP_CENTER);
	label:dock(DOCK_TOP);

	local smButtonSize = GUI.theme.controlWidget.jumpbck.default.image.size;
	local lgButtonSize = GUI.theme.controlWidget.play.default.image.size;
	local lgButtonPos = px(size.x/2 - lgButtonSize.x/2, 0);
	local padding = 8;
	local smButtonPosL = px(lgButtonPos.x - smButtonSize.x - padding, (lgButtonSize.y - smButtonSize.y)*0.5);
	local smButtonPosR = px(lgButtonPos.x + lgButtonSize.x + padding, (lgButtonSize.y - smButtonSize.y)*0.5);

	local frame = GUI.frame(widget, px(0,0), nx(1,1), "");
	local jumpbck = __button(frame, smButtonPosL, smButtonSize, "", "controlWidget.jumpbck");
	local jumpfwd = __button(frame, smButtonPosR, smButtonSize, "", "controlWidget.jumpfwd");
	local play = GUI.playbutton(frame, px(size.x/2 - lgButtonSize.x/2, 0), lgButtonSize, isplaying);
	frame:dock(DOCK_FILL);

	jumpbck.onButtonDown = function(self)
		if(widget.onStepBackward) then
			widget:onStepBackward();
		end
	end

	jumpfwd.onButtonDown = function(self)
		if(widget.onStepForward) then
			widget:onStepForward();
		end
	end

	play.onPlay = function(self)
		if(widget.onPlay) then
			widget:onPlay();
		end
	end

	play.onPause = function(self)
		if(widget.onPause) then
			widget:onPause();
		end
	end

	widget.setText = function(self, str)
		label:setText(str);
	end

	return widget;
end
