
GUI.popupmenu = function(parent, pos, size, items)
    local numItems = table.getn(items)
    local itemHeight = GUI.theme.popupmenu.item.hovered.image.size.y
    local padding = 8
    local popup = GUI.frame(nil, pos, GUISize(SZ_FILL, (numItems*itemHeight)+padding), "popupmenu.frame")
    popup.items = {}
    popup.hovered = 0
    popup.selected = 0

    for index, name in ipairs(items) do
        local item = GUI.label(popup, GUISize(0, (index-1)*itemHeight), GUISize(SZ_FILL, itemHeight), name, "popupmenu.item.default")
        item:anchor(true, false, true, false)
        popup.items[index] = item
    end

    local trackingHandler = {
        onPassiveUpdate = function(self)
            local cursor = GUI.getCursorPosition()
            local hovered = 0
            for index, item in ipairs(popup.items) do
                if(index == popup.selected) then
                    item:setStyle("popupmenu.item.selected")
                elseif(item:containsPoint(cursor.x, cursor.y)) then
                    item:setStyle("popupmenu.item.hovered")
                    hovered = index
                else
                    item:setStyle("popupmenu.item.default")
                end
            end
            popup.hovered = hovered
        end
    }

    popup:addEventHandler(trackingHandler)
    return popup
end

GUI.dropdownmenu = function(parent, pos, width, items, default)
    local ddbutton = GUI.theme.dropdownmenu.button.default.image

    local dropdownmenu = __button(parent, pos, GUISize(SZ_FILL, SZ_WRAP), "", "dropdownmenu.button")
    dropdownmenu:anchor(true, true, true, false)
    local label = GUI.label(dropdownmenu, GUIPos(0, 0), GUISize(SZ_FILL, SZ_FILL), "", "popupmenu.item.default")
    label:anchor(true, false, true, false)
    local popup = GUI.popupmenu(parent, GUIPos(0,0), GUISize(SZ_FILL, SZ_WRAP), items)
    popup:hide()

    if(default == nil) then
        default = 1
    end

    dropdownmenu.addItem = function(name)
        table.insert(self.items, name)
    end

    dropdownmenu.removeItem = function(self, arg)
        if(type(arg) == "string") then
            for index, item in ipairs(self.items) do
                if(item == arg) then
                    table.remove(self.items, index)
                    return
                end
            end
        elseif(type(arg) == "number") then
            table.remove(self.items, arg)
        end
    end

    dropdownmenu.selectItem = function(self, arg)
        if(type(arg) == "string") then
            for index, item in ipairs(self.items) do
                if(item == arg) then
                    arg = index
                    break
                end
            end
        elseif(type(arg) == "number") then
        else
            assert(0) -- unsupported argument.
        end
        if(items[arg]~=nil) then
            self.selected = arg
            if(self.onItemSelected) then
                self:onItemSelected(arg)
            end
        end
    end

    popup.selected = default
    label:setText(items[default])

    dropdownmenu.onButtonDown = function(self)
        popup.hovered = 0

        local dropdownsize = self:getSize()
        local popupsize = popup:getSize()
        local ddpos = self:getWorldPosition()
        ddpos.y = ddpos.y + dropdownsize.y - 1

        popup:moveTo(ddpos.x+2, ddpos.y)
        popup:resize(dropdownsize.x, popupsize.y)
        popup:bringToFront()

        popup:show()
    end

    dropdownmenu.onButtonUp = function(self)
        popup:hide()
        if(popup.hovered > 0 and popup.selected ~= popup.hovered) then
            if(popup.selected > 0) then
                if(self.onItemDeselected) then
                    self:onItemDeselected(items[popup.selected])
                end
            end

            popup.selected = popup.hovered
            popup.hovered = 0


            label:setText(items[popup.selected])
            if(self.onItemSelected) then
                self:onItemSelected(items[popup.selected])
            end
        end
    end

    return dropdownmenu
end
