
GUI.propertygrid = function(parent, pos, size)
    local pgrid = GUI.list(parent, GUIPos(0, 0), GUISize(SZ_FILL, 32), "", false)
    pgrid:anchor(true, false, true, false)

    local indent = 16

    pgrid.__ispgrid = true
    pgrid.items = {}
    pgrid.indent = 0
    pgrid.padding = 1.0
    pgrid.colwidth = pgrid:getWidth()*0.45

    pgrid:setPadding(pgrid.padding)

    pgrid.collapse = function(self, recursively)
        for _, item in pairs(self.items) do
            if(item.sublist) then
                item.sublist.button:minimize()
                item.sublist:collapse(recursively)
            end
        end
    end

    pgrid.expand = function(self, recursively)
        for _, item in pairs(self.items) do
            if(item.sublist) then
                item.sublist.button:maximize()
                item.sublist:expand(recursively)
            end
        end
    end

    pgrid.__newproperty = function(parent, name, value)
        local property = GUI.frame(parent, GUIPos(0,0),  GUISize(SZ_FILL, 24), "")
        property:anchor(true, false, true, false)

        local label1 = GUI.label(property, GUIPos(0, 0), GUISize(pgrid.colwidth, SZ_FILL), name, "propertygrid.key")
        label1:anchor(true, false, false, false)
        label1:setTextAlign(ALIGN_CENTER_LEFT)

        local label2 = GUI.label(property, GUIPos(label1:getWidth()+pgrid.padding, 0.0), GUISize(property:getWidth()-pgrid.padding, SZ_FILL), value, "propertygrid.key")
        label2:anchor(true, false, true, false)
        label2:setTextAlign(ALIGN_CENTER_LEFT)

        property.setValue = function(self, value)
            label2:setText(value)
        end

        return property
    end

    pgrid.__newgroup = function(parent, name, expanded)
        local item = GUI.frame(parent, GUIPos(0,0),  GUISize(SZ_FILL, 32), "propertygrid.groupframe")
        item:anchor(true, false, true, false)

        local minimizeButton = GUI.minimizer(item, GUIPos(parent.indent, 0), true)

        local label1 = GUI.label(item, GUIPos(parent.indent + minimizeButton:getWidth(), 0), GUISize(SZ_FILL, 32), name, "propertygrid.group")
        label1:anchor(true, false, true, false)
        label1:setTextAlign(ALIGN_CENTER_LEFT)

        -- create item sublist
        item.sublist = GUI.list(parent, GUIPos(0,0), GUISize(SZ_FILL, 32), "", false)
        item.sublist:anchor(true, false, true, false)
        item.sublist.items = {}
        item.sublist:setPadding(pgrid.padding)
        item.sublist.__ispgrid = true
        item.sublist.indent = parent.indent + indent
        item.sublist.button = minimizeButton

        item.sublist.__destroy = function(self)
            parent.items[name] = nil
            item.sublist:destroy()
            item:destroy()
        end

        item.sublist.collapse = function(self, recursively)
            if(recursively) then
                for _, item in pairs(self.items) do
                    if(item.sublist) then
                        item.sublist.button:minimize()
                        item.sublist:collapse(recursively)
                    end
                end
            end
            self:hide()
        end

        item.sublist.expand = function(self, recursively)
            if(recursively) then
                for _, item in pairs(self.items) do
                    if(item.sublist) then
                        item.sublist.button:maximize()
                        item.sublist:expand(recursively)
                    end
                end
            end
            self:show()
        end

        function minimizeButton:onMinimize()
            local recursively = false
            if(Input.isModifierDown(KB_LCTRL) or Input.isModifierDown(KB_RCTRL)) then
                recursively = true
            end
            item.sublist:collapse(recursively)
        end

        function minimizeButton:onMaximize()
            local recursively = false
            if(Input.isModifierDown(KB_LCTRL) or Input.isModifierDown(KB_RCTRL)) then
                recursively = true
            end
            item.sublist:expand(recursively)
        end

        if(not(expanded)) then
            minimizeButton:minimize()
        end

        --		there are some useful optimizations to be gained by not doing this.
        --		local visibilityHandler = {
        --			onPassiveUpdate = function(self)
        --				if(Util.numKeys(item.sublist.items)==0) then
        --					self:hide()
        --				else
        --					self:show()
        --				end
        --			end
        --		}

        -- empty groups will not be drawn
        --		item:addEventHandler(visibilityHandler)
        return item
    end

    pgrid.clear = function(self)
        for _, item in pairs(self.items) do
            item.sublist:__destroy()
        end
        self.items = {}
    end

    pgrid.addGroup = function(self, parent, name, expanded)
        if(parent==nil) then
            parent = self
        end
        local item = parent.items[name]
        if(item == nil) then
            item = self.__newgroup(parent, name, expanded)
            parent.items[name] = item
            return item.sublist
        end
        return item.sublist
    end

    pgrid.addProperty = function(self, parent, name, value)
        assert(value)
        if(parent==nil) then
            parent = self
        end
        local item = parent.items[name]
        if(item == nil) then
            item = self.__newproperty(parent, name, value)
            parent.items[name] = item
            return item
        else
            if(value and item.setValue) then
                if(type(value) == "function") then
                    value = value()
                end
                item:setValue(value)
            end
        end
        return item
    end

    --[[
        pgrid.populate() expects a specially formatted array:
        {
            { "+groupA" }, 				-- push group (and expand)
            { "property0", "value0" },	-- NOTE: see below
            { "property1", "value1" },	-- values may be also be an anonymous function.
            { "-groupB" },				-- push group (the '-' is unnecessary groups are collapsed by default)
            { "property0", "value0" },
            { "property1", "value1" },
            { name="property2", value="value2" }, -- this is also accepted
            -1,							-- pop group
            -1,							-- pop group
            -1,							-- pop group
            -- NOTE: it is not necessary to pop trailing groups unless you intend to add
            -- more groups/properties.
        }
        Properties must have non-nil values to avoid being misinterpreted as groups. If that is
        limiting, you can always pass in an anonymous function that returns the value, this will
        work even if the returned value is nil. Another option is to set value to the string literal "nil",
        since this would be the intended output.
    ]]
    pgrid.populate = function(self, group, array)
        local stack = { group }
        for i, v in ipairs(array) do
            local top = stack[#stack]
            if(v == -1) then
                table.remove(stack, #stack)
            elseif(type(v) == "table") then
                local name = v[1] or v.name
                local value = v[2] or v.value
                assert(name and type(name) == "string")
                if(value) then
                    self:addProperty(top, name, value)
                else
                    local expanded = false
                    local count
                    name, count = string.gsub(string.gsub(name, "-", ""), "+", "")

                    if(count > 0) then
                        expanded = true
                    else
                        expanded = false
                    end

                    table.insert(stack, self:addGroup(top, name, expanded))
                end
            end
        end
    end

    return pgrid
end

