
ToolboxMenus = {}

function ToolboxMenus:new()
    local toolMenus = params or {}
    setmetatable(toolMenus, self)
    self.__index = self

    local menuBar = GUI.frame(nil, GUIPos(0, 0), GUISize(SZ_FILL, 35), "")
    menuBar:dock(DOCK_TOP)

    local toolsButton = GUI.button(menuBar, GUIPos(0, 0), GUISize(75, SZ_WRAP), "Tools")
    toolsButton:dock(DOCK_LEFT)

    function toolsButton:onButtonUp()
        local showMenu = toolMenus.activeButton ~= self
        if toolMenus.menu then toolMenus:_dismiss() end
        if showMenu then
            toolMenus.activeButton = self
            local buttonPos = self:getWorldPosition()
            local menuPos = GUIPos(buttonPos.x, buttonPos.y + self:getHeight())
            toolMenus.menu = GUI.frame(nil, menuPos, GUISize(200, SZ_FILL), "popupmenu.frame")
            local menuSize = GUISize(0, 10)
            for name, tool in pairs(Toolbox.tools) do
                if tool ~= toolMenus then
                    local checkbox = GUI.checkbox(toolMenus.menu, px(0, 0), GUISize(SZ_WRAP, SZ_WRAP), tool.active, name, "")
                    checkbox:dock(DOCK_TOP)
                    menuSize.x = math.max(menuSize.x, checkbox:getWidth())
                    menuSize.y = menuSize.y + checkbox:getHeight()
                    checkbox.toolName = name

                    function checkbox:onclick()
                        Toolbox:activateTool(self.toolName, self.checked)
                        toolMenus:_dismiss()
                    end
                end
            end
            toolMenus.menu:resize(menuSize.x, menuSize.y)
        end
    end

    local optionsButton = GUI.button(menuBar, GUIPos(0, 0), GUISize(75, SZ_WRAP), "Options")
    optionsButton:dock(DOCK_LEFT)

    function optionsButton:onButtonUp()
        local showMenu = toolMenus.activeButton ~= self
        if toolMenus.menu then toolMenus:_dismiss() end
        if showMenu then
            toolMenus.activeButton = self
            local buttonPos = self:getWorldPosition()
            local menuPos = GUIPos(buttonPos.x, buttonPos.y + self:getHeight())
            toolMenus.menu = GUI.frame(nil, menuPos, GUISize(200, SZ_FILL), "popupmenu.frame")
            local menuSize = GUISize(0, 10)
            for _, option in ipairs(Toolbox.options) do
                local enabled = option.get()
                local checkbox = GUI.checkbox(toolMenus.menu, px(0, 0), GUISize(SZ_WRAP, SZ_WRAP), enabled, option.name, "")
                checkbox:dock(DOCK_TOP)
                menuSize.x = math.max(menuSize.x, checkbox:getWidth())
                menuSize.y = menuSize.y + checkbox:getHeight()

                function checkbox:onclick()
                    option.set(self.checked)
                    Persist.toolbox.options[option.name] = self.checked
                    System.flushPersist()
                    toolMenus:_dismiss()
                end
            end
            toolMenus.menu:resize(menuSize.x, menuSize.y)
        end
    end

    toolMenus.menuBar = menuBar
    return toolMenus
end

function ToolboxMenus:hide()
    self:_dismiss()
    self.menuBar:hide()
end

function ToolboxMenus:show()
    self.menuBar:show()
end

function ToolboxMenus:_dismiss()
    if self.menu then
        self.menu:hide()

        -- TODO: this is a work around to a bug in the engine.
        -- Sometimes when we destroy a frame, it will crash
        -- in the engine code.  Commented out the destruction
        -- until the bug is fixed.  Until then, that means we will
        -- be leaking a menu everytime one is dismissed.
--        self.menu:destroy()

        self.menu = nil
        self.activeButton = nil
    end
end
