
-- configurators
require "windyday/scripts/configurators/tornado"
require "windyday/scripts/configurators/mouse"
require "windyday/scripts/configurators/hat"
require "windyday/scripts/configurators/squirrel"
require "windyday/scripts/configurators/water"
require "windyday/scripts/configurators/dropeffect"
require "windyday/scripts/configurators/dropground"
require "windyday/scripts/configurators/bubbleflash"
require "windyday/scripts/configurators/apple"
require "windyday/scripts/configurators/raccoons"
require "windyday/scripts/configurators/chipmunks"
require "windyday/scripts/configurators/bird"
require "windyday/scripts/configurators/egg"
require "windyday/scripts/configurators/bear2andfish"
require "windyday/scripts/configurators/turtle"
require "windyday/scripts/configurators/dighole"
require "windyday/scripts/configurators/dighole_molehole"
require "windyday/scripts/configurators/shadow"
require "windyday/scripts/configurators/whitefluff"
require "windyday/scripts/fixup/actorutil"

local LogDebug = true
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end

ActorManager = {
    configurators = {
        -- Actor name = configuration function
        -- The configuration function will be called with the actor as a parameter
        -- after the actor is created
        Slice_Rampage = Water.configWater,
        TornadoProxy = Tornado.configTornado,
		Mouse = Mouse.configMouse,
		Hat = Hat.configHat,
		Squirrel = Squirrel.configSquirrel,
        Apples = Apples.config,
        Raccoons = Raccoons.config,
        Chipmunks = Chipmunks.config,
        DropProxy = DropEffect.configDropEffect,
        DropGround = DropGround.configDropGround,
        whiteBubbleFlash = BubbleFlash.config,
        Bird = Bird.config,
        Egg = Egg.config,
        Turtle = Turtle.config,
        Bear2 = Bear2AndFish.config,
        DigHole = DigHole.configDigHole,
        DigHole_MoleHole = DigHole_MoleHole.configDigHole,
        Shadow_Hat = ShadowConfig.configShadowHat,
        Shadow_Squirrel = ShadowConfig.configShadowSquirrel,
        Shadow_Turtle = ShadowConfig.configShadowTurtle,
        Shadow_Mouse = ShadowConfig.configShadowMouse,
        Shadow_SquirrelNut1 = ShadowConfig.configShadowSquirrelNut1,
        Shadow_SquirrelNut2 = ShadowConfig.configShadowSquirrelNut2,
        Shadow_Bear2 = ShadowConfig.configShadowBear2,
        WhiteFluff = WhiteFluff.config,

        -- Easter Eggs
        Trigger_ChipmunkHoleA = EasterEgg.config,
        Trigger_ChipmunkHoleB = EasterEgg.config,
        Trigger_ChipmunkHoleC = EasterEgg.config,
        Trigger_Raccoons = EasterEgg.config,
        Trigger_StareBear = EasterEgg.config,
        Trigger_TurtleShellShock = EasterEgg.config,

        -- Secondary (B-roll) audio
        Audio_ChipmunkSlapping1 = SecondaryAudioPlayer.config,
        Audio_ChipmunkSlapping2 = SecondaryAudioPlayer.config,
        Audio_MoleLeaving = SecondaryAudioPlayer.config,
        Audio_RaccoonBrook = SecondaryAudioPlayer.config,
        Audio_RaccoonWashing = SecondaryAudioPlayer.config,
        Audio_SquirrelFindHat = SecondaryAudioPlayer.config,
        Audio_MouseLanding = SecondaryAudioPlayer.config,
    },
	activeActors = {},
	scene = nil,
    actorPool = {},
}

function ActorManager:init(scene)
    self.scene = scene
    self.activeActors = {}
    self.scene.actors = self.scene.actors or {}
end

local actorPreCreateList = 
{
    "CameraMan",
    "SunGlow",
    "Iris",
    "Hat",
    "Shadow_Hat",
    "Broom",
    "Env_Forest",
    "Slice_Forest",
    "ForestCanopy",
    "Slice_CenterSpec",
    "Shadow_Mouse",
    "Mouse",
}

function ActorManager:startPreCreate()
    self.preCreateIndex = 1
end

function ActorManager:preCreateUpdate()
    if self.preCreateIndex ~= nil and self.preCreateIndex <= #actorPreCreateList then
        local name = actorPreCreateList[ self.preCreateIndex ]
        Debug("Pre-creating actor object: "..name..", index: "..self.preCreateIndex)

        if self.scene.actors[name] == nil then
            local actorData = def.actors[name]
            local actor     = self.scene:createActor(name, actorData, nil, true )

            self.scene.actors[name] = nil
            self.actorPool[name]    = actor

            actor:setVisibility(false)
            actor:setActive(false)
        end

        self.preCreateIndex = self.preCreateIndex+1
    end
end

function ActorManager:getActor(name, segmentName)
    local actorData = def.actors[name]
	if(self.activeActors[name] == nil) then
		if (actorData == nil) then
            -- if the name does not exist create empty actor
            Debug("Create empty actor object: "..name)

            self.activeActors[name] = self.scene:createTransform(name)
            self.scene.actors[name] = self.activeActors[name]
	        -- attach helper methods to actor
	        self.activeActors[name].getBone = ActorUtil.getBone
	        
            local targetDef = LookAtTargets[name]
            -- If this object is also a look at target then initialize it as such
            if (targetDef) then
                -- Debug model setup
                -- self.activeActors[name] = self.scene:createActor(name,  {
                --     model = "windyday:models/hat.mxm",
                --     materials = { 
                --         { name = "Mat_HatTexture", shader = "windyday:shaders/character.rsfx", technique = "characterbodysimple", renderGroup = 2, textures = { u_MainTex = "windyday:textures/hat_diffuse_325_256px.dds", }, parameters = { u_DarkColor = Color.new( 0.2235294, 0.172549, 0.1176471, 1 ), u_TexContrast = 0.75, u_TexOffset = 0.5, u_AmbientLightColor = Color.new( 1, 1, 1, 1 ), u_LightDirection = Vector.new( 0, -1, 0, 0 ), u_MainTex_ST = Vector.new(1, 1, 0, 0), }, },
                --     },
                -- })

                local target = self.activeActors[name]
                if (target.components == nil or target.components["lookatdetector"] == nil) then
                    local component = nil

                    -- we want to add a LookAtDetector component to this target
                    component = LookAtDetector:new(self.scene.mainCamera, name)

                    -- since there is no bone, apply position data to the target itself
                    -- NOTE: CURRENTLY FLIPPING X ON IMPORT (due to handedness reversal from prototype)
                    local initialPos = Vector.new(targetDef.position[1]*-1, targetDef.position[2], targetDef.position[3])
                    local initialRot = Vector.new(targetDef.rotation[1]*-1, targetDef.rotation[2], targetDef.rotation[3], targetDef.rotation[4])
                    target:setLocal({ position = initialPos, rotation =  initialRot})

                    -- look at data
                    Debug("Adding look at detector to " .. name)

                    -- Assign values from definition
                    component.useBoxCollision   = targetDef.useBoxCollision
                    component.colliderRadius    = targetDef.colliderRadius
                    component.colliderWidth     = targetDef.colliderWidth
                    component.colliderHeight    = targetDef.colliderHeight

                    target:addComponent("lookatdetector", component)
                end
            end

			self.activeActors[name].name = name
			self.activeActors[name].materials = {}

            if (self.configurators[name]) then
                self.configurators[name](self.activeActors[name])
            end
		else
            local actor = self.actorPool[name]
            if actor == nil then
                actor = self.scene:createActor(name, actorData, nil, true)
    			Debug("Created actor object: "..name)
            else
                Debug("Created actor object from pre-create list: "..name)
                actor:setActive(true)
                self.scene.actors[name] = actor
                self.actorPool[name]    = nil
            end
			
	        -- attach helper methods to actor
	        actor.getBone = ActorUtil.getBone
			
            actor.name = name
            self:addAnimationDriver(actor)
			
            --self:checkMaterials(actor, actorData)
			
			self.activeActors[name] = actor
			
	        -- attach helper methods to actor
	        self.activeActors[name].getBone = ActorUtil.getBone

            self.activeActors[name]:setVisibility(false)
            if (self.configurators[name]) then
                self.configurators[name](actor)
            end
        end
        
        Events.send("ActorManager", "onActorCreated", self.activeActors[name])
	end

    local actor = self.activeActors[name]
    if segmentName and
       actor and not (actor.animation and actor.animation[segmentName]) and
       actorData and actorData.animation and actorData.animation[segmentName] then

        -- add the segment's animation and idle animation
        local anim = actorData.animation[segmentName]
        actor.animation = actor.animation or {}
        actor.animation[segmentName] = actor:addAnimation(segmentName, anim[1], anim[2], anim[3], anim[4], anim[5] or true);
        local idleAnimName = segmentName .. "_Idle"
        local idleAnim = actorData.animation[idleAnimName]
        if idleAnim then
            actor.animation[idleAnimName] = actor:addAnimation(idleAnimName, idleAnim[1], idleAnim[2], idleAnim[3], idleAnim[4], idleAnim[5] or true);
        end
    end

    return self.activeActors[name]
end

function ActorManager:getBone(boneDef)
    local actorName = boneDef.actor
    local bonePath = boneDef.bone
    
    local actor = self:getActor(actorName)
    if (bonePath == nil) then
        return actor
    else
        return actor:getBone(bonePath)
    end
end

function ActorManager:addAnimationDriver(actor)
    if (actor.animDriver == nil) then
        actor.animDriver = actor:addComponent("animDriver", AnimationDriver.new())
    end
end

function ActorManager:checkMaterials(actor, actorData)
    local materials = actor:listMaterials()
    if materials and actorData.materials and #materials ~= #actorData.materials then
		Log.error( "Bad material mapping for: "..actor.name )
	end 
end

function ActorManager:destroyActor(name)
	-- Check to see if the object is a valid object before deleting
    if (self.activeActors[name] and status(self.activeActors[name]) ~= nil) then
        self.activeActors[name]:destroy()
        self.activeActors[name] = nil
        -- TODO: should probably switch all usages of activeActors to scene.actors
        if (self.scene.actors) then
            self.scene.actors[name] = nil
        end

        Log.message("ActorManager: destroyed "..name)
        Events.send("ActorManager", "onActorDestroyed", name)
    end
end
