
-- Google Analytics APIs

--require "windyday/scripts/launcher/launcher"
require "windyday/scripts/launcher/config"

-- This UID is property (aka episode) specific
-- as new episodes are released, this should correspondingly be updated
local analyticsUID			= "UA-40905299-1"

--  Session level custom dimensions
ANALYTICS_BUILD_DIMENSION 			= 5
ANALYTICS_PACKAGE_DIMENSION			= 29
ANALYTICS_MOXIE_DIMENSION			= 30
ANALYTICS_USER_TYPE_DIMENSION			= 7
ANALYTICS_SESSION_TYPE_DIMENSION		= 8
ANALYTICS_APP_TYPE_AT_USER_LAUNCH		= 9
ANALYTICS_CLIENT_LOCAL_HOUR			= 10
ANALYTICS_CLIENT_LOCAL_DAY			= 11
ANALYTICS_DAYS_SINCE_FIRST_PROMPT_DIMENSION	= 14
ANALYTICS_NUMBER_OF_HAT_PROMPTS_DIMENSION	= 15
ANALYTICS_SUSTAINED_RUN_COUNT			= 16
ANALYTICS_TOTAL_RUN_COUNT			= 26
ANALYTICS_TOTAL_SESSIONS_COMPLETE		= 31
ANALYTICS_SESSION_WAS_FROM_RESUME		= 19
ANALYTICS_SESSION_LAUNCH_REASON			= 24
ANALYTICS_HOUR_OF_FIRST_ENGAGEMENT		= 3
ANALYTICS_DOGFOOD_BUCKET			= 33

-- Hit level custom dimensions
ANALYTICS_HEADSET_DIMENSION			= 1
ANALYTICS_SOUND_LEVEL_DIMENSION			= 22
ANALYTICS_ACTIVE_SCENE_DIMENSION		= 17
ANALYTICS_TIME_SUBSEGMENT_DIMENSION		= 12
ANALYTICS_TIME_SUBSEGMENT_SCENE_BASED_DIMENSION	= 23 
ANALYTICS_REASON_FOR_ATTRITION			= 18

-- Custom metrics
ANALYTICS_ACTIVE_APPLICATION_METRIC_SLOT	= 3
ANALYTICS_BUILD_METRIC_SLOT			= 4
ANALYTICS_HEADSET_METRIC_SLOT			= 2
ANALYTICS_VOLUME_METRIC_SLOT			= 1
ANALYTICS_RUN_COUNT_SLOT			= 5
ANALYTICS_HAT_SHOWN_COUNT_SLOT			= 6
ANALYTICS_TOTAL_TIME_IN_DOWNLOAD_SLOT		= 7
ANALYTICS_TOTAL_TIME_DOWNLOADING_SLOT		= 8

-- Used for time bins
local ANALYTICS_MILLIS_PER_TIME_BIN		= 5000
local ANALYTICS_MILLIS_PER_FPS_BIN		= 3000
local timeBin 					= 0 
local startTime					= -1
local lastObservedScene				= ""
local showIsPlaying				= false
local fpsBin 					= 0 
local timeAtStartOfFpsBin			= 0

-- for fps tracking
-- JamesB says the current raw values are unreliable, JamesR says to use the build in tracking, needs resolving
local framesThisBin 				= 0
local timeThisSegmentBegan			= 0

local curNumberOfTouchesInTimeBin		= 0
local curNumberOfSwipesInTimeBin		= 0

local curNumberOfTouchesInSceneBin		= 0
local curNumberOfMovesInSceneBin		= 0

-- Used for tracking GWI entry
local timeOfFirstGWI				= -1
local ANALYTICS_SECONDS_UNTIL_SUSTAINED_ENGAGEMENT = 15

local immAccessReported				= false

local promptReason				= nil

-- used to determine whether our interruption was user or system generated
local unmatchedOnUserLeaveHint 			= false

local touchIsDown				= false
local curMoveCount				= 0

-- Make sure the persist tables are setup
Persist = Persist or {}
Persist.windyday = Persist.windyday or {}
Persist.windyday.totalRunCount = Persist.windyday.totalRunCount or 0 
Persist.windyday.sustainedrunCount = Persist.windyday.sustainedrunCount or 0
Persist.windyday.episodesCompleted = Persist.windyday.episodesCompleted or 0
Persist.windyday.pendingUltimateAccess = Persist.windyday.pendingUltimateAccess or false
Persist.windyday.promptsSinceLastEngagement = Persist.windyday.promptsSinceLastEngagement or 0

Persist.windyday.specialCaseFirstAbortValid = Persist.windyday.specialCaseFirstAbortValid or false

AnalyticsManager = {}

-- how many digits we will pad values out to (e.g. "00012" for 12 ) 
local ANALYTICS_ZERO_PAD_DEFAULT		= 5

-- helper function to calculate time deltas accounting for rollover
calculateTimeDelta = function ( startT, endT )
	-- if rollover happens endtime will actually be less then start time
	if endT < startT then
		-- HACK:  there was rollover.  lua does not offer a max value utility, and since 
		-- I don't know what the underlying architecture will be I'm just going to take a 
		-- subset of the delta until we have a proper native API in place.
		return endT
	else 
		return endT - startT
	end
end

--TODO: remove on next build
mangleString = function( intToMangle, numDigits )
	local resultString = ""

	if numDigits == nil then
		numDigits = ANALYTICS_ZERO_PAD_DEFAULT
	end
	if intToMangle ~= nil then
		resultString = string.format("%0"..tostring(numDigits or 0).."i", intToMangle or 0)
	end

	return resultString
end

AnalyticsManager.getTimeBin = function()
	if startTime >= 0 then
		return math.floor(calculateTimeDelta(startTime, Time.milliseconds() ) / ANALYTICS_MILLIS_PER_TIME_BIN)
	else
		-- time bins begin at the hat prompt, we don't care about GWI load
		return 0
	end
end

AnalyticsManager.getTimeBinSceneBased = function()
	if startTime >= 0 then
		return math.floor((Time.milliseconds() - timeThisSegmentBegan) / ANALYTICS_MILLIS_PER_TIME_BIN)
	else
		-- time bins begin at the hat prompt, we don't care about GWI load
		return 0
	end
end

AnalyticsManager.setAllHitDimensions = function()
	Analytics.setCustomDimension(ANALYTICS_TIME_SUBSEGMENT_DIMENSION, mangleString(AnalyticsManager:getTimeBin()))
	Analytics.setCustomDimension(ANALYTICS_TIME_SUBSEGMENT_SCENE_BASED_DIMENSION, mangleString(AnalyticsManager:getTimeBinSceneBased()))
	Analytics.setCustomDimension(ANALYTICS_HEADSET_DIMENSION, mangleString( JavaAudio.getHeadsetState()) )
	Analytics.setCustomDimension(ANALYTICS_SOUND_LEVEL_DIMENSION, mangleString( JavaAudio.getStreamVolume(1), 3))
	Analytics.setCustomDimension(ANALYTICS_ACTIVE_SCENE_DIMENSION, lastObservedScene or "")
end

AnalyticsManager.recordDayOfFirstPrompt = function()

	-- note:  the date is not returned human readable, which means the month is not 0 indexed
	-- our functions work off 0 offset...  I wish this was all just consistent
	local date = Calendar.getDate()
	date.month = date.month-1

	-- mark down the day of our first prompt so we can see how many days away from our first prompt we interacted
	Persist.windyday.yearOfFirstRun = Persist.windyday.yearOfFirstRun or date.year
	Persist.windyday.monthOfFirstRun = Persist.windyday.monthOfFirstRun or date.month
	Persist.windyday.dayOfFirstRun = Persist.windyday.dayOfFirstRun or date.day
	Persist.windyday.hourOfFirstRun = Persist.windyday.hourOfFirstRun or Calendar.getLocalTime().hour
end

AnalyticsManager.initialize = function()

	-- initialize analytics
	Log.message("Initialising Google Analytics.")
	
	-- todo: remove after next build
	if (Analytics.initialize ~= nil) then
		Analytics.initialize(analyticsUID)
	else
		Analytics.initialise(analyticsUID)
	end
	
	if Analytics.setVersionString ~= nil then
		Analytics.setVersionString(System.versionString())
	end

	-- analytics would like to know what segments are playing for scene transitions
	Events.listen(AnalyticsManager, "segmentPlaying")
end

AnalyticsManager.sendInitialStats = function()
	-- tell the service this is a doodle session as opposed to a download session
	Analytics.setCustomDimension(ANALYTICS_BUILD_DIMENSION, System.versionString() or "unknown")
	Analytics.setCustomDimension(ANALYTICS_SESSION_TYPE_DIMENSION, "SESSION_TYPE_DOODLE")
	Analytics.setCustomDimension(ANALYTICS_PACKAGE_DIMENSION, Platform.package)
	Analytics.setCustomDimension(ANALYTICS_MOXIE_DIMENSION, Platform.version)

	Analytics.sendEvent("GeneralInfo " .. Platform.package, "CurrentShow", Platform.package, 0)

	-- set the dogfood bucket
	local config = Config.active()
	Analytics.setCustomDimension(ANALYTICS_DOGFOOD_BUCKET, config.name)

	-- build info
	Analytics.sendEvent("GeneralInfo", "BuildID", System.versionString(), 0)

	-- record how many times the user has previously completed the show
	Analytics.setCustomDimension( ANALYTICS_TOTAL_SESSIONS_COMPLETE, mangleString(Persist.windyday.episodesCompleted))

	-- record the kind of app that was being used at launch
--	local activeTargetInfo = PixelPipe.getActiveTargetInfo()
--	local packageName = "";
--	if (activeTargetInfo ~= nil) then
--		packageName = activeTargetInfo.packageName;
--	end
--	Analytics.setCustomDimension(ANALYTICS_APP_TYPE_AT_USER_LAUNCH, AnalyticsManager.getAppType(packageName))

	-- record the client time so we can filter based on the clients local time (analytics converts to GMT)
	local clientHour = Calendar.getLocalTime().hour
	local localDay = Calendar.getDate().day
	Analytics.setCustomDimension( ANALYTICS_CLIENT_LOCAL_HOUR, mangleString(clientHour, 2))
	Analytics.setCustomDimension( ANALYTICS_CLIENT_LOCAL_DAY, mangleString(localDay, 2))
	
	-- Also, because our .apk is not changing, manually keep track of episode viewings through the persist table
	-- (if we do not do this, it becomes very complicated to establish new vs. returning visitors)
	Analytics.sendEvent("GeneralInfo", "InitialVolume", "", JavaAudio.getStreamVolume(1))
	Analytics.sendEvent("GeneralInfo", "InitialHeadsetState", "", JavaAudio.getHeadsetState())
	Analytics.sendEvent("GeneralInfo", "TimeSubsegmentDuration", "", ANALYTICS_MILLIS_PER_TIME_BIN)
	Analytics.sendEvent("GeneralInfo", "RunCount", mangleString(Persist.windyday.totalRunCount), 0)

	-- allow us to filter out testers in the future
	Analytics.setCustomDimension(ANALYTICS_USER_TYPE_DIMENSION, Global.getUserType() or "")
end


-- listing of scenes we consider to be "main scenes"
-- each of these scenes will be represented as their own view as opposed to side shows and interactions
-- which will be captured through events
local mainScenes = {
	"HatCycle",
	"PepeIntro",
	"ChaseA",
	"BushA",
	"SquirrelA",
	"TeaseA",
	"Rampage1",
	"Rampage2A",
	"Rampage3A",
	"WindUpA",
	"TornadoA",
	"BarrageA",
	"DropA",
	"End1A",
}

-- a listing of counts associated with each scene so we do not inadvertently end up blasting the 
-- server with repeat scenes 
local curCountTable = {} 

AnalyticsManager.segmentPlaying = function(self, segmentName)
	segmentName = segmentName or ""

	local found = false
	for i, name in ipairs(mainScenes) do
		if name == segmentName then
			found = true
		end
	end

	if found == false then
		-- this apparently isn't a segment we're interested in, skip it!
		return
	end

	curCountTable[segmentName] = curCountTable[segmentName] or 0
	if curCountTable[segmentName] == 0 then 
		-- let's also send some touch information for this scene
		self.setAllHitDimensions()
		Analytics.sendEvent("Ux", "touchesInSceneBucket", "", curNumberOfTouchesInSceneBin)
		curNumberOfTouchesInSceneBin = 0

		self.setAllHitDimensions()
		Analytics.sendEvent("Ux", "movesInSceneBucket", "", curNumberOfMovesInSceneBin)
		curNumberOfMovesInSceneBin = 0

		-- and then mark down in our hash that we've seen the scene
		curCountTable[segmentName] = 1
		Analytics.sendView(segmentName)

		-- if this is a sequence we're interested in, report fps for that scene before we switch
		local timeInSegment = Time.milliseconds() - timeThisSegmentBegan
		timeThisSegmentBegan = Time.milliseconds()

	else
		-- we've already seen this scene, no need to blast analytics with data
		curCountTable[segmentName] = curCountTable[segmentName]+1
	end

	lastObservedScene = segmentName
end

AnalyticsManager.update = function(self)
	-- don't start doing time binning until we've actually seen the hat
	if startTime == -1 then
		return
	end

	framesThisBin = framesThisBin + 1

	-- the following code only applies to when the show begins
	-- TODO:  what's the proper mechanism for querying if the game is in show
	if startTime == -1 then 
		return
	end

	-- see if the user has stayed within the app for at least the required engagement period
	-- TODO:  if the app persists past closure this needs a test to see if it's still actually rendering
	local elapsedTime = calculateTimeDelta(startTime, Time.milliseconds() )
	if immAccessReported ~= true and elapsedTime >= (ANALYTICS_SECONDS_UNTIL_SUSTAINED_ENGAGEMENT*1000) then

		-- see AnalyticsManager.GWIAbandoned for explanation of this
		Persist.windyday.specialCaseFirstAbortValid = true

		--  this is the number of actually sustained runs we have made 
		Persist.windyday.sustainedrunCount = Persist.windyday.sustainedrunCount + 1
		Analytics.setCustomDimension(ANALYTICS_SUSTAINED_RUN_COUNT, mangleString(Persist.windyday.sustainedrunCount))

		-- if the user clicked on the hat within allotted time and stayed for allotted time 
		if immAccessReported ~= true then
			immAccessReported = true
			self.setAllHitDimensions()
			Analytics.sendEvent("GWI", "Success", "SustainedEngagement", Persist.windyday.promptsSinceLastEngagement)
		end

		-- ultimate access and immediate access are NOT mutually exclusive by design
		if Persist.windyday.pendingUltimateAccess == true then
			Persist.windyday.pendingUltimateAccess = false

			self.setAllHitDimensions()
			Analytics.sendEvent("GWI", "Success", "UltimateAccess", 1)
		end

		-- reset our counter for how many hat prompts have been initiated since the last sustained engagement
		Persist.windyday.promptsSinceLastEngagement = 0
	end
end

AnalyticsManager.onShowBegan = function(self)
	-- used for time binning
	startTime = Time.milliseconds()
	timeAtStartOfFpsBin = Time.milliseconds()
	showIsPlaying = true

	-- total run count information
	if (Persist.windyday.totalRunCount == nil) then
		Log.error("Persist.windyday.totalRunCount is NIL! Copy /data/data/com.motorola.avatar/shared/persist.lua off the device and send to jritts")
	end
	if (type(Persist.windyday.totalRunCount) ~= "number") then
		Log.error("Persist.windyday.totalRunCount is not a number! Copy /data/data/com.motorola.avatar/shared/persist.lua off the device and send to jritts")
	end

	Persist.windyday.totalRunCount = Persist.windyday.totalRunCount + 1
	Analytics.setCustomDimension( ANALYTICS_TOTAL_RUN_COUNT, mangleString(Persist.windyday.totalRunCount))

	--  add tracking information for the absolute run count (as opposed to the sustained engagement)
	if Persist.windyday.totalRunCount == 0 then 
		Analytics.sendEvent("GeneralInfo", "FirstTimeRun", "", 1)
	end
	
	-- how was the show launched (hat prompt, splash, etc.)
	local showLaunchReason = nil
	if promptReason == "hat_prompt" then 
		showLaunchReason = "hat_prompt"
	elseif Context._getStartReasonString() ~= nil and Context._getStartReasonString() == "splash_play" then 
		showLaunchReason = "splash_play"
	elseif (Context._getStartReasonString() ~= nil) and (Context.getPromptType() == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE) then
		showLaunchReason = "notification_play"
	else
		-- by exclusion, it was the widget or shortcut
		showLaunchReason = "widgetAndShortcut"
	end


	Analytics.setCustomDimension( ANALYTICS_SESSION_LAUNCH_REASON, showLaunchReason)

	-- mark down if this was a resume session or not
	if Context.getCheckpoint() == nil then
		Analytics.setCustomDimension( ANALYTICS_SESSION_WAS_FROM_RESUME, "FromBeginning" )
	else
		Analytics.setCustomDimension( ANALYTICS_SESSION_WAS_FROM_RESUME, "FromResume" )
		lastObservedScene = Context.getCheckpoint() or ""
	end

	self.setAllHitDimensions()
	Analytics.sendEvent("GWI", "ShowBegan", showLaunchReason, 0)
end

AnalyticsManager.onShowEnded = function(self)
	showIsPlaying = false

	local elapsedTime = calculateTimeDelta(startTime, Time.milliseconds() ) / 1000
	if elapsedTime <= ANALYTICS_SECONDS_UNTIL_SUSTAINED_ENGAGEMENT then
		-- it's possible the user is going to return at a later point, and this allows us to qualify those users
		Persist.windyday.pendingUltimateAccess = true

		-- report a quick abort
		self.setAllHitDimensions()
		Analytics.sendEvent("GWI", "Failure", "QuickAbort", elapsedTime)

		-- see AnalyticsManager.GWIAbandoned
		if Persist.windyday.specialCaseFirstAbortValid == false then
			Persist.windyday.specialCaseFirstAbortValid = true
			self.setAllHitDimensions()
			Analytics.sendEvent("GWI", "FailureSpecialCase", "SingleHatQuickAbort", 0)
		end
	end

	Analytics.sendEvent("MangleStore", "ShowEnded_elapsedTime", mangleString(elapsedTime), elapsedTime)
end

AnalyticsManager.onDoNotify = function()
	AnalyticsManager.setAllHitDimensions()
	Analytics.sendEvent("GWI", "Impression", "Flourish", 0)
end

AnalyticsManager.setAppAtTimeOfPrompt = function(self)
--    local appType = AnalyticsManager.getAppType(Query.getActiveActivity())
--	self.setAllHitDimensions()
--	Analytics.sendEvent("GWI", "AppAtTimeOfPromptAccepted", appType, 0)
end

AnalyticsManager.recordTouchDown = function(self)
	-- keep track of the number of touches for each time bucket
	curNumberOfTouchesInTimeBin = curNumberOfTouchesInTimeBin + 1 
	curNumberOfTouchesInSceneBin = curNumberOfTouchesInSceneBin + 1

	touchIsDown = true
end

AnalyticsManager.recordTouchUp = function(self)
	-- this wasn't just a touch but a proper swipe
	if curMoveCount >= 5 then 
		-- keep track of the number of touches for each time bucket
		curNumberOfSwipesInTimeBin = curNumberOfSwipesInTimeBin + 1 
		curNumberOfMovesInSceneBin = curNumberOfMovesInSceneBin + 1
	end 

	curMoveCount = 0
	touchIsDown = false
end

AnalyticsManager.recordMoveEvent = function(self)
	-- moves only occur on swiping motions in between a touch down/up
	curMoveCount = curMoveCount + 1
end

AnalyticsManager.hatPromptIsolated = function(animName)
	-- prior to this there was no way to differentiate hat prompts, so this will be an 
	-- additional piece of info in case we decide this proves useful in the future
	AnalyticsManager.setAllHitDimensions()
	Analytics.sendEvent("GWI", "HatPromptBreakdown", animName, 0)
end

AnalyticsManager.hatPromptBegun = function(self)
	Analytics.sendView("GWI_hat_prompt")
	self.setAllHitDimensions()
	Analytics.sendEvent("GWI", "Impression", "HatPrompt", 0)

	-- hack to circumvent the incredible crazy state machine that is GWI
	promptReason = "hat_prompt"

	-- mark down a time stamp so we can establish the time until click
	timeOfFirstGWI = Time.seconds()

	-- mark down the date if this was our first presentation
	AnalyticsManager:recordDayOfFirstPrompt()
	if Calendar.getDateDifferenceInDays ~= nil then 
		local year = Persist.windyday.yearOfFirstRun;
		local month = Persist.windyday.monthOfFirstRun;
		local day = Persist.windyday.dayOfFirstRun;

		local daysSinceFirstPrompt = math.abs(Calendar.getDateDifferenceInDays(day, month, year))
		Analytics.setCustomDimension( ANALYTICS_DAYS_SINCE_FIRST_PROMPT_DIMENSION, mangleString(daysSinceFirstPrompt))
	end

	-- DSA thinks it could be interesting to know the hour that the very first hat was presented
	Analytics.setCustomDimension(ANALYTICS_HOUR_OF_FIRST_ENGAGEMENT, mangleString(Persist.windyday.hourOfFirstRun, 2))

	-- we are also interested in how many times the hat prompt has been presented, regardless of it being accepted
	Persist.windyday.promptsSinceLastEngagement = Persist.windyday.promptsSinceLastEngagement + 1
	Analytics.setCustomDimension( ANALYTICS_NUMBER_OF_HAT_PROMPTS_DIMENSION, mangleString(Persist.windyday.promptsSinceLastEngagement) )
end

AnalyticsManager.kickupHappened = function(self)
	local timeUntilAccept = Time.seconds()-timeOfFirstGWI

	-- send an event describing how long it took to reach kickup
	-- note, this event will not be sent for show starts that had no hat, so it should not be used for user counts
	self.setAllHitDimensions()
	Analytics.sendEvent("GWI", "Success", "TimeBetweenGWIAndAccess", timeUntilAccept)
	Analytics.sendEvent("MangleStore", "GWI_HatAccept", mangleString(timeUntilAccept), timeUntilAccept)
end

AnalyticsManager.GWIAbandoned = function(self)
	-- previously we aborted the session here but now we are going to do it from the global shutdown
	Analytics.sendEvent("GWI", "Failure", "HatFadedForever", 0)
	Persist.windyday.pendingUltimateAccess = true

	-- This is a very special (and unfortunate) case that has to be handled
	-- to report the metric "how many users aborted and never returned or engaged", we would like to say 
	-- ABORTS { runCount == 0 } - ULTIMATE_ACCESS { runCount == 1 } , but, for users that aborted multiple times, this will overcount
	-- aborts have a many to one relationship with ULTIMATE_ACCESS, and so in this ONE special case we will send up a single event
	-- so that we can count this metric.  
	-- Note that if the user has ever engaged previously, we aren't gong to count this.
	if Persist.windyday.specialCaseFirstAbortValid == false then
		Persist.windyday.specialCaseFirstAbortValid = true
		Analytics.sendEvent("GWI", "FailureSpecialCase", "SingleHatFadedForever", 0)
	end
end

AnalyticsManager.showComplete = function(self)
	showIsPlaying = false
	
	-- increment the number of times the show has been completed, but don't update the custom dimension
	-- updating it now would retroactively update all of the hits in the session so far
	Persist.windyday.episodesCompleted = Persist.windyday.episodesCompleted + 1

	Analytics.setCustomDimension(ANALYTICS_REASON_FOR_ATTRITION, "ShowComplete")
	self.setAllHitDimensions()

	local timeInShow = math.floor((Time.milliseconds() - startTime) / 1000)
	Analytics.sendEvent("GWO", "ShowComplete", "", timeInShow)
end

AnalyticsManager.onUserLeaveHint = function(self)
	unmatchedOnUserLeaveHint = true
end

AnalyticsManager.userLeft = function(self)

	if showIsPlaying == true then 
		-- if user has hinted a leave, it is implied that the next onstop was triggered by the user
		if unmatchedOnUserLeaveHint == true then
			unmatchedOnUserLeaveHint = false
			Analytics.setCustomDimension(ANALYTICS_REASON_FOR_ATTRITION, "UserLeft")
		else
			Analytics.setCustomDimension(ANALYTICS_REASON_FOR_ATTRITION, "UserInterrupted")
		end

		self.setAllHitDimensions()

		local timeInShow = math.floor((Time.milliseconds() - startTime) / 1000)
		Analytics.sendEvent("Attrition", lastObservedScene or "", mangleString(AnalyticsManager:getTimeBinSceneBased()), timeInShow)
	end
end


-- these metrics need reworking by DSA.  These either need to be bucketed... or something, but they need to decide how to measure this
AnalyticsManager.screenEvent = function(self)
	-- TODO
end

AnalyticsManager.volumeUp = function(self)
	-- TODO
end

AnalyticsManager.volumeDown = function(self)
	-- TODO
end

AnalyticsManager.headsetEvent = function(self)
	-- TODO
end

AnalyticsManager.bluetoothEvent = function(self)
	-- TODO
end

AnalyticsManager.getAppType = function(packageName)
    if (packageName) then
        for i, appType in pairs(analyticsAppTypes) do
            if string.find(packageName, appType.appName) then
                return appType.appType or ""
            end
        end
    end
    return "APP_TYPE_OTHER"
end
