
require "common/scripts/moxie/moxie"
require "common/scripts/moxie/shell"

require "windyday/scripts/android/android"
require "windyday/scripts/global"
require "windyday/scripts/prefetch"
require "windyday/scripts/windyday"
require "windyday/scripts/endofshow"

-- Launcher:
require "windyday/scripts/launcher/launcher"

GWIEnabled = true
if (GWIEnabled) then
    Shell.start("launcher_start")
else
	Global.init()
    Global.initShow()
    Global.prefetchCompleted()
    Global.scene.mainCamera.controller:enableTouchControls(true)
    Android.acquireScreen()
    InputManager:enableSensors(true)
    Android.setAlphaRange(1.0, 1.0)
    Global.clearColor:set(1.0, 1.0, 1.0, 1.0)
    Global.startShow()
    if (Context.getCheckpoint() == nil) then
        AnimationManager:next()
        AnimationManager:next()
    end
    Shell.start("windyday")
end

Application.onRotation = function(deviceRotation)
	Analytics.sendEvent("Device", "Rotation", "", deviceRotation)
	Shell.dispatch("onRotation", {deviceRotation});
end

Application.onInterceptTouch = function(windowX, windowY, x, y, eventTime, onTSB)
	-- dispatch to current state
	-- TOOD:  analytics needs hooks for the interpreted event, as sending all touch events will be too much
	return (true == Shell.dispatch("onInterceptTouch", {windowX=windowX, windowY=windowY, x=x, y=y, eventTime=eventTime, onTSB=onTSB}));
end

Application.onPPStateChange = function(window)
	-- dispatch to current state
	Shell.dispatch("onPPStateChange", {window=window});
end

Application.onPPLocSizeChanged = function(appBecameFullScreen, window)
	-- dispatch to current state
	Shell.dispatch("onPPLocSizeChanged", {appBecameFullScreen=appBecameFullScreen, window=window});
end

Application.onSystemUIVizUpdate = function(newVisibility)
    Context.onSystemUIVizUpdate(newVisibility)
end

Application.onActivityEvent = function(eventString)
	-- debug only: exit doodle on dummy activity destruction
	if (eventString == "onStop") then
		Global.shutdown = true
	end

    -- dispatch to current state
    Shell.dispatch("onActivityEvent", {eventString=eventString});

    -- update analytics
	if (eventString == "onStop") then
		AnalyticsManager:userLeft()
	elseif (eventString == "onUserLeaveHint") then
		AnalyticsManager.onUserLeaveHint()
	elseif (eventString == "onNotificationCanceled") then
		Context.onNotificationCanceled()
	end
end

Application.onChangeDisplaySize = function(screenWidth, screenHeight, availWidth, availHeight)
	Log.message("Change display size "..screenWidth..", "..screenHeight..", "..availWidth..", "..availHeight);

	Global.screen_size.x = screenWidth;
	Global.screen_size.y = screenHeight;
	Global.avail_size.x = availWidth;
	Global.avail_size.y = availHeight;

	-- default clip rect to full screen
    Global:setRenderOverTSB(true)

	-- dispatch to current state
	Analytics.sendEvent("Device", "DisplaySizeChange", "{width:"..screenWidth.." height:"..screenHeight.." availableWidth:"..availWidth.." availableHeight:"..availHeight.."}", 0)
	Shell.dispatch("onChangeDisplaySize", {screenWidth=screenWidth, screenHeight=screenHeight, availWidth=availWidth, availHeight=availHeight});
end

Application.onVolumeUp = function(newVolume)
	Log.message("Volume up "..newVolume);

	-- dispatch to current state
	AnalyticsManager:volumeUp()
	Shell.dispatch("onVolumeUp", {newVolume=newVolume});
end

Application.onVolumeDown = function(newVolume)
	Log.message("Volume down "..newVolume);

	-- dispatch to current state
	AnalyticsManager:volumeDown()
	Shell.dispatch("onVolumeDown", {newVolume=newVolume});
end

Application.onScreenEvent = function(newState)
	Log.message("Screen event "..newState);

	-- dispatch to current state
	AnalyticsManager:screenEvent()
	Shell.dispatch("onScreenEvent", {newState=newState});
end

Application.onLocaleChanged = function()
	-- dispatch to current state
	Shell.dispatch("onLocaleChanged", {});
end

Application.onBatteryEvent = function(newState)
	Log.message("Battery event "..newState);

	-- dispatch to current state
	Shell.dispatch("onBatteryEvent", {newState=newState});
end
	
Application.onHeadsetEvent = function(newState)
	Log.message("Headset event "..newState);

	-- dispatch to current state
	AnalyticsManager:headsetEvent()
	Shell.dispatch("onHeadsetEvent", {newState=newState});
end

Application.onBluetoothEvent = function(newState)
	Log.message("Bluetooth event "..newState);

	-- dispatch to current state
	AnalyticsManager:bluetoothEvent()
	Shell.dispatch("onBluetoothEvent", {newState=newState});
end

Application.onKeyPress = function(keyCode, action, isLongPress, downTime)
	Log.message("Keypress event "..keyCode..", "..action..", "..downTime);

	-- dispatch to current state
	Shell.dispatch("onKeyPress", {keyCode=keyCode, action=action, isLongPress=isLongPress, downTime=downTime});
end

Application.onTouchDown = function(pointerId, x, y, eventTime, touchPhase)
	Log.message("Touch down "..pointerId..", "..x..", "..y..", "..eventTime..", "..touchPhase);

	-- dispatch to current state
	AnalyticsManager:recordTouchDown()
	Shell.dispatch("onTouchDown", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase});
	
    Events.send("app", "touchDown", Vector(x, y, 0, 0));
    
	return true;
end

Application.onTouchUp = function(pointerId, x, y, eventTime, touchPhase)
	Log.message("Touch up "..pointerId..", "..x..", "..y..", "..eventTime..", "..touchPhase);

	-- dispatch to current state
	AnalyticsManager:recordTouchUp()
	Shell.dispatch("onTouchUp", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase});
	
    Events.send("app", "touchUp", Vector(x,y, 0, 0));
    
	return true;
end

Application.onTouchMove = function(pointerId, x, y, eventTime, touchPhase)
	Log.message("Touch move "..pointerId..", "..x..", "..y..", "..eventTime..", "..touchPhase);

	-- dispatch to current state
	AnalyticsManager:recordMoveEvent()
	Shell.dispatch("onTouchMove", {pointerId=pointerId, x=x, y=y, eventTime=eventTime, touchPhase=touchPhase});
	
    Events.send("app", "drag", Vector(x, y, 0, 0));
    
	return true;
end

Application.onRequestExit = function()
    Logger:message("Exit requested")
    return true
end

Application.onTerminate = function()
	Shell.onShutdown();
	Global.onShutdown();
end