
local Logger = TaggedLog("SecondaryAudioPlayer", false)

SecondaryAudioPlayer = {}

SecondaryAudioPlayer.config = function(trigger)
    -- Add a SecondaryAudioPlayer to the trigger actor if we have data for it
    local config = secondaryaudio[trigger.name]
    if (config) then
        Logger:message("Adding SecondaryAudioPlayer to " .. trigger.name)
        trigger:addComponent("SecondaryAudioPlayer", SecondaryAudioPlayer:new(config))
    else
        Log.warning("Unable to find SecondaryAudio configuration data for " .. trigger.name)
    end
end

function SecondaryAudioPlayer:new(data)
    local this = Util.copy(data)
    setmetatable(this, self)
    self.__index = self

    this.playing = false
    this.volumeLevel = 0
    this.volumeGoal = 0
    this.startDelay = 0

    return this
end

function SecondaryAudioPlayer:onStart()
    Logger:message("starting " .. self.object.name)
    self.startDelay = self.startFrameDelay / 24

    Events.listen(self, "lookingAt")
    Events.listen(self, "notLookingAt")
end

function SecondaryAudioPlayer:onUpdate()
    self.startDelay = self.startDelay - Time.dt()
    if (not self.playing and self.startDelay < 0) then
        self:playSound()
    end

    if (self.playing) then
        self.volumeLevel = math.lerp(self.volumeLevel, self.volumeGoal, Time.dt() * 5)
        self.sound.volumeControl:setVolume(self.volumeLevel)
    end
end

function SecondaryAudioPlayer:onDestroy()
    if (self.sound) then
        SoundManager:releaseSound(self.sound)
        self.sound = nil
    end
    Events.stopListening(self, "lookingAt")
    Events.stopListening(self, "notLookingAt")

    Logger:message("destroying " .. self.object.name)
end

function SecondaryAudioPlayer:playSound()
    local sound = SoundManager:getSound(self.soundName)
    if (not sound) then
        Log.error("Unable to get sound for SecondaryAudioPlayer: " .. self.soundName)
        return
    end
    local volumeControl = sound.volumeControl

    sound:setLooping(self.looping)
    volumeControl:setGlobalVolumeType("Ambient")
    self.volumeLevel = self.volumeGoal
    volumeControl:setVolume(self.volumeLevel)
    volumeControl.autorelease = false

    sound:play()
    self.sound = sound
    self.playing = true
end

function SecondaryAudioPlayer:lookingAt(name)
    if (self.object.name == name) then
        self.volumeGoal = self.maxVolume or 1
        Logger:message("Looking at " .. name .. ", setting volume goal to " .. self.volumeGoal)
    end
end

function SecondaryAudioPlayer:notLookingAt(name)
    if (self.object.name == name) then
        self.volumeGoal = 0
        Logger:message("Not looking at " .. name .. ", setting volume goal to " .. self.volumeGoal)
    end
end
