
local Logger = TaggedLog("SoundFXPlayer", false)

SoundFXPlayer = {
    disableSFX = false,
}

function SoundFXPlayer:onStart()
    Logger:message("onStart")

    self.currentSegment = nil
    self.currentIndex = 1
    self.timesToPlay = nil

    Events.listen(self, "segmentPlaying")
    local startSegment = AnimationManager:currentSegmentName()
    if (startSegment ~= "") then
        self:segmentPlaying(startSegment)
    end
end

function SoundFXPlayer:onDestroy()
    Events.stopListening(self, "segmentPlaying")
end

function SoundFXPlayer:onUpdate()
    if ((not self.disableSFX) and self.currentSegment and self.currentIndex <= #self.timesToPlay) then
        self.segmentTimer = self.segmentTimer + Time.dt()

        if (self.segmentTimer > self.timesToPlay[self.currentIndex]) then
            local sound = self.currentSegment.sounds[self.currentIndex]
            SoundManager:playSFX(sound.name, sound.volume, sound.looping)
            self.currentIndex = self.currentIndex + 1
        end
    end
end

function SoundFXPlayer:onDestroy()
    Events.stopListening(self, "segmentPlaying")
end

function SoundFXPlayer:segmentPlaying(segmentName)
    Logger:message("segment playing: " .. segmentName)
    self:playSegmentSounds(segmentName)
end

function SoundFXPlayer:playSegmentSounds(segmentName)
    self.currentSegment = nil
    for _, segment in ipairs(soundfx) do
        if (segment.name == segmentName) then
            self.currentSegment = segment
            break
        end
    end
    if (self.currentSegment == nil) then
        Logger:error("Didn't find soundfx for " .. segmentName)
        return
    end

    self.segmentTimer = 0
    self.currentIndex = 1
    self.timesToPlay = {}
    for i, sound in ipairs(self.currentSegment.sounds) do
        local startTime = self.currentSegment.segmentStartFrame
        local playTime = sound.frame
        self.timesToPlay[i] = (playTime - startTime) / 24
    end
end
