
local Logger = TaggedLog("VolumeControl", false)

VolumeControl = {
    currentVolume = 0,
    globalVolume = 0,
    volumeType = "SFX",
    fading = false,
    startTimer = 0,
    fadeTimer = 1.1,
    fadeTime = 1,

    volumeStart = 0,
    volumeEnd = 0,
    time = 0,
    autorelease = true,
    timePlaying = 0,
}

function VolumeControl:new(sound)
    local this = { sound = sound }
    setmetatable(this, self)
    self.__index = self

    this.globalVolume = SoundManager.globalVolumes[self.volumeType]
    this.sound:setVolume(self.currentVolume * self.globalVolume)

    return this
end

function VolumeControl:onUpdate()
    self:updateTiming()
    if (self.fading) then
        self.startTimer = self.startTimer + self.lastDeltaTime
        if (self.startTimer > 0) then
            if (not self.sound:isPlaying()) then
                self.timePlaying = 0
                self.sound:play()
                if (self.volumeType == "Music") then
                    SoundManager.currentMusicBed = self.sound
                    Logger:message("Setting current music bed to: " .. self.sound.name)
                end
            end
            self.fadeTimer = self.fadeTimer + (self.lastDeltaTime / self.fadeTime)
            self.currentVolume = math.lerp(self.volumeStart, self.volumeEnd, self.fadeTimer)
            if (self.fadeTimer > 1) then
                self.fading = false
                if (self.currentVolume < 0.001 and self.autorelease) then
                    Logger:message("autoreleasing fading sound: " .. self.sound.name)
                    self.sound:stop()
                    self:releaseSound()
                    return
                end
            end
        end
    elseif (not self.sound:isPlaying() and self.autorelease) then
        Logger:message("autoreleasing sound: " .. self.sound.name)
        self:releaseSound()
        return
    end

    self.globalVolume = SoundManager.globalVolumes[self.volumeType]
    self.sound:setVolume(self.currentVolume * self.globalVolume)
end

function VolumeControl:forceFade()
    self.fading = false
    if (self.fadeTimer < 1) then
        self.fadeTimer = 1.1
        self.currentVolume = self.volumeEnd
    end
end

function VolumeControl:setTimedFade(startDelay, newFadeTime, newFadeStart, newFadeEnd)
    self.fading = true
    self.startTimer = -startDelay
    self.fadeTimer = 0
    if (newFadeTime < 0.01) then
        self.fadeTimer = 0.99
        self.fadeTime = 1
    else
        self.fadeTime = newFadeTime;
    end

    self.volumeStart = newFadeStart;
    self.volumeEnd = newFadeEnd;
end

function VolumeControl:setVolume(newVolume)
    if (self.fading) then Log.error("VolumeControl: don't set the volume on a fading volume control: " .. self.sound.name .. tostring(debug.traceback())) end
    self.currentVolume = newVolume
end

function VolumeControl:setGlobalVolumeType(volumeType)
    self.volumeType = volumeType
    if (volumeType == "Music") then
        self:setVolume(0)
    else
        self:setVolume(1)
    end
end

function VolumeControl:releaseSound()
    Logger:message("releasing sound: " .. tostring(self.sound.name))
    SoundManager:releaseSound(self.sound)
end

-- To make up for the unreliable results from sound:getSeekPosition(),
-- we try to give a better estimate here.
function VolumeControl:getSeekPosition()
    if (self.timePlaying) then
        return self.timePlaying % self.sound:getDurationSeconds()
    end
    return 0
end

function VolumeControl:updateTiming()
    local updateTime = Time.seconds()
    self.lastUpdateTime = self.lastUpdateTime or updateTime
    self.lastDeltaTime = updateTime - self.lastUpdateTime
    self.lastUpdateTime = updateTime

    if (self.sound:isPlaying()) then
        self.timePlaying = self.timePlaying + self.lastDeltaTime
    end
end
