
local LogDebug = false
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end

-- Handle playing all the animations (including idles) for a given actor
AnimationDriver = {}
AnimationDriver.__index = AnimationDriver

function AnimationDriver.new()
    local this = {
        segmentName = "",
        idleNotification = false,
        mainAnim = nil,
        idleAnim = nil,
        playing = nil,
    }
    return setmetatable(this, AnimationDriver)
end

local function idleSegmentName(segmentName)
    return segmentName .. "_Idle"
end

function AnimationDriver:init(segmentName, animations, notifyOnIdle)
    self:stop();
    self.segmentName = segmentName
    self.idleNotification = notifyOnIdle
    self.mainAnim = animations[segmentName]
    self.idleAnim = animations[idleSegmentName(segmentName)]

    return self.mainAnim
end

function AnimationDriver:onUpdate()
    if (self.playing) then
        self:playIdleIfNeeded()
    end
end

function AnimationDriver:play(startTime)
    self.playing = self.mainAnim
    self.playing:play()
    if (startTime) then
        -- Have to start slightly before the end otherwise
        -- it will just clamp it back to 0
        self.playing:setTime(startTime - 0.001)
    end
end

function AnimationDriver:stop()
    if (self.playing) then
        self.playing:stop()
    end
    self.playing = nil
end

function AnimationDriver:isFinished(checkLooping)
    return Util.isAnimFinished(self.playing, checkLooping)
end

function AnimationDriver:getLoopCount()
    local result = 0
    if (self.idleAnim) then
        result = self.idleAnim:getTime() / self.idleAnim:getDuration()
    end
    return result;
end

function AnimationDriver:playIdleIfNeeded()
    if (self.idleAnim and not self.idleAnim:isPlaying() and Util.isAnimFinished(self.mainAnim, false)) then
        self:stop()
        self.playing = self.idleAnim
        self.playing:play()
        if (self.idleNotification) then
            Events.send(self, "idlePlaying", idleSegmentName(self.segmentName))
        end
    end
end
