
local LogDebug = false
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end

-- Utility that converts a bone data table {actor = name, bone = path} into a bone from the actormanager
local function convertBoneDataToBone(table, datafield)
    if (table[datafield]) then
        table[datafield] = ActorManager:getBone(table[datafield])
    end
end

BlackholeAlpha = {
    initialColor = Color.white,
}

function BlackholeAlpha:new(data)
    local this = shallowcopy(data) or {}
    setmetatable(this, self)
    self.__index = self

    return this
end

function BlackholeAlpha:onStart()
    -- Convert the bone data into bone actor objects
    convertBoneDataToBone(self, "rootBone")
    convertBoneDataToBone(self, "alphaDriver")
    convertBoneDataToBone(self, "colorDriver")


    if (self.material) then
        self.material = self.object.materials[self.material]
        self.initialColor = self.material:getParameters().u_Color
    end
    local fixedScale =  Vector.new(self.fixedScale, self.fixedScale, self.fixedScale)
    if (self.rootBone.setLocalScale) then
        self.rootBone:setLocalScale(fixedScale)
    end
    --if (self.name) then
    --    Log.message(" ********* "..self.name.." initial color = "..tostring(self.initialColor))
    --end
end

function BlackholeAlpha:onUpdate()
    local newColor = self.initialColor:copy()

    if (self.colorDriver) then
        newColor.r = self.colorDriver:getWorld().position.y
        newColor.g = self.colorDriver:getWorld().position.y
        newColor.b = self.colorDriver:getWorld().position.y
    end
    if (self.alphaDriver) then
        if (self.scaleInitialAlpha) then
            newColor.a = self.initialColor.a * self.alphaDriver:getWorld().position.y
        else
            newColor.a = self.alphaDriver:getWorld().position.y
        end
    end

    if (self.material) then
        self.material:setParameter("u_Color", newColor)
    end

    if (self.rotSpeed ~= 0) then

        local transform = self.rootBone:getLocal()
        if (transform) then
            transform.rotation = transform.rotation * Quaternion.fromEulerAngles(Vector(0.0,self.rotSpeed,0.0) * Time.dt())
            self.rootBone:setLocal(transform)
        end
    end

    --if (self.name) then
    --    Log.message(" ********* "..self.name.." set to " .. tostring(self.material:getParameters().u_Color))
    --end
end
