
require "common/scripts/moxie/events"
require "windyday/scripts/orientationremapper"

CameraController  =
{
	touchAnchor	= nil;
	tapThreshold = 10;
	tapTimeThreshold = 0.5;
	rotationScaleX = 0.14;
	currentRotationScaleX = 0.14;
	rotationScaleY = 0.13;
	rotY = 15;
	rotX = 0.0;
	quatY = Quaternion.new();
	quatX =  Quaternion.new();
	rotZ = 0.0;
	northOffset 	= 0.0;
	fov    = 30;
	smoothCamera        = true;
	cameraSpeed         = 16;
	boomLength			= 1.75;
	cameraOrbitSquashY	= 0.5;
	rotationSource		= nil;
	saveRotationSource	= nil;
	touchControls    	= false;
	constrainedTouch 	= true;
	rotationOffset 		= Vector.zero:copy();
	rotationFinal 		= Vector.zero:copy();
	touchStartTime 		= 0.0;
	touchTime 			= 0.0;
	cameraCrane 		= Vector.zero:copy();
	nearClipPlane		= 0.1;
	rotGoalLast			= nil;
	camLocalT			= nil;
	calib 				= nil;
	lastRot				= nil;
	lastUpdateSecs		= nil;
	touchDelta			= nil;
	dragPosition		= nil;
	
	offsetDone			= false;
	setNewHeading		= false;
	currentHeading		= 0;
	startYOffset		= 0;
	startYOffsetQuat	= Quaternion.new();
	lastYawQuat			= Quaternion.new();
	lastYaw				= 0;
	
	yClampStart			= nil;
	yClampFinal			= 0;
	lastYClampFinal		= 0;
	headingClamp		= Quaternion.new();
	hitQuad				= {x=0, y=0, width=0, height=0};
	hitQuadSize			= .875;
	
	
	-- x rotation limit
	softClamp			= true;
	remapper			= shallowcopy(OrientationRemapper);
	
	parent				= nil;
	
	-- good way in stuff
	gwiLock				= false;
	gwiRotResetTime		= 0.0;
	gwiRotOffset		= Vector.zero:copy();
	gwiRotSave			= Vector.zero:copy();
	
	-- input checking
	isDown				= false;
	isUp				= true;
}

CameraController.new = function(self, _rotationSource)
	o = {};
	setmetatable(o, self);
	--o.rotationSource = _rotationSource;
	self.__index = self;
	return o;
end

function CameraController:onStart()
	Events.listen(self, "sceneStarted")
	
	local size = math.clamp01(self.hitQuadSize)
	
	local aspectX = Global.screen_size.x/Global.screen_size.y;
	local aspectY = Global.screen_size.y/Global.screen_size.x;
	
	self.hitQuad = {
		xMin=Global.screen_size.x*(1 - size),
		yMin=Global.screen_size.y*(1 - size),
		xMax=Global.screen_size.x*(size),
		yMax=Global.screen_size.y*(size)
	};

	self.parent = Global.scene.cameraRig;

	if (Platform.win32 or Platform.mac)then
		self.constrainedTouch = false;
	end
	
    self.calib = Quaternion.new();
    if (Platform.win32 or Platform.mac) then
   		self.calib:setAxisAngle(Vector.right, math.rad(0))
    elseif(Platform.android or Platform.iOS ) then
   		self.calib:setAxisAngle(Vector.right, math.rad(-90))
    end
    
	--Screen.sleepTimeout = SleepTimeout.NeverSleep;
    self.camLocalT 		= self.object:getLocal();
    self.rotationFinal  = InputManager:getSensorRotation();--self.rotationSource:getRotation();
    self.rotationFinal  = self.rotationFinal:getEulerAngles();
    
    Events.listen(self, "setSmoothCamera");
    Events.listen(self, "setTouchControls");
    Events.listen(self, "setConstrainedTouchControls");
    self:enableTouchControls(false);
    
    self:setInitialRotation();
    
    if(GWIenabled) then
    	self.softClamp = true;
    	self.gwiLock = true;
    	local localT = self.object:getLocal();
    	self.gwiRotSave = ((InputManager:getSensorRotation()):getEulerAngles()) - localT.rotation:getEulerAngles();
    	self:enableTouchControls(false);
    else
    	self:enableTouchControls(true);
    end
end

function CameraController:enableSensor()
	if(self.gwiLock) then
		self:enableTouchControls(true);
		self.gwiLock = false;
	end
	self:lookAtHeading(self.startYOffset)
end

function CameraController:setTouchControls(touch)
    self:enableTouchControls(touch);
end

function CameraController:setConstrainedTouchControls(constrained)
    self.constrainedTouch = constrained;
end

function CameraController:enableTouchControls(enabled)
    self.touchControls = enabled;
    
    if (self.touchControls) then
        Events.listen(self, "touchDown");
    	Events.listen(self, "touchUp");
    	Events.listen(self, "drag");
    else
        Events.stopListening(self, "touchDown");
        Events.stopListening(self, "touchUp");
        Events.stopListening(self, "drag");
    end
end

function CameraController:setInitialRotation()
 	local localT = self.object:getLocal();
    --yield WaitForSeconds(0.25);
    self.rotY = 0.0;
    self.rotationOffset = Vector.zero:copy();
end

function CameraController:rotateCameraBy(angles) 
    self.rotX = self.rotX + angles.x
    self.rotY = self.rotY + angles.y
    self.rotZ = self.rotZ + angles.z
end

function CameraController:resetRotation() 
-- TODO Reimplement
    self.rotX = 0.0
    self.rotY = 0.0
    self.rotZ = 0.0
    self.rotationOffset = Vector.zero
end

function CameraController:getGyroRotationFinal()
	return self.rotationFinal; 
end

function CameraController:touchDown(touchPosition)
	if(self.isDown) then
	Log.message("Touch: UP NOT RECVD");
		self.isUp = true;
		self:resetInput(touchPosition);
	end
	
	self.isDown = true;

    self.touchAnchor = touchPosition;
    self.touchStartTime = os.time();
    
    if(touchPosition.x < self.hitQuad.xMax and touchPosition.y < self.hitQuad.yMax
    	and touchPosition.x > self.hitQuad.xMin and touchPosition.y > self.hitQuad.yMin ) then
    	self.lockHeading = true;
    end

	self.isUp = false;
end

function CameraController:resetInput(touchPosition)
	if (self.touchAnchor == nil) then return end

    self.touchTime = os.time() - self.touchStartTime;
    
    local toAnchor = (self.touchAnchor - touchPosition);
    
--    local tapped = (self.touchTime < self.tapTimeThreshold) and toAnchor:length() < self.tapThreshold;
--
--    if (tapped) then
--        self.rotY = 0.0;
--        self.rotationOffset = Vector(0,0,0);
--    end
	
	self.lastYClampFinal = self.yClampFinal;
	self.dragPosition = nil;
	self.yClampStart = nil;
    self.lockHeading = false;
    self.dragPosition = nil;
    self.isDown = false;
end

function CameraController:touchUp(touchPosition)
	self.isUp = true;
	self:resetInput(touchPosition);
end

function CameraController:drag(touchDelta)
    
    if(not self.dragPosition) then
    	self.dragPosition = Vector(touchDelta.x, touchDelta.y)
    end
    
    local pos = Vector(touchDelta.x, touchDelta.y);
    
    self.touchDelta = Vector(0.5*(pos.x - self.dragPosition.x), 0.5*(pos.y - self.dragPosition.y), 0.0, 0.0);
    
	self.dragPosition = pos;
    
    local localT = self.object:getLocal()
    local rollAngle, rollQuat = localT.rotation:getRoll()
    self.touchDelta = self.touchDelta:rotateBy(rollQuat:inverse()) 
    
    --update rotation
	self.rotY = self.rotY + self.touchDelta.y;
	self.rotX = self.rotX - self.touchDelta.x;
	self.quatX = Quaternion.axisAngle(Vector.right, self.rotY * self.currentRotationScaleX)
	self.quatY = Quaternion.axisAngle(Vector.up, -self.rotX * self.rotationScaleY)
end

function CameraController:setSmoothCamera(notification)
    self.smoothCamera = notification;
end

function CameraController:lookAt(objT) 
-- TODO Reimplement
--	local cameraLook = Vector(0.0, 0.0, -1.0);
--	cameraLook = cameraLook:rotateBy(self.object:getRotation())
--	local right = Vector(1.0, 0.0, 0.0);
--	right = right:rotateBy(self.object:getRotation())
--	local newLook = (Vector(objT.position.x, 0.0, objT.position.z) - Vector(self.position.x, 0.0, self.position.z)):normalized();
--	
--	local lookDotLook = Vector.dot(cameraLook, newLook);
--	local rightDotLook = Vector.dot(right, newLook);
--	
--	local angleY = 0.0;
--	if(lookDotLook < 0.99 and lookDotLook > -0.99)then
--		angleY = -math.sign(rightDotLook)*math.acos(lookDotLook)*57.2957795;
--	end
--	
--	self.rotX = self.rotX + (angleY/self.rotationScaleY);
end

function CameraController:lookAtHeading(heading)
	self.lastYaw, self.lastYawQuat = self.object:getLocal().rotation:getYaw()
	self.currentHeading = heading
	self.setNewHeading = true
	self.northOffset = self.currentHeading - self.lastYaw;
	self.startYOffsetQuat = Quaternion.axisAngle(Vector.up, self.northOffset);
end

function CameraController:_getHeadingClamp(yaw)
    if(not self.yClampStart) then
    	self.yClampStart = yaw - self.lastYClampFinal;
    end
    
    self.yClampFinal = yaw - self.yClampStart
	
	return Quaternion.axisAngle(Vector.up, self.yClampFinal):inverse();
end

function CameraController:onUpdate()

	local rotSource = InputManager:getSensorRotation()
   	local localT = self.object:getLocal();
   	local lastRotation = localT.rotation;
   	
	self.lastYaw, self.lastYawQuat = rotSource:getYaw()
	
	if(self.lockHeading) then
		self.headingClamp = self:_getHeadingClamp(self.lastYaw)
	end
	
	if (self.constrainedTouch) then
		self.rotY = 0
		self.quatX = Quaternion.new()
	end
	
	if(self.softClamp and (AnimationManager.startSegment == "HatCycle")) then
		--localT.rotation = self:applySoftClamp(rotSource:copy(), rotSourceAngles)
		localT.rotation = self.remapper:remapOrientation(rotSource:copy())
		if( not self.remapper.isCorrecting ) then
			self.softClamp = false
		end
	else
		localT.rotation = rotSource:copy()
	end
	
	-- combine all offsets into final rotation
	localT.rotation = self.startYOffsetQuat*self.quatY*self.quatX*self.headingClamp*localT.rotation

	-- calculate time independently 
	local currTimeSecs = Time.seconds();
	if(self.lastUpdateSecs == nil) then
		self.lastUpdateSecs = currTimeSecs;
	end
	
	local dt = currTimeSecs - self.lastUpdateSecs;
	
	-- Slerp towards goal
	if (not self.softClamp and self.smoothCamera and self.cameraSpeed ~= 0) then
       	localT.rotation = Quaternion.slerp(dt * self.cameraSpeed, lastRotation, localT.rotation );
    end
    
	local camLook = Vector(0.0, 0.0, -1.0, 0);
	camLook = camLook:rotateBy(localT.rotation)
	camLook.w = 0
	camLook:normalize()
	
   	-- Create a new forward vector that is squashed on Y in an ellipsoid
    local forward = Vector(
    						-- X
    						camLook.x,
    
    						-- Y ( solve for Y in: x^2 + (y/0.5)^2 + z^2 = 1.0 ; y = 0.5*( sqrt(1.0 - x^2 - z^2) ))
    						math.sign(camLook.y)*self.cameraOrbitSquashY*
    						math.sqrt(math.clamp(1.0 - camLook.x*camLook.x - 
    						camLook.z*camLook.z, 0.0, 5.6e300)),
    						
    						-- Z
    						camLook.z);
    
    -- add position back in now that rotation has been applied
   	localT.position = self.cameraCrane + forward*self.boomLength
   	self.object:setLocal(localT)
end

function CameraController:applySoftClamp(rotation, angles)
	
	local camLook = Vector(0.0, 0.0, -1.0, 0);
	camLook = camLook:rotateBy(rotation)
	camLook:normalize()
	local worldDown = Vector(0,-1,0,0)
	local worldRight = Vector(1,0,0,0)
	
	local angleX = math.deg(Vector.angle(worldDown, camLook)) - 90
	angleX = angleX
	
	if(angleX < 0.0 ) then
		
		local lerpedAngle		= math.lerp(0, 1, -(angleX)/90)
		local correctionAngle 	= math.clamp(lerpedAngle*60, 0, -(angleX))
		
		local correctionAxis = Vector.cross(worldDown, camLook)
		correctionAxis:normalize()
		
		--Renderer.drawText(10, 310, textStyle, string.format("crctnAxis: <%.2f,%.2f,%.2f>", correctionAxis.x, correctionAxis.y, correctionAxis.z))
		--Renderer.drawText(10, 340, textStyle, string.format("X:%.3f", angleX))
		
		local correctQuat 	= Quaternion.axisAngle(correctionAxis, correctionAngle)
		return Quaternion.shortestDistance(correctQuat*rotation, rotation)
		
	else
		self.softClamp = false
		return rotation
	end
	
end
