

-- at a minimum, a component is just a data container.
DebugCameraController = 
{
	linearVelocity = Vector.new(0,0,0),		-- linear velocity
	angularVelocity = Vector.new(0,0,0),	-- angular velocity
	moveSpeed = 10.0,						-- units per second
	step 	  = 5.0,
	turnSpeed = math.rad(50),							
	lookSpeed = math.rad(50),
	linearDamp = 0.95,
	angularDamp = 0.95,
	sensitivity = 60,
	frame = 0,
	blink = 1.0;
	lastUpdateSecs		= nil;
}

-- components can also act as behaviors
function DebugCameraController:onStart() 
	self.active = false
end

-- demonstrates the use of the Input library to query buttons
function DebugCameraController:onUpdate()

	if(Input.isKeyDown(KB_Q))then
		self.moveSpeed = math.max(self.moveSpeed + self.step, 10.0);
		self.lookSpeed = math.min(math.rad(100), self.lookSpeed*1.25);
		self.turnSpeed = math.min(math.rad(100), self.turnSpeed*1.25);
	end
	if(Input.isKeyDown(KB_E))then
		self.moveSpeed = math.max(self.moveSpeed - self.step, 10.0);
		self.lookSpeed = math.max(0.1, self.lookSpeed*0.75);
		self.turnSpeed = math.max(0.1, self.turnSpeed*0.75);
	end
	
	if((self.frame % 240) == 0) then
		textStyle = { color = Color.new(math.clamp01(self.blink), 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = 1.0 }
		self.blink = self.blink*-1
	end
	self.frame = self.frame +1;
	Renderer.drawText(0, 130, textStyle, string.format("<DEBUG_CAMERA> Speed:%.1f", self.moveSpeed));
		
	local linearAcc = Vector(0,0,0);
	
	if (Input.isKeyDown(KB_W)) then
		linearAcc.z = linearAcc.z - self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_S)) then
		linearAcc.z = linearAcc.z + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_A)) then
		linearAcc.x = linearAcc.x - self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_D)) then
		linearAcc.x = linearAcc.x + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_Z)) then
		linearAcc.y = linearAcc.y + self.moveSpeed;
	end
	
	if (Input.isKeyDown(KB_X)) then
		linearAcc.y = linearAcc.z - self.moveSpeed;
	end
	
	local currTimeSecs = Time.seconds()
	if(self.lastUpdateSecs == nil) then
		self.lastUpdateSecs = currTimeSecs;
	end
	
	local dt = currTimeSecs - self.lastUpdateSecs;
	self.lastUpdateSecs = currTimeSecs;
	
	self.linearVelocity = self.linearVelocity + linearAcc * 2.0 * dt;
	self.linearVelocity.x = math.min(math.max(self.linearVelocity.x, -self.moveSpeed*dt), self.moveSpeed*dt);
	self.linearVelocity.y = math.min(math.max(self.linearVelocity.y, -self.moveSpeed*dt), self.moveSpeed*dt);
	self.linearVelocity.z = math.min(math.max(self.linearVelocity.z, -self.moveSpeed*dt), self.moveSpeed*dt);
	
	local angularAcc = Vector(0,0,0);
	if(Input.isMouseMoving() and Input.getMouseButton(MB_L_BUTTON)) then
		local mouseDelta = Input.getMouseDelta() * self.sensitivity;
		local vp = Renderer.getViewport();
		angularAcc.x = -(mouseDelta.y/vp.height) * self.lookSpeed;
		angularAcc.y = -(mouseDelta.x/vp.width) * self.turnSpeed;
	end
	
	self.angularVelocity = self.angularVelocity + angularAcc * 2.0 * dt;
	self.angularVelocity.x = math.min(math.max(self.angularVelocity.x, -self.lookSpeed*dt), self.lookSpeed*dt);
	self.angularVelocity.y = math.min(math.max(self.angularVelocity.y, -self.turnSpeed*dt), self.turnSpeed*dt);
	
	local localT = self.object:getLocal();
	
	local y = Quaternion.new(); 
	local p = Quaternion.new(); 
		
	y:setAxisAngle(Vector.up, self.angularVelocity.y);
	p:setAxisAngle(Vector.right:rotateBy(localT.rotation), self.angularVelocity.x);
	
	local deltaRot = p * y;
	
	localT.position = localT.position + self.linearVelocity:rotateBy(localT.rotation);
	localT:applyRotation(deltaRot);
	
	self.object:setLocal(localT);

	-- apply damping
	self.linearVelocity = self.linearVelocity * self.linearDamp;
	self.angularVelocity = self.angularVelocity * self.angularDamp;
end
