
require "common/scripts/moxie/moxie"

require "windyday/scripts/util"

local LogDebug = true
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end


DrivenParticles = {
    emitter         = nil,

    emitEnabled     = true,
    prevEmitRate    = 1,

    driverEmitOn    = nil,
    driverEmitRate  = nil,
}

function DrivenParticles:new(data)
    local this = shallowcopy(data) or {}
    setmetatable(this, self)
    self.__index = self
    return this
end

function DrivenParticles:onStart()
    if(not self.emitter) then
        Log.message( "Starting Driven Particles : "..self.emitterName )

        if (self.emitDriver ~= nil) then
            self.driverEmitOn   = ActorManager:getBone(self.emitDriver)
        end
        if (self.rateDriver ~= nil) then
            self.driverEmitRate = ActorManager:getBone(self.rateDriver)
        end

        self.emitter = Global.scene:createParticleEmitter( self.emitterName, "windyday:particles/"..self.particleSystemSpec..".mxp" )
        --Log.message( "     ********** Intial Emit Rate "..self.emitterName.." "..self.emitter:getEmitRate())


        local transform = SRT.new( Vector(0.0,0.0,0.0), Quaternion.identity, Vector.one )
        if (self.emitterPos) then
            transform.position = self.emitterPos:copy()
        end
        if (self.emitterRot) then
            transform.rotation = self.emitterRot:copy()
        end
        if (self.emitterScale) then
            transform.scale = self.emitterScale:copy()
        end
        self.emitter:setLocal( transform )

        -- Add the emitter as a child of this component
        if(self.parented) then
            self.object:addChild(self.emitter)
        end
    end
end

function DrivenParticles:onUpdate()
    -- Cosine curve to multiply in for observation
    -- local debugScalar = ((math.cos(Time.seconds() * 3.14 ) * 0.5) + 0.51) * 500.0

    if (self.emitter) then
        if (self.driverEmitOn and self.driverEmitOn.getWorld) then
            local worldTransform = self.driverEmitOn:getWorld()
            if (worldTransform) then
                if (worldTransform.position.y < 0.5 and self.emitEnabled == true) then
                    if (self.emitter.getEmitRate) then
                        self.prevEmitRate = self.emitter:getEmitRate()
                        --if (self.emitterName) then
                        --    Log.message("     ********** Switching OFF "..self.emitterName.." "..self.prevEmitRate)
                        --end
                    end
                    self.emitter:setEmitRate(0)
                    self.emitEnabled = false
                elseif (worldTransform.position.y > 0.5 and self.emitEnabled == false) then
                    --if (self.emitterName) then
                    --    Log.message("     ********** Switching ON "..self.emitterName.." "..self.prevEmitRate)
                    --end
                    self.emitter:setEmitRate(self.prevEmitRate)
                    self.emitEnabled = true
                end
            end
        end

        if (self.driverEmitRate and self.driverEmitRate.getWorld and self.emitEnabled == true) then
            self.emitter:setEmitRate(self.driverEmitRate:getWorld().position.y)
        end
    end

--    local transform = self.object:getLocal()
--    if (transform) then
--        local eulerRot = transform.rotation:getEulerAngles()
--        Log.message( "     ********** EulerRot ("..eulerRot.x.." , "..eulerRot.y.." , "..eulerRot.z..") ")
--    end

--    local transform = self.emitter:getWorld()
--    if (self.driverEmitOn) then
--        local transform = self.driverEmitOn:getWorld()
--        if (transform and self.emitterName and self.emitter) then
--            Log.message( "     ********** WorldPos "..self.emitterName.." enabled "..self.emitEnabled.." ("..transform.position.y..") rate "..self.emitter:getEmitRate())
--        end
--    end

--    local systemSpec = def.particleSystems.tornadoSpecksLater1
--
--    local transform = self.emitter:getLocal()
--    if (transform) then
--        transform.position.z = systemSpec.emitters[1].transform.position.z + debugScalar
--        Log.message( "     ********** DebugScalar "..debugScalar )
--        self.emitter:setLocal(transform)
--    end

end

function DrivenParticles:onDestroy()
    if(not self.parented and self.emitter) then
        self.emitter:destroy()
        self.emitter = nil
    end
end
