
local Logger = TaggedLog("EasterEgg", false)

EasterEgg =
{
-- public
	actorName = "",
	animationName = "",
	target = nil,
	actor = nil,
	removeOnLookAway = false,
	looping = false,
	useTriggerPos = false,
	position = Vector(),
	scale = Vector.one:copy(),
	rotation = Quaternion.new(),
	
-- private
	homePos = Vector.new(),
	homeRot = Quaternion.new(),
	homeScale = Vector.new(),
	triggered = false,
	characterRoot = nil,
	
 -- audio fields
 	soundName = nil,
	startFrameDelay = 0,

	startDelay = 0,
    sound = nil,
    volumeLevel = 0,
    volumeGoal = 1,
}

EasterEgg.config = function(trigger)
    local config = eastereggs[trigger.name]
    if (config) then
        Logger:message("Adding EasterEgg to " .. trigger.name)
        trigger:addComponent("EasterEgg", EasterEgg:new(config))
    else
        Log.warning("Unable to find EasterEgg configuration data for " .. trigger.name)
    end
end

EasterEgg.new = function(self, config)
	local o = config or {};
	setmetatable(o, self);
	self.__index = self;
	return o;
end

function EasterEgg:onStart()

    Logger:message("set up for " .. self.actorName)

	if (self.target == nil) then
		self.target = self.object.components["lookatdetector"]
	end
	
	self.actor = ActorManager:getActor(self.actorName, self.animationName);
	
	self.actor:setVisibility(true)
	
	if (self.looping) then
		self.actor.animation[self.animationName].play()
	end
	
	local actorT = self.actor:getLocal()
	self.homePos = actorT.position:copy();
	self.homeRot = actorT.rotation:copy();
	self.homeScale = actorT.scale:copy();
		
	if (self.useTriggerPos) then
		
--		local localT = self.object:getLocal()
--		actorT.position = localT.position;
--		actorT.rotation = localT.rotation;
--		actorT.scale 	= localT.scale;
		self.object:setChild(self.actor)
		
	else
		local actorT = self.actor:getLocal()
		actorT.position 	= self.position:copy();
		actorT.rotation 	= self.rotation:copy();
		actorT.scale 		= self.scale:copy();
		self.actor:setLocal(actorT)
	end
	
	-- Audio setup
	self.startDelay = self.startFrameDelay/24.0;
	self.sound = nil

	Events.listen(self, "lookingAt")
	Events.listen(self, "notLookingAt")
end

function EasterEgg:onUpdate()
    if (self.sound and self.sound:isPlaying()) then
        self.volumeLevel = math.lerp(self.volumeLevel, self.volumeGoal, Time.dt() * 5)
        self.sound.volumeControl:setVolume(self.volumeLevel)
    end
end

function EasterEgg:onDestroy()
	if (Global.scene.actors[self.actorName] and self.actor and self.actor:isVisible()) then
		self.actor:setVisibility(false);
		if (self.useTriggerPos) then
--			local actorT = self.actor.getLocal()
--			self.actor.position 	= self.homePos:copy();
--			self.actor.rotation 	= self.homeRot:copy();
--			self.actor.scale 		= self.homeScale:copy();
			self.object:removeChild(self.actor)
		else
			local actorT = self.actor:getLocal()
			actorT.position 	= self.homePos:copy();
			actorT.rotation 	= self.homeRot:copy();
			actorT.scale 		= self.homeScale:copy();
			self.actor:setLocal(actorT)
		end
	end

    if (self.sound) then
        SoundManager:releaseSound(self.sound)
        self.sound = nil
    end

    Events.stopListening(self, "lookingAt")
    Events.stopListening(self, "notLookingAt")
end

function EasterEgg:lookingAt(name)
    if (name ~= self.object.name) then return end
    self.volumeGoal = 0.7
	if (not self.triggered and self.target and self.target.isTriggered) then
		if ((Platform.android  or Platform.iOS)  and self.soundName) then
            DelayScheduler:runAfter(self.startDelay, function() self:playSound() end, self.soundName)
            Logger:message("starting audio with delay: " .. self.startDelay)
        end
		self.triggered = true;
        Logger:message("easter egg triggered: " .. self.actorName)
		if (not self.looping) then
            self.actor.animation[self.animationName]:play()
        end
	end
end

function EasterEgg:notLookingAt(name)
    if (name ~= self.object.name) then return end
    self.volumeGoal = 0

	if(self.removeOnLookAway and self.triggered and self.target and not self.target.isTriggered) then
		self.actor:setVisibility(false)
	end
end

function EasterEgg:playSound()
    Logger:message("playing sound: " .. self.soundName)
    local sound = SoundManager:getSound(self.soundName)
    if (sound) then
        local volumeControl = sound.volumeControl
        self.volumeLevel = self.volumeGoal
        volumeControl:setGlobalVolumeType("Ambient")
        volumeControl:setVolume(self.volumeLevel)
        volumeControl.autorelease = false
        sound:play()
        self.sound = sound
    end
end
