

def.actors.Iris = { 
    model = "windyday:models/iris.mxm",
	animation = {
	    },
	materials = { 
		{ name = "Mat_Iris", shader = "windyday:shaders/unlit.rsfx", technique = "transparent_with_color_clamped", renderGroup = 5, 
		textures = { u_MainTex = "windyday:textures/iris.dds", }, parameters = { u_Color = Color.new( 1.0, 1.0, 1.0, 1.0 ), u_MainTex_ST = Vector.new(1, 1, 0, -0.45), }, },
	}
}

IrisController = {}
IrisController.__index = IrisController;

IrisController.new = function()	


	local self = {
		lookAtTarget = nil,
		lookAtBlend = 0,
		currentState = 0,
		
		irisStage1Range = Vector.new(0.0, -0.45, 0, 0),
		irisStage1Time = 2,
		irisStage1Delay = 2.4,
		
		irisStage2Range = Vector.new(-0.45, -0.5, 0, 0),
		irisStage2Time = .95,
		irisStage2Delay = 0,
		
		irisTimer = 0,
		testTimer = 0,
		closed = false,
		
		activity = { open = "open", stage2 = "stage2", stage3 = "stage3", },
		currentActivity = "open",
		calib = Quaternion.new(),
		rotStart = Quaternion.new(),
	}
	
	return setmetatable(self, IrisController);
end

IrisController.onStart = function(self)
	self.lookAtTarget = ActorManager:getActor("IrisTarget");
	if (not self.lookAtTarget) then 
		Log.message ("ERROR: Iris cant find look at target");
	end
	
	-- rotate by 180
	self.calib:setAxisAngle(Vector.up, math.rad(180))
	
	local localT = self.object:getLocal();
	localT.position = Global.scene.mainCamera:getWorld().position;
	
	local lookAt = Vector(-LookAtTargets.IrisTarget.position[1], 1, -LookAtTargets.IrisTarget.position[3]);
	lookAt:normalize();
	self.lookAtRotation = Quaternion.lookAt(lookAt:normalized());
	
	localT.rotation = self.calib*self.lookAtRotation;
	self.object:setLocal(localT);
	
	Events.listen(self, "segmentPlaying");
end

IrisController.segmentPlaying = function(self, segmentName)

	if (segmentName == "End6A") then
		self:irisStage1Close();
	elseif (segmentName == "End6B") then
		self:irisStage2Close();
	elseif( not self.closed) then
		self:openIris();
	end
end

IrisController.irisStage1Close = function(self)
		
	if (self.currentActivity == self.activity.stage1) then
		return
	end
	
	--self.active = true;
	self.rotStart = Global.scene.mainCamera:getWorld().rotation;
	self.object:setRotation(self.rotStart);
	self.object:setVisibility(true)
	self.currentActivity = self.activity.stage1;
	self.irisTimer = -self.irisStage1Delay / self.irisStage1Time;
	self.lookAtBlend = 0

end
	
IrisController.irisStage2Close = function(self)
	
	if (self.currentActivity == self.activity.stage2) then
		return
	end
	self.rotStart = Global.scene.mainCamera:getWorld().rotation;
	self.object:setRotation(self.rotStart);
	self.object:setVisibility(true)
	self.currentActivity = self.activity.stage2;
	self.irisTimer = -self.irisStage2Delay / self.irisStage2Time;
	self.closed = true;
	self.lookAtBlend = 1

end
	
IrisController.openIris = function(self)
	if (self.currentActivity == self.activity.open) then
		return
	end
	self.object:setVisibility(false)
	self.currentActivity = self.activity.open;
end


IrisController.onUpdate = function(self)

	local localT = self.object:getLocal()
	local camWorldT = Global.scene.mainCamera:getWorld();
	
	localT.position = camWorldT.position
		
	if ( self.currentActivity == self.activity.open ) then
				
			localT.rotation = Quaternion.lerp(self.lookAtBlend, camWorldT.rotation, self.lookAtRotation);
			
			local texOffset = Vector(1, 1, 0, 0.0);
			self.object.materials["Mat_Iris"]:setParameters({u_MainTex_ST = texOffset});
			self.lookAtBlend = 0;
	elseif ( self.currentActivity == self.activity.stage1) then
			self.testTimer = self.testTimer + Time.dt()
			localT.rotation = Quaternion.lerp(self.lookAtBlend, camWorldT.rotation, self.lookAtRotation);
			
			self.irisTimer = self.irisTimer + (Time.dt() / self.irisStage1Time);
			if(self.irisTimer > 1) then
				self.irisTimer = 1
			end
			
			local lerpAmount = math.easeInOutSine(math.clamp01(self.irisTimer));
			local offsetValue = math.lerp(self.irisStage1Range.x, self.irisStage1Range.y, lerpAmount);
			self.object.materials["Mat_Iris"]:setParameters({u_MainTex_ST = Vector( 1, 1, 0, offsetValue)});
			self.lookAtBlend =  math.clamp01(self.irisTimer);
			
	elseif ( self.currentActivity == self.activity.stage2) then
	
			localT.rotation = Quaternion.lerp(self.lookAtBlend, camWorldT.rotation, self.lookAtRotation);
			
			self.irisTimer = self.irisTimer + (Time.dt()/ self.irisStage2Time)
			if(self.irisTimer > 1) then
				if (Windyday.endOfShow == false) then 
					Windyday.endOfShow = true 
				end
			end
			
			local lerpAmount = math.easeInOutSine(math.clamp01(self.irisTimer));
			local offsetValue = math.lerp(self.irisStage2Range.x, self.irisStage2Range.y, lerpAmount);
			self.object.materials["Mat_Iris"]:setParameters({u_MainTex_ST = Vector( 1, 1, 0, offsetValue)});
			
			local fadeAmount = math.clamp(self.irisTimer - 3, 0, 1);
			local irisColor = Vector.lerp(fadeAmount, Vector(1,1,1,1), Vector(1,1,1,0));
			self.object.materials["Mat_Iris"]:setParameters({u_Color = irisColor});
			self.lookAtBlend = 1;
	end
	
	localT.rotation = self.calib*localT.rotation
	self.object:setLocal(localT);
end

