
LaunchHatMover = {}
LaunchHatMover.__index = LaunchHatMover;

LaunchHatMover.new = function()	
	local self = {
		waiting = true,
		ready = false,
        numFramesMoving = 0,
        startDelay = 0,
		transitionTime = 3,
		transitionTimer = 0,
		startPosition = Vector.new(),
		startObject = nil,
		startTransform = nil,
		target = nil,
		targetTransform = "",
		matName = "Mat_HatTexture",
		rotTimer = -1,
	}
	return setmetatable(self, LaunchHatMover);
end

LaunchHatMover.segmentPlaying = function(self, segmentName)
    if (segmentName == "HatCycle" or segmentName == "HatIntro") then return end

    self.object:setVisibility(false)
    self.object:setActive(false)

    Events.stopListening(self, "segmentPlaying")
    self.object:destroy()
end

LaunchHatMover.isReady = function(self)
	return self.ready
end

LaunchHatMover.onStart = function(self)
	Log.error("HatMover onStart")

	local hat = ActorManager:getActor("Hat", "HatCycle")
	hat.animation.HatCycle:play()
	Events.listen(self, "segmentPlaying")
end

LaunchHatMover.startIntro = function(self, startDelay)
	self.waiting = true
	self.startObject = Global.scene.actors.TSBRig
	self.startTransform = "HatPosition|RootNode"
	self.target = Global.scene.actors.TSBRig;
	self.targetTransform = "HatPosition|RootNode";
	self.startDelay = startDelay
end

LaunchHatMover.gotoStartOfShow = function(self)
	Log.error("HatMover gotoStartOfShow")
	self.waiting = false
	Global.scene.actors.Hat.animation.HatCycle:setPaused(true)
	Global.scene.actors.Hat.animation.HatCycle:setTime(0.0)

	self.startPosition = self.object:getLocal().position

    self.startObject = Global.scene.actors.TSBRig
	self.startTransform = "HatPosition|RootNode"
	
	self.target = Global.scene.actors.Hat or ActorManager:getActor("Hat", "HatCycle")
	self.targetTransform = "hat_mox_cn_root|hat_MOXIE_skeleton|RootNode"

	self.object.animation.GWIIdleShort:crossfade(1, false)
end

LaunchHatMover.startShow = function(self)
	Log.error("HatMover startShow")
	self.ready = true

	self.object.animation.HatIntroLocal:crossfade(1, false)
	local realHat = Global.scene.actors.Hat
	realHat.animation.HatCycle:setPaused(false)
	AnimationManager:next()
end

LaunchHatMover.onLateUpdate = function(self)
	
	-- wait til the start delay runs out
	if (self.waiting) then
		self.startDelay = self.startDelay - Time.dt()
		if (self.startDelay < 0) then
			self:gotoStartOfShow() 
		end
		-- if we are still waiting then don't do anything else
		return
	end

    if (self.numFramesMoving > self.transitionTime) then
        -- Render group change must happen after the white field has faded and only needs to happen once
        -- So we clumsily count a few frames here and then switch.
        self.object.materials["Mat_HatTexture"]:setRenderGroup(2);
    else
        self.numFramesMoving = self.numFramesMoving + 1;
    end

	-- get all the positions to interpolate between
	local localT = self.object:getLocal();
	
	local targetT = self.target:getBoneWorld(self.targetTransform);
	local startT = self.startObject:getBoneWorld(self.startTransform);
	
	-- do all this positioning 'n stuff
	self.transitionTimer = self.transitionTimer + Time.dt() / self.transitionTime
	--Log.message(self.transitionTimer)
	local lerpAmount = math.easeInOutSine(math.clamp(self.transitionTimer, 0, 1))
	localT.position = Vector.lerp(lerpAmount, startT.position, targetT.position)
	self.object:setPosition(localT.position);
	
	-- set the hat to rotate to world up, in case the user is being a jerk
	local camTRot = Global.scene.mainCamera:getWorld().rotation;
	local rotLerpAmount = math.clamp(self.transitionTimer * self.transitionTime, 0, 1)
	localT.rotation = Quaternion.lerp(rotLerpAmount, camTRot, Quaternion.identity)
	self.object:setRotation(localT.rotation);		
	
	if (not self.ready and self.transitionTimer > 1.0) then 
		self:startShow()
	end
end





