

LookAtDetector = 
{
	worldPosT = nil,
	isTriggered = false,
	camera = nil,
	--lookAtBone = "mouse_mox_cn_pelvis|mouse_MOXIE_skeleton|RootNode",
	lookAtThreshold = {}, -- the close to 1 this value is the more directly you have to look at it to trigger it
	lookedAt = false,
	debugname = nil,

	toTarget = nil,
	useBoxCollider = false,
	colliderRadius = 0.0,
	colliderWidth = 0.0,
	colliderHeight = 0.0,

	dotDifferential = 0.0,
}

LookAtDetector.basenew = function(self)
	local o = {};
	setmetatable(o, self);
	self.__index = self;
	return o;
end

LookAtDetector.new = function(self, _camera, _debugname)
	if _camera == nil then
		error("\nLookAtDetector requires a camera\n", 2);
	end

	local o = {};
	--return setmetatable({ camera = _camera or nil, worldPosT = nil,isTriggered = false, yRot = LookAtDetector.new() or nil}, self);
	setmetatable(o, self);
	o.camera = _camera;
	o.debugname = _debugname;
	self.__index = self;

	return o;
end
	
LookAtDetector.onStart = function(self)
end

LookAtDetector.getTransform = function(self)
	return self.worldPosT;
end

LookAtDetector.onUpdate = function(self)
	local debugString = "";	

	local posT = self:getTransform();
	if(posT == nil) then
		posT = self.object:getWorld();
	end
	
	-- Don't use full camera pos because it leads to parallax error (since the cam is on a boom)
	-- Do account for the camera height however, so get the rig
	if (Global.scene.cameraRig == nil) then
		return
	end
	local toTarget = posT.position - Global.scene.cameraRig:getWorld().position;

	-- If the camera to target vector has changed then something moved, recalculate or goal value
	if (self.toTarget == nil or self.toTarget ~= toTarget) then
		self.toTarget = toTarget:copy();
		self.lookAtThreshold = self:calculateGoal();
	end

	toTarget:normalize();
	
	local look = self.camera:getWorld():forward();
	
	-- debugString = debugString..string.format(" : toTarget <%6.4f, %6.4f, %6.4f>", self.toTarget.x, self.toTarget.y, self.toTarget.z);
	-- debugString = debugString..string.format(" : lookAtThreshold %f", self.lookAtThreshold[1] );
	-- Log.message(debugString);
	Global.debugString = debugString;

	self.dotDifferential = Vector.dot(look, toTarget);
	local dotDifferential = self.dotDifferential;

	if(dotDifferential >= self.lookAtThreshold[1] and self.isTriggered ~= true) then
		self.isTriggered = true;
		Events.send(self.object, "lookingAt", self.debugname);
		self.lookedAt = true;
		self.object:setDrawDebug(true);
	elseif (dotDifferential < self.lookAtThreshold[1] and self.isTriggered ~= false) then
		self.isTriggered = false;
		Events.send(self.object, "notLookingAt", self.debugname);
		self.lookedAt = false;
		self.object:setDrawDebug(false);
	end
end

LookAtDetector.calculateGoal = function(self)
	-- This is more complicated than is likely necessary
	-- Calculate the minimum result of our dot product for the location and radius
	-- originally generated in the protype.

	local targetLen	= self.toTarget:length();

	local maxAngle	= {}
	if (self.useBoxCollider) then
		maxAngle[1] = math.atan2( self.colliderWidth * 0.5, targetLen );
		maxAngle[2] = math.atan2( self.colliderHeight * 0.5, targetLen );
	else
		maxAngle[1] = math.atan2( self.colliderRadius, targetLen );
	end

	-- Minimum dot-product we should need in our transition check
	local dotGoal = {}
    for i = 1, #maxAngle do
		dotGoal[i] = math.cos(maxAngle[i]);
	end

	return dotGoal;
end

LookAtDetector.getThreshold = function(self)
	return (self and self.lookAtThreshold[1]) or ""
end

LookAtDetector.debugDot = function(self)
	return self.dotDifferential;
end
