
local Logger = TaggedLog("MainActorsDetector", false)

MainActorsDetector = {
    -- Bones of actors that need to be on screen
    actorBoneData = {
        { actor = "Mouse", bone="mouse_mox_cn_pelvis|mouse_MOXIE_skeleton|RootNode" },
        { actor = "Hat", bone="hat_mox_cn_root|hat_MOXIE_skeleton|RootNode" },
        { actor = "LaunchHat", bone="hat_mox_cn_root|hat_MOXIE_skeleton|RootNode" },
    },

    thresholdAngle = 30,
}

function MainActorsDetector:anyOnScreen()
    for _, boneData in ipairs(self.actorBoneData) do
        if (self:onScreen(boneData)) then
            return true
        end
    end
    return false
end

function MainActorsDetector:onScreen(boneData)
    local actor = Global.scene.actors[boneData.actor]
    if (Global.scene.cameraRig == nil or actor == nil or not actor:isVisible()) then
        return false
    end

    -- Simple angle check between the bone and the camera
    local bonePos = actor:getBoneWorld(boneData.bone).position
    local cameraPos = Global.scene.cameraRig:getWorld().position
	
    local cameraToBone = bonePos - cameraPos
	cameraToBone:normalize();
	
    local cameraLook = Global.scene.mainCamera:getWorld():forward()
    local angle = 0.0
    if (cameraToBone and cameraLook) then
        angle = math.deg(Vector.angle(cameraToBone, cameraLook))
    end

    return math.abs(angle) <= self.thresholdAngle
end
