

TargetData = {
		nameOfPosTarget = nil; -- the name of the object being lit
		nameOfMatTarget = nil; -- then name of the object containing the shared material that needs to be updated/lit
		currentDistanceSqrd = math.floatMax;
		
		lightTargetPos = nil; -- reference to transform of the target (the object being lit)
		lightTargetMat = nil; -- reference to shared material of the object being lit
}

local hashIndex = 1;

OmniLightProbe = {
	-- triggers refresh of links (editor only)
	refreshLinks = false;
	lightTargetData = {}; 		-- list of targets
	lightTargetLookUpMap = {}; 	-- hash to target mapping to quickly locate target that this probe is lighting
	cutOffDist = 30.0;
	cutOffDistSqrd = 0.0;
	lightProbeColor = Vector.one;
	
	currentTarget = nil; -- local cache of current target
	radiusSqrd = 0.0;
}

OmniLightProbe.new = function(self, data)
	local o = nil
	if(data ~= nil) then
		o = data;
	else
		o = {};
	end
	setmetatable(o, self);
	self.__index = self;
	return o;
end

function OmniLightProbe:onStart()

	--Events.listen(self, "onSetVisibilityChanged")
	Events.listen(self, "onActorDestroyed")

	self.cutOffDistSqrd = self.cutOffDist*self.cutOffDist;
	
	self.radiusSqrd = self.scale*2.0;
	
	local localT = self.object:getLocal()
	localT.position = self.position
	self.object:setLocal(localT)
	
	
	-- create quick look up map
	for _, td in ipairs(self.lightTargetData) do
		if(td ~= nil) then
			td.hashcode = hashIndex;
			td.hashname = "id_"..hashIndex;
			hashIndex = hashIndex + 1;
			self.lightTargetLookUpMap[td.hashcode] = td;	
		end
		
		td.lightTargetPos = ActorManager:getBone({actor=td.targetActorName, bone = td.targetBone})
		
		td.lightTargetMat = { 
			name = "material_"..td.targetActorName,
			actorName = td.targetActorName,
			setParameters = function(self, paramTable)
				local actor = ActorManager:getActor(self.actorName)
				for name, mat in pairs(actor.materials) do
					mat:setParameters(paramTable)
				end
			end,
		}
		
	end
	
	
end

function OmniLightProbe:onUpdate ()
	self:onLateUpdate()
end

function OmniLightProbe:onSetVisibilityChanged(name, isVisible)
	if(name == self.setName and not isVisible and not self.isDestroyed) then
		Log.message("Actor: "..name.." Destroying OmniLightProbe")
		self.object:destroy()
		self.isDestroyed = true
	end
end

function OmniLightProbe:onActorDestroyed(name, isVisible)
	if(name == self.setName and not self.isDestroyed) then
		Log.message("Actor: "..name.." Destroying OmniLightProbe")
		self.object:destroy()
		self.isDestroyed = true
		Events.stopListening(self, "onActorDestroyed")
	end
end

function OmniLightProbe:onLateUpdate()

	 self.currentTarget = nil;
	
	--get active target
	 for _, targetData in ipairs(self.lightTargetData) do
		
		 if(self.active and targetData ~= nil)then
			
			 if(targetData.lightTargetPos ~= nil) then
				 self.currentTarget = targetData.lightTargetPos;
				 local localT = self.object:getLocal()
				 local targerPos = self.currentTarget:getWorld().position
				 targetData.currentDistanceSqrd = Vector.lengthSq(localT.position - targerPos);
					
				 -- if target is within lighting range add it to the blend manager
				 if(targetData.currentDistanceSqrd < self.cutOffDist * self.cutOffDist) then
					 OmniLightBlendManager:addProbe(targetData, self);
				 end
			 end
		 end
	 end
	
end

function OmniLightProbe:getInvertedLightProbeColor()
	return Vector.one - self.lightProbeColor;
end

function OmniLightProbe:findTargetInThisProbe (targetData)
	return self.lightTargetLookUpMap[targetData.hashcode];
end
