

ScreenFader = {}
ScreenFader.__index = ScreenFader;

ScreenFader.new = function(matName)	

	local self = {
		matName = matName,
		material = nil,
		destroyAtEnd = false,
		
		timer = 0,
		speed = 2,
		idleColor = Vector(1.0,1.0,1.0,1.0),
		fadeOutColor = Vector(1.0,1.0,1.0,0.0),
		startColor = Vector(1.0,1.0,1.0,0.0),
		endColor = Vector(1.0,1.0,1.0,1.0),
	}
	
	return setmetatable(self, ScreenFader);
end

ScreenFader.onStart = function(self)
	self.material = self.object:findMaterial(self.matName)
end

ScreenFader.onUpdate = function(self)

	if (self.timer < 1) then
		self.timer = self.timer + (Time.dt() * self.speed)
		local amount = math.easeInOutSine(math.clamp(self.timer, 0, 1))
		local matColor = Vector.lerp(amount, self.startColor, self.endColor)
		self.material:setParameters({ u_Color = matColor})
		if (self.timer > 1 and self.destroyAtEnd) then 
			local launchHat = Global.scene.actors.LaunchHat
			if (launchHat) then launchHat:setVisibility(true) end
			self.object:destroy()
		end
	end
end

ScreenFader.setDestroyAtEnd = function(self)
	self.destroyAtEnd = true
end

ScreenFader.fadeIn = function(self, delayTime)
	self.timer = -delayTime * self.speed
	self.startColor = self.fadeOutColor
	self.endColor = self.idleColor
end

ScreenFader.fadeOut = function(self, delayTime)
	self.timer = -delayTime * self.speed
	self.startColor = self.idleColor
	self.endColor = self.fadeOutColor
end
