

Shadow = {}

function Shadow:new(owner, shadowName, targetBone)
    local this = {}
	this.debug = false
	this.targetBone = targetBone
	this.baseScale = 1
	this.shadowName = shadowName
	this.shadow = ActorManager:getActor(shadowName)
	local actor = ActorManager:getActor(owner)
	
	actor:addComponent(shadowName, this)
	this.fadeRange = Vector (0.5, 2)
	
    setmetatable(this, self)
    self.__index = self
    return this
end

function Shadow:onDestroy()
	if(self.shadow) then
		ActorManager:destroyActor(self.shadowName)
		self.shadow = nil
	end
end

function Shadow:onUpdate()
	if (not self.targetBone or not self.shadow or not ActorManager.activeActors[self.shadowName]) then return end
	local fadeValue = 0
	local xFadeValue = 0
	local zFadeValue = 0
	local shadowScale = Vector(0, 0, 0)
	
	local boneTransform = self.targetBone:getWorld()
	local targetWorldPos = boneTransform.position
	local targetWorldY = 0

    if (targetWorldPos.y < 0) then
        -- target is below ground so hide the shadow
        self.shadow:setVisibility(false)
        return
    else
        self.shadow:setVisibility   (true)
    end

    local dropProxy = ActorManager.activeActors.DropProxy
    if (dropProxy) then
        targetWorldY = dropProxy:getBoneWorld("setDriver|RootNode").position.y
    end

	fadeValue = math.inverseLerp(self.fadeRange.x, self.fadeRange.y, targetWorldPos.y)
	
	if (self.xFadeRange) then
		xFadeValue = math.inverseLerp(self.xFadeRange.x, self.xFadeRange.y, targetWorldPos.x)
	end
	if (self.zFadeRange) then 
		zFadeValue = math.inverseLerp(self.zFadeRange.x, self.zFadeRange.y, targetWorldPos.z)
	end
	
	local currentColor = Vector(1, 1, 1, 1 - (fadeValue + xFadeValue + zFadeValue))
	
	self.shadow.materials["Mat_Shadow"]:setParameters({ u_Color = currentColor})
	
	local shadowTransform = self.shadow:getLocal()
	
	if (self.baseScale) then 
		local scaleValue = self.baseScale * (1 -  (fadeValue * 0.5))
		shadowTransform.scale = Vector(scaleValue, scaleValue, scaleValue)
	end
	shadowTransform.position = Vector(targetWorldPos.x, targetWorldY, targetWorldPos.z)
	self.shadow:setLocal(shadowTransform)
	
	if (self.debug) then
		Log.error(targetWorldPos.x);
	end
end


function Shadow:setScale(newScale) 
	self.baseScale = newScale
end

function Shadow:setAttenuation(attenMin, attenMax) 
	self.fadeRange = Vector(attenMin, attenMax)
end

function Shadow:setXFade(xMin, xMax) 
	self.xFadeRange = Vector(xMin, xMax);
end

function Shadow:setZFade(zMin, zMax) 
	self.zFadeRange = Vector(zMin, zMax);
end

function Shadow:setDebug() 
	self.debug = true;
end
