
def.actors.SunGlow = { 
    model = "windyday:models/screenquad.mxm",
	animation = {
	    },
	materials = { 
		{ 
			name = "Mat_Quad", shader = "windyday:shaders/unlit.rsfx", technique = "sunglow", renderGroup = 7, 
			textures = {},
			parameters = { u_Color = Color.new( 59/255, 45/255, 30/255, 168/255 ) }
		},
	}
}

SunGlow = {
	camera,
	horizon = Color.new( 0, 0, 0, 0 ),
	canopy = Color.new( 0, 0, 0, 0 ),
	goalGlowHorizon = Color.new( 0, 0, 0, 0 ),
	goalGlowCanopy = Color.new( 0, 0, 0, 0 ),
	horizonIntensity = 0,
	canopyIntensity = 0,
	goalHorizonIntensity = 0,
	goalCanopyIntensity = 0,
	gKeyDown = false,
	offset = nil,
	worldQuat = Quaternion.new(),
	debug = false,
}
SunGlow.__index = SunGlow;

SunGlow.new = function(offset)
	local self = {}
	self = setmetatable(self, SunGlow);
	self.offset = offset or 0;
	return self
end

function SunGlow:onStart()
	self.camera = Global.scene.mainCamera
	self.object:setVisibility(true)
	
	-- parent holds the position
	local localT = self.object:getLocal()
	localT.position.z = -1
	localT.position.y = 0.5
	localT.scale.x = 10
	self.object:setLocal(localT)
	
	
	if(self.debug) then
		Events.listen(self, "touchDown");
		Events.listen(self, "touchUp");
		Events.listen(self, "drag");
	end
end

function SunGlow:onUpdate() 
		
		-- Toggle effect
--		if (Input.isKeyDown(KB_G)) then
--			self.gKeyDown = true
--		elseif(self.gKeyDown) then
--			self.gKeyDown = false
--    		self.object:setVisibility(not self.object:isVisible())
--	    end
		
		--if(self.debug) then
		--	self:updateColors()
		--end

		local rotation = self.camera:getWorld().rotation
		self.worldQuat = rotation:copy()
		
		--local angles = rotation:getEulerAngles()
		local localT = self.object:getLocal()
		
		local forward = Vector.rotateBy(Vector.forward, rotation)
		forward:normalize()
		local right = Vector.rotateBy(Vector.right, rotation)
		right:normalize()
		local up = Vector.rotateBy(Vector.up, rotation)
		up:normalize()
		--local down = up*-1;
		
		--local start = Vector(0, 0.5, -1)
		
		-- project onto the XY plane
		local xyUp = Vector(Vector.dot(right, Vector.up), Vector.dot(up, Vector.up), 0, 0)
		xyUp:normalize()
		
		-- calculate roll from this isolated case
		local roll = math.deg(Vector.angle(Vector.up, xyUp) or 0)
		
		-- half space test to capture which half of the unit-circle/plane we are in ( quadrant 1 & 2 or quadrant 3 & 4)
		local rollHS = Vector.dot(Vector.right, xyUp*-1)
		
		-- debug text
--		Renderer.drawText(10, 100, textStyle, string.format("roll %.2f", roll));
--		Renderer.drawText(10, 130, textStyle, string.format("rollHS %.2f", rollHS));
		
		-- lerp goal color
		self.horizon 			= Color.lerp(Time.dt(), self.horizon, self.goalGlowHorizon)
		self.canopy  			= Color.lerp(Time.dt(), self.canopy, self.goalGlowCanopy)
		self.horizonIntensity 	= math.lerp(self.horizonIntensity, self.goalHorizonIntensity, Time.dt())
		self.canopyIntensity 	= math.lerp(self.canopyIntensity, self.goalCanopyIntensity, Time.dt())
	
		local intensity = Vector.dot(forward, Vector.up);
		
		-- lerp from horizon to canopy
		local color = Color.lerp(math.max(0, intensity), self.horizon, self.canopy)
		local colorIntensity = math.lerp(self.horizonIntensity, self.canopyIntensity, math.max(0, intensity))
		
		--if(self.debug) then
		--	Renderer.drawText(10, 160, textStyle, string.format("HorizonI: %.5f CanopyI: %.5f CurI %.5f", self.goalHorizonIntensity, self.goalCanopyIntensity, colorIntensity));
		--end
		
		color.r = color.r * colorIntensity
		color.g = color.g * colorIntensity
		color.b = color.b * colorIntensity		
		
		self.object.materials["Mat_Quad"]:setParameters({ u_Color = color});
		
		localT.position = self.camera:getWorld().position + forward + Vector.up*(1.0 - intensity)*0.5
		
		local scale = math.clamp01(intensity) + 1.0
		localT.scale = Vector(10 + scale, scale, scale)
		
		--local x = Quaternion.axisAngle(Vector.right, angles.x + self.offset)
		--local y = Quaternion.axisAngle(Vector.up, angles.y + self.offset)
		
		-- remove roll from rotation
		localT.rotation = rotation*Quaternion.axisAngle(Vector.forward, math.sign(rollHS)*roll):inverse()
		self.object:setLocal(localT)
end

function SunGlow:resetGlow()
	self.color = Color.new( 0, 0, 0, 0 );
	self.intensity = 0;
end


function SunGlow:setGlow(horizon, canopy, horizonIntensity, canopyIntensity)
	self.horizon = self.goalGlowHorizon:copy();
	self.canopy = self.goalGlowCanopy:copy();
	self.horizonIntensity = self.goalHorizonIntensity
	self.canopyIntensity = self.goalCanopyIntensity
	
	if(horizon) then
		self.goalGlowHorizon = horizon:copy()
		self.goalHorizonIntensity = horizonIntensity
	else
		self.goalGlowHorizon = Color.new( 0, 0, 0, 0 );
		self.goalHorizonIntensity = 0
	end
	
	if(canopy) then
		self.goalGlowCanopy= canopy:copy()
		self.goalCanopyIntensity = canopyIntensity
	else
		self.goalGlowCanopy = Color.new( 0, 0, 0, 0 );
		self.goalCanopyIntensity = 0
	end
	
end

SunGlow.touchDelta = nil
SunGlow.touchAnchor = Vector.new()

function SunGlow:setSunGlowEnabled(enabled)
	self.object:setVisibility(enabled)
	self.active = enabled
end


function SunGlow:updateColors()

	local yStep = Global.screen_size.y/4
    local textStyle = { color = Color.new(1.0, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = 1.0 }
    Renderer.drawText(10, yStep*2, textStyle, "---------CANOPY---------------");
    Renderer.drawText(10, yStep*3, textStyle, "---------HORIZON----------------");

	if(self.touchDelta) then
	
		-- right or left
		local dot = Vector.dot(self.touchDelta, Vector.right)
		local length = self.touchDelta:length();
	
		if(self.canopyIntensityAdjust) then
			
			if(dot > 0) then -- right
				self.goalCanopyIntensity  = self.goalCanopyIntensity  + self.touchDelta:length()/Global.screen_size.x
			else -- left
				self.goalCanopyIntensity  = self.goalCanopyIntensity  - self.touchDelta:length()/Global.screen_size.x
			end
			
		elseif(self.horizonIntensityAdjust) then
			if(dot > 0) then -- right
				self.goalHorizonIntensity = self.goalHorizonIntensity + self.touchDelta:length()/Global.screen_size.x
			else -- left
				self.goalHorizonIntensity = self.goalHorizonIntensity - self.touchDelta:length()/Global.screen_size.x
			end
		end
	
	end
	
end

function SunGlow:touchDown(touchPosition)
    --self.touchAnchor = touchPosition;
    --self.touchStartTime = os.time();
    
    self.canopyIntensityAdjust = false
	self.horizonIntensityAdjust = false
    
    -- imaginary sliders
    
    local section = math.floor( (touchPosition.y/Global.screen_size.y)*4 ) + 1	
    
    if(section == 3) then -- intensity
    	if(self.goalCanopyIntensity) then
    		self.canopyIntensityAdjust = true
		end
    end
    if(section == 4) then -- intensity
    	if(self.goalHorizonIntensity) then
    		self.horizonIntensityAdjust = true
		end
    end
    
end

function SunGlow:touchUp(touchPosition)
	--if (self.touchAnchor == nil) then return end
	
	self.touchDelta = nil
	self.canopyIntensityAdjust = false
	self.horizonIntensityAdjust = false
end

function SunGlow:drag(touchDelta)
    local delta = Vector(touchDelta.x, touchDelta.y, 0.0, 0.0);
    if(not self.touchDelta) then
    	self.touchDelta = Vector.new();
    else
    	self.touchDelta = delta:copy()
    	self.touchDelta.w = 0;
    	self.touchDelta.z = 0;
    end
end
