
local Logger = TaggedLog("DelayScheduler", false)

DelayScheduler = {}

local DelaySchedulerGameObj = nil

function DelayScheduler:init(scene)
    if (DelaySchedulerGameObj) then self:shutdown() end
    DelaySchedulerGameObj = scene:createTransform("DelayScheduler")

    if (DelaySchedulerGameObj.components == nil or DelaySchedulerGameObj.components.DelayScheduler == nil) then
        DelaySchedulerGameObj:addComponent("DelayScheduler", DelayScheduler)
    end
    self.pending = {}
    Logger:message("init")
end

function DelayScheduler:shutdown()
    Logger:message("shutdown")
    if (DelaySchedulerGameObj) then
        DelaySchedulerGameObj:destroy()
        DelaySchedulerGameObj = nil
    end
end

function DelayScheduler:onDestroy()
    DelaySchedulerGameObj = nil
end

function DelayScheduler:runAfter(delay, func, name)
    name = name or "<function>"
    if (delay == 0) then
        -- call immediately
        Logger:message("Running " .. name .. " immediately")
        func()
    else
        Logger:message("Queueing " .. name .. " for " .. delay)
        table.insert(self.pending, { func = func, delay = delay, timer = 0, name = name})
    end
end

function DelayScheduler:onUpdate()
    local delta = Time.dt()
    local stillPending = {}
    for _, entry in ipairs(self.pending) do
        entry.timer = entry.timer + delta
        if (entry.timer >= entry.delay) then
            Logger:message("Calling: " .. entry.name .. " after " .. entry.delay .. ", with timer = " .. entry.timer)
            entry.func()
        else
            table.insert(stillPending, entry)
        end
    end
    self.pending = stillPending
end
