
Events = {}

local notifications = {}

Events.listen = function(observer, messageName, ...)
-- If the name isn't good, then throw an error and return.
    if (messageName == nil or messageName == "") then
        Log.error("Null name specified for notification in Listen.");
        return
    end
    -- If this specific notification doens't exist yet, then create it.
    if (notifications[messageName] == nil) then
        notifications[messageName] = {};
    end

    local notifyList = notifications[messageName];

    -- If the list of observers doesn't already contains the one that's registering, then add it.
    if (notifyList[observer] == nil) then
        notifyList[observer] = observer;
    end
end

Events.stopListening = function(observer, messageName)
    if (notifications[messageName] == nil) then return end;

    local notifyList = notifications[messageName];

    -- Assuming that this is a valid notification type, remove the observer from the list.
    if (notifyList ~= nil) then
        notifyList[observer] = nil;
        if (Util.numKeys(notifyList) == 0) then
            notifications[messageName] = nil;
        end
    end
end

Events.send = function(sender, messageName, ...)
-- First make sure that the name of the notification is valid.
    if (messageName == nil or messageName == "") then
        Log.error("Null name sent to Send.");
        return;
    end

    -- Obtain the notification list, and make sure that it is valid as well
    local notifyList = notifications[messageName];

    if (notifyList == nil) then
        return;
    end

    -- Create an array to keep track of invalid observers that we need to remove
    local observersToRemove = {};

    -- Iterate through all the objects that have signed up to be notified by this type of notification.
    for key,value in pairs(notifyList) do
        if (value == null) then
            -- observer is invalid
            observersToRemove[key] = notifyList[key];
        else
            -- observer is valid
            if (type(value[messageName]) == "function") then
                value[messageName](value, ...);
            else
                Log.error("Cannot call "..messageName.." on observer "..tostring(key))
            end
        end
    end

    -- Remove all the invalid observers
    for key,value in pairs(observersToRemove) do
        notifyList[observer] = nil;
    end
end
