
require "common/scripts/moxie/moxie"
require "windyday/scripts/analyticsmanager"

require "windyday/scripts/fixup/fixemups"
require "windyday/scripts/fixup/logger"
require "windyday/scripts/fixup/events"
require "windyday/scripts/util"

-- data
require "windyday/scripts/scenedata/segments"
require "windyday/scripts/scenedata/sets"
require "windyday/scripts/scenedata/actors"
require "windyday/scripts/scenedata/lookattargets"
require "windyday/scripts/scenedata/omnilightprobes"
require "windyday/scripts/scenedata/particles"
require "windyday/scripts/scenedata/music"
require "windyday/scripts/scenedata/soundfx"
require "windyday/scripts/scenedata/secondaryaudio"
require "windyday/scripts/scenedata/eastereggs"

-- components
require "windyday/scripts/components/animatedlookatdetector"
require "windyday/scripts/components/animationdriver"
require "windyday/scripts/components/cameracontroller"
require "windyday/scripts/components/debugcamera"
require "windyday/scripts/components/positiontracker"
require "windyday/scripts/components/omnilightprobe"
require "windyday/scripts/components/motiontracker"
require "windyday/scripts/components/drivenkey"
require "windyday/scripts/components/applerippletextures"
require "windyday/scripts/components/hidebyscale"
require "windyday/scripts/components/screenfader"
require "windyday/scripts/components/iconfader"
require "windyday/scripts/components/hilitefader"
require "windyday/scripts/components/iriscontroller"
require "windyday/scripts/endofshow/hatreturnmover"
require "windyday/scripts/components/fovtest"
require "windyday/scripts/components/clearcolorcontrol"
require "windyday/scripts/components/mousematerialcontrol"
require "windyday/scripts/components/hidebehind"
require "windyday/scripts/components/hatfader"
require "windyday/scripts/components/launchhatmover"
require "windyday/scripts/components/mainactorsdetector"
require "windyday/scripts/components/sunglow"
require "windyday/scripts/components/digdirtparticles"
require "windyday/scripts/components/easteregg"
require "windyday/scripts/audio/secondaryaudioplayer"

-- controls
require "windyday/scripts/segment"
require "windyday/scripts/animationmanager"
require "windyday/scripts/actormanager"
require "windyday/scripts/setmanager"
require "windyday/scripts/leafparticles"
require "windyday/scripts/starparticles"
require "windyday/scripts/inputmanager"
require "windyday/scripts/guimanager"
require "windyday/scripts/omnilightblendmanager"
require "windyday/scripts/osd"
require "windyday/scripts/delayscheduler"
require "windyday/scripts/audio/soundmanager"
require "windyday/scripts/audio/musicplayer"
require "windyday/scripts/audio/soundfxplayer"
require "windyday/scripts/audio/volumecontrol"
require "windyday/scripts/replay"

-- stats
require "windyday/scripts/stats" -- please to not require anything after this file.

-- Ensure the persist tables are set up
Persist = Persist or {}
Persist.windyday = Persist.windyday or {}

--Persist.windyday.context = Persist.windyday.context or {}
--Persist.windyday.context.lastCheckpoint = "DropB"

--local profile = require 'common/scripts/moxie/profiler'

mainRenderTargetDescription =
{
	surfaces =
	{
		{
			attachment	= RTA_COLOR0,
			type		= RTF_TEXTURE2D,
			format		= PF_RGBA8,
		},
		{
			attachment	= RTA_DEPTH_STENCIL,
			type		= RTF_TEXTURE2D,
			format		= PF_DEPTH24_STENCIL8,
		}
	}
}

enum( "RVF_MAIN_VIEW", 1 )

mainView = Renderer.createView( "main" )
mainView:setFlags( RVF_MAIN_VIEW )
mainView:setRenderTarget( mainRenderTargetDescription )
--mainView:setSuperSampleRate( 8 )

Global = {}

-- build version:  major, minor, build
Global.screen_size		= Vector.new(384.0, 640.0, 0.0, 0.0)
Global.avail_size		= Vector.new(384.0, 640.0, 0.0, 0.0)
Global.clearColor		= Color.new(0.0,0.0,0.0,0.0);

-- shutdown flag set upon dummy activity closing
Global.shutdown         = false

if(Platform.candidate) then
	Global.showFPS			= false;
	Global.debugRender		= false;
	Global.renderOverTSB    = true;
end

if(Platform.debug or Platform.release) then
	Global.showFPS			= false;
	Global.debugRender		= true;
	Global.renderOverTSB    = true;
end

if (Platform.branch == "R") then
    Global.debugRender      = false;
    Global.showFPS          = false;
end

Global.storyManager = nil

Global.init = function()
	local startScene = Platform.scene

    if (startScene ~= nil and startScene ~= "") then
        Context.onCheckpoint(startScene)
	end

    if StoryManager then
        Global.storyManager = StoryManager:getFSM("windyday")
    end

    RenderDebug.setEnabled(true)
	
	if (Global.scene == nil) then
        Global.scene = Scene.create();
        Global.scene.cameraRig  = Global.scene:createTransform("cameraRig");
        Global.scene.mainCamera = Global.scene:createCamera("camera001");

        Global.scene.cameraRig:addChild(Global.scene.mainCamera);
        Global.scene.mainCamera:setPerspective( --[[ match long axis to unity's aspect ]] 30.0*(1184/1280.0), 0.3, 1000.0);
        Global.scene.cameraRig:setPosition(Vector.new(0.0, 2.0, 0.0));
        Global.scene.mainCamera.controller = Global.scene.mainCamera:addComponent("cameraControl", CameraController:new());
        if(Platform.win32 or Platform.mac) then
            Global.scene.mainCamera.debugcontroller = Global.scene.mainCamera:addComponent("debugcamera", DebugCameraController);
        end
    end

    Global.resources = System.createResourceGroup( "global" )
    
    mainView:setCamera( Global.scene.mainCamera )

	-- Enable google analytics
	AnalyticsManager.initialize()
	AnalyticsManager.sendInitialStats() 

	InputManager:init(Global.scene)
    DelayScheduler:init(Global.scene)

	if(Platform.debug or Platform.release) then
		GUI.loadTheme();
	end
end

Global.initShow = function()
    -- Prefetch common WindyDay assets
    Global.resources:addShader( "common:shaders/simple.rsfx" )
    Global.resources:addShader( "common:shaders/particle.rsfx" )
    Global.resources:addTexture( "common:textures/missingtexture.dds" )
    Global.resources:addActor( def.actors.Hat, "HatCycle" )
    Global.resources:addActor( def.actors.CameraMan, "HatCycle" )
    Global.resources:addActor( def.actors.SunGlow, "HatCycle" )
    Global.resources:addActor( def.actors.Iris, "HatCycle" )
    Global.resources:addActor( def.actors.Mouse, "HatCycle" )

    -- Add particle textures
    Global.resources:addTexture( "windyday:textures/molehill_particle_dirt.dds" )
    Global.resources:addTexture( "windyday:textures/leaves.dds" )
    Global.resources:addTexture( "windyday:textures/stars.dds" )
    Global.resources:addTexture( "windyday:textures/tornado_particle_greyspecks.dds" )
    Global.resources:addTexture( "windyday:textures/tornado_particle_leaves.dds" )
    Global.resources:addTexture( "windyday:textures/whitefluff.dds" )
    Global.resources:addTexture( "windyday:textures/noise.dds" )

    -- prefetch the first segment
    local checkPoint = Context.getCheckpoint()
    if (checkPoint) then
        AsyncLoader.prefetchForSegment(checkPoint)
    else
        AsyncLoader.prefetchIntroMusic()
        AsyncLoader.prefetch()
    end

    Global.resources:prefetch()

--    AsyncLoader.generateStreamingData()

    -- init show systems
    SoundManager.init(Global.scene)
    ActorManager:init(Global.scene)
    SetManager:init(Global.scene)
    OmniLightBlendManager:init(Global.scene)
end

Global.getUserType = function(self)
	return Config.active().name
end

Global.prefetchCompleted = function()
    -- Initialize global stuff after we have pre-fetched
    local cameraMan = ActorManager:getActor("CameraMan", "HatCycle")
    if (cameraMan) then
        Global.scene.cameraRig:addComponent("cameraManTracker", PositionTracker.new(cameraMan, "CameraPivot|CameraMan|RootNode", false))
    end

    local sunGlow = ActorManager:getActor("SunGlow", "HatCycle")
    Global.scene.sunGlow= sunGlow:addComponent("sunGlow", SunGlow.new())

    local iris = ActorManager:getActor("Iris", "HatCycle")
    iris:addComponent("irisController", IrisController.new())
end

-- playthrough recording
Global.startShow = function(self)
    Global.scene.mainCamera.controller.active = true
	Global.scene.mainCamera.controller:enableTouchControls(true);
	if(HitchProfiler ~= nil) then
		HitchProfiler.resetFrameCounter()
	end
	--profile:start()

	Global.startShowTime = os.clock()

    local startScene = Platform.scene
	
    if (startScene == "") then 
		startScene = nil 
	end
	
    AnimationManager:init(Global.scene.mainCamera, startScene)

    -- we are finally displaying the show, take control of the audio stream
    JavaAudio.requestFocus(1)

    LeafParticles:config()
    Global:setRenderOverTSB(false)

	-- initialize the replay system
	-- this will play a previous recording
	-- or start recording the current playthrough
	-- from the given camera.
	Replay:init(Global.scene.mainCamera);
end

Global.showEnded = function(self)
    OmniLightBlendManager:shutdown()

    Global.endShowTime = os.clock()

    Log.message( "Total show time: "..string.format("%.2f", Global.endShowTime-Global.startShowTime).." seconds" )

	--[[
	profile:stop()
	if Platform.android then
    	profile:writeReport( '/sdcard/luaprofile.txt' )
	elseif Platform.win32  or Platform.mac then
		profile:writeReport( 'luaprofile.txt' )
	end
	]]
end

Global.onShutdown = function(self)
    if Global.resources then
        Global.resources:destroy()
    end
	AsyncLoader.shutdown()
    DelayScheduler.shutdown()
    SoundManager.shutdown()
    if(Global.scene) then
		--  where do we have the reason for this
		if (Analytics.dispatch ~= nil) then
			Analytics.dispatch()
		else
			Analytics.forceFlushPendingResults()
		end
		Analytics.sendSessionComplete()

		Global.scene:destroy()
		Global.scene = nil;
		while (System.gc(true) > 0) do
		end

		Log.message("Global.onShutdown = function(self)")
	end

	mainView:destroy()

    if Global.storyManager then
        Global.storyManager:destroy()
        Global.storyManager = nil
    end
end

Global.onUpdate = function(self)
	if(Global.scene == nil) then
		return;
	end

	ActorManager:preCreateUpdate()
		
	AsyncLoader:cleanup();
	InputManager:update();
	AnimationManager:update()
	OmniLightBlendManager:update()
	AnalyticsManager:update()

	if(Global.scene) then
		Global.scene:update();
	end

	if (Input.isKeyDown(KB_F12, true)) then
    	mainView:saveScreenshot()
    end  

--	Stat.updateAll();
end

Global.drawInfo = function(self)
	local textScale = 1.0;
	if(Platform.win32 or Platform.mac) then
		textScale = 0.5;
	end
	
	local y = 10;
	local yStep = 30 * textScale;
	nextLine = function() 
		y = y + yStep;
		return y - yStep;
	end
	if (Global.debugRender) then
	    local segmentName = AnimationManager:currentDisplayName()
		if(segmentName == "") then
			segmentName = "gwi";
		end
	    local title = "[" .. Platform.package .. " | "..segmentName.."]"
	    local fpsString = "["..FPS.string().."]"
		local osdEnabled = "No OSD"
		if( OSD.enabled ) then 
			osdEnabled = "OSD"
		end
		local versionString = "["..System.versionString().." | "..osdEnabled.."]"
		
	    local textStyle = { color = Color.new(0.0, 0.4, 0.6), font = "common:fonts/arial_default_24.fnt", scale = textScale }
	    Renderer.drawText(10, nextLine(), textStyle, title)
	    Renderer.drawText(10, nextLine(), textStyle, versionString)
		Renderer.drawText(10, nextLine(), textStyle, "[time: "..Time.string().."]");
		Renderer.drawText(10, nextLine(), textStyle, fpsString)	
--	    Renderer.drawText(10, nextLine(), textStyle, "["..segmentName.."]")
--	    Renderer.drawText(10, nextLine(), textStyle, fpsString)
				
		if(Memory) then
			local mbs = Memory.numBytesAllocated()/1048576.0;
			local blocks = Memory.numBlocksAllocated();
			Renderer.drawText(10, nextLine(), textStyle, "[mem: "..string.format("%.2f", mbs).." | #:"..blocks.."]");
		end
		
		if(Platform.record and Platform.record ~= "") then
			textStyle = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
			Renderer.drawText(10, nextLine(), textStyle, "[record: "..Platform.record.."]");
		end
		
		if(Platform.replay and Platform.replay ~= "") then
--			textStyle = { color = Color.new(0.75, 0.0, 0.0), font = "common:fonts/arial_default_24.fnt", scale = textScale }
			Renderer.drawText(10, nextLine(), textStyle, "[replay: "..Platform.replay.."]".."[x"..string.format("%.2f", Time.timescale()).."]");
		end
	end
	
	if(Global.showFPS) then
	    local textStyle = { color = Color.new(0.0, 0.25, 0.6), font = "common:fonts/arial_default_24.fnt", scale = 2.0 };
	    local fpsString = "[fps: "..string.format("%.2f", FPS.average()).."]"
	    if Global.startShowTime ~= nil then
	    	local time = os.clock() - Global.startShowTime
	    	if Global.endShowTime ~= nil then
	    		time = Global.endShowTime - Global.startShowTime
	    	end
	    	fpsString = fpsString.."[time: "..string.format("%.2f", time).."]"
	    end
	    Renderer.drawText(10, nextLine(), textStyle, fpsString);
	end
end

Global.onReshape = function( width, height )
	Log.message("Reshape "..width..", "..height);

	mainView:resize( width, height )

	if SceneControlPanel then
		SceneControlPanel:moveTo( (width/2)-125, height-250 )
	end
end

Global.onRender = function()
	mainView:setClearColor( Global.clearColor )
	Global.scene:render( mainView )
	Global:drawInfo();
	Renderer.display( mainView )
end

Global.updateScreenSize = function(self)
	if (Platform.android) then
		if (self.renderOverTSB) then
			-- Use the whole screen
			mainView:setClipRect(0, 0, Global.screen_size.x, Global.screen_size.y)
		else
			-- Clip out the TSB area
			-- mainView:setClipRect(0, Global.screen_size.y - Global.avail_size.y, Global.avail_size.x, Global.avail_size.y)
		end
	end
end

Global.setRenderOverTSB = function(self, render)
    self.renderOverTSB = render
    self:updateScreenSize()
end

-- Clean up the segment table to allow jumping around instead of sequential traversal
Global.fixSegmentTable = function(self)
    local previousSeg = nil
    for _, seg in ipairs(segments) do
        -- build up segment data so all objects showing are listed in each segment
        if (previousSeg) then
            for _, value in ipairs(previousSeg.objectsToShow) do
                -- if previous segment object is not being hidden in the this segment add it to objects to show
                if( not table.find_index(seg.objectsToHide, function(hideObj) return hideObj == value end ) ) then
                    seg.objectsToShow = seg.objectsToShow or {}
                    table.insert(seg.objectsToShow, value)
                end
            end

        end
        previousSeg = seg
    end
end

-- Invoke the cleanup
Global.fixSegmentTable()
