

HSVColor {

	h;
	s;
	v;
}

HSVColor.new = function(self, colorRGB, colorHSV)
	o = {};
	
	setmetatable(o, self);
	self.__index = self;
	
	if(colorRGB)then
		self:fromRGB(colorRGB)
	elseif(colorHSV)then
		self.h = colorHSV.h;
		self.s = colorHSV.s;
		self.v = colorHSV.v;
	end
	
	return o;
end
	
function HSVColor:toRGB()
	return HSVtoRGB(self.h*360, self.s, self.v);
end

function HSVColor:fromRGB(rgb)
	local hsv = RGBtoHSV(rgb.r, rgb.g, rgb.b);
	self.h = hsv.h;
	self.s = hsv.s;
	self.v = hsv.v;
end
	
-- STATIC METHOD
HSVColor.RGBtoHSV = function(r,g,b)
	local h;
	local s;
	local v;
	 
	local min;
	local max;
	local delta;
	
	min = math.minArr( {self.r, self.g, self.b} );
	max = math.maxArr( {self.r, self.g, self.b} );
	v = max;						-- v
	delta = max - min;
	if( max ~= 0 ) then
		s = delta / max;				-- s
	else
		-- r = g = b = 0			-- s = 0, v is undefined
		s = 0.0;
		h = -1.0;
		return HSVColor.new(nil, h,s,v);
	end
	
	if( r == max ) then
		h = ( g - b ) / delta;		-- between yellow & magenta
	elseif( g == max ) then
		h = 2 + ( b - r ) / delta;	-- between cyan & yellow
	else
		h = 4 + ( r - g ) / delta;	-- between magenta & cyan
	end
	h *= 60;						-- degrees
	if( h < 0.0 ) then
		h += 360;
	end
	
	return HSVColor.new(nil, h,s,v);
end
	
HSVColor.HSVtoRGB = function(h, s, v )
	local i;
	local f;
	local p;
	local q;
	local t;
	if( s == 0 ) then
		-- achromatic (grey)
		r = v;
		g = v;
		b = v;
		return Color.new(r,g,b);
	end
	h /= 60;			-- sector 0 to 5
	i = math.floor( h );
	f = h - i;			-- factorial part of h
	p = v * ( 1 - s );
	q = v * ( 1 - s * f );
	t = v * ( 1 - s * ( 1 - f ) );
	
	if( i == 0 ) then
			r = v;
			g = t;
			b = p;
	elseif(i == 1) then
			r = q;
			g = v;
			b = p;
	elseif( i == 2) then
			r = p;
			g = v;
			b = t;
	elseif( i == 3 ) then
			r = p;
			g = q;
			b = v;
	elseif( i == 4 ) then
			r = t;
			g = p;
			b = v;
	else		-- case 5:
			r = v;
			g = p;
			b = q;
			break;
	end
	
	return Color.new(r, g, b)
end
		

