
--[[
  String table must be of the form {[key] = {[locale] = [string]}, ...}

    strings =
    {
        ["hi"] =
        {
            ["en_US"] = "Hello World",
            ["es_US"] = "Hola El Mundo"
        },
        ["bye"] =
        {
            ["en_US"] = "Bye",
            ["es_US"] = "Adios"
        }
    }

  Locale strings are in the Android "<lang>_<country>_<variant>" format defined
  by Locale.toString(). See: http://developer.android.com/reference/java/util/Locale.html#toString%28%29

  From the docs:
    Returns the string representation of this Locale. It consists of the 
    language code, country code and variant separated by underscores. If the 
    language is missing the string begins with an underscore. If the country 
    is missing there are 2 underscores between the language and the variant.
    The variant cannot stand alone without a language and/or country code:
    in this case this method would return the empty string.

    Examples: "en", "en_US", "_US", "en__POSIX", "en_US_POSIX" 
]]

-- TODO: move L10N API to moxie-common
-- TODO: move split to moxie-common-utils

-- string split
-- http://lua-users.org/wiki/SplitJoin: "Function: Split a string with a pattern, Take Three"
function split(str, delim, maxNb)
    -- eliminate bad cases...
    if string.find(str, delim) == nil then
        return { str }
    end
    if maxNb == nil or maxNb < 1 then
        maxNb = 0    -- no limit
    end
    local result = {}
    local pat = "(.-)" .. delim .. "()"
    local nb = 0
    local lastPos
    for part, pos in string.gfind(str, pat) do
        nb = nb + 1
        result[nb] = part
        lastPos = pos
        if nb == maxNb then break end
    end
    -- handle the last field
    if nb ~= maxNb then
        result[nb + 1] = string.sub(str, lastPos)
    end
    return result
end

-- Localization API
L10N = {}

L10N.default  = "en_US"

L10N.init = function()
--    L10N.locale   = Avatar.getLocale() or L10N.default

    L10N.locale   = L10N.default

    L10N.codes    = split(L10N.locale, '_')
    L10N.language = L10N.codes[1] or ""
    L10N.country  = L10N.codes[2] or ""
    L10N.variant  = L10N.codes[3] or ""
end

L10N.get = function(stringTable, key)
    return stringTable[key][L10N.locale]
        or stringTable[key][L10N.language..'_'..L10N.country]
        or stringTable[key][L10N.language..'__'..L10N.variant]
        or stringTable[key]['_'..L10N.country]
        or stringTable[key][L10N.language]
        or stringTable[key]["en_US"]
        or ""
end

L10N.init()
