Context = Context or {}

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns what caused Avatar to launch
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
enum("START_REASON_NORMAL", 0)    -- cloud service
enum("START_REASON_SHORTCUT")     -- launcher shortcut
enum("START_REASON_NOTIFICATION") -- android notification
enum("START_REASON_RESET")        -- forces a reset of GWI
enum("START_REASON_SPLASH")       -- splash screen start button
enum("START_REASON_BROWSER")      -- browser screen button
enum("START_REASON_LAUNCHER")     -- dev launcher
enum("START_REASON_TEST")         -- stab test

-- PRIVATE
Context._getStartReason = function()
    if (Platform ~= nil) then
        if (Platform.reason == "shortcut") then
            return START_REASON_SHORTCUT
        elseif (Platform.reason == "reset") then
            return START_REASON_RESET
        elseif (Platform.reason == "splash_play") then
            return START_REASON_SPLASH
        elseif (Platform.reason == "browser_play") then
            return START_REASON_BROWSER
        elseif (Platform.reason == "notification") then
            return START_REASON_NOTIFICATION
        elseif (Platform.reason == "launcher") then
            return START_REASON_LAUNCHER
        elseif (Platform.reason == "test") then
            return START_REASON_TEST
        else
            return START_REASON_NORMAL
        end
    end
    return START_REASON_NORMAL
end

Context._getStartReasonString = function()
    if (Platform.reason == nil) then
        return "Unknown"
    else
        return Platform.reason
    end
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns what caused Avatar to exit
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
enum("STOP_REASON_NONE", 0)      -- have not yet seen the show
enum("STOP_REASON_GWO")          -- good way out
enum("STOP_REASON_FWO")          -- fast way out
enum("STOP_REASON_TIMEOUT")      -- long timeout off of TSB
enum("STOP_REASON_HIDDEN")       -- blacklist
enum("STOP_REASON_ABNORMAL")     -- reboot or crash
enum("STOP_REASON_NOTIFICATION") -- notification was posted
enum("STOP_REASON_DONE")         -- show has completed

-- PRIVATE
Context._getStopReason = function()
    if (Persist.windyday.context.stopReason == nil) then
        Persist.windyday.context.stopReason = STOP_REASON_NONE
    end
    return Persist.windyday.context.stopReason
end

Context._getStopReasonString = function()
	if Persist.windyday.context.stopReason == nil or Persist.windyday.context.stopReason == STOP_REASON_NONE then 
		return "None"
	elseif Persist.windyday.context.stopReason == STOP_REASON_GWO then
		return "GWO"
	elseif Persist.windyday.context.stopReason == STOP_REASON_FWO then
		return "FWO"
	elseif Persist.windyday.context.stopReason == STOP_REASON_TIMEOUT then
		return "TimedOut"
    elseif Persist.windyday.context.stopReason == STOP_REASON_HIDDEN then
        return "Hidden"
    elseif Persist.windyday.context.stopReason == STOP_REASON_ABNORMAL then
        return "Abnormal"
    elseif Persist.windyday.context.stopReason == STOP_REASON_NOTIFICATION then
        return "Notification"
    elseif Persist.windyday.context.stopReason == STOP_REASON_DONE then
        return "Done"
    end
    return "Unknown"
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns true if GWI should not play for the current phone state
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
Context.isBlacklisted = function()
    return false
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Show events
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

Context.sendStoryEvent = function(what, extra)
	if StoryManager and StoryManager.sendEvent then
		if (extra == nil) then
			extra = ""
		end
		StoryManager:sendEvent(what, extra)
	end
end

-- Call when show completes (iris out)
Context.onGoodWayOut = function()
    Persist.windyday.context.stopReason = STOP_REASON_GWO
    Persist.windyday.context.lastCheckpoint = nil
    Persist.windyday.context.lastRunWasGWO = true
    Persist.windyday.context.notificationCanceled = false
    Persist.windyday.context.abnormalExit = false
    System.flushPersist()

    AnalyticsManager:onShowEnded()
    Context.sendStoryEvent("STORY_STATE_STOPPED", "GWO")

    Avatar.restart(500, true)
    Scheduler.cancel()
end

-- Call when show is interrupted (TSB key or incoming event)
Context.onFastWayOut = function()
    Persist.windyday.context.stopReason = STOP_REASON_FWO
    Persist.windyday.context.lastRunWasGWO = false
    Persist.windyday.context.notificationCanceled = false
    Persist.windyday.context.abnormalExit = false
    System.flushPersist()

    AnalyticsManager:onShowEnded()
	Context.sendStoryEvent("STORY_STATE_STOPPED", "FWO")

    Avatar.restart(500, true)
    Scheduler.cancel()
end

-- Call when the hat times out (long timeout)
Context.onTimeOut = function()
    Persist.windyday.context.stopReason = STOP_REASON_TIMEOUT
    Persist.windyday.context.abnormalExit = false
    System.flushPersist()

    Avatar.restart(500, true)
    Scheduler.cancel()
end

-- Call when the doodle has nothing more to present (i.e. after presenting the splash screen following a GWO)
Context.onDone = function()
    Persist.windyday.context.stopReason = STOP_REASON_DONE
    Persist.windyday.context.abnormalExit = false
    System.flushPersist()
end

-- Call when the hat is hiding from the TSB
Context.onHide = function()
    Persist.windyday.context.stopReason = STOP_REASON_HIDDEN
    AnalyticsManager:GWIAbandoned()
end

-- Call when the notification is posted
Context.onNotification = function()
    Persist.windyday.context.stopReason = STOP_REASON_NOTIFICATION
end

-- Returns true if "tap to continue" notification is active
Context.isNotificationPosted = function()
    return Persist.windyday.context.stopReason == STOP_REASON_NOTIFICATION
end

-- Call when moxie version changes
Context.onVersionUpdate = function()
    Persist.windyday.context = {}
    Persist.windyday.versionUpdated = true
    System.flushPersist()
end

-- Call when the notification is posted
Context.onNotificationCanceled = function()
    Persist.windyday.context.notificationCanceled = true
    System.flushPersist()
end

-- Call when a show checkpoint is crossed
Context.onCheckpoint = function(name)
    Persist.windyday.context.lastCheckpoint = name
    System.flushPersist()
	Context.sendStoryEvent("STORY_STATE_SEGMENT_START", name)
end

-- Returns the last checkpoint hit before the show was interrupted
Context.getCheckpoint = function()
    if (Persist.windyday.context.lastCheckpoint == "") then
        return nil
    end
    return Persist.windyday.context.lastCheckpoint
end

-- Updates the TSB style
Context.onSystemUIVizUpdate = function(newVisibility)
    Context.systemUIViz = newVisibility
end

-- Returns true if the show can start
Context.canShowHat = function()
    return true
end

-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Long timeout on TSB hat
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

-- Returns true if the hat has appeared to the user at least once
Context.getTSBHatShown = function()
    return Persist.windyday.context.tsbHatTimeShown ~= nil
end

-- Call when the hat is shown to the user
Context.onTSBHatShown = function()
    if (not Platform.android) then return end

    -- keep track of the time when the hat was shown
    if (Persist.windyday.context.tsbHatTimeShown == nil) then
        Persist.windyday.context.tsbHatTimeShown = os.time()
        Persist.windyday.context.tsbHatTimePeriod = Config.active().showdayTimeout

        System.flushPersist()
    end
end

-- Check to see if the hat has been shown for too long
Context.isTSBHatTimedOut = function()
    if (Persist.windyday.context.tsbHatTimeShown and type(Persist.windyday.context.tsbHatTimeShown) == "number" and
        Persist.windyday.context.tsbHatTimePeriod and type(Persist.windyday.context.tsbHatTimePeriod) == "number") then

        local shownFor = os.time() - Persist.windyday.context.tsbHatTimeShown
        return shownFor > Persist.windyday.context.tsbHatTimePeriod
    end
    return false
end

-- Return how much time we have left before we should timeout
Context.getTSBHatTimeRemaining = function()
    if (Persist.windyday.context.tsbHatTimeShown and type(Persist.windyday.context.tsbHatTimeShown) == "number" and
        Persist.windyday.context.tsbHatTimePeriod and type(Persist.windyday.context.tsbHatTimePeriod) == "number") then

        local shownFor = os.time() - Persist.windyday.context.tsbHatTimeShown
        return Persist.windyday.context.tsbHatTimePeriod - shownFor
    end
    return Config.active().showdayTimeout
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Returns the current scenario
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
enum("PROMPT_HAT_ON_TSB", 0)                  -- hat appears on the TSB
enum("PROMPT_HAT_UNHIDE")                     -- hat is reappearing after a blacklist condition
enum("PROMPT_HAT_FLOURISH_NOTIFICATION")      -- hat animates off the top of the screen to post a notification
enum("PROMPT_HAT_FLOURISH_NOTIFICATION_IDLE") -- after a screen cycle, hat animates off the top of the screen to post a notification
enum("PROMPT_HAT_FLOURISH_INTRO_RESTART")     -- hat animates into the beginning of the show due to a splash screen start
enum("PROMPT_HAT_FLOURISH_INTRO_START_IDLE")  -- hat animates into the beginning of the show after user taps notification
enum("PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE") -- hat animates off the top of the screen after user taps notification
enum("PROMPT_SHOW_LIBRARY")                    -- display the library screen

Context.getPromptType = function()
    local startReason = Context._getStartReason()
    local stopReason = Context._getStopReason()

    --
    -- spaghetti code that determines what scenario should run next based
    -- on context and history
    --
    if (startReason == START_REASON_RESET) then
        -- debug feature - reset GWI
        return PROMPT_HAT_ON_TSB
    elseif (stopReason == STOP_REASON_NOTIFICATION or startReason == START_REASON_TEST) then
        -- we posted a notification and the user just clicked on it
        if (Context.getCheckpoint() == nil) then
            -- FWI to show start - play the stitched animation
            return PROMPT_HAT_FLOURISH_INTRO_START_IDLE
        else
            -- FWI to middle of show - play the notification animation
            return PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE
        end
    elseif (stopReason == STOP_REASON_HIDDEN) then
        -- bring the hat back from a blacklist state
        return PROMPT_HAT_UNHIDE
    elseif (stopReason == STOP_REASON_DONE and startReason ~= START_REASON_BROWSER) then
        -- we've already done a GWO and shown the splash, so do nothing
        return nil
    elseif (stopReason == STOP_REASON_GWO) then
        -- show library after GWO
        return PROMPT_SHOW_LIBRARY
    elseif (stopReason == STOP_REASON_TIMEOUT) then
        -- wait for a screen cycle, then fly the hat up and post a notification
        return PROMPT_HAT_FLOURISH_NOTIFICATION_IDLE
    elseif (stopReason == STOP_REASON_FWO) then
        -- fly the hat up and post a notification
        return PROMPT_HAT_FLOURISH_NOTIFICATION
    elseif (startReason == START_REASON_SPLASH or startReason == START_REASON_BROWSER or startReason == START_REASON_SHORTCUT) then
        -- splash screen always starts the show immediately
        return PROMPT_HAT_FLOURISH_INTRO_RESTART
    elseif (Persist.windyday.context.notificationCanceled) then
        -- user selected X on notification
        return nil
    elseif (not Context.getTSBHatShown()) then
        -- this is the very first appearance of the hat
        return PROMPT_HAT_ON_TSB
    elseif (stopReason == STOP_REASON_ABNORMAL) then
        -- a reboot interrupted our last session
        if (Persist.windyday.context.lastRunWasGWO) then
            -- reboot followed a GWO, so do nothing
            return nil
        else
            -- otherwise wait for a screen cycle then post a notification
            return PROMPT_HAT_FLOURISH_NOTIFICATION_IDLE
        end
    end

    -- do nothing (unknown reason)
    return nil
end

Context.bypassIdle = function()
    return Context._getStartReason() == START_REASON_TEST
end

Context.isBrowserPlay = function()
    return Context._getStartReason() == START_REASON_BROWSER
end


-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --
-- Initialize persistent state
-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- --

Persist = Persist or {}
Persist.windyday = Persist.windyday or {}
Persist.windyday.context = Persist.windyday.context or {}

-- If we're being reset, clear all persistent context state
if (Context._getStartReason() == START_REASON_RESET) then
    Persist.windyday.context = {}
end

-- If we have upgraded to a new major, minor or build version, clear persist state
if (Context._getStartReason() ~= START_REASON_TEST) then
    if ((Persist.windyday.context.lastPlatformVersionMajor == nil) or (Persist.windyday.context.lastPlatformVersionMajor ~= Platform.versionMajor) or
        (Persist.windyday.context.lastPlatformVersionMinor == nil) or (Persist.windyday.context.lastPlatformVersionMinor ~= Platform.versionMinor) or
        (Persist.windyday.context.lastPlatformVersionBuild == nil) or (Persist.windyday.context.lastPlatformVersionBuild ~= Platform.versionBuild)) then
        Context.onVersionUpdate()
    end
end

Persist.windyday.context.lastPlatformVersionMajor = Platform.versionMajor
Persist.windyday.context.lastPlatformVersionMinor = Platform.versionMinor
Persist.windyday.context.lastPlatformVersionBuild = Platform.versionBuild
Persist.windyday.context.lastPlatformVersionPatch = Platform.versionPatch

-- Detect if the previous run exited due to reboot, battery drain, crash, etc
if (Persist.windyday.context.abnormalExit) then
    Persist.windyday.context.stopReason = STOP_REASON_ABNORMAL
end
Persist.windyday.context.abnormalExit = true

-- Debug info
Persist.windyday.context.lastStopReason = Context._getStopReason()
Persist.windyday.context.lastStopReasonStr = Context._getStopReasonString()
Persist.windyday.context.lastStartReason = Context._getStartReason()
Persist.windyday.context.lastStartReasonStr = Context._getStartReasonString()

-- Flush to disk
System.flushPersist()
