
Launcher.intro = {

    onStartup = function(self)
        InputManager:enableSensors(true)

        ActorManager:startPreCreate() 

        self.interrupted = Context.isBlacklisted()

        -- analytics
        AnalyticsManager:setAppAtTimeOfPrompt()
		
        -- set up visuals
        local tsbRig = Global.scene.actors.TSBRig
        local launchHat = Global.scene.actors.LaunchHat

        self.animationName = "IntroStatic"
        local whiteFadeDelay = 2.0
		local launchHatStartDelay = launchHat.animation.IntroStatic:getDuration() - 1.0


        local prompt = Context.getPromptType()
		if (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART
			or prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE
			or prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE) then

            self.animationName = nil
			launchHatStartDelay = 1.1
			whiteFadeDelay = 0

            Android.onFadeInForest()
        else
            Android.reset()
        end

        self.slideHatTimer = 1
        if (prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE) then
            self.slideHatTimer = 0
        end

        Android.acquireScreen()
 
        local whiteScreen = Global.scene.actors.WhiteScreen
        local whiteScreenFader = whiteScreen.components["WhiteScreenFader"]
        whiteScreen:setVisibility(true) 

        if (not Context.getCheckpoint() or prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE or prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART) then
            self.showRealHatAtIntro = false
            local launchHatMover = launchHat:addComponent("LaunchHatMover", LaunchHatMover.new())
            launchHatMover:startIntro(launchHatStartDelay)

            MusicPlayer:playIntroMusic()
        else
            self.showRealHatAtIntro = true
        end
		
		if (self.animationName) then
			launchHat.animation[self.animationName]:play()
			tsbRig.animation[self.animationName]:play()
		end
		
        whiteScreenFader:fadeIn(whiteFadeDelay)

        self.sceneStartTimer = whiteFadeDelay + 0.75
        self.TSBClipTimer = whiteFadeDelay
        self.sceneStarted = false

        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()
        local prompt = Context.getPromptType()

        if (self.interrupted or Global.shutdown) then
            return "launcher_interrupted"
        end

        if (not self.TSBClipped) then
            self.TSBClipTimer = self.TSBClipTimer - Time.dt()
            if (self.TSBClipTimer < 0) then
                if (Platform.android or Platform.iOS) then
                    -- mainView:setClipRect(0, Global.screen_size.y - Global.avail_size.y, Global.avail_size.x, Global.avail_size.y);
                end
                self.TSBClipped = true
                if(Platform.win32 or Platform.mac) then
                    local fakeScreen = Global.scene.actors.FakeScreen
                    fakeScreen:setVisibility(false)
                end
            end
        end

		-- Slide hat out of the way if not starting from TSB and going to a checkpoint
		if (prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE) then
            -- Update the rig's parent to the right
            local rigRoot = Global.scene.actors.TSBRig.root
            local pos = rigRoot:getPosition()
			self.slideHatTimer = self.slideHatTimer + Time.dt()
            pos.x = pos.x + math.lerpEaseIn(0, 1, self.slideHatTimer)
            rigRoot:setPosition(pos)
		end

        if (not self.sceneStarted) then
            self.sceneStartTimer = self.sceneStartTimer - Time.dt()

            -- Make sure both the scene start timer if done and the hat has
            -- finished sliding off the screen
            if (self.sceneStartTimer < 0 and self.slideHatTimer >= 1) then
                Global:startShow()
                Global.scene.mainCamera.controller:enableSensor()
                local whiteScreen = Global.scene.actors.WhiteScreen
                local whiteScreenFader = whiteScreen.components["WhiteScreenFader"]
                whiteScreenFader:fadeOut(0)
                whiteScreenFader:setDestroyAtEnd()
                Android.setAlphaRange(1.0, 1.0);
                Global.clearColor:set(1.0, 1.0, 1.0, 1.0);

                self.sceneStarted = true

                Android.onFadeInForest()
            end
        end

        if (self.showRealHatAtIntro) then
            if (self.sceneStarted) then
                local realHat = ActorManager:getActor("Hat")
                realHat:setVisibility(true)
                local launchHat = Global.scene.actors.LaunchHat
                launchHat:destroy()
                return "windyday";
            end
        else
            local launchHatMover = Global.scene.actors.LaunchHat.components["LaunchHatMover"]
            if(launchHatMover:isReady()) then
                return "windyday";
            end
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onRender = function(self)
        if (not Android.isPaused()) then
            Global.onRender()
        end
    end,

    onShutdown = function(self)
    end,

    onEvent = function(self, event)
        if(event.type == "onActivityEvent") then
            if (event.data.eventString == "onDummyActivityGainedFocus") then
                Android.onResume()
            elseif (event.data.eventString == "onDummyActivityLostFocus") then
                Android.onPause()
            elseif (event.data.eventString == "onStop") then
                self.interrupted = true
                Android.onResume()
            end
        elseif(event.type == "onChangeDisplaySize") then
            Global:setRenderOverTSB(false)
        elseif(event.type == "onBatteryEvent") then
            if (event.data.newState == 0) then
                self.interrupted = true
                Android.onResume()
            end
        end
    end
}
