
require "windyday/scripts/launcher/load1_assets"

Launcher.load1 = {
    -- whether or not we have already passed through this state
    loaded = false,

    -- next state after loading completes
    nextState = nil,

    -- whether or not to wait for a screen cycle
    doIdle = false,
    doIdleOnNotification = false,

    onStartup = function(self)
        Android.reset()

        -- choose the next state
        self.nextState = nil
        self.doIdle = false
        self.doIdleOnNotification = false
        
        local prompt = Context.getPromptType()

        if (prompt == PROMPT_HAT_FLOURISH_NOTIFICATION) then
            self.nextState = "launcher_notify"
        elseif (prompt == PROMPT_HAT_FLOURISH_NOTIFICATION_IDLE) then
            self.nextState = "launcher_notify"
            self.doIdle = true
        elseif (not Context.bypassIdle() and (prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE or prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE)) then
            self.nextState = "launcher_load2"
            self.doIdle = true
            self.doIdleOnNotification = true
        else
            self.nextState = "launcher_load2"
        end

        -- Initialize the scene & camera and start prefetching of the hat assets
        self:prefetch()

        -- tick continuously
        return true
    end,

    onUpdate = function(self)
        -- go to next state when loading completes
        if (self.loaded or System.getPendingResourceCount() == 0) then
            self:setupScene()
            self.loaded = true

            if (self.doIdle) then
                Launcher.idle.nextState = self.nextState
                Launcher.idle.idleOnNotification = self.doIdleOnNotification
                return "launcher_idle"
            else
                return self.nextState
            end
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onRender = function(self)
    end,

    prefetch = function(self)
        if (self.loaded) then return end

        Global.init()

        if StoryManager and StoryManager.pauseStory then
            StoryManager.pauseStory()
        end

        if (Platform.win32 or Platform.mac) then
            Global.resources:addActor(def.actors.FakeScreen)
        end

        Global.resources:addActor(def.actors.WhiteFade)
        Global.resources:addActor(def.actors.TSBRig)
        Global.resources:addActor(def.actors.LaunchHat)
        Global.resources:prefetch()
    end,

    setupScene = function(self)
        if (self.loaded) then return end

        -- create a transform for screen space items
        local screenSpace = Global.scene:createTransform("LockedToCamera")
        Global.scene.mainCamera:addChild(screenSpace)

        -- make a SRT for all the screen elements
        local screenSpaceSRT = screenSpace:getLocal()
        screenSpaceSRT.position = Vector(0, -2, 0)

        -- create a fake screen shot if we are in windows
        if (Platform.win32 or Platform.mac) then
            local fakeScreen = Global.scene:createActor("FakeScreen", def.actors.FakeScreen)
            local screenFader = fakeScreen:addComponent("FakeScreenFader", ScreenFader.new("Mat_ScreenShot"))
            fakeScreen:setVisibility(true)
            fakeScreen:setLocal(screenSpaceSRT)
            screenSpace:addChild(fakeScreen)
        end

        -- create a white fader for the scene
        local whiteScreen = Global.scene:createActor("WhiteScreen", def.actors.WhiteFade)
        local whiteFader = whiteScreen:addComponent("WhiteScreenFader", ScreenFader.new("Mat_WhiteFade"))
        whiteFader:fadeOut(0)
        whiteScreen:setVisibility(false)
        whiteScreen:setLocal(screenSpaceSRT)
        screenSpace:addChild(whiteScreen)

        -- Create a parent for the camera rig so we can move it while we are animating it
        local tsbRigRoot = Global.scene:createTransform("TSBRigRoot")
        screenSpace:addChild(tsbRigRoot)

        -- create a rig to move the hat into the world
        local tsbRig = Global.scene:createActor("TSBRig", def.actors.TSBRig)
        tsbRig:setLocal(screenSpaceSRT)
        tsbRig.root = tsbRigRoot
        tsbRigRoot:addChild(tsbRig)
        tsbRig:setVisibility(false) -- turn off its visibility, just in case

        -- create a hat to land on the tsb
        local launchHat = Global.scene:createActor("LaunchHat", def.actors.LaunchHat)
        local launchHatMover = launchHat:addComponent("launchHatPosTracker", PositionTracker.new(tsbRig, "HatPosition|RootNode", false))
        launchHatMover:trackRotation()
        Launcher.hat.init(launchHat)

        -- move the screen space transform into position
        local currentSRT = screenSpace:getLocal()
        currentSRT.rotation = Quaternion.fromEulerAngles(Vector(0, 0, 0))
        currentSRT.position = Vector(0, 0, 1.75)
        screenSpace:setLocal(currentSRT)
    end
}
