
Launcher.load2 = {
    -- whether or not we have already passed through this state
    loaded = false,

    -- next state after loading completes
    nextState = nil,

    -- whether or not to wait for a screen cycle
    doIdle = false,

    introAnim = "LoadCoverIntro",
    loopAnim =  "LoadCoverIdle",

    exitTimeLength = 2,  -- Take 2 seconds to float off the screen

    onStartup = function(self)
        Android.reset()

        if StoryManager and StoryManager.resumeStory then
            StoryManager.resumeStory()
        end

        -- choose the next state
        self.nextState = nil
        self.doIdle = false
        self.finishedIntro = true
        self.animation = nil

        local prompt = Context.getPromptType()

        if (prompt == PROMPT_HAT_ON_TSB) then
            self.nextState = "launcher_prompt"
            self.doIdle = true
        elseif (prompt == PROMPT_HAT_UNHIDE) then
            self.nextState = "launcher_fast_prompt"
            self.doIdle = true
        elseif (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART
            or prompt == PROMPT_HAT_FLOURISH_INTRO_MIDDLE_IDLE
            or prompt == PROMPT_HAT_FLOURISH_INTRO_START_IDLE) then

            Android.acquireScreen()
            InputManager:enableSensors(true)

            self.nextState = "launcher_intro"
            self.animation = self.introAnim
            self.finishedIntro = false
            self:playAnimation()

            -- Make sure the mouse gets prefetched before we h`ead into the show
            Global.resources:addActor( def.actors.Mouse, "HatCycle" )

            if (prompt == PROMPT_HAT_FLOURISH_INTRO_RESTART) then
--                if (not Context.isBrowserPlay()) then
                    Avatar.cancelNotification()
  --              end
                Context.onCheckpoint("")
            end
        else
            Log.fatal("Reached launcher_load2 in inapplicable scenario")
        end

        -- start loading assets in the background
        self:prefetch()

        self.interrupted = Context.isBlacklisted()
        self.exiting = false

        -- tick continuously
        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()

        if (self.exiting) then
			-- we are done loading and moving that hat off screen
			if (not self.loaded) then
				Global.prefetchCompleted()
				self.loaded = true
			end

			-- go to next state
			if (self.doIdle) then
				Launcher.idle.nextState = self.nextState
				Launcher.idle.idleOnNotification = false
				return "launcher_idle"
			else
				return self.nextState
			end
        elseif ((self.loaded or System.getPendingResourceCount() == 0) and self.finishedIntro) then
            self.exiting = true
        end

        if (self.animation and (self.interrupted or Global.shutdown)) then
            return "launcher_interrupted"
        end

        if (self.animation and not self:animationPlaying()) then
            self.animation = self.loopAnim
            self:playAnimation()
            self.finishedIntro = true
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onRender = function(self)
        if not (Android.isPaused() or self.interrupted or not self.animation) then
            Global.onRender()
        end
    end,

    onShutdown = function(self)
    end,

    onEvent = function(self, event)
        if (self.animation) then
            if(event.type == "onActivityEvent") then
                if (event.data.eventString == "onDummyActivityGainedFocus") then
                    Android.onResume()
                elseif (event.data.eventString == "onDummyActivityLostFocus") then
                    Android.onPause()
                elseif (event.data.eventString == "onStop") then
                    self.interrupted = true
                    Android.onResume()
                end
            elseif(event.type == "onChangeDisplaySize") then
                Global:setRenderOverTSB(false)
            end
        end
    end,

    prefetch = function(self)
        if (self.loaded) then return end

        -- load to a checkpoint
        Global.initShow()
    end,

    playAnimation = function(self)
        if (self.animation) then
            Global.scene.actors.LaunchHat.animation[self.animation]:play()
            Global.scene.actors.TSBRig.animation[self.animation]:play()
        end
    end,

    animationPlaying = function(self)
        return self.animation and Global.scene.actors.LaunchHat.animation[self.animation]:isPlaying()
    end,

}
