
require "windyday/scripts/analyticsmanager"

Launcher.prompt = {
    new = function(self, animName)
        local this = { 
            animName = animName, 
            returning = false,
        }
        setmetatable(this, self)
        self.__index = self
        return this
    end,

    onStartup = function(self)
        Android.reset()

    	AnalyticsManager.hatPromptIsolated(self.animName)

        -- record when hat is initially shown on the TSB
        if (Context.getPromptType() == PROMPT_HAT_ON_TSB) then
            Context.onTSBHatShown()
        end
        
        -- notify analytics we are showing the hat, plus which app was active at the time
        AnalyticsManager:hatPromptBegun()

        -- for debugging allow the user to tap during prompt on win32
        if (Platform.win32 or Platform.mac) then
            Launcher.hat:setTapAllowed(true)
        end

        -- set up visuals

        -- get the launchHat
        local launchHat = Global.scene.actors.LaunchHat
        local tsbRig = Global.scene.actors.TSBRig
        
        -- save the hat anim for later use
        self.animation = launchHat.animation[self.animName]
        
        -- set an amount of time to run for
        self.animTime = self.animation:getDuration()
        
        -- set an amount of time for an initial delay and add it to the run time
        self.startDelay = 1.0
        self.animTime = self.animTime + self.startDelay

        Launcher.onHatPrompt()

        return true
    end,

    onUpdate = function(self)
        Global.onUpdate()

        -- check if animation is done
        self.animTime = self.animTime - Time.dt()
        if (self.animTime < 0) then
            return "launcher_wait"
        end

        if (self.animTime < self.animation:getDuration() and not self.animation:isPlaying()) then
            -- play the correct animations on the hat and rig
            local launchHat = Global.scene.actors.LaunchHat
            local tsbRig = Global.scene.actors.TSBRig           
            launchHat:setVisibility(true)
            launchHat.animation[self.animName]:play()--:crossfade(1, false)
            tsbRig.animation[self.animName]:play()--:crossfade(1, false)        
        end

        -- hide the hat immediately if we enter a blacklist state
        if (Context.isBlacklisted()) then
            return "launcher_hide"
        end

        -- long timeout
        if (Context.isTSBHatTimedOut()) then
            return "launcher_timeout"
        end
    end,

    onReshape = function(self, width, height)
        Global.onReshape(width, height)
    end,

    onRender = function(self)
        Global.onRender()
    end,

    onShutdown = function(self)
    end,

    onDestroy = function(self)
        if (Platform.win32 or Platform.mac) then
            Launcher.hat:setTapAllowed(false)
        end
    end,

    onEvent = function(self, event)
        -- pass the event onto the hat object to handle input
        return Launcher.hat:onEvent(event)
    end
}
