
require "common/scripts/moxie/moxie"

require "windyday/scripts/util"

local LogDebug = true
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end

LeafParticles = {}

function LeafParticles.config()
    Debug("   *** Configurating LeafParticles    ****")

    local leafRoot = Global.scene:createTransform("LeafParticleRoot")
    local leafDriver = leafRoot:addComponent("LeafParticleDriver", LeafParticlesDriver:new())
    leafDriver.systemDefintions = { "leavesinner", "leavesouter" }

    Debug("   *** DONE Configurating LeafParticles    ****")
end


LeafParticlesDriver = {
    systemDefintions = {},
    particleSystem  = nil,

    lerpTimeScalar  = 0.5,

    speedScale      = 1.0,
    speedScaleGoal  = 1.0,

    baseSpinSpeed   =
    { 
        Vector(0.0, -5.0,0.0),
        Vector(0.0,-10.0,0.0)
    },

    leafEmitters    = {},

    emitEnabled     = true,
    prevEmitRate    = {},
}

function LeafParticlesDriver:new(data)
    local this = shallowcopy(data) or {}
    setmetatable(this, self)
    self.__index = self
    return this
end

function LeafParticlesDriver:onStart()

    for i = 1, #self.systemDefintions do
        Log.message( " Parsing Particle System : "..self.systemDefintions[i] )

        local emitter = Global.scene:createParticleEmitter( self.systemDefintions[i], "windyday:particles/"..self.systemDefintions[i]..".mxp" )
        emitter:setLocal( SRT.new( Vector(0.0,0.0,0.0), Quaternion.identity, Vector.one ) )
        self.leafEmitters[i] = emitter
    end

    Events.listen(self, "windMagnitude");
    -- onStart() is often after the first windMessage, check it here in case we missed it
    if (AnimationManager and AnimationManager.currentWindMagnitude) then
        self:scaleSpeed(AnimationManager:currentWindMagnitude())
    end
end

function LeafParticlesDriver:onUpdate()

    self.speedScale = math.lerp(self.speedScale, self.speedScaleGoal, Time.dt() * self.lerpTimeScalar);

    -- Log.message( " *** PRE-SPAMMING *** ")
    for i = 1, #self.leafEmitters do

        if (self.leafEmitters[i]) then
            -- Log.message( " *** FOUND EMITTER! *** ")
            local transform = self.leafEmitters[i]:getLocal()
            if (transform) then
                -- Log.message( " *** FOUND TRANSFORM! *** ")
                transform.rotation = transform.rotation * Quaternion.fromEulerAngles(self.baseSpinSpeed[i] * self.speedScale * Time.dt())
                -- local eulerRot = transform.rotation:getEulerAngles()
                -- Log.message( "      EulerRot ("..eulerRot.x.." , "..eulerRot.y.." , "..eulerRot.z..") ")

                -- Testing parenting
                -- transform.position = Vector(math.cos(Time.seconds() * 0.25) * 5.0, math.cos(Time.seconds() * 0.25) * 5.0, -15.0)
                -- Log.message( "      Position ("..transform.position.x.." , "..transform.position.y.." , "..transform.position.z..") ")

                self.leafEmitters[i]:setLocal(transform)
            end
        end
    end

    -- Log.message( " *** SPAMMING *** ")
end

function LeafParticlesDriver:onDestroy()
   Events.stopListening(self, "windMagnitude")
end

LeafParticlesDriver.windMagnitude = function(self, magnitude)
    self:scaleSpeed(magnitude)
end

function LeafParticlesDriver:scaleSpeed(scalar)
    self.speedScaleGoal = scalar

    -- Determine if we need to toggle the emmitters on/off
    if (scalar == 0 and self.emitEnabled == true) then
        self.emitEnabled  = false
        self:stopEmitting()
    elseif (self.emitEnabled == false and scalar ~= 0) then
        self.emitEnabled  = true
        self:resumeEmitting()
    end
end

function LeafParticlesDriver:stopEmitting()
    for i = 1, #self.leafEmitters do
        if (self.leafEmitters[i].getEmitRate) then
            self.prevEmitRate[i] = self.leafEmitters[i]:getEmitRate()
        else
            self.prevEmitRate[i] = 1
        end
        self.leafEmitters[i]:setEmitRate(0)
    end
end

function LeafParticlesDriver:resumeEmitting()
    for i = 1, #self.leafEmitters do
        self.leafEmitters[i]:setEmitRate(self.prevEmitRate[i])
    end
end


