
loadSequence = {
	{
	   "HatCycle",
    },
    {
	   "HatIntro",
	   "PepeIntro",
	},
	{
	   "ChaseA",
	   "ChaseB",
	},
	{
	   "BushA",
	   "BushB",
	},
	{
	   "SquirrelA",
	},
	{
	   "TeaseA",
	   "TeaseB",
	},
	{
	   "Rampage1",
	},
	{
	   "Rampage2A",
	},
	{
	   "Rampage2B",
	},
	{
	   "Rampage3A",
	},
	{
	   "Rampage3B",
	},
	{
	   "Rampage3C",
	   "WindUpA",
	},
	{
	   "WindUpB",
	},
	{
	   "WindUpC",
	},
	{
	   "WindUpD",
	   "WindUpE",
	},
	{
	   "TornadoA",
	},
	{
	   "TornadoB",
	},
	{
	   "BarrageA",
	   "BarrageB",
	   "BarrageC",
	   "BarrageD",
	   "BarrageE",
	},
    {
	   "DropA",
    },
    {
	   "DropB",
	   "DropC",
	},
	{
	   "End1A",
	},
	{
	   "End2A",
	},
	{
	   "End3A",
	},
	{
	   "End4A",
	},
	{
	   "End4B",
	},
	{
	   "End4C",
	},
	{
	   "End5A",
	},
	{
	   "End5B",
	},
	{
	   "End5C",
	},
	{
	   "End6A",
	},
	{
	   "End6B",
	}
}

-- create a name to index mapping for segments
-- a segment in this context is a metadata descriptor 
-- containing references to the dependent resources 
-- needed at each point in the show. 
local segmentmapping = {}
for i, seg in ipairs(segments) do
	segmentmapping[seg.name] = i;
end

-- create a name to index mapping for sets
-- a set in this context is a construct meant to represent
-- the static environment at each point in the show. 
local setmapping = {}
for i, set in ipairs(sets) do
	setmapping[set.name] = i;
end

-- create a name to index mapping for the music data per segment
local musicmapping = {}
for i, musicEvent in ipairs(music) do
    musicmapping[musicEvent.name] = i
end

-- create a name to index mapping for the soundfx data per segment
local soundfxmapping = {}
for i, segmentSFX in ipairs(soundfx) do
    soundfxmapping[segmentSFX.name] = i
end

-- AsyncLoader
AsyncLoader = {}

-- prefetch a set
-- prefetches all the resources for all the peices (actors)
-- in the set.
AsyncLoader.prefetchSet = function(set, group)
    if set.pieces then
        --Log.message( "AsyncLoader.prefetchSet: "..set.name..", numPieces: "..#set.pieces )
        for i, piece in ipairs(set.pieces) do
            --Log.message( "lua prefetch set (piece): "..piece )
            local actordef = def.actors[piece];
            --Log.message( "lua prefetch set (actordef): "..actordef.model )
			group:addActor(actordef)
        end

		group:prefetch()
    end
end

AsyncLoader.prefetchMusic = function(musicEvent, group)
    for _, bed in ipairs(musicEvent.musicBedEvents) do
		SoundManager.prefetch(bed.name, group)
    end
    for _, stab in ipairs(musicEvent.musicStabEvents) do
		SoundManager.prefetch(stab.name, group)
    end
end

AsyncLoader.prefetchSFX = function(segmentSFX, group)
    if (segmentSFX) then
        for _, sound in ipairs(segmentSFX.sounds) do
            SoundManager.prefetch(sound.name, group)
        end
    end
end

AsyncLoader.prefetchSecondaryAudio = function(segment, group)
    if (segment.objectsToShow == nil) then return end
    for _, objectName in ipairs(segment.objectsToShow) do
        -- see if there are any secondary audio for this segment that we need to prefetch
        local audio = secondaryaudio[objectName]
        if (audio) then
            SoundManager.prefetch(audio.soundName, group)
        end
        -- see if there are any easter egg sounds for this segment
        local easteregg = eastereggs[objectName]
        if (easteregg and easteregg.soundName) then
            SoundManager.prefetch(easteregg.soundName, group)
        end
    end
end

AsyncLoader.prefetchIntroMusic = function()
    AsyncLoader.prefetchMusic(musicIntro, Global.resources)
end

-- prefetch a segment. 
-- prefetches all the resources for all the actors
-- in the segment, including set peices.
AsyncLoader.prefetchSegment = function(segment, group)
	Log.message( "AsyncLoader.prefetchSegment: "..segment.name )
	local setindex = setmapping[segment.setName];
	AsyncLoader.prefetchSet(sets[setindex], group);
    AsyncLoader.prefetchMusic(music[musicmapping[segment.name]], group)
    AsyncLoader.prefetchSFX(soundfx[soundfxmapping[segment.name]], group)
    AsyncLoader.prefetchSecondaryAudio(segment, group)
    for i, actor in ipairs(segment.actors) do
		local actordef = def.actors[actor.name];
		group:addActor(actordef, segment.name)
	end
	if(segment.objectsToShow ~= nil) then
		for i, actor in ipairs(segment.objectsToShow) do
			local actordef = def.actors[actor];
			if(actordef) then
				group:addActor(actordef, segment.name)
			end

            local easteregg = eastereggs[actor]
            if (easteregg and easteregg.actorName) then
                group:addActor(easteregg.actorName, easteregg.animationName)
            end
		end
	end
	if(segment.prefetch ~= nil) then
		for i, actor in ipairs(segment.prefetch) do
			local actordef = def.actors[actor];
			if(actordef) then
				group:addActor(actordef)
			end
		end
	end
	if(segment.particleSystems ~= nil) then
		for _, uri in ipairs(segment.particleSystems) do
			group:addParticleSystem(uri)
		end
	end
	group:prefetch()

    if Global.storyManager then
        Global.storyManager:prefetchState(segment.name)

        if segment.nextSegments then
            for _, transition in pairs(segment.nextSegments) do
                Global.storyManager:prefetchState(transition)
            end
        end
    end
end

-- This should be called each time a new segment is 
-- started. 
local prefetchIndex = 1;
local prefetchCounter = 1;

local resourceGroups = {}

AsyncLoader.cleanup = function(self)
	System.gc();
end

AsyncLoader.prefetch = function()
	prefetchCounter = prefetchCounter - 1;
	if(prefetchCounter == 0) then
		local batch = loadSequence[prefetchIndex];
		if(batch ~= nil) then
			prefetchCounter = table.getn(batch);
			for i, segname in ipairs(batch) do 
				local segindex = segmentmapping[segname];

				-- track resources in resource group
				local group = System.createResourceGroup( segname )
				resourceGroups[segname] = group
				Log.warning( "Prefetching into resource group: "..segname )
				
				AsyncLoader.prefetchSegment(segments[segindex], group);
			end
		end

		prefetchIndex = prefetchIndex+1;
	end
end

AsyncLoader.prefetchForSegment = function(segmentName)
    -- Find the segment in the batches and update the indices
    for batchIndex, batch in ipairs(loadSequence) do
        local counter = #batch
        for _, seg in ipairs(batch) do
            if (seg == segmentName) then
                -- Prefetch the block for the given segment
                prefetchIndex = batchIndex
                prefetchCounter = 1
                AsyncLoader.prefetch()
                prefetchCounter = counter
                return
            end
            counter = counter - 1
        end
    end
end

-- Call this to move the prefetch index backward (<0) or forward (>0)
-- Should only be used if manually changing the current segment.
AsyncLoader.jump = function(offset)
	prefetchIndex = prefetchIndex+offset;
end

function AsyncLoader.freeResourceGroup( name )
	local group = resourceGroups[name]
	if group then
		Log.warning( "Freeing resource group: "..name )
		group:destroy()
		resourceGroups[name] = nil
	end
end

function AsyncLoader.shutdown()
	-- free any remaining resource groups
	for name, group in pairs(resourceGroups) do
		Log.warning( "Freeing resource group: "..name )
		group:destroy()
	end
end


function AsyncLoader.generateStreamingData()
    local states = {}
    local resources = {}

    local scene = Scene.create()
    local soundEmitter = AudioEmitter.create()

    local function resourceDuration(uri)
        local duration
        if string.endsWith(uri, ".mfb") then
            local fbSampler = FlipbookSampler.create()
            fbSampler:setFlipbook(uri)
            duration = fbSampler:getNumFrames() / 60
            fbSampler:destroy()
        elseif string.endsWith(uri, ".ogg") then
            local sound = soundEmitter:addSound(uri, uri, false, 0, -1)
            duration = sound:getDurationSeconds()
            soundEmitter:removeSound(uri)
        end
        return duration
    end

    local function animationDuration(actorName, animName)
        local actorData = def.actors[actorName]
        if actorData and actorData.animation and actorData.animation[animName] then
            local anim = actorData.animation[animName]
            local startFrame = anim[2]
            local endFrame = anim[3]
            if endFrame == -1 then
                -- load the actor so we can get the animation's real end frame
                local actor = scene:createActor(actorName, actorData)
                if actor and actor.animation and actor.animation[animName] then
                    endFrame = actor.animation[animName]:getStopFrame()
                    actor:destroy()
                    Util.fullGC()
                end
            end
            return (endFrame - startFrame) / 60
        end
    end

    local function minAnimationDuration(actorNames, segmentName)
        local minDuration
        for _, actorName in ipairs(actorNames) do
            local animDuration = animationDuration(actorName, segmentName)
            if minDuration then
                minDuration = math.min(minDuration, animDuration)
            else
                minDuration = animDuration
            end
        end
        return minDuration
    end

    local function convertSegmentResources(resourceGroup)
        local segmentResources = {}
        if resourceGroup.uris then
            for uri, _ in pairs(resourceGroup.uris) do
                local path = uri:gsub(":", "/")
                if not resources[path] then
                    resources[path] = { path = path, duration = resourceDuration(uri) }
                end
                table.insert(segmentResources, path)
            end
        end
        return segmentResources
    end

    local function resourcesForSegment(segmentName)
        AsyncLoader.prefetchForSegment(segmentName)
        local group = resourceGroups[segmentName]
        if group then
            return convertSegmentResources(group)
        end
        return {}
    end

    AsyncLoader.prefetchIntroMusic()

    for order, segmentData in ipairs(segments) do
        local segmentName = segmentData.name
        if not states[segmentName] then
            local state = {}
            states[segmentName] = state

            state.order = order
            state.resources = resourcesForSegment(segmentName)

            state.transitions = {}
            local minSegmentDuration
            if segmentData.nextSegments then
                for _, transition in ipairs(segmentData.nextSegments) do
                    local minTransitionDuration = minAnimationDuration(segmentData.actors, segmentName)
                    state.transitions.default = {
                        toSegment = transition,
                        minDuration = minTransitionDuration,
                    }
                    if minSegmentDuration then
                        minSegmentDuration = math.min(minSegmentDuration, minTransitionDuration)
                    else
                        minSegmentDuration = minTransitionDuration
                    end
                end
            end
            state.minDuration = minSegmentDuration
        end
    end

    -- Compute the base resources needed to start the show
    local baseResourceGroup = Global.resources

    -- Special case the launcher and show start assets
--    baseResourceGroup:addActor(LaunchData.actors.LauncherToNotification)
--    baseResourceGroup:addActor(LaunchData.actors.GoodWayIn)
--    baseResourceGroup:addActor(LaunchData.actors.GWI_Title)
--    baseResourceGroup:addFlipbook("windyday:flipbooks/title.mfb")
--    for _, actorName in ipairs(SceneData.ShowStartActors) do
--        baseResourceGroup:addActor(SceneData.actors[actorName])
--    end

    local rootState = {}
    rootState.order = 0
    rootState.resources = convertSegmentResources(baseResourceGroup)
    states["BaseResources"] = rootState

    local filename = "windyday.xml"
    local file, error = io.open(filename, "w")

    file:write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n")
    file:write("<StoryManifest xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:noNamespaceSchemaLocation=\"ResourceManifestSchema.xsd\">\n")

    file:write("\t<Chunks>\n")
    for _, state in pairs(states) do
        local url = string.format("http://dl.google.com/spotlight/windyday/1/windyday_%03d.zip", state.order)
        file:write("\t\t<Chunk ID=\"" .. tostring(state.order) .. "\" URL=\"" .. url .. "\">\n")
        if state.resources then
            for _, name in ipairs(state.resources) do
                local resource = resources[name]
                if resource then
                    local duration = ""
                    if resource.duration then
                        duration = " duration=\"" .. string.format("%.2f", resource.duration) .. "\""
                    end
                    file:write("\t\t\t<Resource Name=\"" .. resource.path ..  "\"" .. duration .. "/>\n")
                end
            end
        end
        file:write("\t\t</Chunk>\n")
    end
    file:write("\t</Chunks>\n")

    file:write("\t<StateMachines>\n")
    file:write("\t\t<StateMachine Name=\"windyday\" Root=\"BaseResources\">\n")

    file:write("\t\t\t<States>\n")
    for name, state in pairs(states) do
        local minDuration = ""
        if state.minDuration then
            minDuration = string.format(" MinDuration=\"%.2f\"", state.minDuration)
        end
        file:write("\t\t\t\t<State Name=\"" .. name .. "\" Order=\"" .. tostring(state.order) .. "\"" .. minDuration .. ">\n")
        file:write("\t\t\t\t\t<Chunks>\n")
        file:write("\t\t\t\t\t\t<ChunkID>0</ChunkID>\n")
        file:write("\t\t\t\t\t\t<ChunkID>" .. tostring(state.order) .. "</ChunkID>\n")
        file:write("\t\t\t\t\t</Chunks>\n")
        if state.transitions then
            file:write("\t\t\t\t\t<Transitions>\n")
            for targetName, transition in pairs(state.transitions) do
                file:write("\t\t\t\t\t\t<Transition To=\"" .. transition.toSegment .. "\" MinDuration=\"" .. string.format("%.2f", transition.minDuration or 0) .. "\" Target=\"" .. targetName .. "\"/>\n")
            end
            file:write("\t\t\t\t\t</Transitions>\n")
        end
        file:write("\t\t\t\t</State>\n")
    end
    file:write("\t\t\t</States>\n")

    file:write("\t\t</StateMachine>\n")
    file:write("\t</StateMachines>\n")

    file:write("</StoryManifest>\n")
    file:close()

    scene:destroy()
    soundEmitter:destroy()

end
