
require "windyday/scripts/util"

local LogDebug = false
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end

SetManager = {
}

function SetManager:init(scene)
    self.scene = scene
    self.setData = {}
    self.setPieces = {}
    self.showing = {}
    self:preProcessSets()
end

function SetManager:preProcessSets()
    self.setData = {}
    for i, set in ipairs(sets) do
        self.setData[set.name] = set
    end
end

function SetManager:showSet(name)
    Debug("SetManager showing set " .. name)
    local pieces = self.setData[name].pieces;
    -- Remove any set pieces from showing that we need for the new set
    if (pieces) then
        for _, piece in ipairs(pieces) do
            self.showing[piece] = nil
        end
    end

    -- Hide and delete the pieces that are showing, but are not part of the new set
    self:hide(self.showing)
        self:destroy(self.showing)


    -- Load and show the new set's pieces
    self:loadSet(name);
    self.showing = {}
    if (pieces) then
        for _, piece in ipairs(pieces) do
            Debug("Showing: " .. tostring(piece))
            self.showing[piece] = self.setPieces[piece];
        end
    end
    self:show(self.showing)
    
    if (Global.scene.sunGlow) then
    	local horizon = nil;
    	local canopy = nil;
    	local horizonIntensity = nil;
    	local canopyIntensity = nil;
    	
    	if(self.setData[name].sunGlowHorizon) then
    		horizon = Color.new(self.setData[name].sunGlowHorizon[1], self.setData[name].sunGlowHorizon[2], self.setData[name].sunGlowHorizon[3], .65882)
    		horizonIntensity = self.setData[name].sunGlowHorizon[5]
    	end
    	
    	if(self.setData[name].sunGlowCanopy) then
    		canopy = Color.new(self.setData[name].sunGlowCanopy[1], self.setData[name].sunGlowCanopy[2], self.setData[name].sunGlowCanopy[3], .65882)
    		canopyIntensity = self.setData[name].sunGlowCanopy[5]
    	end
    	
    	
		Global.scene.sunGlow:setGlow(horizon, canopy, horizonIntensity, canopyIntensity);
	end
end

function SetManager:loadSet(name)
    self:initSetPieces(self.setData[name])
end

function SetManager:show(pieces)
    self:setVisibility(pieces, true)
end

function SetManager:hide(pieces)
    self:setVisibility(pieces, false)
end

function SetManager:setVisibility(pieces, visible)
    if (pieces) then
        for _, piece in pairs(pieces) do
            if (piece) then
                piece:setVisibility(visible)
                Events.send("SetManager", "onSetVisibilityChanged", piece.name, visible)
            end
        end
    end
end

function SetManager:destroy(pieces)
    if (pieces) then
        for name, piece in pairs(pieces) do
            if (piece) then
                Debug("Destroying: " .. name .. " - " .. tostring(piece))
                self.setPieces[name] = nil
                ActorManager:destroyActor(name)
            end
        end
    end
end

function SetManager:initSetPieces(set)
    if (set and set.pieces) then
        for _, name in ipairs(set.pieces) do
            if (self.setPieces[name] == nil) then
                self.setPieces[name] = ActorManager:getActor(name)
                Debug("SetManager: creating piece " .. name .. " -> " .. tostring(self.setPieces[name]))
                self.setPieces[name]:setVisibility(false)
            end
        end
    end
end
