
require "common/scripts/moxie/moxie"

require "windyday/scripts/util"

local LogDebug = true
local Debug = function(...) end
if (LogDebug) then Debug = Log.message end

StarParticles = {
    particleSystem  = nil,

    spinSpeed       = 
    { 
        Vector(0.0, 5.0,0.0), 
        Vector(0.0,10.0,0.0) 
    },
    baseSpinSpeed   =
    { 
        Vector(0.0, 5.0,0.0), 
        Vector(0.0,10.0,0.0) 
    },

    emitters    = {},

    targetDriver    = nil,
    
    scene = nil,
    
    dropProxy 			= nil, 	-- the parent of all drivers
    starLoopEmitDriver 	= nil, 	-- the bone to drive the star particle loop
    starPepeEmitDriver 	= nil, 	-- the bone to drive the star particle loop
    
    -- particle emitters
    starsPepeEmitter = nil,
    starLoopEmitter = nil,
    starsStartEmitter = nil,
    
    mouseBone = nil,
    starBounceTime = 0,
    
    translationCurves = {},
}

StarParticles.new = function(self, scene)
	local o = {};
	setmetatable(o, self);
	o.scene = scene;
	self.__index = self;
	return o;
end

function StarParticles:onStart()
	
    -- get bones needed to drive the particles
   	self.starLoopEmitDriver = ActorManager:getBone({actor = "DropProxy", bone = "starLoopEmit|RootNode"})
   	self.starPepeEmitDriver = ActorManager:getBone({actor = "DropProxy", bone = "pepeStarsEmitDriver|RootNode"})
   	
   	self.mouseBone = ActorManager:getBone({actor = "Mouse", bone = "mouse_mox_cn_pelvis|mouse_MOXIE_skeleton|RootNode"})
   	
   	-- setup translation curve
   	local eval = function(x) 
   		return math.abs(math.sin((x+.1)*2))*((x)^2)*(x+.1)*0.45
   	end
   	
   	local max = eval(1)
   	
   	for i = 0, 31, 1 do
   		local sample = math.clamp01(eval(i/32))
   		self.translationCurves[i+1] = math.clamp01(sample/max)
   		--Log.message("EVAL "..(i+1).." ="..self.translationCurves[i+1])
   	end
   	
   	Events.listen(self, "segmentPlaying")
   	Events.listen(self, "idlePlaying")
	
end

function StarParticles:starsLoopStartSetup()

	if(not self.starsStartEmitter) then
		
	    -- parse systems
        Log.message( "Parsing Particle System starsLoopStart" )

        self.starsStartEmitter = Global.scene:createParticleEmitter( "starsLoopStart", "windyday:particles/starsstart.mxp" )
		self.starsStartEmitter:setLocal( SRT.new( Vector( 0.0, 0.001, 0.0, 0.0 ), Quaternion.identity, Vector.one ) )
     end
     
     self.starsStartEmitter:setActive(false)
	 self.starsStartEmitter:setVisibility(false)
     self.starsStartEmitter.timer = 0.0;
     self.starsStartEmitter.translateTimer = 0.0;
     self.starsStartEmitter.translateDuration = 7.0;
     self.starsStartEmitter.isConfigured = true;
     self.starsStartEmitter.isPaused = true;
end

function StarParticles:starsLoopSetup()

	if(not self.starLoopEmitter) then
		
	    -- parse systems
        Log.message( " Parsing Particle System starsLoop" )
        
        self.starLoopEmitter = Global.scene:createParticleEmitter( "starsPepeEmitter", "windyday:particles/starsloop.mxp" )
		self.starLoopEmitter:setLocal( SRT.new( Vector( 0.0, 0.0, 0.0 ), Quaternion.identity, Vector.one ) )
		self.starLoopEmitter.material = self.starLoopEmitter:getMaterial()
		
     end
     
     self.starLoopEmitter:setActive(false)
     self.starLoopEmitter:setVisibility(false)
     self.starLoopEmitter.timer = 0.0;
     self.starLoopEmitter.duration = 1.5;
     self.starLoopEmitter.isConfigured = true;
end

function StarParticles:starsPepeSetup()

	if(not self.starsPepeEmitter) then
		
	    -- parse systems
        Log.message( " Parsing Particle System starsLoop" )
        
        self.starsPepeEmitter = Global.scene:createParticleEmitter( "starsPepeEmitter", "windyday:particles/pepestars.mxp" )
		self.starsPepeEmitter:setLocal( SRT.new( Vector( 0.0, 0.0, 0.0 ), Quaternion.identity, Vector.one ) )
		self.starsPepeEmitter.material = self.starsPepeEmitter:getMaterial()
     end
     
     self.starsPepeEmitter:setActive(false)
     self.starsPepeEmitter.timer = 0.0;
     self.starsPepeEmitter.duration = 2.5;
     self.starsPepeEmitter.hasStarted = false;
     self.starsPepeEmitter.isConfigured = true;
end

function StarParticles:onVisible( isVisible )
	if(isVisible) then
		-- run the visible update
		self.onUpdate = self.onUpdateVisible;
		
		-- this doesn't get called as expected when scene starts from a pertinent segment
		self:segmentPlaying(AnimationManager.current.displayName)
		
		
    else
    	-- hide update method
    	self.onUpdate = nil
    end
    
end

function StarParticles:updateStarsLoopStart()
	if(self.starsStartEmitter ~= nil) then
		self.starsStartEmitter.timer = self.starsStartEmitter.timer + Time.dt()
		if(self.starsStartEmitter.timer >= 1.0 and not self.starsStartEmitter:isActive()) then
			self.starsStartEmitter:setActive(true)
			self.starsStartEmitter:setVisibility(true)
		end
		
		if(self.starsStartEmitter:isActive() and not self.starsStartEmitter.isPaused) then
			local localT = self.starsStartEmitter:getLocal()
			
			if(self.starsStartEmitter.translateTimer < self.starsStartEmitter.translateDuration) then
				self.starsStartEmitter.translateTimer = self.starsStartEmitter.translateTimer + Time.dt()
				local sampleAt = self.starsStartEmitter.translateTimer/self.starsStartEmitter.translateDuration
				local sample = math.spline(sampleAt, self.translationCurves, 32)
	   			--	Log.message("SPLINE: time:"..sampleAt.." = "..sample)
	   			self.speed = sample*300 - localT.position.y
				localT.position.y = sample*300
				self.starsStartEmitter:setLocal(localT)
			else
				localT.position.y = localT.position.y + self.speed
				self.starsStartEmitter:setLocal(localT)
				--self.starsStartEmitter:setActive(false)
				--self.starsStartEmitter:setVisibility(false)
			end
		end
	end
end

function StarParticles:updateStarsLoop()
	if(self.starLoopEmitter ~= nil and self.starLoopEmitter.timer < self.starLoopEmitter.duration and not self.starLoopEmitter.isPaused) then
		self.starLoopEmitter.timer = self.starLoopEmitter.timer + Time.dt()
		if(self.starLoopEmitter.timer >= self.starLoopEmitter.duration) then
			self.starLoopEmitter:setActive(true)
     		self.starLoopEmitter:setVisibility(true)
		end
	end
end

function StarParticles:updateStarsStart()

	if(self.starsPepeEmitter ~= nil) then
	
		if(not self.starsPepeEmitter.hasStarted) then
			self.starsPepeEmitter.timer = self.starsPepeEmitter.timer + Time.dt()
			if(self.starsPepeEmitter.timer >= 1.0) then
				self.starsPepeEmitter.timer = 0
				self.starsPepeEmitter:setActive(true)
				self.starsPepeEmitter.hasStarted = true
			end
		end
		
		if(self.starsPepeEmitter.hasStarted) then
			local starsT = self.mouseBone:getWorld()
			local localT = self.starsPepeEmitter:getLocal()
			localT.position = starsT.position:copy()
			self.starsPepeEmitter:setLocal(localT)
			
			-- timer to kill the emitter
--			self.starsPepeEmitter.timer = self.starsPepeEmitter.timer + Time.dt()
--			if(self.starsPepeEmitter.timer >= self.starsPepeEmitter.duration) then
--				self.starsPepeEmitter:setActive(false)
--				self.starsPepeEmitter:setVisibility(false)
--			end
		end
	end
end

function StarParticles:onUpdateVisible()
	
    self:updateStarsStart();
    self:updateStarsLoopStart();
    self:updateStarsLoop();
 	
	local position = self.starLoopEmitDriver:getLocal().position;
	if (self.starLoopEmitter ~= nil and position.y > .1 and (not self.starLoopEmitter.visible)) then
		self.starLoopEmitter:setVisibility(true)
		self.starLoopEmitter.visible = true;
	elseif(self.starLoopEmitter ~= nil and position.y < .1 and self.starLoopEmitter.visible) then
		self.starLoopEmitter:setVisibility(false)
		self.starLoopEmitter.visible = false;
	end

end

function StarParticles:onDestroy()
    if(self.starLoopEmitter) then
        self.starLoopEmitter:destroy()
		self.starLoopEmitter = nil
    end
	if(self.starPepeEmitter) then
		self.starPepeEmitter:destroy()
		self.starPepeEmitter = nil
	end
	if(self.starsStartEmitter) then
		self.starsStartEmitter:destroy()
		self.starsStartEmitter = nil
	end
end

function StarParticles:scaleSpeed(scalar) 
    for i = 1, #self.baseSpinSpeed do
        self.spinSpeed[i] = self.baseSpinSpeed[i] * scalar
    end
end

function StarParticles:idlePlaying(idleName)
		if (idleName == "BarrageE_Idle") then
			Log.message("------------ STARS IDLING --------------")
			self.starsStartEmitter:setTimeScale(0)
			self.starLoopEmitter:setTimeScale(0)
			self.starsStartEmitter.isPaused = true
			self.starLoopEmitter.isPaused = true
		end
end
function StarParticles:segmentPlaying(segmentName)
		
		if (segmentName == "BarrageE") then
			if(not self.starPepeEmitter)then
				self:starsPepeSetup();
			end
			if(not self.starLoopEmitter)then
				self:starsLoopSetup();
			end
			if(not self.starsStartEmitter)then
				self:starsLoopStartSetup();
			end
		end
		
		if (segmentName == "DropA") then
		
			-- scene must be starting from this point
			if(not self.starLoopEmitter)then
				self:starsLoopSetup();
				self.starLoopEmitter:setActive(true)
     			self.starLoopEmitter:setVisibility(true)
     			self.starLoopEmitter.isPaused = true
			end
			if(not self.starsStartEmitter)then
				self:starsLoopStartSetup();
				self.starsStartEmitter:setActive(true)
     			self.starsStartEmitter:setVisibility(true)
     			self.starsStartEmitter.isPaused = true
			end
			
			-- resume from idle
			if (self.starsStartEmitter ~= nil and self.starsStartEmitter.isPaused) then
				self.starsStartEmitter:setTimeScale(1)
				self.starsStartEmitter.isPaused = false
			end
			
			if (self.starLoopEmitter ~= nil and self.starLoopEmitter.isPaused) then
				self.starLoopEmitter:setTimeScale(1);
				self.starLoopEmitter.isPaused = false
			end
		end
end
